// Copyright (c) 2013-2016 CoolGames

/******************************************************************************
 * Spine Runtimes Software License
 * Version 2.3
 * 
 * Copyright (c) 2013-2015, Esoteric Software
 * All rights reserved.
 * 
 * You are granted a perpetual, non-exclusive, non-sublicensable and
 * non-transferable license to use, install, execute and perform the Spine
 * Runtimes Software (the "Software") and derivative works solely for personal
 * or internal use. Without the written permission of Esoteric Software (see
 * Section 2 of the Spine Software License Agreement), you may not (a) modify,
 * translate, adapt or otherwise create derivative works, improvements of the
 * Software or develop new applications using the Software or (b) remove,
 * delete, alter or obscure any trademarks or any copyright, trademark, patent
 * or other intellectual property or proprietary rights notices on or in the
 * Software, including any copy thereof. Redistributions in binary or source
 * form must include this license and terms.
 * 
 * THIS SOFTWARE IS PROVIDED BY ESOTERIC SOFTWARE "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL ESOTERIC SOFTWARE BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

var spine = {
	radDeg: 180 / Math.PI,
	degRad: Math.PI / 180,
	temp: [],
    Float32Array: (typeof(Float32Array) === 'undefined') ? Array : Float32Array,
    Uint16Array: (typeof(Uint16Array) === 'undefined') ? Array : Uint16Array
};

spine.BoneData = function (name, parent) {
	this.name = name;
	this.parent = parent;
};
spine.BoneData.prototype = {
	length: 0,
	x: 0, y: 0,
	rotation: 0,
	scaleX: 1, scaleY: 1,
	inheritScale: true,
	inheritRotation: true,
	flipX: false, flipY: false
};

spine.BlendMode = {
	normal: 0,
	additive: 1,
	multiply: 2,
	screen: 3
};

spine.SlotData = function (name, boneData) {
	this.name = name;
	this.boneData = boneData;
};
spine.SlotData.prototype = {
	r: 1, g: 1, b: 1, a: 1,
	attachmentName: null,
	blendMode: spine.BlendMode.normal
};

spine.IkConstraintData = function (name) {
	this.name = name;
	this.bones = [];
};
spine.IkConstraintData.prototype = {
	target: null,
	bendDirection: 1,
	mix: 1
};

spine.Bone = function (boneData, skeleton, parent) {
	this.data = boneData;
	this.skeleton = skeleton;
	this.parent = parent;
	this.setToSetupPose();
};
spine.Bone.yDown = false;
spine.Bone.prototype = {
	x: 0, y: 0,
	rotation: 0, rotationIK: 0,
	scaleX: 1, scaleY: 1,
	flipX: false, flipY: false,
	m00: 0, m01: 0, worldX: 0, // a b x
	m10: 0, m11: 0, worldY: 0, // c d y
	worldRotation: 0,
	worldScaleX: 1, worldScaleY: 1,
	worldFlipX: false, worldFlipY: false,
	updateWorldTransform: function () {
		var parent = this.parent;
		if (parent) {
			this.worldX = this.x * parent.m00 + this.y * parent.m01 + parent.worldX;
			this.worldY = this.x * parent.m10 + this.y * parent.m11 + parent.worldY;
			if (this.data.inheritScale) {
				this.worldScaleX = parent.worldScaleX * this.scaleX;
				this.worldScaleY = parent.worldScaleY * this.scaleY;
			} else {
				this.worldScaleX = this.scaleX;
				this.worldScaleY = this.scaleY;
			}
			this.worldRotation = this.data.inheritRotation ? (parent.worldRotation + this.rotationIK) : this.rotationIK;
			this.worldFlipX = parent.worldFlipX != this.flipX;
			this.worldFlipY = parent.worldFlipY != this.flipY;
		} else {
			var skeletonFlipX = this.skeleton.flipX, skeletonFlipY = this.skeleton.flipY;
			this.worldX = skeletonFlipX ? -this.x : this.x;
			this.worldY = (skeletonFlipY != spine.Bone.yDown) ? -this.y : this.y;
			this.worldScaleX = this.scaleX;
			this.worldScaleY = this.scaleY;
			this.worldRotation = this.rotationIK;
			this.worldFlipX = skeletonFlipX != this.flipX;
			this.worldFlipY = skeletonFlipY != this.flipY;
		}
		var radians = this.worldRotation * spine.degRad;
		var cos = Math.cos(radians);
		var sin = Math.sin(radians);
		if (this.worldFlipX) {
			this.m00 = -cos * this.worldScaleX;
			this.m01 = sin * this.worldScaleY;
		} else {
			this.m00 = cos * this.worldScaleX;
			this.m01 = -sin * this.worldScaleY;
		}
		if (this.worldFlipY != spine.Bone.yDown) {
			this.m10 = -sin * this.worldScaleX;
			this.m11 = -cos * this.worldScaleY;
		} else {
			this.m10 = sin * this.worldScaleX;
			this.m11 = cos * this.worldScaleY;
		}
	},
	setToSetupPose: function () {
		var data = this.data;
		this.x = data.x;
		this.y = data.y;
		this.rotation = data.rotation;
		this.rotationIK = this.rotation;
		this.scaleX = data.scaleX;
		this.scaleY = data.scaleY;
		this.flipX = data.flipX;
		this.flipY = data.flipY;
	},
	worldToLocal: function (world) {
		var dx = world[0] - this.worldX, dy = world[1] - this.worldY;
		var m00 = this.m00, m10 = this.m10, m01 = this.m01, m11 = this.m11;
		if (this.worldFlipX != (this.worldFlipY != spine.Bone.yDown)) {
			m00 = -m00;
			m11 = -m11;
		}
		var invDet = 1 / (m00 * m11 - m01 * m10);
		world[0] = dx * m00 * invDet - dy * m01 * invDet;
		world[1] = dy * m11 * invDet - dx * m10 * invDet;
	},
	localToWorld: function (local) {
		var localX = local[0], localY = local[1];
		local[0] = localX * this.m00 + localY * this.m01 + this.worldX;
		local[1] = localX * this.m10 + localY * this.m11 + this.worldY;
	}
};

spine.Slot = function (slotData, bone) {
	this.data = slotData;
	this.bone = bone;
	this.setToSetupPose();
};
spine.Slot.prototype = {
	r: 1, g: 1, b: 1, a: 1,
	_attachmentTime: 0,
	attachment: null,
	attachmentVertices: [],
	setAttachment: function (attachment) {
		if (this.attachment == attachment) return;
		this.attachment = attachment;
		this._attachmentTime = this.bone.skeleton.time;
		this.attachmentVertices.length = 0;
	},
	setAttachmentTime: function (time) {
		this._attachmentTime = this.bone.skeleton.time - time;
	},
	getAttachmentTime: function () {
		return this.bone.skeleton.time - this._attachmentTime;
	},
	setToSetupPose: function () {
		var data = this.data;
		this.r = data.r;
		this.g = data.g;
		this.b = data.b;
		this.a = data.a;

		if (!data.attachmentName)
			this.setAttachment(null);
		else {
			var slotDatas = this.bone.skeleton.data.slots;
			for (var i = 0, n = slotDatas.length; i < n; i++) {
				if (slotDatas[i] == data) {
					this.attachment = null;
					this.setAttachment(this.bone.skeleton.getAttachmentBySlotIndex(i, data.attachmentName));
					break;
				}
			}
		}
	}
};

spine.IkConstraint = function (data, skeleton) {
	this.data = data;
	this.mix = data.mix;
	this.bendDirection = data.bendDirection;

	this.bones = [];
	for (var i = 0, n = data.bones.length; i < n; i++)
		this.bones.push(skeleton.findBone(data.bones[i].name));
	this.target = skeleton.findBone(data.target.name);
};
spine.IkConstraint.prototype = {
	apply: function () {
		var target = this.target;
		var bones = this.bones;
		switch (bones.length) {
		case 1:
			spine.IkConstraint.apply1(bones[0], target.worldX, target.worldY, this.mix);
			break;
		case 2:
			spine.IkConstraint.apply2(bones[0], bones[1], target.worldX, target.worldY, this.bendDirection, this.mix);
			break;
		}
	}
};
/** Adjusts the bone rotation so the tip is as close to the target position as possible. The target is specified in the world
 * coordinate system. */
spine.IkConstraint.apply1 = function (bone, targetX, targetY, alpha) {
	var parentRotation = (!bone.data.inheritRotation || !bone.parent) ? 0 : bone.parent.worldRotation;
	var rotation = bone.rotation;
	var rotationIK = Math.atan2(targetY - bone.worldY, targetX - bone.worldX) * spine.radDeg;
	if (bone.worldFlipX != (bone.worldFlipY != spine.Bone.yDown)) rotationIK = -rotationIK;
	rotationIK -= parentRotation;
	bone.rotationIK = rotation + (rotationIK - rotation) * alpha;
};
/** Adjusts the parent and child bone rotations so the tip of the child is as close to the target position as possible. The
 * target is specified in the world coordinate system.
 * @param child Any descendant bone of the parent. */
spine.IkConstraint.apply2 = function (parent, child, targetX, targetY, bendDirection, alpha) {
	var childRotation = child.rotation, parentRotation = parent.rotation;
	if (!alpha) {
		child.rotationIK = childRotation;
		parent.rotationIK = parentRotation;
		return;
	}
	var positionX, positionY, tempPosition = spine.temp;
	var parentParent = parent.parent;
	if (parentParent) {
		tempPosition[0] = targetX;
		tempPosition[1] = targetY;
		parentParent.worldToLocal(tempPosition);
		targetX = (tempPosition[0] - parent.x) * parentParent.worldScaleX;
		targetY = (tempPosition[1] - parent.y) * parentParent.worldScaleY;
	} else {
		targetX -= parent.x;
		targetY -= parent.y;
	}
	if (child.parent == parent) {
		positionX = child.x;
		positionY = child.y;
	} else {
		tempPosition[0] = child.x;
		tempPosition[1] = child.y;
		child.parent.localToWorld(tempPosition);
		parent.worldToLocal(tempPosition);
		positionX = tempPosition[0];
		positionY = tempPosition[1];
	}
	var childX = positionX * parent.worldScaleX, childY = positionY * parent.worldScaleY;
	var offset = Math.atan2(childY, childX);
	var len1 = Math.sqrt(childX * childX + childY * childY), len2 = child.data.length * child.worldScaleX;
	// Based on code by Ryan Juckett with permission: Copyright (c) 2008-2009 Ryan Juckett, http://www.ryanjuckett.com/
	var cosDenom = 2 * len1 * len2;
	if (cosDenom < 0.0001) {
		child.rotationIK = childRotation + (Math.atan2(targetY, targetX) * spine.radDeg - parentRotation - childRotation) * alpha;
		return;
	}
	var cos = (targetX * targetX + targetY * targetY - len1 * len1 - len2 * len2) / cosDenom;
	if (cos < -1)
		cos = -1;
	else if (cos > 1)
		cos = 1;
	var childAngle = Math.acos(cos) * bendDirection;
	var adjacent = len1 + len2 * cos, opposite = len2 * Math.sin(childAngle);
	var parentAngle = Math.atan2(targetY * adjacent - targetX * opposite, targetX * adjacent + targetY * opposite);
	var rotation = (parentAngle - offset) * spine.radDeg - parentRotation;
	if (rotation > 180)
		rotation -= 360;
	else if (rotation < -180) //
		rotation += 360;
	parent.rotationIK = parentRotation + rotation * alpha;
	rotation = (childAngle + offset) * spine.radDeg - childRotation;
	if (rotation > 180)
		rotation -= 360;
	else if (rotation < -180) //
		rotation += 360;
	child.rotationIK = childRotation + (rotation + parent.worldRotation - child.parent.worldRotation) * alpha;
};

spine.Skin = function (name) {
	this.name = name;
	this.attachments = {};
};
spine.Skin.prototype = {
	addAttachment: function (slotIndex, name, attachment) {
		this.attachments[slotIndex + ":" + name] = attachment;
	},
	getAttachment: function (slotIndex, name) {
		return this.attachments[slotIndex + ":" + name];
	},
	_attachAll: function (skeleton, oldSkin) {
		for (var key in oldSkin.attachments) {
			var colon = key.indexOf(":");
			var slotIndex = parseInt(key.substring(0, colon));
			var name = key.substring(colon + 1);
			var slot = skeleton.slots[slotIndex];
			if (slot.attachment && slot.attachment.name == name) {
				var attachment = this.getAttachment(slotIndex, name);
				if (attachment) slot.setAttachment(attachment);
			}
		}
	}
};

spine.Animation = function (name, timelines, duration) {
	this.name = name;
	this.timelines = timelines;
	this.duration = duration;
};
spine.Animation.prototype = {
	apply: function (skeleton, lastTime, time, loop, events) {
		if (loop && this.duration != 0) {
			time %= this.duration;
			lastTime %= this.duration;
		}
		var timelines = this.timelines;
		for (var i = 0, n = timelines.length; i < n; i++)
			timelines[i].apply(skeleton, lastTime, time, events, 1);
	},
	mix: function (skeleton, lastTime, time, loop, events, alpha) {
		if (loop && this.duration != 0) {
			time %= this.duration;
			lastTime %= this.duration;
		}
		var timelines = this.timelines;
		for (var i = 0, n = timelines.length; i < n; i++)
			timelines[i].apply(skeleton, lastTime, time, events, alpha);
	}
};
spine.Animation.binarySearch = function (values, target, step) {
	var low = 0;
	var high = Math.floor(values.length / step) - 2;
	if (!high) return step;
	var current = high >>> 1;
	while (true) {
		if (values[(current + 1) * step] <= target)
			low = current + 1;
		else
			high = current;
		if (low == high) return (low + 1) * step;
		current = (low + high) >>> 1;
	}
};
spine.Animation.binarySearch1 = function (values, target) {
	var low = 0;
	var high = values.length - 2;
	if (!high) return 1;
	var current = high >>> 1;
	while (true) {
		if (values[current + 1] <= target)
			low = current + 1;
		else
			high = current;
		if (low == high) return low + 1;
		current = (low + high) >>> 1;
	}
};
spine.Animation.linearSearch = function (values, target, step) {
	for (var i = 0, last = values.length - step; i <= last; i += step)
		if (values[i] > target) return i;
	return -1;
};

spine.Curves = function (frameCount) {
	this.curves = []; // type, x, y, ...
	//this.curves.length = (frameCount - 1) * 19/*BEZIER_SIZE*/;
};
spine.Curves.prototype = {
	setLinear: function (frameIndex) {
		this.curves[frameIndex * 19/*BEZIER_SIZE*/] = 0/*LINEAR*/;
	},
	setStepped: function (frameIndex) {
		this.curves[frameIndex * 19/*BEZIER_SIZE*/] = 1/*STEPPED*/;
	},
	/** Sets the control handle positions for an interpolation bezier curve used to transition from this keyframe to the next.
	 * cx1 and cx2 are from 0 to 1, representing the percent of time between the two keyframes. cy1 and cy2 are the percent of
	 * the difference between the keyframe's values. */
	setCurve: function (frameIndex, cx1, cy1, cx2, cy2) {
		var subdiv1 = 1 / 10/*BEZIER_SEGMENTS*/, subdiv2 = subdiv1 * subdiv1, subdiv3 = subdiv2 * subdiv1;
		var pre1 = 3 * subdiv1, pre2 = 3 * subdiv2, pre4 = 6 * subdiv2, pre5 = 6 * subdiv3;
		var tmp1x = -cx1 * 2 + cx2, tmp1y = -cy1 * 2 + cy2, tmp2x = (cx1 - cx2) * 3 + 1, tmp2y = (cy1 - cy2) * 3 + 1;
		var dfx = cx1 * pre1 + tmp1x * pre2 + tmp2x * subdiv3, dfy = cy1 * pre1 + tmp1y * pre2 + tmp2y * subdiv3;
		var ddfx = tmp1x * pre4 + tmp2x * pre5, ddfy = tmp1y * pre4 + tmp2y * pre5;
		var dddfx = tmp2x * pre5, dddfy = tmp2y * pre5;

		var i = frameIndex * 19/*BEZIER_SIZE*/;
		var curves = this.curves;
		curves[i++] = 2/*BEZIER*/;
		
		var x = dfx, y = dfy;
		for (var n = i + 19/*BEZIER_SIZE*/ - 1; i < n; i += 2) {
			curves[i] = x;
			curves[i + 1] = y;
			dfx += ddfx;
			dfy += ddfy;
			ddfx += dddfx;
			ddfy += dddfy;
			x += dfx;
			y += dfy;
		}
	},
	getCurvePercent: function (frameIndex, percent) {
		percent = percent < 0 ? 0 : (percent > 1 ? 1 : percent);
		var curves = this.curves;
		var i = frameIndex * 19/*BEZIER_SIZE*/;
		var type = curves[i];
		if (type === 0/*LINEAR*/) return percent;
		if (type == 1/*STEPPED*/) return 0;
		i++;
		var x = 0;
		for (var start = i, n = i + 19/*BEZIER_SIZE*/ - 1; i < n; i += 2) {
			x = curves[i];
			if (x >= percent) {
				var prevX, prevY;
				if (i == start) {
					prevX = 0;
					prevY = 0;
				} else {
					prevX = curves[i - 2];
					prevY = curves[i - 1];
				}
				return prevY + (curves[i + 1] - prevY) * (percent - prevX) / (x - prevX);
			}
		}
		var y = curves[i - 1];
		return y + (1 - y) * (percent - x) / (1 - x); // Last point is 1,1.
	}
};

spine.RotateTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, angle, ...
	this.frames.length = frameCount * 2;
};
spine.RotateTimeline.prototype = {
	boneIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 2;
	},
	setFrame: function (frameIndex, time, angle) {
		frameIndex *= 2;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = angle;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var bone = skeleton.bones[this.boneIndex];

		if (time >= frames[frames.length - 2]) { // Time is after last frame.
			var amount = bone.data.rotation + frames[frames.length - 1] - bone.rotation;
			while (amount > 180)
				amount -= 360;
			while (amount < -180)
				amount += 360;
			bone.rotation += amount * alpha;
			return;
		}

		// Interpolate between the previous frame and the current frame.
		var frameIndex = spine.Animation.binarySearch(frames, time, 2);
		var prevFrameValue = frames[frameIndex - 1];
		var frameTime = frames[frameIndex];
		var percent = 1 - (time - frameTime) / (frames[frameIndex - 2/*PREV_FRAME_TIME*/] - frameTime);
		percent = this.curves.getCurvePercent(frameIndex / 2 - 1, percent);

		var amount = frames[frameIndex + 1/*FRAME_VALUE*/] - prevFrameValue;
		while (amount > 180)
			amount -= 360;
		while (amount < -180)
			amount += 360;
		amount = bone.data.rotation + (prevFrameValue + amount * percent) - bone.rotation;
		while (amount > 180)
			amount -= 360;
		while (amount < -180)
			amount += 360;
		bone.rotation += amount * alpha;
	}
};

spine.TranslateTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, x, y, ...
	this.frames.length = frameCount * 3;
};
spine.TranslateTimeline.prototype = {
	boneIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 3;
	},
	setFrame: function (frameIndex, time, x, y) {
		frameIndex *= 3;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = x;
		this.frames[frameIndex + 2] = y;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var bone = skeleton.bones[this.boneIndex];

		if (time >= frames[frames.length - 3]) { // Time is after last frame.
			bone.x += (bone.data.x + frames[frames.length - 2] - bone.x) * alpha;
			bone.y += (bone.data.y + frames[frames.length - 1] - bone.y) * alpha;
			return;
		}

		// Interpolate between the previous frame and the current frame.
		var frameIndex = spine.Animation.binarySearch(frames, time, 3);
		var prevFrameX = frames[frameIndex - 2];
		var prevFrameY = frames[frameIndex - 1];
		var frameTime = frames[frameIndex];
		var percent = 1 - (time - frameTime) / (frames[frameIndex + -3/*PREV_FRAME_TIME*/] - frameTime);
		percent = this.curves.getCurvePercent(frameIndex / 3 - 1, percent);

		bone.x += (bone.data.x + prevFrameX + (frames[frameIndex + 1/*FRAME_X*/] - prevFrameX) * percent - bone.x) * alpha;
		bone.y += (bone.data.y + prevFrameY + (frames[frameIndex + 2/*FRAME_Y*/] - prevFrameY) * percent - bone.y) * alpha;
	}
};

spine.ScaleTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, x, y, ...
	this.frames.length = frameCount * 3;
};
spine.ScaleTimeline.prototype = {
	boneIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 3;
	},
	setFrame: function (frameIndex, time, x, y) {
		frameIndex *= 3;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = x;
		this.frames[frameIndex + 2] = y;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var bone = skeleton.bones[this.boneIndex];

		if (time >= frames[frames.length - 3]) { // Time is after last frame.
			bone.scaleX += (bone.data.scaleX * frames[frames.length - 2] - bone.scaleX) * alpha;
			bone.scaleY += (bone.data.scaleY * frames[frames.length - 1] - bone.scaleY) * alpha;
			return;
		}

		// Interpolate between the previous frame and the current frame.
		var frameIndex = spine.Animation.binarySearch(frames, time, 3);
		var prevFrameX = frames[frameIndex - 2];
		var prevFrameY = frames[frameIndex - 1];
		var frameTime = frames[frameIndex];
		var percent = 1 - (time - frameTime) / (frames[frameIndex + -3/*PREV_FRAME_TIME*/] - frameTime);
		percent = this.curves.getCurvePercent(frameIndex / 3 - 1, percent);

		bone.scaleX += (bone.data.scaleX * (prevFrameX + (frames[frameIndex + 1/*FRAME_X*/] - prevFrameX) * percent) - bone.scaleX) * alpha;
		bone.scaleY += (bone.data.scaleY * (prevFrameY + (frames[frameIndex + 2/*FRAME_Y*/] - prevFrameY) * percent) - bone.scaleY) * alpha;
	}
};

spine.ColorTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, r, g, b, a, ...
	this.frames.length = frameCount * 5;
};
spine.ColorTimeline.prototype = {
	slotIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 5;
	},
	setFrame: function (frameIndex, time, r, g, b, a) {
		frameIndex *= 5;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = r;
		this.frames[frameIndex + 2] = g;
		this.frames[frameIndex + 3] = b;
		this.frames[frameIndex + 4] = a;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var r, g, b, a;
		if (time >= frames[frames.length - 5]) {
			// Time is after last frame.
			var i = frames.length - 1;
			r = frames[i - 3];
			g = frames[i - 2];
			b = frames[i - 1];
			a = frames[i];
		} else {
			// Interpolate between the previous frame and the current frame.
			var frameIndex = spine.Animation.binarySearch(frames, time, 5);
			var prevFrameR = frames[frameIndex - 4];
			var prevFrameG = frames[frameIndex - 3];
			var prevFrameB = frames[frameIndex - 2];
			var prevFrameA = frames[frameIndex - 1];
			var frameTime = frames[frameIndex];
			var percent = 1 - (time - frameTime) / (frames[frameIndex - 5/*PREV_FRAME_TIME*/] - frameTime);
			percent = this.curves.getCurvePercent(frameIndex / 5 - 1, percent);

			r = prevFrameR + (frames[frameIndex + 1/*FRAME_R*/] - prevFrameR) * percent;
			g = prevFrameG + (frames[frameIndex + 2/*FRAME_G*/] - prevFrameG) * percent;
			b = prevFrameB + (frames[frameIndex + 3/*FRAME_B*/] - prevFrameB) * percent;
			a = prevFrameA + (frames[frameIndex + 4/*FRAME_A*/] - prevFrameA) * percent;
		}
		var slot = skeleton.slots[this.slotIndex];
		if (alpha < 1) {
			slot.r += (r - slot.r) * alpha;
			slot.g += (g - slot.g) * alpha;
			slot.b += (b - slot.b) * alpha;
			slot.a += (a - slot.a) * alpha;
		} else {
			slot.r = r;
			slot.g = g;
			slot.b = b;
			slot.a = a;
		}
	}
};

spine.AttachmentTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, ...
	this.frames.length = frameCount;
	this.attachmentNames = [];
	this.attachmentNames.length = frameCount;
};
spine.AttachmentTimeline.prototype = {
	slotIndex: 0,
	getFrameCount: function () {
		return this.frames.length;
	},
	setFrame: function (frameIndex, time, attachmentName) {
		this.frames[frameIndex] = time;
		this.attachmentNames[frameIndex] = attachmentName;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) {
			if (lastTime > time) this.apply(skeleton, lastTime, Number.MAX_VALUE, null, 0);
			return;
		} else if (lastTime > time) //
			lastTime = -1;

		var frameIndex = time >= frames[frames.length - 1] ? frames.length - 1 : spine.Animation.binarySearch1(frames, time) - 1;
		if (frames[frameIndex] < lastTime) return;

		var attachmentName = this.attachmentNames[frameIndex];
		skeleton.slots[this.slotIndex].setAttachment(
			!attachmentName ? null : skeleton.getAttachmentBySlotIndex(this.slotIndex, attachmentName));
	}
};

spine.EventTimeline = function (frameCount) {
	this.frames = []; // time, ...
	this.frames.length = frameCount;
	this.events = [];
	this.events.length = frameCount;
};
spine.EventTimeline.prototype = {
	getFrameCount: function () {
		return this.frames.length;
	},
	setFrame: function (frameIndex, time, event) {
		this.frames[frameIndex] = time;
		this.events[frameIndex] = event;
	},
	/** Fires events for frames > lastTime and <= time. */
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		if (!firedEvents) return;

		var frames = this.frames;
		var frameCount = frames.length;

		if (lastTime > time) { // Fire events after last time for looped animations.
			this.apply(skeleton, lastTime, Number.MAX_VALUE, firedEvents, alpha);
			lastTime = -1;
		} else if (lastTime >= frames[frameCount - 1]) // Last time is after last frame.
			return;
		if (time < frames[0]) return; // Time is before first frame.

		var frameIndex;
		if (lastTime < frames[0])
			frameIndex = 0;
		else {
			frameIndex = spine.Animation.binarySearch1(frames, lastTime);
			var frame = frames[frameIndex];
			while (frameIndex > 0) { // Fire multiple events with the same frame.
				if (frames[frameIndex - 1] != frame) break;
				frameIndex--;
			}
		}
		var events = this.events;
		for (; frameIndex < frameCount && time >= frames[frameIndex]; frameIndex++)
			firedEvents.push(events[frameIndex]);
	}
};

spine.DrawOrderTimeline = function (frameCount) {
	this.frames = []; // time, ...
	this.frames.length = frameCount;
	this.drawOrders = [];
	this.drawOrders.length = frameCount;
};
spine.DrawOrderTimeline.prototype = {
	getFrameCount: function () {
		return this.frames.length;
	},
	setFrame: function (frameIndex, time, drawOrder) {
		this.frames[frameIndex] = time;
		this.drawOrders[frameIndex] = drawOrder;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var frameIndex;
		if (time >= frames[frames.length - 1]) // Time is after last frame.
			frameIndex = frames.length - 1;
		else
			frameIndex = spine.Animation.binarySearch1(frames, time) - 1;

		var drawOrder = skeleton.drawOrder;
		var slots = skeleton.slots;
		var drawOrderToSetupIndex = this.drawOrders[frameIndex];
		if (!drawOrderToSetupIndex) {
			for (var i = 0, n = slots.length; i < n; i++)
				drawOrder[i] = slots[i];
		} else {
			for (var i = 0, n = drawOrderToSetupIndex.length; i < n; i++)
				drawOrder[i] = skeleton.slots[drawOrderToSetupIndex[i]];
		}

	}
};

spine.FfdTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = [];
	this.frames.length = frameCount;
	this.frameVertices = [];
	this.frameVertices.length = frameCount;
};
spine.FfdTimeline.prototype = {
	slotIndex: 0,
	attachment: 0,
	getFrameCount: function () {
		return this.frames.length;
	},
	setFrame: function (frameIndex, time, vertices) {
		this.frames[frameIndex] = time;
		this.frameVertices[frameIndex] = vertices;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var slot = skeleton.slots[this.slotIndex];
		if (slot.attachment != this.attachment) return;

		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var frameVertices = this.frameVertices;
		var vertexCount = frameVertices[0].length;

		var vertices = slot.attachmentVertices;
		if (vertices.length != vertexCount) alpha = 1;
		vertices.length = vertexCount;

		if (time >= frames[frames.length - 1]) { // Time is after last frame.
			var lastVertices = frameVertices[frames.length - 1];
			if (alpha < 1) {
				for (var i = 0; i < vertexCount; i++)
					vertices[i] += (lastVertices[i] - vertices[i]) * alpha;
			} else {
				for (var i = 0; i < vertexCount; i++)
					vertices[i] = lastVertices[i];
			}
			return;
		}

		// Interpolate between the previous frame and the current frame.
		var frameIndex = spine.Animation.binarySearch1(frames, time);
		var frameTime = frames[frameIndex];
		var percent = 1 - (time - frameTime) / (frames[frameIndex - 1] - frameTime);
		percent = this.curves.getCurvePercent(frameIndex - 1, percent < 0 ? 0 : (percent > 1 ? 1 : percent));

		var prevVertices = frameVertices[frameIndex - 1];
		var nextVertices = frameVertices[frameIndex];

		if (alpha < 1) {
			for (var i = 0; i < vertexCount; i++) {
				var prev = prevVertices[i];
				vertices[i] += (prev + (nextVertices[i] - prev) * percent - vertices[i]) * alpha;
			}
		} else {
			for (var i = 0; i < vertexCount; i++) {
				var prev = prevVertices[i];
				vertices[i] = prev + (nextVertices[i] - prev) * percent;
			}
		}
	}
};

spine.IkConstraintTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, mix, bendDirection, ...
	this.frames.length = frameCount * 3;
};
spine.IkConstraintTimeline.prototype = {
	ikConstraintIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 3;
	},
	setFrame: function (frameIndex, time, mix, bendDirection) {
		frameIndex *= 3;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = mix;
		this.frames[frameIndex + 2] = bendDirection;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) return; // Time is before first frame.

		var ikConstraint = skeleton.ikConstraints[this.ikConstraintIndex];

		if (time >= frames[frames.length - 3]) { // Time is after last frame.
			ikConstraint.mix += (frames[frames.length - 2] - ikConstraint.mix) * alpha;
			ikConstraint.bendDirection = frames[frames.length - 1];
			return;
		}

		// Interpolate between the previous frame and the current frame.
		var frameIndex = spine.Animation.binarySearch(frames, time, 3);
		var prevFrameMix = frames[frameIndex + -2/*PREV_FRAME_MIX*/];
		var frameTime = frames[frameIndex];
		var percent = 1 - (time - frameTime) / (frames[frameIndex + -3/*PREV_FRAME_TIME*/] - frameTime);
		percent = this.curves.getCurvePercent(frameIndex / 3 - 1, percent);

		var mix = prevFrameMix + (frames[frameIndex + 1/*FRAME_MIX*/] - prevFrameMix) * percent;
		ikConstraint.mix += (mix - ikConstraint.mix) * alpha;
		ikConstraint.bendDirection = frames[frameIndex + -1/*PREV_FRAME_BEND_DIRECTION*/];
	}
};

spine.FlipXTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, flip, ...
	this.frames.length = frameCount * 2;
};
spine.FlipXTimeline.prototype = {
	boneIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 2;
	},
	setFrame: function (frameIndex, time, flip) {
		frameIndex *= 2;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = flip ? 1 : 0;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) {
			if (lastTime > time) this.apply(skeleton, lastTime, Number.MAX_VALUE, null, 0);
			return;
		} else if (lastTime > time) //
			lastTime = -1;
		var frameIndex = (time >= frames[frames.length - 2] ? frames.length : spine.Animation.binarySearch(frames, time, 2)) - 2;
		if (frames[frameIndex] < lastTime) return;
		skeleton.bones[this.boneIndex].flipX = frames[frameIndex + 1] != 0;
	}
};

spine.FlipYTimeline = function (frameCount) {
	this.curves = new spine.Curves(frameCount);
	this.frames = []; // time, flip, ...
	this.frames.length = frameCount * 2;
};
spine.FlipYTimeline.prototype = {
	boneIndex: 0,
	getFrameCount: function () {
		return this.frames.length / 2;
	},
	setFrame: function (frameIndex, time, flip) {
		frameIndex *= 2;
		this.frames[frameIndex] = time;
		this.frames[frameIndex + 1] = flip ? 1 : 0;
	},
	apply: function (skeleton, lastTime, time, firedEvents, alpha) {
		var frames = this.frames;
		if (time < frames[0]) {
			if (lastTime > time) this.apply(skeleton, lastTime, Number.MAX_VALUE, null, 0);
			return;
		} else if (lastTime > time) //
			lastTime = -1;
		var frameIndex = (time >= frames[frames.length - 2] ? frames.length : spine.Animation.binarySearch(frames, time, 2)) - 2;
		if (frames[frameIndex] < lastTime) return;
		skeleton.bones[this.boneIndex].flipY = frames[frameIndex + 1] != 0;
	}
};

spine.SkeletonData = function () {
	this.bones = [];
	this.slots = [];
	this.skins = [];
	this.events = [];
	this.animations = [];
	this.ikConstraints = [];
};
spine.SkeletonData.prototype = {
	name: null,
	defaultSkin: null,
	width: 0, height: 0,
	version: null, hash: null,
	/** @return May be null. */
	findBone: function (boneName) {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++)
			if (bones[i].name == boneName) return bones[i];
		return null;
	},
	/** @return -1 if the bone was not found. */
	findBoneIndex: function (boneName) {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++)
			if (bones[i].name == boneName) return i;
		return -1;
	},
	/** @return May be null. */
	findSlot: function (slotName) {
		var slots = this.slots;
		for (var i = 0, n = slots.length; i < n; i++) {
			if (slots[i].name == slotName) return slot[i];
		}
		return null;
	},
	/** @return -1 if the bone was not found. */
	findSlotIndex: function (slotName) {
		var slots = this.slots;
		for (var i = 0, n = slots.length; i < n; i++)
			if (slots[i].name == slotName) return i;
		return -1;
	},
	/** @return May be null. */
	findSkin: function (skinName) {
		var skins = this.skins;
		for (var i = 0, n = skins.length; i < n; i++)
			if (skins[i].name == skinName) return skins[i];
		return null;
	},
	/** @return May be null. */
	findEvent: function (eventName) {
		var events = this.events;
		for (var i = 0, n = events.length; i < n; i++)
			if (events[i].name == eventName) return events[i];
		return null;
	},
	/** @return May be null. */
	findAnimation: function (animationName) {
		var animations = this.animations;
		for (var i = 0, n = animations.length; i < n; i++)
			if (animations[i].name == animationName) return animations[i];
		return null;
	},
	/** @return May be null. */
	findIkConstraint: function (ikConstraintName) {
		var ikConstraints = this.ikConstraints;
		for (var i = 0, n = ikConstraints.length; i < n; i++)
			if (ikConstraints[i].name == ikConstraintName) return ikConstraints[i];
		return null;
	}
};

spine.Skeleton = function (skeletonData) {
	this.data = skeletonData;

	this.bones = [];
	for (var i = 0, n = skeletonData.bones.length; i < n; i++) {
		var boneData = skeletonData.bones[i];
		var parent = !boneData.parent ? null : this.bones[skeletonData.bones.indexOf(boneData.parent)];
		this.bones.push(new spine.Bone(boneData, this, parent));
	}

	this.slots = [];
	this.drawOrder = [];
	for (var i = 0, n = skeletonData.slots.length; i < n; i++) {
		var slotData = skeletonData.slots[i];
		var bone = this.bones[skeletonData.bones.indexOf(slotData.boneData)];
		var slot = new spine.Slot(slotData, bone);
		this.slots.push(slot);
		this.drawOrder.push(slot);
	}
	
	this.ikConstraints = [];
	for (var i = 0, n = skeletonData.ikConstraints.length; i < n; i++)
		this.ikConstraints.push(new spine.IkConstraint(skeletonData.ikConstraints[i], this));

	this.boneCache = [];
	this.updateCache();
};
spine.Skeleton.prototype = {
	x: 0, y: 0,
	skin: null,
	r: 1, g: 1, b: 1, a: 1,
	time: 0,
	flipX: false, flipY: false,
	/** Caches information about bones and IK constraints. Must be called if bones or IK constraints are added or removed. */
	updateCache: function () {
		var ikConstraints = this.ikConstraints;
		var ikConstraintsCount = ikConstraints.length;

		var arrayCount = ikConstraintsCount + 1;
		var boneCache = this.boneCache;
		if (boneCache.length > arrayCount) boneCache.length = arrayCount;
		for (var i = 0, n = boneCache.length; i < n; i++)
			boneCache[i].length = 0;
		while (boneCache.length < arrayCount)
			boneCache[boneCache.length] = [];

		var nonIkBones = boneCache[0];
		var bones = this.bones;

		outer:
		for (var i = 0, n = bones.length; i < n; i++) {
			var bone = bones[i];
			var current = bone;
			do {
				for (var ii = 0; ii < ikConstraintsCount; ii++) {
					var ikConstraint = ikConstraints[ii];
					var parent = ikConstraint.bones[0];
					var child= ikConstraint.bones[ikConstraint.bones.length - 1];
					while (true) {
						if (current == child) {
							boneCache[ii].push(bone);
							boneCache[ii + 1].push(bone);
							continue outer;
						}
						if (child == parent) break;
						child = child.parent;
					}
				}
				current = current.parent;
			} while (current);
			nonIkBones[nonIkBones.length] = bone;
		}
	},
	/** Updates the world transform for each bone. */
	updateWorldTransform: function () {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++) {
			var bone = bones[i];
			bone.rotationIK = bone.rotation;
		}
		var i = 0, last = this.boneCache.length - 1;
		while (true) {
			var cacheBones = this.boneCache[i];
			for (var ii = 0, nn = cacheBones.length; ii < nn; ii++)
				cacheBones[ii].updateWorldTransform();
			if (i == last) break;
			this.ikConstraints[i].apply();
			i++;
		}
	},
	/** Sets the bones and slots to their setup pose values. */
	setToSetupPose: function () {
		this.setBonesToSetupPose();
		this.setSlotsToSetupPose();
	},
	setBonesToSetupPose: function () {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++)
			bones[i].setToSetupPose();

		var ikConstraints = this.ikConstraints;
		for (var i = 0, n = ikConstraints.length; i < n; i++) {
			var ikConstraint = ikConstraints[i];
			ikConstraint.bendDirection = ikConstraint.data.bendDirection;
			ikConstraint.mix = ikConstraint.data.mix;
		}
	},
	setSlotsToSetupPose: function () {
		var slots = this.slots;
		var drawOrder = this.drawOrder;
		for (var i = 0, n = slots.length; i < n; i++) {
			drawOrder[i] = slots[i];
			slots[i].setToSetupPose(i);
		}
	},
	/** @return May return null. */
	getRootBone: function () {
		return this.bones.length ? this.bones[0] : null;
	},
	/** @return May be null. */
	findBone: function (boneName) {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++)
			if (bones[i].data.name == boneName) return bones[i];
		return null;
	},
	/** @return -1 if the bone was not found. */
	findBoneIndex: function (boneName) {
		var bones = this.bones;
		for (var i = 0, n = bones.length; i < n; i++)
			if (bones[i].data.name == boneName) return i;
		return -1;
	},
	/** @return May be null. */
	findSlot: function (slotName) {
		var slots = this.slots;
		for (var i = 0, n = slots.length; i < n; i++)
			if (slots[i].data.name == slotName) return slots[i];
		return null;
	},
	/** @return -1 if the bone was not found. */
	findSlotIndex: function (slotName) {
		var slots = this.slots;
		for (var i = 0, n = slots.length; i < n; i++)
			if (slots[i].data.name == slotName) return i;
		return -1;
	},
	setSkinByName: function (skinName) {
		var skin = this.data.findSkin(skinName);
		if (!skin) throw "Skin not found: " + skinName;
		this.setSkin(skin);
	},
	/** Sets the skin used to look up attachments before looking in the {@link SkeletonData#getDefaultSkin() default skin}. 
	 * Attachments from the new skin are attached if the corresponding attachment from the old skin was attached. If there was 
	 * no old skin, each slot's setup mode attachment is attached from the new skin.
	 * @param newSkin May be null. */
	setSkin: function (newSkin) {
		if (newSkin) {
			if (this.skin)
				newSkin._attachAll(this, this.skin);
			else {
				var slots = this.slots;
				for (var i = 0, n = slots.length; i < n; i++) {
					var slot = slots[i];
					var name = slot.data.attachmentName;
					if (name) {
						var attachment = newSkin.getAttachment(i, name);
						if (attachment) slot.setAttachment(attachment);
					}
				}
			}
		}
		this.skin = newSkin;
	},
	/** @return May be null. */
	getAttachmentBySlotName: function (slotName, attachmentName) {
		return this.getAttachmentBySlotIndex(this.data.findSlotIndex(slotName), attachmentName);
	},
	/** @return May be null. */
	getAttachmentBySlotIndex: function (slotIndex, attachmentName) {
		if (this.skin) {
			var attachment = this.skin.getAttachment(slotIndex, attachmentName);
			if (attachment) return attachment;
		}
		if (this.data.defaultSkin) return this.data.defaultSkin.getAttachment(slotIndex, attachmentName);
		return null;
	},
	/** @param attachmentName May be null. */
	setAttachment: function (slotName, attachmentName) {
		var slots = this.slots;
		for (var i = 0, n = slots.length; i < n; i++) {
			var slot = slots[i];
			if (slot.data.name == slotName) {
				var attachment = null;
				if (attachmentName) {
					attachment = this.getAttachmentBySlotIndex(i, attachmentName);
					if (!attachment) throw "Attachment not found: " + attachmentName + ", for slot: " + slotName;
				}
				slot.setAttachment(attachment);
				return;
			}
		}
		throw "Slot not found: " + slotName;
	},
	/** @return May be null. */
	findIkConstraint: function (ikConstraintName) {
		var ikConstraints = this.ikConstraints;
		for (var i = 0, n = ikConstraints.length; i < n; i++)
			if (ikConstraints[i].data.name == ikConstraintName) return ikConstraints[i];
		return null;
	},
	update: function (delta) {
		this.time += delta;
	}
};

spine.EventData = function (name) {
	this.name = name;
};
spine.EventData.prototype = {
	intValue: 0,
	floatValue: 0,
	stringValue: null
};

spine.Event = function (data) {
	this.data = data;
};
spine.Event.prototype = {
	intValue: 0,
	floatValue: 0,
	stringValue: null
};

spine.AttachmentType = {
	region: 0,
	boundingbox: 1,
	mesh: 2,
	skinnedmesh: 3
};

spine.RegionAttachment = function (name) {
	this.name = name;
	this.offset = [];
	this.offset.length = 8;
	this.uvs = [];
	this.uvs.length = 8;
};
spine.RegionAttachment.prototype = {
	type: spine.AttachmentType.region,
	x: 0, y: 0,
	rotation: 0,
	scaleX: 1, scaleY: 1,
	width: 0, height: 0,
	r: 1, g: 1, b: 1, a: 1,
	path: null,
	rendererObject: null,
	regionOffsetX: 0, regionOffsetY: 0,
	regionWidth: 0, regionHeight: 0,
	regionOriginalWidth: 0, regionOriginalHeight: 0,
	setUVs: function (u, v, u2, v2, rotate) {
		var uvs = this.uvs;
		if (rotate) {
			uvs[2/*X2*/] = u;
			uvs[3/*Y2*/] = v2;
			uvs[4/*X3*/] = u;
			uvs[5/*Y3*/] = v;
			uvs[6/*X4*/] = u2;
			uvs[7/*Y4*/] = v;
			uvs[0/*X1*/] = u2;
			uvs[1/*Y1*/] = v2;
		} else {
			uvs[0/*X1*/] = u;
			uvs[1/*Y1*/] = v2;
			uvs[2/*X2*/] = u;
			uvs[3/*Y2*/] = v;
			uvs[4/*X3*/] = u2;
			uvs[5/*Y3*/] = v;
			uvs[6/*X4*/] = u2;
			uvs[7/*Y4*/] = v2;
		}
	},
	updateOffset: function () {
		var regionScaleX = this.width / this.regionOriginalWidth * this.scaleX;
		var regionScaleY = this.height / this.regionOriginalHeight * this.scaleY;
		var localX = -this.width / 2 * this.scaleX + this.regionOffsetX * regionScaleX;
		var localY = -this.height / 2 * this.scaleY + this.regionOffsetY * regionScaleY;
		var localX2 = localX + this.regionWidth * regionScaleX;
		var localY2 = localY + this.regionHeight * regionScaleY;
		var radians = this.rotation * spine.degRad;
		var cos = Math.cos(radians);
		var sin = Math.sin(radians);
		var localXCos = localX * cos + this.x;
		var localXSin = localX * sin;
		var localYCos = localY * cos + this.y;
		var localYSin = localY * sin;
		var localX2Cos = localX2 * cos + this.x;
		var localX2Sin = localX2 * sin;
		var localY2Cos = localY2 * cos + this.y;
		var localY2Sin = localY2 * sin;
		var offset = this.offset;
		offset[0/*X1*/] = localXCos - localYSin;
		offset[1/*Y1*/] = localYCos + localXSin;
		offset[2/*X2*/] = localXCos - localY2Sin;
		offset[3/*Y2*/] = localY2Cos + localXSin;
		offset[4/*X3*/] = localX2Cos - localY2Sin;
		offset[5/*Y3*/] = localY2Cos + localX2Sin;
		offset[6/*X4*/] = localX2Cos - localYSin;
		offset[7/*Y4*/] = localYCos + localX2Sin;
	},
	computeVertices: function (x, y, bone, vertices) {
		x += bone.worldX;
		y += bone.worldY;
		var m00 = bone.m00, m01 = bone.m01, m10 = bone.m10, m11 = bone.m11;
		var offset = this.offset;
		vertices[0/*X1*/] = offset[0/*X1*/] * m00 + offset[1/*Y1*/] * m01 + x;
		vertices[1/*Y1*/] = offset[0/*X1*/] * m10 + offset[1/*Y1*/] * m11 + y;
		vertices[2/*X2*/] = offset[2/*X2*/] * m00 + offset[3/*Y2*/] * m01 + x;
		vertices[3/*Y2*/] = offset[2/*X2*/] * m10 + offset[3/*Y2*/] * m11 + y;
		vertices[4/*X3*/] = offset[4/*X3*/] * m00 + offset[5/*X3*/] * m01 + x;
		vertices[5/*X3*/] = offset[4/*X3*/] * m10 + offset[5/*X3*/] * m11 + y;
		vertices[6/*X4*/] = offset[6/*X4*/] * m00 + offset[7/*Y4*/] * m01 + x;
		vertices[7/*Y4*/] = offset[6/*X4*/] * m10 + offset[7/*Y4*/] * m11 + y;
	}
};

spine.MeshAttachment = function (name) {
	this.name = name;
};
spine.MeshAttachment.prototype = {
	type: spine.AttachmentType.mesh,
	vertices: null,
	uvs: null,
	regionUVs: null,
	triangles: null,
	hullLength: 0,
	r: 1, g: 1, b: 1, a: 1,
	path: null,
	rendererObject: null,
	regionU: 0, regionV: 0, regionU2: 0, regionV2: 0, regionRotate: false,
	regionOffsetX: 0, regionOffsetY: 0,
	regionWidth: 0, regionHeight: 0,
	regionOriginalWidth: 0, regionOriginalHeight: 0,
	edges: null,
	width: 0, height: 0,
	updateUVs: function () {
		var width = this.regionU2 - this.regionU, height = this.regionV2 - this.regionV;
		var n = this.regionUVs.length;
		if (!this.uvs || this.uvs.length != n) {
            this.uvs = new spine.Float32Array(n);
		}
		if (this.regionRotate) {
			for (var i = 0; i < n; i += 2) {
                this.uvs[i] = this.regionU + this.regionUVs[i + 1] * width;
                this.uvs[i + 1] = this.regionV + height - this.regionUVs[i] * height;
			}
		} else {
			for (var i = 0; i < n; i += 2) {
                this.uvs[i] = this.regionU + this.regionUVs[i] * width;
                this.uvs[i + 1] = this.regionV + this.regionUVs[i + 1] * height;
			}
		}
	},
	computeWorldVertices: function (x, y, slot, worldVertices) {
		var bone = slot.bone;
		x += bone.worldX;
		y += bone.worldY;
		var m00 = bone.m00, m01 = bone.m01, m10 = bone.m10, m11 = bone.m11;
		var vertices = this.vertices;
		var verticesCount = vertices.length;
		if (slot.attachmentVertices.length == verticesCount) vertices = slot.attachmentVertices;
		for (var i = 0; i < verticesCount; i += 2) {
			var vx = vertices[i];
			var vy = vertices[i + 1];
			worldVertices[i] = vx * m00 + vy * m01 + x;
			worldVertices[i + 1] = vx * m10 + vy * m11 + y;
		}
	}
};

spine.SkinnedMeshAttachment = function (name) {
	this.name = name;
};
spine.SkinnedMeshAttachment.prototype = {
	type: spine.AttachmentType.skinnedmesh,
	bones: null,
	weights: null,
	uvs: null,
	regionUVs: null,
	triangles: null,
	hullLength: 0,
	r: 1, g: 1, b: 1, a: 1,
	path: null,
	rendererObject: null,
	regionU: 0, regionV: 0, regionU2: 0, regionV2: 0, regionRotate: false,
	regionOffsetX: 0, regionOffsetY: 0,
	regionWidth: 0, regionHeight: 0,
	regionOriginalWidth: 0, regionOriginalHeight: 0,
	edges: null,
	width: 0, height: 0,
	updateUVs: function (u, v, u2, v2, rotate) {
		var width = this.regionU2 - this.regionU, height = this.regionV2 - this.regionV;
		var n = this.regionUVs.length;
		if (!this.uvs || this.uvs.length != n) {
            this.uvs = new spine.Float32Array(n);
		}
		if (this.regionRotate) {
			for (var i = 0; i < n; i += 2) {
                this.uvs[i] = this.regionU + this.regionUVs[i + 1] * width;
                this.uvs[i + 1] = this.regionV + height - this.regionUVs[i] * height;
			}
		} else {
			for (var i = 0; i < n; i += 2) {
                this.uvs[i] = this.regionU + this.regionUVs[i] * width;
                this.uvs[i + 1] = this.regionV + this.regionUVs[i + 1] * height;
			}
		}
	},
	computeWorldVertices: function (x, y, slot, worldVertices) {
		var skeletonBones = slot.bone.skeleton.bones;
		var weights = this.weights;
		var bones = this.bones;

		var w = 0, v = 0, b = 0, f = 0, n = bones.length, nn;
		var wx, wy, bone, vx, vy, weight;
		if (!slot.attachmentVertices.length) {
			for (; v < n; w += 2) {
				wx = 0;
				wy = 0;
				nn = bones[v++] + v;
				for (; v < nn; v++, b += 3) {
					bone = skeletonBones[bones[v]];
					vx = weights[b];
					vy = weights[b + 1];
					weight = weights[b + 2];
					wx += (vx * bone.m00 + vy * bone.m01 + bone.worldX) * weight;
					wy += (vx * bone.m10 + vy * bone.m11 + bone.worldY) * weight;
				}
				worldVertices[w] = wx + x;
				worldVertices[w + 1] = wy + y;
			}
		} else {
			var ffd = slot.attachmentVertices;
			for (; v < n; w += 2) {
				wx = 0;
				wy = 0;
				nn = bones[v++] + v;
				for (; v < nn; v++, b += 3, f += 2) {
					bone = skeletonBones[bones[v]];
					vx = weights[b] + ffd[f];
					vy = weights[b + 1] + ffd[f + 1];
					weight = weights[b + 2];
					wx += (vx * bone.m00 + vy * bone.m01 + bone.worldX) * weight;
					wy += (vx * bone.m10 + vy * bone.m11 + bone.worldY) * weight;
				}
				worldVertices[w] = wx + x;
				worldVertices[w + 1] = wy + y;
			}
		}
	}
};

spine.BoundingBoxAttachment = function (name) {
	this.name = name;
	this.vertices = [];
};
spine.BoundingBoxAttachment.prototype = {
	type: spine.AttachmentType.boundingbox,
	computeWorldVertices: function (x, y, bone, worldVertices) {
		x += bone.worldX;
		y += bone.worldY;
		var m00 = bone.m00, m01 = bone.m01, m10 = bone.m10, m11 = bone.m11;
		var vertices = this.vertices;
		for (var i = 0, n = vertices.length; i < n; i += 2) {
			var px = vertices[i];
			var py = vertices[i + 1];
			worldVertices[i] = px * m00 + py * m01 + x;
			worldVertices[i + 1] = px * m10 + py * m11 + y;
		}
	}
};

spine.AnimationStateData = function (skeletonData) {
	this.skeletonData = skeletonData;
	this.animationToMixTime = {};
};
spine.AnimationStateData.prototype = {
	defaultMix: 0,
	setMixByName: function (fromName, toName, duration) {
		var from = this.skeletonData.findAnimation(fromName);
		if (!from) throw "Animation not found: " + fromName;
		var to = this.skeletonData.findAnimation(toName);
		if (!to) throw "Animation not found: " + toName;
		this.setMix(from, to, duration);
	},
	setMix: function (from, to, duration) {
		this.animationToMixTime[from.name + ":" + to.name] = duration;
	},
	getMix: function (from, to) {
		var key = from.name + ":" + to.name;
		return this.animationToMixTime.hasOwnProperty(key) ? this.animationToMixTime[key] : this.defaultMix;
	}
};

spine.TrackEntry = function () {};
spine.TrackEntry.prototype = {
	next: null, previous: null,
	animation: null,
	loop: false,
	delay: 0, time: 0, lastTime: -1, endTime: 0,
	timeScale: 1,
	mixTime: 0, mixDuration: 0, mix: 1,
	onStart: null, onEnd: null, onComplete: null, onEvent: null
};

spine.AnimationState = function (stateData) {
	this.data = stateData;
	this.tracks = [];
	this.events = [];
};
spine.AnimationState.prototype = {
	onStart: null,
	onEnd: null,
	onComplete: null,
	onEvent: null,
	timeScale: 1,
	update: function (delta) {
		delta *= this.timeScale;
		for (var i = 0; i < this.tracks.length; i++) {
			var current = this.tracks[i];
			if (!current) continue;

			current.time += delta * current.timeScale;
			if (current.previous) {
				var previousDelta = delta * current.previous.timeScale;
				current.previous.time += previousDelta;
				current.mixTime += previousDelta;
			}

			var next = current.next;
			if (next) {
				next.time = current.lastTime - next.delay;
				if (next.time >= 0) this.setCurrent(i, next);
			} else {
				// End non-looping animation when it reaches its end time and there is no next entry.
				if (!current.loop && current.lastTime >= current.endTime) this.clearTrack(i);
			}
		}
	},
	apply: function (skeleton) {
		for (var i = 0; i < this.tracks.length; i++) {
			var current = this.tracks[i];
			if (!current) continue;

			this.events.length = 0;

			var time = current.time;
			var lastTime = current.lastTime;
			var endTime = current.endTime;
			var loop = current.loop;
			if (!loop && time > endTime) time = endTime;

			var previous = current.previous;
			if (!previous) {
				if (current.mix == 1)
					current.animation.apply(skeleton, current.lastTime, time, loop, this.events);
				else
					current.animation.mix(skeleton, current.lastTime, time, loop, this.events, current.mix);
			} else {
				var previousTime = previous.time;
				if (!previous.loop && previousTime > previous.endTime) previousTime = previous.endTime;
				previous.animation.apply(skeleton, previousTime, previousTime, previous.loop, null);

				var alpha = current.mixTime / current.mixDuration * current.mix;
				if (alpha >= 1) {
					alpha = 1;
					current.previous = null;
				}
				current.animation.mix(skeleton, current.lastTime, time, loop, this.events, alpha);
			}

			for (var ii = 0, nn = this.events.length; ii < nn; ii++) {
				var event = this.events[ii];
				if (current.onEvent) current.onEvent(i, event);
				if (this.onEvent) this.onEvent(i, event);
			}

			// Check if completed the animation or a loop iteration.
			if (loop ? (lastTime % endTime > time % endTime) : (lastTime < endTime && time >= endTime)) {
				var count = Math.floor(time / endTime);
				if (current.onComplete) current.onComplete(i, count);
				if (this.onComplete) this.onComplete(i, count);
			}

			current.lastTime = current.time;
		}
	},
	clearTracks: function () {
		for (var i = 0, n = this.tracks.length; i < n; i++)
			this.clearTrack(i);
		this.tracks.length = 0; 
	},
	clearTrack: function (trackIndex) {
		if (trackIndex >= this.tracks.length) return;
		var current = this.tracks[trackIndex];
		if (!current) return;

		if (current.onEnd) current.onEnd(trackIndex);
		if (this.onEnd) this.onEnd(trackIndex);

		this.tracks[trackIndex] = null;
	},
	_expandToIndex: function (index) {
		if (index < this.tracks.length) return this.tracks[index];
		while (index >= this.tracks.length)
			this.tracks.push(null);
		return null;
	},
	setCurrent: function (index, entry) {
		var current = this._expandToIndex(index);
		if (current) {
			var previous = current.previous;
			current.previous = null;

			if (current.onEnd) current.onEnd(index);
			if (this.onEnd) this.onEnd(index);

			entry.mixDuration = this.data.getMix(current.animation, entry.animation);
			if (entry.mixDuration > 0) {
				entry.mixTime = 0;
				// If a mix is in progress, mix from the closest animation.
				if (previous && current.mixTime / current.mixDuration < 0.5)
					entry.previous = previous;
				else
					entry.previous = current;
			}
		}

		this.tracks[index] = entry;

		if (entry.onStart) entry.onStart(index);
		if (this.onStart) this.onStart(index);
	},
	setAnimationByName: function (trackIndex, animationName, loop) {
		var animation = this.data.skeletonData.findAnimation(animationName);
		if (!animation) throw "Animation not found: " + animationName;
		return this.setAnimation(trackIndex, animation, loop);
	},
	/** Set the current animation. Any queued animations are cleared. */
	setAnimation: function (trackIndex, animation, loop) {
		var entry = new spine.TrackEntry();
		entry.animation = animation;
		entry.loop = loop;
		entry.endTime = animation.duration;
		this.setCurrent(trackIndex, entry);
		return entry;
	},
	addAnimationByName: function (trackIndex, animationName, loop, delay) {
		var animation = this.data.skeletonData.findAnimation(animationName);
		if (!animation) throw "Animation not found: " + animationName;
		return this.addAnimation(trackIndex, animation, loop, delay);
	},
	/** Adds an animation to be played delay seconds after the current or last queued animation.
	 * @param delay May be <= 0 to use duration of previous animation minus any mix duration plus the negative delay. */
	addAnimation: function (trackIndex, animation, loop, delay) {
		var entry = new spine.TrackEntry();
		entry.animation = animation;
		entry.loop = loop;
		entry.endTime = animation.duration;

		var last = this._expandToIndex(trackIndex);
		if (last) {
			while (last.next)
				last = last.next;
			last.next = entry;
		} else
			this.tracks[trackIndex] = entry;

		if (delay <= 0) {
			if (last)
				delay += last.endTime - this.data.getMix(last.animation, animation);
			else
				delay = 0;
		}
		entry.delay = delay;

		return entry;
	},
	/** May be null. */
	getCurrent: function (trackIndex) {
		if (trackIndex >= this.tracks.length) return null;
		return this.tracks[trackIndex];
	}
};

spine.SkeletonJson = function (attachmentLoader) {
	this.attachmentLoader = attachmentLoader;
};
spine.SkeletonJson.prototype = {
	scale: 1,
	readSkeletonData: function (root, name) {
		var skeletonData = new spine.SkeletonData();
		skeletonData.name = name;

		// Skeleton.
		var skeletonMap = root["skeleton"];
		if (skeletonMap) {
			skeletonData.hash = skeletonMap["hash"];
			skeletonData.version = skeletonMap["spine"];
			skeletonData.width = skeletonMap["width"] || 0;
			skeletonData.height = skeletonMap["height"] || 0;
		}

		// Bones.
		var bones = root["bones"];
		for (var i = 0, n = bones.length; i < n; i++) {
			var boneMap = bones[i];
			var parent = null;
			if (boneMap["parent"]) {
				parent = skeletonData.findBone(boneMap["parent"]);
				if (!parent) throw "Parent bone not found: " + boneMap["parent"];
			}
			var boneData = new spine.BoneData(boneMap["name"], parent);
			boneData.length = (boneMap["length"] || 0) * this.scale;
			boneData.x = (boneMap["x"] || 0) * this.scale;
			boneData.y = (boneMap["y"] || 0) * this.scale;
			boneData.rotation = (boneMap["rotation"] || 0);
			boneData.scaleX = boneMap.hasOwnProperty("scaleX") ? boneMap["scaleX"] : 1;
			boneData.scaleY = boneMap.hasOwnProperty("scaleY") ? boneMap["scaleY"] : 1;
			boneData.inheritScale = boneMap.hasOwnProperty("inheritScale") ? boneMap["inheritScale"] : true;
			boneData.inheritRotation = boneMap.hasOwnProperty("inheritRotation") ? boneMap["inheritRotation"] : true;
			skeletonData.bones.push(boneData);
		}

		// IK constraints.
		var ik = root["ik"];
		if (ik) {
			for (var i = 0, n = ik.length; i < n; i++) {
				var ikMap = ik[i];
				var ikConstraintData = new spine.IkConstraintData(ikMap["name"]);

				var bones = ikMap["bones"];
				for (var ii = 0, nn = bones.length; ii < nn; ii++) {
					var bone = skeletonData.findBone(bones[ii]);
					if (!bone) throw "IK bone not found: " + bones[ii];
					ikConstraintData.bones.push(bone);
				}

				ikConstraintData.target = skeletonData.findBone(ikMap["target"]);
				if (!ikConstraintData.target) throw "Target bone not found: " + ikMap["target"];

				ikConstraintData.bendDirection = (!ikMap.hasOwnProperty("bendPositive") || ikMap["bendPositive"]) ? 1 : -1;
				ikConstraintData.mix = ikMap.hasOwnProperty("mix") ? ikMap["mix"] : 1;

				skeletonData.ikConstraints.push(ikConstraintData);
			}
		}

		// Slots.
		var slots = root["slots"];
		for (var i = 0, n = slots.length; i < n; i++) {
			var slotMap = slots[i];
			var boneData = skeletonData.findBone(slotMap["bone"]);
			if (!boneData) throw "Slot bone not found: " + slotMap["bone"];
			var slotData = new spine.SlotData(slotMap["name"], boneData);

			var color = slotMap["color"];
			if (color) {
				slotData.r = this.toColor(color, 0);
				slotData.g = this.toColor(color, 1);
				slotData.b = this.toColor(color, 2);
				slotData.a = this.toColor(color, 3);
			}

			slotData.attachmentName = slotMap["attachment"];
			slotData.blendMode = spine.BlendMode[slotMap["blend"] || "normal"];

			skeletonData.slots.push(slotData);
		}

		// Skins.
		var skins = root["skins"];
		for (var skinName in skins) {
			if (!skins.hasOwnProperty(skinName)) continue;
			var skinMap = skins[skinName];
			var skin = new spine.Skin(skinName);
			for (var slotName in skinMap) {
				if (!skinMap.hasOwnProperty(slotName)) continue;
				var slotIndex = skeletonData.findSlotIndex(slotName);
				var slotEntry = skinMap[slotName];
				for (var attachmentName in slotEntry) {
					if (!slotEntry.hasOwnProperty(attachmentName)) continue;
					var attachment = this.readAttachment(skin, attachmentName, slotEntry[attachmentName]);
					if (attachment) skin.addAttachment(slotIndex, attachmentName, attachment);
				}
			}
			skeletonData.skins.push(skin);
			if (skin.name == "default") skeletonData.defaultSkin = skin;
		}

		// Events.
		var events = root["events"];
		for (var eventName in events) {
			if (!events.hasOwnProperty(eventName)) continue;
			var eventMap = events[eventName];
			var eventData = new spine.EventData(eventName);
			eventData.intValue = eventMap["int"] || 0;
			eventData.floatValue = eventMap["float"] || 0;
			eventData.stringValue = eventMap["string"] || null;
			skeletonData.events.push(eventData);
		}

		// Animations.
		var animations = root["animations"];
		for (var animationName in animations) {
			if (!animations.hasOwnProperty(animationName)) continue;
			this.readAnimation(animationName, animations[animationName], skeletonData);
		}

		return skeletonData;
	},
	readAttachment: function (skin, name, map) {
		name = map["name"] || name;

		var type = spine.AttachmentType[map["type"] || "region"];
		var path = map["path"] || name;
		
		var scale = this.scale;
		if (type == spine.AttachmentType.region) {
			var region = this.attachmentLoader.newRegionAttachment(skin, name, path);
			if (!region) return null;
			region.path = path;
			region.x = (map["x"] || 0) * scale;
			region.y = (map["y"] || 0) * scale;
			region.scaleX = map.hasOwnProperty("scaleX") ? map["scaleX"] : 1;
			region.scaleY = map.hasOwnProperty("scaleY") ? map["scaleY"] : 1;
			region.rotation = map["rotation"] || 0;
			region.width = (map["width"] || 0) * scale;
			region.height = (map["height"] || 0) * scale;

			var color = map["color"];
			if (color) {
				region.r = this.toColor(color, 0);
				region.g = this.toColor(color, 1);
				region.b = this.toColor(color, 2);
				region.a = this.toColor(color, 3);
			}

			region.updateOffset();
			return region;
		} else if (type == spine.AttachmentType.mesh) {
			var mesh = this.attachmentLoader.newMeshAttachment(skin, name, path);
			if (!mesh) return null;
			mesh.path = path; 
			mesh.vertices = this.getFloatArray(map, "vertices", scale);
			mesh.triangles = this.getIntArray(map, "triangles");
			mesh.regionUVs = this.getFloatArray(map, "uvs", 1);
			mesh.updateUVs();

			color = map["color"];
			if (color) {
				mesh.r = this.toColor(color, 0);
				mesh.g = this.toColor(color, 1);
				mesh.b = this.toColor(color, 2);
				mesh.a = this.toColor(color, 3);
			}

			mesh.hullLength = (map["hull"] || 0) * 2;
			if (map["edges"]) mesh.edges = this.getIntArray(map, "edges");
			mesh.width = (map["width"] || 0) * scale;
			mesh.height = (map["height"] || 0) * scale;
			return mesh;
		} else if (type == spine.AttachmentType.skinnedmesh) {
			var mesh = this.attachmentLoader.newSkinnedMeshAttachment(skin, name, path);
			if (!mesh) return null;
			mesh.path = path;

			var uvs = this.getFloatArray(map, "uvs", 1);
			var vertices = this.getFloatArray(map, "vertices", 1);
			var weights = [];
			var bones = [];
			for (var i = 0, n = vertices.length; i < n; ) {
				var boneCount = vertices[i++] | 0;
				bones[bones.length] = boneCount;
				for (var nn = i + boneCount * 4; i < nn; ) {
					bones[bones.length] = vertices[i];
					weights[weights.length] = vertices[i + 1] * scale;
					weights[weights.length] = vertices[i + 2] * scale;
					weights[weights.length] = vertices[i + 3];
					i += 4;
				}
			}
			mesh.bones = bones;
			mesh.weights = weights;
			mesh.triangles = this.getIntArray(map, "triangles");
			mesh.regionUVs = uvs;
			mesh.updateUVs();
			
			color = map["color"];
			if (color) {
				mesh.r = this.toColor(color, 0);
				mesh.g = this.toColor(color, 1);
				mesh.b = this.toColor(color, 2);
				mesh.a = this.toColor(color, 3);
			}
			
			mesh.hullLength = (map["hull"] || 0) * 2;
			if (map["edges"]) mesh.edges = this.getIntArray(map, "edges");
			mesh.width = (map["width"] || 0) * scale;
			mesh.height = (map["height"] || 0) * scale;
			return mesh;
		} else if (type == spine.AttachmentType.boundingbox) {
			var attachment = this.attachmentLoader.newBoundingBoxAttachment(skin, name);
			var vertices = map["vertices"];
			for (var i = 0, n = vertices.length; i < n; i++)
				attachment.vertices.push(vertices[i] * scale);
			return attachment;
		}
		throw "Unknown attachment type: " + type;
	},
	readAnimation: function (name, map, skeletonData) {
		var timelines = [];
		var duration = 0;

		var slots = map["slots"];
		for (var slotName in slots) {
			if (!slots.hasOwnProperty(slotName)) continue;
			var slotMap = slots[slotName];
			var slotIndex = skeletonData.findSlotIndex(slotName);

			for (var timelineName in slotMap) {
				if (!slotMap.hasOwnProperty(timelineName)) continue;
				var values = slotMap[timelineName];
				if (timelineName == "color") {
					var timeline = new spine.ColorTimeline(values.length);
					timeline.slotIndex = slotIndex;

					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						var color = valueMap["color"];
						var r = this.toColor(color, 0);
						var g = this.toColor(color, 1);
						var b = this.toColor(color, 2);
						var a = this.toColor(color, 3);
						timeline.setFrame(frameIndex, valueMap["time"], r, g, b, a);
						this.readCurve(timeline, frameIndex, valueMap);
						frameIndex++;
					}
					timelines.push(timeline);
					duration = Math.max(duration, timeline.frames[timeline.getFrameCount() * 5 - 5]);

				} else if (timelineName == "attachment") {
					var timeline = new spine.AttachmentTimeline(values.length);
					timeline.slotIndex = slotIndex;

					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						timeline.setFrame(frameIndex++, valueMap["time"], valueMap["name"]);
					}
					timelines.push(timeline);
					duration = Math.max(duration, timeline.frames[timeline.getFrameCount() - 1]);

				} else
					throw "Invalid timeline type for a slot: " + timelineName + " (" + slotName + ")";
			}
		}

		var bones = map["bones"];
		for (var boneName in bones) {
			if (!bones.hasOwnProperty(boneName)) continue;
			var boneIndex = skeletonData.findBoneIndex(boneName);
			if (boneIndex == -1) throw "Bone not found: " + boneName;
			var boneMap = bones[boneName];

			for (var timelineName in boneMap) {
				if (!boneMap.hasOwnProperty(timelineName)) continue;
				var values = boneMap[timelineName];
				if (timelineName == "rotate") {
					var timeline = new spine.RotateTimeline(values.length);
					timeline.boneIndex = boneIndex;

					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						timeline.setFrame(frameIndex, valueMap["time"], valueMap["angle"]);
						this.readCurve(timeline, frameIndex, valueMap);
						frameIndex++;
					}
					timelines.push(timeline);
					duration = Math.max(duration, timeline.frames[timeline.getFrameCount() * 2 - 2]);

				} else if (timelineName == "translate" || timelineName == "scale") {
					var timeline;
					var timelineScale = 1;
					if (timelineName == "scale")
						timeline = new spine.ScaleTimeline(values.length);
					else {
						timeline = new spine.TranslateTimeline(values.length);
						timelineScale = this.scale;
					}
					timeline.boneIndex = boneIndex;

					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						var x = (valueMap["x"] || 0) * timelineScale;
						var y = (valueMap["y"] || 0) * timelineScale;
						timeline.setFrame(frameIndex, valueMap["time"], x, y);
						this.readCurve(timeline, frameIndex, valueMap);
						frameIndex++;
					}
					timelines.push(timeline);
					duration = Math.max(duration, timeline.frames[timeline.getFrameCount() * 3 - 3]);

				} else if (timelineName == "flipX" || timelineName == "flipY") {
					var x = timelineName == "flipX";
					var timeline = x ? new spine.FlipXTimeline(values.length) : new spine.FlipYTimeline(values.length);
					timeline.boneIndex = boneIndex;

					var field = x ? "x" : "y";
					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						timeline.setFrame(frameIndex, valueMap["time"], valueMap[field] || false);
						frameIndex++;
					}
					timelines.push(timeline);
					duration = Math.max(duration, timeline.frames[timeline.getFrameCount() * 2 - 2]);
				} else
					throw "Invalid timeline type for a bone: " + timelineName + " (" + boneName + ")";
			}
		}

		var ikMap = map["ik"];
		for (var ikConstraintName in ikMap) {
			if (!ikMap.hasOwnProperty(ikConstraintName)) continue;
			var ikConstraint = skeletonData.findIkConstraint(ikConstraintName);
			var values = ikMap[ikConstraintName];
			var timeline = new spine.IkConstraintTimeline(values.length);
			timeline.ikConstraintIndex = skeletonData.ikConstraints.indexOf(ikConstraint);
			var frameIndex = 0;
			for (var i = 0, n = values.length; i < n; i++) {
				var valueMap = values[i];
				var mix = valueMap.hasOwnProperty("mix") ? valueMap["mix"] : 1;
				var bendDirection = (!valueMap.hasOwnProperty("bendPositive") || valueMap["bendPositive"]) ? 1 : -1;
				timeline.setFrame(frameIndex, valueMap["time"], mix, bendDirection);
				this.readCurve(timeline, frameIndex, valueMap);
				frameIndex++;
			}
			timelines.push(timeline);
			duration = Math.max(duration, timeline.frames[timeline.frameCount * 3 - 3]);
		}

		var ffd = map["ffd"];
		for (var skinName in ffd) {
			var skin = skeletonData.findSkin(skinName);
			var slotMap = ffd[skinName];
			for (slotName in slotMap) {
				var slotIndex = skeletonData.findSlotIndex(slotName);
				var meshMap = slotMap[slotName];
				for (var meshName in meshMap) {
					var values = meshMap[meshName];
					var timeline = new spine.FfdTimeline(values.length);
					var attachment = skin.getAttachment(slotIndex, meshName);
					if (!attachment) throw "FFD attachment not found: " + meshName;
					timeline.slotIndex = slotIndex;
					timeline.attachment = attachment;
					
					var isMesh = attachment.type == spine.AttachmentType.mesh;
					var vertexCount;
					if (isMesh)
						vertexCount = attachment.vertices.length;
					else
						vertexCount = attachment.weights.length / 3 * 2;

					var frameIndex = 0;
					for (var i = 0, n = values.length; i < n; i++) {
						var valueMap = values[i];
						var vertices;
						if (!valueMap["vertices"]) {
							if (isMesh)
								vertices = attachment.vertices;
							else {
								vertices = [];
								vertices.length = vertexCount;
							}
						} else {
							var verticesValue = valueMap["vertices"];
							var vertices = [];
							vertices.length = vertexCount;
							var start = valueMap["offset"] || 0;
							var nn = verticesValue.length;
							if (this.scale == 1) {
								for (var ii = 0; ii < nn; ii++)
									vertices[ii + start] = verticesValue[ii];
							} else {
								for (var ii = 0; ii < nn; ii++)
									vertices[ii + start] = verticesValue[ii] * this.scale;
							}
							if (isMesh) {
								var meshVertices = attachment.vertices;
								for (var ii = 0, nn = vertices.length; ii < nn; ii++)
									vertices[ii] += meshVertices[ii];
							}
						}
						
						timeline.setFrame(frameIndex, valueMap["time"], vertices);
						this.readCurve(timeline, frameIndex, valueMap);
						frameIndex++;
					}
					timelines[timelines.length] = timeline;
					duration = Math.max(duration, timeline.frames[timeline.frameCount - 1]);
				}
			}
		}

		var drawOrderValues = map["drawOrder"];
		if (!drawOrderValues) drawOrderValues = map["draworder"];
		if (drawOrderValues) {
			var timeline = new spine.DrawOrderTimeline(drawOrderValues.length);
			var slotCount = skeletonData.slots.length;
			var frameIndex = 0;
			for (var i = 0, n = drawOrderValues.length; i < n; i++) {
				var drawOrderMap = drawOrderValues[i];
				var drawOrder = null;
				if (drawOrderMap["offsets"]) {
					drawOrder = [];
					drawOrder.length = slotCount;
					for (var ii = slotCount - 1; ii >= 0; ii--)
						drawOrder[ii] = -1;
					var offsets = drawOrderMap["offsets"];
					var unchanged = [];
					unchanged.length = slotCount - offsets.length;
					var originalIndex = 0, unchangedIndex = 0;
					for (var ii = 0, nn = offsets.length; ii < nn; ii++) {
						var offsetMap = offsets[ii];
						var slotIndex = skeletonData.findSlotIndex(offsetMap["slot"]);
						if (slotIndex == -1) throw "Slot not found: " + offsetMap["slot"];
						// Collect unchanged items.
						while (originalIndex != slotIndex)
							unchanged[unchangedIndex++] = originalIndex++;
						// Set changed items.
						drawOrder[originalIndex + offsetMap["offset"]] = originalIndex++;
					}
					// Collect remaining unchanged items.
					while (originalIndex < slotCount)
						unchanged[unchangedIndex++] = originalIndex++;
					// Fill in unchanged items.
					for (var ii = slotCount - 1; ii >= 0; ii--)
						if (drawOrder[ii] == -1) drawOrder[ii] = unchanged[--unchangedIndex];
				}
				timeline.setFrame(frameIndex++, drawOrderMap["time"], drawOrder);
			}
			timelines.push(timeline);
			duration = Math.max(duration, timeline.frames[timeline.getFrameCount() - 1]);
		}

		var events = map["events"];
		if (events) {
			var timeline = new spine.EventTimeline(events.length);
			var frameIndex = 0;
			for (var i = 0, n = events.length; i < n; i++) {
				var eventMap = events[i];
				var eventData = skeletonData.findEvent(eventMap["name"]);
				if (!eventData) throw "Event not found: " + eventMap["name"];
				var event = new spine.Event(eventData);
				event.intValue = eventMap.hasOwnProperty("int") ? eventMap["int"] : eventData.intValue;
				event.floatValue = eventMap.hasOwnProperty("float") ? eventMap["float"] : eventData.floatValue;
				event.stringValue = eventMap.hasOwnProperty("string") ? eventMap["string"] : eventData.stringValue;
				timeline.setFrame(frameIndex++, eventMap["time"], event);
			}
			timelines.push(timeline);
			duration = Math.max(duration, timeline.frames[timeline.getFrameCount() - 1]);
		}

		skeletonData.animations.push(new spine.Animation(name, timelines, duration));
	},
	readCurve: function (timeline, frameIndex, valueMap) {
		var curve = valueMap["curve"];
		if (!curve) 
			timeline.curves.setLinear(frameIndex);
		else if (curve == "stepped")
			timeline.curves.setStepped(frameIndex);
		else if (curve instanceof Array)
			timeline.curves.setCurve(frameIndex, curve[0], curve[1], curve[2], curve[3]);
	},
	toColor: function (hexString, colorIndex) {
		if (hexString.length != 8) throw "Color hexidecimal length must be 8, recieved: " + hexString;
		return parseInt(hexString.substring(colorIndex * 2, (colorIndex * 2) + 2), 16) / 255;
	},
	getFloatArray: function (map, name, scale) {
		var list = map[name];
		var values = new spine.Float32Array(list.length);
		var i = 0, n = list.length;
		if (scale == 1) {
			for (; i < n; i++)
				values[i] = list[i];
		} else {
			for (; i < n; i++)
				values[i] = list[i] * scale;
		}
		return values;
	},
	getIntArray: function (map, name) {
		var list = map[name];
		var values = new spine.Uint16Array(list.length);
		for (var i = 0, n = list.length; i < n; i++)
			values[i] = list[i] | 0;
		return values;
	}
};

spine.Atlas = function (atlasText, textureLoader) {
	this.textureLoader = textureLoader;
	this.pages = [];
	this.regions = [];

	var reader = new spine.AtlasReader(atlasText);
	var tuple = [];
	tuple.length = 4;
	var page = null;
	while (true) {
		var line = reader.readLine();
		if (line === null) break;
		line = reader.trim(line);
		if (!line.length)
			page = null;
		else if (!page) {
			page = new spine.AtlasPage();
			page.name = line;

			if (reader.readTuple(tuple) == 2) { // size is only optional for an atlas packed with an old TexturePacker.
				page.width = parseInt(tuple[0]);
				page.height = parseInt(tuple[1]);
				reader.readTuple(tuple);
			}
			page.format = spine.Atlas.Format[tuple[0]];

			reader.readTuple(tuple);
			page.minFilter = spine.Atlas.TextureFilter[tuple[0]];
			page.magFilter = spine.Atlas.TextureFilter[tuple[1]];

			var direction = reader.readValue();
			page.uWrap = spine.Atlas.TextureWrap.clampToEdge;
			page.vWrap = spine.Atlas.TextureWrap.clampToEdge;
			if (direction == "x")
				page.uWrap = spine.Atlas.TextureWrap.repeat;
			else if (direction == "y")
				page.vWrap = spine.Atlas.TextureWrap.repeat;
			else if (direction == "xy")
				page.uWrap = page.vWrap = spine.Atlas.TextureWrap.repeat;

			textureLoader.load(page, line, this);

			this.pages.push(page);

		} else {
			var region = new spine.AtlasRegion();
			region.name = line;
			region.page = page;

			region.rotate = reader.readValue() == "true";

			reader.readTuple(tuple);
			var x = parseInt(tuple[0]);
			var y = parseInt(tuple[1]);

			reader.readTuple(tuple);
			var width = parseInt(tuple[0]);
			var height = parseInt(tuple[1]);

			region.u = x / page.width;
			region.v = y / page.height;
			if (region.rotate) {
				region.u2 = (x + height) / page.width;
				region.v2 = (y + width) / page.height;
			} else {
				region.u2 = (x + width) / page.width;
				region.v2 = (y + height) / page.height;
			}
			region.x = x;
			region.y = y;
			region.width = Math.abs(width);
			region.height = Math.abs(height);

			if (reader.readTuple(tuple) == 4) { // split is optional
				region.splits = [parseInt(tuple[0]), parseInt(tuple[1]), parseInt(tuple[2]), parseInt(tuple[3])];

				if (reader.readTuple(tuple) == 4) { // pad is optional, but only present with splits
					region.pads = [parseInt(tuple[0]), parseInt(tuple[1]), parseInt(tuple[2]), parseInt(tuple[3])];

					reader.readTuple(tuple);
				}
			}

			region.originalWidth = parseInt(tuple[0]);
			region.originalHeight = parseInt(tuple[1]);

			reader.readTuple(tuple);
			region.offsetX = parseInt(tuple[0]);
			region.offsetY = parseInt(tuple[1]);

			region.index = parseInt(reader.readValue());

			this.regions.push(region);
		}
	}
};
spine.Atlas.prototype = {
	findRegion: function (name) {
		var regions = this.regions;
		for (var i = 0, n = regions.length; i < n; i++)
			if (regions[i].name == name) return regions[i];
		return null;
	},
	dispose: function () {
		var pages = this.pages;
		for (var i = 0, n = pages.length; i < n; i++)
			this.textureLoader.unload(pages[i].rendererObject);
	},
	updateUVs: function (page) {
		var regions = this.regions;
		for (var i = 0, n = regions.length; i < n; i++) {
			var region = regions[i];
			if (region.page != page) continue;
			region.u = region.x / page.width;
			region.v = region.y / page.height;
			if (region.rotate) {
				region.u2 = (region.x + region.height) / page.width;
				region.v2 = (region.y + region.width) / page.height;
			} else {
				region.u2 = (region.x + region.width) / page.width;
				region.v2 = (region.y + region.height) / page.height;
			}
		}
	}
};

spine.Atlas.Format = {
	alpha: 0,
	intensity: 1,
	luminanceAlpha: 2,
	rgb565: 3,
	rgba4444: 4,
	rgb888: 5,
	rgba8888: 6
};

spine.Atlas.TextureFilter = {
	nearest: 0,
	linear: 1,
	mipMap: 2,
	mipMapNearestNearest: 3,
	mipMapLinearNearest: 4,
	mipMapNearestLinear: 5,
	mipMapLinearLinear: 6
};

spine.Atlas.TextureWrap = {
	mirroredRepeat: 0,
	clampToEdge: 1,
	repeat: 2
};

spine.AtlasPage = function () {};
spine.AtlasPage.prototype = {
	name: null,
	format: null,
	minFilter: null,
	magFilter: null,
	uWrap: null,
	vWrap: null,
	rendererObject: null,
	width: 0,
	height: 0
};

spine.AtlasRegion = function () {};
spine.AtlasRegion.prototype = {
	page: null,
	name: null,
	x: 0, y: 0,
	width: 0, height: 0,
	u: 0, v: 0, u2: 0, v2: 0,
	offsetX: 0, offsetY: 0,
	originalWidth: 0, originalHeight: 0,
	index: 0,
	rotate: false,
	splits: null,
	pads: null
};

spine.AtlasReader = function (text) {
	this.lines = text.split(/\r\n|\r|\n/);
};
spine.AtlasReader.prototype = {
	index: 0,
	trim: function (value) {
		return value.replace(/^\s+|\s+$/g, "");
	},
	readLine: function () {
		if (this.index >= this.lines.length) return null;
		return this.lines[this.index++];
	},
	readValue: function () {
		var line = this.readLine();
		var colon = line.indexOf(":");
		if (colon == -1) throw "Invalid line: " + line;
		return this.trim(line.substring(colon + 1));
	},
	/** Returns the number of tuple values read (1, 2 or 4). */
	readTuple: function (tuple) {
		var line = this.readLine();
		var colon = line.indexOf(":");
		if (colon == -1) throw "Invalid line: " + line;
		var i = 0, lastMatch = colon + 1;
		for (; i < 3; i++) {
			var comma = line.indexOf(",", lastMatch);
			if (comma == -1) break;
			tuple[i] = this.trim(line.substr(lastMatch, comma - lastMatch));
			lastMatch = comma + 1;
		}
		tuple[i] = this.trim(line.substring(lastMatch));
		return i + 1;
	}
};

spine.AtlasAttachmentLoader = function (atlas) {
	this.atlas = atlas;
};
spine.AtlasAttachmentLoader.prototype = {
	newRegionAttachment: function (skin, name, path) {
		var region = this.atlas.findRegion(path);
		if (!region) throw "Region not found in atlas: " + path + " (region attachment: " + name + ")";
		var attachment = new spine.RegionAttachment(name);
		attachment.rendererObject = region;
		attachment.setUVs(region.u, region.v, region.u2, region.v2, region.rotate);
		attachment.regionOffsetX = region.offsetX;
		attachment.regionOffsetY = region.offsetY;
		attachment.regionWidth = region.width;
		attachment.regionHeight = region.height;
		attachment.regionOriginalWidth = region.originalWidth;
		attachment.regionOriginalHeight = region.originalHeight;
		return attachment;
	},
	newMeshAttachment: function (skin, name, path) {
		var region = this.atlas.findRegion(path);
		if (!region) throw "Region not found in atlas: " + path + " (mesh attachment: " + name + ")";
		var attachment = new spine.MeshAttachment(name);
		attachment.rendererObject = region;
		attachment.regionU = region.u;
		attachment.regionV = region.v;
		attachment.regionU2 = region.u2;
		attachment.regionV2 = region.v2;
		attachment.regionRotate = region.rotate;
		attachment.regionOffsetX = region.offsetX;
		attachment.regionOffsetY = region.offsetY;
		attachment.regionWidth = region.width;
		attachment.regionHeight = region.height;
		attachment.regionOriginalWidth = region.originalWidth;
		attachment.regionOriginalHeight = region.originalHeight;
		return attachment;
	},
	newSkinnedMeshAttachment: function (skin, name, path) {
		var region = this.atlas.findRegion(path);
		if (!region) throw "Region not found in atlas: " + path + " (skinned mesh attachment: " + name + ")";
		var attachment = new spine.SkinnedMeshAttachment(name);
		attachment.rendererObject = region;
		attachment.regionU = region.u;
		attachment.regionV = region.v;
		attachment.regionU2 = region.u2;
		attachment.regionV2 = region.v2;
		attachment.regionRotate = region.rotate;
		attachment.regionOffsetX = region.offsetX;
		attachment.regionOffsetY = region.offsetY;
		attachment.regionWidth = region.width;
		attachment.regionHeight = region.height;
		attachment.regionOriginalWidth = region.originalWidth;
		attachment.regionOriginalHeight = region.originalHeight;
		return attachment;
	},
	newBoundingBoxAttachment: function (skin, name) {
		return new spine.BoundingBoxAttachment(name);
	}
};

spine.SkeletonBounds = function () {
	this.polygonPool = [];
	this.polygons = [];
	this.boundingBoxes = [];
};
spine.SkeletonBounds.prototype = {
	minX: 0, minY: 0, maxX: 0, maxY: 0,
	update: function (skeleton, updateAabb) {
		var slots = skeleton.slots;
		var slotCount = slots.length;
		var x = skeleton.x, y = skeleton.y;
		var boundingBoxes = this.boundingBoxes;
		var polygonPool = this.polygonPool;
		var polygons = this.polygons;

		boundingBoxes.length = 0;
		for (var i = 0, n = polygons.length; i < n; i++)
			polygonPool.push(polygons[i]);
		polygons.length = 0;

		for (var i = 0; i < slotCount; i++) {
			var slot = slots[i];
			var boundingBox = slot.attachment;
			if (boundingBox.type != spine.AttachmentType.boundingbox) continue;
			boundingBoxes.push(boundingBox);

			var poolCount = polygonPool.length, polygon;
			if (poolCount > 0) {
				polygon = polygonPool[poolCount - 1];
				polygonPool.splice(poolCount - 1, 1);
			} else
				polygon = [];
			polygons.push(polygon);

			polygon.length = boundingBox.vertices.length;
			boundingBox.computeWorldVertices(x, y, slot.bone, polygon);
		}

		if (updateAabb) this.aabbCompute();
	},
	aabbCompute: function () {
		var polygons = this.polygons;
		var minX = Number.MAX_VALUE, minY = Number.MAX_VALUE, maxX = Number.MIN_VALUE, maxY = Number.MIN_VALUE;
		for (var i = 0, n = polygons.length; i < n; i++) {
			var vertices = polygons[i];
			for (var ii = 0, nn = vertices.length; ii < nn; ii += 2) {
				var x = vertices[ii];
				var y = vertices[ii + 1];
				minX = Math.min(minX, x);
				minY = Math.min(minY, y);
				maxX = Math.max(maxX, x);
				maxY = Math.max(maxY, y);
			}
		}
		this.minX = minX;
		this.minY = minY;
		this.maxX = maxX;
		this.maxY = maxY;
	},
	/** Returns true if the axis aligned bounding box contains the point. */
	aabbContainsPoint: function (x, y) {
		return x >= this.minX && x <= this.maxX && y >= this.minY && y <= this.maxY;
	},
	/** Returns true if the axis aligned bounding box intersects the line segment. */
	aabbIntersectsSegment: function (x1, y1, x2, y2) {
		var minX = this.minX, minY = this.minY, maxX = this.maxX, maxY = this.maxY;
		if ((x1 <= minX && x2 <= minX) || (y1 <= minY && y2 <= minY) || (x1 >= maxX && x2 >= maxX) || (y1 >= maxY && y2 >= maxY))
			return false;
		var m = (y2 - y1) / (x2 - x1);
		var y = m * (minX - x1) + y1;
		if (y > minY && y < maxY) return true;
		y = m * (maxX - x1) + y1;
		if (y > minY && y < maxY) return true;
		var x = (minY - y1) / m + x1;
		if (x > minX && x < maxX) return true;
		x = (maxY - y1) / m + x1;
		if (x > minX && x < maxX) return true;
		return false;
	},
	/** Returns true if the axis aligned bounding box intersects the axis aligned bounding box of the specified bounds. */
	aabbIntersectsSkeleton: function (bounds) {
		return this.minX < bounds.maxX && this.maxX > bounds.minX && this.minY < bounds.maxY && this.maxY > bounds.minY;
	},
	/** Returns the first bounding box attachment that contains the point, or null. When doing many checks, it is usually more
	 * efficient to only call this method if {@link #aabbContainsPoint(float, float)} returns true. */
	containsPoint: function (x, y) {
		var polygons = this.polygons;
		for (var i = 0, n = polygons.length; i < n; i++)
			if (this.polygonContainsPoint(polygons[i], x, y)) return this.boundingBoxes[i];
		return null;
	},
	/** Returns the first bounding box attachment that contains the line segment, or null. When doing many checks, it is usually
	 * more efficient to only call this method if {@link #aabbIntersectsSegment(float, float, float, float)} returns true. */
	intersectsSegment: function (x1, y1, x2, y2) {
		var polygons = this.polygons;
		for (var i = 0, n = polygons.length; i < n; i++)
			if (polygons[i].intersectsSegment(x1, y1, x2, y2)) return this.boundingBoxes[i];
		return null;
	},
	/** Returns true if the polygon contains the point. */
	polygonContainsPoint: function (polygon, x, y) {
		var nn = polygon.length;
		var prevIndex = nn - 2;
		var inside = false;
		for (var ii = 0; ii < nn; ii += 2) {
			var vertexY = polygon[ii + 1];
			var prevY = polygon[prevIndex + 1];
			if ((vertexY < y && prevY >= y) || (prevY < y && vertexY >= y)) {
				var vertexX = polygon[ii];
				if (vertexX + (y - vertexY) / (prevY - vertexY) * (polygon[prevIndex] - vertexX) < x) inside = !inside;
			}
			prevIndex = ii;
		}
		return inside;
	},
	/** Returns true if the polygon contains the line segment. */
	polygonIntersectsSegment: function (polygon, x1, y1, x2, y2) {
		var nn = polygon.length;
		var width12 = x1 - x2, height12 = y1 - y2;
		var det1 = x1 * y2 - y1 * x2;
		var x3 = polygon[nn - 2], y3 = polygon[nn - 1];
		for (var ii = 0; ii < nn; ii += 2) {
			var x4 = polygon[ii], y4 = polygon[ii + 1];
			var det2 = x3 * y4 - y3 * x4;
			var width34 = x3 - x4, height34 = y3 - y4;
			var det3 = width12 * height34 - height12 * width34;
			var x = (det1 * width34 - width12 * det2) / det3;
			if (((x >= x3 && x <= x4) || (x >= x4 && x <= x3)) && ((x >= x1 && x <= x2) || (x >= x2 && x <= x1))) {
				var y = (det1 * height34 - height12 * det2) / det3;
				if (((y >= y3 && y <= y4) || (y >= y4 && y <= y3)) && ((y >= y1 && y <= y2) || (y >= y2 && y <= y1))) return true;
			}
			x3 = x4;
			y3 = y4;
		}
		return false;
	},
	getPolygon: function (attachment) {
		var index = this.boundingBoxes.indexOf(attachment);
		return index == -1 ? null : this.polygons[index];
	},
	getWidth: function () {
		return this.maxX - this.minX;
	},
	getHeight: function () {
		return this.maxY - this.minY;
	}
};

//******************************************************
//*****    Global Initialization
//******************************************************

    var mainCanvasId = "canvasBackground";
    var mainCanvas = document.getElementById(mainCanvasId);

    var testMode = false;

    var builderVersion = "1.8.1.0";    // Version of the builder when building the solution
    var buildDate = "01-07-2016";    // Date when building the solution
    var buildTime = "13:38:39";    // Time when building the solution
    var buildTags = ["big", "landscape", "endscreen_endless", "theme_olympus", "game_bejeweled", "timed", "gameui_difficulty", "boostermedia2", "final"];    // Tags used in this build

//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//*****    The Basic Engine
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************

/////////////////////////////////////////////////////////////
// tbBrowser: Browser detection functionality
//
// Copyright 2013-2014 CoolGames
// Last Changed: 18-11-2014
/////////////////////////////////////////////////////////////

var TBBrowser = function(userAgent) {
	"use strict";

	this.userAgent = userAgent;
	if (this.userAgent === undefined) {
		this.userAgent = (navigator !== undefined) ? navigator.userAgent : '';
	}
	this.detect();
};

TBBrowser.prototype.getFirstMatch = function(regex) {
	"use strict";
	var match = this.userAgent.match(regex);
	return (match && match.length > 1 && match[1]) || '';
};

TBBrowser.prototype.detect = function() {
	"use strict";
	var iosdevice = this.getFirstMatch(/(ipod|iphone|ipad)/i).toLowerCase(),
		likeAndroid = /like android/i.test(this.userAgent),
		android = !likeAndroid && /android/i.test(this.userAgent),
		versionIdentifier = this.getFirstMatch(/version\/(\d+(\.\d+)?)/i),
		tablet = /tablet/i.test(this.userAgent),
		mobile = !tablet && /[^-]mobi/i.test(this.userAgent);

	this.browser = {};
	this.os = {};
	this.features = {};

	//Opera
	if (/opera|opr/i.test(this.userAgent)) {
		this.name = 'Opera';
		this.browser.opera = true;
		this.browser.version = versionIdentifier || this.getFirstMatch(/(?:opera|opr)[\s\/](\d+(\.\d+)?)/i);
	}
	//Windows Phone
	else if (/windows phone/i.test(this.userAgent)) {
		this.name = 'Windows Phone';
		this.os.windowsphone = true;
		this.browser.msie = true;
		this.browser.version = this.getFirstMatch(/iemobile\/(\d+(\.\d+)?)/i);
	}
	//Internet Explorer
	else if (/msie|trident/i.test(this.userAgent)) {
		this.name = 'Internet Explorer';
		this.browser.msie = true;
		this.browser.version = this.getFirstMatch(/(?:msie |rv:)(\d+(\.\d+)?)/i);
	}
	//Chrome
	else if (/chrome|crios|crmo/i.test(this.userAgent)) {
		this.name = 'Chrome';
		this.browser.chrome = true;
		this.browser.version = this.getFirstMatch(/(?:chrome|crios|crmo)\/(\d+(\.\d+)?)/i);
	}
	//iOS
	else if (iosdevice) {
		this.name = iosdevice == 'iphone' ? 'iPhone' : iosdevice == 'ipad' ? 'iPad' : 'iPod';
		if (versionIdentifier) {
			this.browser.version = versionIdentifier;
		}
	}
	//Sailfish
	else if (/sailfish/i.test(this.userAgent)) {
		this.name = 'Sailfish';
		this.browser.sailfish = true;
		this.browser.version = this.getFirstMatch(/sailfish\s?browser\/(\d+(\.\d+)?)/i);
	}
	//SeaMonkey
	else if (/seamonkey\//i.test(this.userAgent)) {
		this.name = 'SeaMonkey';
		this.browser.seamonkey = true;
		this.browser.version = this.getFirstMatch(/seamonkey\/(\d+(\.\d+)?)/i);
	}
	//Firefox
	else if (/firefox|iceweasel/i.test(this.userAgent)) {
		this.name = 'Firefox';
		this.browser.firefox = true;
		this.browser.version = this.getFirstMatch(/(?:firefox|iceweasel)[ \/](\d+(\.\d+)?)/i);
		if (/\((mobile|tablet);[^\)]*rv:[\d\.]+\)/i.test(this.userAgent)) {
			this.os.firefoxos = true;
		}
	}
	//Amazon Silk
	else if (/silk/i.test(this.userAgent)) {
		this.name = 'Amazon Silk';
		this.browser.silk = true;
		this.browser.version = this.getFirstMatch(/silk\/(\d+(\.\d+)?)/i);
	}
	//Android
	else if (android) {
		this.name = 'Android';
		this.browser.stockandroid = true;
		this.browser.version = versionIdentifier;
	}
	//PhantomJS
	else if (/phantom/i.test(this.userAgent)) {
		this.name = 'PhantomJS';
		this.browser.phantom = true;
		this.browser.version = this.getFirstMatch(/phantomjs\/(\d+(\.\d+)?)/i);
	}
	//BlackBerry
	else if (/blackberry|\bbb\d+/i.test(this.userAgent) || /rim\stablet/i.test(this.userAgent)) {
		this.name = 'BlackBerry';
		this.browser.blackberry = true;
		this.browser.version = versionIdentifier || this.getFirstMatch(/blackberry[\d]+\/(\d+(\.\d+)?)/i);
	}
	//WebOS
	else if (/(web|hpw)os/i.test(this.userAgent)) {
		this.name = 'WebOS';
		this.browser.webos = true;
		this.browser.version = versionIdentifier || this.getFirstMatch(/w(?:eb)?osbrowser\/(\d+(\.\d+)?)/i);
		if (/touchpad\//i.test(this.userAgent)) {
			this.features.touchpad = true;
		}
	}
	//Bada
	else if (/bada/i.test(this.userAgent)) {
		this.name = 'Bada';
		this.browser.bada = true;
		this.browser.version = this.getFirstMatch(/dolfin\/(\d+(\.\d+)?)/i);
	}
	//Tizen
	else if (/tizen/i.test(this.userAgent)) {
		this.name = 'Tizen';
		this.browser.tizen = true;
		this.browser.version = this.getFirstMatch(/(?:tizen\s?)?browser\/(\d+(\.\d+)?)/i) || versionIdentifier;
	}
	//Safari
	else if (/safari/i.test(this.userAgent)) {
		this.name = 'Safari';
		this.browser.safari = true;
		this.browser.version = versionIdentifier;
	}
	//Unknown!
	else {
		// Do nothing
	}

	//Set webkit or gecko flag for browsers based on these engines
	if (/(apple)?webkit/i.test(this.userAgent)) {
		this.name = this.name || "Webkit";
		this.browser.webkit = true;
		if (!this.browser.version && versionIdentifier) {
			this.browser.version = versionIdentifier;
		}
	}
	else if (!this.opera && /gecko\//i.test(this.userAgent)) {
		this.name = this.name || "Gecko";
		this.browser.gecko = true;
		this.browser.version = this.browser.version || this.getFirstMatch(/gecko\/(\d+(\.\d+)?)/i);
	}

	//Set OS flags for platforms that have multiple browsers
	if (android || this.silk) {
		this.os.android = true;
	}
	else if (iosdevice) {
		this.os.ios = true;
	}

	//OS version extraction
	var osVersion = '';
	if (iosdevice) {
		osVersion = this.getFirstMatch(/os (\d+([_\s]\d+)*) like mac os x/i);
		osVersion = osVersion.replace(/[_\s]/g, '.');
	}
	else if (android) {
		osVersion = this.getFirstMatch(/android[ \/-](\d+(\.\d+)*)/i);
	}
	else if (this.windowsphone) {
		osVersion = this.getFirstMatch(/windows phone (?:os)?\s?(\d+(\.\d+)*)/i);
	}
	else if (this.webos) {
		osVersion = this.getFirstMatch(/(?:web|hpw)os\/(\d+(\.\d+)*)/i);
	}
	else if (this.blackberry) {
		osVersion = this.getFirstMatch(/rim\stablet\sos\s(\d+(\.\d+)*)/i);
	}
	else if (this.bada) {
		osVersion = this.getFirstMatch(/bada\/(\d+(\.\d+)*)/i);
	}
	else if (this.tizen) {
		osVersion = this.getFirstMatch(/tizen[\/\s](\d+(\.\d+)*)/i);
	}

	if (osVersion) {
		this.os.version = osVersion;
	}

	//Device type extraction
	var osMajorVersion = osVersion.split('.')[0];
	if (tablet || iosdevice == 'ipad' || (android && (osMajorVersion == 3 || (osMajorVersion == 4 && !mobile))) || this.silk) {
		this.features.tablet = true;
	}
	else if (mobile || iosdevice == 'iphone' || iosdevice == 'ipod' || android || this.blackberry || this.webos || this.bada) {
		this.features.mobile = true;
	}

	// Graded Browser Support
	// http://developer.yahoo.com/yui/articles/gbs
	this.grade = {a: false, c: false, x: false};
	if ((this.msie && this.browser.version >= 10) ||
		(this.chrome && this.browser.version >= 20) ||
		(this.firefox && this.browser.version >= 20.0) ||
		(this.safari && this.browser.version >= 6) ||
		(this.opera && this.browser.version >= 10.0) ||
		(this.ios && this.os.version && this.os.version.split(".")[0] >= 6)) {
		this.grade.a = true;
	}
	else if ((this.msie && this.browser.version < 10) ||
		(this.chrome && this.browser.version < 20) ||
		(this.firefox && this.browser.version < 20.0) ||
		(this.safari && this.browser.version < 6) ||
		(this.opera && this.browser.version < 10.0) ||
		(this.ios && this.os.version && this.os.version.split(".")[0] < 6) ) {
		this.grade.c = true;
	}
	else {
		this.grade.x = true;
	}

	try {
		this.browser.versionAsNumber = this.browser.version ? parseFloat(this.browser.version.match(/\d+(\.\d+)?/)[0], 10) : 0;
	}
	catch (e) {
		this.browser.versionAsNumber = 0;
	}

	try {
		this.os.versionAsNumber = this.os.version ? parseFloat(this.os.version.match(/\d+(\.\d+)?/)[0], 10) : 0;
	}
	catch (e) {
		this.os.versionAsNumber = 0;
	}

};
var tbBrowser = new TBBrowser();

/////////////////////////////////////////////////////////////
// tbMath: The engine part dealing mathematical calculations
//		It contains classes dealing with vectors, bounding boxes
//		random number, etc.
//
// Copyright 2013-2015 CoolGames
// Last Changed: 08-06-2015
/////////////////////////////////////////////////////////////

//
// The math class tbMath that extends the Math class with other useful methods
//

var TBMath = function() {
	"use strict";
};

TBMath.prototype.degToRad = function(degrees) {
	"use strict";
	return Math.PI * degrees/180;
};

TBMath.prototype.radToDeg = function(rad) {
	"use strict";
	return 180*rad/Math.PI;
};

TBMath.prototype.distance = function(x1, y1, x2, y2) {
	"use strict";
	return Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
};

TBMath.prototype.direction = function(x1, y1, x2, y2) {
	"use strict";
	var xx = x2-x1, yy = y2-y1;
	return tbMath.radToDeg(Math.atan2(-yy, xx));
};

TBMath.prototype.projectionX = function(direction, length) {
	"use strict";
	return length * Math.cos(tbMath.degToRad(direction));
};

TBMath.prototype.projectionY = function(direction, length) {
	"use strict";
	return - length * Math.sin(tbMath.degToRad(direction));
};

var tbMath = new TBMath();


//
// The vector class tbVector
//

var tbVector = function(x,y) {
	"use strict";
	this.x = x;
	this.y = y;
};

function tbVectorFromPolar(direction, length) {
	"use strict";
	var x = length * Math.cos(tbMath.degToRad(direction)),
		y = - length * Math.sin(tbMath.degToRad(direction));
	return new tbVector(x,y);
}

tbVector.prototype.assignVector = function(v) {
	"use strict";
	this.x = v.x;
	this.y = v.y;
};

tbVector.prototype.assignXY = function(x, y) {
	"use strict";
	this.x = x;
	this.y = y;
};

tbVector.prototype.assign = function(vector_or_x, undefined_or_y) {
	"use strict";
	if (undefined_or_y === undefined) {
		this.x = vector_or_x.x;
		this.y = vector_or_x.y;
	}
	else {
		this.x = vector_or_x;
		this.y = undefined_or_y;
	}
};

tbVector.prototype.magnitude = function() {
	"use strict";
	return Math.sqrt(this.x*this.x + this.y*this.y);
};

tbVector.prototype.length = function() {
	"use strict";
	return Math.sqrt(this.x*this.x + this.y*this.y);
};

tbVector.prototype.lengthSquared = function() {
	"use strict";
	return this.x*this.x + this.y*this.y;
};

tbVector.prototype.distanceSquared = function(other) {
	"use strict";
	var dx = (this.x - other.x),
		dy = (this.y - other.y);
	return dx * dx + dy * dy;
};

tbVector.prototype.distance = function(other) {
	"use strict";
	var dx = (this.x - other.x),
		dy = (this.y - other.y);
	return Math.sqrt(dx * dx + dy * dy);
};

tbVector.prototype.distanceSquaredXY = function(x, y) {
	"use strict";
	var dx = (this.x - x),
		dy = (this.y - y);
	return dx * dx + dy * dy;
};

tbVector.prototype.distanceXY = function(x, y) {
	"use strict";
	var dx = (this.x - x),
		dy = (this.y - y);
	return Math.sqrt(dx * dx + dy * dy);
};

tbVector.prototype.direction = function() {
	"use strict";
	return tbMath.radToDeg(Math.atan2(-this.y, this.x));
};

tbVector.prototype.clone = function() {
	"use strict";
	return new tbVector(this.x, this.y);
};

tbVector.prototype.add = function(vector) {
	"use strict";
	return new tbVector(this.x + vector.x, this.y + vector.y);
};

tbVector.prototype.addDirect = function(vector) {
	"use strict";
	this.x += vector.x;
	this.y += vector.y;
};

tbVector.prototype.subtract = function(vector) {
	"use strict";
	return new tbVector(this.x - vector.x, this.y - vector.y);
};

tbVector.prototype.subtractDirect = function(vector) {
	"use strict";
	this.x -= vector.x;
	this.y -= vector.y;
};

tbVector.prototype.negate = function() {
	"use strict";
	return new tbVector(-this.x, -this.y);
};

tbVector.prototype.negateDirect = function() {
	"use strict";
	this.x = -this.x;
	this.y = -this.y;
};

tbVector.prototype.scale = function(scalar) {
	"use strict";
	return new tbVector(scalar * this.x, scalar * this.y);
};

tbVector.prototype.scaleDirect = function(scalar) {
	"use strict";
	this.x *= scalar;
	this.y *= scalar;
};

tbVector.prototype.multiply = function(vector) {
	"use strict";
	return new tbVector(vector.x * this.x, vector.y * this.y);
};

tbVector.prototype.multiplyDirect = function(vector) {
	"use strict";
	this.x *= vector.x;
	this.y *= vector.y;
};

tbVector.prototype.rotate = function(angle) {
	"use strict";
	var s = Math.sin(angle*Math.PI/180),
		c = Math.cos(angle*Math.PI/180);
	return new tbVector(c * this.x + s * this.y, -s * this.x + c * this.y);
};

tbVector.prototype.rotateDirect = function(angle) {
	"use strict";
	var s = Math.sin(angle*Math.PI/180),
		c = Math.cos(angle*Math.PI/180),
		x = this.x;
	this.x = c * this.x + s * this.y;
	this.y = -s * x + c * this.y;
};

tbVector.prototype.dot = function(vector) {
	"use strict";
	return (this.x * vector.x + this.y * vector.y);
};

tbVector.prototype.perpendicular = function() {
	"use strict";
	return new tbVector(this.y, -this.x);
};

tbVector.prototype.perpendicularDirect = function() {
	"use strict";
	var x = this.x;
	this.x = this.y;
	this.y = -x;
};

tbVector.prototype.normalize = function() {
	"use strict";
	var m = this.magnitude();
	if (m === 0) {
		return new tbVector(0, 0);
	}
	return new tbVector(this.x / m, this.y / m);
};

tbVector.prototype.normalizeDirect = function() {
	"use strict";
	var m = this.magnitude();
	if (m === 0) {
		this.x = 0; this.y = 0;
		return;
	}
	this.x /= m;
	this.y /= m;
};

tbVector.prototype.normal = function() {
	"use strict";
	return (this.perpendicular()).normalize();
};

tbVector.prototype.normalDirect = function() {
	"use strict";
	this.perpendicularDirect();
	this.normalizeDirect();
};

tbVector.prototype.draw = function(x, y, color) {
	"use strict";
	var sc = Math.min(8,this.length()/4),
		vec1 = this.subtract(this.normalize().scale(2*sc)),
		vec2 = vec1.add(this.normal().scale(sc)),
		vec3 = vec1.add(this.normal().scale(-sc)),
		context = tbGraphics.context;
	context.strokeStyle = color;
	context.beginPath();
	context.moveTo(x, y);
	context.lineTo(x + vec1.x, y + vec1.y);
	context.lineTo(x + vec2.x, y + vec2.y);
	context.lineTo(x + this.x, y + this.y);
	context.lineTo(x + vec3.x, y + vec3.y);
	context.lineTo(x + vec1.x, y + vec1.y);
	context.stroke();
};


//
// A random number generator class
// Uses a linear congruential generator using GCC's constants
//

var useOldRandomNumberConstants = false;

// The constructor
var tbRandom = function(seed) {
	"use strict";
	if (useOldRandomNumberConstants) {
		// This version does not work correctly because JavaScript does not support 64-bit integers
		this.m = 0x80000000;		// 2**31;
		this.a = 1103515245;
		this.c = 12345;
	} else {
		this.m = 4294967296;		// 2**32;
		this.a = 1664525;
		this.c = 1013904223;
	}
	if (seed === undefined) {
		this.state = Math.floor(Math.random() * (this.m-1));
	} else {
		this.state = seed;
	}
};

// returns integer in range [0,2^31> (but with weak randomness in lower bits). Don't use directly
tbRandom.prototype._next = function() {
	"use strict";
	this.state = (this.a * this.state + this.c) % this.m;
	return this.state;
};

// returns a clone of the object
tbRandom.prototype.clone = function() {
	"use strict";
	var res = new tbRandom();
	res.m = this.m;
	res.a = this.a;
	res.c = this.c;
	res.state = this.state;
	return res;
};

// returns real in range [0,n>
tbRandom.prototype.random = function(n) {
	"use strict";
	var nn = 1;
	if (n !== undefined) { nn = n; }
	return nn*(this._next() / this.m);
};

// returns integer in range [0,n]
tbRandom.prototype.iRandom = function(n) {
	"use strict";
	var nn = 1;
	if (n !== undefined) { nn = n; }
	return Math.floor(this.random(nn+1));
};

// returns real in range [n1,n2>
tbRandom.prototype.randomRange = function(n1,n2) {
	"use strict";
	return n1 + this.random(n2-n1);
};

// returns integer in range [n1,n2]
tbRandom.prototype.iRandomRange = function(n1,n2) {
	"use strict";
	return n1 + this.iRandom(n2-n1);
};

// Returns randomly one of the arguments
tbRandom.prototype.choice = function() {
	"use strict";
	if (arguments.length === 0) { return 0; }
	return arguments[this.iRandomRange(0, arguments.length-1)];
};

// We define one global random number generator
var defaultRandom = new tbRandom();
/////////////////////////////////////////////////////////////
// tbPlatforms: Contains objects for device detection and
//              platform specific options
//
// Copyright 2013-2014 CoolGames
// Last Changed: 16-7-2014
/////////////////////////////////////////////////////////////


var tbPlatforms = {

	// Enable or disable platform specific hacks
	// TODO: default is true, but it should be based on detected device
	Hacks: {
		// On our Android 4.1 Sony test device, the context is not properly reset each frame
		// therefore, we add a hack to the tbGraphics.clear function by drawing an invisible
		// line
		ANDROID_41_CONTEXT_RESET_BUG: true,

		// If we use Cordova to build an app on iOS, the game becomes slow if images are loaded
		// that are not used. This hack enabled immediate use of an image after it is loaded
		DRAW_IMAGE_AFTER_LOADING: false
	}
};/////////////////////////////////////////////////////////////
// tbLoader: The engine part dealing with loading
//
// Copyright 2013-2015 CoolGames
// Last Changed: 9-6-2015
/////////////////////////////////////////////////////////////

// The constructor for the loader object
var TBLoader = function() {
	"use strict";
	this._location = "";				// Location of the data files
	this._loading = [];					// Whether this moment is being loaded or has been loaded before
	this._total = [];					// Total number of items to be loaded for each moment
	this._loaded  = [];					// Number of items that are loaded for each moment
	this._failed = [];					// Number of items that failed for each moment
	this._items = [];					// The items to be loaded for each moment
	this.addMoment("start");
	this.addMoment("load");
	this.addMoment("game");
};

// Set the data location
TBLoader.prototype.setDataLocation = function(location) { 
	"use strict";
	this._location = location;
	if (this._location !== "" && this._location[this._location.length-1] !== "/") {
		this._location += "/";
	}
};

// Get the data location
TBLoader.prototype.getDataLocation = function() { 
	"use strict";
	return this._location;
};

// Initialize a moment
TBLoader.prototype.addMoment = function(moment) { 
	"use strict";
	if (this._items[moment]) { return; }			// It already exists
	this._total[moment] = 0;
	this._loaded[moment] = 0;
	this._failed[moment] = 0;
	this._items[moment] = [];
	this._loading[moment] = false;
};

// Return total number of items to load for the moment
TBLoader.prototype.total = function(moment) { 
	"use strict";
	if (!this._items[moment]) { return 0; }
	return this._total[moment];
};

// Return number of loaded items for the moment
TBLoader.prototype.loaded = function(moment) { 
	"use strict";
	if (!this._items[moment]) { return 0; }
	return this._loaded[moment];
};

// Return number of failed items for the moment
TBLoader.prototype.failed = function(moment) { 
	"use strict";
	if (!this._items[moment]) { return 0; }
	return this._failed[moment];
};

// Return whether all items are loaded for the moment
TBLoader.prototype.complete = function(moment) { 
	"use strict";
	if (!this._items[moment]) { return true; }
	return (this._loaded[moment] + this._failed[moment] === this._total[moment]);
};

// Return percentage of items that are loaded for the moment
TBLoader.prototype.percentage = function(moment) { 
	"use strict";
	if (!this._items[moment]) { return 100; }
	return (100*(this._loaded[moment] + this._failed[moment]) / this._total[moment]);
};

// Report that a resource is correctly loaded
TBLoader.prototype.loadComplete = function(moment) {
	"use strict";
	this._loaded[moment] += 1;
	if (this.complete(moment)) {
		tbGame.addSpecial("Load Complete", { moment: moment} ); 
	}
};

// Report that a resource failed to load
TBLoader.prototype.loadFailed = function(moment) {
	"use strict";
	this._failed[moment] += 1;
	tbGame.addSpecial("Load Failed", { moment: moment} ); 
};

// Add a resource to be loaded, number indicated the number of subresources
TBLoader.prototype.addResource = function(resource, moment, number) {
	"use strict";
	if (!this._items[moment]) {
		this.addMoment(moment);
	}
	this._items[moment].push(resource);
	this._total[moment] += number;
};

// start the actual loading for the moment
TBLoader.prototype.startLoading = function(moment) {
	"use strict";
	var i;
	// Check whether it is or has already been loaded
	if (this._loading[moment]) { return; }
	this._loading[moment] = true;
	// If nothing to load, generate event and return
	if (!this._items[moment] || this._items[moment].length === 0) {
		tbGame.addSpecial("Load Complete", { moment: moment} );
		return;
	}
	// Start loading the items
	for (i =0 ; i < this._items[moment].length; i += 1) {
		this._items[moment][i].startLoading(moment, this._location);
	}
};
	
// Create the loader
var tbLoader = new TBLoader();
/////////////////////////////////////////////////////////////
// tbGraphics: The engine part dealing with graphics
//
// Copyright 2013-2015 CoolGames
// Last Changed: 28-06-2015
/////////////////////////////////////////////////////////////

//
// GRAPHICS
//

// The constructor for the graphics object
var TBGraphics = function(mainCanvas) {
	"use strict";
	this.canvas = undefined;					// The current draw canvas
	this.context = undefined;					// The current drawing context
	this.width = 0;								// The width of the current canvas
	this.height = 0;							// The height of the current canvas
	if (mainCanvas) {
		this.setCanvas(mainCanvas);
	}
};

// Set the current canvas by id
TBGraphics.prototype.setCanvasId = function(id) {
	"use strict";
	this.canvas = document.getElementById(id);
	this.setCanvas(this.canvas);
};

// Set the current canvas
TBGraphics.prototype.setCanvas = function(canvas) {
	"use strict";
	this.canvas = canvas;
	this.context = canvas.getContext("2d");
	this.width = canvas.width;
	this.height = canvas.height;
};

// Clear the current canvas to transparent
TBGraphics.prototype.clear = function() {
	"use strict";
	this.context.clearRect(0, 0, this.width, this.height);
	// On Android 4.1 do a hack to force clearing
	if (tbPlatforms.Hacks.ANDROID_41_CONTEXT_RESET_BUG) {
		this.context.beginPath();
		this.context.moveTo(0, 0);
		this.context.lineTo(-1, -1);
		this.context.closePath();
		this.context.stroke();
	}
};

// Clear the current canvas to the indicated color
TBGraphics.prototype.clearColor = function(color) {
	"use strict";
	var oldColor = this.context.fillStyle;
	this.context.fillStyle = color;
	this.context.fillRect(0, 0, this.width, this.height);
	this.context.fillStyle = oldColor;
};

// Draw a rectangle
TBGraphics.prototype.drawRectangle = function(x, y, w, h, fillColor, strokeColor, width) {
	"use strict";
	this.context.save();
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fillRect(x, y, w, h);
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.strokeRect(x, y, w, h);
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fillRect(x, y, w, h);
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.strokeRect(x, y, w, h);
		}
	}
	this.context.restore();
};

// Draw a rounded rectangle
TBGraphics.prototype.drawRoundedRectangle = function(x, y, w, h, r, fillColor, strokeColor, width) {
	"use strict";
	this.context.save();
	if (r === undefined) { r = 8; }
	if (w < 2 * r) { r = Math.floor(w / 2); }
	if (h < 2 * r) { r = Math.floor(h / 2); }
	this.context.beginPath();
	this.context.moveTo(x + r, y);
	this.context.arcTo(x + w, y, x + w, y + h, r);
	this.context.arcTo(x + w, y + h, x, y + h, r);
	this.context.arcTo(x, y + h, x, y, r);
	this.context.arcTo(x, y, x + w, y, r);
	this.context.closePath();
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fill();
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.stroke();
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fill();
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.stroke();
		}
	}
	this.context.restore();
};

// Draw a disk
TBGraphics.prototype.drawDisk = function(x, y, r, fillColor, strokeColor, width) {
	"use strict";
	this.context.save();
	this.context.beginPath();
	this.context.arc(x, y, r, 0, 2 * Math.PI, false);
	this.context.closePath();
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fill();
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.stroke();
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fill();
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.stroke();
		}
	}
	this.context.restore();
};

// Draw an ellipse
TBGraphics.prototype.drawEllipse = function(x, y, rx, ry, fillColor, strokeColor, width) {
	"use strict";
	// Create the path using transformations
	this.context.save();
	this.context.beginPath();
	this.context.translate(x - rx, y - ry);
	this.context.scale(rx, ry);
	this.context.arc(1, 1, 1, 0, 2 * Math.PI, false);
	this.context.restore();
	// Now draw it
	this.context.save();
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fill();
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.stroke();
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fill();
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.stroke();
		}
	}
	this.context.restore();
};

// Draw a line
TBGraphics.prototype.drawLine = function(x1, y1, x2, y2, strokeColor, width) {
	"use strict";
	this.context.save();
	this.context.beginPath();
	this.context.moveTo(x1, y1);
	this.context.lineTo(x2, y2);
	if (width) { this.context.lineWidth = width; }
	this.context.strokeStyle = strokeColor;
	this.context.stroke();
	this.context.restore();
};

// Draw a triangle
TBGraphics.prototype.drawTriangle = function(x1, y1, x2, y2, x3, y3, fillColor, strokeColor, width) {
	"use strict";
	this.drawPolygon([x1, x2, x3], [y1, y2, y3], fillColor, strokeColor, width);
};

// Draw a polygon
TBGraphics.prototype.drawPolygon = function(x, y, fillColor, strokeColor, width) {
	"use strict";
	var i;
	this.context.save();
	this.context.beginPath();
	this.context.moveTo(x[0], y[0]);
	for (i = 1; i < x.length; i += 1) {
		this.context.lineTo(x[i], y[i]);
	}
	this.context.closePath();
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fill();
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.stroke();
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fill();
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.stroke();
		}
	}
	this.context.restore();
};

// Draw a polyline
TBGraphics.prototype.drawPolyLine = function(x, y, strokeColor, width) {
	"use strict";
	var i;
	this.context.save();
	this.context.beginPath();
	this.context.moveTo(x[0], y[0]);
	for (i = 1; i < x.length; i += 1) {
		this.context.lineTo(x[i], y[i]);
	}
	if (width) { this.context.lineWidth = width; }
	this.context.strokeStyle = strokeColor;
	this.context.stroke();
	this.context.restore();
};

// Draw a text
TBGraphics.prototype.drawText = function(text, x, y, fillColor, font, strokeColor, width) {
	"use strict";
	this.context.save();
	this.context.font = font;
	// Compatibility with the past
	if (strokeColor === false) {
		this.context.fillStyle = fillColor;
		this.context.fillText(text, x, y);
	} else if (strokeColor === true) {
		this.context.strokeStyle = fillColor;
		this.context.strokeText(text, x, y);
	} else {
		// New method
		if (fillColor !== undefined) {
			this.context.fillStyle = fillColor;
			this.context.fillText(text, x, y);
		}
		if (strokeColor !== undefined) {
			if (width) { this.context.lineWidth = width; }
			this.context.strokeStyle = strokeColor;
			this.context.strokeText(text, x, y);
		}
	}
	this.context.restore();
};

// Return the width of a text
TBGraphics.prototype.textWidth = function(text, font) {
	"use strict";
	this.context.font = font;
	return this.context.measureText(text).width;
};


// Create the graphics object
var tbGraphics = new TBGraphics(mainCanvas);


//
// TEXTURES
//

// The constructor for the texture
var tbTexture = function(name, number, loading) {
	"use strict";
	this.name = name;
	this.number = number;
	this.loading = loading;
	this.bitmaps = [];
	this.filenames = [];
	tbLoader.addResource(this, this.loading, this.number);
};

// Callback when a bitmap is correctly loaded
tbTexture.prototype._bitmapLoaded = function(moment) {
	"use strict";
	tbLoader.loadComplete(moment);
};

// Callback when a bitmap failed to load
tbTexture.prototype._bitmapFailed = function(moment) {
	"use strict";
	tbLoader.loadFailed(moment);
};

// Start the loading of the texture, location is the added location part
tbTexture.prototype.startLoading = function(moment, location) {
	"use strict";
	var self = this,
		i, fname,
		bmLoad = function(e) {
			if (tbPlatforms.Hacks.DRAW_IMAGE_AFTER_LOADING) {
				tbGraphics.context.drawImage(this, 0, 0, 1, 1, -2, -2, 1, 1);
			}
			self._bitmapLoaded(moment);
		},
		bmFail = function(e) { self._bitmapFailed(moment); };
	for (i = 0; i<this.bitmaps.length; i += 1) {
		fname = this.filenames[i];
		if (fname.toLowerCase().indexOf("http:") !== 0 && fname.toLowerCase().indexOf("https:") !== 0) {
			fname = location + fname;
		}
		this.bitmaps[i].src = fname;
		this.bitmaps[i].addEventListener("load", bmLoad, false);
		this.bitmaps[i].addEventListener("error", bmFail, false);
	}
};

// Return whether correctly loaded
tbTexture.prototype.complete = function() {
	"use strict";
	var i;
	for (i = 0; i<this.bitmaps.length; i += 1) {
		if (!this.bitmaps[i].complete || this.bitmaps[i].width === 0) {
			return false;
		}
	}
	return true;
};

// Adds a bitmap to the texture
tbTexture.prototype.addBitmap = function(number, filename) {
	"use strict";
	if (number >= 0 && number < this.number) {
		this.bitmaps[number] = new Image();
		this.filenames[number] = filename;
	}
};

// Adds an existing image to the texture
tbTexture.prototype.addImage = function(number, image) {
	"use strict";
	if (number >= 0 && number < this.number) {
		this.bitmaps[number] = image;
		this.filenames[number] = "";
	}
};

// Draws the source part of the texture at the destination
tbTexture.prototype.drawPart = function(number, sx, sy, sw, sh, dx, dy, dw, dh) {
	"use strict";
	if (this.bitmaps[number] && this.bitmaps[number].complete) {
		if (dw === undefined) { dw = sw; }
		if (dh === undefined) { dh = sh; }
		// Solve problem with wrong sizes
		if (sw <= 0 || sh <= 0) { return; }
		if (Math.round(dw) === 0 || Math.round(dh) === 0) { return; }
		tbGraphics.context.drawImage(this.bitmaps[number], sx, sy, sw, sh, dx, dy, dw, dh);
	}
};


//
// IMAGES
//

// The constructor for the image
var tbImage = function(name, texture, number, width, height, xOrigin, yOrigin, combinedWidth, combinedHeight, perRow) {
	"use strict";
	this.name = name;
	this.texture = texture;
	this.number = number;
	this.width = width;
	this.height = height;
	this.xOrigin = xOrigin;
	this.yOrigin = yOrigin;
	this.combinedWidth = combinedWidth;
	this.combinedHeight = combinedHeight;
	this.perRow = perRow;
	this.subTexIndex = [];
	this.subX = [];
	this.subY = [];
	this.subW = [];
	this.subH = [];
	this.subXOff = [];
	this.subYOff = [];
};

// Adds a subimage for the image
tbImage.prototype.addImage = function(number, texnumb, x, y, w, h, xOff, yOff) {
	"use strict";
	if (number >= 0 && number < this.number) {
		this.subTexIndex[number] = texnumb;
		this.subX[number] = x;
		this.subY[number] = y;
		this.subW[number] = w;
		this.subH[number] = h;
		this.subXOff[number] = xOff;
		this.subYOff[number] = yOff;
	}
};

// Create an image, loaded from a file
function tbImageLoad(name, filename, width, height, xOrigin, yOrigin) {
	"use strict";
	var tex, img;
	tex = new tbTexture(name + "_Texture", 1, name);
	tex.addBitmap(0,filename);
	img = new tbImage(name, tex, 1, width, height, xOrigin, yOrigin, width, height, 1);
	img.addImage(0, 0, 0, 0, width, height, 0, 0);
	tbLoader.startLoading(name);
	return img;
}

// Create an image with subimages, loaded from a file
function tbImageLoadExt(name, filename, total, perRow, width, height, xOrigin, yOrigin) {
	"use strict";
	var tex, img, i;
	tex = new tbTexture(name + "_Texture", 1, name);
	tex.addBitmap(0,filename);
	img = new tbImage(name, tex, total, width, height, xOrigin, yOrigin, perRow * width, (1 + Math.floor((total-1) / perRow)) * height, perRow);
	for (i=0; i<total; i += 1) {
		img.addImage(i, 0, (i % perRow)*width, (Math.floor(i / perRow))*height, width, height, 0, 0);
	}
	tbLoader.startLoading(name);
	return img;
}

// Create an image, from an Image
function tbImageFromImage(name, image, width, height, xOrigin, yOrigin) {
	"use strict";
	var tex, img;
	tex = new tbTexture(name + "_Texture", 1, name);
	tex.addImage(0,image);
	img = new tbImage(name, tex, 1, width, height, xOrigin, yOrigin, width, height, 1);
	img.addImage(0, 0, 0, 0, width, height, 0, 0);
	return img;
}

// Create an image with subimages, from an Image
function tbImageFromImageExt(name, image, total, perRow, width, height, xOrigin, yOrigin) {
	"use strict";
	var tex, img, i;
	tex = new tbTexture(name + "_Texture", 1, name);
	tex.addImage(0,image);
	img = new tbImage(name, tex, total, width, height, xOrigin, yOrigin, perRow * width, (1 + Math.floor((total-1) / perRow)) * height, perRow);
	for (i=0; i<total; i += 1) {
		img.addImage(i, 0, (i % perRow)*width, (Math.floor(i / perRow))*height, width, height, 0, 0);
	}
	return img;
}



// Return whether correctly loaded
tbImage.prototype.complete = function() {
	"use strict";
	return this.texture.complete();
};

// Draws subimage number at the destination
tbImage.prototype.drawSimple = function(number, dx, dy) {
	"use strict";
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number], this.subY[number], this.subW[number], this.subH[number],
		dx-this.xOrigin+this.subXOff[number], dy-this.yOrigin+this.subYOff[number]);
};

// Draws subimage number at the destination with an alpha setting
tbImage.prototype.drawAlpha = function(number, dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha;
	context.globalAlpha = alpha;
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number], this.subY[number], this.subW[number], this.subH[number],
		dx-this.xOrigin+this.subXOff[number], dy-this.yOrigin+this.subYOff[number]);
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draws subimage number at the destination transformed and with an alpha setting
tbImage.prototype.drawExt = function(number, dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context;
	// Solve problem with 0 scale
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; }
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Set the transformation and alpha
	context.save();
	context.translate(dx,dy);
	context.rotate(-angle * Math.PI/180);     // from degrees to radians and counter-clockwise
	context.scale(xScale,yScale);
	context.globalAlpha = alpha;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number], this.subY[number], this.subW[number], this.subH[number],
		this.subXOff[number]-this.xOrigin, this.subYOff[number]-this.yOrigin);
	// Restore transformation and alpha
	context.restore();
};

// Draws subimage number in the region
tbImage.prototype.drawStretched = function(number, dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		scx, scy;
	if (this.width === 0 || this.height === 0) { return; }
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Set alpha
	context.globalAlpha = alpha;
	// Draw it
	scx = dw / this.width;
	scy = dh / this.height;
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number], this.subY[number], this.subW[number], this.subH[number],
		dx+scx*this.subXOff[number], dy+scy*this.subYOff[number], scx*this.subW[number], scy*this.subH[number]);
	// Restore alpha
	context.globalAlpha = oldalpha;
};


// Draws subimage number in nine-sliced to fill the region
tbImage.prototype.drawNineSlice = function(number, dx, dy, dw, dh, alpha, leftSize, rightSize, topSize, bottomSize) {
	"use strict";
	var sw0, sw1, sw2, sx0, sx1, sx2, sh0, sh1, sh2, sy0, sy1, sy2, sw, sh,
		dw0, dw1, dw2, dx0, dx1, dx2, dh0, dh1, dh2, dy0, dy1, dy2;
	if (this.width === 0 || this.height === 0) { return; }
	// Get the correct size of the pieces when not defined
	if (leftSize === undefined) { leftSize = Math.floor(this.width / 3); }
	if (rightSize === undefined) { rightSize = Math.floor(this.width / 3); }
	if (topSize === undefined) { topSize = Math.floor(this.height / 3); }
	if (bottomSize === undefined) { bottomSize = Math.floor(this.height / 3); }
	// Correct sizes when small
	if (dw < leftSize + rightSize) {
		leftSize = Math.round(leftSize * dw / (leftSize + rightSize));
		rightSize = dw - leftSize;
	}
	if (dh < topSize + bottomSize) {
		topSize = Math.round(topSize * dh / (topSize + bottomSize));
		bottomSize = dh - topSize;
	}
	// Determine positions and size of the pieces in the image
	sw = this.width; sw0 = leftSize; sw1 = this.width - leftSize - rightSize; sw2 = rightSize;
	sx0 = 0; sx1 = sw0; sx2 = sw0 + sw1;
	sh = this.height; sh0 = topSize; sh1 = this.height - topSize - bottomSize; sh2 = bottomSize;
	sy0 = 0; sy1 = sh0; sy2 = sh0 + sh1;
	// Determine positions and size of the pieces in the destination
	dw0 = leftSize; dw1 = dw - leftSize - rightSize; dw2 = rightSize;
	dx0 = dx; dx1 = dx + dw0; dx2 = dx + dw0 + dw1;
	dh0 = topSize; dh1 = dh - topSize - bottomSize; dh2 = bottomSize;
	dy0 = dy; dy1 = dy + dh0; dy2 = dy + dh0 + dh1;
	// Now draw it, looking at the various cases
	if (dw === this.width && dh === this.height) {
		this.drawAlpha(number, dx + this.xOrigin, dy + this.yOrigin, alpha);
	} else if (dh === this.height) {
		this.drawPartStretched(number, sx1, sy0, sw1, sh, dx1, dy0, dw1, dh, alpha);
		this.drawPart(number, sx0, sy0, sw0, sh, dx0, dy0, alpha);
		this.drawPart(number, sx2, sy0, sw2, sh, dx2, dy0, alpha);
	} else if (dw === this.width) {
		this.drawPartStretched(number, sx0, sy1, sw, sh1, dx0, dy1, dw, dh1, alpha);
		this.drawPart(number, sx0, sy0, sw, sh0, dx0, dy0, alpha);
		this.drawPart(number, sx0, sy2, sw, sh2, dx0, dy2, alpha);
	} else {
		// middle piece
		this.drawPartStretched(number, sx1, sy1, sw1, sh1, dx1, dy1, dw1, dh1, alpha); 
		// side pieces
		this.drawPartStretched(number, sx1, sy0, sw1, sh0, dx1, dy0, dw1, dh0, alpha);
		this.drawPartStretched(number, sx0, sy1, sw0, sh1, dx0, dy1, dw0, dh1, alpha);
		this.drawPartStretched(number, sx2, sy1, sw2, sh1, dx2, dy1, dw2, dh1, alpha);
		this.drawPartStretched(number, sx1, sy2, sw1, sh2, dx1, dy2, dw1, dh2, alpha);
		// corner pieces
		this.drawPart(number, sx0, sy0, sw0, sh0, dx0, dy0, alpha);
		this.drawPart(number, sx0, sy2, sw0, sh2, dx0, dy2, alpha);
		this.drawPart(number, sx2, sy0, sw2, sh0, dx2, dy0, alpha);
		this.drawPart(number, sx2, sy2, sw2, sh2, dx2, dy2, alpha);
	}
};

// Draw a number of copies of the image
tbImage.prototype.drawMultiple = function(number, hNumb, vNumb, dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		subx, suby, subw, subh, xx, yy, i, j;
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Set alpha
	context.globalAlpha = alpha;
	// Draw it
	subx = this.subX[number]; suby = this.subY[number]; subw = this.subW[number]; subh = this.subH[number];
	xx = dx-this.xOrigin+this.subXOff[number];
	yy = dy-this.yOrigin+this.subYOff[number];
	for (i=0; i<hNumb; i += 1) {
		for (j=0; j<vNumb; j += 1) {
			this.texture.drawPart(this.subTexIndex[number],subx,suby,subw,subh,xx+i*this.width,yy+j*this.height);
		}
	}
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draw a number of copies of the image with a transformation
tbImage.prototype.drawMultipleExt = function(number, hNumb, vNumb, dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var xinc = xScale*this.width,
		yinc = yScale*this.height,
		hxinc = xinc*Math.cos(angle*Math.PI/180),
		hyinc = -xinc*Math.sin(angle*Math.PI/180),
		vxinc = yinc*Math.sin(angle*Math.PI/180),
		vyinc = yinc*Math.cos(angle*Math.PI/180),
		i, j;
	for (i=0; i<hNumb; i += 1) {
		for (j=0; j<vNumb; j += 1) {
			this.drawExt(number,dx+i*hxinc+j*vxinc,dy+i*hyinc+j*vyinc,xScale,yScale,angle,alpha);
		}
	}
};

// Draw the image tiled in an area with an offset
tbImage.prototype.drawTiledOffset = function(number, xOff, yOff, dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		xx, yy, nx, ny, xdest, ydest, subx, suby, subw, subh, i, j;
	if (this.width === 0 || this.height === 0) { return; }
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Set alpha
	context.globalAlpha = alpha;
	// Draw it
	xx = Math.round(xOff-this.xOrigin) % this.width;
	if (xx > 0) { xx -= this.width; }
	yy = Math.round(yOff-this.yOrigin) % this.height;
	if (yy > 0) { yy -= this.height; }
	nx = Math.ceil((dw-xx)/this.width);
	ny = Math.ceil((dh-yy)/this.height);
	xx += dx+this.subXOff[number];
	yy += dy+this.subYOff[number];
	for (i=0; i<nx; i += 1) {
		for (j=0; j<ny; j += 1)
		{
			subx = this.subX[number]; suby = this.subY[number]; subw = this.subW[number]; subh = this.subH[number];
			xdest = xx+i*this.width;
			ydest = yy+j*this.height;
			if (xdest < dx) { subx += dx-xdest; subw -= dx-xdest; xdest = dx; }
			if (xdest+subw >= dx+dw) { subw = dx+dw-xdest; }
			if (ydest < dy) { suby += dy-ydest; subh -= dy-ydest; ydest = dy; }
			if (ydest+subh >= dy+dh) { subh = dy+dh-ydest; }
			if (subw > 0 && subh > 0) {
				this.texture.drawPart(this.subTexIndex[number],subx,suby,subw,subh,xdest,ydest);
			}
		}
	}
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draw the image tiled in an area
tbImage.prototype.drawTiled = function(number, dx, dy, dw, dh, alpha) {
	"use strict";
	this.drawTiledOffset(number, 0, 0, dx, dy, dw, dh, alpha);
};

// Draws part of subimage number at the destination with an alpha setting
tbImage.prototype.drawPart = function(number, sx, sy, sw, sh, dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		subx, suby, subw, subh;
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Check whether there is anything to draw
	subx = this.subXOff[number]; suby = this.subYOff[number]; subw = this.subW[number]; subh = this.subH[number];
	sx -= subx; sy -= suby;
	if (sx+sw <= 0 || sy+sh <= 0 || sx >= subw || sy >= subh) { return; }
	// Handle case when sx,sy does not lie inside the area
	if (sx<0) { sw += sx; dx -= sx; sx = 0; }
	if (sy<0) { sh += sy; dy -= sy; sy = 0; }
	// Handle case when it sticks out of the area
	if (sx+sw > subw) { sw = subw-sx; }
	if (sy+sh > subh) { sh = subh-sy;	}
	// Set alpha
	context.globalAlpha = alpha;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number]+sx, this.subY[number]+sy, sw, sh, dx, dy);
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draw the image part tiled in an area with an offset
tbImage.prototype.drawPartTiledOffset = function(number, xOff, yOff, sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	var xx, yy, nx, ny, xdest, ydest, subx, suby, subw, subh, i, j;
	if (sw <= 0 || sh <= 0) { return; }
	xx = Math.round(xOff) % sw;
	if (xx > 0) { xx -= sw; }
	yy = Math.round(yOff) % sh;
	if (yy > 0) { yy -= sh; }
	nx = Math.ceil((dw-xx)/sw);
	ny = Math.ceil((dh-yy)/sh);
	xx += dx;
	yy += dy;
	for (i=0; i<nx; i += 1) {
		for (j=0; j<ny; j += 1) {
			subx = sx; suby = sy; subw = sw; subh = sh;
			xdest = xx+i*sw;
			ydest = yy+j*sh;
			if (xdest < dx) { subx += dx-xdest; subw -= dx-xdest; xdest = dx; }
			if (xdest+subw >= dx+dw) { subw = dx+dw-xdest; }
			if (ydest < dy) { suby += dy-ydest; subh -= dy-ydest; ydest = dy; }
			if (ydest+subh >= dy+dh) { subh = dy+dh-ydest; }
			if (subw > 0 && subh > 0) {
				this.drawPart(number,subx,suby,subw,subh,xdest,ydest,alpha);
			}
		}
	}
};

// Draw part of the image tiled in an area
tbImage.prototype.drawPartTiled = function(number, sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	this.drawPartTiledOffset(number, 0, 0, sx, sy, sw, sh, dx, dy, dw, dh, alpha);
};

// Draws part of subimage number stretched in the destination rectangle with an alpha setting
tbImage.prototype.drawPartStretched = function(number, sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		scx, scy, subx, suby, subw, subh;
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Check whether there is anything to draw
	scx = dw / sw;
	scy = dh / sh;
	subx = this.subXOff[number]; suby = this.subYOff[number]; subw = this.subW[number]; subh = this.subH[number];
	sx -= subx; sy -= suby;
	if (sx+sw <= 0 || sy+sh <= 0 || sx >= subw || sy >= subh) { return; }
	// Handle case when sx,sy does not lie inside the area
	if (sx<0) { sw += sx; dw += scx*sx; dx -= scx*sx; sx = 0; }
	if (sy<0) { sh += sy; dh += scy*sy; dy -= scy*sy; sy = 0; }
	// Handle case when it sticks out of the area
	if (sx+sw > subw) {dw -= scx*(sw-subw+sx); sw = subw-sx; }
	if (sy+sh > subh) {dh -= scy*(sh-subh+sy); sh = subh-sy; }
	// Set alpha
	context.globalAlpha = alpha;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number]+sx, this.subY[number]+sy, sw, sh, dx, dy, dw, dh);
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draws part of subimage number at the destination transformed with an alpha setting
tbImage.prototype.drawPartExt = function(number, sx, sy, sw, sh, dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context,
		newx, newy, subx, suby, subw, subh;
	// Solve problem with 0 scale
	if (Math.round(sw) === 0 || Math.round(sh) === 0) { return; }
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; }
	// Get the number in range
	number = (Math.round(number) % this.number + this.number) % this.number;
	// Check whether there is anything to draw
	subx = this.subXOff[number]; suby = this.subYOff[number]; subw = this.subW[number]; subh = this.subH[number];
	newx = 0; newy = 0;
	sx -= subx; sy -= suby;
	if (sx+sw <= 0 || sy+sh <= 0 || sx >= subw || sy >= subh) { return; }
	// Handle case when sx,sy does not lie inside the area
	if (sx<0) { sw += sx; newx -= sx; sx = 0; }
	if (sy<0) { sh += sy; newy -= sy; sy = 0; }
	// Handle case when it sticks out of the area
	if (sx+sw > subw) { sw = subw-sx; }
	if (sy+sh > subh) { sh = subh-sy;	}
	// Set the transformation and alpha
	context.save();
	context.translate(dx,dy);
	context.rotate(-angle * Math.PI/180);     // from degrees to radians and counter-clockwise
	context.scale(xScale,yScale);
	context.globalAlpha = alpha;
	// Draw it
	this.texture.drawPart(this.subTexIndex[number],
		this.subX[number]+sx, this.subY[number]+sy, sw, sh, newx, newy);
	// Restore transformation and alpha
	context.restore();
};

// Draws the combined at the destination
tbImage.prototype.drawCombinedSimple = function(dx, dy) {
	"use strict";
	var n, xx, yy;
	// Draw it
	for (n = 0; n < this.number; n += 1) {
		xx = dx + this.width * (n % this.perRow);
		yy = dy + this.height * Math.floor(n / this.perRow);
		this.texture.drawPart(this.subTexIndex[n],
			this.subX[n], this.subY[n], this.subW[n], this.subH[n],
			xx-this.xOrigin+this.subXOff[n], yy-this.yOrigin+this.subYOff[n]);
	}
};

// Draws the combined image at the destination with an alpha setting
tbImage.prototype.drawCombinedAlpha = function(dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		n, xx, yy;
	// Set the alpha
	context.globalAlpha = alpha;
	// Draw it
	for (n = 0; n < this.number; n += 1) {
		xx = dx + this.width * (n % this.perRow);
		yy = dy + this.height * Math.floor(n / this.perRow);
		this.texture.drawPart(this.subTexIndex[n],
			this.subX[n], this.subY[n], this.subW[n], this.subH[n],
			xx-this.xOrigin+this.subXOff[n], yy-this.yOrigin+this.subYOff[n]);
	}
	// Restore alpha
	context.globalAlpha = oldalpha;
};

// Draws the combined image at the destination transformed and with an alpha setting
tbImage.prototype.drawCombinedExt = function(dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context;
	// Solve problem with 0 scale
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; }
	// Set the transformation and alpha
	context.save();
	context.translate(dx, dy);
	context.rotate(-angle * Math.PI/180);     // from degrees to radians and counter-clockwise
	context.scale(xScale, yScale);
	context.globalAlpha = alpha;
	// Draw it
	this.drawCombinedSimple(0, 0);
	// Restore transformation
	context.restore();
};

// Draws the combined image in the region
tbImage.prototype.drawCombinedStretched = function(dx, dy, dw, dh, alpha) {
	"use strict";
	var scx, scy;
	if (this.combinedWidth === 0 || this.combinedHeight === 0) { return; }
	scx = dw / this.combinedWidth;
	scy = dh / this.combinedHeight;
	this.drawCombinedExt(dx+this.xOrigin*scx, dy+this.yOrigin*scy, scx, scy, 0, alpha);
};

// Draws part of the combined image at the destination with an alpha setting
tbImage.prototype.drawCombinedPart = function(sx, sy, sw, sh, dx, dy, alpha) {
	"use strict";
	var n, xx, yy;
	// Check whether there is anything to draw and, if so, call normal methods
	for (n = 0; n < this.number; n += 1) {
		xx = this.width * (n % this.perRow);
		yy = this.height * Math.floor(n / this.perRow);
		if (xx > sx + sw) { continue; }
		if (xx+this.width < sx) { continue; }
		if (yy > sy + sh) { continue; }
		if (yy+this.height < sy) { continue; }
		this.drawPart(n, sx-xx, sy-yy, sw, sh, dx, dy, alpha);
	}
};

// Draws part of the combined image stretched in the destination rectangle with an alpha setting
tbImage.prototype.drawCombinedPartStretched = function(sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	var n, xx, yy;
	// Check whether there is anything to draw and, if so, call normal methods
	for (n = 0; n < this.number; n += 1) {
		xx = this.width * (n % this.perRow);
		yy = this.height * Math.floor(n / this.perRow);
		if (xx > sx + sw) { continue; }
		if (xx+this.width < sx) { continue; }
		if (yy > sy + sh) { continue; }
		if (yy+this.height < sy) { continue; }
		this.drawPartStretched(n, sx-xx, sy-yy, sw, sh, dx, dy, dw, dh, alpha);
	}
};


//
// SURFACES
//

// The constructor for the surface object
var tbSurface = function(width, height) {
	"use strict";
	this.canvas = document.createElement('canvas');
	this.context = this.canvas.getContext("2d");
	this.width = width;
	this.height = height;
	this.xOrigin = 0;
	this.yOrigin = 0;
	this.canvas.width = width;
	this.canvas.height = height;
	this.clear();
	this.oldCanvas = undefined;
};

// Change the size of the surface
tbSurface.prototype.changeSize = function(width, height) {
	"use strict";
	this.width = width;
	this.height = height;
	this.canvas.width = width;
	this.canvas.height = height;
	this.clear();
};

// Return a copy (clone) of the surface
tbSurface.prototype.clone = function() {
	"use strict";
	var surf = new tbSurface(this.width, this.height);
	surf.xOrigin = this.xOrigin;
	surf.yOrigin = this.yOrigin;
	surf.setTarget();
	this.drawSimple(0,0);
	surf.resetTarget();
	return surf;
};

// Set the surface as render target
tbSurface.prototype.setTarget = function() {
	"use strict";
	this.oldCanvas = tbGraphics.canvas;
	tbGraphics.setCanvas(this.canvas);
};

// Resets the render target to the previous one
tbSurface.prototype.resetTarget = function() {
	"use strict";
	if (tbGraphics.canvas !== this.canvas) { return; }
	if (this.oldCanvas === undefined) { return; }
	tbGraphics.setCanvas(this.oldCanvas);
	this.oldCanvas = undefined;
};

// Clear the surface to transparent
tbSurface.prototype.clear = function() {
	"use strict";
	this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
};

// Clear the surface to the indicated color
tbSurface.prototype.clearColor = function(color) {
	"use strict";
	this.context.fillStyle = color;
	this.context.fillRect(0, 0, this.canvas.width, this.canvas.height);
};

// Draws the surface at the destination location
tbSurface.prototype.drawSimple = function(dx,dy) {
	"use strict";
	tbGraphics.context.drawImage(this.canvas, dx - this.xOrigin, dy - this.yOrigin);
};

// Draws the surface at the destination with an alpha setting
tbSurface.prototype.drawAlpha = function(dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha;
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas, dx - this.xOrigin, dy - this.yOrigin);
	context.globalAlpha = oldalpha;
};

// Draws the surface at the destination transformed and with an alpha setting
tbSurface.prototype.drawExt = function(dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context;
	// Solve problem with 0 scale
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; }
	context.save();
	context.translate(dx, dy);
	context.rotate(-angle * Math.PI/180);
	context.scale(xScale, yScale);
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas, -this.xOrigin, -this.yOrigin);
	context.restore();
};

// Draws the surface in the region
tbSurface.prototype.drawStretched = function(dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha;
	if (this.width === 0 || this.height === 0) { return; }
	if (Math.round(dw) === 0 || Math.round(dh) === 0) { return; }
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas, dx, dy, dw, dh);
	context.globalAlpha = oldalpha;
};



// Draws the surface in nine-sliced to fill the region
tbSurface.prototype.drawNineSlice = function(dx, dy, dw, dh, alpha, leftSize, rightSize, topSize, bottomSize) {
	"use strict";
	var sw0, sw1, sw2, sx0, sx1, sx2, sh0, sh1, sh2, sy0, sy1, sy2, sw, sh,
		dw0, dw1, dw2, dx0, dx1, dx2, dh0, dh1, dh2, dy0, dy1, dy2;
	if (this.width === 0 || this.height === 0) { return; }
	// Get the correct size of the pieces when not defined
	if (leftSize === undefined) { leftSize = Math.floor(this.width / 3); }
	if (rightSize === undefined) { rightSize = Math.floor(this.width / 3); }
	if (topSize === undefined) { topSize = Math.floor(this.height / 3); }
	if (bottomSize === undefined) { bottomSize = Math.floor(this.height / 3); }
	// Correct sizes when small
	if (dw < leftSize + rightSize) {
		leftSize = Math.round(leftSize * dw / (leftSize + rightSize));
		rightSize = dw - leftSize;
	}
	if (dh < topSize + bottomSize) {
		topSize = Math.round(topSize * dh / (topSize + bottomSize));
		bottomSize = dh - topSize;
	}
	// Determine positions and size of the pieces in the image
	sw = this.width; sw0 = leftSize; sw1 = this.width - leftSize - rightSize; sw2 = rightSize;
	sx0 = 0; sx1 = sw0; sx2 = sw0 + sw1;
	sh = this.height; sh0 = topSize; sh1 = this.height - topSize - bottomSize; sh2 = bottomSize;
	sy0 = 0; sy1 = sh0; sy2 = sh0 + sh1;
	// Determine positions and size of the pieces in the destination
	dw0 = leftSize; dw1 = dw - leftSize - rightSize; dw2 = rightSize;
	dx0 = dx; dx1 = dx + dw0; dx2 = dx + dw0 + dw1;
	dh0 = topSize; dh1 = dh - topSize - bottomSize; dh2 = bottomSize;
	dy0 = dy; dy1 = dy + dh0; dy2 = dy + dh0 + dh1;
	// Now draw it, looking at the various cases
	if (dw === this.width && dh === this.height) {
		this.drawAlpha(dx + this.xOrigin, dy + this.yOrigin, alpha);
	} else if (dh === this.height) {
		this.drawPartStretched(sx1, sy0, sw1, sh, dx1, dy0, dw1, dh, alpha);
		this.drawPart(sx0, sy0, sw0, sh, dx0, dy0, alpha);
		this.drawPart(sx2, sy0, sw2, sh, dx2, dy0, alpha);
	} else if (dw === this.width) {
		this.drawPartStretched(sx0, sy1, sw, sh1, dx0, dy1, dw, dh1, alpha);
		this.drawPart(sx0, sy0, sw, sh0, dx0, dy0, alpha);
		this.drawPart(sx0, sy2, sw, sh2, dx0, dy2, alpha);
	} else {
		// middle piece
		this.drawPartStretched(sx1, sy1, sw1, sh1, dx1, dy1, dw1, dh1, alpha); 
		// side pieces
		this.drawPartStretched(sx1, sy0, sw1, sh0, dx1, dy0, dw1, dh0, alpha);
		this.drawPartStretched(sx0, sy1, sw0, sh1, dx0, dy1, dw0, dh1, alpha);
		this.drawPartStretched(sx2, sy1, sw2, sh1, dx2, dy1, dw2, dh1, alpha);
		this.drawPartStretched(sx1, sy2, sw1, sh2, dx1, dy2, dw1, dh2, alpha);
		// corner pieces
		this.drawPart(sx0, sy0, sw0, sh0, dx0, dy0, alpha);
		this.drawPart(sx0, sy2, sw0, sh2, dx0, dy2, alpha);
		this.drawPart(sx2, sy0, sw2, sh0, dx2, dy0, alpha);
		this.drawPart(sx2, sy2, sw2, sh2, dx2, dy2, alpha);
	}
};

// Draw a number of copies of the surface
tbSurface.prototype.drawMultiple = function(hNumb, vNumb, dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		i, j;
	context.globalAlpha = alpha;
	for (i=0; i<hNumb; i += 1) {
		for (j=0; j<vNumb; j += 1) {
			tbGraphics.context.drawImage(this.canvas, 
				dx - this.xOrigin + i*this.width, dy - this.yOrigin + j*this.height);
		}
	}
	context.globalAlpha = oldalpha;
};

// Draw a number of copies of the surface with a transformation
tbSurface.prototype.drawMultipleExt = function(hNumb, vNumb, dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var xinc = xScale*this.width,
		yinc = yScale*this.height,
		hxinc = xinc*Math.cos(angle*Math.PI/180),
		hyinc = -xinc*Math.sin(angle*Math.PI/180),
		vxinc = yinc*Math.sin(angle*Math.PI/180),
		vyinc = yinc*Math.cos(angle*Math.PI/180),
		i, j;
	for (i=0; i<hNumb; i += 1) {
		for (j=0; j<vNumb; j += 1) {
			this.drawExt(dx+i*hxinc+j*vxinc,dy+i*hyinc+j*vyinc,xScale,yScale,angle,alpha);
		}
	}
};

// Draws the surface tiled in an area with an offset
tbSurface.prototype.drawTiledOffset = function(xOff, yOff, dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha,
		xx, yy, nx, ny, xdest, ydest, subx, suby, subw, subh, i, j;
	if (this.width === 0 || this.height === 0) { return; }
	context.globalAlpha = alpha;
	// Draw it
	xx = Math.round(xOff-this.xOrigin) % this.width;
	if (xx > 0) { xx -= this.width; }
	yy = Math.round(yOff-this.yOrigin) % this.height;
	if (yy > 0) { yy -= this.height; }
	nx = Math.ceil((dw-xx)/this.width);
	ny = Math.ceil((dh-yy)/this.height);
	xx += dx;
	yy += dy;
	for (i=0; i<nx; i += 1) {
		for (j=0; j<ny; j += 1) {
			subx = 0; suby = 0; subw = this.width; subh = this.height;
			xdest = xx+i*this.width;
			ydest = yy+j*this.height;
			if (xdest < dx) { subx += dx-xdest; subw -= dx-xdest; xdest = dx; }
			if (xdest+subw >= dx+dw) { subw = dx+dw-xdest; }
			if (ydest < dy) { suby += dy-ydest; subh -= dy-ydest; ydest = dy; }
			if (ydest+subh >= dy+dh) { subh = dy+dh-ydest; }
			if (subw > 0 && subh > 0) {
				tbGraphics.context.drawImage(this.canvas,subx,suby,subw,subh,xdest,ydest,subw,subh);
			}
		}
	}
	context.globalAlpha = oldalpha;
};

// Draws the surface tiles in an area
tbSurface.prototype.drawTiled = function(dx, dy, dw, dh, alpha) {
	"use strict";
	this.drawTiledOffset(0, 0, dx, dy, dw, dh, alpha);
};

// Draws part of the surface at the destination with an alpha setting
tbSurface.prototype.drawPart = function(sx, sy, sw, sh, dx, dy, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha;
	if (sw <= 0 || sh <= 0) { return; }
	if (sx+sw > this.width) { sw = this.width - sx; }
	if (sy+sh > this.height) { sh = this.height - sy; }
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas,sx, sy, sw, sh, dx, dy, sw, sh);
	context.globalAlpha = oldalpha;
};

// Draw the surface part tiled in an area with an offset
tbSurface.prototype.drawPartTiledOffset = function(xOff, yOff, sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	var xx, yy, nx, ny, xdest, ydest, subx, suby, subw, subh, i, j;
	if (sw <= 0 || sh <= 0) { return; }
	if (sx+sw > this.width) { sw = this.width - sx; }
	if (sy+sh > this.height) { sh = this.height - sy; }
	xx = Math.round(xOff) % sw;
	if (xx > 0) { xx -= sw; }
	yy = Math.round(yOff) % sh;
	if (yy > 0) { yy -= sh; }
	nx = Math.ceil((dw-xx)/sw);
	ny = Math.ceil((dh-yy)/sh);
	xx += dx;
	yy += dy;
	for (i=0; i<nx; i += 1) {
		for (j=0; j<ny; j += 1) {
			subx = sx; suby = sy; subw = sw; subh = sh;
			xdest = xx+i*sw;
			ydest = yy+j*sh;
			if (xdest < dx) { subx += dx-xdest; subw -= dx-xdest; xdest = dx; }
			if (xdest+subw >= dx+dw) { subw = dx+dw-xdest; }
			if (ydest < dy) { suby += dy-ydest; subh -= dy-ydest; ydest = dy; }
			if (ydest+subh >= dy+dh) { subh = dy+dh-ydest; }
			if (subw > 0 && subh > 0) {
				this.drawPart(subx,suby,subw,subh,xdest,ydest,alpha);
			}
		}
	}
};

// Draw part of the surface tiled in an area
tbSurface.prototype.drawPartTiled = function(sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	this.drawPartTiledOffset(0, 0, sx, sy, sw, sh, dx, dy, dw, dh, alpha);
};

// Draws part of the surface stretched in the destination rectangle with an alpha setting
tbSurface.prototype.drawPartStretched = function(sx, sy, sw, sh, dx, dy, dw, dh, alpha) {
	"use strict";
	var context = tbGraphics.context,
		oldalpha = context.globalAlpha;
	if (sw <= 0 || sh <= 0) { return; }
	if (sx+sw > this.width) { sw = this.width - sx; }
	if (sy+sh > this.height) { sh = this.height - sy; }
	if (Math.round(dw) === 0 || Math.round(dh) === 0) { return; }
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas,sx, sy, sw, sh, dx, dy, dw, dh);
	context.globalAlpha = oldalpha;
};

// Draws part of the surface at the destination transformed with an alpha setting
tbSurface.prototype.drawPartExt = function(sx, sy, sw, sh, dx, dy, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context;
	// Solve problem with 0 scale
	if (sw <= 0 || sh <= 0) { return; }
	if (sx+sw > this.width) { sw = this.width - sx; }
	if (sy+sh > this.height) { sh = this.height - sy; }
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; }
	context.save();
	context.translate(dx,dy);
	context.rotate(-angle * Math.PI/180);
	context.scale(xScale, yScale);
	context.globalAlpha = alpha;
	tbGraphics.context.drawImage(this.canvas,sx, sy, sw, sh, 0, 0, sw, sh);
	context.restore();
};

/////////////////////////////////////////////////////////////
// tbFont: The engine part dealing with bitmap and vector fonts
//
// Copyright 2013-2014 CoolGames
// Last Changed: 16-07-2014
/////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////
// BITMAP FONTS
//////////////////////////////////////////////////////

var alignLeft = "left";
var alignCenter = "center";
var alignRight = "right";
var baselineTop = "top";
var baselineMiddle = "middle";
var baselineBase = "base";
var baselineBottom = "bottom";

// The constructor for the font
var tbBitmapFont = function(name) {
	"use strict";
	this.name = name;
	this.align = alignLeft;
	this.baseline = baselineBase;
	this.spacing = 0;
	this.tracking = 0;
};

// Return whether correctly loaded
tbBitmapFont.prototype.complete = function() {
	"use strict";
	return this.image.texture.complete();
};

// Split the string in lines with maximal width w
tbBitmapFont.prototype._split = function(str, w) {
	"use strict";
	// Split in lines
	var lines = [],
		ind, wl, ln, st;
	if (w === undefined) {
		lines = str.split('\n');
		return lines;
	}
	if (w < 0) { w = 0; }
	ind = 0;
	wl = 0;
	lines[0] = "";
	while (ind < str.length) {
		if (str[ind] === '\n') {
			lines.push("");
			wl = 0;
			ind += 1;
		} else {
			ln = lines.length-1;
			wl += this.width[str.charCodeAt(ind)];
			if (wl > w && lines[ln].length > 0) {
				st = lines[ln].length-1;
				while (st >= 0 && lines[ln][st] !== " ") { st -= 1; }
				if (st >= 0) {
					ind = ind - lines[ln].length + st + 1;
					lines[ln] = lines[ln].substr(0,st);
				}
				lines.push("");
				wl = 0;
			} else {
				lines[ln] += str[ind];
				wl += this.tracking;
				ind += 1;
			}
		}
	}
	return lines;
};

// Return the width of a line
tbBitmapFont.prototype._textWidth = function(line) {
	"use strict";
	var  wl = 0, i;
	for (i = line.length-1; i >= 0; i -= 1) {
		wl += this.width[line.charCodeAt(i)] + this.tracking;
	}
	return wl - this.tracking; // Remove the last tracking
};

// Return the width of the string
tbBitmapFont.prototype.textWidth = function(str, w) {
	"use strict";
	var lines = this._split(str, w), wmax = 0, j;
	for (j = lines.length-1; j >= 0; j -= 1) {
		wmax = Math.max(wmax, this._textWidth(lines[j]));
	}
	return wmax;
};

tbBitmapFont.prototype.textHeight = function(str, w) {
	"use strict";
	var lines = this._split(str, w);
	return lines.length * this.height + (lines.length-1) * this.spacing;
};

// Draws the string at the destination
tbBitmapFont.prototype.drawSimple = function(str, dx, dy, w) {
	"use strict";
	// Split in lines
	var lines = this._split(str, w),
		th = lines.length * this.height + (lines.length-1) * this.spacing,
		xst = dx, yst = dy,
		xx, yy, i, j, chr, ind;
	switch (this.baseline) {
		case baselineTop: yst -= this.top; break;
		case baselineMiddle: yst -= this.top + Math.round(th / 2); break;
		case baselineBase: yst -= this.base; break;
		case baselineBottom: yst -= this.top + th; break;
	}
	// Now draw the lines
	yy = yst;
	for (j = 0; j < lines.length; j += 1) {
		xx = xst;
		switch (this.align) {
			case alignLeft: xx = xst; break;
			case alignCenter: xx = xst - Math.round(this._textWidth(lines[j]) / 2); break;
			case alignRight: xx = xst - this._textWidth(lines[j]); break;
		}
		for (i = 0; i < lines[j].length; i += 1) {
			chr = lines[j].charCodeAt(i);
			ind = this.index[chr];
			if (ind >= 0) { this.image.drawSimple(ind, xx-this.left[chr], yy); }
			xx += this.width[chr] + this.tracking;
		}
		yy += this.height + this.spacing;
	}
};

// Draws the string at the destination with an alpha setting
tbBitmapFont.prototype.drawAlpha = function(str, dx, dy, alpha, w) {
	"use strict";
	var context = tbGraphics.context;
	context.save();
	context.globalAlpha = alpha;
	this.drawSimple(str, dx, dy, w);
	context.restore();
};

// Draws the string at the destination transformed and with an alpha setting
tbBitmapFont.prototype.drawExt = function(str, dx, dy, xScale, yScale, angle, alpha, w) {
	"use strict";
	var context = tbGraphics.context;
	if (Math.abs(xScale) < 0.0001 || Math.abs(yScale) < 0.0001) { return; } // Solve problem with 0 scale
	context.save();
	context.translate(dx,dy);
	context.rotate(-angle * Math.PI / 180);     // from degrees to radians and counter-clockwise
	context.scale(xScale,yScale);
	context.globalAlpha = alpha;
	this.drawSimple(str, 0, 0, w/xScale);
	context.restore();
};


//////////////////////////////////////////////////////
// VECTOR FONT FACES
//////////////////////////////////////////////////////

// Create a loader object that is responsible for loading the fontface
var tbVectorFontLoader = function(fontFace, woffPath, ttfPath, loadMoment) {
	"use strict";
	// Info
	this.fontFace = fontFace;
	this.woffPath = woffPath;
	this.ttfPath = ttfPath;
	// Measurement to check whether the font has been loaded
	this._fallbackMeasurements = [
		{ text: "MiHhX!@v&Qq", width: -1, font: "sans-serif" },
		{ text: "MiHhX!@v&Qq", width: -1, font: "serif" },
		{ text: "AaMm#@!Xx67", width: -1, font: "sans-serif" },
		{ text: "AaMm#@!Xx67", width: -1, font: "serif" }
	];
	this.success = false;
	// Add it to the loader
	tbLoader.addResource(this, loadMoment, 1);
};

// Compute the width of a text
tbVectorFontLoader.prototype._computeWidth = function(font, fallback, text) {
	"use strict";
	var width;
	tbGraphics.context.save();
	tbGraphics.context.font = "250pt " + font + ", " + fallback;
	width = tbGraphics.context.measureText(text).width;
	tbGraphics.context.restore();
	return width;
};

// Add the fontface to the style
tbVectorFontLoader.prototype._addToCss = function(location) {
	"use strict";
	// Define css
	var cssCode =	"@font-face {" +
						"font-family: " + this.fontFace + ";" +
						"src: url('" + location + this.woffPath + "') format('woff'), " +
							 "url('" + location + this.ttfPath  + "') format('truetype');" +
					"}",
	// Create CSS element
	styleElement = document.createElement("style");
	styleElement.id = this.fontFace + "_fontface";
	styleElement.type = "text/css";
	if (styleElement.styleSheet) {
		styleElement.styleSheet.cssText = cssCode;
	} else {
		styleElement.appendChild(document.createTextNode(cssCode));
	}
	// Add CSS element to head
	document.getElementsByTagName("head")[0].appendChild(styleElement);
};

// Add an element to the DOM to force loading the font
tbVectorFontLoader.prototype._addToDom = function() {
	"use strict";
	var i, measurement;
	// Create the DOM element to start the loading
	this.DOMElement = document.createElement("span");
	this.DOMElement.style.position = "absolute";
	this.DOMElement.style.left = "-9999px";
	this.DOMElement.style.top = "-9999px";
	this.DOMElement.style.visibility = "hidden";
	this.DOMElement.style.fontSize = "250pt";
	this.DOMElement.id = this.fontFace + "_loader";
	// Add the element to the DOM
	document.body.appendChild(this.DOMElement);
	// Do initial _fallbackMeasurements
	for (i = 0; i < this._fallbackMeasurements.length; i += 1)
	{
		measurement = this._fallbackMeasurements[i];
		measurement.width = this._computeWidth(measurement.font, measurement.font, measurement.text);
	}
};

// This loop checks whether the font is loaded
tbVectorFontLoader.prototype._cycle = function() {
	"use strict";
	var i, measurement, that, width;
	// Do the check
	for (i = 0; i < this._fallbackMeasurements.length; i += 1) {
		measurement = this._fallbackMeasurements[i];
		width = this._computeWidth(this.fontFace, measurement.font, measurement.text);
		// Check whether width is changed (then the font is loaded)
		if(measurement.width !== width) {
			tbLoader.loadComplete(this.loadMoment);
			document.body.removeChild(this.DOMElement);
			this.success = true;
			return;
		}
	}
	// If not loaded, loop
	that = this;
	window.setTimeout(function() { that._cycle(); }, 33);
};

// Start the loading (called by tbLoader) by adding the fontface and an element to the DOM and checking
tbVectorFontLoader.prototype.startLoading = function(moment, location) {
	"use strict";
	this._addToCss(location);
	this._addToDom();
	this.loadMoment = moment;
	this._cycle();
};

//Returns whether loading is complete (true)
tbVectorFontLoader.prototype.complete = function() {
	"use strict";
	return this.success;
};


//////////////////////////////////////////////////////
// VECTOR FONTS
//////////////////////////////////////////////////////

//
// Construction
//

// Constructor for a font with a given fontface and fallback font
var tbVectorFont = function (fontFace, fallbackFontFace) {
	"use strict";
	this.fontFace = fontFace;
	this.fallbackFontFace = fallbackFontFace;
	this.setDefault();
};

// Create a clone of the font with all the same property values
tbVectorFont.prototype.clone = function() {
	"use strict";
	var result = new tbVectorFont(this.fontFace, this.fallbackFontFace);
	//Font
	result.fontStyle = this.fontStyle;
	result.fontWeight = this.fontWeight;
	result.fontCase = this.fontCase;
	result.fontSize = this.fontSize;
	//Fill
	result.fill = this.fill;
	result.fillAlpha = this.fillAlpha;
	result.fillMode = this.fillMode;
	result.fillColor = this.fillColor;
	result.fillPattern = {image: this.fillPattern.image, repeatH: this.fillPattern.repeatH, repeatV: this.fillPattern.repeatV};
	result.fillGradient = {vertical: this.fillGradient.vertical, number:this.fillGradient.number, colors:this.fillGradient.colors.slice(0),
			size: this.fillGradient.size, offset: this.fillGradient.offset};
	result.fillStyle = this.fillStyle;
	//Stoke
	result.stroke = this.stroke;
	result.strokeAlpha = this.strokeAlpha;
	result.strokeMode = this.strokeMode;
	result.strokeColor = this.strokeColor;
	result.strokeStyle = this.strokeStyle;
	result.strokeWidth = this.strokeWidth;
	result.strokeBehind = this.strokeBehind;
	result.strokeJoin = this.strokeJoin;
	//Shadow
	result.shadow = { active: this.shadow.active, color:this.shadow.color, offsetX: this.shadow.offsetX, offsetY: this.shadow.offsetY, blur: this.shadow.blur };
	//Text Alignment
	result.align = this.align;
	result.baseline = this.baseline;
	result.offsetFactor = this.offsetFactor;
	result.spacing = this.spacing;
	return result;
};

//
// Settings
//

// Set all properties to default
tbVectorFont.prototype.setDefault = function() {
	"use strict";
	//Font
	this.fontStyle = "";
	this.fontWeight = "";
	this.fontCase = "normal";
	this.fontSize = 12;
	//Fill
	this.fill = true;
	this.fillAlpha = 1;
	this.fillMode = 0;
	this.fillColor = "black";
	this.fillPattern = {image: undefined, subImage: 0, repeatH: true, repeatV: true};
	this.fillGradient = {vertical: true, number:3, colors:["red","white","blue"], size: 0.6, offset: 0};
	this.fillStyle = undefined;
	//Stroke
	this.stroke = false;
	this.strokeAlpha = 1;
	this.strokeMode = 0;
	this.strokeColor = "black";
	this.strokeStyle = undefined;
	this.strokeWidth = 1.0;
	this.strokeBehind = false;
	this.strokeJoin = "miter";
	//Shadow
	this.shadow = { active: false, color:'rgba(10, 10, 10, 0.3)', offsetX: 3, offsetY: 3, blur: 1 };
	//Text Alignment
	this.align = "left";
	this.baseline = "top";
	this.offsetFactor = 0;
	this.spacing = 0;
};

// Set a number of properties at once
tbVectorFont.prototype.setProperties = function(properties) {
	"use strict";
	// Font Face
	if (properties.fontFace !== undefined) { this.fontFace = properties.fontFace; }
	if (properties.fallbackFontFace !== undefined) { this.fallbackFontFace = properties.fallbackFontFace; }
	//Font
	if (properties.fontStyle !== undefined) { this.fontStyle = properties.fontStyle; }
	if (properties.fontWeight !== undefined) { this.fontWeight = properties.fontWeight; }
	if (properties.fontCase !== undefined) { this.fontCase = properties.fontCase; }
	if (properties.fontSize !== undefined) { this.fontSize = properties.fontSize; }
	//Fill
	if (properties.fill !== undefined) { this.fill = properties.fill; }
	if (properties.fillAlpha !== undefined) { this.fillAlpha = properties.fillAlpha; }
	if (properties.fillColor !== undefined) { this.fillMode = 0; this.fillColor = properties.fillColor; }
	if (properties.fillPattern !== undefined) { this.fillMode = 1; this.fillPattern = properties.fillPattern; }
	if (properties.fillGradient !== undefined) { this.fillMode = 2; this.fillGradient = properties.fillGradient; }
	if (properties.fillStyle !== undefined) { this.fillMode = 3; this.fillStyle = properties.fillStyle; }
	//Stroke
	if (properties.stroke !== undefined) { this.stroke = properties.stroke; }
	if (properties.strokeAlpha !== undefined) { this.strokeAlpha = properties.strokeAlpha; }
	if (properties.strokeColor !== undefined) { this.strokeMode = 0; this.strokeColor = properties.strokeColor; }
	if (properties.strokeStyle !== undefined) { this.strokeMode = 3; this.strokeStyle = properties.strokeStyle; }
	if (properties.strokeWidth !== undefined) { this.strokeWidth = properties.strokeWidth; }
	if (properties.strokeBehind !== undefined) { this.strokeBehind = properties.strokeBehind; }
	if (properties.strokeJoin !== undefined) { this.strokeJoin = properties.strokeJoin; }
	//Shadow
	if (properties.shadow !== undefined) { this.shadow = properties.shadow; }
	//Text Alignment
	if (properties.align !== undefined) { this.align = properties.align; }
	if (properties.baseline !== undefined) { this.baseline = properties.baseline; }
	if (properties.offsetFactor !== undefined) { this.offsetFactor = properties.offsetFactor; }
	if (properties.spacing !== undefined) { this.spacing = properties.spacing; }
};

tbVectorFont.prototype.setFontFace = function(fontFace) {
	"use strict";
	this.fontFace = fontFace;
};

tbVectorFont.prototype.getFontFace = function() {
	"use strict";
	return this.fontFace;
};

tbVectorFont.prototype.setFallbackFontFace = function(fallbackFontFace) {
	"use strict";
	this.fallbackFontFace = fallbackFontFace;
};

tbVectorFont.prototype.getFallbackFontFace = function() {
	"use strict";
	return this.fallbackFontFace;
};

tbVectorFont.prototype.setFontStyle = function(fontStyle) {
	"use strict";
	this.fontStyle = (fontStyle === undefined) ? "" : fontStyle;
};

tbVectorFont.prototype.getFontStyle = function() {
	"use strict";
	return this.fontStyle;
};

tbVectorFont.prototype.setFontWeight = function(fontWeight) {
	"use strict";
	this.fontWeight = (fontWeight === undefined) ? "" : fontWeight;
};

tbVectorFont.prototype.getFontWeight = function() {
	"use strict";
	return this.fontWeight;
};

tbVectorFont.prototype.setFontCase = function(fontCase) {
	"use strict";
	this.fontCase = (fontCase === undefined) ? "normal" : fontCase;
};

tbVectorFont.prototype.getFontCase = function() {
	"use strict";
	return this.fontCase;
};

tbVectorFont.prototype.setFontSize = function(fontSize) {
	"use strict";
	this.fontSize = (fontSize === undefined) ? 12 : fontSize;
};

tbVectorFont.prototype.getFontSize = function() {
	"use strict";
	return this.fontSize;
};

tbVectorFont.prototype.setFill = function(fill) {
	"use strict";
	this.fill = (fill === undefined) ? true : fill;
};

tbVectorFont.prototype.getFill = function() {
	"use strict";
	return this.fill;
};

tbVectorFont.prototype.setFillAlpha = function(alpha) {
	"use strict";
	this.fillAlpha = (alpha === undefined) ? 1 : alpha;
};

tbVectorFont.prototype.getFillAlpha = function() {
	"use strict";
	return this.fillAlpha;
};

tbVectorFont.prototype.setFillColor = function(color) {
	"use strict";
	this.fillMode = 0;
	this.fillColor = (color === undefined) ? "black" : color;
};

tbVectorFont.prototype.getFillColor = function() {
	"use strict";
	return this.fillColor;
};

tbVectorFont.prototype.setFillPattern = function(obj, subImage, repeatH, repeatV) {
	"use strict";
	if (obj === undefined) { return; }
	this.fillMode = 1;
	if (obj instanceof tbImage) {
		this.fillPattern = {image: obj, subImage: subImage, repeatH: repeatH, repeatV: repeatV};
	} else {
		this.fillPattern = {image: obj.image, subImage: obj.subImage, repeatH: obj.repeatH, repeatV: obj.repeatV};
	}
};

tbVectorFont.prototype.getFillPattern = function() {
	"use strict";
	return {image: this.fillPattern.image, subImage: this.fillPattern.subImage, repeatH: this.fillPattern.repeatH, repeatV: this.fillPattern.repeatV	};
};

tbVectorFont.prototype.setFillGradient = function(obj, number, colors, size, offset) {
	"use strict";
	if (obj === undefined) { return; }
	this.fillMode = 2;
	if (obj instanceof Object)
	{
		this.fillGradient.vertical = obj.vertical;
		this.fillGradient.number = obj.number;
		this.fillGradient.colors = obj.colors.slice(0);
		this.fillGradient.size = obj.size;
		this.fillGradient.offset = obj.offset;
	} else {
		this.fillGradient.vertical = obj;
		this.fillGradient.number = number;
		this.fillGradient.colors = colors.slice(0);
		this.fillGradient.size = (size === undefined) ? 0.6 : size;
		this.fillGradient.offset = (offset === undefined) ? 0 : offset;
	}
};

tbVectorFont.prototype.getFillGradient = function() {
	"use strict";
	return {
		vertical: this.fillGradient.vertical,
		number: this.fillGradient.number,
		colors: this.fillGradient.colors.slice(0),
		size: this.fillGradient.size,
		offset: this.fillGradient.offset
	};
};

tbVectorFont.prototype.setFillStyle = function(style) {
	"use strict";
	this.fillMode = 3;
	this.fillStyle = (style === undefined) ? "black" : style;
};

tbVectorFont.prototype.getFillStyle = function() {
	"use strict";
	return this.fillStyle;
};

tbVectorFont.prototype.setStroke = function(stroke) {
	"use strict";
	this.stroke = (stroke === undefined) ? false : stroke;
};

tbVectorFont.prototype.getStroke = function() {
	"use strict";
	return this.stroke;
};

tbVectorFont.prototype.setStrokeAlpha = function(alpha) {
	"use strict";
	this.strokeAlpha = (alpha === undefined) ? 1 : alpha;
};

tbVectorFont.prototype.getStrokeAlpha = function() {
	"use strict";
	return this.strokeAlpha;
};

tbVectorFont.prototype.setStrokeColor = function(color) {
	"use strict";
	this.strokeMode = 0;
	this.strokeColor = (color === undefined) ? "black" : color;
};

tbVectorFont.prototype.getStrokeColor = function() {
	"use strict";
	return this.strokeColor;
};

tbVectorFont.prototype.setStrokeStyle = function(style) {
	"use strict";
	this.strokeMode = 3;
	this.strokeStyle = (style === undefined) ? "black" : style;
};

tbVectorFont.prototype.getStrokeStyle = function() {
	"use strict";
	return this.strokeStyle;
};

tbVectorFont.prototype.setStrokeWidth = function(width) {
	"use strict";
	this.strokeWidth = (width === undefined) ? 1 : width;
};

tbVectorFont.prototype.getStrokeWidth = function() {
	"use strict";
	return this.strokeWidth;
};

tbVectorFont.prototype.setStrokeBehind = function(behind) {
	"use strict";
	this.strokeBehind = (behind === undefined) ? false : behind;
};

tbVectorFont.prototype.getStrokeBehind  = function() {
	"use strict";
	return this.strokeBehind;
};

tbVectorFont.prototype.setStrokeJoin = function(join) {
	"use strict";
	this.strokeJoin = (join === undefined) ? "miter" : join;
};

tbVectorFont.prototype.getStrokeJoin = function() {
	"use strict";
	return this.strokeJoin;
};

tbVectorFont.prototype.setShadow = function(obj, color, offsetX, offsetY, blur) {
	"use strict";
	if (obj === undefined) {
		this.shadow = { active: false, color: 'rgba(10, 10, 10, 0.3)', offsetX: 3, offsetY: 3, blur: 1 };
	} else if (obj instanceof Object)
	{
		this.shadow = { active: obj.active, color: obj.color, offsetX: obj.offsetX, offsetY: obj.offsetY, blur: obj.blur };
	} else if (color === undefined) {
		this.shadow.active = obj;
	} else {
		this.shadow = { active: obj, color: color, offsetX: offsetX, offsetY: offsetY, blur: blur };
	}
};

tbVectorFont.prototype.getShadow = function() {
	"use strict";
	return { active: this.shadow.active, color: this.shadow.color, offsetX: this.shadow.offsetX, offsetY: this.shadow.offsetY, blur: this.shadow.blur };
};

tbVectorFont.prototype.setAlign = function(align) {
	"use strict";
	this.align = (align === undefined) ? "left" : align;
};

tbVectorFont.prototype.getAlign = function() {
	"use strict";
	return this.align;
};

tbVectorFont.prototype.setBaseline = function(baseline) {
	"use strict";
	this.baseline = (baseline === undefined) ? "top" : baseline;
};

tbVectorFont.prototype.getBaseline = function() {
	"use strict";
	return this.baseline;
};

tbVectorFont.prototype.setOffsetFactor = function(offsetFactor) {
	"use strict";
	this.offsetFactor = (offsetFactor === undefined) ? 0 : offsetFactor;
};

tbVectorFont.prototype.getOffsetFactor = function() {
	"use strict";
	return this.offsetFactor;
};

tbVectorFont.prototype.setLineSpacing = function(spacing) {
	"use strict";
	this.spacing = (spacing === undefined) ? 0 : spacing;
};

tbVectorFont.prototype.getLineSpacing = function() {
	"use strict";
	return this.spacing;
};


//
// Size
//

// Create the font string for the font
tbVectorFont.prototype._getFontString = function() {
	"use strict";
	return this.fontStyle + " " + this.fontWeight + " " + this.fontSize + "px " + this.fontFace + ", " + this.fallbackFontFace;
};

// Compute the text width for the collection of lines
tbVectorFont.prototype._textWidth = function(lines) {
	"use strict";
	var largestWidth = 0, li;
	for (li = 0; li < lines.length; li += 1) {
		largestWidth = Math.max(largestWidth, lines[li].width);
	}
	return largestWidth;
};

// Compute the text height for the collection of lines
tbVectorFont.prototype._textHeight = function(lines) {
	"use strict";
	return this.fontSize * lines.length + this.spacing * (lines.length - 1);
};

// Compute the text box for the collection of lines placed at (x,y)
tbVectorFont.prototype._textBox = function(lines, x, y) {
	"use strict";
	var bx = x, by = y,
		bw = this._textWidth(lines),
		bh = this._textHeight(lines);
	switch (this.align){
		case 'right': bx += -bw; break;
		case 'center': bx += -bw/2; break;
	}
	switch (this.baseline){
		case 'base':
		case 'bottom': by += -bh + Math.round(this.offsetFactor * this.fontSize); break;
		case 'middle': by += -bh/2 + Math.round(this.offsetFactor * this.fontSize / 2); break;
	}
    return {x: bx, y: by, width: bw, height: bh};
};

// Split the text in lines at \n and using the optional width
tbVectorFont.prototype._splitText = function(text, width) {
	"use strict";
	var li, wi, currentLine, words, spaceWidth, wordWidth, currentWidth, lines, output = [],
		context = tbGraphics.context;
	context.font = this._getFontString();
	// Adapt case if required
	switch (this.fontCase) {
		case "upper" : text = text.toUpperCase(); break;
		case "lower" : text = text.toLowerCase(); break;
	}
	// Simple case
	if (width === undefined) {
		lines = text.split('\n');
		for (li = 0; li < lines.length; li += 1) {
			output.push({ text: lines[li], width: context.measureText(lines[li]).width });
		}
		return output;
	}
	// split on \n
	lines = text.split('\n');
	//split on width
	spaceWidth = context.measureText(' ').width;
	for (li = 0; li < lines.length; li += 1) {
		words = lines[li].split(' ');
		currentLine = words[0];
		currentWidth = context.measureText(words[0]).width;
		for (wi = 1; wi < words.length; wi += 1) {
			wordWidth = context.measureText(words[wi]).width;
			if (currentWidth + spaceWidth + wordWidth < width) {
				currentLine += ' ' + words[wi];
				currentWidth += spaceWidth + wordWidth;
			} else {
				output.push({ text: currentLine, width: currentWidth });
				currentLine = words[wi];
				currentWidth = wordWidth;
			}
		}
		output.push({ text: currentLine, width: currentWidth });
	}
	return output;
};

// Return the width of the textbox for the current font
tbVectorFont.prototype.textWidth = function(text, width) {
	"use strict";
	var w;
	tbGraphics.context.save();
	w = this._textWidth(this._splitText(text, width));
	tbGraphics.context.restore();
	return w;
};

// Return the height of the textbox for the current font
tbVectorFont.prototype.textHeight = function(text, width) {
	"use strict";
	var h;
	tbGraphics.context.save();
	h = this._textHeight(this._splitText(text, width));
	tbGraphics.context.restore();
	return h;
};

// Return the textbox for the current font when the text is place at x,y
tbVectorFont.prototype.textBox = function(text, x, y, width) {
	"use strict";
	var h;
	tbGraphics.context.save();
	h = this._textBox(this._splitText(text, width), x, y);
	tbGraphics.context.restore();
	return h;
};

// Return the largest font size for the text to fit the box. split indicated whether to split lines
tbVectorFont.prototype._textFitSize = function(size, text, width, height, split) {
	"use strict";
	var oldSize = this.fontSize, lines, result;
	this.fontSize = size;
	if (split) {
		lines = this._splitText(text, width);
	} else {
		lines = this._splitText(text);
	}
	result = this._textWidth(lines) <= width && this._textHeight(lines) <= height;
	this.fontSize = oldSize;
	return result;
};

// Return the largest font size for the text to fit the box. split indicated whether to split lines
tbVectorFont.prototype.textFitSize = function(text, width, height, split) {
	"use strict";
	var size = 0, increment = 32;
	if (split === undefined) { split = false; }
	tbGraphics.context.save();
	while (this._textFitSize(size+increment, text, width, height, split)) { size += increment; }
	while (increment >= 2) {
		increment /= 2;
		if (this._textFitSize(size+increment, text, width, height, split)) { size += increment; }
	}
	tbGraphics.context.restore();
	return Math.max(4, size);
};

//
// Drawing
//

// Create the pattern for drawing the text
tbVectorFont.prototype._createPattern = function() {
	"use strict";
	var img = this.fillPattern.image,
		surface = new tbSurface(img.width, img.height),
		h = this.fillPattern.repeatH,
		v = this.fillPattern.repeatV,
		repeat;
	if (h && v) {
		repeat = "repeat";
	} else if (h && !v) {
		repeat = "repeat-x";
	} else if (!h && v) {
		repeat = "repeat-y";
	} else if (!h && !v) {
		repeat = "no-repeat";
	}
	surface.setTarget();
	img.drawSimple(this.fillPattern.subImage, 0, 0);
	surface.resetTarget();
	return tbGraphics.context.createPattern(surface.canvas, repeat);
};

// Create the gradient for drawing
tbVectorFont.prototype._createGradient = function(bx, by, bw, bh) {
	"use strict";
	var size = Math.max(0.01, this.fillGradient.size),
		offset = this.fillGradient.offset,
		gradient, step, i;
	// Direction
	if (this.fillGradient.vertical) {
		offset = bh/2 + offset*bh;
		size *= 0.5*bh;
		gradient = tbGraphics.context.createLinearGradient(bx, by + offset-size, bx, by + offset+size);
	} else {
		offset = bw/2 + offset*bw;
		size *= 0.5*bw;
		gradient = tbGraphics.context.createLinearGradient(bx + offset-size, by, bx  + offset+size, by);
	}
	// Colors
	step = 1 / (this.fillGradient.number-1);
	for (i = 0; i < this.fillGradient.number; i += 1) {
		gradient.addColorStop(i*step, this.fillGradient.colors[i]);
	}
	return gradient;
};

// Draw the fill
tbVectorFont.prototype._drawFill = function(context, lines, bx, by, bw, bh, alpha) {
	"use strict";
	var li, xOffset, yOffset;
	//Setting the shadow
	if (this.shadow.active) {
		context.shadowColor = this.shadow.color;
		context.shadowOffsetX = this.shadow.offsetX;
		context.shadowOffsetY = this.shadow.offsetY;
		context.shadowBlur = this.shadow.blur;
	} else {
		context.shadowColor = undefined;
		context.shadowOffsetX = 0;
		context.shadowOffsetY = 0;
		context.shadowBlur = 0;
	}
	// Set alpha and fill mode
	context.globalAlpha = alpha * this.fillAlpha;
	switch (this.fillMode) {
		case 0:  context.fillStyle = this.fillColor; break;
		case 1:  context.fillStyle = this._createPattern(); break;
		case 2:  context.fillStyle = this._createGradient(bx, by, bw, bh); break;
		case 3:  context.fillStyle = this.fillStyle; break;
		default: context.fillStyle = this.fillColor; break;
	}
	// Draw the lines
	for (li = 0; li < lines.length; li += 1) {
		xOffset = 0;
		switch (this.align) {
			case 'right': xOffset = (bw - lines[li].width); break;
			case 'center': xOffset = (bw - lines[li].width)/2; break;
		}
		yOffset = this.fontSize * (li + 1) + this.spacing * li;
		if (this.fillMode === 2 && this.fillGradient.vertical) {
			context.fillStyle = this._createGradient(bx, by + yOffset - this.fontSize, bw, this.fontSize);
		}
		context.fillText(lines[li].text, bx + xOffset, by + yOffset);
	}
};

// Draw the stroke
tbVectorFont.prototype._drawStroke = function(context, lines, bx, by, bw, bh, alpha) {
	"use strict";
	var li, xOffset, yOffset;
	//Setting the shadow
	if (!this.fill && this.shadow.active) {
		context.shadowColor = this.shadow.color;
		context.shadowOffsetX = this.shadow.offsetX;
		context.shadowOffsetY = this.shadow.offsetY;
		context.shadowBlur = this.shadow.blur;
	} else {
		context.shadowColor = undefined;
		context.shadowOffsetX = 0;
		context.shadowOffsetY = 0;
		context.shadowBlur = 0;
	}
	// Set alpha and stroke mode
	context.globalAlpha = alpha * this.strokeAlpha;
	switch (this.strokeMode) {
		case 0: context.strokeStyle = this.strokeColor; break;
		case 3: context.strokeStyle = this.strokeStyle; break;
	}
	context.lineWidth = this.strokeWidth;
	context.lineJoin = this.strokeJoin;
	// Draw the lines
	for (li = 0; li < lines.length; li += 1) {
		xOffset = 0;
		switch (this.align) {
			case 'right': xOffset = (bw - lines[li].width); break;
			case 'center': xOffset = (bw - lines[li].width)/2; break;
		}
		yOffset = this.fontSize * (li + 1) + this.spacing * li;
		context.strokeText(lines[li].text, bx + xOffset, by + yOffset);
	}
};

// Do the actual drawing of the font
tbVectorFont.prototype._draw = function(context, text, x, y, alpha, width) {
	"use strict";
	var lines = this._splitText(text, width),
		bx = x, by = y,
		bw = this._textWidth(lines),
		bh = this._textHeight(lines);
	// Compute the correct position
	context.textAlign = 'left';
	context.textBaseline = 'bottom';
	switch (this.align){
		case 'right': bx += -bw; break;
		case 'center': bx += -bw/2; break;
	}
	switch (this.baseline){
		case 'base':
		case 'bottom': by += -bh + Math.round(this.offsetFactor * this.fontSize); break;
		case 'middle': by += -bh/2 + Math.round(this.offsetFactor * this.fontSize/2); break;
	}
	// Setting the font
	context.font = this._getFontString();
	// Now do the drawing
	if (this.stroke && this.strokeBehind) { this._drawStroke(context, lines, bx, by, bw, bh, alpha); }
	if (this.fill) { this._drawFill(context, lines, bx, by, bw, bh, alpha); }
	if (this.stroke && !this.strokeBehind) { this._drawStroke(context, lines, bx, by, bw, bh, alpha); }
};

// Draw simple
tbVectorFont.prototype.drawSimple = function(text, x, y, width) {
	"use strict";
	var context = tbGraphics.context;
	if (this.fill && this.fillMode === 1) {		// For patterns we must always use transformations
		this.drawExt(text, x, y, 1, 1, 0, 1, width);
	} else {
		context.save();
		this._draw(context, text, x, y, 1, width);
		context.restore();
	}
};

// Draw with alpha
tbVectorFont.prototype.drawAlpha = function(text, x, y, alpha, width) {
	"use strict";
	var context = tbGraphics.context;
	if (this.fill && this.fillMode === 1) {		// For patterns we must always use transformations
		this.drawExt(text, x, y, 1, 1, 0, alpha, width);
	} else {
		context.save();
		this._draw(context, text, x, y, alpha, width);
		context.restore();
	}
};

// Draw with transformation
tbVectorFont.prototype.drawExt = function(text, x, y, xScale, yScale, angle, alpha, width) {
	"use strict";
	var context = tbGraphics.context;
	context.save();
	context.translate(x, y);
	context.rotate(-angle * Math.PI/180);
	context.scale(xScale, yScale);
	try {
		this._draw(context, text, 0, 0, alpha, width);
	} catch (e) {
	}
	context.restore();
};
/////////////////////////////////////////////////////////////
// tbAudio: The engine part dealing with audio and sounds
//
// Copyright 2013-2015 CoolGames
// Last Changed: 26-11-2015
/////////////////////////////////////////////////////////////

var TBAudio = function() {
	"use strict";
    //settings
    this.maxNoOfChannels = 10;
    this._maxConSnds = -1;
	this._maxConSndsPolicy = "stop_lowest_prio";
    //determine audio support
    this.usingWebAudio = false;
    this.usingHTMLAudio = false;
    this.audioSupported = false;
	
    //check webAudio
    var audioObj, self = this,
        ContextClass = (typeof AudioContext !== 'undefined')? AudioContext : ((typeof webkitAudioContext !== 'undefined')? webkitAudioContext : undefined);
    if(ContextClass){
        this.usingWebAudio = true;
    }//check html5 audio
    else if(typeof Audio !== 'undefined'){
        try{
            if(typeof (new Audio()).canPlayType !== 'undefined'){
                this.usingHTMLAudio = true;
            }
        }catch(ignore){}
    }
	this.audioSupported = this.usingWebAudio || this.usingHTMLAudio;
	if(this.usingHTMLAudio && tbBrowser.browser.stockandroid){
		this.maxConcurrentSounds(1);
	}
	
    //check browser codec support
    if(this.audioSupported){
        try{
			audioObj = new Audio();
			this._codecs = {
				'ogg': !!audioObj.canPlayType('audio/ogg; codecs="vorbis"').replace(/^no$/, ''),
				'mp3': !!audioObj.canPlayType('audio/mpeg;').replace(/^no$/, ''),
				'opus': !!audioObj.canPlayType('audio/ogg; codecs="opus"').replace(/^no$/, ''),
				'wav': !!audioObj.canPlayType('audio/wav; codecs="1"').replace(/^no$/, ''),
				'm4a': !!(audioObj.canPlayType('audio/x-m4a;') || audioObj.canPlayType('audio/aac;')).replace(/^no$/, ''),
				'mp4': !!(audioObj.canPlayType('audio/x-mp4;') || audioObj.canPlayType('audio/aac;')).replace(/^no$/, ''),
				'weba': !!audioObj.canPlayType('audio/webm; codecs="vorbis"').replace(/^no$/, '')
			};
        }catch(ignore){
			// To solve an issue with the Edge browser on which Audio does not seem to be supported
			this._codecs = {
				'ogg': false,
				'mp3': true,
				'opus': false,
				'wav': false,
				'm4a': false,
				'mp4': false,
				'weba': false
			};
			if (testMode) { console.log("Cannot create Audio object to determine codec support. Only supporting MP3."); }
		}
   }

    //initialise context
    this._activeInstances = [];
    this._inactiveSources = {};
    this._tagVolumeMap = {};
    this._tagVolumeMapBackup = {};
    this._fadingTags = [];
    this._noPlayingInstances = 0;
	if(this.usingWebAudio){
	   //set-up AudioContext
		this._context = new ContextClass();
		this._createGain = (typeof this._context.createGain === "function")? function(){return self._context.createGain();} : ((typeof this._context.createGainNode === "function")? function(){return self._context.createGainNode();} : function(){});
		this._gainNodes = {};
		//create master gain to which all other nodes connect
		this._masterGain = this._createGain();
		if(this._masterGain === undefined){
			this.usingHTMLAudio = true;
			this._loadAudioFile = TBAudio.prototype._loadHTMLAudio;
			return;
		}
		this._masterGain.connect(this._context.destination);
		this._gainNodes["master"] = this._masterGain;

		//set-up loading function
		this._loadAudioFile = TBAudio.prototype._loadWebAudio;
    }else if(this.usingHTMLAudio){
        this._loadAudioFile = TBAudio.prototype._loadHTMLAudio;
    }
    else{
        this._loadAudioFile = function(){};
    }
};

TBAudio.prototype.maxConcurrentSounds = function(max){
    "use strict";
    if(max !== undefined){
        this._maxConSnds = max;
	}
    return this._maxConSnds;
};

TBAudio.prototype.maxConcurrentSoundsPolicy = function(policy){
    "use strict";
    if(policy !== undefined){
        this._maxConSndsPolicy = policy;
    }
    return this._maxConSndsPolicy;
};

TBAudio.prototype._handleStep = function(timePassed){
    "use strict";
    var i, instance, handler, tag, value;
    for(i=0; i<this._activeInstances.length; i+=1){
        instance = this._activeInstances[i];
        if(instance && instance.enginePaused === 0){
            if(!instance.paused){
                instance._timer += timePassed;
                //if instance playback is delayed, check if delay has passed
                if(instance._delayed && instance._timer >= instance.startDelay){
                    instance._delayed = false;
                    this._startPlaying(instance, instance._startOffset);
                }//if instance has reached end sound loop or deactivate
                //not required for web audio because of "onended" event
                else if(instance.playing && this.usingHTMLAudio){
                    if(this.getPosition(instance.id) >= instance.duration ){
                        if(!instance.looping){
                        	instance._source.pause();
                        	tbAudio._handleEndSound(instance);
                    	}else {
                            //web audio is looped automatically, for HTML audio playing pointer has to be reset to start of sound
                        	//check and when necessary wait till audio is in "canPlayThrough" state before updating position
                        	try{
                            	instance._source.pause();
                            	instance._source.currentTime = instance._startOffset/1000;
								if(instance._source.readyState === 4){
									instance._source.play();
								}else{
									handler = (function(){
										var inst = instance;
										return {
											ready: function(){
												inst._source.play();
												inst._source.removeEventListener("canplaythrough", handler.ready, false);
											}
										};
									}());
									instance._source.addEventListener("canplaythrough", handler.ready, false);
								}
							}catch(e){
								if(testMode){
									console.log("error in step when position > duration: " + e.name + ", " + e.message);
								}
							}
                        }
                    }
                }
            }//if instance paused for specific duration, check if duration has passed
            else if(instance._pauseDuration) {
                instance._pauseTimer += timePassed;
                if(instance._pauseTimer >= instance._pauseDuration){
                    this.resume(instance.id);
                }
            }
        }
    }

    //update gain on sounds that are fading
    for(i=this._fadingTags.length-1; i>=0; i-=1){
        tag = this._fadingTags[i];
		if(!this.isPaused(tag.id) && tag.enginePaused === 0){
			tag.timer += timePassed;
			if(tag.timer >= tag.duration){
				tbAudio.setGain(tag.id, tag.targetValue);
				if(this._tagVolumeMapBackup[tag.id] !== undefined){
					this._tagVolumeMapBackup[tag.id] = tag.targetValue;
				}
				if(tag.callback){
					tag.callback();
				}
				this._fadingTags.splice(i, 1);
			}else{
				value = tag.ease(tag.timer, tag.start, tag.targetValue - tag.start, tag.duration);
				tbAudio.setGain(tag.id, value);
				if(this._tagVolumeMapBackup[tag.id] !== undefined){
					this._tagVolumeMapBackup[tag.id] = value;
				}
			}
		}
    }
};

TBAudio.prototype._deactivateInstance = function(instance){
    "use strict";
    //add audio object to inactive sound queue for reuse (without loading)
    if(!this._inactiveSources[instance.sound.sprite.name]){
        this._inactiveSources[instance.sound.sprite.name] = [instance._source];
    }else if(this._inactiveSources[instance.sound.sprite.name].length < this.maxNoOfChannels){
        this._inactiveSources[instance.sound.sprite.name].push(instance._source);
    }
};

TBAudio.prototype._findActiveInstances = function(tag){
    "use strict";
    var i, instance,retVal;
    retVal = [];
    for(i=0; i<this._activeInstances.length; i+=1){
        instance = this._activeInstances[i];
        if(instance){
            if(instance.tags.indexOf(tag) >= 0){
                retVal.push(instance);
            }
        }
    }
    return retVal;
};

TBAudio.prototype._checkPlayPrio = function(sound){
    "use strict";
    if(this._maxConSnds > 0 && this._noPlayingInstances >= this._maxConSnds){
    	switch(this._maxConSndsPolicy){
            case "cancel_new":
                return false;
            case "stop_lowest_prio":
                var i, inst, lowest;
                //find currently playing
                for(i=0; i<this._activeInstances.length; i+=1){
                    inst = this._activeInstances[i];
                    if(inst && inst.playing && !inst.paused){
                        if(lowest === undefined || lowest.priority < inst.priority){
                            lowest = inst;
                        }
                    }
                }
                if(lowest.priority > sound._priority){
                    this.stop(lowest.id);
                    return true;
                }
                return false;
            break;
        }
    }
    return true;
};

TBAudio.prototype._connectWebAudioNode = function(instance){
    "use strict";
	var t, node, totGain = 1;
	for(t=0; t<instance.tags.length; t+=1){
		if(tbAudio._tagVolumeMap[instance.tags[t]] !== undefined){
			totGain *= tbAudio._tagVolumeMap[instance.tags[t]];
		}
	}
	node = this._createGain();
	node.gain.value = totGain;
    node.connect(this._masterGain);
    this._gainNodes[instance.id] = node;
    instance._source.connect(node);
 };

TBAudio.prototype._disconnectWebAudioNode = function(instance){
    "use strict";
    instance._source.disconnect(0);
    if(this._gainNodes[instance.id]){
        this._gainNodes[instance.id].disconnect(0);
        delete this._gainNodes[instance.id];
    }
};

TBAudio.prototype._createSoundSourceInstance = function(sound){
	"use strict";
	var source;
	if(!sound.sprite || !sound.sprite.audio){
		return;
	}
	if(this.usingWebAudio){
		 //create a new source node and add the node to the graph
		source = this._context.createBufferSource();
		source.buffer = sound.sprite.audio;
		source.loopStart = sound.startOffset/1000;
		source.loopEnd = (sound.startOffset + sound.duration)/1000;
		return source;
	}
	if(this.usingHTMLAudio){
		//clone audio node
		source = sound.sprite.audio.cloneNode(true);
		source.volume = 0;
		return source;
		
	}
};

TBAudio.prototype._createSoundInstance = function(sound){
    "use strict";
    var i, instance, sources, s;
    if(this.usingWebAudio){
        s = this._createSoundSourceInstance(sound);
        if(s){
        	instance = new tbSoundInstance(sound,s);
        }
    }else if(this.usingHTMLAudio){
        //check if any audio sources are currently unused,
        //otherwise create new audio source
    	sources = this._inactiveSources[sound.sprite.name];
    	if(!sources){
    		return;
    	}
    	if(sources.length > 0 ){
    		instance = new tbSoundInstance(sound, sources.pop());            
		}else{
			s = this._createSoundSourceInstance(sound);
			if(s){
				instance = new tbSoundInstance(sound, s);
			}
        }         
    }

    //add sound instance to activeInstance array
    if(instance){
        if(this.usingWebAudio){
        	this._connectWebAudioNode(instance);
        }
    	for(i=0; i<this._activeInstances.length; i+=1){
            if(this._activeInstances[i] === undefined){
                this._activeInstances[i] = instance;
                return instance;
            }
        }
        this._activeInstances.push(instance);
    }
	return instance;
};

TBAudio.prototype._selectFormat = function(filenames){
    "use strict";
    var i, ext;
    for(i=0; i<filenames.length; i+=1){
        ext = filenames[i].split('.').pop();
        if(this._codecs[ext]){
            return filenames[i];
        }
    }
    return false;
};

TBAudio.prototype._loadHTMLAudio = function(sprite, urls, moment){
    "use strict";
    var url = tbAudio._selectFormat(urls),
        onerror = function(){
    		tbLoader.loadFailed(moment);
        },
        onload = function(){
        	var audioObjClone;
        	sprite.loaded = true;
        	tbLoader.loadComplete(moment);
        	//round up the duration when using HTML5 Audio to account for the lower precision
        	sprite.duration = Math.ceil(sprite.audio.duration * 1000);
            //remove event handlers
            sprite.audio.removeEventListener("canplaythrough", onload, false);
            sprite.audio.removeEventListener("error", onerror, false);
            //already create one copy of source so it will be pre loaded
            audioObjClone = sprite.audio.cloneNode(true);
            tbAudio._inactiveSources[sprite.name].push(audioObjClone);
        };
    if(url){
    	sprite.audio = new Audio();
    	sprite.audio.src = url;
    	sprite.audio.autoplay = false;
    	sprite.audio.preload = "auto";
    	sprite.audio.addEventListener("canplaythrough", onload, false);
    	sprite.audio.addEventListener("error", onerror, false);
    	sprite.audio.load();
    }else{
        onerror();
    }
};

TBAudio.prototype._loadWebAudio = function(sprite, urls, moment){
    "use strict";
    var url = tbAudio._selectFormat(urls),
        onerror = function(){
            tbLoader.loadFailed(moment);
        },
        onload = function(buffer){
        	sprite.audio = buffer;
            sprite.duration = buffer.duration*1000;
            sprite.loaded = true;
            tbLoader.loadComplete(moment);
        },
        request = new XMLHttpRequest();
     // load the buffer from the URL
    request.open("GET", url, true);
    request.responseType = "arraybuffer";
    request.onload = function() {
        // decode the buffer into an audio source
        tbAudio._context.decodeAudioData(
            request.response,
            function(buffer) {
                if(buffer) {
                    onload(buffer);
                }
            },
            function() {
              onerror();
            }
        );
    };
    request.onerror = function() {
      // if there is an error, switch the sound to HTML Audio
        if(typeof Audio !== "undefined") {
            tbAudio.usingWebAudio = false;
            tbAudio.usingHTMLAudio = true;
            tbAudio._loadAudioFile = TBAudio.prototype._loadHTMLAudio;
            //TODO RELOAD ALL EARLIER SOUNDS AGAIN??
            tbAudio._loadAudioFile(sprite, urls, moment);
        }
    };
    try {
        request.send();
    } catch (e) {}
};

//create and play a new soundInstance
TBAudio.prototype.play = function(item, delay, fadeDuration, fadeEase){
    "use strict";
    if(item instanceof tbSound){
    	if(this._checkPlayPrio(item)){
    		var instance = this._createSoundInstance(item);
    		if(!instance){
    			return -1;
    		}
            instance.startDelay = delay || 0;
            instance._delayed = (delay > 0);
            instance.fadeDuration = fadeDuration || 0;
            instance.fadeEase = fadeEase || function(t,b,c,d){return (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;};
            if(!instance._delayed){
            	this._startPlaying(instance, instance._startOffset);
            }
            return instance.id;
    	}
    	return -1;
    }
};

TBAudio.prototype._startPlaying = function(instance, offset){
    "use strict";
    var position;
    if(typeof offset !== "number"){
		offset = 0;
	}
    this._updateGain(instance.id);
    
	if(instance.fadeDuration > 0){
        var gain = this.getTagGain(instance.id);
        this.setGain(instance.id, 0);
        this.fade(instance.id, gain, instance.fadeDuration, instance.fadeEase);
        instance.fadeDuration = 0;
        instance.fadeEase = undefined;
    }
    if(this.usingWebAudio){
        position = offset - instance._startOffset;
    	instance._startTime = this._context.currentTime*1000 - position;
        //instance._source.loop = instance.looping;
        instance._source.onended = function(){tbAudio._handleEndSound(instance);};
        try{
			//check for old function calls as well as new ones
        	if(instance._source.start){
				instance._source.start(0, offset/1000, (instance.duration - position)/1000);
			}else if(instance._source.noteGrainOn){
				instance._source.noteGrainOn(0, offset/1000, (instance.duration - position)/1000);
			}
			instance._started = true;
			instance.playing = true;
			this._noPlayingInstances += 1;
			//must be called after starting sound for Chrome 42
			instance._source.loop = instance.looping;
		}catch(e){
			if(testMode){
				console.log("error in starting sound: " + e.name + ", " + e.message);
			}
		}
    }else if(this.usingHTMLAudio){
    	if(instance._source.readyState !== 4){
			//if not yet ready to play, wait till can play through then play
    		var handler = (function(){
				var inst = instance;
				return {
					ready: function(){
						inst._source.currentTime = offset/1000;
						inst._source.play();
						inst._started = true;
						inst._source.removeEventListener("canplaythrough", handler.ready, false);
					}
				};
			}());
			instance._source.addEventListener("canplaythrough", handler.ready, false);
		}else{
			//already ready so play immediately
			instance._source.currentTime = offset/1000;
			instance._source.play();
			instance._started = true;
		}
		instance.playing = true;
		this._noPlayingInstances += 1;
    }
};

//resume (or start) active instances that match tag
TBAudio.prototype.resume = function(tag, fadeDuration, fadeEase, enginePause){
    "use strict";
    var i, instance, gain, tag, fadingTag, source,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        if(((instance.paused || !instance.playing) && !enginePause) || (!instance.paused && enginePause)){
            if(!enginePause){
				
                //check to see if instance was already fading before pause
            	for(i=this._fadingTags.length-1; i>=0; i-=1){
                    tag = this._fadingTags[i];
            		if(tag.id === instance.id){
            			fadingTag = tag;
            			fadeDuration = 0;
            			fadeEase = undefined;
            			break;
            		}
            	}
            	instance.paused = false;
				instance.fadeDuration = fadeDuration || 0;
				instance.fadeEase = fadeEase || function(t,b,c,d){return (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;};
				if(instance._fadedOnPause){
					if(fadeDuration === undefined){
						instance.fadeDuration = instance._fadedOnPause.duration;
					}
					if(fadeEase === undefined){
						instance.fadeEase = instance._fadedOnPause.ease;
					}
					gain = instance._fadedOnPause.gain;
					instance._fadedOnPause = undefined;
				}
			}
            if(this.usingWebAudio){
            	//audio node must be recreated, after sound being stopped on pause it cannot be restarted
            	source = this._createSoundSourceInstance(instance.sound);
            	if(source){
                	instance._source = source;
                    this._connectWebAudioNode(instance);
            	}
            }
            if(gain !== undefined){
            	tbAudio.setGain(instance.id, gain);
            }
            
            //the progress in the sound already made before pausing must be given as a parameter
        	this._startPlaying(instance, instance._startOffset + (instance._pausedPosition || 0));
            
            if(fadingTag !== undefined){
            	tbAudio.setGain(instance.id, fadingTag.ease(fadingTag.timer, fadingTag.start, fadingTag.targetValue - fadingTag.start, fadingTag.duration));
            	tbAudio.fade(instance.id, fadingTag.targetValue, fadingTag.duration - fadingTag.timer, fadingTag.ease, fadingTag.callback);
            }
        }
    }
};

//pause playing active instances that match tag
TBAudio.prototype.pause = function(tag, duration, fadeDuration, fadeEase, enginePause){
    "use strict";
    var i, instance, position,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        if(!instance.paused){
			instance.fadeDuration = fadeDuration || 0;
            if(instance.fadeDuration > 0){
				instance.fadeEase = fadeEase || function(t,b,c,d){return (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;};
				//remember gain previous to fading so it can be reset on resume
				instance._fadedOnPause = {gain : tbAudio.getGain(instance.id), duration : instance.fadeDuration, ease: instance.fadeEase};
				tbAudio.fade(instance.id, 0, instance.fadeDuration, instance.fadeEase, function(){tbAudio.pause(instance.id, duration);});
            }else{
            	position = this.getPosition(instance.id);
            	instance._pausedPosition = position;
            	if(!enginePause){
					instance.paused = true;
					instance._pauseTimer = 0;
					instance._pauseDuration = duration;
					this._noPlayingInstances -= 1;
				}
				if(this.usingWebAudio){
					instance._source.onended = function(){};
					if(instance.playing){
						if(instance._started){
							try{
								if(instance._source.stop){
									instance._source.stop(0);
								}else if(instance._source.noteOff){
									instance._source.noteOff(0);
								}
							}catch(e){
								if(testMode){
									console.log("error in stopping sound for pause: "+ e.name + ", " + e.message);								
								}
							}
							instance._started = false;
						}
					}
					this._disconnectWebAudioNode(instance);
				}
				else if(this.usingHTMLAudio){
					instance._source.pause();
				}
			}
        }
    }
};

TBAudio.prototype._handleEndSound = function(instance){
    "use strict";
	if(this._tagVolumeMap[instance.id]){
		delete this._tagVolumeMap[instance.id];
	}
	if(!instance.paused){
		this._noPlayingInstances -= 1;
	}
	if(this.usingWebAudio){
		instance._started = false;
		instance.playing = false;
		this._disconnectWebAudioNode(instance);
	}
	else if(this.usingHTMLAudio){
		//store source for future use
		this._deactivateInstance(instance);
	}
	//remove instance from active instances list
	this._activeInstances[this._activeInstances.indexOf(instance)] = undefined;
};

TBAudio.prototype.stop = function(tag, fadeDuration, fadeEase){
    "use strict";
    var i, instance,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        instance.fadeDuration = fadeDuration || 0;
		if(instance.fadeDuration > 0){
			instance.fadeEase = fadeEase || function(t,b,c,d){return (t==0) ? b : c * Math.pow(2, 10 * (t/d - 1)) + b;};
			tbAudio.fade(instance.id, 0, instance.fadeDuration, instance.fadeEase, function(){tbAudio.stop(instance.id);});
		}else{
			//remove instances from volume map
			if(this._tagVolumeMap[instance.id]){
				delete this._tagVolumeMap[instance.id];
			}
			//update number of sounds playing
			if(instance.playing && !instance.paused){
				this._noPlayingInstances -= 1;
			}
			if(this.usingWebAudio){
				if(instance.playing && !instance.paused && !instance._delayed){
					//make sure sound has already started otherwise an error may occur
					if(instance._started){
						try{
							if(instance._source.stop){
								instance._source.stop(0);
							} else if(instance._source.noteOff){
								instance._source.noteOff(0);
							}
						}catch(e){
							if(testMode){
								console.log("error in stopping sound: " + e.name +", " + e.message);
							}
						}
						instance._started = false;
					}
					this._disconnectWebAudioNode(instance);
				}
			}
			else if(this.usingHTMLAudio){
			    if(!instance._delayed){
			    	instance._source.pause();
			    }
				//reset instance and store for future use
				this._deactivateInstance(instance);
			}
			//remove instance from active instances list
			this._activeInstances[this._activeInstances.indexOf(instance)] = undefined;
			instance.playing = false;
		}
    }
};

TBAudio.prototype.enginePause = function(tag){
    "use strict";
    var i, instance,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
		if(instance.enginePaused === 0){
			tbAudio.pause(instance.id, 0, 0, null, true);
		}
		instance.enginePaused += 1;
    }
	for(i=0; i<this._fadingTags.length; i+=1){
        if(tag === 'master' || this._fadingTags[i].id === tag){
            this._fadingTags[i].enginePaused += 1;
		}
    }
};

TBAudio.prototype.engineResume = function(tag){
    "use strict";
    var i, instance,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
		instance.enginePaused = Math.max(instance.enginePaused - 1, 0);
		if(instance.enginePaused === 0){
			tbAudio.resume(instance.id, 0, null, true);
		}
	}
	for(i=0; i<this._fadingTags.length; i+=1){
        if(tag === 'master' || this._fadingTags[i].id === tag){
            this._fadingTags[i].enginePaused = Math.max(this._fadingTags[i].enginePaused - 1, 0);
		}
    }
};

TBAudio.prototype.engineResumeForced = function(tag){
    "use strict";
    var i,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        items[i].enginePaused = 0;
		tbAudio.resume(items[i].id, 0, null, true);
    }
	for(i=0; i<this._fadingTags.length; i+=1){
        if(tag === 'master' || this._fadingTags[i].id === tag){
            this._fadingTags[i].enginePaused = 0;
		}
    }
};


TBAudio.prototype.fade = function(tag, targetValue, duration, easeFunction, callback){
    "use strict";
    var i;
    for(i=0; i<this._fadingTags.length; i+=1){
        if(this._fadingTags[i].id === tag){
            this._fadingTags.splice(i, 1);
            break;
        }
    }
    this._fadingTags.push({id:tag, targetValue:targetValue, ease:easeFunction || function(t,b,c,d){return (t==d) ? b+c : c * (-Math.pow(2, -10 * t/d) + 1) + b;},
            duration:duration, timer:0, start:this.getTagGain(tag), callback: callback, enginePaused: 0});
};

TBAudio.prototype.mute = function(tag){
    "use strict";
    var oldVolume;
    if(this._tagVolumeMapBackup[tag] === undefined){
    	oldVolume = (this._tagVolumeMap[tag] !== undefined)?  this._tagVolumeMap[tag] : 1;
    	//set tag gain value to 0
        this.setGain(tag, 0);
    	//store previous gain value of tag
    	this._tagVolumeMapBackup[tag] = oldVolume;
    }
};

TBAudio.prototype.unmute = function(tag){
    "use strict";
    //if a previous gain value was stored, reset tag gain to this value
    if(this._tagVolumeMapBackup[tag] !== undefined){
        this.setGain(tag, this._tagVolumeMapBackup[tag]);
        delete this._tagVolumeMapBackup[tag];
    }
};

TBAudio.prototype.position = function(tag, position){
    "use strict";
    var i, instance, source,
        setPos,
        items = this._findActiveInstances(tag);
    if(!isNaN(position) && position >= 0){
    	for(i=0; i<items.length; i++){
    		instance = items[i];
    		position = position % instance.duration;
            if(this.usingWebAudio){
            	 //cannot go back in buffer so stop current buffer and create new one that starts at given position
            	if(!instance.paused){
            		instance._source.onended = function(){};
            		//make sure sound is already started before trying to stop it
                	if(instance._started){
						try{
							if(instance._source.stop){
								instance._source.stop(0);
							} else if(instance._source.noteOff){
								instance._source.noteOff(0);
							}
						}
						catch(e){
							if(testMode){
								console.log("error in stopping sound for position: " + e.name + ", " +e.message);  
							}
						}
						instance._started = false;
					}
                    this._disconnectWebAudioNode(instance);
                    this._noPlayingInstances -= 1;
                    //create new buffer and start playing from requested position
                    source = this._createSoundSourceInstance(instance.sound);
                    if(source){
                        instance._source = source;
                        this._connectWebAudioNode(instance);
                        this._startPlaying(instance, instance._startOffset + position);
                    }
                }else{
                    instance._pausedPosition = position;
                }
            }else if(this.usingHTMLAudio){
                //check and when necessary wait for sound to be "canPlayThrough" then jump to requested position
            	if(instance._source.readyState === 4){
                    instance._source.currentTime = (instance._startOffset + position)/1000;
                }else{
                    (setPos = function(){
                        var inst = instance,
                            pos = position;
                        return{
                            handler: function(){
                                inst._source.currentTime = (inst._startOffset + pos)/1000;
                                inst._source.removeEventListener("canplaythrough", setPos.handler, false);
                            }
                        };
                    }());
                    instance._source.addEventListener("canplaythrough", setPos.handler, false);
                }
            }
        }
    }
};

TBAudio.prototype.rewind = function(tag){
    "use strict";
    tbAudio.position(tag, 0);
};

TBAudio.prototype.setLooping = function(tag, looping){
    "use strict";
    var i,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        items[i].looping = looping;
		if(this.usingWebAudio){
			//set property on buffer so webaudio api will automatically take care of the looping
			items[i]._source.loop = looping;
		}
    }
};

TBAudio.prototype.setPriority = function(tag, priority){
    "use strict";
    var i,
        items = this._findActiveInstances(tag);
    for(i=0; i<items.length; i+=1){
        items[i].priority = priority;
    }
};

TBAudio.prototype.getInstances = function(tag){
    "use strict";
     var i, instance,
        retVal = [];
    for(i=0; i<this._activeInstances.length; i+=1){
        instance = this._activeInstances[i];
        if(instance){
            if(instance.tags.indexOf(tag) >= 0){
                retVal.push(instance.id);
            }
        }
    }
    return retVal;
};

TBAudio.prototype.getTagGain = function(tag){
    "use strict";
    if(this._tagVolumeMap[tag] !== undefined){
        return this._tagVolumeMap[tag];
    }
    return 1;
};

TBAudio.prototype.getGain = function(id){
    "use strict";
    var t,
        totGain = 1,
        instance = this._findActiveInstances(id)[0];
    if(instance){
        for(t=0; t<instance.tags.length; t+=1){
            if(this._tagVolumeMap[instance.tags[t]] !== undefined){
				totGain *= this._tagVolumeMap[instance.tags[t]];
			}
        }
    }
    return Math.round(100*totGain)/100;
};

TBAudio.prototype.setGain = function(tag, volume){
    "use strict";
    var i, instance, t,
		totGain = 1,
        items = this._findActiveInstances(tag);
    //update tagVolumeMap
    this._tagVolumeMap[tag] = volume;
    //unmute sound if was muted
    if(this._tagVolumeMapBackup[tag]){
    	delete this._tagVolumeMapBackup[tag];
    }
    //update volume of all instances that have tag
    for(i=0; i<items.length; i+=1){
    	instance = items[i];
    	if(instance.tags.indexOf(tag) >= 0){
    		for(t=0; t<instance.tags.length; t+=1){
                if(this._tagVolumeMap[instance.tags[t]] !== undefined){
    				totGain *= this._tagVolumeMap[instance.tags[t]];
    			}
            }
            totGain = Math.round(100*totGain)/100;
            if(this.usingWebAudio){
                this._gainNodes[instance.id].gain.value = totGain;
            }else if(this.usingHTMLAudio){
                instance._source.volume = totGain;
            }
    	}
    }
};

TBAudio.prototype._updateGain = function(tag){
	"use strict";
    var i, instance, t,
	totGain = 1,
    items = this._findActiveInstances(tag);
    
    //update volume of all instances that have tag
    for(i=0; i<items.length; i+=1){
    	instance = items[i];
		for(t=0; t<instance.tags.length; t+=1){
            if(this._tagVolumeMap[instance.tags[t]] !== undefined){
				totGain *= this._tagVolumeMap[instance.tags[t]];
			}
        }
        totGain = Math.round(100*totGain)/100;
        if(this.usingWebAudio){
            this._gainNodes[instance.id].gain.value = totGain;
        }else if(this.usingHTMLAudio){
            instance._source.volume = totGain;
        }
	}
};

TBAudio.prototype.getTags = function(id){
    "use strict";
    var i, instance, t, tag,
        items = this._findActiveInstances(id),
        retVal = [];
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        for(t=0; t<instance.tags.length; t+=1){
            tag = instance.tags[t];
            if(retVal.indexOf(tag) < 0){
                retVal.push(tag);
            }
        }
    }
    return retVal;
};

TBAudio.prototype.addTags = function(id, tags){
    "use strict";
    var i, instance, t,
        items = this._findActiveInstances(id);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        tags = [].concat(tags);
        for(t=0; t<tags.length; t+=1){
            if(instance.tags.indexOf(tags[t]) < 0){
                instance.tags.push(tags[t]);
            }
        }
    }
    this._updateGain(id);
};

TBAudio.prototype.removeTags = function(id, tags){
    "use strict";
    var i, instance, t, tag, idx,
        items = this._findActiveInstances(id);
    for(i=0; i<items.length; i+=1){
        instance = items[i];
        tags = [].concat(tags);
        for(t=0; t<tags.length; t+=1){
            tag = tags[t];
            if(tag !== instance.id && tag !== 'master'){
                idx = instance.tags.indexOf(tag);
                if(idx >= 0){
                    instance.tags.splice(idx, 1);
                }
            }
        }
    }
    return instance.tags;
};


TBAudio.prototype.isPaused = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return instance.paused;
    }
};

TBAudio.prototype.isPlaying = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return instance.playing;
    }
};

TBAudio.prototype.isActive = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return true;
    }
    return false;
};

TBAudio.prototype.isLooping = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return instance.looping;
    }
};

TBAudio.prototype.getFading = function(id){
    "use strict";
    var i;
    for(i=0; i<this._fadingTags.length; i+=1){
        if(this._fadingTags[i].id === id){
            return this._fadingTags[i].targetValue;
        }
    }
    return -1;
};

TBAudio.prototype.getPriority = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return instance.priority;
    }
};

TBAudio.prototype.getDuration = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        return instance.sound.duration;
    }
};

TBAudio.prototype.getPosition = function(id){
    "use strict";
    var instance = this._findActiveInstances(id)[0];
    if(instance){
        if(this.usingWebAudio){
            if(instance.paused){
                return instance._pausedPosition;
            }
            return (tbAudio._context.currentTime*1000 - instance._startTime)%instance.duration;
        }
        if(tbAudio.usingHTMLAudio){
            return Math.ceil(instance._source.currentTime*1000 - instance._startOffset);
        }   
    }
};

//create the tbAudio object
var tbAudio = new TBAudio();

//
// AudioSprites
//

//The constructor for a tbAudioSprite object
var tbAudioSprite = function(name, mp3Name, oggName, loading){
	"use strict";
	this.name = name;
	this.mp3Name = mp3Name;
	this.oggName = oggName;
	this.loadingMoment = loading;
    this.loaded = false;
    this.audio = null;
    tbLoader.addResource(this, this.loadingMoment, 1);
};

tbAudioSprite.prototype.startLoading = function(moment, location){
	"use strict";
	var fnameMp3, fnameOgg;
	fnameMp3 = this.mp3Name;
	if (fnameMp3.toLowerCase().indexOf("http:") !== 0 && fnameMp3.toLowerCase().indexOf("https:") !== 0) {
		fnameMp3 = location + fnameMp3;
	}
	fnameOgg = this.oggName;
	if (fnameOgg.toLowerCase().indexOf("http:") !== 0 && fnameOgg.toLowerCase().indexOf("https:") !== 0) {
		fnameOgg = location + fnameOgg;
	}
	tbAudio._inactiveSources[this.name] = [];
    tbAudio._loadAudioFile(this, [fnameOgg, fnameMp3], moment);
};

tbAudioSprite.prototype.complete = function(){
	"use strict";
	return this.loaded;
};

//
// Sounds
//

// The constructor for a tbSound object
var tbSound = function(name, sprite, offset, duration, gain, priority, tags) {
	"use strict";
	// Copy the arguments
	this.name = name;
	this.sprite = sprite;
	this.startOffset = offset;
	this.duration = duration;

    tbAudio.setGain(this.name, ((gain !== undefined)? gain: 1));
    this._priority = (priority !== undefined)? priority : 10;
    this.tags = [];
    if(tags){
        this.tags = this.tags.concat(tags);
    }
    if(this.tags.indexOf(this.name) < 0){
        this.tags.push(this.name);
    }
};

tbSound.prototype.complete = function(){
	"use strict";
	return this.sprite.complete();
};

// Gets or Sets the priority of the sound object
tbSound.prototype.priority = function(priority) {
	"use strict";
	if(priority !== undefined){
		this._priority = priority;
	}
	return this._priority;
};

// Gets the tags of the sound object
tbSound.prototype.getTags = function() {
	"use strict";
	//return shallow copy of array
	return this.tags.slice(0);
};

// Adds tags to the sound object
tbSound.prototype.addTags = function(tags) {
	"use strict";
	var t;
	tags = [].concat(tags);
	for(t=0; t<tags.length; t+=1){
		if(this.tags.indexOf(tags[t]) < 0){
			this.tags.push(tags[t]);
		}
	}
};

// Removes tags from the sound object
tbSound.prototype.removeTags = function(tags) {
	"use strict";
	var t, tag, idx;
	tags = [].concat(tags);
	for(t=0; t<tags.length; t+=1){
		tag = tags[t];
		if(tag !== this.name){
			idx = this.tags.indexOf(tag);
			if(idx >= 0){
				this.tags.splice(idx, 1);
			}
		}
	}
};

//
// SoundInstances
//

// The constructor for a soundInstance
var tbSoundInstance = function(sound, source) {
	"use strict";
	this.sound = sound;
    this._startOffset = this.sound.startOffset;
    this._endTime = this._startOffset + this.sound.duration;
    this._source = source;
    this.duration = this.sound.duration;
    this._init();
};

tbSoundInstance.prototype._init = function(){
    "use strict";
    this.id =  Math.round(Date.now() * Math.random()) + "";
    this.tags = ["master", this.id].concat(this.sound.tags);
    this.priority = (this.sound._priority !== undefined)? this.sound._priority : 10;
    this.looping = false;
    this.playing = false;
    this.paused = false;
	this.enginePaused = 0;
    this._timer = 0;
    this._delayed = false;
	this._started = false;
	this._pausedPosition = 0;
    this.startDelay = 0;

	//set initial gain
	var t, totGain = 1;
	for(t=0; t<this.tags.length; t+=1){
		if(tbAudio._tagVolumeMap[this.tags[t]] !== undefined){
			totGain *= tbAudio._tagVolumeMap[this.tags[t]];
		}
	}
	if(tbAudio.usingWebAudio){
		//tbAudio._gainNodes[this.id].gain.value = totGain;
	}else if(tbAudio.usingHTMLAudio){
		this._source.volume = totGain;
	}
};/////////////////////////////////////////////////////////////
// tbData: The engine part dealing with data files
//
// Copyright 2013 CoolGames
// Last Changed: 9-9-2013
/////////////////////////////////////////////////////////////

// The constructor for the data file object
var tbDataFile = function(name, fileName, info) {
	"use strict";
	this.name = name;
	this.fileName = fileName;
	this.info = info;
};


// The constructor for the text object
var tbText = function(name) {
	"use strict";
	this.name = name;
	this.text = "";						// The text
	this.complete = false;
	this.failed = false;
};

// Loading state changed
tbText.prototype._loaded = function(request) {
	"use strict";
	if (request.readyState === 4) {
		this.complete = true;
		this.failed = (request.status !== 200);
		if (this.failed) {
			tbGame.addSpecial("Get Failed", {name: this.name} );
		} else {
			this.text = request.responseText;
			tbGame.addSpecial("Get Complete", {name: this.name} );
		}
	}
};

// Posting state changed
tbText.prototype._posted = function(request) {
	"use strict";
	if (request.readyState === 4) {
		this.complete = true;
		this.failed = (request.status !== 200);
		if (this.failed) {
			tbGame.addSpecial("Post Failed", {name: this.name} );
		} else {
			tbGame.addSpecial("Post Complete", {name: this.name} );
		}
	}
};

// Get the text from an url
tbText.prototype.get = function(url) {
	"use strict";
	var self = this,
		request = new XMLHttpRequest(),
		ready = function() { self._loaded(request); };
	this.complete = false;
	request.open("GET", url);
	request.onreadystatechange = ready;
	request.send(null);
};

// Post the text to an url
tbText.prototype.post = function(url) {
	"use strict";
	var self = this,
		request = new XMLHttpRequest(),
		ready = function() { self._posted(request); };
	this.complete = false;
	request.open("POST", url);
	request.setRequestHeader("Content-Type", "text/plain;charset=UTF-8");
	request.onreadystatechange = ready;
	request.send(this.text);
};

// Get the text from an url synchronous
tbText.prototype.getSynchronous = function(url) {
	"use strict";
	var request = new XMLHttpRequest();
	request.open("GET", url, false);
	try {
		request.send();
	} catch (e) {
		return false;
	}
	this.complete = true;
	this.failed = (request.status !== 200);
	if (this.failed) {
		return false;
	}
	this.text = request.responseText;
	return true;
};

// Post the text to an url synchronous
tbText.prototype.postSynchronous = function(url) {
	"use strict";
	var request = new XMLHttpRequest();
	request.open("POST", url);
	request.setRequestHeader("Content-Type", "text/plain;charset=UTF-8");
	try {
		request.send(this.text);
	} catch(e) {
		return false;
	}
	this.complete = true;
	this.failed = (request.status !== 200);
	return (!this.failed);
};

// Parse the text as a JSON object
tbText.prototype.parseJSON = function() {
	"use strict";
	if (JSON === undefined) {
		return ({ });
	} else {
		return JSON.parse(this.text);
	}
};

// Stringify an object into a JSON text
tbText.prototype.stringifyJSON = function(obj) {
	"use strict";
	if (JSON === undefined) {
		this.text = "";
	} else {
		this.text = JSON.stringify(obj);
	}
};

tbText.prototype._browserSupportsLocalStorage = function() {
	try {
		return 'localStorage' in window && window['localStorage'] !== null;
	} catch (e) {
		return false;
	}
};

tbText.prototype.localStorageStore = function(key) {
	"use strict";

	if (this._browserSupportsLocalStorage()) {
		var localStorage = window.localStorage;
		localStorage.setItem(key || this.name, this.text);
	}
};

tbText.prototype.localStorageRetrieve = function(key) {
	"use strict";

	if (this._browserSupportsLocalStorage()) {
		var localStorage = window.localStorage;
		this.text = localStorage.getItem(key || this.name);
	}
	else {
		this.text = null;
	}
};

/////////////////////////////////////////////////////////////
// tbGame: The game object that handles the main loop and
// all the event processing
//
// Copyright 2013-2015 CoolGames
// Last Changed: 3-9-2015
/////////////////////////////////////////////////////////////

var TBGame = function(mainCanvas) {
	"use strict";
	if (mainCanvas) { this.mainCanvas = mainCanvas; }
	// Initialize the tokens
	this.clear();
	// Initialize the event handlers
	var self = this;
	this._initEventCounters();
	this._initClick(self);
	this._initTouch(self);
	this._initKeyboard(self);
	this._initAlarm(self);
	this._initSpecial(self);
	this._initPause(self);
	this._initLoop(self);
};

// Clears the game state, removing all tokens and elements
TBGame.prototype.clear = function() {
	"use strict";
	this._tokens = [];
	this._tokenOrderChanged = false;
	this._elements = [];
	this._elementOrderChanged = false;
};

// Clears all events
TBGame.prototype.clearAllEvents = function() {
	"use strict";
	this.clearClickEvents();
	this.clearTouchEvents();
	this.clearKeyboardEvents();
	this.clearAlarmEvents();
	this.clearSpecialEvents();
};

// Sets the main canvas (required for finding touch event positions)
TBGame.prototype.setMainCanvas = function(mainCanvas) {
	"use strict";
	this.mainCanvas = mainCanvas;
};

// ********************************************
// ** Event counters
// ** For each class of events we maintain the number of tokens that
// ** use it. We only run through the array when this number is positive
// ********************************************

// Initialize the event counters
TBGame.prototype._initEventCounters = function() {
	"use strict";
	this._numberBeginStepEvents = 0;
	this._numberClickEvents = 0;
	this._numberTouchEvents = 0;
	this._numberKeyEvents = 0;
	this._numberAlarmEvents = 0;
	this._numberSpecialEvents = 0;
	this._numberEndStepEvents = 0;
	this._numberImmediateClickEvents = 0;
	this._numberImmediateTouchEvents = 0;
	this._numberImmediateKeyEvents = 0;
	this._numberImmediateSpecialEvents = 0;
};

// Add to the event counters
TBGame.prototype._addEventCounters = function(token) {
	"use strict";
	if (token.handleBeginStep) { this._numberBeginStepEvents += 1; }
	if (token.handleClick) { this._numberClickEvents += 1; }
	if (token.handleTouchDown || token.handleTouchUp) { this._numberTouchEvents += 1; }
	if (token.handleMouseWheel) { this._numberTouchEvents += 1; }
	if (token.handleKeyDown || token.handleKeyUp) { this._numberKeyEvents += 1; }
	if (token.handleAlarm) { this._numberAlarmEvents += 1; }
	if (token.handleSpecial) { this._numberSpecialEvents += 1; }
	if (token.handleEndStep) { this._numberEndStepEvents += 1; }
	if (token.handleImmediateClick) { this._numberImmediateClickEvents += 1; }
	if (token.handleImmediateTouchDown || token.handleImmediateTouchUp) { this._numberImmediateTouchEvents += 1; }
	if (token.handleImmediateMouseWheel) { this._numberImmediateTouchEvents += 1; }
	if (token.handleImmediateKeyDown || token.handleImmediateKeyUp) { this._numberImmediateKeyEvents += 1; }
	if (token.handleImmediateSpecial) { this._numberImmediateSpecialEvents += 1; }
};

// Subtract from the event counters
TBGame.prototype._subtractEventCounters = function(token) {
	"use strict";
	if (token.handleBeginStep) { this._numberBeginStepEvents -= 1; }
	if (token.handleClick) { this._numberClickEvents -= 1; }
	if (token.handleTouchDown || token.handleTouchUp) { this._numberTouchEvents -= 1; }
	if (token.handleMouseWheel) { this._numberTouchEvents -= 1; }
	if (token.handleKeyDown || token.handleKeyUp) { this._numberKeyEvents -= 1; }
	if (token.handleAlarm) { this._numberAlarmEvents -= 1; }
	if (token.handleSpecial) { this._numberSpecialEvents -= 1; }
	if (token.handleEndStep) { this._numberEndStepEvents -= 1; }
	if (token.handleImmediateClick) { this._numberImmediateClickEvents -= 1; }
	if (token.handleImmediateTouchDown || token.handleImmediateTouchUp) { this._numberImmediateTouchEvents -= 1; }
	if (token.handleImmediateMouseWheel) { this._numberImmediateTouchEvents -= 1; }
	if (token.handleImmediateKeyDown || token.handleImmediateKeyUp) { this._numberImmediateKeyEvents -= 1; }
	if (token.handleImmediateSpecial) { this._numberImmediateSpecialEvents -= 1; }
};

// ********************************************
// ** Dealing with the click
// ********************************************

// Initialize the click handling
TBGame.prototype._initClick = function(self) {
	"use strict";
	window.addEventListener('click', function(e) {self._doClick(e); } , false);
	this.clearClickEvents();
};

// Clears the click events
TBGame.prototype.clearClickEvents = function() {
	"use strict";
	this._clickInfo = {x:0, y:0};
	this._clickAction = [];				// The clicks since the last step
};

// Add a click event
TBGame.prototype.addClickEvent = function(x, y) {
	"use strict";
	var i, tok, handled;
	this._clickInfo.x = x;
	this._clickInfo.y = y;
	this._clickAction.push( { x: this._clickInfo.x, y: this._clickInfo.y } );
	// Handle the immediate events
	if (this._numberImmediateClickEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateClick) {
				handled = tok.handleImmediateClick(this._clickInfo.x, this._clickInfo.y);
				if (handled === true) { break; }
			}
		}
	}
};

// Handle a click event
TBGame.prototype._doClick = function(e) {
	"use strict";
	var canvas, bbox, x, y;
	if (this.mainCanvas === undefined) { return; }
	if (this._paused > 0) { return; }
	canvas = this.mainCanvas;
	bbox = canvas.getBoundingClientRect();
	x = (e.clientX - bbox.left) * (canvas.width / bbox.width);
	y = (e.clientY - bbox.top) * (canvas.height / bbox.height);
	e.preventDefault();
	this.addClickEvent(x, y);
};

// Send the correct click events to the tokens
TBGame.prototype._handleClickEvents = function() {
	"use strict";
	var c, i, tok, act;
	if (this._clickAction.length === 0) { return; }
	// Send the events to the tokens
	if (this._numberClickEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleClick) {
				for (c = 0; c < this._clickAction.length; c += 1) {
					act = this._clickAction[c];
					if (act.handled !== true) {
						act.handled = tok.handleClick(act.x, act.y);
					}
				}
			}
		}
	}
	// Clear the click events
	this._clickAction = [];
};

// Return the click x position
TBGame.prototype.clickX = function() {
	"use strict";
	return this._clickInfo.x;
};

// Return the click y position
TBGame.prototype.clickY = function() {
	"use strict";
	return this._clickInfo.y;
};


// ********************************************
// ** Dealing with the touch
// ********************************************

// Initialize the touch handling
TBGame.prototype._initTouch = function(self) {
	"use strict";
	window.addEventListener('mousedown', function(e) {self._doMouseDown(e); } , false);
	window.addEventListener('mouseup', function(e) {self._doMouseUp(e); } , false);
	window.addEventListener('mousemove', function(e) {self._doMouseMove(e); } , false);
	window.addEventListener('touchstart', function(e) {self._doTouchStart(e); } , false);
	window.addEventListener('touchend', function(e) {self._doTouchEnd(e); } , false);
	window.addEventListener('touchmove', function(e) {self._doTouchMove(e); } , false);
	window.addEventListener('touchleave', function(e) {self._doTouchLeave(e); } , false);
	window.addEventListener('touchcancel', function(e) {self._doTouchCancel(e); } , false);
	window.addEventListener("mousewheel", function(e) {self._doMouseWheel(e); } , false);     // mouse wheel IE9, Chrome, Safari, Opera
	window.addEventListener("DOMMouseScroll", function(e) {self._doMouseWheel(e); } , false); // Firefox

	this.clearTouchEvents();
	// Hack to avoid touch event also firing mouse events
	this._lastTouchDown = 0;
	this._lastTouchUp = 0;
};

// Clears the touch events
TBGame.prototype.clearTouchEvents = function() {
	"use strict";
	var i;
	this._touchInfo = [];				// The information about the touch event
	for (i=0; i < 16; i += 1) {
		this._touchInfo[i] = {id: -1, down: false, x: 0, y: 0};
	}
	this._touchAction = [];				// The finger that were pressed or released since the last step
};

// We have to determine the finger ourselves as it turns out
// devices like iPad do not follow the numbering standards
// for touch identifiers.
TBGame.prototype._findFinger = function(id) {
	"use strict";
	var finger = -1,
		i;
	// Check whether it exists
	for (i=0; i<16; i += 1) {
		if (this._touchInfo[i].id === id) {
			finger = i;
			break;
		}
	}
	// If not, find first one that is not down
	if (finger === -1) {
		for (i=0; i<16; i += 1) {
			if (!this._touchInfo[i].down) {
				finger = i;
				this._touchInfo[i].id = id;
				break;
			}
		}
	}
	return finger;
};

// Add a touch move event
TBGame.prototype._addTouchMove = function(id, x, y) {
	"use strict";
	var finger, canvas, bbox;
	if (this.mainCanvas === undefined) { return; }
	finger = this._findFinger(id);
	if (finger === -1) { return; }
	canvas = this.mainCanvas;
	bbox = canvas.getBoundingClientRect();
	this._touchInfo[finger].x = (x - bbox.left) * (canvas.width / bbox.width);
	this._touchInfo[finger].y = (y - bbox.top) * (canvas.height / bbox.height);
};

// Add a touch down event
TBGame.prototype._addTouchDownEvent = function(id) {
	"use strict";
	var finger = this._findFinger(id),
		i, tok, handled;
	if (finger === -1) { return; }
	if (this._touchInfo[finger].down) { return; }
	this._touchAction.push( { finger: finger, x: this._touchInfo[finger].x, y: this._touchInfo[finger].y, down: true } );
	this._touchInfo[finger].down = true;
	// Handle the immediate events
	if (this._numberImmediateTouchEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateTouchDown) {
				handled = tok.handleImmediateTouchDown(finger, this._touchInfo[finger].x, this._touchInfo[finger].y);
				if (handled === true) { break; }
			}
		}
	}
};

// Add a touch up event
TBGame.prototype._addTouchUpEvent = function(id) {
	"use strict";
	var finger = this._findFinger(id),
		i, tok, handled;
	if (finger === -1) { return; }
	if (!this._touchInfo[finger].down) { return; }
	this._touchAction.push( { finger: finger, x: this._touchInfo[finger].x, y: this._touchInfo[finger].y, down: false } );
	this._touchInfo[finger].down = false;
	// Handle the immediate events
	if (this._numberImmediateTouchEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateTouchUp) {
				handled = tok.handleImmediateTouchUp(finger, this._touchInfo[finger].x, this._touchInfo[finger].y);
				if (handled === true) { break; }
			}
		}
	}
};

// Add a touch down event for a finger (rather than an id)
TBGame.prototype.addTouchDownEvent = function(finger) {
	"use strict";
	this._addTouchDownEvent(this._touchInfo[finger].id);
};

// Add a touch up event for a finger (rather than an id)
TBGame.prototype.addTouchUpEvent = function(finger) {
	"use strict";
	this._addTouchUpEvent(this._touchInfo[finger].id);
};

//Add a mouse wheel event
TBGame.prototype.addMouseWheelEvent = function(delta) {
	"use strict";
	var finger = 0,
		i, tok, handled;
	this._touchAction.push( { finger: finger, x: this._touchInfo[finger].x, y: this._touchInfo[finger].y, wheelDelta: delta } );
	// Handle the immediate events
	if (this._numberImmediateTouchEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateMouseWheel) {
				handled = tok.handleImmediateMouseWheel(delta, this._touchInfo[finger].x, this._touchInfo[finger].y);
				if (handled === true) { break; }
			}
		}
	}
};

// Handle a mouse button down event
TBGame.prototype._doMouseDown = function(e) {
	"use strict";
	if (this._paused > 0) { return; }
	e.preventDefault();
	window.focus();					// Because we cancel this with the previous command
	if (this._lastTouchDown >= Date.now()-1000) { return; }	// Hack to avoid a mouse down soon after a touch down event
	this._addTouchMove(0,e.clientX,e.clientY);
	this._addTouchDownEvent(0);
};

// Handle a mouse up event
TBGame.prototype._doMouseUp = function(e) {
	"use strict";
	if (this._paused > 0) { return; }
	e.preventDefault();
	if (this._lastTouchUp >= Date.now()-1000) { return; }	// Hack to avoid a mouse down soon after a touch up event
	this._addTouchMove(0,e.clientX,e.clientY);
	this._addTouchUpEvent(0);
};

// Handle a mouse move event
TBGame.prototype._doMouseMove = function(e) {
	"use strict";
	if (this._paused > 0) { return; }
	e.preventDefault();
	this._addTouchMove(0,e.clientX,e.clientY);
};

// Handle a mouse wheel event
TBGame.prototype._doMouseWheel = function(e) {
	"use strict";
	var delta;
	if (this._paused > 0) { return; }
	e.preventDefault();
	window.focus();					// Because we cancel this with the previous command
	delta = Math.max(-1, Math.min(1, (e.wheelDelta || -e.detail)));
	this.addMouseWheelEvent(delta);
};

// Handle a touch start event
TBGame.prototype._doTouchStart = function(e) {
	"use strict";
	var touches = e.changedTouches,
		i;
	this._lastTouchDown = Date.now();
	if (this._paused > 0) { return; }
	e.preventDefault();
	window.focus();					// Because we cancel this with the previous command
	for (i=0; i<touches.length; i += 1) {
		this._addTouchMove(touches[i].identifier,touches[i].clientX,touches[i].clientY);
		this._addTouchDownEvent(touches[i].identifier);
	}
};

// Handle a touch end event
TBGame.prototype._doTouchEnd = function(e) {
	"use strict";
	var touches = e.changedTouches,
		i;
	this._lastTouchUp = Date.now();
	if (this._paused > 0) { return; }
	e.preventDefault();
	for (i=0; i<touches.length; i += 1)	{
		this._addTouchMove(touches[i].identifier,touches[i].clientX,touches[i].clientY);
		this._addTouchUpEvent(touches[i].identifier);
	}
};

// Handle a touch leave event
TBGame.prototype._doTouchLeave = function(e) {
	"use strict";
	var touches = e.changedTouches,
		i;
	this._lastTouchUp = Date.now();
	if (this._paused > 0) { return; }
	e.preventDefault();
	for (i=0; i<touches.length; i += 1)	{
		this._addTouchMove(touches[i].identifier,touches[i].clientX,touches[i].clientY);
		this._addTouchUpEvent(touches[i].identifier);
	}
};

// Handle a touch cancel event
TBGame.prototype._doTouchCancel = function(e) {
	"use strict";
	var touches = e.changedTouches,
		i;
	this._lastTouchUp = Date.now();
	if (this._paused > 0) { return; }
	e.preventDefault();
	for (i=0; i<touches.length; i += 1)	{
		this._addTouchMove(touches[i].identifier,touches[i].clientX,touches[i].clientY);
		this._addTouchUpEvent(touches[i].identifier);
	}
};

// Handle a touch move event
TBGame.prototype._doTouchMove = function(e) {
	"use strict";
	var touches = e.changedTouches,
		i;
	if (this._paused > 0) { return; }
	e.preventDefault();
	for (i=0; i<touches.length; i += 1)	{
		this._addTouchMove(touches[i].identifier,touches[i].clientX,touches[i].clientY);
	}
};

// Send the correct touch events to the tokens
TBGame.prototype._handleTouchEvents = function() {
	"use strict";
	var c, i, tok, act;
	if (this._touchAction.length === 0) { return; }
	// Send the events to the tokens
	if (this._numberTouchEvents > 0) {
		for (i = this._tokens.length-1; i >= 0; i -= 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && (tok.handleTouchDown || tok.handleTouchUp || tok.handleMouseWheel)) {
				for (c = 0; c < this._touchAction.length; c += 1) {
					act = this._touchAction[c];
					if (act.handled !== true) {
						if (act.wheelDelta !== undefined && tok.handleMouseWheel) {
							act.handled = tok.handleMouseWheel(act.wheelDelta, act.x, act.y);
						} else if (act.down && tok.handleTouchDown) {
							act.handled = tok.handleTouchDown(act.finger, act.x, act.y);
						} else if (!act.down && tok.handleTouchUp) {
							act.handled = tok.handleTouchUp(act.finger, act.x, act.y);
						}
					}
				}
			}
		}
	}
	// Clear the touch events
	this._touchAction = [];
};

// Check whether a touch finger is down
TBGame.prototype.touchDown = function(finger) {
	"use strict";
	return this._touchInfo[finger].down;
};

// Returns the touch id
TBGame.prototype.touchId = function(finger) {
	"use strict";
	return this._touchInfo[finger].id;
};

// Return the touch x position
TBGame.prototype.touchX = function(finger) {
	"use strict";
	return this._touchInfo[finger].x;
};

// Return the touch y position
TBGame.prototype.touchY = function(finger) {
	"use strict";
	return this._touchInfo[finger].y;
};


// ********************************************
// ** Dealing with the keyboard
// ********************************************

// Initialize the keyboard handling
TBGame.prototype._initKeyboard = function(self) {
	"use strict";
	window.addEventListener('keydown', function(e) {self._doKeyDown(e); } , false);
	window.addEventListener('keyup', function(e) {self._doKeyUp(e); } , false);
	this.clearKeyboardEvents();
};

// Clears the keyboard handling
TBGame.prototype.clearKeyboardEvents = function() {
	"use strict";
	var i;
	this._keyDown = [];							// Whether the key is currently down
	for (i=0; i< 256; i += 1) {
		this._keyDown[i] = false;
	}
	this._keyAction = [];						// The keys that were pressed or released since the last step
};

// Add a key down event
TBGame.prototype.addKeyDownEvent = function(keyCode) {
	"use strict";
	var i, tok, handled;
	if (this._keyDown[keyCode]) { return; }
	this._keyAction.push( { key: keyCode, down: true } );
	this._keyDown[keyCode] = true;
	// Handle the immediate events
	if (this._numberImmediateKeyEvents > 0) {
		for (i = 0; i < this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateKeyDown) {
				handled = tok.handleImmediateKeyDown(keyCode);
				if (handled === true) { break; }
			}
		}
	}
};

// Add a key up event
TBGame.prototype.addKeyUpEvent = function(keyCode) {
	"use strict";
	var i, tok, handled;
	if (!this._keyDown[keyCode]) { return; }
	this._keyAction.push( { key: keyCode, down: false } );
	this._keyDown[keyCode] = false;
	// Handle the immediate events
	if (this._numberImmediateKeyEvents > 0) {
		for (i = 0; i < this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateKeyUp) {
				handled = tok.handleImmediateKeyUp(keyCode);
				if (handled === true) { break; }
			}
		}
	}
};

// Handle a key down event
TBGame.prototype._doKeyDown = function(e) {
	"use strict";
	if (this._paused > 0) { return; }
	if([32, 37, 38, 39, 40].indexOf(e.keyCode) > -1) {
		e.preventDefault();
	}
	this.addKeyDownEvent(e.keyCode);
};

// Handle a key up event
TBGame.prototype._doKeyUp = function(e) {
	"use strict";
	if (this._paused > 0) { return; }
	if([32, 37, 38, 39, 40].indexOf(e.keyCode) > -1) {
		e.preventDefault();
	}
	this.addKeyUpEvent(e.keyCode);
};

// Send the correct key events to the tokens
TBGame.prototype._handleKeyEvents = function() {
	"use strict";
	var c, i, tok, act;
	if (this._keyAction.length === 0) { return; }
	// Send the events to the tokens
	if (this._numberKeyEvents > 0) {
		for (i=0; i<this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && (tok.handleKeyDown || tok.handleKeyUp)) {
				for (c=0; c<this._keyAction.length; c += 1) {
					act = this._keyAction[c];
					if (act.handled !== true) {
						if (act.down && tok.handleKeyDown) {
							act.handled = tok.handleKeyDown(act.key);
						} else if (!act.down && tok.handleKeyUp) {
							act.handled = tok.handleKeyUp(act.key);
						}
					}
				}
			}
		}
	}
	// Clear the keyboard events
	this._keyAction = [];
};

// Check whether a key is down
TBGame.prototype.keyboardKeyDown = function(keyCode) {
	"use strict";
	return this._keyDown[keyCode];
};


// ********************************************
// ** Dealing with step events
// ********************************************

// Send a begin step event to the tokens
TBGame.prototype._handleBeginStepEvents = function() {
	"use strict";
	var i, tok;
	if (this._numberBeginStepEvents > 0) {
		for (i=0; i<this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.handleBeginStep && tok.active && tok._paused <= 0) {
				if (tok.handleBeginStep(this.stepTime) === true) { return; }
			}
		}
	}
};

// Send an end step event to the tokens
TBGame.prototype._handleEndStepEvents = function() {
	"use strict";
	var i, tok;
	if (this._numberEndStepEvents > 0) {
		for (i=0; i<this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.handleEndStep && tok.active && tok._paused <= 0) {
				if (tok.handleEndStep(this.stepTime) === true) { return; }
			}
		}
	}
};

// ********************************************
// ** Dealing with alarm events
// ********************************************

// Initialize the alarm events
TBGame.prototype._initAlarm = function(self) {
	"use strict";
	this._alarmId = 0;
	this.clearAlarmEvents();
};

// Clears the alarm events
TBGame.prototype.clearAlarmEvents = function() {
	"use strict";
	this._alarmInfo = [];
	this._alarmAction = [];
};

// Find an alarm index
TBGame.prototype._findAlarmIndex = function(id, token) {
	"use strict";
	var i;
	if (token === undefined) { token = null; }
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		if (this._alarmInfo[i].id === id && this._alarmInfo[i].token === token) { return i; }
	}
	return -1;
};

// Add a new alarm
TBGame.prototype.addAlarm = function(time, repeats, token) {
	"use strict";
	if (repeats === undefined) { repeats = 1; }
	if (token === undefined) { token = null; }
	while (this.existsAlarm(this._alarmId, token)) { this._alarmId += 1; }
	this._alarmInfo.push( { id: this._alarmId, time: time, repeats: repeats,
							token: token, timeLeft: time, repeatsLeft: repeats-1, paused: 0 } );
	this._alarmId += 1;
	return this._alarmId-1;
};

// Remove an existing alarm
TBGame.prototype.removeAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { this._alarmInfo[ind].id = undefined; }
};

// Checks whether an alarm with the id exists
TBGame.prototype.existsAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	return ind >= 0;
};

// Set a new or existing alarm
TBGame.prototype.setAlarm = function(id, time, repeats, token) {
	"use strict";
	var n = this._alarmInfo.length,
		i;
	if (repeats === undefined) { repeats = 1; }
	if (token === undefined) { token = null; }
	// Check whether it exists
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		if (this._alarmInfo[i].id === id && this._alarmInfo[i].token === token) { n = i; }
	}
	// Find an unused spot
	if (n === this._alarmInfo.length) {
		for (i = 0; i < this._alarmInfo.length; i += 1) {
			if (this._alarmInfo[i].id === undefined) { n = i; }
		}
	}
	this._alarmInfo[n] = { id: id, time: time, repeats: repeats,
							token: token, timeLeft: time, repeatsLeft: repeats-1, paused: 0 };
};

// Clear an existing alarm
TBGame.prototype.clearAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) {
		this._alarmInfo[ind].timeLeft = 0;
		this._alarmInfo[ind].repeatsLeft = 0;
	}
};

// Clear all alarms
TBGame.prototype.clearAllAlarms = function() {
	"use strict";
	var i;
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		this._alarmInfo[i].timeLeft = 0;
		this._alarmInfo[i].repeatsLeft = 0;
	}
};

// Get an existing alarm
TBGame.prototype.getAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { return this._alarmInfo[ind].timeLeft; } else { return undefined; }
};

// Get an existing alarm percentage
TBGame.prototype.getAlarmPercentage = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { return 100*this._alarmInfo[ind].timeLeft/this._alarmInfo[ind].time; } else { return undefined; }
};

// Pauses an alarm
TBGame.prototype.pauseAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { this._alarmInfo[ind].paused += 1; }
};

// Resumes an alarm
TBGame.prototype.resumeAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { this._alarmInfo[ind].paused = Math.max(0, this._alarmInfo[ind].paused-1); }
};

// Resumes an alarm forced
TBGame.prototype.resumeForcedAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { this._alarmInfo[ind].paused = 0; }
};

// Returns whether the alarm is paused
TBGame.prototype.isPausedAlarm = function(id, token) {
	"use strict";
	var ind = this._findAlarmIndex(id, token);
	if (ind >= 0) { return this._alarmInfo[ind].paused > 0; } else { return false; }
};

// Pause all alarms
TBGame.prototype.pauseAllAlarms = function() {
	"use strict";
	var i;
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		this._alarmInfo[i].paused += 1;
	}
};

// Resume all alarms
TBGame.prototype.resumeAllAlarms = function() {
	"use strict";
	var i;
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		this._alarmInfo[i].paused = Math.max(0, this._alarmInfo[i].paused-1);
	}
};

// Resume all alarms forced
TBGame.prototype.resumeForcedAllAlarms = function() {
	"use strict";
	var i;
	for (i = 0; i < this._alarmInfo.length; i += 1) {
		this._alarmInfo[i].paused = 0;
	}
};

// Update the alarms
TBGame.prototype._updateAlarms = function(timePassed) {
	"use strict";
	var i, alarm;
	this._alarmAction.length = 0;
	for (i=0; i< this._alarmInfo.length; i += 1) {
		alarm = this._alarmInfo[i];
		if (alarm.id !== undefined && alarm.paused === 0 && (alarm.timeLeft > 0 || alarm.repeatsLeft > 0)) {
			alarm.timeLeft -= timePassed;
			if (alarm.timeLeft <= 0) {
				this._alarmAction.push({id: alarm.id, token: alarm.token});
				if (alarm.repeatsLeft > 0) {
					alarm.repeatsLeft -= 1;
					alarm.timeLeft += alarm.time;
				} else {
					alarm.timeLeft = 0;
				}
			}
		}
	}
};

// Send alarm events to the tokens
TBGame.prototype._handleAlarmEvents = function() {
	"use strict";
	var i, tok, j, act;
	this._updateAlarms(this.stepTime);
	if (this._numberAlarmEvents > 0 && this._alarmAction.length > 0) {
		for (i = 0; i < this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.handleAlarm && tok.active) {
				for (j = 0; j < this._alarmAction.length; j += 1) {
					act = this._alarmAction[j];
					if (act.handled !== true && (act.token === null || act.token === tok)) {
						act.handled = tok.handleAlarm(act.id);
					}
				}
			}
		}
	}
};


// ********************************************
// ** Dealing with special events
// ********************************************

// Initialize the special events
TBGame.prototype._initSpecial = function(self) {
	"use strict";
	this.clearSpecialEvents();
};

// Clear the special events
TBGame.prototype.clearSpecialEvents = function() {
	"use strict";
	this._specialAction = [];
};

// Add a special event
TBGame.prototype.addSpecial = function(id, args, token) {
	"use strict";
	var i, tok, handled;
	if (token === undefined) { token = null; }
	this._specialAction.push( {id: id, args: args, token: token } );
	// Handle the immediate events
	if (this._numberImmediateSpecialEvents > 0) {
		for (i = 0; i < this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.active && tok._paused <= 0 && tok.handleImmediateSpecial && (token === null || token === tok)) {
				handled = tok.handleImmediateSpecial(id, args);
				if (handled === true) { break; }
			}
		}
	}
};

// Send special events to the tokens
TBGame.prototype._handleSpecialEvents = function() {
	"use strict";
	var i, j, tok, act;
	if (this._numberSpecialEvents > 0 && this._specialAction.length > 0) {
		for (i = 0; i < this._tokens.length; i += 1) {
			tok = this._tokens[i];
			if (tok && tok.handleSpecial && tok.active && tok._paused <= 0) {
				for (j = 0; j < this._specialAction.length; j += 1) {
					act = this._specialAction[j];
					if (act.handled !== true && (act.token === null || act.token === tok)) {
						act.handled = tok.handleSpecial(act.id, act.args);
					}
				}
			}
		}
	}
	this._specialAction.length = 0;
};


// ********************************************
// ** Dealing with drawing events
// ********************************************

// Draws an element
TBGame.prototype._drawElement = function(ind) {
	"use strict";
	var element = this._elements[ind];
	if (!element.visible) { return; }
	// Set the correct canvas
	if (element.canvas !== undefined && element.canvas !== tbGraphics.canvas) {
		tbGraphics.setCanvas(element.canvas);
	}
	// Now draw it if required
	if (tbGraphics.canvas.drawTokens === false && element.forceDraw !== true) { return; }
	if (element._kind === 0) {	// Image
		if (element._paused <= 0) { element.subImage += element.animSpeed*this.stepTime/1000; }
		if (element.xscale === 1 && element.yscale === 1 && element.angle === 0) {
			if (element.alpha === 1) {
				element.image.drawSimple(element.subImage, element.x, element.y);
			} else {
				element.image.drawAlpha(element.subImage, element.x, element.y, element.alpha);
			}
		} else {
			element.image.drawExt(element.subImage, element.x, element.y, element.xscale, element.yscale, element.angle, element.alpha);
		}
	}
	if (element._kind === 1) {	// Text
		if (element.xscale === 1 && element.yscale === 1 && element.angle === 0) {
			if (element.alpha === 1) {
				element.font.drawSimple(element.text, element.x, element.y);
			} else {
				element.font.drawAlpha(element.text, element.x, element.y, element.alpha);
			}
		} else {
			element.font.drawExt(element.text, element.x, element.y, element.xscale, element.yscale, element.angle, element.alpha);
		}
	}
};

// Draws a token
TBGame.prototype._drawToken = function(ind) {
	"use strict";
	var tok = this._tokens[ind];
	if (!tok.visible) { return; }
	// Set the correct canvas
	if (tok.canvas !== undefined && tok.canvas !== tbGraphics.canvas) {
		tbGraphics.setCanvas(tok.canvas);
	}
	// Now draw it if required
	if (tbGraphics.canvas.drawTokens === false && tok.forceDraw !== true) { return; }
	if (tok.handleDraw) { return tok.handleDraw(); }
};

// Handle the drawing and send draw events to the tokens
TBGame.prototype._handleDrawEvents = function() {
	"use strict";
	var nt = 0, ne = 0;
	while (nt < this._tokens.length || ne < this._elements.length) {
		if (ne === this._elements.length) {
			if (this._drawToken(nt) === true) { return; }
			nt += 1;
		} else if (nt === this._tokens.length) {
			this._drawElement(ne);
			ne += 1;
		} else if (this._elements[ne].layer > this._tokens[nt].layer ||
					(this._elements[ne].layer === this._tokens[nt].layer &&
						this._elements[ne].depth > this._tokens[nt].depth)) {
			this._drawElement(ne);
			ne += 1;
		} else {
			if (this._drawToken(nt) === true) { return; }
			nt += 1;
		}
	}
};


// ********************************************
// ** Dealing with pausing
// ********************************************

// Initialize the pausing
TBGame.prototype._initPause = function(self) {
	"use strict";
	this._paused = 0;
	this._pausedDraw = false;
};

// Pause the loop
TBGame.prototype.pause = function(pausedDraw) {
	"use strict";
	this._paused += 1;
	if (pausedDraw === undefined) { pausedDraw = false; }
	this._pausedDraw = pausedDraw;
};

// Resume the loop
TBGame.prototype.resume = function() {
	"use strict";
	if (this._paused === 0) { return; }
	this.lastTime = Date.now(); // To avoid jumps in game time and big step time
	this._paused -= 1;
};

// Resume the loop
TBGame.prototype.resumeForced = function() {
	"use strict";
	if (this._paused === 0) { return; }
	this.lastTime = Date.now(); // To avoid jumps in game time and big step time
	this._paused = 0;
};

// Returns whether the loop is paused
TBGame.prototype.isPaused = function() {
	"use strict";
	return this._paused > 0;
};


// ********************************************
// ** Dealing with the global game loop
// ********************************************

window.__timingStartTime = 0;			// The time just before the step is started
window.__timingFinishTime = 0;			// The time just after the step is finished
window.__timingStepTime = 0;			// The time the step took
window.__timingBetweenTime = 0;			// The time between the start of the current step and the finish of the previous
window.__timingWaitTime = 0;			// The wait time when using setTimeout
window.__timingFrames = 60;				// The desired number of frames per second when using setTimeout
window.__timingInterval = 0;			// The minimal interval used in setTimeout (to give the system some time for processing)
window.__timingForceTimeout = false;	// Whether to force using setTimeout (and not requestAnimationFrame)

// Define the callback function to be called in each step
window.__timingDoAStep = function () {
	"use strict";
	// Remember the current time
	window.__timingStartTime = Date.now();
	window.__timingBetweenTime = window.__timingStartTime - window.__timingFinishTime;
	// Perform a step in the loop
	tbGame._stepLoop();
	// Remember the time when it is finished
	window.__timingFinishTime = Date.now();
	window.__timingStepTime = window.__timingFinishTime - window.__timingStartTime;
	// Set the correct wait time and requestthe next frame
	window.__timingWaitTime = Math.max(window.__timingInterval, 1000/window.__timingFrames - window.__timingStepTime);
	window.__timingRequestNextAnimationFrame(window.__timingDoAStep);
} ;

// Define our own request animation frame function
(function() {
	"use strict";
	// Default use the timeout function
	window.__timingRequestNextAnimationFrame =
		function(callback) {
			window.setTimeout(callback, window.__timingWaitTime);
		};
	// If request animation frame is supported, use that instead
	if (!window.__timingForceTimeout) {
		window.__timingRequestNextAnimationFrame = window.requestAnimationFrame ||
			window.webkitRequestAnimationFrame ||
			window.mozRequestAnimationFrame ||
			window.msRequestAnimationFrame ||
			window.oRequestAnimationFrame ||
			window.__timingRequestNextAnimationFrame;
	}
}());

TBGame.prototype.supportsRequestAnimationFrame = function () {
// Returns whether the system supports the use of request animation frame (in some form)
	"use strict";
	return window.requestAnimationFrame !== undefined ||
		window.webkitRequestAnimationFrame !== undefined ||
		window.mozRequestAnimationFrame !== undefined ||
		window.msRequestAnimationFrame !== undefined ||
		window.oRequestAnimationFrame !== undefined;
};

TBGame.prototype.forceUseTimeout = function (force) {
// forces the use of the timeout function instead of request animation frame (or not)
	"use strict";
	window.__timingForceTimeout = force;
	window.__timingDefineFunction();
};

TBGame.prototype.setLoopSpeed = function (framesPerSecond) {
// Sets the number of frames per second (when requestAnimationFrame does not exist)
	"use strict";
	window.__timingFrames = framesPerSecond;
};

TBGame.prototype.setLoopInterval = function (interval) {
// Sets the minimal interval for setTimeout (when requestAnimationFrame does not exist)
	"use strict";
	window.__timingInterval = interval;
};

// Initialize the loop
TBGame.prototype._initLoop = function(self) {
	"use strict";
	// Initialize time (all in milliseconds)
	this.startTime = Date.now();				// The time the game started
	this.lastTime = this.startTime;				// The time for the last step
	this.gameTime = 0;							// The time the game has run so far (sum of step times)
	this.stepTime = 0;							// The time of the last step
	this.maxStepTime = 200;						// The maximal step time passed back in the step events
	this.totalSteps = 0;						// The total number of steps since the start
	// Start the start loading
	this.loadingMoment = "";
	// Set the first loop call using out own version of window.requestNext AnimationFram
	window.__timingRequestNextAnimationFrame(window.__timingDoAStep);
};

// Function to order the tokens and elements and clean them up
TBGame.prototype._order = function() {
	"use strict";

	function sortfunction(a, b){
		return (a.layer === b.layer) ? b.depth-a.depth :((a.layer > b.layer) ? -1 : 1);
	}

	var i, n;
	// Handle the tokens
	n = 0;
	for (i=0; i<this._tokens.length; i += 1) {
		if (this._tokens[i]) {
			// activate when new
			if (this._tokens[i]._activateNew) {
				this._tokens[i]._activateNew = false;
				this._tokens[i].active = true;
			}
			//shift it in the array
			this._tokens[n] = this._tokens[i];
			n += 1;
		}
	}
	this._tokens.length = n;
	if (this._tokenOrderChanged) {
		this._tokens.sort(sortfunction);
	}
	this._tokenOrderChanged = false;
	// Handle the elements
	n = 0;
	for (i=0; i<this._elements.length; i += 1) {
		if (this._elements[i]) {
			this._elements[n] = this._elements[i];
			n += 1;
		}
	}
	this._elements.length = n;
	if (this._elementOrderChanged) {
		this._elements.sort(sortfunction);
	}
	this._elementOrderChanged = false;
};

// Handle a time step in the loop
TBGame.prototype._stepLoop = function() {
	"use strict";
	var newTime;
	// Handle paused mode
	if (this._paused > 0) {
		if (this._pausedDraw) {
			this._order();
			this._handleDrawEvents();
		}
		return;
	}
	// Calculate the time passed
	newTime = Date.now();
	// Solve a strange bugin which the time passed becomes NaN
	if(typeof newTime !== "number"){
		newTime = this.lastTime;
	}
	this.stepTime = Math.min(this.maxStepTime, newTime - this.lastTime);
	this.gameTime += this.stepTime;
	this.totalSteps += 1;
	// Deal with loading
	if (this.loadingMoment === "") {
		this.loadingMoment = "start";
		tbLoader.startLoading(this.loadingMoment);
	}
	if (this.loadingMoment === "start" && tbLoader.complete(this.loadingMoment)) {
		this.loadingMoment = "load";
		tbLoader.startLoading(this.loadingMoment);
	}
	if (this.loadingMoment === "load" && tbLoader.complete(this.loadingMoment)) {
		this.loadingMoment = "game";
		tbLoader.startLoading(this.loadingMoment);
	}
	// Handle the sounds
	if (typeof tbAudio !== "undefined") {
		tbAudio._handleStep(this.stepTime);
	}
	// Handle the events
	this._handleBeginStepEvents();
	this._handleClickEvents();
	this._handleTouchEvents();
	this._handleKeyEvents();
	this._handleAlarmEvents();
	this._handleSpecialEvents();
	this._handleEndStepEvents();
	// Order and clean up
	this._order();
	// Finally draw
	this._handleDrawEvents();
	// Update the time
	this.lastTime = newTime;
};

// ********************************************
// ** Adding and removing tokens
// ********************************************

// Add a token to the game; the optional activate argument indicates whether to activate it immediately
TBGame.prototype.addToken = function(token, activate) {
	"use strict";
	// Fill in properties if they are missing
	if (token.group === undefined) { token.group = 0; }
	if (token.visible === undefined) { token.visible = true; }
	if (token.active === undefined) { token.active = true; }
	if (token.depth === undefined) { token.depth = 0; }
	if (token.layer === undefined) { token.layer = 0; }
	if (token._paused === undefined) { token._paused = 0; }
	if (token._tags === undefined) { token._tags = []; }
	// Set activation
	token._activateNew = false;
	if (activate !== undefined && activate === false) {
		token._activateNew = true;
		token.active = false;
	}
	this._tokens.push(token);
	this._tokenOrderChanged = true;
	this._addEventCounters(token);
	// Add it to the array
	if (token.handleAdd) {
		token.handleAdd();
	}
};

// Change the depth of a token (also when it is not in the game!)
TBGame.prototype.setTokenDepth = function(token, depth) {
	"use strict";
	if (token.depth !== depth) { this._tokenOrderChanged = true; }
	token.depth = depth;
};

// Change the layer of a token (also when it is not in the game!)
TBGame.prototype.setTokenLayer = function(token, layer) {
	"use strict";
	if (token.layer !== layer) { this._tokenOrderChanged = true; }
	token.layer = layer;
};

// Clears the tags of a token
TBGame.prototype.clearTokenTags = function(token) {
	"use strict";
	token._tags = [];
};

// Returns (a copy of) the tags of a token
TBGame.prototype.getTokenTags = function(token) {
	"use strict";
	var t, tags = [];
	if (token._tags === undefined) { token._tags = []; }
	for (t = token._tags.length-1; t >= 0; t -= 1) { tags.push(token._tags[t]); }
	return tags;
};

// Add tags to a token
TBGame.prototype.addTokenTags = function(token, tags) {
	"use strict";
	var t;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (token._tags === undefined) { token._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		if (token._tags.indexOf(tags[t]) < 0) {
			token._tags.push(tags[t]);
		}
	}
};

// Remove tags from a token
TBGame.prototype.removeTokenTags = function(token, tags) {
	"use strict";
	var t, ind;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (token._tags === undefined) { token._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		ind = token._tags.indexOf(tags[t]);
		if (ind >= 0) { token._tags.splice(ind, 1); }
	}
};

// Returns all tokens that contain the tag or satisfy the select function
TBGame.prototype.findTokens = function (arg) {
	"use strict";
	var tokens = [],
		i, token;
	if (arg === undefined || arg === "all" || arg === "master") {
		for (i = 0; i < this._tokens.length; i += 1) {
			token = this._tokens[i];
			if (token !== undefined) { tokens.push(token); }
		}
	} else if (typeof arg === "function" ) {
		for (i = 0; i < this._tokens.length; i += 1) {
			token = this._tokens[i];
			if (token !== undefined && arg(token)) { tokens.push(token); }
		}
	} else {
		for (i = 0; i < this._tokens.length; i += 1) {
			token = this._tokens[i];
			if (token !== undefined && token._tags.indexOf(arg) >= 0) { tokens.push(token); }
		}
	}
	return tokens;
};

// Find all tokens (for compatibility)
TBGame.prototype.findAllTokens = function () {
	"use strict";
	return this.findTokens();
};

// Pause a single token
TBGame.prototype.pauseToken = function(token) {
	"use strict";
	token._paused += 1;
};

// Pause a number of tokens
TBGame.prototype.pauseTokens = function(arg) {
	"use strict";
	var i, tokens = this.findTokens(arg);
	for (i = 0; i < tokens.length; i += 1) { this.pauseToken(tokens[i]); }
};

// Resume a single token
TBGame.prototype.resumeToken = function(token) {
	"use strict";
	token._paused = Math.max(0, token._paused-1);
};

// Resume a number of tokens
TBGame.prototype.resumeTokens = function(arg) {
	"use strict";
	var i, tokens = this.findTokens(arg);
	for (i = 0; i < tokens.length; i += 1) { this.resumeToken(tokens[i]); }
};

// Force resume a single token
TBGame.prototype.resumeForcedToken = function(token) {
	"use strict";
	token._paused = 0;
};

// Force resume a number of tokens
TBGame.prototype.resumeForcedTokens = function(arg) {
	"use strict";
	var i, tokens = this.findTokens(arg);
	for (i = 0; i < tokens.length; i += 1) { this.resumeForcedToken(tokens[i]); }
};

// Remove a token from the game
TBGame.prototype.removeToken = function(token) {
	"use strict";
	var ind = this._tokens.indexOf(token);
	if (ind < 0) { return; }
	if (this._tokens[ind].handleRemove) {
		this._tokens[ind].handleRemove();
	}
	this._subtractEventCounters(this._tokens[ind]);
	this._tokens[ind] = undefined;
};

// Removes all tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.removeTokens = function (arg) {
	"use strict";
	var i, tokens = this.findTokens(arg);
	for (i = 0; i < tokens.length; i += 1) { this.removeToken(tokens[i]); }
};

// Remove all tokens (for compatibility)
TBGame.prototype.removeAllTokens = function () {
	"use strict";
	this.removeTokens();
};

// Counting all tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.countTokens = function (arg) {
	"use strict";
	var tokens = this.findTokens(arg);
	return tokens.length;
};

// Count all tokens (for compatibility)
TBGame.prototype.countAllTokens = function () {
	"use strict";
	this.countTokens();
};


// ********************************************
// ** Adding and removing images
// ********************************************

// Add an image to the game
TBGame.prototype.addImage = function(image, subImage, animSpeed, x, y, depth, layer) {
	"use strict";
	return this.addImageExt(image, subImage, animSpeed, x, y, 1, 1, 0, 1, depth, layer);
};

// Add a simple image to the game
TBGame.prototype.addImageSimple = function(image, x, y, depth, layer) {
	"use strict";
	return this.addImageExt(image, 0, 0, x, y, 1, 1, 0, 1, depth, layer);
};

// Add an image with alpha value to the game
TBGame.prototype.addImageAlpha = function(image, subImage, animSpeed, x, y, alpha, depth, layer) {
	"use strict";
	return this.addImageExt(image, subImage, animSpeed, x, y, 1, 1, 0, alpha, depth, layer);
};

// Add an image with alpha and transformation to the game
TBGame.prototype.addImageExt = function(image, subImage, animSpeed, x, y, xscale, yscale, angle, alpha, depth, layer) {
	"use strict";
	if (layer === undefined) { layer = 0; }
	this._elements.push({ _kind: 0, image: image, subImage: subImage, animSpeed: animSpeed, visible: true,
						x: x, y: y, xscale: xscale, yscale: yscale, angle: angle, alpha: alpha, depth: depth, layer: layer,
						_paused: 0, _tags: [] });
	this._elementOrderChanged = true;
	return this._elements[this._elements.length-1];
};

// Add an existing image token to the game
TBGame.prototype.addImageToken = function(imageToken) {
	"use strict";
	this._elements.push(imageToken);
	this._elementOrderChanged = true;
};

// Change the depth of an image token
TBGame.prototype.setImageDepth = function(imageToken, depth) {
	"use strict";
	this._elementOrderChanged = (imageToken.depth !== depth);
	imageToken.depth = depth;
};

// Change the layer of an image token
TBGame.prototype.setImageLayer = function(imageToken, layer) {
	"use strict";
	this._elementOrderChanged = (imageToken.layer !== layer);
	imageToken.layer = layer;
};

// Clears the tags of an image token
TBGame.prototype.clearImageTags = function(imageToken) {
	"use strict";
	imageToken._tags = [];
};

// Returns (a copy of) the tags of an image token
TBGame.prototype.getImageTags = function(imageToken) {
	"use strict";
	var t, tags = [];
	if (imageToken._tags === undefined) { imageToken._tags = []; }
	for (t = imageToken._tags.length-1; t >= 0; t -= 1) { tags.push(imageToken._tags[t]); }
	return tags;
};

// Add tags to an image token
TBGame.prototype.addImageTags = function(imageToken, tags) {
	"use strict";
	var t;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (imageToken._tags === undefined) { imageToken._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		if (imageToken._tags.indexOf(tags[t]) < 0) {
			imageToken._tags.push(tags[t]);
		}
	}
};

// Remove tags from an image token
TBGame.prototype.removeImageTags = function(imageToken, tags) {
	"use strict";
	var t, ind;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (imageToken._tags === undefined) { imageToken._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		ind = imageToken._tags.indexOf(tags[t]);
		if (ind >= 0) { imageToken._tags.splice(ind, 1); }
	}
};

// Returns all images that contain the tag or satisfy the select function
TBGame.prototype.findImages = function (arg) {
	"use strict";
	var imageTokens = [],
		i, imageToken;
	if (arg === undefined || arg === "all" || arg === "master") {
		for (i = 0; i < this._elements.length; i += 1) {
			imageToken = this._elements[i];
			if (imageToken !== undefined && imageToken._kind === 0) { imageTokens.push(imageToken); }
		}
	} else if (typeof arg === "function" ) {
		for (i = 0; i < this._elements.length; i += 1) {
			imageToken = this._elements[i];
			if (imageToken !== undefined && imageToken._kind === 0 && arg(imageToken)) { imageTokens.push(imageToken); }
		}
	} else {
		for (i = 0; i < this._elements.length; i += 1) {
			imageToken = this._elements[i];
			if (imageToken !== undefined && imageToken._kind === 0 && imageToken._tags.indexOf(arg) >= 0) { imageTokens.push(imageToken); }
		}
	}
	return imageTokens;
};

// Find all image tokens (for compatibility)
TBGame.prototype.findAllImages = function () {
	"use strict";
	return this.findImages();
};

// Pause a single image token
TBGame.prototype.pauseImage = function(imageToken) {
	"use strict";
	imageToken._paused += 1;
};

// Pause a number of image tokens
TBGame.prototype.pauseImages = function(arg) {
	"use strict";
	var i, imageTokens = this.findImages(arg);
	for (i = 0; i < imageTokens.length; i += 1) { this.pauseImage(imageTokens[i]); }
};

// Resume a single image token
TBGame.prototype.resumeImage = function(imageToken) {
	"use strict";
	imageToken._paused = Math.max(0, imageToken._paused-1);
};

// Resume a number of image tokens
TBGame.prototype.resumeImages = function(arg) {
	"use strict";
	var i, imageTokens = this.findImages(arg);
	for (i = 0; i < imageTokens.length; i += 1) { this.resumeImage(imageTokens[i]); }
};

// Force resume a single image token
TBGame.prototype.resumeForcedImage= function(imageToken) {
	"use strict";
	imageToken._paused = 0;
};

// Force resume a number of images token
TBGame.prototype.resumeForcedImages = function(arg) {
	"use strict";
	var i, imageTokens = this.findImages(arg);
	for (i = 0; i < imageTokens.length; i += 1) { this.resumeForcedImage(imageTokens[i]); }
};

// Remove an image token from the game
TBGame.prototype.removeImage = function(imageToken) {
	"use strict";
	var ind = this._elements.indexOf(imageToken);
	if (ind < 0) { return; }
	this._elements[ind] = undefined;
};

// Removes all image tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.removeImages = function (arg) {
	"use strict";
	var i, imageTokens = this.findImages(arg);
	for (i = 0; i < imageTokens.length; i += 1) { this.removeImage(imageTokens[i]); }
};

// Remove all image tokens (for compatibility)
TBGame.prototype.removeAllImages = function () {
	"use strict";
	this.removeImages();
};

// Counting all image tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.countImages = function (arg) {
	"use strict";
	var imageTokens = this.findImages(arg);
	return imageTokens.length;
};

// Count all image tokens (for compatibility)
TBGame.prototype.countAllImages = function () {
	"use strict";
	this.countImages();
};

// ********************************************
// ** Adding and removing texts
// ********************************************

// Add a simple text to the game
TBGame.prototype.addTextSimple = function(text, font, x, y, depth, layer) {
	"use strict";
	return this.addTextExt(text, font, x, y, 1, 1, 0, 1, depth, layer);
};

// Add a text with alpha value to the game
TBGame.prototype.addTextAlpha = function(text, font, x, y, alpha, depth, layer) {
	"use strict";
	return this.addTextExt(text, font, x, y, 1, 1, 0, alpha, depth, layer);
};

// Add a text with alpha and transformation to the game
TBGame.prototype.addTextExt = function(text, font, x, y, xscale, yscale, angle, alpha, depth, layer) {
	"use strict";
	if (layer === undefined) { layer = 0; }
	this._elements.push({ _kind: 1, text: text, font: font, visible: true,
						x: x, y: y, xscale: xscale, yscale: yscale, angle: angle, alpha: alpha, depth: depth, layer: layer,
						_tags: [] });
	this._elementOrderChanged = true;
	return this._elements[this._elements.length-1];
};

// Add an existing text token to the game
TBGame.prototype.addTextToken = function(textToken) {
	"use strict";
	this._elements.push(textToken);
	this._elementOrderChanged = true;
};

// Change the depth of a text token
TBGame.prototype.setTextDepth = function(textToken, depth) {
	"use strict";
	this._elementOrderChanged = (textToken.depth !== depth);
	textToken.depth = depth;
};

// Change the layer of a text token
TBGame.prototype.setTextLayer = function(textToken, layer) {
	"use strict";
	this._elementOrderChanged = (textToken.layer !== layer);
	textToken.layer = layer;
};

// Clears the tags of a text token
TBGame.prototype.clearTextTags = function(textToken) {
	"use strict";
	textToken._tags = [];
};

// Returns (a copy of) the tags of a text token
TBGame.prototype.getTextTags = function(textToken) {
	"use strict";
	var t, tags = [];
	if (textToken._tags === undefined) { textToken._tags = []; }
	for (t = textToken._tags.length-1; t >= 0; t -= 1) { tags.push(textToken._tags[t]); }
	return tags;
};

// Add tags to a text token
TBGame.prototype.addTextTags = function(textToken, tags) {
	"use strict";
	var t;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (textToken._tags === undefined) { textToken._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		if (textToken._tags.indexOf(tags[t]) < 0) {
			textToken._tags.push(tags[t]);
		}
	}
};

// Remove tags from a text token
TBGame.prototype.removeTextTags = function(textToken, tags) {
	"use strict";
	var t, ind;
	tags = [].concat(tags);				// Such that it also works for a single value
	if (textToken._tags === undefined) { textToken._tags = []; }
	for (t = tags.length-1; t >= 0; t -= 1) {
		ind = textToken._tags.indexOf(tags[t]);
		if (ind >= 0) { textToken._tags.splice(ind, 1); }
	}
};

// Returns all texts that contain the tag or satisfy the select function
TBGame.prototype.findTexts = function (arg) {
	"use strict";
	var textTokens = [],
		i, textToken;
	if (arg === undefined || arg === "all" || arg === "master") {
		for (i = 0; i < this._elements.length; i += 1) {
			textToken = this._elements[i];
			if (textToken !== undefined && textToken._kind === 1) { textTokens.push(textToken); }
		}
	} else if (typeof arg === "function" ) {
		for (i = 0; i < this._elements.length; i += 1) {
			textToken = this._elements[i];
			if (textToken !== undefined && textToken._kind === 1 && arg(textToken)) { textTokens.push(textToken); }
		}
	} else {
		for (i = 0; i < this._elements.length; i += 1) {
			textToken = this._elements[i];
			if (textToken !== undefined && textToken._kind === 1 && textToken._tags.indexOf(arg) >= 0) { textTokens.push(textToken); }
		}
	}
	return textTokens;
};

// Find all text tokens (for compatibility)
TBGame.prototype.findAllTexts = function () {
	"use strict";
	return this.findTexts();
};

// Remove a text token from the game
TBGame.prototype.removeText = function(textToken) {
	"use strict";
	var ind = this._elements.indexOf(textToken);
	if (ind < 0) { return; }
	this._elements[ind] = undefined;
};

// Removes all text tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.removeTexts = function (arg) {
	"use strict";
	var i, textTokens = this.findTexts(arg);
	for (i = 0; i < textTokens.length; i += 1) { this.removeText(textTokens[i]); }
};

// Remove all text tokens (for compatibility)
TBGame.prototype.removeAllTexts = function () {
	"use strict";
	this.removeTexts();
};

// Counting all text tokens that satisfy the argument (tag or function or nothing)
TBGame.prototype.countTexts = function (arg) {
	"use strict";
	var textTokens = this.findTexts(arg);
	return textTokens.length;
};

// Count all text tokens (for compatibility)
TBGame.prototype.countAllTexts = function () {
	"use strict";
	this.countTexts();
};


// Now generate the tbGame object
var tbGame = new TBGame(mainCanvas);
/////////////////////////////////////////////////////////////
// tbConstants: The engine part dealing constants and basic functions
//
// Copyright 2013 CoolGames
// Last Changed: 14-5-2013
/////////////////////////////////////////////////////////////


var tbKeyCodes = {
	vkCancel: 3,
	vkHelp: 6,
	vkBackspace: 8,
	vkTab: 9,
	vkClear: 12,
	vkReturn: 13,
	vkEnter: 14,
	vkShift: 16,
	vkControl: 17,
	vkAlt: 18,
	vkPause: 19,
	vkCapslock: 20,
	vkEscape: 27,
	vkSpace: 32,
	vkPageup: 33,
	vkPagedown: 34,
	vkEnd: 35,
	vkHome: 36,
	vkLeft: 37,
	vkUp: 38,
	vkRight: 39,
	vkDown: 40,
	vkPrintscreen: 44,
	vkInsert: 45,
	vkDelete: 46,
	vk0: 48,
	vk1: 49,
	vk2: 50,
	vk3: 51,
	vk4: 52,
	vk5: 53,
	vk6: 54,
	vk7: 55,
	vk8: 56,
	vk9: 57,
	vkSemicolon: 59,
	vkEquals: 61,
	vkA: 65,
	vkB: 66,
	vkC: 67,
	vkD: 68,
	vkE: 69,
	vkF: 70,
	vkG: 71,
	vkH: 72,
	vkI: 73,
	vkJ: 74,
	vkK: 75,
	vkL: 76,
	vkM: 77,
	vkN: 78,
	vkO: 79,
	vkP: 80,
	vkQ: 81,
	vkR: 82,
	vkS: 83,
	vkT: 84,
	vkU: 85,
	vkV: 86,
	vkW: 87,
	vkX: 88,
	vkY: 89,
	vkZ: 90,
	vkContextmenu: 93,
	vkNumpad0: 96,
	vkNumpad1: 97,
	vkNumpad2: 98,
	vkNumpad3: 99,
	vkNumpad4: 100,
	vkNumpad5: 101,
	vkNumpad6: 102,
	vkNumpad7: 103,
	vkNumpad8: 104,
	vkNumpad9: 105,
	vkMultiply: 106,
	vkAdd: 107,
	vkSeparator: 108,
	vkSubtract: 109,
	vkDecimal: 110,
	vkDivide: 111,
	vkF1: 112,
	vkF2: 113,
	vkF3: 114,
	vkF4: 115,
	vkF5: 116,
	vkF6: 117,
	vkF7: 118,
	vkF8: 119,
	vkF9: 120,
	vkF10: 121,
	vkF11: 122,
	vkF12: 123,
	vkF13: 124,
	vkF14: 125,
	vkF15: 126,
	vkF16: 127,
	vkF17: 128,
	vkF18: 129,
	vkF19: 130,
	vkF20: 131,
	vkF21: 132,
	vkF22: 133,
	vkF23: 134,
	vkF24: 135,
	vkNumlock: 144,
	vkScrolllock: 145,
	vkComma: 188,
	vkPeriod: 190,
	vkSlash: 191,
	vkBackquote: 192,
	vkOpenbracket: 219,
	vkBackslash: 220,
	vkClosebracket: 221,
	vkQuote: 222,
	vkMeta: 224
};
/////////////////////////////////////////////////////////////
// tbMasks: The engine part dealing collision masks
//
// Copyright 2013 CoolGames
// Last Changed: 9-9-2013
/////////////////////////////////////////////////////////////

// The mask constants
var maskUndefined = -1;
var maskPoint = 0;
var maskLine = 1;
var maskRectangle = 2;
var maskDisk = 3;
var maskPolygon = 4;

// The constructor for the mask
var tbMask = function(kind, args) {
	"use strict";
	var i;
	this.kind = kind;									// The kind of mask
	this.polygon = null;								// The mask area object (depending on the kind)
	switch (this.kind) {
		case maskPoint:
			this.polygon = {x: [args.x], y: [args.y]};
			this.bBoxLeft = args.x;
			this.bBoxTop = args.y;
			this.bBoxRight = args.x;
			this.bBoxBottom = args.y;
			break;
		case maskRectangle:
			this.polygon = {x: [args.x, args.x+args.w-1, args.x+args.w-1, args.x, args.x],
								y: [args.y, args.y, args.y+args.h-1, args.y+args.h-1, args.y]};
			this.bBoxLeft = args.x;
			this.bBoxTop = args.y;
			this.bBoxRight = args.x+args.w-1;
			this.bBoxBottom = args.y+args.h-1;
			break;
		case maskDisk:
			this.polygon = {x: [], y: []};
			this.bBoxLeft = args.x-args.r;
			this.bBoxTop = args.y-args.r;
			this.bBoxRight = args.x+args.r;
			this.bBoxBottom = args.y+args.r;
			break;
		case maskLine:
			this.polygon = {x: [args.x1, args.x2], y: [args.y1, args.y2]};
			this.bBoxLeft = Math.min(args.x1, args.x2);
			this.bBoxTop = Math.min(args.y1, args.y2);
			this.bBoxRight = Math.max(args.x1, args.x2);
			this.bBoxBottom = Math.max(args.y1, args.y2);
			break;
		case maskPolygon:
			this.polygon = {x: [], y: []};
			this.bBoxLeft = args.x[0]; this.bBoxTop = args.y[0]; this.bBoxRight = args.x[0]; this.bBoxBottom = args.y[0];
			for (i=0; i< args.x.length; i += 1) {
				this.polygon.x.push(args.x[i]);
				this.polygon.y.push(args.y[i]);
				this.bBoxLeft = Math.min(this.bBoxLeft,args.x[i]);
				this.bBoxTop = Math.min(this.bBoxTop,args.y[i]);
				this.bBoxRight = Math.max(this.bBoxRight,args.x[i]);
				this.bBoxBottom = Math.max(this.bBoxBottom,args.y[i]);
			}
			this.polygon.x.push(args.x[0]);
			this.polygon.y.push(args.y[0]);
			break;
		default:
			this.bBoxLeft = 0;
			this.bBoxTop = 0;
			this.bBoxRight = -1;
			this.bBoxBottom = -1;
	}
};

// Create a point mask
var tbMaskPoint = function(x, y) {
	"use strict";
	return new tbMask(maskPoint, {x: x, y: y});
};

// Create a rectangle mask
var tbMaskRectangle = function(x, y, w, h) {
	"use strict";
	return new tbMask(maskRectangle, {x: x, y: y, w: w, h: h});
};

// Create an disk mask
var tbMaskDisk = function(x, y, r) {
	"use strict";
	return new tbMask(maskDisk, {x: x, y: y, r: r});
};

// Create a line segment mask
var tbMaskLine = function(x1, y1, x2, y2) {
	"use strict";
	return new tbMask(maskLine, {x1: x1, y1: y1, x2: x2, y2: y2});
};

// Create a triangular mask
var tbMaskTriangle = function(x1, y1, x2, y2, x3, y3) {
	"use strict";
	return new tbMask(maskPolygon, {x: [x1, x2, x3], y: [y1, y2, y3] });
};

// Create a polygonal mask
var tbMaskPolygon = function(x, y) {
	"use strict";
	return new tbMask(maskPolygon, {x: x, y: y});
};

// Create a rectangle mask for a sub-image
var tbMaskSubImage = function(image, number) {
	"use strict";
	// Get the number in range
	number = (Math.round(number) % image.number + image.number) % image.number;
	// Compute the mask
	return new tbMask(maskRectangle, {x: image.subXOff[number]-image.xOrigin,
										y: image.subYOff[number]-image.yOrigin,
										w: image.subW[number],
										h: image.subH[number]});
};

// Create a rectangle mask for an image
var tbMaskImage = function(image) {
	"use strict";
	var x1 = 1000000, x2 = -1000000, y1 = 1000000, y2 = -1000000,
		number, nx1, nx2, ny1, ny2;
	for (number=0; number<image.number; number += 1) {
		nx1 = image.subXOff[number]-image.xOrigin;
		nx2 = nx1 + image.subW[number] - 1;
		ny1 = image.subYOff[number]-image.yOrigin;
		ny2 = ny1 + image.subH[number] - 1;
		if (nx1 < x1) { x1 = nx1; }
		if (nx2 > x2) { x2 = nx2; }
		if (ny1 < y1) { y1 = ny1; }
		if (ny2 > y2) { y2 = ny2; }
	}
	return new tbMask(maskRectangle, {x: x1, y: y1, w: x2-x1+1, h:y2-y1+1});
};


//
// Transformations. All these methods return a new mask. They do not change the current mask
//

tbMask.prototype.clone = function() {
	"use strict";
	var mask = new tbMask(maskUndefined,{ }),
		i;
	mask.kind = this.kind;
	mask.bBoxLeft = this.bBoxLeft;
	mask.bBoxRight = this.bBoxRight;
	mask.bBoxTop = this.bBoxTop;
	mask.bBoxBottom = this.bBoxBottom;
	mask.polygon = {x: [], y: []};
	for (i=0; i<this.polygon.x.length; i += 1) { mask.polygon.x[i] = this.polygon.x[i]; }
	for (i=0; i<this.polygon.y.length; i += 1) { mask.polygon.y[i] = this.polygon.y[i]; }
	return mask;
};

tbMask.prototype.translate = function(tx, ty) {
	"use strict";
	var mask = this.clone(),
		i;
	mask.bBoxLeft += tx;
	mask.bBoxRight += tx;
	mask.bBoxTop += ty;
	mask.bBoxBottom += ty;
	for (i=0; i<mask.polygon.x.length; i += 1) { mask.polygon.x[i] += tx; }
	for (i=0; i<mask.polygon.y.length; i += 1) { mask.polygon.y[i] += ty; }
	return mask;
};

tbMask.prototype.scale = function(sc) {
	"use strict";
	var mask = this.clone(),
		i;
	mask.bBoxLeft *= sc;
	mask.bBoxRight *= sc;
	mask.bBoxTop *= sc;
	mask.bBoxBottom *= sc;
	for (i=0; i<mask.polygon.x.length; i += 1) { mask.polygon.x[i] *= sc; }
	for (i=0; i<mask.polygon.y.length; i += 1) { mask.polygon.y[i] *= sc; }
	return mask;
};

tbMask.prototype.rotate = function(angle) {
	"use strict";
	var point0, point1, r, center, pointx, pointy, i;
	switch (this.kind) {
		case maskPoint:
			point0 = new tbVector(this.polygon.x[0],this.polygon.y[0]);
			point0 = point0.rotate(angle);
			return tbMaskPoint(point0.x,point0.y);
			break;
		case maskLine:
			point0 = new tbVector(this.polygon.x[0],this.polygon.y[0]);
			point0 = point0.rotate(angle);
			point1 = new tbVector(this.polygon.x[1],this.polygon.y[1]);
			point1 = point1.rotate(angle);
			return tbMaskLine(point0.x,point0.y,point1.x,point1.y);
			break;
		case maskDisk:
			r = (this.bBoxRight-this.bBoxLeft)/2;
			center = new tbVector(this.bBoxLeft+r,this.bBoxTop+r);
			center = center.rotate(angle);
			return tbMaskDisk(center.x,center.y,r);
			break;
		default:
			pointx = [];
			pointy = [];
			for (i=0; i<this.polygon.x.length-1; i += 1) {
				point0 = new tbVector(this.polygon.x[i],this.polygon.y[i]);
				point0 = point0.rotate(angle);
				pointx.push(point0.x);
				pointy.push(point0.y);
			}
			return tbMaskPolygon(pointx,pointy);
			break;
	}
};


//
// Collision checking, using SAT method
//

// Project a polygonal mask placed at x,y on the vector
tbMask.prototype._project = function(x,y,axis) {
	"use strict";
	var v = new tbVector(0,0),
		scalar,
		minVal = 1000000000,
		maxVal = -10000000000,
		i;
	for (i=0; i < this.polygon.x.length; i += 1) {
		v.x = x + this.polygon.x[i];
		v.y = y + this.polygon.y[i];
		scalar = v.dot(axis);
		minVal = Math.min(minVal,scalar);
		maxVal = Math.max(maxVal,scalar);
	}
	return {min: minVal, max:maxVal};
};

// Return the axis for the polygon mask
tbMask.prototype._getAxes = function() {
	"use strict";
	var v1 = new tbVector(0,0),
		v2 = new tbVector(0,0),
		axes = [],
		i;
	for (i=0; i < this.polygon.x.length-1; i += 1) {
		v1.x = this.polygon.x[i];
		v1.y = this.polygon.y[i];
		v2.x = this.polygon.x[i+1];
		v2.y = this.polygon.y[i+1];
		axes.push(v1.subtract(v2).normal());
	}
	return axes;
};

// Determines whether this mask at position x,y collides with mask at position mX, mY
tbMask.prototype.collisionMask = function(x, y, mask, mX, mY) {
	"use strict";
	var r, axes, pr1, pr2, i;
	// First check the bounding box
	if (mX + mask.bBoxRight < x+this.bBoxLeft || mX + mask.bBoxLeft > x+this.bBoxRight) { return false; }
	if (mY + mask.bBoxBottom < y+this.bBoxTop || mY + mask.bBoxTop > y+this.bBoxBottom) { return false; }
	// Now do the easy cases
	if (this.kind === maskRectangle && mask.kind === maskRectangle) {
		return true;
	}
	if (mask.kind === maskDisk) {
		r = (mask.bBoxRight-mask.bBoxLeft)/2;
		return this.collisionDisk(x,y,mX+mask.bBoxLeft+r,mY+mask.bBoxTop+r,r);
	}
	if (this.kind === maskDisk) {
		r = (this.bBoxRight-this.bBoxLeft)/2;
		return mask.collisionDisk(mX,mY,x+this.bBoxLeft+r,y+this.bBoxTop+r,r);
	}
	if (mask.kind === maskPoint) {
		return this.collisionPoint(x,y,mX+mask.bBoxLeft,mY+mask.bBoxTop);
	}
	if (this.kind === maskPoint) {
		return mask.collisionPoint(mX,mY,x+this.bBoxLeft,y+this.bBoxTop);
	}
	// General case
	axes = this._getAxes().concat(mask._getAxes());
	for (i=0; i<axes.length; i += 1) {
		pr1 = this._project(x,y,axes[i]);
		pr2 = mask._project(mX,mY,axes[i]);
		if (pr1.max < pr2.min || pr2.max < pr1.min) { return false; }
	}
	return true;
};

// Determine whether the mask at position x,y collides with a disk
tbMask.prototype.collisionDisk = function(x, y, colX, colY, colR) {
	"use strict";
	var dx, dy, r, axes, nx, ny, nd, i, dist, v1, v2, scalar, pr1, pr2;
	// First check the bounding box
	if (colX+colR < x+this.bBoxLeft || colX-colR > x+this.bBoxRight) { return false; }
	if (colY+colR < y+this.bBoxTop || colY-colR > y+this.bBoxBottom) { return false; }
	// Now do precise checking
	switch (this.kind) {
		case maskPoint:
			dx = colX-(x+this.bBoxLeft);
			dy = colY-(y+this.bBoxTop);
			return dx*dx+dy*dy <= colR*colR;
			break;
		case maskDisk:
			r = (this.bBoxRight-this.bBoxLeft)/2;
			dx = colX-(x+this.bBoxLeft+r);
			dy = colY-(y+this.bBoxTop+r);
			return dx*dx+dy*dy <= (r+colR)*(r+colR);
			break;
		default:
			axes = this._getAxes();
			// Find nearest point on polygon
			nx = 0; ny = 0; nd = 1000000000;
			for (i=0; i< this.polygon.x.length; i += 1)
			{
				dx = x+this.polygon.x[i] - colX;
				dy = y+this.polygon.y[i] - colY;
				dist = dx*dx+dy*dy;
				if (dist<=nd) { nx = x+this.polygon.x[i]; ny = y+this.polygon.y[i]; nd = dist; }
			}
			v1 = new tbVector(colX,colY);
			v2 = new tbVector(nx,ny);
			axes.push(v1.subtract(v2).normalize());
			// Now do the projections
			for (i=0; i<axes.length; i += 1) {
				scalar = v1.dot(axes[i]);
				pr1 = {min: scalar-colR, max:scalar+colR};
				pr2 = this._project(x,y,axes[i]);
				if (pr1.max < pr2.min || pr2.max < pr1.min) { return false; }
			}
			return true;
			break;
	}
};

// Determine whether the mask at position x,y collides with a point
tbMask.prototype.collisionPoint = function(x, y, colX, colY) {
	"use strict";
	var r, dx, dy, x1, y1, x2, y2, yy, axes, v, pr1, pr2, i;
	// First check the bounding box
	if (colX < x+this.bBoxLeft || colX > x+this.bBoxRight) { return false; }
	if (colY < y+this.bBoxTop || colY > y+this.bBoxBottom) { return false; }
 	// Now do precise checking
	switch (this.kind) {
		case maskPoint:
		case maskRectangle:
			return true;
			break;
		case maskDisk:
			r = (this.bBoxRight-this.bBoxLeft)/2;
			dx = colX-(x+this.bBoxLeft+r);
			dy = colY-(y+this.bBoxTop+r);
			return dx*dx+dy*dy <= r*r;
			break;
		case maskLine:
			x1 = x + this.polygon.x[0];
			y1 = y + this.polygon.y[0];
			x2 = x + this.polygon.x[1];
			y2 = y + this.polygon.y[1];
			if (colX === x1) { return (colY === y1); }
			if (colX === x2) { return (colY === y2); }
			yy = y1 + (colX-x1)*(y2-y1)/(x2-x1);
			return (Math.abs(yy-colY) < 1);
			break;
		case maskPolygon:
			axes = this._getAxes();
			for (i=0; i<axes.length; i += 1) {
				v = new tbVector(colX,colY);
				pr1 = v.dot(axes[i]);
				pr2 = this._project(x,y,axes[i]);
				if (pr1 < pr2.min || pr2.max < pr1) { return false; }
			}
			return true;
			break;
		default:
			return false;
	}
};

// Draw the mask (for testing)
tbMask.prototype.draw = function(x, y, color) {
	"use strict";
	var context = tbGraphics.context,
		r, i;
	context.fillStyle = color;
	context.strokeStyle = color;
	switch (this.kind) {
		case maskPoint:
			context.fillRect(x+this.bBoxLeft-1, y+this.bBoxTop-1, 3, 3);
			break;
		case maskRectangle:
			context.fillRect(x+this.bBoxLeft, y+this.bBoxTop, this.bBoxRight-this.bBoxLeft+1, this.bBoxBottom-this.bBoxTop+1);
			break;
		case maskDisk:
			r = (this.bBoxRight-this.bBoxLeft)/2;
			context.beginPath();
			context.arc(x+this.bBoxLeft+r,y+this.bBoxTop+r,r,0,2 * Math.PI, false);
			context.closePath();
			context.fill();
			break;
		case maskLine:
			context.beginPath();
			context.moveTo(x + this.polygon.x[0],y + this.polygon.y[0]);
			context.lineTo(x + this.polygon.x[1],y + this.polygon.y[1]);
			context.stroke();
			break;
		case maskPolygon:
			context.beginPath();
			context.moveTo(x + this.polygon.x[0],y + this.polygon.y[0]);
			for (i=1; i<this.polygon.x.length-1; i += 1) {
				context.lineTo(x + this.polygon.x[i],y + this.polygon.y[i]);
			}
			context.closePath();
			context.fill();
			break;
	}
};

// Draw the bounding box of the mask (for testing)
tbMask.prototype.drawBox = function(x, y, color) {
	"use strict";
	var context = tbGraphics.context;
	context.strokeStyle = color;
	context.strokeRect(x+this.bBoxLeft, y+this.bBoxTop, this.bBoxRight-this.bBoxLeft+1, this.bBoxBottom-this.bBoxTop+1);
};


/////////////////////////////////////////////////////////////
// tbTimeLine: The engine part dealing with time lines that
// can perform actions at certain moments in time.
//
// Copyright 2013 CoolGames
// Last Changed: 9-9-2013
/////////////////////////////////////////////////////////////


var tbTimeLine = function() {
// The constructor for the time line object
	// A time line is a token so we nee some properties used by the engine
	"use strict";
	this.depth = 10000000;				// Handle as early as possible
	this.visible = false;
	this.active = true;
	this.group = "Engine";
	// Own properties
	this._moments = [];					// The moments at which actions must be performed
	this._running = false;				// Whether the time line is running
	this._paused = false;				// Whether the time line is paused
	this._loop = false;					// Whether to loop
	this._speed = 1;					// Speed with which to run
	this._lastTime = -1;				// The previously handled time
	this._nextMoment = -1000000;		// The next moment to handle
};


//
// Help methods
//

tbTimeLine.prototype._sort = function() {
// Function to sort the moments
	"use strict";

	var sortfunction = function (a, b) {
		return (a.moment - b.moment);
	};
	
	this._moments.sort(sortfunction);
};


//
// Maintaining the time line
//

tbTimeLine.prototype.clone = function() {
// Returns a clone of the time line
	"use strict";
	var res = new tbTimeLine(),
		i;
	for (i = 0; i < this._moments.length; i += 1) {
		res._moments.push( { moment: this._moments[i].moment, action: this._moments[i].action } );
	}
	res._loop = this._loop;
	return res;
};

tbTimeLine.prototype.merge = function(timeLine) {
// Merges the moments of the time line into the current one
	"use strict";
	var i;
	for (i = 0; i < timeLine._moments.length; i += 1) {
		this._moments.push( { moment: timeLine._moments[i].moment, action: timeLine._moments[i].action } );
	}
	this._sort();
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.addMoment = function(moment, action) {
// Adds a moment to the time line
	"use strict";
	var ind, i;
	if (this._moments.length === 0 || this._moments[this._moments.length-1].moment <= moment) {
		this._moments.push({ moment: moment, action: action });
	} else {
		ind = 0;
		while (this._moments[ind].moment <= moment) { ind += 1; }
		for (i = this._moments.length; i > ind; i -= 1) { 
			this._moments[i] = this._moments[i-1];
		}
		this._moments[ind] = { moment: moment, action: action };
	}
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.changeMoment = function(moment, action) {
// Changes the action at a moment in the time line
	"use strict";
	this.deleteMoment(moment);
	this.addMoment(moment, action);
};

tbTimeLine.prototype.deleteMoment = function(moment) {
// Removes all moments at the given moment
	"use strict";
	this.deleteRange(moment, moment);
};

tbTimeLine.prototype.shiftMoment = function(moment, amount) {
// Shift all moments at the given moment
	"use strict";
	this.shiftRange(moment, moment, amount);
};

tbTimeLine.prototype.duplicateMoment = function(moment, newMoment) {
// Duplicates all moments at the given moment
	"use strict";
	this.duplicateRange(moment, moment, newMoment);
};

tbTimeLine.prototype.deleteRange = function(moment0, moment1) {
// Removes all moments in the range
	"use strict";
	var ind = 0,
		i;
	for (i = 0; i < this._moments.length; i += 1) {
		if (this._moments[i].moment < moment0) {
			ind += 1;
		} else if (this._moments[i].moment > moment1) {
			this._moments[ind] = this._moments[i];
			ind += 1;
		}
	}
	this._moments.length = ind;
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.shiftRange = function(moment0, moment1, amount) {
// Shifts all moments in the range
	"use strict";
	var i;
	for (i = 0; i < this._moments.length; i += 1) {
		if (this._moments[i].moment >= moment0 && this._moments[i].moment <= moment1) {
			this._moments[i].moment += amount;
		}
	}
	this._sort();
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.duplicateRange = function(moment0, moment1, newMoment) {
// Duplicates all moments in the range starting at the new moment
	"use strict";
	var shift = newMoment - moment0,
		i;
	for (i = 0; i < this._moments.length; i += 1) {
		if (this._moments[i].moment >= moment0 && this._moments[i].moment <= moment1) {
			this._moments.push( { moment: this._moments[i].moment + shift, action: this._moments[i].action } );
		}
	}
	this._sort();
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.stretchRange = function(moment0, moment1, factor) {
// Stretches all moments in the range
	"use strict";
	var shift = (moment1-moment0) * (factor-1),   // Amount to shift moments after the range
		i;
	for (i = 0; i < this._moments.length; i += 1) {
		if (this._moments[i].moment >= moment0 && this._moments[i].moment <= moment1) {
			this._moments[i].moment = moment0 + (this._moments[i].moment-moment0) * factor;
		} else if (this._moments[i].moment > moment1) {
			this._moments[i].moment += shift;
		}
	}
	this._sort();
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.reverseRange = function(moment0, moment1) {
// Reverses all moments in the range
	"use strict";
	var i;
	for (i = 0; i < this._moments.length; i += 1) {
		if (this._moments[i].moment >= moment0 && this._moments[i].moment <= moment1) {
			this._moments[i].moment = moment1 - (this._moments[i].moment-moment0);
		}
	}
	this._sort();
	this._nextMoment = -1000000;  // invalidate next moment
};

tbTimeLine.prototype.deleteAll = function() {
// Removes all moments
	"use strict";
	if (this._moments.length > 0) {
		this.deleteRange(0, this._moments[this._moments.length-1].moment);
	}
};

tbTimeLine.prototype.shiftAll = function(amount) {
// Shifts all moments
	"use strict";
	if (this._moments.length > 0) {
		this.shiftRange(0, this._moments[this._moments.length-1].moment, amount);
	}
};

tbTimeLine.prototype.duplicateAll = function(newMoment) {
// Duplicates all moments starting at the new moment
	"use strict";
	if (this._moments.length > 0) {
		this.duplicateRange(0, this._moments[this._moments.length-1].moment, newMoment);
	}
};

tbTimeLine.prototype.stretchAll = function(factor) {
// Stretches all moments
	"use strict";
	if (this._moments.length > 0) {
		this.stretchRange(0, this._moments[this._moments.length-1].moment, factor);
	}
};

tbTimeLine.prototype.reverseAll = function() {
// Reverses all moments
	"use strict";
	if (this._moments.length > 0) {
		this.stretchRange(0, this._moments[this._moments.length-1].moment);
	}
};


//
// Running the time line
//

tbTimeLine.prototype.start = function() {
// Start the time line to run at the start
	"use strict";
	this._running = true;
	this._paused = false;
	if (this._speed < 0 && this._moments.length > 0) {
		this._lastTime = this._moments[this._moments.length-1].moment + 1;
	} else {
		this._lastTime = -1;
	}
	this._nextMoment = -1000000;
	// Add it to the game (removing it first when it already exists)
	tbGame.removeToken(this);
	tbGame.addToken(this);
};
	
tbTimeLine.prototype.startMoment = function(moment) {
// Start the time line to run at the indicated moment
	"use strict";
	this.start();
	this.setMoment(moment);
};
	
tbTimeLine.prototype.setMoment = function(moment) {
// 	Set the moment from where to run
	"use strict";
	if (this._speed < 0) {
		this._lastTime = moment + 1;
	} else {
		this._lastTime = moment - 1;
	}
	this._nextMoment = -1000000;
};

tbTimeLine.prototype.getMoment = function() {
// 	Returns the current time moment
	"use strict";
	return this._lastTime;
};
	
tbTimeLine.prototype.rewind = function() {
// 	Set the moment from where to run to the start
	"use strict";
	if (this._speed < 0 && this._moments.length > 0) {
		this.setMoment(this._moments[this._moments.length-1].moment);
	} else {
		this.setMoment(0);
	}
};
	
tbTimeLine.prototype.stop = function() {
// Stop the time line
	"use strict";
	this._running = false;
	tbGame.removeToken(this);
};
	
tbTimeLine.prototype.running = function() {
// Returns whether the time line is running
	"use strict";
	return this._running;
};
	
tbTimeLine.prototype.pause = function() {
// Pause the time line
	"use strict";
	this._paused = true;
	tbGame.removeToken(this);
};
	
tbTimeLine.prototype.resume = function() {
// Resumes the time line
	"use strict";
	this._paused = false;
	// Add it to the game (removing it first when it already exists)
	tbGame.removeToken(this);
	tbGame.addToken(this);
};
	
tbTimeLine.prototype.paused = function() {
// Returns whether the time line is paused
	"use strict";
	return (this._running && this._paused);
};
	
tbTimeLine.prototype.setLooping = function(loop) {
// Set whether to loop
	"use strict";
	this._loop = loop;
};
	
tbTimeLine.prototype.getLooping = function() {
// Return whether to loop
	"use strict";
	return this._loop ;
};
	
tbTimeLine.prototype.setSpeed = function(speed) {
// 	Set the speed with which to run
	"use strict";
	this._speed = speed;
};

tbTimeLine.prototype.getSpeed = function() {
// 	Returns the speed with which to run
	"use strict";
	return this._speed;
};
	
//
// Callback method to handle all
//

tbTimeLine.prototype.handleMoments = function(timePassed) { 
// This is where the time line does its work
	"use strict";
	if (!this._running || this._paused) { return; }
	if (this._speed === 0) { return; }
	if (this._speed > 0) {
		// Skip moment that did lie in the past
		if (this._nextMoment < 0) { this._nextMoment = 0; }
		while (this._nextMoment < this._moments.length && this._moments[this._nextMoment].moment <= this._lastTime) {
			this._nextMoment += 1;
		}
		// Handle moments that where passed
		this._lastTime += this._speed * timePassed;
		while (this._nextMoment >= 0 && this._nextMoment < this._moments.length && this._moments[this._nextMoment].moment <= this._lastTime) {
			this._moments[this._nextMoment].action(this._moments[this._nextMoment].moment, this);
			this._nextMoment += 1;
		}
		// See whether we are done
		if (this._nextMoment >= this._moments.length) {
			if (this._loop) {
				this.rewind();
			} else {
				this.stop();
			}
		}
	} else {
		// Skip moment that did lie in the past
		if (this._nextMoment < 0) { this._nextMoment = this._moments.length-1; }
		while (this._nextMoment >= 0 && this._moments[this._nextMoment].moment >= this._lastTime) { 
			this._nextMoment -= 1;
		}
		// Handle moments that where passed
		this._lastTime += this._speed * timePassed;
		while (this._nextMoment >= 0 && this._moments[this._nextMoment].moment >= this._lastTime) {
			this._moments[this._nextMoment].action(this._moments[this._nextMoment].moment, this);
			this._nextMoment -= 1;
		}
		// See whether we are done
		if (this._nextMoment < 0 && this._lastTime <= 0) {
			if (this._loop) {
				this.rewind();
			} else {
				this.stop();
			}
		}
	}
};

tbTimeLine.prototype.handleBeginStep = function(timePassed) { 
// You can override this but make sure you at least include this call
	"use strict";
	this.handleMoments(timePassed);
};
/////////////////////////////////////////////////////////////
// tbKeyAreas: The engine part dealing with virtual key
// code areas that can be used to simulate key presses
//
// Copyright 2013 CoolGames
// Last Changed: 9-9-2013
/////////////////////////////////////////////////////////////

// The constructor for the key area object
var tbKeyArea = function () {
	"use strict";
	// Properties used by the engine
	this.depth = 10000000;				// Handle as early as possible
	this.visible = false;
	this.active = true;
	this.group = "Engine";
	// Own properties
	this._areas = [];
	this._fingerIndex = [];
	this.clear();
	this.sliding = false;
	// Add it to the game engine
	tbGame.addToken(this);
};

//
// help methods
//

// Check whether the area contains a point
tbKeyArea.prototype._containsPoint = function(area,x,y) { 
	"use strict";
	if (!area) { return false; }
	return area.mask.collisionPoint(0, 0, x, y);
};

//
// The event handlers
//

// handleBeginStep is called each step before processing other events.
tbKeyArea.prototype.handleBeginStep = function(timePassed) { 
	"use strict";
	var finger, x, y, ind, area;
	if (!this.sliding) { return; }
	for (finger = 0; finger < 16; finger += 1) {
		if (tbGame.touchDown(finger)) {
			x = tbGame.touchX(finger);
			y = tbGame.touchY(finger);
			ind = this._fingerIndex[finger];
			area = this._areas[ind];
			// Check whether a finger moved out of a region
			if (ind >= 0 && area && area.selected && !this._containsPoint(area,x,y)) {
				tbGame.addKeyUpEvent(area.keyCode);
				area.selected = false;
				this._fingerIndex[finger] = -1;
			}		
			// Check whether the finger moved into a region
			this.handleTouchDown(finger,x,y);
		}
	}	
};

// handleTouchDown is called when a touch is started or if a mouse button is pressed.
tbKeyArea.prototype.handleTouchDown = function(finger, x, y) { 
	"use strict";
	var i;
	if (this._fingerIndex[finger] >= 0) { return; }
	for (i=0; i<this._areas.length; i += 1) {
		if (this._areas[i] && this._containsPoint(this._areas[i],x,y) && !this._areas[i].selected) {
			tbGame.addKeyDownEvent(this._areas[i].keyCode);
			this._areas[i].selected = true;
			this._fingerIndex[finger] = i;
			return;
		}
	}
}
;
// handleTouchUp is called when a touch is ended or if a mouse button is released.
tbKeyArea.prototype.handleTouchUp = function(finger, x, y) { 
	"use strict";
	var ind = this._fingerIndex[finger];
	if (ind >= 0 && this._areas[ind] && this._areas[ind].selected) {
		tbGame.addKeyUpEvent(this._areas[ind].keyCode);
		this._areas[ind].selected = false;
	}		
	this._fingerIndex[finger] = -1;
};

//
// The user methods
//

// Add a rectangle with the given id
tbKeyArea.prototype.addRectangle = function(id, x, y, w, h, keyCode) {
	"use strict";
	var mask = tbMaskRectangle(x, y, w, h);
	this._areas.push( { keyCode: keyCode, mask: mask, id:id, selected: false } );
};

// Add a disk with the given id
tbKeyArea.prototype.addDisk = function(id, x, y, r, keyCode) {
	"use strict";
	var mask = tbMaskDisk(x, y, r);
	this._areas.push( { keyCode: keyCode, mask: mask, id:id, selected: false } );
};

// Add a triangle with the given id
tbKeyArea.prototype.addTriangle = function(id, x1, y1, x2, y2, x3, y3, keyCode) {
	"use strict";
	var mask = tbMaskTriangle(x1, y1, x2, y2, x3, y3);
	this._areas.push( { keyCode: keyCode, mask: mask, id:id, selected: false } );
};

// Add a polygon with the given id
tbKeyArea.prototype.addPolygon = function(id, x, y, keyCode) {
	"use strict";
	var mask = tbMaskPolygon(x, y);
	this._areas.push( { keyCode: keyCode, mask: mask, id:id, selected: false } );
};

// Add a mask at a location with the given id
tbKeyArea.prototype.addMask = function(id, x, y, mask, keyCode) {
	"use strict";
	this._areas.push( { keyCode: keyCode, mask: mask.translate(x,y), id:id, selected: false } );
};

// Remove an area
tbKeyArea.prototype.removeArea = function(areaId) {
	"use strict";
	var i;
	for (i=0; i<this._areas.length; i += 1) {
		if (this._areas[i].id === areaId) {
			this._areas[i] = null;
		}
	}
};

// Remove a key code
tbKeyArea.prototype.removeKeyCode = function(keyCode) {
	"use strict";
	var i;
	for (i=0; i<this._areas.length; i += 1) {
		if (this._areas[i].keyCode === keyCode) { 
			this._areas[i] = null;
		}
	}
};

// Remove all (only moment we actually delete entries in the array!)
tbKeyArea.prototype.clear = function() {
	"use strict";
	var i;
	this._areas.length = 0;
	for (i=0; i<16; i += 1) {
		this._fingerIndex[i] = -1;
	}
};

tbKeyArea.prototype.draw = function(colUp, colDown, colText) {
	"use strict";
	var i, area, xx, yy;
	for (i=0; i<this._areas.length; i += 1) {
		area = this._areas[i];
		if (area) { 
			if (area.selected) {
				area.mask.draw(0,0,colDown); 
			} else {
				area.mask.draw(0,0,colUp);
			}
			xx = (area.mask.bBoxLeft+area.mask.bBoxRight)/2;
			yy = (area.mask.bBoxTop+area.mask.bBoxBottom)/2;
			tbGraphics.drawText("id: " + area.id, xx - 20, yy - 10, colText, "16px Arial");
			tbGraphics.drawText("key: " + area.keyCode, xx - 20, yy + 10, colText, "16px Arial");
		}
	}
};


/////////////////////////////////////////////////////////////
// tbParticles: This engine component deals with particle systems
//
// This is a complete rewrite, replacing the old particle engine
//
// Copyright 2014 CoolGames
// Last Changed: 16-07-2014
/////////////////////////////////////////////////////////////


var tbParticle = {};

tbParticle.generator = new tbRandom();			// The random number generator we use
tbParticle.particlePool = [];					// The pool of particles that can be reused


// ********************************************
// ** Distribution functions
// ********************************************

tbParticle.distributionRandom = function(min, max) {
	"use strict";
	return tbParticle.generator.randomRange(min, max);
};

tbParticle.distributionIRandom = function(min, max) {
	"use strict";
	return tbParticle.generator.iRandomRange(min, max);
};

tbParticle.distributionRounded = function(min, max, step) {
	"use strict";
	return step * tbParticle.generator.iRandomRange(min/step, max/step);
};


// ********************************************
// ** Support functions
// ********************************************

// Clone a distribution value
tbParticle.cloneDistributionValue = function(value) {
	"use strict";
	if (typeof value !== "object" || value === null || value.min === undefined) {
		if (Array.isArray(value)) {
			return value.slice();
		}
		return value;
	}
	return { min: value.min, max: value.max, distribution: value.distribution, parameter: value.parameter };
};

// Compute a value from a distribution object
tbParticle.computeDistributionValue = function(value) {
	"use strict";
	if (typeof value !== "object" || value === null || value.min === undefined) {
		if (Array.isArray(value)) {
			return value[tbParticle.generator.iRandomRange(0, value.length-1)];
		}
		return value;
	}
	if (value.max === undefined) {
		if (Array.isArray(value.min)) {
			return value.min[tbParticle.generator.iRandomRange(0, value.min.length-1)];
		}
		return value.min;
	}
	if (value.distribution === undefined) {
		return tbParticle.generator.randomRange(value.min, value.max);
	}
	return value.distribution(value.min, value.max, value.parameter);
};

// Creates a value for a particle type
tbParticle.createInterpolationValue = function(begin, change, interpolation, cycle, absolute, parameter) {
	"use strict";
	if (change === undefined) { return begin; }
	if (interpolation === undefined) { return { begin: begin, change: change }; }
	if (cycle === undefined) { cycle = 1; }
	if (absolute === undefined) { absolute = false; }
	return { begin: begin, change: change, interpolation: interpolation, cycle: cycle, absolute: absolute, parameter: parameter };
};

// Clone a value for a particle type
tbParticle.cloneInterpolationValue = function(value) {
	"use strict";
	if (typeof value !== "object" || value === null || value.begin === undefined) {
		return tbParticle.cloneDistributionValue(value);
	}
	return {
		begin: value.begin === undefined ? undefined : tbParticle.cloneDistributionValue(value.begin),
		change: value.change === undefined ? undefined : tbParticle.cloneDistributionValue(value.change),
		interpolation: value.interpolation,
		cycle: value.cycle === undefined ? undefined : tbParticle.cloneDistributionValue(value.cycle),
		absolute: value.absolute,
		parameter: value.parameter === undefined ? undefined : tbParticle.cloneDistributionValue(value.parameter)
	};
};

// Initializes a value for a particle, based on the value in the particle type
tbParticle.initInterpolationValue = function(value) {
	"use strict";
	var val;
	if (typeof value !== "object" || value === null || value.begin === undefined) {
		return { value: tbParticle.computeDistributionValue(value) };
	}
	val = tbParticle.computeDistributionValue(value.begin);
	if (value.change === undefined) {
		return { value: val };
	}
	return {
		value: val,
		begin: val,
		change: tbParticle.computeDistributionValue(value.change),
		interpolation: value.interpolation,
		cycle: value.cycle === undefined ? 1 : tbParticle.computeDistributionValue(value.cycle),
		absolute: value.absolute === undefined ? false : value.absolute,
		parameter: tbParticle.computeDistributionValue(value.parameter)
	};
};

// Updates a value for a particle, based on the current time and duration
tbParticle.updateInterpolationValue = function(value, time, duration) {
	"use strict";
	if (value.begin === undefined) { return; }			// value is constant
	if (value.interpolation === undefined) {			// simple linear interpolation
		value.value = value.begin + time*value.change/duration;
	} else {
		if (!value.absolute) { time /= duration; }
		time = time % value.cycle;
		value.value = value.interpolation(time, value.begin, value.change, value.cycle, value.parameter);
	}
};


// ********************************************
// ** Defining particles
// ********************************************

// The constructor for a particle
tbParticle.Particle = function() {
	"use strict";
};

// The constructor for a particle
tbParticle.Particle.prototype.init = function(token, pType, x, y) {
	"use strict";
	this.token = token;
	this.pType = pType;
	// Settings related to the image
	this.shape = tbParticle.initInterpolationValue(pType.shape).value;
	this.image = tbParticle.initInterpolationValue(pType.image).value;
	this.subImage = tbParticle.initInterpolationValue(pType.subImage);
	this.autoSubImage = pType.autoSubImage;
	this.absoluteSubImage = pType.absoluteSubImage;
	this.alpha = tbParticle.initInterpolationValue(pType.alpha);
	this.imageColorMode = pType.imageColorMode;
	this.colorMode = pType.colorMode;
	this.colorRed = tbParticle.initInterpolationValue(pType.colorRed);
	this.colorGreen = tbParticle.initInterpolationValue(pType.colorGreen);
	this.colorBlue = tbParticle.initInterpolationValue(pType.colorBlue);
	this.colorHue = tbParticle.initInterpolationValue(pType.colorHue);
	this.colorSaturation = tbParticle.initInterpolationValue(pType.colorSaturation);
	this.colorLight = tbParticle.initInterpolationValue(pType.colorLight);
	this.additive = pType.additive;
	this.drawFunction = pType.drawFunction;
	if (this.shape === "image" && (this.image !== undefined || this.image !== null) && this.imageColorMode !== "none") {
		this.surf = new tbSurface(this.image.width, this.image.height);
	}
	// Settings related to the lifeTime
	this.delay = tbParticle.initInterpolationValue(pType.delay).value;
	this.lifeTime = tbParticle.initInterpolationValue(pType.lifeTime).value;
	this.stepBeginFunction = pType.stepBeginFunction;
	this.stepEndFunction = pType.stepEndFunction;
	this.time = 0;
	this.alive = true;
	// Settings related to secondary particles
	this.emitType = pType.emitType;
	if (this.emitType !== null) {
		this.emitAmount = tbParticle.initInterpolationValue(pType.emitAmount);
		this.emitAtEnd = pType.emitAtEnd;
		this.emitNumber = tbParticle.computeDistributionValue(pType.emitNumber);
		this.emitDelay = tbParticle.computeDistributionValue(pType.emitDelay);
		this.emitInterval = tbParticle.initInterpolationValue(pType.emitInterval);
		this.emitCreated = 0;
		this.emitNext = 0;
	}
	this.effectType = pType.effectType;
	if (this.effectType !== null) {
		this.effectAmount = tbParticle.initInterpolationValue(pType.effectAmount);
		this.effectAtEnd = pType.effectAtEnd;
		this.effectNumber = tbParticle.computeDistributionValue(pType.effectNumber);
		this.effectDelay = tbParticle.computeDistributionValue(pType.effectDelay);
		this.effectInterval = tbParticle.initInterpolationValue(pType.effectInterval);
		this.effectCreated = 0;
		this.effectNext = 0;
	}
	// Settings related to the motion
	this.x = x;
	this.y = y;
	this.followEffect = pType.followEffect;
	this.positionX = tbParticle.initInterpolationValue(pType.positionX);
	this.positionY = tbParticle.initInterpolationValue(pType.positionY);
	this.speed = tbParticle.initInterpolationValue(pType.speed);
	this.direction = tbParticle.initInterpolationValue(pType.direction);
	this.forceDirection = tbParticle.initInterpolationValue(pType.forceDirection);
	this.forceMagnitude = tbParticle.initInterpolationValue(pType.forceMagnitude);
	this.forceX = 0;
	this.forceY = 0;
	this.moveX = 0;
	this.moveY = 0;
	this.currentX = this.x + this.positionX.value;
	this.currentY = this.y + this.positionY.value;
	this.previousX = this.currentX;
	this.previousY = this.currentY;
	// Settings related to the appearance
	this.autoOrientation = pType.autoOrientation;
	this.orientation = tbParticle.initInterpolationValue(pType.orientation);
	this.size = tbParticle.initInterpolationValue(pType.size);
	this.scaleX = tbParticle.initInterpolationValue(pType.scaleX);
	this.scaleY = tbParticle.initInterpolationValue(pType.scaleY);
};

// Create secondary particles
tbParticle.Particle.prototype.createEmit = function() {
	"use strict";
	var t, numb, j, pType, xx, yy, particle;
	if (this.emitType === null) { return; }
	// Determine the number of particles to create
	tbParticle.updateInterpolationValue(this.emitAmount, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.emitInterval, this.time, this.lifeTime);
	if (this.time >= this.lifeTime) {
		if (!this.emitAtEnd) { return; }
		numb =  this.emitAmount.value*(this.emitNumber - this.emitCreated);
	} else {
		numb = 0;
		t = this.time - this.emitDelay;
		while (t >= this.emitNext && numb < this.emitNumber - this.emitCreated) {
			numb +=  this.emitAmount.value;
			this.emitNext += this.emitInterval.value;
		}
	}
	// Now create the particles
	for (j = 0; j < numb; j += 1) {
		pType = tbParticle.computeDistributionValue(this.emitType);
		xx = this.currentX;
		yy = this.currentY;
		if (this.followEffect) {
			xx += this.token.x;
			yy += this.token.y;
		}
		particle = new tbParticle.Particle();
		particle.init(this.token, pType,xx, yy);
		this.token.particles.push(particle);
	}
	this.emitCreated += numb;
	// Remove when finished
	if (this.emitCreated >= this.emitNumber) { this.emitType = null; }
};

tbParticle.Particle.prototype.createEffect = function() {
	"use strict";
	var t, numb, j, eType, xx, yy;
	if (this.effectType === null) { return; }
	// Determine the number of effects to create
	tbParticle.updateInterpolationValue(this.effectAmount, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.effectInterval, this.time, this.lifeTime);
	if (this.time >= this.lifeTime) {
		if (!this.effectAtEnd) { return; }
		numb = this.effectAmount.value*(this.effectNumber - this.effectCreated);
	} else {
		numb = 0;
		t = this.time - this.effectDelay;
		while (t >= this.effectNext && numb < this.effectNumber - this.effectCreated) {
			numb += this.effectAmount.value;
			this.effectNext += this.effectInterval.value;
		}
	}
	// Now create the effects
	for (j = 0; j < numb; j += 1) {
		eType = tbParticle.computeDistributionValue(this.effectType);
		xx = this.currentX;
		yy = this.currentY;
		if (this.followEffect) {
			xx += this.token.x;
			yy += this.token.y;
		}
		tbParticle.showEffect(eType,xx, yy, this.token.layer, this.token.depth, tbGame.getTokenTags(this.token));
	}
	this.effectCreated += numb;
	// Remove when finished
	if (this.effectCreated >= this.effectNumber) { this.effectType = null; }
};


// Update the particle
tbParticle.Particle.prototype.update = function(timePassed) {
	"use strict";
	var dir;
	// Update the life time
	if (this.delay > 0) {
		this.delay -= timePassed;
		if (this.delay < 0) { this.time = -this.delay; }
	} else {
		this.time += timePassed;
	}
	if (this.delay > 0) { return; }
	if (this.time >= this.lifeTime) {
		this.createEmit();
		this.createEffect();
		this.alive = false;
		return;
	}
	if (this.stepBeginFunction !== null) { this.stepBeginFunction(this, this.time, this.lifeTime); }
	// Settings related to the motion
	this.previousX = this.currentX;
	this.previousY = this.currentY;
	tbParticle.updateInterpolationValue(this.positionX, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.positionY, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.speed, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.direction, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.forceDirection, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.forceMagnitude, this.time, this.lifeTime);
	this.forceX += timePassed * tbMath.projectionX(this.forceDirection.value, this.forceMagnitude.value);
	this.forceY += timePassed * tbMath.projectionY(this.forceDirection.value, this.forceMagnitude.value);
	this.moveX += timePassed * (tbMath.projectionX(this.direction.value, this.speed.value) + this.forceX);
	this.moveY += timePassed * (tbMath.projectionY(this.direction.value, this.speed.value) + this.forceY);
	this.currentX = this.x + this.positionX.value + this.moveX;
	this.currentY = this.y + this.positionY.value + this.moveY;
	// Settings related to the appearance
	tbParticle.updateInterpolationValue(this.orientation, this.time, this.lifeTime);
	if (this.autoOrientation) {
		dir = tbMath.direction(this.previousX, this.previousY, this.currentX, this.currentY);
		this.orientation.value = dir;
	}
	tbParticle.updateInterpolationValue(this.size, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.scaleX, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.scaleY, this.time, this.lifeTime);
	// Settings related to the image or shape
	tbParticle.updateInterpolationValue(this.colorRed, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.colorGreen, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.colorBlue, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.colorHue, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.colorSaturation, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.colorLight, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.subImage, this.time, this.lifeTime);
	tbParticle.updateInterpolationValue(this.alpha, this.time, this.lifeTime);
	if (this.autoSubImage) {
		dir = tbMath.direction(this.previousX, this.previousY, this.currentX, this.currentY);
		this.subImage.value = this.image.number * this.direction / 360;
	}
	// Create secondary particles
	this.createEmit();
	this.createEffect();
	// Call step end function
	if (this.stepEndFunction !== null) { this.stepEndFunction(this, this.time, this.lifeTime, timePassed); }
};

// Draw the particle
tbParticle.Particle.prototype.draw = function() {
	"use strict";
	var xx = this.currentX,
		yy = this.currentY,
		sub = this.subImage.value,
		scaleX = this.size.value * this.scaleX.value,
		scaleY = this.size.value * this.scaleX.value,
		orient = this.orientation.value,
		alpha = this.alpha.value,
		color = "white",
		context = tbGraphics.context,
		img = this.image;
	// See if anything must be drawn
	if (!this.alive) { return; }
	if (this.delay > 0) { return; }
	if (Math.abs(scaleX) < 0.0001 || Math.abs(scaleY) < 0.0001) { return; }
	// Follow the token, if required
	if (this.followEffect) {
		xx += this.token.x; yy += this.token.y;
	}
	// Set the color
	if (this.colorMode === "hsl") {
		color = "hsl(" + Math.round(this.colorHue.value).toString() + "," +
					Math.round(this.colorSaturation.value).toString() + "%,"  +
					Math.round(this.colorLight.value).toString() + "%)";
	} else {
		color = "rgb(" + Math.round(this.colorRed.value).toString() + "," +
					Math.round(this.colorGreen.value).toString() + ","  +
					Math.round(this.colorBlue.value).toString() + ")";
	}
	// Set the transformation and alpha
	context.save();
	context.translate(xx, yy);
	context.rotate(-orient * Math.PI/180);
	context.scale(scaleX, scaleY);
	if (this.additive) { context.globalCompositeOperation = "lighter"; }
	// Now draw it
	switch (this.shape) {
		case "function":
			if (this.drawFunction !== undefined && this.drawFunction !== null) {
				this.drawFunction(this, color, alpha);
			}
			break;
		case "rectangle":
			context.globalAlpha = alpha;
			tbGraphics.drawRectangle(-0.5, -0.5, 1, 1, color);
			break;
		case "disk":
			context.globalAlpha = alpha;
			tbGraphics.drawDisk(0, 0, 0.5, color);
			break;
		default:
			context.globalAlpha = alpha;
			if (this.image !== undefined && this.image !== null) {
				if (this.imageColorMode === "color" || this.imageColorMode === "merge") {
					this.surf.clearColor(color);
					this.surf.setTarget();
					this.surf.context.globalCompositeOperation = "destination-in";
					if (this.image instanceof tbImage) {
						this.image.drawSimple(0, this.image.xOrigin, this.image.yOrigin);
					} else {
						this.image.drawSimple(0, 0);
					}
					if (this.imageColorMode === "merge") {
						this.surf.context.globalCompositeOperation = "lighter";
						if (this.image instanceof tbImage) {
							this.image.drawSimple(0, this.image.xOrigin, this.image.yOrigin);
						} else {
							this.image.drawSimple(0, 0);
						}
					}
					this.surf.resetTarget();
					img = this.surf;
				}
				if (img instanceof tbImage) {
					if (!this.absoluteSubImage) { sub *= img.number-0.0001; }
					sub = Math.floor(sub);
					if (this.autoSubImage) { orient = 0; }
					img.drawAlpha(sub, 0,0);
				} else {
					img.drawAlpha(-img.width/2, -img.height/2);
				}
			}
			break;
	}
	// Restore the transformation
	context.restore();
};


// ********************************************
// ** Defining particle types
// ********************************************

// The default particle type
tbParticle.defaultParticleType = {
	// Settings related to the image
	shape : "image",
	image : null,
	subImage : 0,
	autoSubImage : false,
	absoluteSubImage : true,
	alpha : 1,
	imageColorMode : "none",
	colorMode : "rgb",
	colorRed : 255,
	colorGreen : 255,
	colorBlue: 255,
	colorHue : 0,
	colorSaturation : 100,
	colorLight : 50,
	additive : false,
	drawFunction : null,
	// Settings related to the life time
	delay : 0,
	lifeTime : 10000000,
	stepBeginFunction : null,
	stepEndFunction : null,
	//Settings related to secondary particles
	emitType : null,
	emitAtEnd : true,
	emitAmount : 1,
	emitNumber : 1,
	emitDelay : 10000000,
	emitInterval : 0,
	effectType : null,
	effectAtEnd : true,
	effectAmount : 1,
	effectNumber : 1,
	effectDelay : 10000000,
	effectInterval : 0,
	// Settings related to the position
	positionX : 0,
	positionY : 0,
	speed : 0,
	direction : 0,
	forceDirection : 0,
	forceMagnitude : 0,
	followEffect : false,
	// Settings related to the transformation
	autoOrientation : false,
	orientation : 0,
	size : 1,
	scaleX : 1,
	scaleY : 1
};

// Return a clone of a particle type; if properties is defined these are put in the particle type
tbParticle.cloneParticleType = function (pType, properties) {
	"use strict";
	var attr, newType = {};
	for (attr in pType) {
		if (pType.hasOwnProperty(attr)) {
			newType[attr] = tbParticle.cloneInterpolationValue(pType[attr]);
		}
	}
	if (properties !== undefined) {
		tbParticle.changeParticleType(newType, properties);
	}
	return newType;
};

// Set a number of properties in the particle type
tbParticle.changeParticleType = function (pType, properties) {
	"use strict";
	var attr;
	for (attr in properties) {
		if (properties.hasOwnProperty(attr)) {
			pType[attr] = tbParticle.cloneInterpolationValue(properties[attr]);
		}
	}
	return pType;
};

// Return a default particle type; if properties is defined these are put in the particle type
tbParticle.createParticleType = function (properties) {
	"use strict";
	return tbParticle.cloneParticleType(tbParticle.defaultParticleType, properties);
};


// ********************************************
// ** Defining particle effects
// ********************************************

tbParticle.createParticleEffect = function() {
	"use strict";
	return { emitters: [] };
};

// Add a particle emitter for a particular type during a particular time, after a delay
tbParticle.addEmitter = function(effect, pType, x, y, number, delay, interval, properties ) {
	"use strict";
	var newType = tbParticle.cloneParticleType(pType);
	if (properties !== undefined) { tbParticle.changeParticleType(newType, properties); }
	effect.emitters.push({ pType: newType, x: x, y: y, number: number, delay: delay, interval: interval });
};

// Add a single particle a particular type, after a delay
tbParticle.addSingle = function(effect, pType, x, y, delay, properties ) {
	"use strict";
	var newType = tbParticle.cloneParticleType(pType);
	if (properties !== undefined) { tbParticle.changeParticleType(newType, properties); }
	effect.emitters.push({ pType: newType, x: x, y: y, number: 1, delay: delay, interval: 0 });
};

// Add multiple particles of a particular type, after a delay
tbParticle.addMultiple = function(effect, pType, x, y, number, delay, properties ) {
	"use strict";
	var newType = tbParticle.cloneParticleType(pType);
	if (properties !== undefined) { tbParticle.changeParticleType(newType, properties); }
	effect.emitters.push({ pType: newType, x: x, y: y, number: number, delay: delay, interval: 0 });
};

// Show a particle effect. Returns the corresponding token.
tbParticle.showEffect = function(effect, x, y, layer, depth, tags) {
	"use strict";
	return new tbParticle.ParticleToken(effect, x, y, layer, depth, tags);
};


// ********************************************
// ** Defining particle tokens (used internally)
// ********************************************

// Constructor for a particle token
tbParticle.ParticleToken = function(pEffect, x, y, layer, depth, tags) {
	"use strict";
	var i, emitter;
	// Token properties used by the engine
	if (layer === undefined) { layer = 0; }
	if (depth === undefined) { depth = 0; }
	this.layer = layer;
	this.depth = depth;
	this.visible = true;
	this.active = true;
	this.group = 0;
	// Own properties
	this.x = x;
	this.y = y;
	this.effect = pEffect;
	this.lifeTime = 0;
	// Create the emitters
	this.emitters = [];
	for (i = 0; i < this.effect.emitters.length; i += 1) {
		emitter = this.effect.emitters[i];
		this.emitters.push({
			pType: emitter.pType,
			x: emitter.x, y: emitter.y,
			number: tbParticle.computeDistributionValue(emitter.number),
			delay: tbParticle.computeDistributionValue(emitter.delay),
			interval: emitter.interval, created: 0, next: 0
		});
	}
	// Create the particles
	this.particles = [];
	// Add it to the game engine
	tbGame.addToken(this);
	if (tags !== undefined) { tbGame.addTokenTags(this, tags); }
};

tbParticle.ParticleToken.prototype.removeParticles = function() {
// Remove all particles from the system
	"use strict";
	var i;
	for (i = 0; i < this.particles.length; i += 1) {
		tbParticle.particlePool.push(this.particles[i]);
	}
	this.particles.length = 0;
};

tbParticle.ParticleToken.prototype.particleCount = function() {
// Returns the number of particles
	"use strict";
	return this.particles.length;
};

// Handles the step event, updating all particles and creating new ones
tbParticle.ParticleToken.prototype.handleEndStep = function(timePassed) {
	"use strict";
	var i, j, n, t, numb, particle, emitter, pType;
	this.lifeTime += timePassed/1000;
	// Update the current particles, also removing dead ones
	n = 0;
	for (i = 0; i < this.particles.length; i += 1) {
		particle = this.particles[i];
		particle.update(timePassed/1000);
		if (particle.alive) {
			this.particles[n] = particle;
			n += 1;
		} else {
			tbParticle.particlePool.push(particle);
		}
	}
	this.particles.length = n;
	// Create new particles based on the emitters, and remove emitters that are finished
	n = 0;
	for (i = 0; i < this.emitters.length; i += 1) {
		emitter = this.emitters[i];
		t = this.lifeTime - emitter.delay;
		numb = 0;
		// Determine the number of particles to create
		while (t >= emitter.next && numb < emitter.number - emitter.created) { numb += 1; emitter.next += tbParticle.computeDistributionValue(emitter.interval); }
		// Now create the particles
		for (j = 0; j < numb; j += 1) {
			pType = tbParticle.computeDistributionValue(emitter.pType);
			if (tbParticle.particlePool.length > 0) {
				particle = tbParticle.particlePool.pop();
			} else {
				particle = new tbParticle.Particle();
			}
			if (pType.followEffect) {
				particle.init(this, pType, tbParticle.computeDistributionValue(emitter.x), tbParticle.computeDistributionValue(emitter.y));
			} else {
				particle.init(this, pType, this.x + tbParticle.computeDistributionValue(emitter.x), this.y + tbParticle.computeDistributionValue(emitter.y));
			}
			this.particles.push(particle);
		}
		emitter.created += numb;
		// Keep it when not finished
		if (emitter.created < emitter.number) {
			this.emitters[n] = emitter;
			n += 1;
		}
	}
	this.emitters.length = n;
	// remove the token when done
	if (this.emitters.length === 0 && this.particles.length === 0) { tbGame.removeToken(this); }
};

// Handles the draw event, drawing all particles
tbParticle.ParticleToken.prototype.handleDraw = function() {
	"use strict";
	var i;
	for (i = 0; i < this.particles.length; i += 1) {
		if (this.particles[i] !== undefined) {
			this.particles[i].draw();
		}
	}
};
/////////////////////////////////////////////////////////////
// tbAnimation: Contains objects and methods useful for animations
//
// Currently it contains a collection of easy functions and 
// the tbAnimator class.
//
// Copyright 2013-2015 CoolGames
// Last Changed: 17-11-2015
/////////////////////////////////////////////////////////////


// *******************************
// Ease Functions
// *******************************

var tbEase = {

//
// Basic ease functions
//

	easeConstant: function (time, begin, change, duration) {
		"use strict";
		return begin;
	},
	
	easeLinear: function (time, begin, change, duration) {
		"use strict";
		var val = time/duration;
		return begin + change * val;
	},
	
	easeInPower: function (time, begin, change, duration, power) {
		"use strict";
		var val;
		if (power === undefined) { power = 3; }
		val = Math.pow(time/duration, power);
		return begin + change * val;
	},
	
	easeOutPower: function (time, begin, change, duration, power) {
		"use strict";
		var val = tbEase.easeInPower(duration-time, 1, -1, duration, power);
		return begin + change * val;
	},
	
	easeInOutPower: function (time, begin, change, duration, power) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInPower(time, 0, 0.5, duration/2, power);
		} else {
			val = tbEase.easeInPower(duration-time, 1, -0.5, duration/2, power);
		}
		return begin + change * val;
	},
	
	easeOutInPower: function (time, begin, change, duration, power) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInPower(duration/2-time, 0.5, -0.5, duration/2, power);
		} else {
			val = tbEase.easeInPower(time-duration/2, 0.5, 0.5, duration/2, power);
		}
		return begin + change * val;
	},
	
	easeInQuadratic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInPower(time, begin, change, duration, 2);
	},
	
	easeOutQuadratic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutPower(time, begin, change, duration, 2);
	},
	
	easeInOutQuadratic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInOutPower(time, begin, change, duration, 2);
	},
	
	easeOutInQuadratic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutInPower(time, begin, change, duration, 2);
	},
	
	easeInCubic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInPower(time, begin, change, duration, 3);
	},
	
	easeOutCubic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutPower(time, begin, change, duration, 3);
	},
	
	easeInOutCubic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInOutPower(time, begin, change, duration, 3);
	},
	
	easeOutInCubic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutInPower(time, begin, change, duration, 3);
	},
	
	easeInQuartic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInPower(time, begin, change, duration, 4);
	},
	
	easeOutQuartic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutPower(time, begin, change, duration, 4);
	},
	
	easeInOutQuartic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInOutPower(time, begin, change, duration, 4);
	},
	
	easeOutInQuartic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutInPower(time, begin, change, duration, 4);
	},
	
	easeInQuintic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInPower(time, begin, change, duration, 5);
	},
	
	easeOutQuintic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutPower(time, begin, change, duration, 5);
	},
	
	easeInOutQuintic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeInOutPower(time, begin, change, duration, 5);
	},
	
	easeOutInQuintic: function (time, begin, change, duration) {
		"use strict";
		return tbEase.easeOutInPower(time, begin, change, duration, 5);
	},
	
	easeInExponential: function (time, begin, change, duration, power) {
		"use strict";
		var val, base;
		if (power === undefined) { power = 8; }
		base = Math.pow(2, -power);			// The exponential function will not cover the whole range from 0-1 so we must correct it
		val = (Math.pow(2, power * time/duration - power) - base) / (1-base);
		return begin + change * val;
	},
	
	easeOutExponential: function (time, begin, change, duration, power) {
		"use strict";
		var val = tbEase.easeInExponential(duration-time, 1, -1, duration, power);
		return begin + change * val;
	},
	
	easeInOutExponential: function (time, begin, change, duration, power) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInExponential(time, 0, 0.5, duration/2, power);
		} else {
			val = tbEase.easeInExponential(duration-time, 1, -0.5, duration/2, power);
		}
		return begin + change * val;
	},
	
	easeOutInExponential: function (time, begin, change, duration, power) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInExponential(duration/2-time, 0.5, -0.5, duration/2, power);
		} else {
			val = tbEase.easeInExponential(time-duration/2, 0.5, 0.5, duration/2, power);
		}
		return begin + change * val;
	},
	
	easeInSine: function (time, begin, change, duration) {
		"use strict";
		var val = 1 - Math.cos(time/duration * Math.PI/2);
		return begin + change * val;
	},
	
	easeOutSine: function (time, begin, change, duration) {
		"use strict";
		var val = tbEase.easeInSine(duration-time, 1, -1, duration);
		return begin + change * val;
	},
	
	easeInOutSine: function (time, begin, change, duration) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInSine(time, 0, 0.5, duration/2);
		} else {
			val = tbEase.easeInSine(duration-time, 1, -0.5, duration/2);
		}
		return begin + change * val;
	},
	
	easeOutInSine: function (time, begin, change, duration) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInSine(duration/2-time, 0.5, -0.5, duration/2);
		} else {
			val = tbEase.easeInSine(time-duration/2, 0.5, 0.5, duration/2);
		}
		return begin + change * val;
	},
	
	easeInCircular: function (time, begin, change, duration) {
		"use strict";
		var val = 1 - Math.sqrt(1 - Math.pow(time/duration, 2));
		return begin + change * val;
	},
	
	easeOutCircular: function (time, begin, change, duration) {
		"use strict";
		var val = tbEase.easeInCircular(duration-time, 1, -1, duration);
		return begin + change * val;
	},
	
	easeInOutCircular: function (time, begin, change, duration) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInCircular(time, 0, 0.5, duration/2);
		} else {
			val = tbEase.easeInCircular(duration-time, 1, -0.5, duration/2);
		}
		return begin + change * val;
	},
	
	easeOutInCircular: function (time, begin, change, duration) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInCircular(duration/2-time, 0.5, -0.5, duration/2);
		} else {
			val = tbEase.easeInCircular(time-duration/2, 0.5, 0.5, duration/2);
		}
		return begin + change * val;
	},
	
	easeInElastic: function (time, begin, change, duration, oscillations, stiffness) {
		"use strict";
		var val;
		if (oscillations === undefined) { oscillations = 3; }
		if (stiffness === undefined) { stiffness = 8; }
		val = Math.sin((1-time/duration)*2*Math.PI*oscillations + Math.PI/2);			// Create the correct sign
		val *= tbEase.easeInExponential(time, 0, 1, duration, stiffness);				// Multiply with an exponential ease in
		return begin + change * val;
	},

	easeOutElastic: function (time, begin, change, duration, oscillations, stiffness) {
		"use strict";
		var val = tbEase.easeInElastic(duration-time, 1, -1, duration, oscillations, stiffness);
		return begin + change * val;
	},
	
	easeInOutElastic: function (time, begin, change, duration, oscillations, stiffness) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInElastic(time, 0, 0.5, duration/2, oscillations, stiffness);
		} else {
			val = tbEase.easeInElastic(duration-time, 1, -0.5, duration/2, oscillations, stiffness);
		}
		return begin + change * val;
	},
	
	easeOutInElastic: function (time, begin, change, duration, oscillations, stiffness) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInElastic(duration/2-time, 0.5, -0.5, duration/2, oscillations, stiffness);
		} else {
			val = tbEase.easeInElastic(time-duration/2, 0.5, 0.5, duration/2, oscillations, stiffness);
		}
		return begin + change * val;
	},
	
	easeInOvershoot: function (time, begin, change, duration, overshoot) {
		"use strict";
		var val;
		if (overshoot === undefined) { overshoot = 1.70158; }
		val = (1+overshoot) * Math.pow(time/duration, 3) - overshoot * Math.pow(time/duration, 2);
		return begin + change * val;

	},
	
	easeOutOvershoot: function (time, begin, change, duration, overshoot) {
		"use strict";
		var val = tbEase.easeInOvershoot(duration-time, 1, -1, duration, overshoot);
		return begin + change * val;
	},
	
	easeInOutOvershoot: function (time, begin, change, duration, overshoot) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInOvershoot(time, 0, 0.5, duration/2, overshoot);
		} else {
			val = tbEase.easeInOvershoot(duration-time, 1, -0.5, duration/2, overshoot);
		}
		return begin + change * val;
	},
	
	easeOutInOvershoot: function (time, begin, change, duration, overshoot) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInOvershoot(duration/2-time, 0.5, -0.5, duration/2, overshoot);
		} else {
			val = tbEase.easeInOvershoot(time-duration/2, 0.5, 0.5, duration/2, overshoot);
		}
		return begin + change * val;
	},

	easeInBounce: function (time, begin, change, duration, number, factor) {
		"use strict";
		var bounces, i, w, width, height, val;
		// Correct the arguments
		if (number === undefined) { number = 4; }
		if (factor === undefined) { factor = 2; } else { factor = Math.sqrt(factor); }
		// Create the list of bounces
		bounces = [1];
		width = 1;
		for (i = 1; i < number; i += 1) {
			bounces.push(bounces[i-1]*factor);
			width += bounces[i];
		}
		// Compute total size
		width -= bounces[number-1]/2;		// We don't use the last half bounce
		height = Math.pow(bounces[number-1],2);
		time = time/duration * width;
		// Find the correct bounce
		w = 0;
		for (i = 0; i < number; i += 1) {
			if (time > bounces[i]) { time -= bounces[i];} else { w = bounces[i]; break; }
		}
		// Now compute the value
		val = -4 * Math.pow(time-w/2,2) + w * w;
		return begin + change * val / height;
	},

	easeOutBounce: function (time, begin, change, duration, number, factor) {
		"use strict";
		var val = tbEase.easeInBounce(duration-time, 1, -1, duration, number, factor);
		return begin + change * val;
	},
	
	easeInOutBounce: function (time, begin, change, duration, number, factor) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInBounce(time, 0, 0.5, duration/2, number, factor);
		} else {
			val = tbEase.easeInBounce(duration-time, 1, -0.5, duration/2, number, factor);
		}
		return begin + change * val;
	},
	
	easeOutInBounce: function (time, begin, change, duration, number, factor) {
		"use strict";
		var val;
		if (time < duration/2) {
			val = tbEase.easeInBounce(duration/2-time, 0.5, -0.5, duration/2, number, factor);
		} else {
			val = tbEase.easeInBounce(time-duration/2, 0.5, 0.5, duration/2, number, factor);
		}
		return begin + change * val;
	},
	
//
// Creating repeated ease functions
//

	repeatConstant : 0,
	repeatRepeat : 1,
	repeatMirror : 2,
	repeatFlip : 3,
	repeatContinuous : 4,
	repeatContinuousMirror : 5,

	easeCreateRepeat: function (func, repeat) {
		"use strict";
		switch (repeat) {
			case tbEase.repeatConstant:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					if (time < 0) { return begin; }
					if (time > duration) { return begin + change; }
					return func(time, begin, change, duration, arg0, arg1, arg2);
				};
			case tbEase.repeatRepeat:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					return func(time - Math.floor(time/duration)*duration, begin, change, duration, arg0, arg1, arg2);
				};
			case tbEase.repeatMirror:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					var val;
					if (Math.floor(time/duration) % 2 === 0) {
						val = func(time - Math.floor(time/duration)*duration, 0, 1, duration, arg0, arg1, arg2);
					} else {
						val = func(duration - time + Math.floor(time/duration)*duration, 0, 1, duration, arg0, arg1, arg2);
					}
					return begin + change * val;
				};
			case tbEase.repeatFlip:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					var val = func(time - Math.floor(time/duration)*duration, 0, 1, duration, arg0, arg1, arg2);
					if (Math.floor(time/duration) % 2 !== 0) {val = 1-val; }
					return begin + change * val;
				};
			case tbEase.repeatContinuous:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					var offset = Math.floor(time/duration),
						val = func(time - Math.floor(time/duration)*duration, 0, 1, duration, arg0, arg1, arg2);
					return begin + change * (offset + val);
				};
			case tbEase.repeatContinuousMirror:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					var val, offset = Math.floor(time/duration);
					if (Math.floor(time/duration) % 2 === 0) {
						val = func(time - Math.floor(time/duration)*duration, 0, 1, duration, arg0, arg1, arg2);
					} else {
						val = func(duration - time + Math.floor(time/duration)*duration, 1, -1, duration, arg0, arg1, arg2);
					}
					return begin + change * (offset + val);
				};
			default:
				return function (time, begin, change, duration, arg0, arg1, arg2) {
					return func(time, begin, change, duration, arg0, arg1, arg2);
				};
		}
	},

//
// Creating ease functions
//

	_easeCreate: function (kind, coef5, coef4, coef3, coef2, coef1, normalize) {
		"use strict";
		var tot;
		if (coef1 === undefined) { coef1 = 0; }
		if (coef2 === undefined) { coef2 = 0; }
		if (coef3 === undefined) { coef3 = 0; }
		if (coef4 === undefined) { coef4 = 0; }
		if (coef5 === undefined) { coef5 = 0; }
		if (normalize === undefined) { normalize = true; }
		tot = coef1 + coef2 + coef3 + coef4 + coef5;
		if (tot === 0) { tot = 1; }
		if (!normalize) { tot = 1; }
		
		switch (kind) {
			case 0:	return function(time, begin, change, duration) {
						var t = time/duration,
							val = coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t; 
						return begin + change * val/tot;
					};
			case 1:return function(time, begin, change, duration) {
						var t = (duration - time)/duration,
							val = 1 - (coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t); 
						return begin + change * val/tot;
					};
			case 2:	return function(time, begin, change, duration) {
						var t, val;
						if (time < duration/2) {
							t = time/(duration/2);
							val = 0.5 * (coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t); 
						} else {
							t = (duration-time)/(duration/2);
							val = 1 - 0.5 * (coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t); 
						
						}
						return begin + change * val/tot;
					};
			case 3:	return function(time, begin, change, duration) {
						var t, val;
						if (time < duration/2) {
							t = 1-time/(duration/2);
							val = 0.5 - 0.5 * (coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t); 
						} else {
							t = time/(duration/2) - 1;
							val = 0.5 + 0.5 * (coef5 * Math.pow(t, 5) + coef4 * Math.pow(t, 4) + coef3 * Math.pow(t, 3) + coef2 * Math.pow(t, 2) + coef1 * t); 
						
						}
						return begin + change * val/tot;
					};
		}
	},
	
	easeCreateIn: function (coef5, coef4, coef3, coef2, coef1, normalize) {
		"use strict";
		return tbEase._easeCreate(0, coef5, coef4, coef3, coef2, coef1, normalize);
	},
	
	easeCreateOut: function (coef5, coef4, coef3, coef2, coef1, normalize) {
		"use strict";
		return tbEase._easeCreate(1, coef5, coef4, coef3, coef2, coef1, normalize);
	},
	
	easeCreateInOut: function (coef5, coef4, coef3, coef2, coef1, normalize) {
		"use strict";
		return tbEase._easeCreate(2, coef5, coef4, coef3, coef2, coef1, normalize);
	},
	
	easeCreateOutIn: function (coef5, coef4, coef3, coef2, coef1, normalize) {
		"use strict";
		return tbEase._easeCreate(3, coef5, coef4, coef3, coef2, coef1, normalize);
	},

//
// Combined ease functions
//

	easeCreateCombined: function (funcs, flips, lengths) {
		"use strict";
		var j, ww = 0, hh, maxHeight = 1, startHeight = [0], startWidth = [0];
		// Make sure the flips array is correct
		if (flips === undefined) { flips = []; }
		while (flips.length < funcs.length) { flips.push(false); }
		// Make sure the lengths array is correct
		if (lengths === undefined) { lengths = []; }
		while (lengths.length < funcs.length) { lengths.push(1/funcs.length); }
		for (j = 0; j < funcs.length; j += 1) { ww += lengths[j]; }
		for (j = 0; j < funcs.length; j += 1) { lengths[j] /= ww; }
		// Compute the start width, height and maximal height
		for (j = 0; j < funcs.length; j += 1) {
			startWidth.push(startWidth[j] + lengths[j]);
			if (funcs[j] === tbEase.easeConstant) { hh = 0; }
			else if (flips[j]) { hh = -1; } 
			else { hh = 1; }
			startHeight.push(startHeight[j] + hh);
			maxHeight = Math.max(maxHeight, startHeight[j+1]);
		}
		
		return function(time, begin, change, duration, arg0, arg1, arg2) {
			var ind, i, val;
			// Compute the function index
			ind = funcs.length-1;
			for (i = 0; i < funcs.length; i += 1) {
				if (time/duration <= startWidth[i+1]) { ind = i; break; }
			}
			// Now compute the value
			val = funcs[ind](time/duration - startWidth[ind], 0, 1, lengths[ind], arg0, arg1, arg2);
			if (flips[ind]) { val = -val; }
			return begin + (startHeight[ind] + val) * change / maxHeight; 
		};
	}

};


/////////////////////////////////////////////////////////////
// tbAnimator: The engine part dealing with animating values over
// time, which can be used for e.g. creating animations.
/////////////////////////////////////////////////////////////

// The constructor for the tbAnimator object
/**
 * @constructor
 */
var tbAnimator = function(target, tracks, events) {
	"use strict";
	this.paused = 0;				// How often paused
	this.currentTime = 0;			// Current time in milliseconds (or whatever the user uses)
	this.speed = 1;					// The speed (multiplication factor)
	this.looping = false;			// Whether to loop the global animator
	this.target = target;			// The target object (often a token)
	this.duration = 0;				// The total duration of the animator
	this.tracks = [];				// The tracks
	this.events = [];				// The events
	if (tracks !== undefined) { this.addTracks(tracks); }
	if (events !== undefined) { this.addTEvents(events); }
};


// *****************************
// Dealing with tracks
// *****************************

// Add a collection of tracks
tbAnimator.prototype.addTracks = function(tracks) {
	"use strict";
	var i;
	if (tracks !== undefined) {
		tracks = [].concat(tracks);
		for (i = 0; i < tracks.length; i += 1) {
			this.addTrack(tracks[i].kind, tracks[i].target, tracks[i].func, tracks[i].arg, tracks[i].keys);
		}
	}
};

// Add a track; returns the id
tbAnimator.prototype.addTrack = function(kind, target, func, arg, keys) {
	"use strict";
	var result;
	this.tracks.push({kind: kind, target: target, func: func, arg: arg,
		keys: [], looping: false, rounding: "none", onlyChanged: true, lastValue: undefined});
	result = this.tracks[this.tracks.length-1];
	if (keys !== undefined) { this.addKeys(result, keys); }
	return result;	
};

// Remove a track
tbAnimator.prototype.removeTrack = function(track) {
	"use strict";
	var ind = this.tracks.indexOf(track);
	if (ind < 0) { return; }
	this.tracks.splice(ind);
};

// Remove all tracks from the animator
tbAnimator.prototype.removeAllTracks = function() {
	"use strict";
	this.tracks = [];
};

// Set whether a track loops
tbAnimator.prototype.setTrackLooping = function(track, looping) {
	"use strict";
	track.looping = looping;
};

// Set the rounding for a track
tbAnimator.prototype.setTrackRounding = function(track, rounding) {
	"use strict";
	track.rounding = rounding;
};

// Set whether a track loops
tbAnimator.prototype.setTrackOnlyChanged = function(track, onlyChanged) {
	"use strict";
	track.onlyChanged = onlyChanged;
};

// Returns the duration of the track
tbAnimator.prototype.getTrackDuration = function(track) {
	"use strict";
	var len;
	len = track.keys.length;
	if (len === 0) { return 0; }
	return track.keys[len-1].time;
};

// Computes the total duration of the animator
tbAnimator.prototype._computeDuration = function() {
	"use strict";
	var i;
	this.duration = 0;
	for (i = 0; i < this.tracks.length; i += 1) {
		this.duration = Math.max(this.duration, this.getTrackDuration(this.tracks[i]));
	}
	this.duration = Math.max(this.duration, this.getEventsDuration());
};

// Returns the value of the track at the given time
tbAnimator.prototype.getTrackValue = function(track, time) {
	"use strict";
	var theValue, len, ind, key1, key2;
	len = track.keys.length;
	if (len === 0) { return 0; }
	// Deal with looping
	if (track.looping) {
		time = time % track.keys[len-1].time;
		if (time < 0) { time += track.keys[len-1].time; }
	}
	// Find the value
	if (time <= track.keys[0].time) {
		theValue = track.keys[0].value;
	} else if (time >= track.keys[len-1].time) {
		theValue = track.keys[len-1].value;
	} else {
		ind = 0;
		while (track.keys[ind].time < time) { ind += 1; }
		key1 = track.keys[ind-1];
		key2 = track.keys[ind];
		theValue = key1.ease(time-key1.time, key1.value, key2.value-key1.value, key2.time-key1.time);
	}
	// Deal with rounding
	switch (track.rounding) {
		case "round": theValue = Math.round(theValue); break;
		case "floor": theValue = Math.floor(theValue); break;
		case "ceil": theValue = Math.ceil(theValue); break;
	}
	// Ready
	return theValue;
};

// Apply the value in the track
tbAnimator.prototype._applyValue = function(track, time) {
	"use strict";
	var theValue, target;
	if (track.keys.length === 0) { return; }
	theValue = this.getTrackValue(track, time);
	// Check whether changed
	if (theValue === track.lastValue && track.onlyChanged) {
		return;
	}		
	// Apply the value
	target = track.target === undefined ? this.target : track.target;
	if (target === undefined && track.kind !== "function") { return; }
	switch (track.kind) {
		case "image": target.image = theValue; break;
		case "subImage": target.subImage = theValue; break;
		case "alpha": target.alpha = theValue; break;
		case "xScale": target.xScale = theValue; break;
		case "yScale": target.yScale = theValue; break;
		case "scale": target.scale = theValue; break;
		case "angle": target.angle = theValue; break;
		case "x": target.x = theValue; break;
		case "y": target.y = theValue; break;
		case "speed": target.speed = theValue; break;
		case "function":
			if (target === undefined) {
				track.func(theValue, track.arg);
			} else {
				track.func.call(target, theValue, track.arg);
			}
			break;
	}
};


// *****************************
// Dealing with keys
// *****************************

// Add a collection of keys to a track, time is now an absolute property
tbAnimator.prototype.addKeys = function(track, keys) {
	"use strict";
	var i;
	if (keys !== undefined) {
		keys = [].concat(keys);
		for (i = 0; i < keys.length; i += 1) {
			this.addKey(track, keys[i].time, keys[i].value, keys[i].ease, keys[i].arg);
		}
	}
};

// Add a key to a track, time is now an absolute property
tbAnimator.prototype.addKey = function(track, time, value, ease, arg) {
	"use strict";
	var result;
	if (ease === undefined) { ease = tbEase.easeLinear; }
	track.keys.push( {time: time, value: value, ease: ease, arg: arg} );
	result = track.keys[track.keys.length-1];
	this._sortKeys(track);
	this._computeDuration();
	return result;
};

// Append a collection of keys to a track, time is now a relative property
tbAnimator.prototype.appendKeys = function(track, keys) {
	"use strict";
	var i;
	if (keys !== undefined) {
		keys = [].concat(keys);
		for (i = 0; i < keys.length; i += 1) {
			this.appendKey(track, keys[i].time, keys[i].value, keys[i].ease, keys[i].arg);
		}
	}
};

// Append a key to a track, time is now a relative property
tbAnimator.prototype.appendKey = function(track, time, value, ease, arg) {
	"use strict";
	var result, lastTime;
	lastTime = track.keys.length === 0 ? 0 : track.keys[track.keys.length-1].time;
	if (ease === undefined) { ease = tbEase.easeLinear; }
	track.keys.push( {time: lastTime + time, value: value, ease: ease, arg: arg} );
	result = track.keys[track.keys.length-1];
	this._computeDuration();
	return result;
};

// Remove a key
tbAnimator.prototype.removeKey = function(track, key) {
	"use strict";
	var ind = track.indexOf(key);
	if (ind < 0) { return; }
	track.splice(ind);
	this._computeDuration();
};

// Remove all keys (in an interval) from the track
tbAnimator.prototype.removeKeys = function(track, startTime, endTime) {
	"use strict";
	var i;
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = track.keys.length - 1; i >= 0; i -= 1) {
		if (track.keys[i].time >= startTime && track.keys[i].time <= endTime) {
			track.keys.splice(i);
		}
	}
	this._computeDuration();
};

// Remove all keys (in an interval) from all tracks
tbAnimator.prototype.removeAllKeys = function(startTime, endTime) {
	"use strict";
	var i;
	for (i = 0; i < this.tracks.length; i += 1) {
		this.removeKeys(this.tracks[i], startTime, endTime);
	}
};

// Shift all keys (in an interval) in the track by a particular amount of time
tbAnimator.prototype.shiftKeys = function(track, amount, startTime, endTime) {
	"use strict";
	var i;
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = 0; i < track.keys.length; i += 1) {
		if (track.keys[i].time >= startTime && track.keys[i].time <= endTime) {
			track.keys[i].time += amount;
			if (track.keys[i].time < 0) { track.keys[i].time = 0; }
		}
	}
	this._sortKeys(track);
	this._computeDuration();
};

// Shift all keys (in an interval) in all tracks
tbAnimator.prototype.shiftAllKeys = function(amount, startTime, endTime) {
	"use strict";
	var i;
	for (i = 0; i < this.tracks.length; i += 1) {
		this.shiftKeys(this.tracks[i], amount, startTime, endTime);
	}
};

// Stretch all keys (in an interval) in the track by multiplying the time with a factor
tbAnimator.prototype.stretchKeys = function(track, factor, startTime, endTime) {
	"use strict";
	var i;
	if (factor < 0) { return; }
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = 0; i < track.keys.length; i += 1) {
		if (track.keys[i].time >= startTime && track.keys[i].time <= endTime) {
			track.keys[i].time *= factor;
		}
	}
	this._sortKeys(track);
	this._computeDuration();
};

// Stretch all keys (in an interval) in all tracks
tbAnimator.prototype.stretchAllKeys = function(factor, startTime, endTime) {
	"use strict";
	var i;
	for (i = 0; i < this.tracks.length; i += 1) {
		this.stretchKeys(this.tracks[i], factor, startTime, endTime);
	}
};

// Sort the keys in the track by time
tbAnimator.prototype._sortKeys = function(track) {
	"use strict";
	track.keys.sort( function(a, b) { return a.time - b.time; } );
};


// *****************************
// Dealing with events
//		- When an event returns true, is stops the progress of time
// *****************************

// Add a collection of events
tbAnimator.prototype.addEvents = function(events) {
	"use strict";
	var i;
	if (events !== undefined) {
		events = [].concat(events);
		for (i = 0; i < events.length; i += 1) {
			this.addEvent(events[i].target, events[i].time, events[i].callback, events[i].arg);
		}
	}
};

// Add an event; Returns the event object
tbAnimator.prototype.addEvent = function(target, time, callback, arg) {
	"use strict";
	var result;
	this.events.push( {target: target, time: time, callback: callback, arg: arg} ) ;
	result = this.events[this.events.length-1];
	this._sortEvents();
	this._computeDuration();
	return result;
};

// Append a collection of events, time is now a relative property
tbAnimator.prototype.appendEvents = function(events) {
	"use strict";
	var i;
	if (events !== undefined) {
		events = [].concat(events);
		for (i = 0; i < events.length; i += 1) {
			this.appendEvent(events[i].target, events[i].time, events[i].callback, events[i].arg);
		}
	}
};

tbAnimator.prototype.appendEvent = function(target, time, callback, arg) {
	"use strict";
	var result, lastTime;
	lastTime = this.events.length === 0 ? 0 : this.events[this.events.length-1].time;
	this.events.push( {target: target, time: lastTime + time, callback: callback, arg: arg} ) ;
	result = this.events[this.events.length-1];
	this._computeDuration();
	return result;
};

// Remove an event
tbAnimator.prototype.removeEvent = function(event) {
	"use strict";
	var ind = this.events.indexOf(event);
	if (ind >= 0) { this.events.splice(ind); }
	this._computeDuration();
};

// Remove all events (in a time interval) from the animator
tbAnimator.prototype.removeEvents = function(startTime, endTime) {
	"use strict";
	var i;
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = this.events.length - 1; i >= 0; i -= 1) {
		if (this.events[i].time >= startTime && this.events[i].time <= endTime) {
			this.events.splice(i);
		}
	}
	this._computeDuration();
};

// Shift all events (in an interval) by a particular amount of time
tbAnimator.prototype.shiftEvents = function(time, startTime, endTime) {
	"use strict";
	var  i;
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = 0; i < this.events.length; i += 1) {
		if (this.events[i].time >= startTime && this.events[i].time <= endTime) {
			this.events[i].time += time;
		}
	}
	this._sortEvents();
	this._computeDuration();
};

// Stretch all events (in an interval) by multiplying the time with a factor
tbAnimator.prototype.stretchEvents = function(factor, startTime, endTime) {
	"use strict";
	var i;
	if (startTime === undefined) { startTime = -99999999; }
	if (endTime === undefined) { endTime = 99999999; }
	for (i = 0; i < this.events.length; i += 1) {
		if (this.events[i].time >= startTime && this.events[i].time <=endTime) {
			this.events[i].time *= factor;
		}
	}
	this._sortEvents();
	this._computeDuration();
};

// Returns the duration of the events
tbAnimator.prototype.getEventsDuration = function() {
	"use strict";
	var len;
	len = this.events.length;
	if (len === 0) { return 0; }
	return this.events[len-1].time;
};


// Sort the events by time
tbAnimator.prototype._sortEvents = function() {
	"use strict";
	this.events.sort( function(a, b) { return a.time - b.time; } );
};

// Find all events in the time interval (taking speed sign and looping into account)
// Do not report events at the startTime to avoid double execution
tbAnimator.prototype._findEvents = function(startTime, endTime) {
	"use strict";
	var i, result = [];
	if (this.speed > 0) {
		if (endTime >= startTime) {
			for (i = 0; i < this.events.length; i += 1) {
				if (this.events[i].time > startTime && this.events[i].time <= endTime) { 
					result.push(this.events[i]); 
				}
			}
		} else {
			for (i = 0; i < this.events.length; i += 1) {
				if (this.events[i].time > startTime) { 
					result.push(this.events[i]); 
				}
			}
			for (i = 0; i < this.events.length; i += 1) {
				if (this.events[i].time <= endTime) { 
					result.push(this.events[i]); 
				}
			}
		}
	} else {
		if (endTime <= startTime) {
			for (i = this.events.length-1; i >= 0; i -= 1) {
				if (this.events[i].time < startTime && this.events[i].time >= endTime) { 
					result.push(this.events[i]); 
				}
			}
		} else {
			for (i = this.events.length-1; i >= 0; i -= 1) {
				if (this.events[i].time < startTime) { 
					result.push(this.events[i]); 
				}
			}
			for (i = this.events.length-1; i >= 0; i -= 1) {
				if (this.events[i].time >= endTime) { 
					result.push(this.events[i]); 
				}
			}
		}
	}
	return result;
};

// Executes an event; returns whether to break
tbAnimator.prototype._executeEvent = function(event) {
	"use strict";
	var target, result;
	target = event.target === undefined ? this.target : event.target;
	if (target === undefined) {
		result = event.callback(event.arg);
	} else {
		result = event.callback.call(target, event.arg);
	}
	return result;
};


// *****************************
// Pausing and resuming
// *****************************

// Pause the animator (these stack)
tbAnimator.prototype.pause = function() {
	"use strict";
	this.paused += 1;
};

// Resumes the animator
tbAnimator.prototype.resume = function() {
	"use strict";
	this.paused -= 1;
};

// Resumes the animator immediately
tbAnimator.prototype.resumeForced = function() {
	"use strict";
	this.paused = 0;
};

// Returns whether the animator is paused
tbAnimator.prototype.isPaused = function() {
	"use strict";
	return this.paused > 0;
};

// Set the speed
tbAnimator.prototype.setSpeed = function(speed) {
	"use strict";
	this.speed = speed;
};

// Returns the speed
tbAnimator.prototype.getSpeed = function() {
	"use strict";
	return this.speed;
};

// Set the looping
tbAnimator.prototype.setLooping = function(looping) {
	"use strict";
	this.looping = looping;
};

// Returns the looping
tbAnimator.prototype.getLooping = function() {
	"use strict";
	return this.looping;
};

// Returns the total duration
tbAnimator.prototype.getDuration = function() {
	"use strict";
	return this.duration;
};


// *****************************
// Changing time
// *****************************

// Correct the time, taking looping into account
tbAnimator.prototype._correctTime = function(time) {
	"use strict";
	if (this.looping) {
		time = time % this.duration;
		if (time < 0) { time += this.duration; }
	} else {
		time = Math.max(0, Math.min(this.duration, time));
	}
	return time;
};

// Sets the time, changing all values (unless not allowed) but NOT calling events
tbAnimator.prototype.setTime = function(newTime, noChange) {
	"use strict";
	var i;
	this.currentTime = this._correctTime(newTime);
	if (noChange) { return; }
	for (i = 0; i < this.tracks.length; i += 1) {
		this._applyValue(this.tracks[i], this.currentTime);
	}
};

// Update the time, changing all values and calling events and checking breakpoints
tbAnimator.prototype.updateTime = function(timePassed) {
	"use strict";
	var newTime, events, i;
	if (this.paused > 0) { return; }
	// Compute the new time
	newTime = this._correctTime(this.currentTime + this.speed * timePassed);
	// Handle events
	events = this._findEvents(this.currentTime, newTime);
	for (i = 0; i < events.length; i += 1) {
		this.setTime(events[i].time);
		if (this._executeEvent(events[i])) { 
			this.currentTime = events[i].time - 0.0000001 * Math.sign(this.speed);	// To make sure it is tested again the next step
			return;
		}
	}
	// Set the new time
	this.setTime(newTime);
};




/////////////////////////////////////////////////////////////
// tbSpine: The engine part dealing with spine animations
//
// Copyright 2016 CoolGames
// Last Changed: 22-3-2016
/////////////////////////////////////////////////////////////

// Note that we reference the various methods and properties in the spine runtime
// by string, to avoid problems with obfuscation. The spine runtime cannot be obfuscated.

var tbSpine = function(animationJson, namespace) {
// The constructor for the spine object
	"use strict";
	this.animationJson = animationJson;
	this.setNamespace(namespace);
	this.skeleton = null;
	this.skeletonData = null;
	this.state = null;
	this.stateDate = null;
	this._isPaused = 0;
	this.speed = 1;
	this._events = [];
	this._loadSkeleton(this.animationJson);
	this._onComplete = null;
};

// Clones the spine (such that you can use multiple at the same moment)
tbSpine.prototype.clone = function() {
	"use strict";
	return new tbSpine(this.animationJson, this.namespace);
};

// Sets the namespace (such that we can change the visuals without redoing changing the animation)
tbSpine.prototype.setNamespace = function(namespace) {
	"use strict";
	if(namespace === undefined){namespace = window;}
	this.namespace = namespace;
};


// *****************************
// Dealing with animations
// *****************************

// Sets the animation to play
tbSpine.prototype.setAnimation = function(animation, loop) {
	"use strict";
	if(this.state){
		this.state["setAnimationByName"](0, animation, loop);
	}
};

// Add an animation to be played after the current animation
tbSpine.prototype.addAnimation = function(animation, loop, delay) {
	"use strict";
	if(delay === undefined || delay < 0){ delay = 0;}
	if(this.state){
		this.state["addAnimationByName"](0, animation, loop, delay);
	}
};

// Sets transition smoothness of two animations
tbSpine.prototype.mixAnimations = function(firstAnimation, secondAnimation, duration){
	"use strict";
	if(duration === undefined){ duration = 0.2;}
	if(this.stateData){
		this.stateData["setMixByName"](firstAnimation, secondAnimation, duration);
	}
};


// *****************************
// Setting properties
// *****************************

// Sets the speed with which to play (1 is default, must be >= 0)
tbSpine.prototype.setSpeed = function(speed) {
	"use strict";
	if(speed === undefined){ speed = 1; }
	if(speed < 0) { speed = 0; }
	this.speed = speed;
};

// Returns the speed with which to play
tbSpine.prototype.getSpeed = function() {
	"use strict";
	return this.speed;
};


// *****************************
// Dealing with events
// *****************************

// Sets an event to call a particular callback
tbSpine.prototype.setEventCallback = function(event, callback) {
	"use strict";
	var i;
	var self = this;
	// Check whether it already exists
	for(i = self._events.length-1; i >= 0 ; i -= 1){
		if(self._events[i].event === event){
			self._events[i].callback = callback;
			return;
		}
	}
	this._events.push({event: event, callback: callback});
};

// Get the callback for a particular event
// Get the callback for a particular event
tbSpine.prototype.getEventCallback = function(event) {
	"use strict";
	var i, _callback;
	var self = this;
	for(i = self._events.length-1; i >= 0 ; i -= 1){
		if(self._events[i].event === event){
			_callback = self._events[i].callback;
		}
	}
	return _callback;
};

// Clear the event callback
tbSpine.prototype.clearEventCallback = function(event) {
	"use strict";
	var i;
	var self = this;
	for(i = self._events.length-1; i >= 0 ; i -= 1){
		if(self._events[i].event === event){
			self._events.splice(i, 1);
		}
	}
};

// Clear all event callbacks
tbSpine.prototype.clearAllEventCallbacks = function() {
	"use strict";
	this._events = [];
};

// Sets the callback to be called on complete
tbSpine.prototype.setOnComplete = function(callback) {
	"use strict";
	this._onComplete = callback;
};

// Returns the callback on complete
tbSpine.prototype.getOnComplete = function() {
	"use strict";
	return this._onComplete;
};

// Clears the callback on complete
tbSpine.prototype.clearOnComplete = function() {
	"use strict";
	this._onComplete = null;
};

//Initialize spines callback functions
tbSpine.prototype._initEvents = function(){
	"use strict";
	var self = this;
	if(this.state){
		this.state["onEvent"] = function(trackIndex, event){
			var i;
			for(i = 0; i < self._events.length; i += 1){
				if(self._events[i].event === event["data"]["name"]){
					if (self._events[i].callback) {self._events[i].callback();}
				}
			}
		};
		this.state["onComplete"] = function(trackIndex, count){
			if(self._onComplete){self._onComplete();}
		};
	}
};


// *****************************
// Pausing and resuming
// *****************************

// Pauses the animation, that is, no updates to the time are applied. This method stacks, so multiple pauses must be followed by multiple resumes
tbSpine.prototype.pause = function() {
	"use strict";
	this._isPaused += 1;
};

// Resumes the animation. This method stacks, so multiple pauses must be followed by multiple resumes
tbSpine.prototype.resume = function() {
	"use strict";
	this._isPaused -= 1;
};

// Resumes the animation immediately, ignoring the stacking of pauses
tbSpine.prototype.resumeForced = function() {
	"use strict";
	this._isPaused = 0;
};

// Returns whether the animation is paused
tbSpine.prototype.isPaused = function() {
	"use strict";
	return this._isPaused > 0;
};


// *****************************
// Changing time
// *****************************

// Update the time, changing all values and calling events
tbSpine.prototype.updateTime = function(timePassed) {
	"use strict";
	if(this._isPaused > 0){return;}
	this.state["update"]((timePassed / 1000) * this.speed);
	this.state["apply"](this.skeleton);
	this.skeleton["updateWorldTransform"]();
};


// *****************************
// Drawing
// *****************************

// Draw at a position
tbSpine.prototype.drawSimple = function(mX, mY) {
	"use strict";
	this.drawExt(mX, mY, undefined, undefined, undefined , undefined);
};

// Draw at a position with an alpha value
tbSpine.prototype.drawAlpha = function(mX, mY, alpha) {
	"use strict";
	this.drawExt(mX, mY, undefined, undefined, undefined , alpha);
};

// Draw at a position with a transformation and an alpha value
tbSpine.prototype.drawExt = function(mX, mY, xScale, yScale, angle, alpha) {
	"use strict";
	var context = tbGraphics.context,
		drawOrder = this.skeleton["drawOrder"],
		i, slot, attachment, bone, x, y, rotation, img;
	// COrrect the values
	if(mX === undefined){ mX = 0; }
	if(mY === undefined){ mY = 0; }
	if(alpha === undefined){ alpha = 1; }
	if(xScale === undefined){ xScale = 1;}
	if(yScale === undefined){ yScale = 1;}
	if(angle === undefined){angle = 0;}
	if(alpha === undefined){alpha = 1;}
	// Set the global transformation
	context.save();
	context.translate(mX, mY);
	context.scale(xScale,yScale);
	context.rotate(angle);
	// Now draw the skeleton
	for (i = 0; i < drawOrder.length; i += 1) {
		slot = drawOrder[i];
		attachment = slot["attachment"];
		if (!(attachment instanceof spine["RegionAttachment"])) { continue; }
		bone = slot["bone"];
		// Find the position, etc.
		x = bone["worldX"] + attachment["x"] * bone["m00"] + attachment["y"] * bone["m01"];
		y = bone["worldY"] + attachment["x"] * bone["m10"] + attachment["y"] * bone["m11"];
		rotation = -(bone["worldRotation"] + attachment["rotation"]) * Math.PI / 180;
		img = this.namespace[attachment["name"]];
		// Now transfrom and draw
		context.save();
		context.translate(imageScalePercentage * x / 100, imageScalePercentage * y / 100);
		context.globalAlpha = alpha * slot["a"];
		context.scale(attachment["scaleX"], attachment["scaleY"]);
		context.rotate(rotation);
		img.drawSimple(0,-img.width / 2, -img.height/2);
		context.restore();
	}
	context.restore();
};

// *****************************
// Internal Functions
// *****************************

//Loading the skeleton
tbSpine.prototype._loadSkeleton = function(skeletonText){
	"use strict";
	// Create the json
	var json = new spine["SkeletonJson"]({
		"newRegionAttachment": function (skin, name, path) {
			var attachment = new spine["RegionAttachment"](name);
			attachment["rendererObject"] = undefined;
			return attachment;
		},
		"newBoundingBoxAttachment": function (skin, name) {
			return new spine["BoundingBoxAttachment"](name);
		}
	});
	// Create the skeleton data
	this.skeletonData = json["readSkeletonData"](JSON.parse(skeletonText));
	// Mirror the vertical direction
	spine["Bone"]["yDown"] = true;
	// Create the skeleton 
	this.skeleton = new spine["Skeleton"](this.skeletonData);
	this.skeleton["updateWorldTransform"]();
	// Create the space
	this.stateData = new spine["AnimationStateData"](this.skeletonData);
	this.state = new spine["AnimationState"](this.stateData);
	
	// Initialize the events
	this._initEvents();
};

//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//*****    The Before Game Scripts
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************


//******************************************************
//*****    Script Resource: TG_SettingsFunctions
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

var TG = window["TG_InitSettings"] || {};
TG.size = (TG["size"] !== undefined) ? TG["size"] : "big";
TG.usesFullScreen = TG["usesFullScreen"];
TG.scaleFactor =(TG.size === "big")? 1 : 0.5;

//setup layers before reading settings, so they can be referenced in settings files
TG.frameworkBackgroundLayer = 20;
TG.gameBackgroundLayer = 10;
TG.gameLayer = 0;
TG.gameUILayer = -10;
TG.gameEffectsLayer = -20;
TG.frameworkUILayer = -30;
TG.frameworkEffectsLayer = -40;

/* exported tgScale */
function tgScale(value, type) {
	"use strict";
	var i, scale = function(v, t) {
		switch (t) {
		case "floor":
			return Math.floor(TG.scaleFactor * v);
		case "round":
			return Math.round(TG.scaleFactor * v);
		default:
			return TG.scaleFactor * v;
		}
	};
	if (typeof value === "number") {
		return scale(value, type);
	}
	if (Object.prototype.toString.call(value) === "[object Array]") {
		for (i = 0; i < value.length; i++) {
			value[i] = tgScale(value[i], type);
		}
		return value;
	}
	if (typeof value === "object") {
		for (i in value) {
			if (value.hasOwnProperty(i)) {
			    value[i] = tgScale(value[i], type);
			}
		}
		return value;
	}
}

/*exported tgSelect */
function tgSelect(obj){
	"use strict";
	if(TG.size === "big"){
		return (obj.big !== undefined)? obj.big : obj;
	}
	return (obj.small !== undefined)? obj.small : obj;
}

//******************************************************
//*****    Script Resource: booster_wrapper
//******************************************************

var BoosterSdk = (function(){

	function BoosterSdk(publisherSettings, gameOrientation, pauseCallback, unpauseCallback, splashFinishedCallback, context)
	{
//      this.gameId = "BM_gameId_Placeholder";
//      this.gameCategory = "BM_gameId_Placeholder";
//      this.developerId = "BM_developerCode_Placeholder";
//      this.gameCode = "BM_gameCode_Placeholder";
		this.orientation = gameOrientation;

		this.pauseCallback = pauseCallback;
		this.unpauseCallback = unpauseCallback;
		this.splashFinishedCallback = splashFinishedCallback;
		this.callbackContext = context;
		
		this.adChannel = publisherSettings["adChannel"];
		
		this.gameCode = publisherSettings["gameCode"];

		this.gameName = publisherSettings["gameName"];
		this.gameVersion = publisherSettings["gameVersion"];
		this.gameCategory = publisherSettings["gameCategory"];
		this.developerId = publisherSettings["developerId"];

		this.enableAds = publisherSettings["enableAds"];
		this.adFreq = publisherSettings["adFreq"];
		this.firstAd = publisherSettings["firstAd"];
		
		this.moreGames = publisherSettings["moreGames"];
		this.moreGamesURL = publisherSettings["moreGamesURL"];
		
		this.tabIcon = publisherSettings["tabIcon"];
		
		//console.log(this.gameName);

		var interval = setInterval(function(){

			if(this.ready())
			{
				this.setupBoosterSdk();
				clearInterval(interval);
			}

		}.bind(this), 100);
	}

	BoosterSdk.prototype.ready = function() {
		return window["Booster"]["ready"] && window["Booster"]["Ad"];
	};

	BoosterSdk.prototype.setupBoosterSdk = function() {

		new Booster["Init"]({
			"orientation": this.orientation,
			"splash": true,
			"minimalUI": true,
			"rotationOnHide": function() 
			{
				window.setTimeout(function() {
					TG.controller.handleResume();
				}, 400);
			},
			"rotationOnShow": function ()
			{
				TG.controller.handlePause();
			}
		});

		adSense = new Booster["Ad"]({
			"type": "adsense",
			"size": "300x250",
			"channelID": this.adChannel
		});
		
		community = new Booster["Community"]({
			"position": 1,
			"gameCode": this.gameCode
			 //"tabIcon": this.tabIcon
		});

		analytics = new Booster["Analytics"]({
			"gameName": this.gameName,
			"gameId": this.gameVersion,
			"gameCategory": this.gameCategory,
			"developer": this.developerId
		});

		moregames = new Booster["Moregames"]();
		
		try{
			freemium = new Booster["Freemium"]();
		}catch(e){}
		

		var this_ = this;

		Booster["onSplashFinishedEvent"] = function() {
			this_.splashFinishedCallback.call(this_.callbackContext);
		};

		Booster["onOpenTab"] = function() {
			this_.pauseCallback.call(this_.callbackContext);
		};

		Booster["onCloseTab"] = function() {
			this_.unpauseCallback.call(this_.callbackContext);
		};

	};

	return BoosterSdk;


})();











//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//*****    The Resources
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************


//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Namespaces
//******************************************************
//******************************************************
//******************************************************


//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Data Files
//******************************************************
//******************************************************
//******************************************************

    var throbber = new tbDataFile("throbber", "media/throbber.png");

//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Textures
//******************************************************
//******************************************************
//******************************************************

    var StartTexture = new tbTexture("StartTexture", 2, "start");
        StartTexture.addBitmap(0, "media/StartTexture0.png");
        StartTexture.addBitmap(1, "media/StartTexture1.png");
    var StartScreenTexture = new tbTexture("StartScreenTexture", 1, "load");
        StartScreenTexture.addBitmap(0, "media/StartScreenTexture0.png");
    var LevelMapScreenTexture = new tbTexture("LevelMapScreenTexture", 1, "load");
        LevelMapScreenTexture.addBitmap(0, "media/LevelMapScreenTexture0.png");
    var LevelEndTexture = new tbTexture("LevelEndTexture", 2, "load");
        LevelEndTexture.addBitmap(0, "media/LevelEndTexture0.png");
        LevelEndTexture.addBitmap(1, "media/LevelEndTexture1.png");
    var MenuTexture = new tbTexture("MenuTexture", 2, "load");
        MenuTexture.addBitmap(0, "media/MenuTexture0.png");
        MenuTexture.addBitmap(1, "media/MenuTexture1.png");
    var GameTexture = new tbTexture("GameTexture", 3, "load");
        GameTexture.addBitmap(0, "media/GameTexture0.png");
        GameTexture.addBitmap(1, "media/GameTexture1.png");
        GameTexture.addBitmap(2, "media/GameTexture2.png");
    var GameStaticTexture = new tbTexture("GameStaticTexture", 2, "load");
        GameStaticTexture.addBitmap(0, "media/GameStaticTexture0.png");
        GameStaticTexture.addBitmap(1, "media/GameStaticTexture1.png");

//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Images
//******************************************************
//******************************************************
//******************************************************

    var imageScalePercentage = 100;

    var s_loadingbar_background = new tbImage("s_loadingbar_background", StartScreenTexture, 1, 42, 32, 0, 0, 42, 32, 1);
        s_loadingbar_background.addImage(0, 0, 929, 1, 42, 32, 0, 0);
    var s_loadingbar_fill = new tbImage("s_loadingbar_fill", StartScreenTexture, 1, 20, 12, 0, 0, 20, 12, 1);
        s_loadingbar_fill.addImage(0, 0, 977, 1, 20, 12, 0, 0);
    var s_level_0 = new tbImage("s_level_0", LevelMapScreenTexture, 1, 125, 140, 0, 0, 125, 140, 1);
        s_level_0.addImage(0, 0, 129, 1, 125, 140, 0, 0);
    var s_level_1 = new tbImage("s_level_1", LevelMapScreenTexture, 1, 125, 140, 0, 0, 125, 140, 1);
        s_level_1.addImage(0, 0, 257, 1, 125, 140, 0, 0);
    var s_level_2 = new tbImage("s_level_2", LevelMapScreenTexture, 1, 125, 140, 0, 0, 125, 140, 1);
        s_level_2.addImage(0, 0, 1, 1, 125, 140, 0, 0);
    var s_level_3 = new tbImage("s_level_3", LevelMapScreenTexture, 1, 125, 140, 0, 0, 125, 140, 1);
        s_level_3.addImage(0, 0, 385, 1, 125, 140, 0, 0);
    var s_level_lock = new tbImage("s_level_lock", LevelMapScreenTexture, 1, 48, 70, 0, 0, 48, 70, 1);
        s_level_lock.addImage(0, 0, 777, 113, 48, 69, 0, 1);
    var s_level_stars = new tbImage("s_level_stars", LevelMapScreenTexture, 1, 126, 46, 0, 0, 126, 46, 1);
        s_level_stars.addImage(0, 0, 513, 1, 126, 45, 0, 1);
    var s_level2_0 = new tbImage("s_level2_0", LevelMapScreenTexture, 1, 84, 87, 0, 0, 84, 87, 1);
        s_level2_0.addImage(0, 0, 897, 97, 84, 87, 0, 0);
    var s_level2_1 = new tbImage("s_level2_1", LevelMapScreenTexture, 1, 84, 87, 0, 0, 84, 87, 1);
        s_level2_1.addImage(0, 0, 897, 1, 84, 87, 0, 0);
    var s_level2_2 = new tbImage("s_level2_2", LevelMapScreenTexture, 1, 84, 87, 0, 0, 84, 87, 1);
        s_level2_2.addImage(0, 0, 601, 113, 84, 87, 0, 0);
    var s_level2_3 = new tbImage("s_level2_3", LevelMapScreenTexture, 1, 84, 87, 0, 0, 84, 87, 1);
        s_level2_3.addImage(0, 0, 513, 49, 84, 87, 0, 0);
    var s_level2_arrow_right = new tbImage("s_level2_arrow_right", LevelMapScreenTexture, 2, 60, 108, 0, 0, 60, 216, 1);
        s_level2_arrow_right.addImage(0, 0, 833, 1, 60, 108, 0, 0);
        s_level2_arrow_right.addImage(1, 0, 641, 1, 60, 108, 0, 0);
    var s_level2_arrow_left = new tbImage("s_level2_arrow_left", LevelMapScreenTexture, 2, 60, 108, 0, 0, 60, 216, 1);
        s_level2_arrow_left.addImage(0, 0, 705, 1, 60, 108, 0, 0);
        s_level2_arrow_left.addImage(1, 0, 769, 1, 60, 108, 0, 0);
    var s_level2_lock = new tbImage("s_level2_lock", LevelMapScreenTexture, 1, 84, 87, 0, 0, 84, 87, 1);
        s_level2_lock.addImage(0, 0, 689, 113, 84, 87, 0, 0);
    var s_pop_medal = new tbImage("s_pop_medal", LevelEndTexture, 8, 378, 378, 189, 189, 3024, 378, 8);
        s_pop_medal.addImage(0, 0, 617, 1, 349, 241, 3, 69);
        s_pop_medal.addImage(1, 0, 617, 529, 346, 267, 5, 54);
        s_pop_medal.addImage(2, 0, 617, 249, 348, 276, 20, 56);
        s_pop_medal.addImage(3, 1, 1, 1, 342, 288, 26, 50);
        s_pop_medal.addImage(4, 1, 689, 1, 319, 292, 22, 46);
        s_pop_medal.addImage(5, 1, 1, 297, 337, 304, 14, 41);
        s_pop_medal.addImage(6, 0, 1, 633, 343, 305, 12, 41);
        s_pop_medal.addImage(7, 1, 345, 1, 341, 304, 13, 41);
    var s_medal_shadow = new tbImage("s_medal_shadow", LevelEndTexture, 1, 195, 208, 0, 0, 195, 208, 1);
        s_medal_shadow.addImage(0, 1, 745, 513, 189, 204, 3, 1);
    var s_medal_shine = new tbImage("s_medal_shine", LevelEndTexture, 6, 195, 208, 0, 0, 1170, 208, 6);
        s_medal_shine.addImage(0, 1, 545, 513, 193, 207, 1, 1);
        s_medal_shine.addImage(1, 1, 345, 313, 193, 207, 1, 1);
        s_medal_shine.addImage(2, 1, 689, 297, 193, 207, 1, 1);
        s_medal_shine.addImage(3, 0, 353, 633, 193, 207, 1, 1);
        s_medal_shine.addImage(4, 0, 553, 801, 193, 207, 1, 1);
        s_medal_shine.addImage(5, 0, 753, 801, 193, 207, 1, 1);
    var s_icon_toggle_hard = new tbImage("s_icon_toggle_hard", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_hard.addImage(0, 0, 953, 617, 67, 67, 0, 0);
    var s_icon_toggle_medium = new tbImage("s_icon_toggle_medium", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_medium.addImage(0, 0, 537, 633, 67, 67, 0, 0);
    var s_icon_toggle_easy = new tbImage("s_icon_toggle_easy", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_easy.addImage(0, 0, 465, 633, 67, 67, 0, 0);
    var s_flagIcon_us = new tbImage("s_flagIcon_us", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_us.addImage(0, 0, 969, 169, 48, 36, 0, 6);
    var s_flagIcon_gb = new tbImage("s_flagIcon_gb", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_gb.addImage(0, 0, 969, 209, 48, 36, 0, 6);
    var s_flagIcon_nl = new tbImage("s_flagIcon_nl", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_nl.addImage(0, 0, 969, 249, 48, 36, 0, 6);
    var s_flagIcon_tr = new tbImage("s_flagIcon_tr", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_tr.addImage(0, 0, 969, 329, 48, 36, 0, 6);
    var s_flagIcon_de = new tbImage("s_flagIcon_de", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_de.addImage(0, 0, 409, 729, 48, 36, 0, 6);
    var s_flagIcon_fr = new tbImage("s_flagIcon_fr", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_fr.addImage(0, 0, 393, 809, 48, 36, 0, 6);
    var s_flagIcon_br = new tbImage("s_flagIcon_br", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_br.addImage(0, 0, 409, 769, 48, 36, 0, 6);
    var s_flagIcon_es = new tbImage("s_flagIcon_es", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_es.addImage(0, 0, 969, 369, 48, 36, 0, 6);
    var s_flagIcon_jp = new tbImage("s_flagIcon_jp", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_jp.addImage(0, 0, 969, 289, 48, 36, 0, 6);
    var s_flagIcon_ru = new tbImage("s_flagIcon_ru", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_ru.addImage(0, 0, 969, 129, 48, 36, 0, 6);
    var s_flagIcon_ar = new tbImage("s_flagIcon_ar", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_ar.addImage(0, 0, 865, 385, 48, 36, 0, 6);
    var s_flagIcon_kr = new tbImage("s_flagIcon_kr", MenuTexture, 1, 48, 48, 0, 0, 48, 48, 1);
        s_flagIcon_kr.addImage(0, 0, 337, 801, 48, 36, 0, 6);
    var s_tutorialButton_close = new tbImage("s_tutorialButton_close", MenuTexture, 1, 66, 65, 0, 0, 66, 65, 1);
        s_tutorialButton_close.addImage(0, 0, 929, 761, 65, 65, 0, 0);
    var s_tutorialButton_next = new tbImage("s_tutorialButton_next", MenuTexture, 1, 66, 65, 0, 0, 66, 65, 1);
        s_tutorialButton_next.addImage(0, 0, 337, 729, 66, 65, 0, 0);
    var s_tutorialButton_previous = new tbImage("s_tutorialButton_previous", MenuTexture, 1, 66, 65, 0, 0, 66, 65, 1);
        s_tutorialButton_previous.addImage(0, 0, 953, 689, 66, 65, 0, 0);
    var s_logo_tinglygames = new tbImage("s_logo_tinglygames", MenuTexture, 1, 240, 240, 0, 0, 240, 240, 1);
        s_logo_tinglygames.addImage(0, 0, 617, 1, 240, 240, 0, 0);
    var s_logo_coolgames = new tbImage("s_logo_coolgames", MenuTexture, 1, 240, 240, 0, 0, 240, 240, 1);
        s_logo_coolgames.addImage(0, 0, 617, 249, 240, 167, 0, 36);
    var s_logo_tinglygames_start = new tbImage("s_logo_tinglygames_start", StartScreenTexture, 1, 156, 54, 0, 0, 156, 54, 1);
        s_logo_tinglygames_start.addImage(0, 0, 617, 1, 156, 53, 0, 0);
    var s_logo_coolgames_start = new tbImage("s_logo_coolgames_start", StartScreenTexture, 1, 300, 104, 0, 0, 300, 104, 1);
        s_logo_coolgames_start.addImage(0, 0, 777, 1, 150, 104, 75, 0);
    var s_ui_cup_highscore = new tbImage("s_ui_cup_highscore", GameTexture, 1, 32, 28, 0, 0, 32, 28, 1);
        s_ui_cup_highscore.addImage(0, 0, 233, 785, 32, 28, 0, 0);
    var s_ui_cup_score = new tbImage("s_ui_cup_score", GameTexture, 1, 28, 24, 0, 0, 28, 24, 1);
        s_ui_cup_score.addImage(0, 0, 937, 473, 28, 24, 0, 0);
    var s_ui_divider = new tbImage("s_ui_divider", GameStaticTexture, 1, 94, 2, 0, 0, 94, 2, 1);
        s_ui_divider.addImage(0, 0, 905, 1, 94, 2, 0, 0);
    var s_ui_background_blank = new tbImage("s_ui_background_blank", GameStaticTexture, 1, 140, 580, 0, 0, 140, 580, 1);
        s_ui_background_blank.addImage(0, 0, 617, 1, 140, 580, 0, 0);
    var s_ui_highscore = new tbImage("s_ui_highscore", GameStaticTexture, 1, 26, 36, 13, 12, 26, 36, 1);
        s_ui_highscore.addImage(0, 0, 905, 9, 26, 36, 0, 0);
    var s_ui_timeleft = new tbImage("s_ui_timeleft", GameStaticTexture, 1, 20, 26, 0, 0, 20, 26, 1);
        s_ui_timeleft.addImage(0, 0, 1001, 1, 20, 26, 0, 0);
    var s_ui_smiley_hard = new tbImage("s_ui_smiley_hard", GameTexture, 1, 22, 22, 11, 11, 22, 22, 1);
        s_ui_smiley_hard.addImage(0, 0, 1001, 497, 22, 22, 0, 0);
    var s_ui_smiley_medium = new tbImage("s_ui_smiley_medium", GameTexture, 1, 22, 22, 11, 11, 22, 22, 1);
        s_ui_smiley_medium.addImage(0, 0, 1001, 841, 22, 22, 0, 0);
    var s_ui_smiley_easy = new tbImage("s_ui_smiley_easy", GameTexture, 1, 22, 22, 11, 11, 22, 22, 1);
        s_ui_smiley_easy.addImage(0, 0, 1001, 817, 22, 22, 0, 0);
    var s_ui_crown = new tbImage("s_ui_crown", GameTexture, 1, 24, 20, 12, 10, 24, 20, 1);
        s_ui_crown.addImage(0, 0, 969, 497, 24, 20, 0, 0);
    var s_ui_heart = new tbImage("s_ui_heart", GameTexture, 1, 28, 24, 14, 12, 28, 24, 1);
        s_ui_heart.addImage(0, 0, 305, 785, 26, 23, 1, 1);
    var s_tile_flash = new tbImage("s_tile_flash", GameTexture, 1, 70, 70, 35, 35, 70, 70, 1);
        s_tile_flash.addImage(0, 2, 337, 217, 70, 70, 0, 0);
    var s_bomb_explosion_center = new tbImage("s_bomb_explosion_center", GameTexture, 8, 300, 300, 150, 150, 2400, 300, 8);
        s_bomb_explosion_center.addImage(0, 0, 929, 817, 69, 79, 117, 107);
        s_bomb_explosion_center.addImage(1, 1, 785, 593, 212, 205, 38, 38);
        s_bomb_explosion_center.addImage(2, 1, 537, 593, 244, 245, 16, 20);
        s_bomb_explosion_center.addImage(3, 1, 281, 593, 253, 253, 11, 13);
        s_bomb_explosion_center.addImage(4, 1, 281, 313, 272, 275, 5, 3);
        s_bomb_explosion_center.addImage(5, 1, 1, 313, 275, 280, 4, 2);
        s_bomb_explosion_center.addImage(6, 1, 689, 297, 280, 287, 1, 3);
        s_bomb_explosion_center.addImage(7, 1, 689, 1, 280, 291, 4, 6);
    var s_bomb_explosion = new tbImage("s_bomb_explosion", GameTexture, 7, 150, 150, 75, 75, 1050, 150, 7);
        s_bomb_explosion.addImage(0, 1, 1, 601, 141, 141, 4, 4);
        s_bomb_explosion.addImage(1, 1, 1, 745, 141, 141, 5, 5);
        s_bomb_explosion.addImage(2, 1, 281, 849, 117, 117, 17, 17);
        s_bomb_explosion.addImage(3, 2, 785, 193, 71, 71, 40, 40);
        s_bomb_explosion.addImage(4, 2, 697, 273, 63, 63, 44, 44);
        s_bomb_explosion.addImage(5, 2, 409, 273, 69, 69, 41, 41);
        s_bomb_explosion.addImage(6, 0, 273, 785, 27, 27, 62, 62);
    var s_nova_lightning_01 = new tbImage("s_nova_lightning_01", GameTexture, 1, 560, 70, 0, 35, 560, 70, 1);
        s_nova_lightning_01.addImage(0, 0, 1, 65, 560, 59, 0, 5);
    var s_nova_lightning_02 = new tbImage("s_nova_lightning_02", GameTexture, 1, 560, 70, 0, 35, 560, 70, 1);
        s_nova_lightning_02.addImage(0, 0, 1, 129, 560, 59, 0, 5);
    var s_nova_lightning_03 = new tbImage("s_nova_lightning_03", GameTexture, 1, 560, 70, 0, 35, 560, 70, 1);
        s_nova_lightning_03.addImage(0, 0, 1, 1, 560, 59, 0, 5);
    var s_hypercube_lightning = new tbImage("s_hypercube_lightning", GameTexture, 16, 260, 120, 0, 60, 4160, 120, 16);
        s_hypercube_lightning.addImage(0, 2, 185, 209, 71, 70, 17, 25);
        s_hypercube_lightning.addImage(1, 2, 105, 209, 71, 69, 17, 25);
        s_hypercube_lightning.addImage(2, 0, 1, 841, 224, 54, 17, 33);
        s_hypercube_lightning.addImage(3, 0, 561, 953, 71, 69, 172, 26);
        s_hypercube_lightning.addImage(4, 0, 705, 249, 257, 109, 2, 6);
        s_hypercube_lightning.addImage(5, 0, 1, 785, 225, 52, 17, 34);
        s_hypercube_lightning.addImage(6, 0, 233, 817, 225, 52, 17, 34);
        s_hypercube_lightning.addImage(7, 0, 465, 817, 225, 67, 17, 27);
        s_hypercube_lightning.addImage(8, 0, 697, 817, 225, 52, 17, 34);
        s_hypercube_lightning.addImage(9, 0, 705, 361, 257, 109, 2, 6);
        s_hypercube_lightning.addImage(10, 0, 697, 873, 224, 67, 18, 27);
        s_hypercube_lightning.addImage(11, 0, 233, 873, 224, 64, 18, 29);
        s_hypercube_lightning.addImage(12, 0, 465, 889, 204, 57, 30, 30);
        s_hypercube_lightning.addImage(13, 0, 353, 193, 192, 52, 38, 32);
        s_hypercube_lightning.addImage(14, 0, 705, 473, 182, 42, 43, 37);
        s_hypercube_lightning.addImage(15, 0, 1, 897, 182, 42, 43, 37);
    var s_hint = new tbImage("s_hint", GameTexture, 10, 200, 200, 100, 100, 2000, 200, 10);
        s_hint.addImage(0, 2, 441, 1, 107, 107, 47, 47);
        s_hint.addImage(1, 2, 761, 1, 103, 103, 49, 49);
        s_hint.addImage(2, 2, 313, 113, 99, 99, 51, 51);
        s_hint.addImage(3, 2, 1, 1, 110, 110, 45, 45);
        s_hint.addImage(4, 1, 561, 441, 119, 119, 41, 41);
        s_hint.addImage(5, 1, 561, 313, 126, 126, 37, 37);
        s_hint.addImage(6, 1, 537, 841, 130, 130, 35, 35);
        s_hint.addImage(7, 1, 785, 801, 131, 132, 35, 34);
        s_hint.addImage(8, 1, 145, 601, 131, 132, 35, 34);
        s_hint.addImage(9, 1, 145, 737, 131, 132, 35, 34);
    var s_selection = new tbImage("s_selection", GameTexture, 1, 68, 68, 34, 34, 68, 68, 1);
        s_selection.addImage(0, 2, 553, 273, 68, 68, 0, 0);
    var s_pop_floater = new tbImage("s_pop_floater", GameTexture, 8, 378, 378, 189, 189, 3024, 378, 8);
        s_pop_floater.addImage(0, 0, 569, 1, 349, 241, 3, 69);
        s_pop_floater.addImage(1, 0, 353, 249, 346, 267, 5, 54);
        s_pop_floater.addImage(2, 0, 1, 193, 348, 276, 20, 56);
        s_pop_floater.addImage(3, 0, 353, 521, 342, 288, 26, 50);
        s_pop_floater.addImage(4, 0, 697, 521, 319, 292, 22, 46);
        s_pop_floater.addImage(5, 1, 345, 1, 337, 304, 14, 41);
        s_pop_floater.addImage(6, 0, 1, 473, 343, 305, 12, 41);
        s_pop_floater.addImage(7, 1, 1, 1, 341, 304, 13, 41);
    var s_btn_big_restart = new tbImage("s_btn_big_restart", LevelEndTexture, 2, 154, 152, 0, 0, 308, 152, 2);
        s_btn_big_restart.addImage(0, 0, 353, 849, 154, 152, 0, 0);
        s_btn_big_restart.addImage(1, 1, 345, 529, 154, 152, 0, 0);
    var s_btn_big_start = new tbImage("s_btn_big_start", LevelEndTexture, 2, 154, 152, 0, 0, 308, 152, 2);
        s_btn_big_start.addImage(0, 1, 1, 609, 154, 152, 0, 0);
        s_btn_big_start.addImage(1, 1, 161, 609, 154, 152, 0, 0);
    var s_btn_small_exit = new tbImage("s_btn_small_exit", MenuTexture, 2, 100, 92, 0, 0, 200, 92, 2);
        s_btn_small_exit.addImage(0, 0, 865, 97, 100, 92, 0, 0);
        s_btn_small_exit.addImage(1, 0, 865, 289, 100, 92, 0, 0);
    var s_btn_small_pause = new tbImage("s_btn_small_pause", GameTexture, 2, 100, 92, 0, 0, 200, 92, 2);
        s_btn_small_pause.addImage(0, 2, 209, 113, 100, 92, 0, 0);
        s_btn_small_pause.addImage(1, 2, 105, 113, 100, 92, 0, 0);
    var s_btn_small_options = new tbImage("s_btn_small_options", MenuTexture, 2, 100, 92, 0, 0, 200, 92, 2);
        s_btn_small_options.addImage(0, 0, 865, 1, 100, 92, 0, 0);
        s_btn_small_options.addImage(1, 0, 865, 193, 100, 92, 0, 0);
    var s_btn_small_retry = new tbImage("s_btn_small_retry", LevelEndTexture, 2, 100, 92, 0, 0, 200, 92, 2);
        s_btn_small_retry.addImage(0, 1, 889, 393, 100, 92, 0, 0);
        s_btn_small_retry.addImage(1, 1, 889, 297, 100, 92, 0, 0);
    var s_btn_standard = new tbImage("s_btn_standard", MenuTexture, 2, 96, 92, 0, 0, 192, 92, 2);
        s_btn_standard.addImage(0, 0, 849, 617, 96, 92, 0, 0);
        s_btn_standard.addImage(1, 0, 233, 729, 96, 92, 0, 0);
    var s_btn_toggle = new tbImage("s_btn_toggle", MenuTexture, 2, 162, 92, 0, 0, 324, 92, 2);
        s_btn_toggle.addImage(0, 0, 849, 521, 162, 92, 0, 0);
        s_btn_toggle.addImage(1, 0, 849, 425, 162, 92, 0, 0);
    var s_icon_toggle_fxoff = new tbImage("s_icon_toggle_fxoff", MenuTexture, 2, 227, 92, 0, 0, 454, 92, 2);
        s_icon_toggle_fxoff.addImage(0, 0, 617, 425, 227, 92, 0, 0);
        s_icon_toggle_fxoff.addImage(1, 0, 697, 713, 227, 92, 0, 0);
    var s_icon_toggle_fxon = new tbImage("s_icon_toggle_fxon", MenuTexture, 2, 227, 92, 0, 0, 454, 92, 2);
        s_icon_toggle_fxon.addImage(0, 0, 465, 713, 227, 92, 0, 0);
        s_icon_toggle_fxon.addImage(1, 0, 1, 729, 227, 92, 0, 0);
    var s_icon_toggle_musicoff = new tbImage("s_icon_toggle_musicoff", MenuTexture, 2, 227, 92, 0, 0, 454, 92, 2);
        s_icon_toggle_musicoff.addImage(0, 0, 233, 633, 227, 92, 0, 0);
        s_icon_toggle_musicoff.addImage(1, 0, 1, 633, 227, 92, 0, 0);
    var s_icon_toggle_musicon = new tbImage("s_icon_toggle_musicon", MenuTexture, 2, 227, 92, 0, 0, 454, 92, 2);
        s_icon_toggle_musicon.addImage(0, 0, 617, 617, 227, 92, 0, 0);
        s_icon_toggle_musicon.addImage(1, 0, 617, 521, 227, 92, 0, 0);
    var s_btn_bigtext = new tbImage("s_btn_bigtext", LevelEndTexture, 2, 137, 104, 0, 0, 274, 104, 2);
        s_btn_bigtext.addImage(0, 1, 545, 313, 137, 104, 0, 0);
        s_btn_bigtext.addImage(1, 1, 321, 689, 137, 104, 0, 0);
    var s_icon_toggle_sfx_on = new tbImage("s_icon_toggle_sfx_on", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_sfx_on.addImage(0, 0, 969, 89, 49, 31, 7, 17);
    var s_icon_toggle_sfx_off = new tbImage("s_icon_toggle_sfx_off", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_sfx_off.addImage(0, 0, 969, 1, 53, 31, 7, 17);
    var s_icon_toggle_music_on = new tbImage("s_icon_toggle_music_on", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_music_on.addImage(0, 0, 449, 809, 38, 41, 13, 16);
    var s_icon_toggle_music_off = new tbImage("s_icon_toggle_music_off", MenuTexture, 1, 67, 67, 0, 0, 67, 67, 1);
        s_icon_toggle_music_off.addImage(0, 0, 969, 41, 51, 41, 8, 16);
    var s_overlay_options = new tbImage("s_overlay_options", MenuTexture, 1, 610, 628, 0, 0, 610, 628, 1);
        s_overlay_options.addImage(0, 0, 1, 1, 609, 627, 1, 1);
    var s_overlay_assignment = new tbImage("s_overlay_assignment", GameStaticTexture, 1, 608, 340, 0, 0, 608, 340, 1);
        s_overlay_assignment.addImage(0, 0, 1, 1, 607, 339, 1, 1);
    var s_tutorial = new tbImage("s_tutorial", MenuTexture, 1, 564, 524, 0, 0, 564, 524, 1);
        s_tutorial.addImage(0, 1, 1, 1, 564, 524, 0, 0);
    var s_overlay_endless = new tbImage("s_overlay_endless", LevelEndTexture, 1, 608, 628, 0, 0, 608, 628, 1);
        s_overlay_endless.addImage(0, 0, 1, 1, 608, 627, 0, 1);
    var s_screen_start = new tbImage("s_screen_start", StartTexture, 4, 576, 320, 0, 0, 1152, 640, 2);
        s_screen_start.addImage(0, 0, 1, 329, 576, 320, 0, 0);
        s_screen_start.addImage(1, 1, 1, 1, 576, 320, 0, 0);
        s_screen_start.addImage(2, 0, 1, 1, 576, 320, 0, 0);
        s_screen_start.addImage(3, 0, 1, 657, 576, 320, 0, 0);
    var s_hypercube = new tbImage("s_hypercube", GameTexture, 16, 114, 114, 57, 57, 1824, 114, 16);
        s_hypercube.addImage(0, 1, 921, 801, 102, 111, 6, 0);
        s_hypercube.addImage(1, 1, 673, 841, 108, 113, 5, 0);
        s_hypercube.addImage(2, 1, 145, 873, 105, 112, 6, 0);
        s_hypercube.addImage(3, 2, 113, 1, 105, 110, 5, 1);
        s_hypercube.addImage(4, 1, 921, 921, 102, 102, 6, 6);
        s_hypercube.addImage(5, 1, 1, 889, 104, 112, 6, 2);
        s_hypercube.addImage(6, 0, 921, 1, 102, 114, 6, 0);
        s_hypercube.addImage(7, 0, 921, 121, 102, 112, 6, 0);
        s_hypercube.addImage(8, 2, 1, 113, 102, 102, 6, 6);
        s_hypercube.addImage(9, 2, 553, 1, 102, 106, 6, 2);
        s_hypercube.addImage(10, 2, 657, 1, 102, 105, 6, 3);
        s_hypercube.addImage(11, 2, 873, 1, 102, 102, 6, 6);
        s_hypercube.addImage(12, 2, 873, 105, 102, 102, 6, 6);
        s_hypercube.addImage(13, 1, 401, 849, 103, 112, 6, 0);
        s_hypercube.addImage(14, 2, 225, 1, 102, 109, 6, 1);
        s_hypercube.addImage(15, 2, 329, 1, 103, 109, 6, 0);
    var s_nova_gem = new tbImage("s_nova_gem", GameTexture, 22, 76, 76, 38, 38, 1672, 76, 22);
        s_nova_gem.addImage(0, 0, 321, 945, 73, 76, 1, 0);
        s_nova_gem.addImage(1, 0, 1, 945, 76, 76, 0, 0);
        s_nova_gem.addImage(2, 0, 161, 945, 76, 74, 0, 2);
        s_nova_gem.addImage(3, 0, 753, 945, 76, 72, 0, 3);
        s_nova_gem.addImage(4, 0, 673, 945, 76, 74, 0, 2);
        s_nova_gem.addImage(5, 0, 241, 945, 76, 72, 0, 3);
        s_nova_gem.addImage(6, 2, 497, 113, 73, 74, 0, 2);
        s_nova_gem.addImage(7, 2, 1, 217, 70, 70, 3, 3);
        s_nova_gem.addImage(8, 0, 401, 953, 73, 70, 3, 3);
        s_nova_gem.addImage(9, 0, 81, 945, 76, 71, 0, 2);
        s_nova_gem.addImage(10, 2, 657, 113, 73, 71, 0, 2);
        s_nova_gem.addImage(11, 2, 937, 209, 70, 70, 3, 3);
        s_nova_gem.addImage(12, 0, 481, 953, 73, 70, 3, 3);
        s_nova_gem.addImage(13, 0, 929, 905, 76, 70, 0, 3);
        s_nova_gem.addImage(14, 2, 561, 193, 73, 73, 0, 0);
        s_nova_gem.addImage(15, 2, 417, 193, 70, 73, 3, 0);
        s_nova_gem.addImage(16, 2, 865, 209, 70, 70, 3, 3);
        s_nova_gem.addImage(17, 2, 641, 193, 70, 72, 3, 1);
        s_nova_gem.addImage(18, 2, 713, 193, 70, 72, 3, 1);
        s_nova_gem.addImage(19, 2, 737, 113, 70, 73, 3, 0);
        s_nova_gem.addImage(20, 2, 489, 193, 70, 73, 3, 0);
        s_nova_gem.addImage(21, 2, 265, 217, 70, 70, 3, 3);
    var s_logo = new tbImage("s_logo", StartScreenTexture, 1, 610, 418, 0, 0, 610, 418, 1);
        s_logo.addImage(0, 0, 1, 1, 609, 417, 1, 1);
    var s_tutorial_01 = new tbImage("s_tutorial_01", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_01.addImage(0, 1, 585, 337, 349, 152, 1, 12);
    var s_tutorial_02 = new tbImage("s_tutorial_02", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_02.addImage(0, 1, 585, 1, 349, 139, 1, 25);
    var s_tutorial_03 = new tbImage("s_tutorial_03", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_03.addImage(0, 1, 585, 497, 349, 188, 1, 1);
    var s_tutorial_04 = new tbImage("s_tutorial_04", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_04.addImage(0, 1, 585, 145, 349, 188, 1, 1);
    var s_tutorial_05 = new tbImage("s_tutorial_05", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_05.addImage(0, 0, 585, 729, 349, 188, 1, 1);
    var s_tutorial_06 = new tbImage("s_tutorial_06", GameStaticTexture, 1, 350, 190, 0, 0, 350, 190, 1);
        s_tutorial_06.addImage(0, 0, 585, 585, 349, 138, 1, 26);
    var s_gem_red = new tbImage("s_gem_red", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_red.addImage(0, 2, 385, 409, 58, 58, 9, 9);
        s_gem_red.addImage(1, 2, 449, 409, 58, 58, 9, 9);
        s_gem_red.addImage(2, 2, 641, 409, 58, 58, 9, 9);
        s_gem_red.addImage(3, 2, 577, 409, 58, 58, 9, 9);
        s_gem_red.addImage(4, 2, 193, 409, 58, 58, 9, 9);
        s_gem_red.addImage(5, 2, 129, 409, 58, 58, 9, 9);
        s_gem_red.addImage(6, 2, 713, 401, 58, 58, 9, 9);
        s_gem_red.addImage(7, 2, 65, 409, 58, 58, 9, 9);
    var s_gem_white = new tbImage("s_gem_white", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_white.addImage(0, 0, 969, 241, 50, 58, 13, 9);
        s_gem_white.addImage(1, 0, 969, 305, 50, 58, 13, 9);
        s_gem_white.addImage(2, 0, 969, 369, 50, 58, 13, 9);
        s_gem_white.addImage(3, 0, 969, 433, 50, 58, 13, 9);
        s_gem_white.addImage(4, 2, 65, 473, 50, 58, 13, 9);
        s_gem_white.addImage(5, 1, 865, 937, 50, 58, 13, 9);
        s_gem_white.addImage(6, 2, 969, 409, 50, 58, 13, 9);
        s_gem_white.addImage(7, 2, 705, 465, 50, 58, 13, 9);
    var s_gem_yellow = new tbImage("s_gem_yellow", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_yellow.addImage(0, 2, 321, 417, 58, 58, 9, 9);
        s_gem_yellow.addImage(1, 2, 1, 417, 58, 58, 9, 9);
        s_gem_yellow.addImage(2, 2, 257, 417, 58, 58, 9, 9);
        s_gem_yellow.addImage(3, 2, 777, 409, 58, 58, 9, 9);
        s_gem_yellow.addImage(4, 2, 841, 409, 58, 58, 9, 9);
        s_gem_yellow.addImage(5, 2, 905, 409, 58, 58, 9, 9);
        s_gem_yellow.addImage(6, 2, 513, 409, 58, 58, 9, 9);
        s_gem_yellow.addImage(7, 2, 257, 353, 58, 58, 9, 9);
    var s_gem_blue = new tbImage("s_gem_blue", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_blue.addImage(0, 2, 521, 345, 58, 58, 9, 9);
        s_gem_blue.addImage(1, 2, 329, 289, 58, 58, 9, 9);
        s_gem_blue.addImage(2, 2, 961, 281, 58, 58, 9, 9);
        s_gem_blue.addImage(3, 2, 1, 289, 58, 58, 9, 9);
        s_gem_blue.addImage(4, 2, 201, 281, 58, 58, 9, 9);
        s_gem_blue.addImage(5, 2, 833, 281, 58, 58, 9, 9);
        s_gem_blue.addImage(6, 2, 137, 281, 58, 58, 9, 9);
        s_gem_blue.addImage(7, 2, 809, 113, 58, 58, 9, 9);
    var s_gem_green = new tbImage("s_gem_green", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_green.addImage(0, 2, 769, 273, 58, 58, 9, 9);
        s_gem_green.addImage(1, 2, 73, 281, 58, 58, 9, 9);
        s_gem_green.addImage(2, 1, 673, 961, 58, 58, 9, 9);
        s_gem_green.addImage(3, 2, 265, 289, 58, 58, 9, 9);
        s_gem_green.addImage(4, 2, 769, 337, 58, 58, 9, 9);
        s_gem_green.addImage(5, 2, 65, 345, 58, 58, 9, 9);
        s_gem_green.addImage(6, 2, 649, 345, 58, 58, 9, 9);
        s_gem_green.addImage(7, 2, 833, 345, 58, 58, 9, 9);
    var s_gem_orange = new tbImage("s_gem_orange", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_orange.addImage(0, 2, 897, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(1, 2, 961, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(2, 2, 585, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(3, 2, 897, 281, 58, 58, 9, 9);
        s_gem_orange.addImage(4, 2, 129, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(5, 2, 193, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(6, 2, 393, 345, 58, 58, 9, 9);
        s_gem_orange.addImage(7, 2, 457, 345, 58, 58, 9, 9);
    var s_gem_purple = new tbImage("s_gem_purple", GameTexture, 8, 76, 76, 38, 38, 608, 76, 8);
        s_gem_purple.addImage(0, 2, 713, 345, 54, 54, 11, 11);
        s_gem_purple.addImage(1, 2, 177, 473, 54, 54, 11, 11);
        s_gem_purple.addImage(2, 2, 121, 473, 54, 54, 11, 11);
        s_gem_purple.addImage(3, 1, 481, 969, 54, 54, 11, 11);
        s_gem_purple.addImage(4, 1, 369, 969, 54, 54, 11, 11);
        s_gem_purple.addImage(5, 1, 313, 969, 54, 54, 11, 11);
        s_gem_purple.addImage(6, 1, 425, 969, 54, 54, 11, 11);
        s_gem_purple.addImage(7, 1, 257, 969, 54, 54, 11, 11);
    var s_bomb_gem = new tbImage("s_bomb_gem", GameTexture, 9, 76, 76, 38, 38, 684, 76, 9);
        s_bomb_gem.addImage(0, 2, 625, 273, 68, 68, 4, 4);
        s_bomb_gem.addImage(1, 2, 417, 113, 74, 74, 1, 1);
        s_bomb_gem.addImage(2, 0, 833, 945, 76, 76, 0, 0);
        s_bomb_gem.addImage(3, 0, 889, 473, 46, 46, 15, 15);
        s_bomb_gem.addImage(4, 2, 1, 353, 58, 58, 9, 9);
        s_bomb_gem.addImage(5, 2, 481, 273, 68, 68, 4, 4);
        s_bomb_gem.addImage(6, 2, 577, 113, 74, 74, 1, 1);
        s_bomb_gem.addImage(7, 1, 785, 937, 76, 76, 0, 0);
        s_bomb_gem.addImage(8, 2, 321, 353, 58, 58, 9, 9);
    var s_background = new tbImage("s_background", GameStaticTexture, 4, 576, 320, 0, 0, 1152, 640, 2);
        s_background.addImage(0, 0, 1, 673, 576, 320, 0, 0);
        s_background.addImage(1, 1, 1, 329, 576, 320, 0, 0);
        s_background.addImage(2, 0, 1, 345, 576, 320, 0, 0);
        s_background.addImage(3, 1, 1, 1, 576, 320, 0, 0);
    var s_ui_background_bejeweled = new tbImage("s_ui_background_bejeweled", GameStaticTexture, 1, 140, 592, 0, 0, 140, 592, 1);
        s_ui_background_bejeweled.addImage(0, 0, 761, 1, 138, 578, 1, 11);

//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Fonts
//******************************************************
//******************************************************
//******************************************************


//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Vector Fonts
//******************************************************
//******************************************************
//******************************************************

    var f_defaultLoader = new tbVectorFontLoader("f_default", "fonts/f_default.woff", "fonts/f_default.ttf", "fonts");
    var f_default = new tbVectorFont("f_default", "Arial");
       f_default.setFontSize(12);
       f_default.setFill(true);
       f_default.setFillColor("Black");
       f_default.setFillAlpha(100/100);
       f_default.setStroke(false);
       f_default.setStrokeColor("Black");
       f_default.setStrokeWidth(1);
       f_default.setStrokeJoin("miter");
       f_default.setStrokeAlpha(100/100);
       f_default.setStrokeBehind(false);
       f_default.setAlign("left");
       f_default.setBaseline("top");
       f_default.setOffsetFactor(0/100);
       f_default.setLineSpacing(0);
    var ff_opensans_extraboldLoader = new tbVectorFontLoader("ff_opensans_extrabold", "fonts/ff_opensans_extrabold.woff", "fonts/ff_opensans_extrabold.ttf", "fonts");
    var ff_dimbo_regularLoader = new tbVectorFontLoader("ff_dimbo_regular", "fonts/ff_dimbo_regular.woff", "fonts/ff_dimbo_regular.ttf", "fonts");
    var ff_opensans_boldLoader = new tbVectorFontLoader("ff_opensans_bold", "fonts/ff_opensans_bold.woff", "fonts/ff_opensans_bold.ttf", "fonts");
    var ff_opensans_bolditalicLoader = new tbVectorFontLoader("ff_opensans_bolditalic", "fonts/ff_opensans_bolditalic.woff", "fonts/ff_opensans_bolditalic.ttf", "fonts");
    var f_game_ui_tiny = new tbVectorFont("ff_opensans_bold", "Arial");
       f_game_ui_tiny.setFontSize(11);
       f_game_ui_tiny.setFill(true);
       f_game_ui_tiny.setFillColor("#799EC5");
       f_game_ui_tiny.setFillAlpha(100/100);
       f_game_ui_tiny.setStroke(false);
       f_game_ui_tiny.setStrokeColor("White");
       f_game_ui_tiny.setStrokeWidth(1);
       f_game_ui_tiny.setStrokeJoin("miter");
       f_game_ui_tiny.setStrokeAlpha(100/100);
       f_game_ui_tiny.setStrokeBehind(false);
       f_game_ui_tiny.setAlign("center");
       f_game_ui_tiny.setBaseline("middle");
       f_game_ui_tiny.setOffsetFactor(0/100);
       f_game_ui_tiny.setLineSpacing(0);
    var f_game_ui = new tbVectorFont("ff_opensans_bold", "Arial");
       f_game_ui.setFontSize(23);
       f_game_ui.setFill(true);
       f_game_ui.setFillColor("#799EC5");
       f_game_ui.setFillAlpha(100/100);
       f_game_ui.setStroke(false);
       f_game_ui.setStrokeColor("Black");
       f_game_ui.setStrokeWidth(1);
       f_game_ui.setStrokeJoin("miter");
       f_game_ui.setStrokeAlpha(100/100);
       f_game_ui.setStrokeBehind(false);
       f_game_ui.setAlign("center");
       f_game_ui.setBaseline("middle");
       f_game_ui.setOffsetFactor(0/100);
       f_game_ui.setLineSpacing(0);
    var f_game_ui_large = new tbVectorFont("ff_opensans_bolditalic", "Arial");
       f_game_ui_large.setFontSize(52);
       f_game_ui_large.setFill(true);
       f_game_ui_large.setFillColor("#172348");
       f_game_ui_large.setFillAlpha(100/100);
       f_game_ui_large.setStroke(false);
       f_game_ui_large.setStrokeColor("Black");
       f_game_ui_large.setStrokeWidth(1);
       f_game_ui_large.setStrokeJoin("miter");
       f_game_ui_large.setStrokeAlpha(100/100);
       f_game_ui_large.setStrokeBehind(false);
       f_game_ui_large.setAlign("center");
       f_game_ui_large.setBaseline("middle");
       f_game_ui_large.setOffsetFactor(0/100);
       f_game_ui_large.setLineSpacing(0);
    var floaterFontFaceLoader = new tbVectorFontLoader("floaterFontFace", "fonts/floaterFontFace.woff", "fonts/floaterFontFace.ttf", "fonts");
    var floaterNumberFontFaceLoader = new tbVectorFontLoader("floaterNumberFontFace", "fonts/floaterNumberFontFace.woff", "fonts/floaterNumberFontFace.ttf", "fonts");
    var floaterFontText1 = new tbVectorFont("floaterFontFace", "Arial");
       floaterFontText1.setFontSize(24);
       floaterFontText1.setFontWeight("normal");
       floaterFontText1.setFill(true);
       floaterFontText1.setFillColor("#FFDE00");
       floaterFontText1.setFillAlpha(100/100);
       floaterFontText1.setStroke(true);
       floaterFontText1.setStrokeColor("#6F1F00");
       floaterFontText1.setStrokeWidth(4);
       floaterFontText1.setStrokeJoin("miter");
       floaterFontText1.setStrokeAlpha(100/100);
       floaterFontText1.setStrokeBehind(true);
       floaterFontText1.setShadow(true, "rgba(57,0,0,0.46)", 0, 4, 2);
       floaterFontText1.setAlign("left");
       floaterFontText1.setBaseline("top");
       floaterFontText1.setOffsetFactor(0/100);
       floaterFontText1.setLineSpacing(0);
    var floaterFontText2 = new tbVectorFont("floaterFontFace", "Arial");
       floaterFontText2.setFontSize(28);
       floaterFontText2.setFontWeight("normal");
       floaterFontText2.setFill(true);
       floaterFontText2.setFillGradient(true, 2, ["#FFF600", "#00DB48", "blue"], 65/100, 2/100);
       floaterFontText2.setFillAlpha(100/100);
       floaterFontText2.setStroke(true);
       floaterFontText2.setStrokeColor("#073400");
       floaterFontText2.setStrokeWidth(4);
       floaterFontText2.setStrokeJoin("miter");
       floaterFontText2.setStrokeAlpha(100/100);
       floaterFontText2.setStrokeBehind(true);
       floaterFontText2.setShadow(true, "rgba(0,57,43,0.47)", 0, 4, 2);
       floaterFontText2.setAlign("left");
       floaterFontText2.setBaseline("top");
       floaterFontText2.setOffsetFactor(0/100);
       floaterFontText2.setLineSpacing(0);
    var floaterFontText3 = new tbVectorFont("floaterFontFace", "Arial");
       floaterFontText3.setFontSize(30);
       floaterFontText3.setFontWeight("normal");
       floaterFontText3.setFill(true);
       floaterFontText3.setFillGradient(true, 3, ["#FFF600", "#FF8236", "#FF0096"], 71/100, -10/100);
       floaterFontText3.setFillAlpha(100/100);
       floaterFontText3.setStroke(true);
       floaterFontText3.setStrokeColor("#4F0027");
       floaterFontText3.setStrokeWidth(4);
       floaterFontText3.setStrokeJoin("miter");
       floaterFontText3.setStrokeAlpha(100/100);
       floaterFontText3.setStrokeBehind(true);
       floaterFontText3.setShadow(true, "rgba(41,0,0,0.48)", 0, 5, 2);
       floaterFontText3.setAlign("left");
       floaterFontText3.setBaseline("top");
       floaterFontText3.setOffsetFactor(0/100);
       floaterFontText3.setLineSpacing(0);
    var floaterFontText4 = new tbVectorFont("floaterFontFace", "Arial");
       floaterFontText4.setFontSize(34);
       floaterFontText4.setFontWeight("normal");
       floaterFontText4.setFill(true);
       floaterFontText4.setFillGradient(true, 3, ["#00FCFF", "#893DFB", "#FF00E4"], 72/100, -4/100);
       floaterFontText4.setFillAlpha(100/100);
       floaterFontText4.setStroke(true);
       floaterFontText4.setStrokeColor("#001637");
       floaterFontText4.setStrokeWidth(4);
       floaterFontText4.setStrokeJoin("miter");
       floaterFontText4.setStrokeAlpha(100/100);
       floaterFontText4.setStrokeBehind(true);
       floaterFontText4.setShadow(true, "rgba(0,35,75,0.49)", 0, 6, 2);
       floaterFontText4.setAlign("left");
       floaterFontText4.setBaseline("top");
       floaterFontText4.setOffsetFactor(0/100);
       floaterFontText4.setLineSpacing(0);
    var floaterFontNumberPositive = new tbVectorFont("floaterNumberFontFace", "Arial");
       floaterFontNumberPositive.setFontSize(30);
       floaterFontNumberPositive.setFill(true);
       floaterFontNumberPositive.setFillColor("White");
       floaterFontNumberPositive.setFillAlpha(100/100);
       floaterFontNumberPositive.setStroke(true);
       floaterFontNumberPositive.setStrokeColor("#00106F");
       floaterFontNumberPositive.setStrokeWidth(2);
       floaterFontNumberPositive.setStrokeJoin("miter");
       floaterFontNumberPositive.setStrokeAlpha(100/100);
       floaterFontNumberPositive.setStrokeBehind(false);
       floaterFontNumberPositive.setShadow(true, "rgba(0,4,57,0.51)", 0, 4, 2);
       floaterFontNumberPositive.setAlign("left");
       floaterFontNumberPositive.setBaseline("top");
       floaterFontNumberPositive.setOffsetFactor(0/100);
       floaterFontNumberPositive.setLineSpacing(0);
    var floaterFontNumberNegative = new tbVectorFont("floaterNumberFontFace", "Arial");
       floaterFontNumberNegative.setFontSize(30);
       floaterFontNumberNegative.setFontWeight("normal");
       floaterFontNumberNegative.setFill(true);
       floaterFontNumberNegative.setFillColor("#FF1E00");
       floaterFontNumberNegative.setFillAlpha(100/100);
       floaterFontNumberNegative.setStroke(true);
       floaterFontNumberNegative.setStrokeColor("#3F0000");
       floaterFontNumberNegative.setStrokeWidth(2);
       floaterFontNumberNegative.setStrokeJoin("miter");
       floaterFontNumberNegative.setStrokeAlpha(100/100);
       floaterFontNumberNegative.setStrokeBehind(false);
       floaterFontNumberNegative.setShadow(true, "rgba(57,0,0,0.49)", 0, 4, 2);
       floaterFontNumberNegative.setAlign("left");
       floaterFontNumberNegative.setBaseline("top");
       floaterFontNumberNegative.setOffsetFactor(0/100);
       floaterFontNumberNegative.setLineSpacing(0);
    var f_themeDefaultLoader = new tbVectorFontLoader("f_themeDefault", "fonts/f_themeDefault.woff", "fonts/f_themeDefault.ttf", "fonts");
    var f_themeDefault = new tbVectorFont("f_themeDefault", "Arial");
       f_themeDefault.setFontSize(40);
       f_themeDefault.setFill(true);
       f_themeDefault.setFillColor("Black");
       f_themeDefault.setFillAlpha(100/100);
       f_themeDefault.setStroke(false);
       f_themeDefault.setStrokeColor("White");
       f_themeDefault.setStrokeWidth(5);
       f_themeDefault.setStrokeJoin("miter");
       f_themeDefault.setStrokeAlpha(100/100);
       f_themeDefault.setStrokeBehind(true);
       f_themeDefault.setAlign("left");
       f_themeDefault.setBaseline("top");
       f_themeDefault.setOffsetFactor(0/100);
       f_themeDefault.setLineSpacing(0);
    var f_timeBonusLoader = new tbVectorFontLoader("f_timeBonus", "fonts/f_timeBonus.woff", "fonts/f_timeBonus.ttf", "fonts");
    var f_timeBonus = new tbVectorFont("f_timeBonus", "Arial");
       f_timeBonus.setFontSize(35);
       f_timeBonus.setFontWeight("bold");
       f_timeBonus.setFontCase("upper");
       f_timeBonus.setFill(true);
       f_timeBonus.setFillColor("Black");
       f_timeBonus.setFillAlpha(100/100);
       f_timeBonus.setStroke(true);
       f_timeBonus.setStrokeColor("White");
       f_timeBonus.setStrokeWidth(4);
       f_timeBonus.setStrokeJoin("miter");
       f_timeBonus.setStrokeAlpha(100/100);
       f_timeBonus.setStrokeBehind(true);
       f_timeBonus.setAlign("center");
       f_timeBonus.setBaseline("bottom");
       f_timeBonus.setOffsetFactor(0/100);
       f_timeBonus.setLineSpacing(0);
    var f_floaters = new tbVectorFont("floaterFontFace", "Arial");
       f_floaters.setFontSize(12);
       f_floaters.setFill(true);
       f_floaters.setFillColor("Black");
       f_floaters.setFillAlpha(100/100);
       f_floaters.setStroke(false);
       f_floaters.setStrokeColor("Black");
       f_floaters.setStrokeWidth(1);
       f_floaters.setStrokeJoin("miter");
       f_floaters.setStrokeAlpha(100/100);
       f_floaters.setStrokeBehind(false);
       f_floaters.setAlign("left");
       f_floaters.setBaseline("top");
       f_floaters.setOffsetFactor(0/100);
       f_floaters.setLineSpacing(0);
    var f_stage_start = new tbVectorFont("f_timeBonus", "Arial");
       f_stage_start.setFontSize(72);
       f_stage_start.setFill(true);
       f_stage_start.setFillColor("White");
       f_stage_start.setFillAlpha(100/100);
       f_stage_start.setStroke(true);
       f_stage_start.setStrokeColor("#FF0080");
       f_stage_start.setStrokeWidth(6);
       f_stage_start.setStrokeJoin("miter");
       f_stage_start.setStrokeAlpha(100/100);
       f_stage_start.setStrokeBehind(true);
       f_stage_start.setAlign("left");
       f_stage_start.setBaseline("top");
       f_stage_start.setOffsetFactor(0/100);
       f_stage_start.setLineSpacing(0);
    var f_stage_end = new tbVectorFont("f_timeBonus", "Arial");
       f_stage_end.setFontSize(64);
       f_stage_end.setFill(true);
       f_stage_end.setFillColor("#F5FF02");
       f_stage_end.setFillAlpha(100/100);
       f_stage_end.setStroke(true);
       f_stage_end.setStrokeColor("#FF0080");
       f_stage_end.setStrokeWidth(6);
       f_stage_end.setStrokeJoin("miter");
       f_stage_end.setStrokeAlpha(100/100);
       f_stage_end.setStrokeBehind(true);
       f_stage_end.setAlign("left");
       f_stage_end.setBaseline("top");
       f_stage_end.setOffsetFactor(0/100);
       f_stage_end.setLineSpacing(0);
    var f_stage_fail = new tbVectorFont("f_timeBonus", "Arial");
       f_stage_fail.setFontSize(72);
       f_stage_fail.setFill(true);
       f_stage_fail.setFillColor("#A91B80");
       f_stage_fail.setFillAlpha(100/100);
       f_stage_fail.setStroke(true);
       f_stage_fail.setStrokeColor("White");
       f_stage_fail.setStrokeWidth(6);
       f_stage_fail.setStrokeJoin("miter");
       f_stage_fail.setStrokeAlpha(100/100);
       f_stage_fail.setStrokeBehind(true);
       f_stage_fail.setAlign("left");
       f_stage_fail.setBaseline("top");
       f_stage_fail.setOffsetFactor(0/100);
       f_stage_fail.setLineSpacing(0);

//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Audio Sprites
//******************************************************
//******************************************************
//******************************************************

    var as_music = new tbAudioSprite("as_music", "audio/as_music.mp3", "audio/as_music.ogg", "audio_music");
    var audioSprite = new tbAudioSprite("audioSprite", "audio/audioSprite.mp3", "audio/audioSprite.ogg", "audio");

//******************************************************
//******************************************************
//******************************************************
//*****    Definition of the Sound Resources
//******************************************************
//******************************************************
//******************************************************

    var a_music = new tbSound("a_music", as_music, 0, 29538, 40/100, 1, ["music"]);
    var a_match_01 = new tbSound("a_match_01", audioSprite, 0, 562, 100/100, 10, ["game"]);
    var a_match_02 = new tbSound("a_match_02", audioSprite, 2000, 562, 100/100, 10, ["game"]);
    var a_match_03 = new tbSound("a_match_03", audioSprite, 4000, 562, 100/100, 10, ["game"]);
    var a_match_04 = new tbSound("a_match_04", audioSprite, 6000, 562, 100/100, 10, ["game"]);
    var a_combo_01 = new tbSound("a_combo_01", audioSprite, 8000, 1190, 100/100, 10, ["game"]);
    var a_combo_02 = new tbSound("a_combo_02", audioSprite, 11000, 1212, 100/100, 10, ["game"]);
    var a_combo_03 = new tbSound("a_combo_03", audioSprite, 14000, 1156, 100/100, 10, ["game"]);
    var a_combo_04 = new tbSound("a_combo_04", audioSprite, 17000, 1272, 100/100, 10, ["game"]);
    var a_bomb_creation = new tbSound("a_bomb_creation", audioSprite, 20000, 741, 100/100, 10, ["game"]);
    var a_bomb_used = new tbSound("a_bomb_used", audioSprite, 22000, 1001, 100/100, 10, ["game"]);
    var a_lightning_creation = new tbSound("a_lightning_creation", audioSprite, 25000, 686, 100/100, 10, ["game"]);
    var a_lightning_used = new tbSound("a_lightning_used", audioSprite, 27000, 1353, 100/100, 10, ["game"]);
    var a_superLightning_creation = new tbSound("a_superLightning_creation", audioSprite, 30000, 916, 100/100, 10, ["game"]);
    var a_superLightning_used = new tbSound("a_superLightning_used", audioSprite, 32000, 1759, 100/100, 10, ["game"]);
    var a_gameui_timewarning_second = new tbSound("a_gameui_timewarning_second", audioSprite, 35000, 246, 100/100, 10, ["game"]);
    var a_levelStart = new tbSound("a_levelStart", audioSprite, 37000, 1002, 100/100, 10, ["sfx"]);
    var a_levelComplete = new tbSound("a_levelComplete", audioSprite, 40000, 1002, 100/100, 10, ["sfx"]);
    var a_mouseDown = new tbSound("a_mouseDown", audioSprite, 43000, 471, 100/100, 10, ["sfx"]);
    var a_levelend_star_01 = new tbSound("a_levelend_star_01", audioSprite, 45000, 1161, 100/100, 10, ["sfx"]);
    var a_levelend_star_02 = new tbSound("a_levelend_star_02", audioSprite, 48000, 1070, 100/100, 10, ["sfx"]);
    var a_levelend_star_03 = new tbSound("a_levelend_star_03", audioSprite, 51000, 1039, 100/100, 10, ["sfx"]);
    var a_levelend_fail = new tbSound("a_levelend_fail", audioSprite, 54000, 1572, 100/100, 10, ["sfx"]);
    var a_levelend_score_counter = new tbSound("a_levelend_score_counter", audioSprite, 57000, 54, 100/100, 10, ["sfx"]);
    var a_levelend_score_end = new tbSound("a_levelend_score_end", audioSprite, 59000, 888, 100/100, 10, ["sfx"]);
    var a_medal = new tbSound("a_medal", audioSprite, 61000, 1225, 100/100, 10, ["sfx"]);

//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//*****    The Game Scripts
//******************************************************
//******************************************************
//******************************************************
//******************************************************
//******************************************************


//******************************************************
//*****    Script Resource: framework_dictionary
//******************************************************

// **********************************************
// Localization file for product: framework dictionary
//
// created on 18-3-2016 10:34:02
// **********************************************

/*exported dictionaryVar*/
var dictionaryVar = dictionaryVar || {};
dictionaryVar["nl-nl"] = dictionaryVar["nl-nl"] || {};
dictionaryVar["nl-nl"]["loadingScreenLoading"] = "Laden...";
dictionaryVar["nl-nl"]["startScreenPlay"] = "SPELEN";
dictionaryVar["nl-nl"]["levelMapScreenTotalScore"] = "Totale score";
dictionaryVar["nl-nl"]["levelEndScreenTitle_level"] = "Level <VALUE>";
dictionaryVar["nl-nl"]["levelEndScreenTitle_difficulty"] = "Goed Gedaan!";
dictionaryVar["nl-nl"]["levelEndScreenTitle_endless"] = "Level <VALUE>";
dictionaryVar["nl-nl"]["levelEndScreenTotalScore"] = "Totale score";
dictionaryVar["nl-nl"]["levelEndScreenSubTitle_levelFailed"] = "Level niet gehaald";
dictionaryVar["nl-nl"]["levelEndScreenTimeLeft"] = "Tijd over";
dictionaryVar["nl-nl"]["levelEndScreenTimeBonus"] = "Tijdbonus";
dictionaryVar["nl-nl"]["levelEndScreenHighScore"] = "High score";
dictionaryVar["nl-nl"]["optionsStartScreen"] = "Hoofdmenu";
dictionaryVar["nl-nl"]["optionsQuit"] = "Stop";
dictionaryVar["nl-nl"]["optionsResume"] = "Terug naar spel";
dictionaryVar["nl-nl"]["optionsTutorial"] = "Speluitleg";
dictionaryVar["nl-nl"]["optionsHighScore"] = "High scores";
dictionaryVar["nl-nl"]["optionsMoreGames"] = "Meer Spellen";
dictionaryVar["nl-nl"]["optionsDifficulty_easy"] = "Makkelijk";
dictionaryVar["nl-nl"]["optionsDifficulty_medium"] = "Gemiddeld";
dictionaryVar["nl-nl"]["optionsDifficulty_hard"] = "Moeilijk";
dictionaryVar["nl-nl"]["optionsMusic_on"] = "Aan";
dictionaryVar["nl-nl"]["optionsMusic_off"] = "Uit";
dictionaryVar["nl-nl"]["optionsSFX_on"] = "Aan";
dictionaryVar["nl-nl"]["optionsSFX_off"] = "Uit";
dictionaryVar["nl-nl"]["optionsLang_en-us"] = "Engels (US)";
dictionaryVar["nl-nl"]["optionsLang_en-gb"] = "Engels (GB)";
dictionaryVar["nl-nl"]["optionsLang_nl-nl"] = "Nederlands";
dictionaryVar["nl-nl"]["gameEndScreenTitle"] = "Gefeliciteerd!\nJe hebt gewonnen.";
dictionaryVar["nl-nl"]["gameEndScreenBtnText"] = "Ga verder";
dictionaryVar["nl-nl"]["optionsTitle"] = "Instellingen";
dictionaryVar["nl-nl"]["optionsQuitConfirmationText"] = "Pas op!\n\nAls je nu stopt verlies je alle voortgang in dit level. Weet je zeker dat je wilt stoppen?";
dictionaryVar["nl-nl"]["optionsQuitConfirmBtn_No"] = "Nee";
dictionaryVar["nl-nl"]["optionsQuitConfirmBtn_Yes"] = "Ja, ik weet het zeker";
dictionaryVar["nl-nl"]["levelMapScreenTitle"] = "Kies een level";
dictionaryVar["nl-nl"]["optionsRestartConfirmationText"] = "Pas op!\n\nAls je nu herstart verlies je alle voortgang in dit level. Weet je zeker dat je wilt herstarten?";
dictionaryVar["nl-nl"]["optionsRestart"] = "Herstart";
dictionaryVar["nl-nl"]["optionsSFXBig_on"] = "Geluid aan";
dictionaryVar["nl-nl"]["optionsSFXBig_off"] = "Geluid uit";
dictionaryVar["nl-nl"]["optionsAbout_title"] = "Over ons";
dictionaryVar["nl-nl"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\nCopyright © 2016";
dictionaryVar["nl-nl"]["optionsAbout_backBtn"] = "Terug";
dictionaryVar["nl-nl"]["optionsAbout_version"] = "versie:";
dictionaryVar["nl-nl"]["optionsAbout"] = "Over ons";
dictionaryVar["nl-nl"]["levelEndScreenMedal"] = "VERBETERD!";
dictionaryVar["nl-nl"]["startScreenQuestionaire"] = "Wat vind jij?";
dictionaryVar["nl-nl"]["levelMapScreenWorld_0"] = "Kies een level";
dictionaryVar["nl-nl"]["startScreenByTinglyGames"] = "door: CoolGames";
dictionaryVar["nl-nl"]["optionsLang_de-de"] = "Duits";
dictionaryVar["nl-nl"]["optionsLang_tr-tr"] = "Turks";
dictionaryVar["nl-nl"]["optionsAbout_header"] = "Ontwikkeld door:";
dictionaryVar["nl-nl"]["levelEndScreenViewHighscoreBtn"] = "Scores bekijken";
dictionaryVar["nl-nl"]["levelEndScreenSubmitHighscoreBtn"] = "Score verzenden";
dictionaryVar["nl-nl"]["challengeStartScreenTitle_challengee_friend"] = "Je bent uitgedaagd door:";
dictionaryVar["nl-nl"]["challengeStartTextScore"] = "Punten van <NAME>:";
dictionaryVar["nl-nl"]["challengeStartTextTime"] = "Tijd van <NAME>:";
dictionaryVar["nl-nl"]["challengeStartScreenToWin"] = "Te winnen aantal Fairplay munten:";
dictionaryVar["nl-nl"]["challengeEndScreenWinnings"] = "Je hebt <AMOUNT> Fairplay munten gewonnen!";
dictionaryVar["nl-nl"]["challengeEndScreenOutcomeMessage_WON"] = "Je hebt de uitdaging gewonnen!";
dictionaryVar["nl-nl"]["challengeEndScreenOutcomeMessage_LOST"] = "Je hebt de uitdaging verloren.";
dictionaryVar["nl-nl"]["challengeEndScreenOutcomeMessage_TIED"] = "Jullie hebben gelijk gespeeld.";
dictionaryVar["nl-nl"]["challengeCancelConfirmText"] = "Je staat op het punt de uitdaging te annuleren. Je inzet wordt teruggestort minus de uitdagingskosten. Weet je zeker dat je de uitdaging wilt annuleren? ";
dictionaryVar["nl-nl"]["challengeCancelConfirmBtn_yes"] = "Ja";
dictionaryVar["nl-nl"]["challengeCancelConfirmBtn_no"] = "Nee";
dictionaryVar["nl-nl"]["challengeEndScreensBtn_submit"] = "Verstuur uitdaging";
dictionaryVar["nl-nl"]["challengeEndScreenBtn_cancel"] = "Annuleer uitdaging";
dictionaryVar["nl-nl"]["challengeEndScreenName_you"] = "Jij";
dictionaryVar["nl-nl"]["challengeEndScreenChallengeSend_error"] = "Er is een fout opgetreden bij het versturen van de uitdaging. Probeer het later nog een keer.";
dictionaryVar["nl-nl"]["challengeEndScreenChallengeSend_success"] = "Je uitdaging is verstuurd!";
dictionaryVar["nl-nl"]["challengeCancelMessage_error"] = "Er is een fout opgetreden bij het annuleren van de uitdaging. Probeer het later nog een keer.";
dictionaryVar["nl-nl"]["challengeCancelMessage_success"] = "De uitdaging is geannuleerd.";
dictionaryVar["nl-nl"]["challengeEndScreenScoreSend_error"] = "Er is een fout opgetreden tijdens de communicatie met de server. Probeer het later nog een keer.";
dictionaryVar["nl-nl"]["challengeStartScreenTitle_challengee_stranger"] = "Jouw tegenstander:";
dictionaryVar["nl-nl"]["challengeStartScreenTitle_challenger_friend"] = "Jouw tegenstander:";
dictionaryVar["nl-nl"]["challengeStartScreenTitle_challenger_stranger"] = "Je zet een uitdaging voor:";
dictionaryVar["nl-nl"]["challengeStartTextTime_challenger"] = "Speel het spel en zet een tijd neer.";
dictionaryVar["nl-nl"]["challengeStartTextScore_challenger"] = "Speel het spel en zet een score neer.";
dictionaryVar["nl-nl"]["challengeForfeitConfirmText"] = "Je staat op het punt de uitdaging op te geven. Weet je zeker dat je dit wilt doen?";
dictionaryVar["nl-nl"]["challengeForfeitConfirmBtn_yes"] = "Ja";
dictionaryVar["nl-nl"]["challengeForfeitConfirmBtn_no"] = "Nee";
dictionaryVar["nl-nl"]["challengeForfeitMessage_success"] = "Je hebt de uitdaging opgegeven.";
dictionaryVar["nl-nl"]["challengeForfeitMessage_error"] = "Er is een fout opgetreden tijdens het opgeven van de uitdaging. Probeer het later nog een keer.";
dictionaryVar["nl-nl"]["optionsChallengeForfeit"] = "Geef op";
dictionaryVar["nl-nl"]["optionsChallengeCancel"] = "Stop";
dictionaryVar["nl-nl"]["challengeLoadingError_notValid"] = "Sorry, deze uitdaging kan niet meer gespeeld worden.";
dictionaryVar["nl-nl"]["challengeLoadingError_notStarted"] = "Kan de server niet bereiken. Probeer het later nog een keer.";
dictionaryVar["nl-nl"]["levelEndScreenHighScore_time"] = "Beste tijd:";
dictionaryVar["nl-nl"]["levelEndScreenTotalScore_time"] = "Totale tijd:";
dictionaryVar["nl-nl"]["optionsLang_fr-fr"] = "Frans";
dictionaryVar["nl-nl"]["optionsLang_ko-kr"] = "Koreaans";
dictionaryVar["nl-nl"]["optionsLang_ar-eg"] = "Arabisch";
dictionaryVar["nl-nl"]["optionsLang_es-es"] = "Spaans";
dictionaryVar["nl-nl"]["optionsLang_pt-br"] = "Braziliaans-Portugees";
dictionaryVar["nl-nl"]["optionsLang_ru-ru"] = "Russisch";
dictionaryVar["nl-nl"]["optionsExit"] = "Stoppen";
dictionaryVar["nl-nl"]["levelEndScreenTotalScore_number"] = "Totale score:";
dictionaryVar["nl-nl"]["levelEndScreenHighScore_number"] = "Topscore:";
dictionaryVar["nl-nl"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> heeft 72 uur om de uitdaging aan te nemen of te weigeren. Als <NAME> je uitdaging weigert of niet accepteert binnen 72 uur worden je inzet en uitdagingskosten teruggestort.";
dictionaryVar["nl-nl"]["challengeEndScreenChallengeSend_submessage_stranger"] = "Als niemand binnen 72 uur je uitdaging accepteert, worden je inzet en uitdagingskosten teruggestort.";
dictionaryVar["nl-nl"]["challengeForfeitMessage_winnings"] = "<NAME> heeft <AMOUNT> Fairplay munten gewonnen!";
dictionaryVar["nl-nl"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["nl-nl"]["optionsLang_jp-jp"] = "Japans";
dictionaryVar["en-us"] = dictionaryVar["en-us"] || {};
dictionaryVar["en-us"]["loadingScreenLoading"] = "Loading...";
dictionaryVar["en-us"]["startScreenPlay"] = "PLAY";
dictionaryVar["en-us"]["levelMapScreenTotalScore"] = "Total score";
dictionaryVar["en-us"]["levelEndScreenTitle_level"] = "Level <VALUE>";
dictionaryVar["en-us"]["levelEndScreenTitle_difficulty"] = "Well done!";
dictionaryVar["en-us"]["levelEndScreenTitle_endless"] = "Stage <VALUE>";
dictionaryVar["en-us"]["levelEndScreenTotalScore"] = "Total score";
dictionaryVar["en-us"]["levelEndScreenSubTitle_levelFailed"] = "Level failed";
dictionaryVar["en-us"]["levelEndScreenTimeLeft"] = "Time remaining";
dictionaryVar["en-us"]["levelEndScreenTimeBonus"] = "Time bonus";
dictionaryVar["en-us"]["levelEndScreenHighScore"] = "High score";
dictionaryVar["en-us"]["optionsStartScreen"] = "Main menu";
dictionaryVar["en-us"]["optionsQuit"] = "Quit";
dictionaryVar["en-us"]["optionsResume"] = "Resume";
dictionaryVar["en-us"]["optionsTutorial"] = "How to play";
dictionaryVar["en-us"]["optionsHighScore"] = "High scores";
dictionaryVar["en-us"]["optionsMoreGames"] = "More Games";
dictionaryVar["en-us"]["optionsDifficulty_easy"] = "Easy";
dictionaryVar["en-us"]["optionsDifficulty_medium"] = "Medium";
dictionaryVar["en-us"]["optionsDifficulty_hard"] = "Difficult";
dictionaryVar["en-us"]["optionsMusic_on"] = "On";
dictionaryVar["en-us"]["optionsMusic_off"] = "Off";
dictionaryVar["en-us"]["optionsSFX_on"] = "On";
dictionaryVar["en-us"]["optionsSFX_off"] = "Off";
dictionaryVar["en-us"]["optionsLang_en-us"] = "English (US)";
dictionaryVar["en-us"]["optionsLang_en-gb"] = "English (GB)";
dictionaryVar["en-us"]["optionsLang_nl-nl"] = "Dutch";
dictionaryVar["en-us"]["gameEndScreenTitle"] = "Congratulations!\nYou have completed the game.";
dictionaryVar["en-us"]["gameEndScreenBtnText"] = "Continue";
dictionaryVar["en-us"]["optionsTitle"] = "Settings";
dictionaryVar["en-us"]["optionsQuitConfirmationText"] = "Attention!\n\nIf you quit now you will lose all progress made during this level. Are you sure you want to quit?";
dictionaryVar["en-us"]["optionsQuitConfirmBtn_No"] = "No";
dictionaryVar["en-us"]["optionsQuitConfirmBtn_Yes"] = "Yes, I'm sure";
dictionaryVar["en-us"]["levelMapScreenTitle"] = "Select a level";
dictionaryVar["en-us"]["optionsRestartConfirmationText"] = "Attention!\n\nIf you restart now you will lose all progress made during this level. Are you sure you want to restart?";
dictionaryVar["en-us"]["optionsRestart"] = "Restart";
dictionaryVar["en-us"]["optionsSFXBig_on"] = "Sound on";
dictionaryVar["en-us"]["optionsSFXBig_off"] = "Sound off";
dictionaryVar["en-us"]["optionsAbout_title"] = "About";
dictionaryVar["en-us"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["en-us"]["optionsAbout_backBtn"] = "Back";
dictionaryVar["en-us"]["optionsAbout_version"] = "version:";
dictionaryVar["en-us"]["optionsAbout"] = "About";
dictionaryVar["en-us"]["levelEndScreenMedal"] = "IMPROVED!";
dictionaryVar["en-us"]["startScreenQuestionaire"] = "What do you think?";
dictionaryVar["en-us"]["levelMapScreenWorld_0"] = "Select a level";
dictionaryVar["en-us"]["startScreenByTinglyGames"] = "by: CoolGames";
dictionaryVar["en-us"]["optionsLang_de-de"] = "German";
dictionaryVar["en-us"]["optionsLang_tr-tr"] = "Turkish";
dictionaryVar["en-us"]["optionsAbout_header"] = "Developed by:";
dictionaryVar["en-us"]["levelEndScreenViewHighscoreBtn"] = "View scores";
dictionaryVar["en-us"]["levelEndScreenSubmitHighscoreBtn"] = "Submit score";
dictionaryVar["en-us"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["en-us"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["en-us"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["en-us"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["en-us"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["en-us"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["en-us"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["en-us"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["en-us"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["en-us"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["en-us"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["en-us"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["en-us"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["en-us"]["challengeEndScreenName_you"] = "You";
dictionaryVar["en-us"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["en-us"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["en-us"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["en-us"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["en-us"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["en-us"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["en-us"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["en-us"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["en-us"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["en-us"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["en-us"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["en-us"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["en-us"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["en-us"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["en-us"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["en-us"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["en-us"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["en-us"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["en-us"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["en-us"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["en-us"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["en-us"]["optionsLang_fr-fr"] = "French";
dictionaryVar["en-us"]["optionsLang_ko-kr"] = "Korean";
dictionaryVar["en-us"]["optionsLang_ar-eg"] = "Arabic";
dictionaryVar["en-us"]["optionsLang_es-es"] = "Spanish";
dictionaryVar["en-us"]["optionsLang_pt-br"] = "Brazilian-Portuguese";
dictionaryVar["en-us"]["optionsLang_ru-ru"] = "Russian";
dictionaryVar["en-us"]["optionsExit"] = "Exit";
dictionaryVar["en-us"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["en-us"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["en-us"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["en-us"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["en-us"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["en-us"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["en-us"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["en-gb"] = dictionaryVar["en-gb"] || {};
dictionaryVar["en-gb"]["loadingScreenLoading"] = "Loading...";
dictionaryVar["en-gb"]["startScreenPlay"] = "PLAY";
dictionaryVar["en-gb"]["levelMapScreenTotalScore"] = "Total score";
dictionaryVar["en-gb"]["levelEndScreenTitle_level"] = "Level <VALUE>";
dictionaryVar["en-gb"]["levelEndScreenTitle_difficulty"] = "Well done!";
dictionaryVar["en-gb"]["levelEndScreenTitle_endless"] = "Stage <VALUE>";
dictionaryVar["en-gb"]["levelEndScreenTotalScore"] = "Total score";
dictionaryVar["en-gb"]["levelEndScreenSubTitle_levelFailed"] = "Level failed";
dictionaryVar["en-gb"]["levelEndScreenTimeLeft"] = "Time remaining";
dictionaryVar["en-gb"]["levelEndScreenTimeBonus"] = "Time bonus";
dictionaryVar["en-gb"]["levelEndScreenHighScore"] = "High score";
dictionaryVar["en-gb"]["optionsStartScreen"] = "Main menu";
dictionaryVar["en-gb"]["optionsQuit"] = "Quit";
dictionaryVar["en-gb"]["optionsResume"] = "Resume";
dictionaryVar["en-gb"]["optionsTutorial"] = "How to play";
dictionaryVar["en-gb"]["optionsHighScore"] = "High scores";
dictionaryVar["en-gb"]["optionsMoreGames"] = "More Games";
dictionaryVar["en-gb"]["optionsDifficulty_easy"] = "Easy";
dictionaryVar["en-gb"]["optionsDifficulty_medium"] = "Medium";
dictionaryVar["en-gb"]["optionsDifficulty_hard"] = "Difficult";
dictionaryVar["en-gb"]["optionsMusic_on"] = "On";
dictionaryVar["en-gb"]["optionsMusic_off"] = "Off";
dictionaryVar["en-gb"]["optionsSFX_on"] = "On";
dictionaryVar["en-gb"]["optionsSFX_off"] = "Off";
dictionaryVar["en-gb"]["optionsLang_en-us"] = "English (US)";
dictionaryVar["en-gb"]["optionsLang_en-gb"] = "English (GB)";
dictionaryVar["en-gb"]["optionsLang_nl-nl"] = "Dutch";
dictionaryVar["en-gb"]["gameEndScreenTitle"] = "Congratulations!\nYou have completed the game.";
dictionaryVar["en-gb"]["gameEndScreenBtnText"] = "Continue";
dictionaryVar["en-gb"]["optionsTitle"] = "Settings";
dictionaryVar["en-gb"]["optionsQuitConfirmationText"] = "Attention!\n\nIf you quit now you will lose all progress made during this level. Are you sure you want to quit?";
dictionaryVar["en-gb"]["optionsQuitConfirmBtn_No"] = "No";
dictionaryVar["en-gb"]["optionsQuitConfirmBtn_Yes"] = "Yes, I'm sure";
dictionaryVar["en-gb"]["levelMapScreenTitle"] = "Select a level";
dictionaryVar["en-gb"]["optionsRestartConfirmationText"] = "Attention!\n\nIf you restart now you will lose all progress made during this level. Are you sure you want to restart?";
dictionaryVar["en-gb"]["optionsRestart"] = "Restart";
dictionaryVar["en-gb"]["optionsSFXBig_on"] = "Sound on";
dictionaryVar["en-gb"]["optionsSFXBig_off"] = "Sound off";
dictionaryVar["en-gb"]["optionsAbout_title"] = "About";
dictionaryVar["en-gb"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["en-gb"]["optionsAbout_backBtn"] = "Back";
dictionaryVar["en-gb"]["optionsAbout_version"] = "version:";
dictionaryVar["en-gb"]["optionsAbout"] = "About";
dictionaryVar["en-gb"]["levelEndScreenMedal"] = "IMPROVED!";
dictionaryVar["en-gb"]["startScreenQuestionaire"] = "What do you think?";
dictionaryVar["en-gb"]["levelMapScreenWorld_0"] = "Select a level";
dictionaryVar["en-gb"]["startScreenByTinglyGames"] = "by: CoolGames";
dictionaryVar["en-gb"]["optionsLang_de-de"] = "German";
dictionaryVar["en-gb"]["optionsLang_tr-tr"] = "Turkish";
dictionaryVar["en-gb"]["optionsAbout_header"] = "Developed by:";
dictionaryVar["en-gb"]["levelEndScreenViewHighscoreBtn"] = "View scores";
dictionaryVar["en-gb"]["levelEndScreenSubmitHighscoreBtn"] = "Submit score";
dictionaryVar["en-gb"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["en-gb"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["en-gb"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["en-gb"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["en-gb"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["en-gb"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["en-gb"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["en-gb"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["en-gb"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["en-gb"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["en-gb"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["en-gb"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["en-gb"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["en-gb"]["challengeEndScreenName_you"] = "You";
dictionaryVar["en-gb"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["en-gb"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["en-gb"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["en-gb"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["en-gb"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["en-gb"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["en-gb"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["en-gb"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["en-gb"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["en-gb"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["en-gb"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you wish to proceed?";
dictionaryVar["en-gb"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["en-gb"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["en-gb"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["en-gb"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["en-gb"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["en-gb"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["en-gb"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["en-gb"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["en-gb"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["en-gb"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["en-gb"]["optionsLang_fr-fr"] = "French";
dictionaryVar["en-gb"]["optionsLang_ko-kr"] = "Korean";
dictionaryVar["en-gb"]["optionsLang_ar-eg"] = "Arabic";
dictionaryVar["en-gb"]["optionsLang_es-es"] = "Spanish";
dictionaryVar["en-gb"]["optionsLang_pt-br"] = "Brazilian-Portuguese";
dictionaryVar["en-gb"]["optionsLang_ru-ru"] = "Russian";
dictionaryVar["en-gb"]["optionsExit"] = "Exit";
dictionaryVar["en-gb"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["en-gb"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["en-gb"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["en-gb"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["en-gb"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["en-gb"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["en-gb"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["de-de"] = dictionaryVar["de-de"] || {};
dictionaryVar["de-de"]["loadingScreenLoading"] = "Laden ...";
dictionaryVar["de-de"]["startScreenPlay"] = "SPIELEN";
dictionaryVar["de-de"]["levelMapScreenTotalScore"] = "Gesamtpunkte";
dictionaryVar["de-de"]["levelEndScreenTitle_level"] = "Level <VALUE>";
dictionaryVar["de-de"]["levelEndScreenTitle_difficulty"] = "Sehr gut!";
dictionaryVar["de-de"]["levelEndScreenTitle_endless"] = "Stufe <VALUE>";
dictionaryVar["de-de"]["levelEndScreenTotalScore"] = "Gesamtpunkte";
dictionaryVar["de-de"]["levelEndScreenSubTitle_levelFailed"] = "Level nicht geschafft";
dictionaryVar["de-de"]["levelEndScreenTimeLeft"] = "Restzeit";
dictionaryVar["de-de"]["levelEndScreenTimeBonus"] = "Zeitbonus";
dictionaryVar["de-de"]["levelEndScreenHighScore"] = "Highscore";
dictionaryVar["de-de"]["optionsStartScreen"] = "Hauptmenü";
dictionaryVar["de-de"]["optionsQuit"] = "Beenden";
dictionaryVar["de-de"]["optionsResume"] = "Weiterspielen";
dictionaryVar["de-de"]["optionsTutorial"] = "So wird gespielt";
dictionaryVar["de-de"]["optionsHighScore"] = "Highscores";
dictionaryVar["de-de"]["optionsMoreGames"] = "Weitere Spiele";
dictionaryVar["de-de"]["optionsDifficulty_easy"] = "Einfach";
dictionaryVar["de-de"]["optionsDifficulty_medium"] = "Mittel";
dictionaryVar["de-de"]["optionsDifficulty_hard"] = "Schwer";
dictionaryVar["de-de"]["optionsMusic_on"] = "EIN";
dictionaryVar["de-de"]["optionsMusic_off"] = "AUS";
dictionaryVar["de-de"]["optionsSFX_on"] = "EIN";
dictionaryVar["de-de"]["optionsSFX_off"] = "AUS";
dictionaryVar["de-de"]["optionsLang_en-us"] = "Englisch (US)";
dictionaryVar["de-de"]["optionsLang_en-gb"] = "Englisch (GB)";
dictionaryVar["de-de"]["optionsLang_nl-nl"] = "Holländisch";
dictionaryVar["de-de"]["gameEndScreenTitle"] = "Glückwunsch!\nDu hast das Spiel abgeschlossen.";
dictionaryVar["de-de"]["gameEndScreenBtnText"] = "Weiter";
dictionaryVar["de-de"]["optionsTitle"] = "Einstellungen";
dictionaryVar["de-de"]["optionsQuitConfirmationText"] = "Achtung!\n\nWenn du jetzt aufhörst, verlierst du alle in diesem Level gemachten Fortschritte. Willst du wirklich aufhören?";
dictionaryVar["de-de"]["optionsQuitConfirmBtn_No"] = "NEIN";
dictionaryVar["de-de"]["optionsQuitConfirmBtn_Yes"] = "Ja, ich bin mir sicher";
dictionaryVar["de-de"]["levelMapScreenTitle"] = "Wähle ein Level";
dictionaryVar["de-de"]["optionsRestartConfirmationText"] = "Achtung!\n\nWenn du jetzt neu startest, verlierst du alle in diesem Level gemachten Fortschritte. Willst du wirklich neu starten?";
dictionaryVar["de-de"]["optionsRestart"] = "Neustart";
dictionaryVar["de-de"]["optionsSFXBig_on"] = "Sound EIN";
dictionaryVar["de-de"]["optionsSFXBig_off"] = "Sound AUS";
dictionaryVar["de-de"]["optionsAbout_title"] = "Über";
dictionaryVar["de-de"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["de-de"]["optionsAbout_backBtn"] = "Zurück";
dictionaryVar["de-de"]["optionsAbout_version"] = "Version:";
dictionaryVar["de-de"]["optionsAbout"] = "Über";
dictionaryVar["de-de"]["levelEndScreenMedal"] = "VERBESSERT!";
dictionaryVar["de-de"]["startScreenQuestionaire"] = "Deine Meinung zählt!";
dictionaryVar["de-de"]["levelMapScreenWorld_0"] = "Wähle ein Level";
dictionaryVar["de-de"]["startScreenByTinglyGames"] = "von: CoolGames";
dictionaryVar["de-de"]["optionsLang_de-de"] = "Deutsch";
dictionaryVar["de-de"]["optionsLang_tr-tr"] = "Türkisch";
dictionaryVar["de-de"]["optionsAbout_header"] = "Entwickelt von:";
dictionaryVar["de-de"]["levelEndScreenViewHighscoreBtn"] = "Punktzahlen ansehen";
dictionaryVar["de-de"]["levelEndScreenSubmitHighscoreBtn"] = "Punktzahl senden";
dictionaryVar["de-de"]["challengeStartScreenTitle_challengee_friend"] = "Dein Gegner:";
dictionaryVar["de-de"]["challengeStartTextScore"] = "Punktzahl von <NAME>:";
dictionaryVar["de-de"]["challengeStartTextTime"] = "Zeit von <NAME>:";
dictionaryVar["de-de"]["challengeStartScreenToWin"] = "Einsatz:";
dictionaryVar["de-de"]["challengeEndScreenWinnings"] = "Du hast <AMOUNT> Fairmünzen gewonnen!";
dictionaryVar["de-de"]["challengeEndScreenOutcomeMessage_WON"] = "Du hast die Partie gewonnen!";
dictionaryVar["de-de"]["challengeEndScreenOutcomeMessage_LOST"] = "Leider hat Dein Gegner die Partie gewonnen.";
dictionaryVar["de-de"]["challengeEndScreenOutcomeMessage_TIED"] = "Gleichstand!";
dictionaryVar["de-de"]["challengeCancelConfirmText"] = "Willst Du Deine Wette wirklich zurückziehen? Dein Wetteinsatz wird Dir zurückgegeben minus die Einsatzgebühr.";
dictionaryVar["de-de"]["challengeCancelConfirmBtn_yes"] = "Ja";
dictionaryVar["de-de"]["challengeCancelConfirmBtn_no"] = "Nein";
dictionaryVar["de-de"]["challengeEndScreensBtn_submit"] = "Herausfordern";
dictionaryVar["de-de"]["challengeEndScreenBtn_cancel"] = "Zurückziehen";
dictionaryVar["de-de"]["challengeEndScreenName_you"] = "Du";
dictionaryVar["de-de"]["challengeEndScreenChallengeSend_error"] = "Leider ist ein Fehler aufgetreten. Probiere es bitte nochmal später.";
dictionaryVar["de-de"]["challengeEndScreenChallengeSend_success"] = "Herausforderung verschickt!";
dictionaryVar["de-de"]["challengeCancelMessage_error"] = "Leider ist ein Fehler aufgetreten. Probiere es bitte nochmal später.";
dictionaryVar["de-de"]["challengeCancelMessage_success"] = "Du hast die Wette erfolgreich zurückgezogen.";
dictionaryVar["de-de"]["challengeEndScreenScoreSend_error"] = "Leider ist ein Fehler aufgetreten. Probiere es bitte nochmal später.";
dictionaryVar["de-de"]["challengeStartScreenTitle_challengee_stranger"] = "Dein Gegner wird:";
dictionaryVar["de-de"]["challengeStartScreenTitle_challenger_friend"] = "Du hast den folgenden Spieler herausgefordert:";
dictionaryVar["de-de"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["de-de"]["challengeStartTextTime_challenger"] = "Spiel um die niedrigste Zeit!";
dictionaryVar["de-de"]["challengeStartTextScore_challenger"] = "Spiel um die hochste Punktzahl!";
dictionaryVar["de-de"]["challengeForfeitConfirmText"] = "Willst du Die Partie wirklich aufgeben?";
dictionaryVar["de-de"]["challengeForfeitConfirmBtn_yes"] = "Ja";
dictionaryVar["de-de"]["challengeForfeitConfirmBtn_no"] = "Nein";
dictionaryVar["de-de"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["de-de"]["challengeForfeitMessage_error"] = "Leider ist ein Fehler aufgetreten. Probiere es bitte nochmal später.";
dictionaryVar["de-de"]["optionsChallengeForfeit"] = "Aufgeben";
dictionaryVar["de-de"]["optionsChallengeCancel"] = "Zurückziehen";
dictionaryVar["de-de"]["challengeLoadingError_notValid"] = "Leider ist diese Partie nicht mehr aktuel.";
dictionaryVar["de-de"]["challengeLoadingError_notStarted"] = "Leider ist ein Fehler aufgetreten. Es konnte keiner Verbindung zum Server hergestellt werden. Versuche es bitte nochmal später.";
dictionaryVar["de-de"]["levelEndScreenHighScore_time"] = "Bestzeit:";
dictionaryVar["de-de"]["levelEndScreenTotalScore_time"] = "Gesamtzeit:";
dictionaryVar["de-de"]["optionsLang_fr-fr"] = "Französisch";
dictionaryVar["de-de"]["optionsLang_ko-kr"] = "Koreanisch";
dictionaryVar["de-de"]["optionsLang_ar-eg"] = "Arabisch";
dictionaryVar["de-de"]["optionsLang_es-es"] = "Spanisch";
dictionaryVar["de-de"]["optionsLang_pt-br"] = "Portugiesisch (Brasilien)";
dictionaryVar["de-de"]["optionsLang_ru-ru"] = "Russisch";
dictionaryVar["de-de"]["optionsExit"] = "Verlassen";
dictionaryVar["de-de"]["levelEndScreenTotalScore_number"] = "Gesamtpunktzahl:";
dictionaryVar["de-de"]["levelEndScreenHighScore_number"] = "Highscore:";
dictionaryVar["de-de"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> hat 72 Stunden um die Wette anzunehmen oder abzulehnen. Sollte <NAME> nicht reagieren oder ablehnen werden Dir Dein Wetteinsatz und die Gebühr zurückerstattet.";
dictionaryVar["de-de"]["challengeEndScreenChallengeSend_submessage_stranger"] = "Als niemanden Deine Herausforderung innerhalb von 72 Stunden annimmt, werden Dir Deinen Wetteinsatz Einsatzgebühr zurückerstattet.";
dictionaryVar["de-de"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["de-de"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["de-de"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["fr-fr"] = dictionaryVar["fr-fr"] || {};
dictionaryVar["fr-fr"]["loadingScreenLoading"] = "Chargement...";
dictionaryVar["fr-fr"]["startScreenPlay"] = "JOUER";
dictionaryVar["fr-fr"]["levelMapScreenTotalScore"] = "Score total";
dictionaryVar["fr-fr"]["levelEndScreenTitle_level"] = "Niveau <VALUE>";
dictionaryVar["fr-fr"]["levelEndScreenTitle_difficulty"] = "Bien joué !";
dictionaryVar["fr-fr"]["levelEndScreenTitle_endless"] = "Scène <VALUE>";
dictionaryVar["fr-fr"]["levelEndScreenTotalScore"] = "Score total";
dictionaryVar["fr-fr"]["levelEndScreenSubTitle_levelFailed"] = "Échec du niveau";
dictionaryVar["fr-fr"]["levelEndScreenTimeLeft"] = "Temps restant";
dictionaryVar["fr-fr"]["levelEndScreenTimeBonus"] = "Bonus de temps";
dictionaryVar["fr-fr"]["levelEndScreenHighScore"] = "Meilleur score";
dictionaryVar["fr-fr"]["optionsStartScreen"] = "Menu principal";
dictionaryVar["fr-fr"]["optionsQuit"] = "Quitter";
dictionaryVar["fr-fr"]["optionsResume"] = "Reprendre";
dictionaryVar["fr-fr"]["optionsTutorial"] = "Comment jouer";
dictionaryVar["fr-fr"]["optionsHighScore"] = "Meilleurs scores";
dictionaryVar["fr-fr"]["optionsMoreGames"] = "Plus de jeux";
dictionaryVar["fr-fr"]["optionsDifficulty_easy"] = "Facile";
dictionaryVar["fr-fr"]["optionsDifficulty_medium"] = "Moyen";
dictionaryVar["fr-fr"]["optionsDifficulty_hard"] = "Difficile";
dictionaryVar["fr-fr"]["optionsMusic_on"] = "Avec";
dictionaryVar["fr-fr"]["optionsMusic_off"] = "Sans";
dictionaryVar["fr-fr"]["optionsSFX_on"] = "Avec";
dictionaryVar["fr-fr"]["optionsSFX_off"] = "Sans";
dictionaryVar["fr-fr"]["optionsLang_en-us"] = "Anglais (US)";
dictionaryVar["fr-fr"]["optionsLang_en-gb"] = "Anglais (UK)";
dictionaryVar["fr-fr"]["optionsLang_nl-nl"] = "Néerlandais";
dictionaryVar["fr-fr"]["gameEndScreenTitle"] = "Félicitations !\nVous avez terminé le jeu.";
dictionaryVar["fr-fr"]["gameEndScreenBtnText"] = "Continuer";
dictionaryVar["fr-fr"]["optionsTitle"] = "Paramètres";
dictionaryVar["fr-fr"]["optionsQuitConfirmationText"] = "Attention !\n\nEn quittant maintenant, vous perdrez votre progression pour le niveau en cours. Quitter quand même ?";
dictionaryVar["fr-fr"]["optionsQuitConfirmBtn_No"] = "Non";
dictionaryVar["fr-fr"]["optionsQuitConfirmBtn_Yes"] = "Oui";
dictionaryVar["fr-fr"]["levelMapScreenTitle"] = "Choisir un niveau";
dictionaryVar["fr-fr"]["optionsRestartConfirmationText"] = "Attention !\n\nEn recommençant maintenant, vous perdrez votre progression pour le niveau en cours. Recommencer quand même ?";
dictionaryVar["fr-fr"]["optionsRestart"] = "Recommencer";
dictionaryVar["fr-fr"]["optionsSFXBig_on"] = "Avec son";
dictionaryVar["fr-fr"]["optionsSFXBig_off"] = "Sans son";
dictionaryVar["fr-fr"]["optionsAbout_title"] = "À propos";
dictionaryVar["fr-fr"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["fr-fr"]["optionsAbout_backBtn"] = "Retour";
dictionaryVar["fr-fr"]["optionsAbout_version"] = "Version :";
dictionaryVar["fr-fr"]["optionsAbout"] = "À propos";
dictionaryVar["fr-fr"]["levelEndScreenMedal"] = "RECORD BATTU !";
dictionaryVar["fr-fr"]["startScreenQuestionaire"] = "Un avis sur le jeu ?";
dictionaryVar["fr-fr"]["levelMapScreenWorld_0"] = "Choisir un niveau";
dictionaryVar["fr-fr"]["startScreenByTinglyGames"] = "Un jeu CoolGames";
dictionaryVar["fr-fr"]["optionsLang_de-de"] = "Allemand";
dictionaryVar["fr-fr"]["optionsLang_tr-tr"] = "Turc";
dictionaryVar["fr-fr"]["optionsAbout_header"] = "Développé par :";
dictionaryVar["fr-fr"]["levelEndScreenViewHighscoreBtn"] = "Voir les scores";
dictionaryVar["fr-fr"]["levelEndScreenSubmitHighscoreBtn"] = "Publier un score";
dictionaryVar["fr-fr"]["challengeStartScreenTitle_challengee_friend"] = "Votre adversaire :";
dictionaryVar["fr-fr"]["challengeStartTextScore"] = "Son score :";
dictionaryVar["fr-fr"]["challengeStartTextTime"] = "Son temps :";
dictionaryVar["fr-fr"]["challengeStartScreenToWin"] = "À gagner :";
dictionaryVar["fr-fr"]["challengeEndScreenWinnings"] = "Vous avez gagné <AMOUNT> fairpoints.";
dictionaryVar["fr-fr"]["challengeEndScreenOutcomeMessage_WON"] = "Vainqueur !";
dictionaryVar["fr-fr"]["challengeEndScreenOutcomeMessage_LOST"] = "Zut !";
dictionaryVar["fr-fr"]["challengeEndScreenOutcomeMessage_TIED"] = "Ex-aequo !";
dictionaryVar["fr-fr"]["challengeCancelConfirmText"] = "Si vous annulez, on vous remboursera le montant du pari moins les frais de pari. Voulez-vous continuer ? ";
dictionaryVar["fr-fr"]["challengeCancelConfirmBtn_yes"] = "Oui";
dictionaryVar["fr-fr"]["challengeCancelConfirmBtn_no"] = "Non";
dictionaryVar["fr-fr"]["challengeEndScreensBtn_submit"] = "Lancer le défi";
dictionaryVar["fr-fr"]["challengeEndScreenBtn_cancel"] = "Annuler le défi";
dictionaryVar["fr-fr"]["challengeEndScreenName_you"] = "Moi";
dictionaryVar["fr-fr"]["challengeEndScreenChallengeSend_error"] = "Une erreur est survenue. Veuillez réessayer ultérieurement.";
dictionaryVar["fr-fr"]["challengeEndScreenChallengeSend_success"] = "Défi lancé.";
dictionaryVar["fr-fr"]["challengeCancelMessage_error"] = "Une erreur est survenue. Veuillez réessayer ultérieurement.";
dictionaryVar["fr-fr"]["challengeCancelMessage_success"] = "Défi annulé.";
dictionaryVar["fr-fr"]["challengeEndScreenScoreSend_error"] = "Une erreur est survenue. Veuillez réessayer ultérieurement.";
dictionaryVar["fr-fr"]["challengeStartScreenTitle_challengee_stranger"] = "Votre adversaire :";
dictionaryVar["fr-fr"]["challengeStartScreenTitle_challenger_friend"] = "Votre adversaire :";
dictionaryVar["fr-fr"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["fr-fr"]["challengeStartTextTime_challenger"] = "Finissez le plus vite possible !";
dictionaryVar["fr-fr"]["challengeStartTextScore_challenger"] = "Essayez d’atteindre le plus haut score !";
dictionaryVar["fr-fr"]["challengeForfeitConfirmText"] = "Voulez-vous vraiment abandonner la partie ?";
dictionaryVar["fr-fr"]["challengeForfeitConfirmBtn_yes"] = "Oui";
dictionaryVar["fr-fr"]["challengeForfeitConfirmBtn_no"] = "Non";
dictionaryVar["fr-fr"]["challengeForfeitMessage_success"] = "Vous avez abandonné.";
dictionaryVar["fr-fr"]["challengeForfeitMessage_error"] = "Une erreur est survenue. Veuillez réessayer ultérieurement.";
dictionaryVar["fr-fr"]["optionsChallengeForfeit"] = "Abandonner";
dictionaryVar["fr-fr"]["optionsChallengeCancel"] = "Annuler";
dictionaryVar["fr-fr"]["challengeLoadingError_notValid"] = "Désolé, cette partie n'existe plus.";
dictionaryVar["fr-fr"]["challengeLoadingError_notStarted"] = "Une erreur de connexion est survenue. Veuillez réessayer ultérieurement.";
dictionaryVar["fr-fr"]["levelEndScreenHighScore_time"] = "Meilleur temps :";
dictionaryVar["fr-fr"]["levelEndScreenTotalScore_time"] = "Temps total :";
dictionaryVar["fr-fr"]["optionsLang_fr-fr"] = "Français";
dictionaryVar["fr-fr"]["optionsLang_ko-kr"] = "Coréen";
dictionaryVar["fr-fr"]["optionsLang_ar-eg"] = "Arabe";
dictionaryVar["fr-fr"]["optionsLang_es-es"] = "Espagnol";
dictionaryVar["fr-fr"]["optionsLang_pt-br"] = "Portugais - Brésilien";
dictionaryVar["fr-fr"]["optionsLang_ru-ru"] = "Russe";
dictionaryVar["fr-fr"]["optionsExit"] = "Quitter";
dictionaryVar["fr-fr"]["levelEndScreenTotalScore_number"] = "Score total :";
dictionaryVar["fr-fr"]["levelEndScreenHighScore_number"] = "Meilleur score :";
dictionaryVar["fr-fr"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> a 72 heures pour accepter votre défi. Si <NAME> refuse ou n’accepte pas dans ce délai vous serez remboursé le montant de votre pari et les frais de pari.";
dictionaryVar["fr-fr"]["challengeEndScreenChallengeSend_submessage_stranger"] = "Si personne n’accepte votre pari d’ici 72 heures, on vous remboursera le montant du pari y compris les frais.";
dictionaryVar["fr-fr"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["fr-fr"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["fr-fr"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["pt-br"] = dictionaryVar["pt-br"] || {};
dictionaryVar["pt-br"]["loadingScreenLoading"] = "Carregando...";
dictionaryVar["pt-br"]["startScreenPlay"] = "JOGAR";
dictionaryVar["pt-br"]["levelMapScreenTotalScore"] = "Pontuação";
dictionaryVar["pt-br"]["levelEndScreenTitle_level"] = "Nível <VALUE>";
dictionaryVar["pt-br"]["levelEndScreenTitle_difficulty"] = "Muito bem!";
dictionaryVar["pt-br"]["levelEndScreenTitle_endless"] = "Fase <VALUE>";
dictionaryVar["pt-br"]["levelEndScreenTotalScore"] = "Pontuação";
dictionaryVar["pt-br"]["levelEndScreenSubTitle_levelFailed"] = "Tente novamente";
dictionaryVar["pt-br"]["levelEndScreenTimeLeft"] = "Tempo restante";
dictionaryVar["pt-br"]["levelEndScreenTimeBonus"] = "Bônus de tempo";
dictionaryVar["pt-br"]["levelEndScreenHighScore"] = "Recorde";
dictionaryVar["pt-br"]["optionsStartScreen"] = "Menu principal";
dictionaryVar["pt-br"]["optionsQuit"] = "Sair";
dictionaryVar["pt-br"]["optionsResume"] = "Continuar";
dictionaryVar["pt-br"]["optionsTutorial"] = "Como jogar";
dictionaryVar["pt-br"]["optionsHighScore"] = "Recordes";
dictionaryVar["pt-br"]["optionsMoreGames"] = "Mais jogos";
dictionaryVar["pt-br"]["optionsDifficulty_easy"] = "Fácil";
dictionaryVar["pt-br"]["optionsDifficulty_medium"] = "Médio";
dictionaryVar["pt-br"]["optionsDifficulty_hard"] = "Difícil";
dictionaryVar["pt-br"]["optionsMusic_on"] = "Sim";
dictionaryVar["pt-br"]["optionsMusic_off"] = "Não";
dictionaryVar["pt-br"]["optionsSFX_on"] = "Sim";
dictionaryVar["pt-br"]["optionsSFX_off"] = "Não";
dictionaryVar["pt-br"]["optionsLang_en-us"] = "Inglês (EUA)";
dictionaryVar["pt-br"]["optionsLang_en-gb"] = "Inglês (GB)";
dictionaryVar["pt-br"]["optionsLang_nl-nl"] = "Holandês";
dictionaryVar["pt-br"]["gameEndScreenTitle"] = "Parabéns!\nVocê concluiu o jogo.";
dictionaryVar["pt-br"]["gameEndScreenBtnText"] = "Continuar";
dictionaryVar["pt-br"]["optionsTitle"] = "Configurações";
dictionaryVar["pt-br"]["optionsQuitConfirmationText"] = "Atenção!\n\nSe você sair agora, perderá todo progresso realizado neste nível. Deseja mesmo sair?";
dictionaryVar["pt-br"]["optionsQuitConfirmBtn_No"] = "Não";
dictionaryVar["pt-br"]["optionsQuitConfirmBtn_Yes"] = "Sim, tenho certeza.";
dictionaryVar["pt-br"]["levelMapScreenTitle"] = "Selecione um nível";
dictionaryVar["pt-br"]["optionsRestartConfirmationText"] = "Atenção!\n\nSe você reiniciar agora, perderá todo progresso realizado neste nível. Deseja mesmo reiniciar?";
dictionaryVar["pt-br"]["optionsRestart"] = "Reiniciar";
dictionaryVar["pt-br"]["optionsSFXBig_on"] = "Com som";
dictionaryVar["pt-br"]["optionsSFXBig_off"] = "Sem som";
dictionaryVar["pt-br"]["optionsAbout_title"] = "Sobre";
dictionaryVar["pt-br"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["pt-br"]["optionsAbout_backBtn"] = "Voltar";
dictionaryVar["pt-br"]["optionsAbout_version"] = "versão:";
dictionaryVar["pt-br"]["optionsAbout"] = "Sobre";
dictionaryVar["pt-br"]["levelEndScreenMedal"] = "MELHOROU!";
dictionaryVar["pt-br"]["startScreenQuestionaire"] = "O que você achou?";
dictionaryVar["pt-br"]["levelMapScreenWorld_0"] = "Selecione um nível";
dictionaryVar["pt-br"]["startScreenByTinglyGames"] = "da: CoolGames";
dictionaryVar["pt-br"]["optionsLang_de-de"] = "Alemão";
dictionaryVar["pt-br"]["optionsLang_tr-tr"] = "Turco";
dictionaryVar["pt-br"]["optionsAbout_header"] = "Desenvolvido por:";
dictionaryVar["pt-br"]["levelEndScreenViewHighscoreBtn"] = "Ver pontuações";
dictionaryVar["pt-br"]["levelEndScreenSubmitHighscoreBtn"] = "Enviar recorde";
dictionaryVar["pt-br"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["pt-br"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["pt-br"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["pt-br"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["pt-br"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["pt-br"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["pt-br"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["pt-br"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["pt-br"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["pt-br"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["pt-br"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["pt-br"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["pt-br"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["pt-br"]["challengeEndScreenName_you"] = "You";
dictionaryVar["pt-br"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["pt-br"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["pt-br"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["pt-br"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["pt-br"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["pt-br"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["pt-br"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["pt-br"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["pt-br"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["pt-br"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["pt-br"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["pt-br"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["pt-br"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["pt-br"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["pt-br"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["pt-br"]["optionsChallengeForfeit"] = "Desistir";
dictionaryVar["pt-br"]["optionsChallengeCancel"] = "Sair do Jogo";
dictionaryVar["pt-br"]["challengeLoadingError_notValid"] = "Desculpe, este desafio não é mais válido.";
dictionaryVar["pt-br"]["challengeLoadingError_notStarted"] = "Impossível conectar ao servidor. Por favor, tente novamente mais tarde.";
dictionaryVar["pt-br"]["levelEndScreenHighScore_time"] = "Tempo recorde:";
dictionaryVar["pt-br"]["levelEndScreenTotalScore_time"] = "Tempo total:";
dictionaryVar["pt-br"]["optionsLang_fr-fr"] = "Francês";
dictionaryVar["pt-br"]["optionsLang_ko-kr"] = "Coreano";
dictionaryVar["pt-br"]["optionsLang_ar-eg"] = "Árabe";
dictionaryVar["pt-br"]["optionsLang_es-es"] = "Espanhol";
dictionaryVar["pt-br"]["optionsLang_pt-br"] = "Português do Brasil";
dictionaryVar["pt-br"]["optionsLang_ru-ru"] = "Russo";
dictionaryVar["pt-br"]["optionsExit"] = "Saída";
dictionaryVar["pt-br"]["levelEndScreenTotalScore_number"] = "Pontuação total:";
dictionaryVar["pt-br"]["levelEndScreenHighScore_number"] = "Pontuação máxima:";
dictionaryVar["pt-br"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["pt-br"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["pt-br"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["pt-br"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["pt-br"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["es-es"] = dictionaryVar["es-es"] || {};
dictionaryVar["es-es"]["loadingScreenLoading"] = "Cargando...";
dictionaryVar["es-es"]["startScreenPlay"] = "JUGAR";
dictionaryVar["es-es"]["levelMapScreenTotalScore"] = "Punt. total";
dictionaryVar["es-es"]["levelEndScreenTitle_level"] = "Nivel <VALUE>";
dictionaryVar["es-es"]["levelEndScreenTitle_difficulty"] = "¡Muy bien!";
dictionaryVar["es-es"]["levelEndScreenTitle_endless"] = "Fase <VALUE>";
dictionaryVar["es-es"]["levelEndScreenTotalScore"] = "Punt. total";
dictionaryVar["es-es"]["levelEndScreenSubTitle_levelFailed"] = "Nivel fallido";
dictionaryVar["es-es"]["levelEndScreenTimeLeft"] = "Tiempo restante";
dictionaryVar["es-es"]["levelEndScreenTimeBonus"] = "Bonif. tiempo";
dictionaryVar["es-es"]["levelEndScreenHighScore"] = "Récord";
dictionaryVar["es-es"]["optionsStartScreen"] = "Menú principal";
dictionaryVar["es-es"]["optionsQuit"] = "Salir";
dictionaryVar["es-es"]["optionsResume"] = "Seguir";
dictionaryVar["es-es"]["optionsTutorial"] = "Cómo jugar";
dictionaryVar["es-es"]["optionsHighScore"] = "Récords";
dictionaryVar["es-es"]["optionsMoreGames"] = "Más juegos";
dictionaryVar["es-es"]["optionsDifficulty_easy"] = "Fácil";
dictionaryVar["es-es"]["optionsDifficulty_medium"] = "Normal";
dictionaryVar["es-es"]["optionsDifficulty_hard"] = "Difícil";
dictionaryVar["es-es"]["optionsMusic_on"] = "Sí";
dictionaryVar["es-es"]["optionsMusic_off"] = "No";
dictionaryVar["es-es"]["optionsSFX_on"] = "Sí";
dictionaryVar["es-es"]["optionsSFX_off"] = "No";
dictionaryVar["es-es"]["optionsLang_en-us"] = "Inglés (EE.UU.)";
dictionaryVar["es-es"]["optionsLang_en-gb"] = "Inglés (GB)";
dictionaryVar["es-es"]["optionsLang_nl-nl"] = "Neerlandés";
dictionaryVar["es-es"]["gameEndScreenTitle"] = "¡Enhorabuena!\nHas terminado el juego.";
dictionaryVar["es-es"]["gameEndScreenBtnText"] = "Continuar";
dictionaryVar["es-es"]["optionsTitle"] = "Ajustes";
dictionaryVar["es-es"]["optionsQuitConfirmationText"] = "¡Aviso!\n\nSi sales ahora, perderás el progreso que hayas realizado en el nivel. ¿Seguro que quieres salir?";
dictionaryVar["es-es"]["optionsQuitConfirmBtn_No"] = "No";
dictionaryVar["es-es"]["optionsQuitConfirmBtn_Yes"] = "Sí, seguro";
dictionaryVar["es-es"]["levelMapScreenTitle"] = "Elige un nivel";
dictionaryVar["es-es"]["optionsRestartConfirmationText"] = "¡Aviso!\n\nSi reinicias ahora, perderás el progreso que hayas realizado en el nivel. ¿Seguro que quieres reiniciar?";
dictionaryVar["es-es"]["optionsRestart"] = "Reiniciar";
dictionaryVar["es-es"]["optionsSFXBig_on"] = "Sonido sí";
dictionaryVar["es-es"]["optionsSFXBig_off"] = "Sonido no";
dictionaryVar["es-es"]["optionsAbout_title"] = "Acerca de";
dictionaryVar["es-es"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["es-es"]["optionsAbout_backBtn"] = "Atrás";
dictionaryVar["es-es"]["optionsAbout_version"] = "versión:";
dictionaryVar["es-es"]["optionsAbout"] = "Acerca de";
dictionaryVar["es-es"]["levelEndScreenMedal"] = "¡SUPERADO!";
dictionaryVar["es-es"]["startScreenQuestionaire"] = "¿Qué te parece?";
dictionaryVar["es-es"]["levelMapScreenWorld_0"] = "Elige un nivel";
dictionaryVar["es-es"]["startScreenByTinglyGames"] = "de: CoolGames";
dictionaryVar["es-es"]["optionsLang_de-de"] = "Alemán";
dictionaryVar["es-es"]["optionsLang_tr-tr"] = "Turco";
dictionaryVar["es-es"]["optionsAbout_header"] = "Desarrollado por:";
dictionaryVar["es-es"]["levelEndScreenViewHighscoreBtn"] = "Ver puntuaciones";
dictionaryVar["es-es"]["levelEndScreenSubmitHighscoreBtn"] = "Enviar puntuación";
dictionaryVar["es-es"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["es-es"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["es-es"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["es-es"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["es-es"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["es-es"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["es-es"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["es-es"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["es-es"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["es-es"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["es-es"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["es-es"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["es-es"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["es-es"]["challengeEndScreenName_you"] = "You";
dictionaryVar["es-es"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["es-es"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["es-es"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["es-es"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["es-es"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["es-es"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["es-es"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["es-es"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["es-es"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["es-es"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["es-es"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["es-es"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["es-es"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["es-es"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["es-es"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["es-es"]["optionsChallengeForfeit"] = "Rendirse";
dictionaryVar["es-es"]["optionsChallengeCancel"] = "Abandonar";
dictionaryVar["es-es"]["challengeLoadingError_notValid"] = "Lo sentimos, este reto ya no es válido.";
dictionaryVar["es-es"]["challengeLoadingError_notStarted"] = "Imposible conectar con el servidor. Inténtalo más tarde.";
dictionaryVar["es-es"]["levelEndScreenHighScore_time"] = "Mejor tiempo:";
dictionaryVar["es-es"]["levelEndScreenTotalScore_time"] = "Tiempo total:";
dictionaryVar["es-es"]["optionsLang_fr-fr"] = "Francés";
dictionaryVar["es-es"]["optionsLang_ko-kr"] = "Coreano";
dictionaryVar["es-es"]["optionsLang_ar-eg"] = "Árabe";
dictionaryVar["es-es"]["optionsLang_es-es"] = "Español";
dictionaryVar["es-es"]["optionsLang_pt-br"] = "Portugués brasileño";
dictionaryVar["es-es"]["optionsLang_ru-ru"] = "Ruso";
dictionaryVar["es-es"]["optionsExit"] = "Salir";
dictionaryVar["es-es"]["levelEndScreenTotalScore_number"] = "Puntos totales:";
dictionaryVar["es-es"]["levelEndScreenHighScore_number"] = "Mejor puntuación:";
dictionaryVar["es-es"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["es-es"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["es-es"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["es-es"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["es-es"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["tr-tr"] = dictionaryVar["tr-tr"] || {};
dictionaryVar["tr-tr"]["loadingScreenLoading"] = "Yükleniyor...";
dictionaryVar["tr-tr"]["startScreenPlay"] = "OYNA";
dictionaryVar["tr-tr"]["levelMapScreenTotalScore"] = "Toplam skor";
dictionaryVar["tr-tr"]["levelEndScreenTitle_level"] = "Seviye <VALUE>";
dictionaryVar["tr-tr"]["levelEndScreenTitle_difficulty"] = "Bravo!";
dictionaryVar["tr-tr"]["levelEndScreenTitle_endless"] = "Bölüm <VALUE>";
dictionaryVar["tr-tr"]["levelEndScreenTotalScore"] = "Toplam skor";
dictionaryVar["tr-tr"]["levelEndScreenSubTitle_levelFailed"] = "Seviye başarısız";
dictionaryVar["tr-tr"]["levelEndScreenTimeLeft"] = "Kalan Süre";
dictionaryVar["tr-tr"]["levelEndScreenTimeBonus"] = "Süre Bonusu";
dictionaryVar["tr-tr"]["levelEndScreenHighScore"] = "Yüksek skor";
dictionaryVar["tr-tr"]["optionsStartScreen"] = "Ana menü";
dictionaryVar["tr-tr"]["optionsQuit"] = "Çık";
dictionaryVar["tr-tr"]["optionsResume"] = "Devam et";
dictionaryVar["tr-tr"]["optionsTutorial"] = "Nasıl oynanır";
dictionaryVar["tr-tr"]["optionsHighScore"] = "Yüksek skorlar";
dictionaryVar["tr-tr"]["optionsMoreGames"] = "Daha Fazla Oyun";
dictionaryVar["tr-tr"]["optionsDifficulty_easy"] = "Kolay";
dictionaryVar["tr-tr"]["optionsDifficulty_medium"] = "Orta";
dictionaryVar["tr-tr"]["optionsDifficulty_hard"] = "Zorluk";
dictionaryVar["tr-tr"]["optionsMusic_on"] = "Açık";
dictionaryVar["tr-tr"]["optionsMusic_off"] = "Kapalı";
dictionaryVar["tr-tr"]["optionsSFX_on"] = "Açık";
dictionaryVar["tr-tr"]["optionsSFX_off"] = "Kapalı";
dictionaryVar["tr-tr"]["optionsLang_en-us"] = "İngilizce (US)";
dictionaryVar["tr-tr"]["optionsLang_en-gb"] = "İngilizce (GB)";
dictionaryVar["tr-tr"]["optionsLang_nl-nl"] = "Hollandaca";
dictionaryVar["tr-tr"]["gameEndScreenTitle"] = "Tebrikler!\nOyunu tamamladın.";
dictionaryVar["tr-tr"]["gameEndScreenBtnText"] = "Devam";
dictionaryVar["tr-tr"]["optionsTitle"] = "Ayarlar";
dictionaryVar["tr-tr"]["optionsQuitConfirmationText"] = "Dikkat!\nŞimdi çıkarsan bu seviyede yapılan tüm ilerleme kaybedilecek. Çıkmak istediğinizden emin misiniz?";
dictionaryVar["tr-tr"]["optionsQuitConfirmBtn_No"] = "Hayır";
dictionaryVar["tr-tr"]["optionsQuitConfirmBtn_Yes"] = "Evet, eminim";
dictionaryVar["tr-tr"]["levelMapScreenTitle"] = "Bir seviye seç";
dictionaryVar["tr-tr"]["optionsRestartConfirmationText"] = "Dikkat!\nŞimdi tekrar başlarsan bu seviyede yapılan tüm ilerleme kaybedilecek. Baştan başlamak istediğinden emin misin?";
dictionaryVar["tr-tr"]["optionsRestart"] = "Tekrar başlat";
dictionaryVar["tr-tr"]["optionsSFXBig_on"] = "Ses açık";
dictionaryVar["tr-tr"]["optionsSFXBig_off"] = "Ses kapalı";
dictionaryVar["tr-tr"]["optionsAbout_title"] = "Hakkında";
dictionaryVar["tr-tr"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["tr-tr"]["optionsAbout_backBtn"] = "Geri";
dictionaryVar["tr-tr"]["optionsAbout_version"] = "sürüm:";
dictionaryVar["tr-tr"]["optionsAbout"] = "Hakkında";
dictionaryVar["tr-tr"]["levelEndScreenMedal"] = "İYİLEŞTİ!";
dictionaryVar["tr-tr"]["startScreenQuestionaire"] = "Ne dersin?";
dictionaryVar["tr-tr"]["levelMapScreenWorld_0"] = "Bir seviye seç";
dictionaryVar["tr-tr"]["startScreenByTinglyGames"] = "tarafından: CoolGames";
dictionaryVar["tr-tr"]["optionsLang_de-de"] = "Almanca";
dictionaryVar["tr-tr"]["optionsLang_tr-tr"] = "Türkçe";
dictionaryVar["tr-tr"]["optionsAbout_header"] = "Hazırlayan:";
dictionaryVar["tr-tr"]["levelEndScreenViewHighscoreBtn"] = "Puanları göster:";
dictionaryVar["tr-tr"]["levelEndScreenSubmitHighscoreBtn"] = "Puan gönder";
dictionaryVar["tr-tr"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["tr-tr"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["tr-tr"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["tr-tr"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["tr-tr"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["tr-tr"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["tr-tr"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["tr-tr"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["tr-tr"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["tr-tr"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["tr-tr"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["tr-tr"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["tr-tr"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["tr-tr"]["challengeEndScreenName_you"] = "You";
dictionaryVar["tr-tr"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["tr-tr"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["tr-tr"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["tr-tr"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["tr-tr"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["tr-tr"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["tr-tr"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["tr-tr"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["tr-tr"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["tr-tr"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["tr-tr"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["tr-tr"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["tr-tr"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["tr-tr"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["tr-tr"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["tr-tr"]["optionsChallengeForfeit"] = "Vazgeç";
dictionaryVar["tr-tr"]["optionsChallengeCancel"] = "Çıkış";
dictionaryVar["tr-tr"]["challengeLoadingError_notValid"] = "Üzgünüz, bu zorluk artık geçerli değil.";
dictionaryVar["tr-tr"]["challengeLoadingError_notStarted"] = "Sunucuya bağlanılamıyor. Lütfen daha sonra tekrar deneyin.";
dictionaryVar["tr-tr"]["levelEndScreenHighScore_time"] = "En İyi Zaman:";
dictionaryVar["tr-tr"]["levelEndScreenTotalScore_time"] = "Toplam Zaman:";
dictionaryVar["tr-tr"]["optionsLang_fr-fr"] = "Fransızca";
dictionaryVar["tr-tr"]["optionsLang_ko-kr"] = "Korece";
dictionaryVar["tr-tr"]["optionsLang_ar-eg"] = "Arapça";
dictionaryVar["tr-tr"]["optionsLang_es-es"] = "İspanyolca";
dictionaryVar["tr-tr"]["optionsLang_pt-br"] = "Brezilya Portekizcesi";
dictionaryVar["tr-tr"]["optionsLang_ru-ru"] = "Rusça";
dictionaryVar["tr-tr"]["optionsExit"] = "Çıkış";
dictionaryVar["tr-tr"]["levelEndScreenTotalScore_number"] = "Toplam Puan:";
dictionaryVar["tr-tr"]["levelEndScreenHighScore_number"] = "Yüksek Puan:";
dictionaryVar["tr-tr"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["tr-tr"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["tr-tr"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["tr-tr"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["tr-tr"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["ru-ru"] = dictionaryVar["ru-ru"] || {};
dictionaryVar["ru-ru"]["loadingScreenLoading"] = "Загрузка...";
dictionaryVar["ru-ru"]["startScreenPlay"] = "ИГРАТЬ";
dictionaryVar["ru-ru"]["levelMapScreenTotalScore"] = "Общий счет";
dictionaryVar["ru-ru"]["levelEndScreenTitle_level"] = "Уровень <VALUE>";
dictionaryVar["ru-ru"]["levelEndScreenTitle_difficulty"] = "Хороший результат!";
dictionaryVar["ru-ru"]["levelEndScreenTitle_endless"] = "Этап <VALUE>";
dictionaryVar["ru-ru"]["levelEndScreenTotalScore"] = "Общий счет";
dictionaryVar["ru-ru"]["levelEndScreenSubTitle_levelFailed"] = "Уровень не пройден";
dictionaryVar["ru-ru"]["levelEndScreenTimeLeft"] = "Оставшееся время";
dictionaryVar["ru-ru"]["levelEndScreenTimeBonus"] = "Дополнительное время";
dictionaryVar["ru-ru"]["levelEndScreenHighScore"] = "Рекорд";
dictionaryVar["ru-ru"]["optionsStartScreen"] = "Главное меню";
dictionaryVar["ru-ru"]["optionsQuit"] = "Выйти";
dictionaryVar["ru-ru"]["optionsResume"] = "Продолжить";
dictionaryVar["ru-ru"]["optionsTutorial"] = "Как играть";
dictionaryVar["ru-ru"]["optionsHighScore"] = "Рекорды";
dictionaryVar["ru-ru"]["optionsMoreGames"] = "Больше игр";
dictionaryVar["ru-ru"]["optionsDifficulty_easy"] = "Легкий";
dictionaryVar["ru-ru"]["optionsDifficulty_medium"] = "Средний";
dictionaryVar["ru-ru"]["optionsDifficulty_hard"] = "Сложный";
dictionaryVar["ru-ru"]["optionsMusic_on"] = "Вкл.";
dictionaryVar["ru-ru"]["optionsMusic_off"] = "Выкл.";
dictionaryVar["ru-ru"]["optionsSFX_on"] = "Вкл.";
dictionaryVar["ru-ru"]["optionsSFX_off"] = "Выкл.";
dictionaryVar["ru-ru"]["optionsLang_en-us"] = "Английский (США)";
dictionaryVar["ru-ru"]["optionsLang_en-gb"] = "Английский (ВБ)";
dictionaryVar["ru-ru"]["optionsLang_nl-nl"] = "Нидерландский";
dictionaryVar["ru-ru"]["gameEndScreenTitle"] = "Поздравляем!\nВы прошли игру.";
dictionaryVar["ru-ru"]["gameEndScreenBtnText"] = "Продолжить";
dictionaryVar["ru-ru"]["optionsTitle"] = "Настройки";
dictionaryVar["ru-ru"]["optionsQuitConfirmationText"] = "Внимание!\n\nЕсли вы выйдете сейчас, уровень не будет засчитан. Вы уверены, что хотите выйти?";
dictionaryVar["ru-ru"]["optionsQuitConfirmBtn_No"] = "Нет";
dictionaryVar["ru-ru"]["optionsQuitConfirmBtn_Yes"] = "Да, выйти";
dictionaryVar["ru-ru"]["levelMapScreenTitle"] = "Выберите уровень";
dictionaryVar["ru-ru"]["optionsRestartConfirmationText"] = "Внимание!\n\nЕсли вы сейчас начнете игру заново, уровень не будет засчитан. Вы уверены, что хотите начать заново?";
dictionaryVar["ru-ru"]["optionsRestart"] = "Заново";
dictionaryVar["ru-ru"]["optionsSFXBig_on"] = "Звук вкл.";
dictionaryVar["ru-ru"]["optionsSFXBig_off"] = "Звук выкл.";
dictionaryVar["ru-ru"]["optionsAbout_title"] = "О программе";
dictionaryVar["ru-ru"]["optionsAbout_text"] = "© CoolGames\nwww.coolgames.com\n2016";
dictionaryVar["ru-ru"]["optionsAbout_backBtn"] = "Назад";
dictionaryVar["ru-ru"]["optionsAbout_version"] = "Версия:";
dictionaryVar["ru-ru"]["optionsAbout"] = "О программе";
dictionaryVar["ru-ru"]["levelEndScreenMedal"] = "НОВЫЙ РЕКОРД!";
dictionaryVar["ru-ru"]["startScreenQuestionaire"] = "Как вам игра?";
dictionaryVar["ru-ru"]["levelMapScreenWorld_0"] = "Выберите уровень";
dictionaryVar["ru-ru"]["startScreenByTinglyGames"] = "Разработчики: CoolGames";
dictionaryVar["ru-ru"]["optionsLang_de-de"] = "Немецкий";
dictionaryVar["ru-ru"]["optionsLang_tr-tr"] = "Турецкий";
dictionaryVar["ru-ru"]["optionsAbout_header"] = "Developed by:";
dictionaryVar["ru-ru"]["levelEndScreenViewHighscoreBtn"] = "View scores";
dictionaryVar["ru-ru"]["levelEndScreenSubmitHighscoreBtn"] = "Submit score";
dictionaryVar["ru-ru"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["ru-ru"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["ru-ru"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["ru-ru"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["ru-ru"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["ru-ru"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["ru-ru"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["ru-ru"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["ru-ru"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["ru-ru"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["ru-ru"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["ru-ru"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["ru-ru"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["ru-ru"]["challengeEndScreenName_you"] = "You";
dictionaryVar["ru-ru"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["ru-ru"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["ru-ru"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["ru-ru"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["ru-ru"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["ru-ru"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["ru-ru"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["ru-ru"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["ru-ru"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["ru-ru"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["ru-ru"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["ru-ru"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["ru-ru"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["ru-ru"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["ru-ru"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["ru-ru"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["ru-ru"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["ru-ru"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["ru-ru"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["ru-ru"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["ru-ru"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["ru-ru"]["optionsLang_fr-fr"] = "Французский";
dictionaryVar["ru-ru"]["optionsLang_ko-kr"] = "Корейский";
dictionaryVar["ru-ru"]["optionsLang_ar-eg"] = "Арабский";
dictionaryVar["ru-ru"]["optionsLang_es-es"] = "Испанский";
dictionaryVar["ru-ru"]["optionsLang_pt-br"] = "Бразильский португальский";
dictionaryVar["ru-ru"]["optionsLang_ru-ru"] = "Русский";
dictionaryVar["ru-ru"]["optionsExit"] = "Exit";
dictionaryVar["ru-ru"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["ru-ru"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["ru-ru"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["ru-ru"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["ru-ru"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["ru-ru"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["ru-ru"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["ar-eg"] = dictionaryVar["ar-eg"] || {};
dictionaryVar["ar-eg"]["loadingScreenLoading"] = "يتم الآن التحميل...";
dictionaryVar["ar-eg"]["startScreenPlay"] = "تشغيل";
dictionaryVar["ar-eg"]["levelMapScreenTotalScore"] = "النتيجة الإجمالية";
dictionaryVar["ar-eg"]["levelEndScreenTitle_level"] = "المستوى <VALUE>";
dictionaryVar["ar-eg"]["levelEndScreenTitle_difficulty"] = "أحسنت!";
dictionaryVar["ar-eg"]["levelEndScreenTitle_endless"] = "المرحلة <VALUE>";
dictionaryVar["ar-eg"]["levelEndScreenTotalScore"] = "النتيجة الإجمالية";
dictionaryVar["ar-eg"]["levelEndScreenSubTitle_levelFailed"] = "لقد فشلت في اجتياز هذا المستوى";
dictionaryVar["ar-eg"]["levelEndScreenTimeLeft"] = "الوقت المتبقي";
dictionaryVar["ar-eg"]["levelEndScreenTimeBonus"] = "مكافأة الوقت";
dictionaryVar["ar-eg"]["levelEndScreenHighScore"] = "أعلى نتيجة";
dictionaryVar["ar-eg"]["optionsStartScreen"] = "القائمة الرئيسية";
dictionaryVar["ar-eg"]["optionsQuit"] = "الخروج من اللعبة";
dictionaryVar["ar-eg"]["optionsResume"] = "استئناف";
dictionaryVar["ar-eg"]["optionsTutorial"] = "طريقة اللعب";
dictionaryVar["ar-eg"]["optionsHighScore"] = "أعلى النتائج";
dictionaryVar["ar-eg"]["optionsMoreGames"] = "المزيد من الألعاب";
dictionaryVar["ar-eg"]["optionsDifficulty_easy"] = "سهل";
dictionaryVar["ar-eg"]["optionsDifficulty_medium"] = "متوسط";
dictionaryVar["ar-eg"]["optionsDifficulty_hard"] = "صعب";
dictionaryVar["ar-eg"]["optionsMusic_on"] = "تشغيل الموسيقى";
dictionaryVar["ar-eg"]["optionsMusic_off"] = "إيقاف الموسيقى";
dictionaryVar["ar-eg"]["optionsSFX_on"] = "تشغيل المؤثرات الصوتية";
dictionaryVar["ar-eg"]["optionsSFX_off"] = "إيقاف المؤثرات الصوتية";
dictionaryVar["ar-eg"]["optionsLang_en-us"] = "الإنجليزية (الولايات المتحدة)";
dictionaryVar["ar-eg"]["optionsLang_en-gb"] = "الإنجليزية (المملكة المتحدة)";
dictionaryVar["ar-eg"]["optionsLang_nl-nl"] = "الهولندية";
dictionaryVar["ar-eg"]["gameEndScreenTitle"] = "تهانينا!\nلقد أكملت اللعبة.";
dictionaryVar["ar-eg"]["gameEndScreenBtnText"] = "متابعة";
dictionaryVar["ar-eg"]["optionsTitle"] = "الإعدادات";
dictionaryVar["ar-eg"]["optionsQuitConfirmationText"] = "انتبه!n\n\إذا خرجت من اللعبة الآن، فستفقد كل التقدم الذي أحرزته خلال هذا المستوى. هل أنت متأكد من أنك تريد الخروج من اللعبة؟";
dictionaryVar["ar-eg"]["optionsQuitConfirmBtn_No"] = "لا";
dictionaryVar["ar-eg"]["optionsQuitConfirmBtn_Yes"] = "نعم، متأكد";
dictionaryVar["ar-eg"]["levelMapScreenTitle"] = "تحديد مستوى";
dictionaryVar["ar-eg"]["optionsRestartConfirmationText"] = "انتبه!\n\nإذا قمت بإعادة التشغيل الآن، فستفقد كل التقدم الذي أحرزته خلال هذا المستوى. هل أنت متأكد من أنك تريد إعادة التشغيل؟";
dictionaryVar["ar-eg"]["optionsRestart"] = "إعادة التشغيل";
dictionaryVar["ar-eg"]["optionsSFXBig_on"] = "تشغيل الصوت";
dictionaryVar["ar-eg"]["optionsSFXBig_off"] = "إيقاف الصوت";
dictionaryVar["ar-eg"]["optionsAbout_title"] = "حول";
dictionaryVar["ar-eg"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["ar-eg"]["optionsAbout_backBtn"] = "السابق";
dictionaryVar["ar-eg"]["optionsAbout_version"] = "الإصدار:";
dictionaryVar["ar-eg"]["optionsAbout"] = "حول";
dictionaryVar["ar-eg"]["levelEndScreenMedal"] = "لقد تحسّنت!";
dictionaryVar["ar-eg"]["startScreenQuestionaire"] = "ما رأيك؟";
dictionaryVar["ar-eg"]["levelMapScreenWorld_0"] = "تحديد مستوى";
dictionaryVar["ar-eg"]["startScreenByTinglyGames"] = "بواسطة: CoolGames";
dictionaryVar["ar-eg"]["optionsLang_de-de"] = "الألمانية";
dictionaryVar["ar-eg"]["optionsLang_tr-tr"] = "التركية";
dictionaryVar["ar-eg"]["optionsAbout_header"] = "Developed by:";
dictionaryVar["ar-eg"]["levelEndScreenViewHighscoreBtn"] = "View scores";
dictionaryVar["ar-eg"]["levelEndScreenSubmitHighscoreBtn"] = "Submit score";
dictionaryVar["ar-eg"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["ar-eg"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["ar-eg"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["ar-eg"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["ar-eg"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["ar-eg"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["ar-eg"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["ar-eg"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["ar-eg"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["ar-eg"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["ar-eg"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["ar-eg"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["ar-eg"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["ar-eg"]["challengeEndScreenName_you"] = "You";
dictionaryVar["ar-eg"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["ar-eg"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["ar-eg"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["ar-eg"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["ar-eg"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["ar-eg"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["ar-eg"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["ar-eg"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["ar-eg"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["ar-eg"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["ar-eg"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["ar-eg"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["ar-eg"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["ar-eg"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["ar-eg"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["ar-eg"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["ar-eg"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["ar-eg"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["ar-eg"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["ar-eg"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["ar-eg"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["ar-eg"]["optionsLang_fr-fr"] = "الفرنسية";
dictionaryVar["ar-eg"]["optionsLang_ko-kr"] = "الكورية";
dictionaryVar["ar-eg"]["optionsLang_ar-eg"] = "العربية";
dictionaryVar["ar-eg"]["optionsLang_es-es"] = "الإسبانية";
dictionaryVar["ar-eg"]["optionsLang_pt-br"] = "البرازيلية - البرتغالية";
dictionaryVar["ar-eg"]["optionsLang_ru-ru"] = "الروسية";
dictionaryVar["ar-eg"]["optionsExit"] = "Exit";
dictionaryVar["ar-eg"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["ar-eg"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["ar-eg"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["ar-eg"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["ar-eg"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["ar-eg"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["ar-eg"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["ko-kr"] = dictionaryVar["ko-kr"] || {};
dictionaryVar["ko-kr"]["loadingScreenLoading"] = "불러오기 중...";
dictionaryVar["ko-kr"]["startScreenPlay"] = "PLAY";
dictionaryVar["ko-kr"]["levelMapScreenTotalScore"] = "총 점수";
dictionaryVar["ko-kr"]["levelEndScreenTitle_level"] = "레벨 <VALUE>";
dictionaryVar["ko-kr"]["levelEndScreenTitle_difficulty"] = "잘 했어요!";
dictionaryVar["ko-kr"]["levelEndScreenTitle_endless"] = "스테이지 <VALUE>";
dictionaryVar["ko-kr"]["levelEndScreenTotalScore"] = "총 점수";
dictionaryVar["ko-kr"]["levelEndScreenSubTitle_levelFailed"] = "레벨 실패";
dictionaryVar["ko-kr"]["levelEndScreenTimeLeft"] = "남은 시간";
dictionaryVar["ko-kr"]["levelEndScreenTimeBonus"] = "시간 보너스";
dictionaryVar["ko-kr"]["levelEndScreenHighScore"] = "최고 점수";
dictionaryVar["ko-kr"]["optionsStartScreen"] = "메인 메뉴";
dictionaryVar["ko-kr"]["optionsQuit"] = "종료";
dictionaryVar["ko-kr"]["optionsResume"] = "계속";
dictionaryVar["ko-kr"]["optionsTutorial"] = "게임 방법";
dictionaryVar["ko-kr"]["optionsHighScore"] = "최고 점수";
dictionaryVar["ko-kr"]["optionsMoreGames"] = "더 많은 게임";
dictionaryVar["ko-kr"]["optionsDifficulty_easy"] = "간단";
dictionaryVar["ko-kr"]["optionsDifficulty_medium"] = "중";
dictionaryVar["ko-kr"]["optionsDifficulty_hard"] = "상";
dictionaryVar["ko-kr"]["optionsMusic_on"] = "켜기";
dictionaryVar["ko-kr"]["optionsMusic_off"] = "끄기";
dictionaryVar["ko-kr"]["optionsSFX_on"] = "켜기";
dictionaryVar["ko-kr"]["optionsSFX_off"] = "끄기";
dictionaryVar["ko-kr"]["optionsLang_en-us"] = "영어(US)";
dictionaryVar["ko-kr"]["optionsLang_en-gb"] = "영어(GB)";
dictionaryVar["ko-kr"]["optionsLang_nl-nl"] = "네덜란드어";
dictionaryVar["ko-kr"]["gameEndScreenTitle"] = "축하합니다!\n게임을 완료했습니다.";
dictionaryVar["ko-kr"]["gameEndScreenBtnText"] = "계속";
dictionaryVar["ko-kr"]["optionsTitle"] = "설정";
dictionaryVar["ko-kr"]["optionsQuitConfirmationText"] = "주의!\n\n지금 종료하면 이 레벨의 모든 진행 내용을 잃게됩니다. 정말 종료하시겠습니까?";
dictionaryVar["ko-kr"]["optionsQuitConfirmBtn_No"] = "아니오";
dictionaryVar["ko-kr"]["optionsQuitConfirmBtn_Yes"] = "네, 확실합니다";
dictionaryVar["ko-kr"]["levelMapScreenTitle"] = "레벨 선택";
dictionaryVar["ko-kr"]["optionsRestartConfirmationText"] = "주의!\n\n지금 다시 시작하면 이 레벨의 모든 진행 내용을 잃게됩니다. 정말 다시 시작하시겠습니까?";
dictionaryVar["ko-kr"]["optionsRestart"] = "다시 시작";
dictionaryVar["ko-kr"]["optionsSFXBig_on"] = "음향 켜기";
dictionaryVar["ko-kr"]["optionsSFXBig_off"] = "음향 끄기";
dictionaryVar["ko-kr"]["optionsAbout_title"] = "관련 정보";
dictionaryVar["ko-kr"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["ko-kr"]["optionsAbout_backBtn"] = "뒤로";
dictionaryVar["ko-kr"]["optionsAbout_version"] = "버전:";
dictionaryVar["ko-kr"]["optionsAbout"] = "관련 정보";
dictionaryVar["ko-kr"]["levelEndScreenMedal"] = "향상했군요!";
dictionaryVar["ko-kr"]["startScreenQuestionaire"] = "어떻게 생각하세요?";
dictionaryVar["ko-kr"]["levelMapScreenWorld_0"] = "레벨 선택";
dictionaryVar["ko-kr"]["startScreenByTinglyGames"] = "제작: CoolGames";
dictionaryVar["ko-kr"]["optionsLang_de-de"] = "독일어";
dictionaryVar["ko-kr"]["optionsLang_tr-tr"] = "터키어";
dictionaryVar["ko-kr"]["optionsAbout_header"] = "Developed by:";
dictionaryVar["ko-kr"]["levelEndScreenViewHighscoreBtn"] = "View scores";
dictionaryVar["ko-kr"]["levelEndScreenSubmitHighscoreBtn"] = "Submit score";
dictionaryVar["ko-kr"]["challengeStartScreenTitle_challengee_friend"] = "You have been challenged by:";
dictionaryVar["ko-kr"]["challengeStartTextScore"] = "<NAME>'s score:";
dictionaryVar["ko-kr"]["challengeStartTextTime"] = "<NAME>'s time:";
dictionaryVar["ko-kr"]["challengeStartScreenToWin"] = "Amount to win:";
dictionaryVar["ko-kr"]["challengeEndScreenWinnings"] = "You have won <AMOUNT> fairpoints";
dictionaryVar["ko-kr"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["ko-kr"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["ko-kr"]["challengeEndScreenOutcomeMessage_TIED"] = "You tied.";
dictionaryVar["ko-kr"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["ko-kr"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["ko-kr"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["ko-kr"]["challengeEndScreensBtn_submit"] = "Submit challenge";
dictionaryVar["ko-kr"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["ko-kr"]["challengeEndScreenName_you"] = "You";
dictionaryVar["ko-kr"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["ko-kr"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["ko-kr"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["ko-kr"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["ko-kr"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["ko-kr"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["ko-kr"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["ko-kr"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["ko-kr"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["ko-kr"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["ko-kr"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["ko-kr"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["ko-kr"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["ko-kr"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["ko-kr"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["ko-kr"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["ko-kr"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["ko-kr"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["ko-kr"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["ko-kr"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["ko-kr"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["ko-kr"]["optionsLang_fr-fr"] = "프랑스어";
dictionaryVar["ko-kr"]["optionsLang_ko-kr"] = "한국어";
dictionaryVar["ko-kr"]["optionsLang_ar-eg"] = "아라비아어";
dictionaryVar["ko-kr"]["optionsLang_es-es"] = "스페인어";
dictionaryVar["ko-kr"]["optionsLang_pt-br"] = "포르투갈어(브라질)";
dictionaryVar["ko-kr"]["optionsLang_ru-ru"] = "러시아어";
dictionaryVar["ko-kr"]["optionsExit"] = "Exit";
dictionaryVar["ko-kr"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["ko-kr"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["ko-kr"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["ko-kr"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["ko-kr"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["ko-kr"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["ko-kr"]["optionsLang_jp-jp"] = "Japanese";
dictionaryVar["jp-jp"] = dictionaryVar["jp-jp"] || {};
dictionaryVar["jp-jp"]["loadingScreenLoading"] = "ロード中…";
dictionaryVar["jp-jp"]["startScreenPlay"] = "プレイ";
dictionaryVar["jp-jp"]["levelMapScreenTotalScore"] = "トータルスコア";
dictionaryVar["jp-jp"]["levelEndScreenTitle_level"] = "レベル <VALUE>";
dictionaryVar["jp-jp"]["levelEndScreenTitle_difficulty"] = "やったね！";
dictionaryVar["jp-jp"]["levelEndScreenTitle_endless"] = "ステージ <VALUE>";
dictionaryVar["jp-jp"]["levelEndScreenTotalScore"] = "トータルスコア";
dictionaryVar["jp-jp"]["levelEndScreenSubTitle_levelFailed"] = "ゲームオーバー";
dictionaryVar["jp-jp"]["levelEndScreenTimeLeft"] = "残り時間";
dictionaryVar["jp-jp"]["levelEndScreenTimeBonus"] = "タイムボーナス";
dictionaryVar["jp-jp"]["levelEndScreenHighScore"] = "ハイスコア";
dictionaryVar["jp-jp"]["optionsStartScreen"] = "メインメニュー";
dictionaryVar["jp-jp"]["optionsQuit"] = "やめる";
dictionaryVar["jp-jp"]["optionsResume"] = "再開";
dictionaryVar["jp-jp"]["optionsTutorial"] = "あそび方";
dictionaryVar["jp-jp"]["optionsHighScore"] = "ハイスコア";
dictionaryVar["jp-jp"]["optionsMoreGames"] = "他のゲーム";
dictionaryVar["jp-jp"]["optionsDifficulty_easy"] = "初級";
dictionaryVar["jp-jp"]["optionsDifficulty_medium"] = "中級";
dictionaryVar["jp-jp"]["optionsDifficulty_hard"] = "上級";
dictionaryVar["jp-jp"]["optionsMusic_on"] = "オン";
dictionaryVar["jp-jp"]["optionsMusic_off"] = "オフ";
dictionaryVar["jp-jp"]["optionsSFX_on"] = "オン";
dictionaryVar["jp-jp"]["optionsSFX_off"] = "オフ";
dictionaryVar["jp-jp"]["optionsLang_en-us"] = "英語（米国）";
dictionaryVar["jp-jp"]["optionsLang_en-gb"] = "英語（英国）";
dictionaryVar["jp-jp"]["optionsLang_nl-nl"] = "オランダ語";
dictionaryVar["jp-jp"]["gameEndScreenTitle"] = "おめでとう！\nすべてのレベルをクリアしました。";
dictionaryVar["jp-jp"]["gameEndScreenBtnText"] = "続ける";
dictionaryVar["jp-jp"]["optionsTitle"] = "設定";
dictionaryVar["jp-jp"]["optionsQuitConfirmationText"] = "注意！\n\nここでやめると\n記録がリセットされますが\nよろしいですか？";
dictionaryVar["jp-jp"]["optionsQuitConfirmBtn_No"] = "いいえ、続けます。";
dictionaryVar["jp-jp"]["optionsQuitConfirmBtn_Yes"] = "はい、やめます。";
dictionaryVar["jp-jp"]["levelMapScreenTitle"] = "レベル選択";
dictionaryVar["jp-jp"]["optionsRestartConfirmationText"] = "注意！\n\nここで再スタートすると\n記録がリセットされますが\nよろしいですか？";
dictionaryVar["jp-jp"]["optionsRestart"] = "再スタート";
dictionaryVar["jp-jp"]["optionsSFXBig_on"] = "サウンド オン";
dictionaryVar["jp-jp"]["optionsSFXBig_off"] = "サウンド オフ";
dictionaryVar["jp-jp"]["optionsAbout_title"] = "About";
dictionaryVar["jp-jp"]["optionsAbout_text"] = "CoolGames\nwww.coolgames.com\n© 2016";
dictionaryVar["jp-jp"]["optionsAbout_backBtn"] = "もどる";
dictionaryVar["jp-jp"]["optionsAbout_version"] = "version";
dictionaryVar["jp-jp"]["optionsAbout"] = "About";
dictionaryVar["jp-jp"]["levelEndScreenMedal"] = "記録更新！";
dictionaryVar["jp-jp"]["startScreenQuestionaire"] = "このゲームへの感想";
dictionaryVar["jp-jp"]["levelMapScreenWorld_0"] = "レベル選択";
dictionaryVar["jp-jp"]["startScreenByTinglyGames"] = "by: CoolGames";
dictionaryVar["jp-jp"]["optionsLang_de-de"] = "ドイツ語";
dictionaryVar["jp-jp"]["optionsLang_tr-tr"] = "トルコ語";
dictionaryVar["jp-jp"]["optionsAbout_header"] = "Developed by";
dictionaryVar["jp-jp"]["levelEndScreenViewHighscoreBtn"] = "スコアをみる";
dictionaryVar["jp-jp"]["levelEndScreenSubmitHighscoreBtn"] = "スコア送信";
dictionaryVar["jp-jp"]["challengeStartScreenTitle_challengee_friend"] = "から挑戦を受けました";
dictionaryVar["jp-jp"]["challengeStartTextScore"] = "<NAME>のスコア";
dictionaryVar["jp-jp"]["challengeStartTextTime"] = "<NAME>の時間";
dictionaryVar["jp-jp"]["challengeStartScreenToWin"] = "ポイント数";
dictionaryVar["jp-jp"]["challengeEndScreenWinnings"] = "<AMOUNT>ポイント獲得";
dictionaryVar["jp-jp"]["challengeEndScreenOutcomeMessage_WON"] = "You have won the challenge!";
dictionaryVar["jp-jp"]["challengeEndScreenOutcomeMessage_LOST"] = "You have lost the challenge.";
dictionaryVar["jp-jp"]["challengeEndScreenOutcomeMessage_TIED"] = "同点";
dictionaryVar["jp-jp"]["challengeCancelConfirmText"] = "You are about to cancel the challenge. Your wager will be returned minus the challenge fee. Are you sure you want to cancel the challenge?";
dictionaryVar["jp-jp"]["challengeCancelConfirmBtn_yes"] = "Yes";
dictionaryVar["jp-jp"]["challengeCancelConfirmBtn_no"] = "No";
dictionaryVar["jp-jp"]["challengeEndScreensBtn_submit"] = "あ";
dictionaryVar["jp-jp"]["challengeEndScreenBtn_cancel"] = "Cancel challenge";
dictionaryVar["jp-jp"]["challengeEndScreenName_you"] = "You";
dictionaryVar["jp-jp"]["challengeEndScreenChallengeSend_error"] = "An error occured while submitting the challenge. Please try again later.";
dictionaryVar["jp-jp"]["challengeEndScreenChallengeSend_success"] = "Your challenge has been sent!";
dictionaryVar["jp-jp"]["challengeCancelMessage_error"] = "An error occured while cancelling your challenge. Please try again later.";
dictionaryVar["jp-jp"]["challengeCancelMessage_success"] = "Your challenge has been cancelled.";
dictionaryVar["jp-jp"]["challengeEndScreenScoreSend_error"] = "An error occured while communicating with the server. Please try again later.";
dictionaryVar["jp-jp"]["challengeStartScreenTitle_challengee_stranger"] = "You have been matched with:";
dictionaryVar["jp-jp"]["challengeStartScreenTitle_challenger_friend"] = "You are challenging:";
dictionaryVar["jp-jp"]["challengeStartScreenTitle_challenger_stranger"] = "You are setting a score for:";
dictionaryVar["jp-jp"]["challengeStartTextTime_challenger"] = "Play the game and set a time.";
dictionaryVar["jp-jp"]["challengeStartTextScore_challenger"] = "Play the game and set a score.";
dictionaryVar["jp-jp"]["challengeForfeitConfirmText"] = "You are about to forfeit the challenge. Are you sure you want to proceed?";
dictionaryVar["jp-jp"]["challengeForfeitConfirmBtn_yes"] = "Yes";
dictionaryVar["jp-jp"]["challengeForfeitConfirmBtn_no"] = "No";
dictionaryVar["jp-jp"]["challengeForfeitMessage_success"] = "You have forfeited the challenge.";
dictionaryVar["jp-jp"]["challengeForfeitMessage_error"] = "An error occured while forfeiting the challenge. Please try again later.";
dictionaryVar["jp-jp"]["optionsChallengeForfeit"] = "Forfeit";
dictionaryVar["jp-jp"]["optionsChallengeCancel"] = "Quit";
dictionaryVar["jp-jp"]["challengeLoadingError_notValid"] = "Sorry, this challenge is no longer valid.";
dictionaryVar["jp-jp"]["challengeLoadingError_notStarted"] = "Unable to connect to the server. Please try again later.";
dictionaryVar["jp-jp"]["levelEndScreenHighScore_time"] = "Best time:";
dictionaryVar["jp-jp"]["levelEndScreenTotalScore_time"] = "Total time:";
dictionaryVar["jp-jp"]["optionsLang_fr-fr"] = "French";
dictionaryVar["jp-jp"]["optionsLang_ko-kr"] = "Korean";
dictionaryVar["jp-jp"]["optionsLang_ar-eg"] = "Arabic";
dictionaryVar["jp-jp"]["optionsLang_es-es"] = "Spanish";
dictionaryVar["jp-jp"]["optionsLang_pt-br"] = "Brazilian-Portuguese";
dictionaryVar["jp-jp"]["optionsLang_ru-ru"] = "Russian";
dictionaryVar["jp-jp"]["optionsExit"] = "Exit";
dictionaryVar["jp-jp"]["levelEndScreenTotalScore_number"] = "Total score:";
dictionaryVar["jp-jp"]["levelEndScreenHighScore_number"] = "High score:";
dictionaryVar["jp-jp"]["challengeEndScreenChallengeSend_submessage"] = "<NAME> has 72 hours to accept or decline your challenge. If <NAME> declines or doesn’t accept within 72 hours your wager and challenge fee will be reimbursed.";
dictionaryVar["jp-jp"]["challengeEndScreenChallengeSend_submessage_stranger"] = "If no one accepts your challenge within 72 hours, the amount of your wager and the challenge fee will be returned to you.";
dictionaryVar["jp-jp"]["challengeForfeitMessage_winnings"] = "<NAME> has won <AMOUNT> fairpoints!";
dictionaryVar["jp-jp"]["optionsAbout_header_publisher"] = "Published by:";
dictionaryVar["jp-jp"]["optionsLang_jp-jp"] = "日本語";

//******************************************************
//*****    Script Resource: game_ui_dictionary
//******************************************************

// **********************************************
// Localization file for product: game_ui_dictionary
//
// created on 12-2-2016 14:45:13
// **********************************************

/*exported dictionaryVar*/
var dictionaryVar = dictionaryVar || {};
dictionaryVar["nl-nl"] = dictionaryVar["nl-nl"] || {};
dictionaryVar["nl-nl"]["game_ui_SCORE"] = "SCORE";
dictionaryVar["nl-nl"]["game_ui_STAGE"] = "LEVEL";
dictionaryVar["nl-nl"]["game_ui_LIVES"] = "LEVENS";
dictionaryVar["nl-nl"]["game_ui_TIME"] = "TIJD";
dictionaryVar["nl-nl"]["game_ui_HIGHSCORE"] = "HIGH SCORE";
dictionaryVar["nl-nl"]["game_ui_LEVEL"] = "LEVEL";
dictionaryVar["nl-nl"]["game_ui_time_left"] = "Resterende tijd";
dictionaryVar["nl-nl"]["game_ui_TIME_TO_BEAT"] = "DOELTIJD";
dictionaryVar["nl-nl"]["game_ui_SCORE_TO_BEAT"] = "DOELSCORE";
dictionaryVar["nl-nl"]["game_ui_HIGHSCORE_break"] = "HIGH\nSCORE";
dictionaryVar["en-us"] = dictionaryVar["en-us"] || {};
dictionaryVar["en-us"]["game_ui_SCORE"] = "SCORE";
dictionaryVar["en-us"]["game_ui_STAGE"] = "STAGE";
dictionaryVar["en-us"]["game_ui_LIVES"] = "LIVES";
dictionaryVar["en-us"]["game_ui_TIME"] = "TIME";
dictionaryVar["en-us"]["game_ui_HIGHSCORE"] = "HIGH SCORE";
dictionaryVar["en-us"]["game_ui_LEVEL"] = "LEVEL";
dictionaryVar["en-us"]["game_ui_time_left"] = "Time left";
dictionaryVar["en-us"]["game_ui_TIME_TO_BEAT"] = "TIME TO BEAT";
dictionaryVar["en-us"]["game_ui_SCORE_TO_BEAT"] = "SCORE TO BEAT";
dictionaryVar["en-us"]["game_ui_HIGHSCORE_break"] = "HIGH\nSCORE";
dictionaryVar["en-gb"] = dictionaryVar["en-gb"] || {};
dictionaryVar["en-gb"]["game_ui_SCORE"] = "SCORE";
dictionaryVar["en-gb"]["game_ui_STAGE"] = "STAGE";
dictionaryVar["en-gb"]["game_ui_LIVES"] = "LIVES";
dictionaryVar["en-gb"]["game_ui_TIME"] = "TIME";
dictionaryVar["en-gb"]["game_ui_HIGHSCORE"] = "HIGH SCORE";
dictionaryVar["en-gb"]["game_ui_LEVEL"] = "LEVEL";
dictionaryVar["en-gb"]["game_ui_time_left"] = "Time left";
dictionaryVar["en-gb"]["game_ui_TIME_TO_BEAT"] = "TIME TO BEAT";
dictionaryVar["en-gb"]["game_ui_SCORE_TO_BEAT"] = "SCORE TO BEAT";
dictionaryVar["en-gb"]["game_ui_HIGHSCORE_break"] = "HIGH\nSCORE";
dictionaryVar["de-de"] = dictionaryVar["de-de"] || {};
dictionaryVar["de-de"]["game_ui_SCORE"] = "PUNKTE";
dictionaryVar["de-de"]["game_ui_STAGE"] = "STUFE";
dictionaryVar["de-de"]["game_ui_LIVES"] = "LEBEN";
dictionaryVar["de-de"]["game_ui_TIME"] = "ZEIT";
dictionaryVar["de-de"]["game_ui_HIGHSCORE"] = "HIGHSCORE";
dictionaryVar["de-de"]["game_ui_LEVEL"] = "LEVEL";
dictionaryVar["de-de"]["game_ui_time_left"] = "Restzeit";
dictionaryVar["de-de"]["game_ui_TIME_TO_BEAT"] = "ZEITVORGABE";
dictionaryVar["de-de"]["game_ui_SCORE_TO_BEAT"] = "Zu schlagende Punktzahl";
dictionaryVar["de-de"]["game_ui_HIGHSCORE_break"] = "HIGHSCORE";
dictionaryVar["fr-fr"] = dictionaryVar["fr-fr"] || {};
dictionaryVar["fr-fr"]["game_ui_SCORE"] = "SCORE";
dictionaryVar["fr-fr"]["game_ui_STAGE"] = "SCÈNE";
dictionaryVar["fr-fr"]["game_ui_LIVES"] = "VIES";
dictionaryVar["fr-fr"]["game_ui_TIME"] = "TEMPS";
dictionaryVar["fr-fr"]["game_ui_HIGHSCORE"] = "MEILLEUR SCORE";
dictionaryVar["fr-fr"]["game_ui_LEVEL"] = "NIVEAU";
dictionaryVar["fr-fr"]["game_ui_time_left"] = "Temps restant";
dictionaryVar["fr-fr"]["game_ui_TIME_TO_BEAT"] = "TEMPS À BATTRE";
dictionaryVar["fr-fr"]["game_ui_SCORE_TO_BEAT"] = "SCORE À BATTRE";
dictionaryVar["fr-fr"]["game_ui_HIGHSCORE_break"] = "MEILLEUR\nSCORE";
dictionaryVar["pt-br"] = dictionaryVar["pt-br"] || {};
dictionaryVar["pt-br"]["game_ui_SCORE"] = "PONTOS";
dictionaryVar["pt-br"]["game_ui_STAGE"] = "FASE";
dictionaryVar["pt-br"]["game_ui_LIVES"] = "VIDAS";
dictionaryVar["pt-br"]["game_ui_TIME"] = "TEMPO";
dictionaryVar["pt-br"]["game_ui_HIGHSCORE"] = "RECORDE";
dictionaryVar["pt-br"]["game_ui_LEVEL"] = "NÍVEL";
dictionaryVar["pt-br"]["game_ui_time_left"] = "Tempo restante";
dictionaryVar["pt-br"]["game_ui_TIME_TO_BEAT"] = "HORA DE ARRASAR";
dictionaryVar["pt-br"]["game_ui_SCORE_TO_BEAT"] = "RECORDE A SER SUPERADO";
dictionaryVar["pt-br"]["game_ui_HIGHSCORE_break"] = "RECORDE";
dictionaryVar["es-es"] = dictionaryVar["es-es"] || {};
dictionaryVar["es-es"]["game_ui_SCORE"] = "PUNTOS";
dictionaryVar["es-es"]["game_ui_STAGE"] = "FASE";
dictionaryVar["es-es"]["game_ui_LIVES"] = "VIDAS";
dictionaryVar["es-es"]["game_ui_TIME"] = "TIEMPO";
dictionaryVar["es-es"]["game_ui_HIGHSCORE"] = "RÉCORD";
dictionaryVar["es-es"]["game_ui_LEVEL"] = "NIVEL";
dictionaryVar["es-es"]["game_ui_time_left"] = "Tiempo restante";
dictionaryVar["es-es"]["game_ui_TIME_TO_BEAT"] = "TIEMPO OBJETIVO";
dictionaryVar["es-es"]["game_ui_SCORE_TO_BEAT"] = "PUNTUACIÓN OBJETIVO";
dictionaryVar["es-es"]["game_ui_HIGHSCORE_break"] = "RÉCORD";
dictionaryVar["tr-tr"] = dictionaryVar["tr-tr"] || {};
dictionaryVar["tr-tr"]["game_ui_SCORE"] = "SKOR";
dictionaryVar["tr-tr"]["game_ui_STAGE"] = "BÖLÜM";
dictionaryVar["tr-tr"]["game_ui_LIVES"] = "HAYATLAR";
dictionaryVar["tr-tr"]["game_ui_TIME"] = "SÜRE";
dictionaryVar["tr-tr"]["game_ui_HIGHSCORE"] = "YÜKSEK SKOR";
dictionaryVar["tr-tr"]["game_ui_LEVEL"] = "SEVİYE";
dictionaryVar["tr-tr"]["game_ui_time_left"] = "Kalan zaman";
dictionaryVar["tr-tr"]["game_ui_TIME_TO_BEAT"] = "BİTİRME ZAMANI";
dictionaryVar["tr-tr"]["game_ui_SCORE_TO_BEAT"] = "BİTİRME PUANI";
dictionaryVar["tr-tr"]["game_ui_HIGHSCORE_break"] = "YÜKSEK\nSKOR";
dictionaryVar["ru-ru"] = dictionaryVar["ru-ru"] || {};
dictionaryVar["ru-ru"]["game_ui_SCORE"] = "РЕЗУЛЬТАТ";
dictionaryVar["ru-ru"]["game_ui_STAGE"] = "ЭТАП";
dictionaryVar["ru-ru"]["game_ui_LIVES"] = "ЖИЗНИ";
dictionaryVar["ru-ru"]["game_ui_TIME"] = "ВРЕМЯ";
dictionaryVar["ru-ru"]["game_ui_HIGHSCORE"] = "РЕКОРД";
dictionaryVar["ru-ru"]["game_ui_LEVEL"] = "УРОВЕНЬ";
dictionaryVar["ru-ru"]["game_ui_time_left"] = "Time left";
dictionaryVar["ru-ru"]["game_ui_TIME_TO_BEAT"] = "TIME TO BEAT";
dictionaryVar["ru-ru"]["game_ui_SCORE_TO_BEAT"] = "SCORE TO BEAT";
dictionaryVar["ru-ru"]["game_ui_HIGHSCORE_break"] = "РЕКОРД";
dictionaryVar["ar-eg"] = dictionaryVar["ar-eg"] || {};
dictionaryVar["ar-eg"]["game_ui_SCORE"] = "النتيجة";
dictionaryVar["ar-eg"]["game_ui_STAGE"] = "مرحلة";
dictionaryVar["ar-eg"]["game_ui_LIVES"] = "عدد المحاولات";
dictionaryVar["ar-eg"]["game_ui_TIME"] = "الوقت";
dictionaryVar["ar-eg"]["game_ui_HIGHSCORE"] = "أعلى نتيجة";
dictionaryVar["ar-eg"]["game_ui_LEVEL"] = "مستوى";
dictionaryVar["ar-eg"]["game_ui_time_left"] = "Time left";
dictionaryVar["ar-eg"]["game_ui_TIME_TO_BEAT"] = "TIME TO BEAT";
dictionaryVar["ar-eg"]["game_ui_SCORE_TO_BEAT"] = "SCORE TO BEAT";
dictionaryVar["ar-eg"]["game_ui_HIGHSCORE_break"] = "أعلى نتيجة";
dictionaryVar["ko-kr"] = dictionaryVar["ko-kr"] || {};
dictionaryVar["ko-kr"]["game_ui_SCORE"] = "점수";
dictionaryVar["ko-kr"]["game_ui_STAGE"] = "스테이지";
dictionaryVar["ko-kr"]["game_ui_LIVES"] = "기회";
dictionaryVar["ko-kr"]["game_ui_TIME"] = "시간";
dictionaryVar["ko-kr"]["game_ui_HIGHSCORE"] = "최고 점수";
dictionaryVar["ko-kr"]["game_ui_LEVEL"] = "레벨";
dictionaryVar["ko-kr"]["game_ui_time_left"] = "Time left";
dictionaryVar["ko-kr"]["game_ui_TIME_TO_BEAT"] = "TIME TO BEAT";
dictionaryVar["ko-kr"]["game_ui_SCORE_TO_BEAT"] = "SCORE TO BEAT";
dictionaryVar["ko-kr"]["game_ui_HIGHSCORE_break"] = "최고 점수";
dictionaryVar["jp-jp"] = dictionaryVar["jp-jp"] || {};
dictionaryVar["jp-jp"]["game_ui_SCORE"] = "スコア";
dictionaryVar["jp-jp"]["game_ui_STAGE"] = "ステージ";
dictionaryVar["jp-jp"]["game_ui_LIVES"] = "ライフ";
dictionaryVar["jp-jp"]["game_ui_TIME"] = "タイム";
dictionaryVar["jp-jp"]["game_ui_HIGHSCORE"] = "ハイスコア";
dictionaryVar["jp-jp"]["game_ui_LEVEL"] = "レベル";
dictionaryVar["jp-jp"]["game_ui_time_left"] = "残り時間";
dictionaryVar["jp-jp"]["game_ui_TIME_TO_BEAT"] = "クリアまであと";
dictionaryVar["jp-jp"]["game_ui_SCORE_TO_BEAT"] = "クリアまであと";
dictionaryVar["jp-jp"]["game_ui_HIGHSCORE_break"] = "ハイ\nスコア";



//******************************************************
//*****    Script Resource: framework_standalone_settings
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************
/* exported tgSettingsVar */
var tgSettingsVar = {};
/* exported initTGSettings */
var initTGSettings = function(){
	"use strict";
	tgSettingsVar = {
		languages: {
			initial: "en-us",
			available: ["en-us", "en-gb", "nl-nl", "de-de", "fr-fr", "pt-br", "es-es", "tr-tr", "ru-ru", "ar-eg", "ko-kr", "jp-jp"]
		},
		canvasses: {
			defaultHeight: tgScale(1040),
			defaultImageHeight: tgScale(960),
			defaultWidth: tgScale(640),
			defaultImageWidth: tgScale(640),
			defaultTop: tgScale(0),
			minTop: tgScale(-80),
			defaultLeft: 0,
			minHeight: tgScale(780),
			canvasBackground: {
				id: "canvasBackground",
				depth: 50
			},
			canvasGame: {
				id: "canvasGame",
				depth: 100,
				top: tgScale(200, "round"),
				left: tgScale(40, "round"),
				width: tgScale(560, "round"),
				height: tgScale(560, "round")
			},
			canvasGameUI: {
				id: "canvasGameUI",
				depth: 150,
				top: 0,
				left: 0,
				height: tgScale(120, "round")
			},
			canvasMain: {
				id: "canvasMain",
				depth: 200
			}
		},
		canvasses_landscape: {
			defaultHeight: tgScale(640),
			defaultImageHeight: tgScale(640),
			defaultWidth: tgScale(1152),
			defaultImageWidth: tgScale(1152),
			defaultTop: tgScale(0),
			minTop: tgScale(0),
			defaultLeft: 0,
			minHeight: tgScale(640),
			minWidth: tgScale(850),
			canvasBackground: {
				id: "canvasBackground",
				depth: 50
			},
			canvasGame: {
				id: "canvasGame",
				depth: 100,
				top: tgScale(40, "round"),
				left: tgScale(296, "round"),
				width: tgScale(560, "round"),
				height: tgScale(560, "round")
			},
			canvasGameUI: {
				id: "canvasGameUI",
				depth: 150,
				top: 0,
				left: tgScale(151),
				width: tgScale(140)
			},
			canvasMain: {
				id: "canvasMain",
				depth: 200
			}
		},
		buttonDefinitions: {
			"bigPlay": {
				type: "text",
				sprite: s_btn_bigtext,
				leftWidth: tgScale(38),
				rightWidth: tgScale(99),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 46,
						small: 30
					}),
					fillColor: "#01198a",
					shadow: {
						active: true,
						color: "#7bfdff",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 46,
					small: 30
				})
			},
			"difficulty_toggle": {
				type: "toggleText",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "0",
					sprite: s_icon_toggle_easy,
					textKey: "optionsDifficulty_easy"
				}, {
					id: "1",
					sprite: s_icon_toggle_medium,
					textKey: "optionsDifficulty_medium"
				}, {
					id: "2",
					sprite: s_icon_toggle_hard,
					textKey: "optionsDifficulty_hard"
				}],
				toggleImageOffsetX: tgScale(30),
				toggleImageOffsetY: tgScale(12),
				textOffsetX: tgScale(10),
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			},
			"music_toggle": {
				type: "toggle",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "on",
					sprite: s_icon_toggle_musicon,
					textKey: "optionsMusic_on"
				}, {
					id: "off",
					sprite: s_icon_toggle_musicoff,
					textKey: "optionsMusic_off"
				}],
				toggleImageOffsetX: tgScale(30),
				toggleImageOffsetY: tgScale(12),
				textOffsetX : 0,
				textOffsetY: 2,
				textPaddingY: tgScale(30)
			},
			"sfx_toggle": {
				type: "toggle",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "on",
					sprite: s_icon_toggle_fxon,
					textKey: "optionsSFX_on"
				}, {
					id: "off",
					sprite: s_icon_toggle_fxoff,
					textKey: "optionsSFX_off"
				}],
				toggleImageOffsetX: tgScale(30),
				toggleImageOffsetY: tgScale(12),
				textOffsetX : 0,
				textOffsetY: 2,
				textPaddingY: tgScale(30)
			},
			"music_big_toggle": {
				type: "toggleText",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "on",
					sprite: (typeof s_icon_toggle_music_on !== "undefined")? s_icon_toggle_music_on : undefined,
					textKey: "optionsMusic_on"
				}, {
					id: "off",
					sprite: (typeof s_icon_toggle_music_off !== "undefined")? s_icon_toggle_music_off : undefined,
					textKey: "optionsMusic_off"
				}],
				toggleImageOffsetX: tgScale(28, "round"),
				toggleImageOffsetY: tgScale(10),
				textOffsetX : tgScale(10),
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			},
			"sfx_big_toggle": {
				type: "toggleText",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "on",
					sprite: (typeof s_icon_toggle_sfx_on !== "undefined")? s_icon_toggle_sfx_on : undefined,
					textKey: "optionsSFXBig_on"
				}, {
					id: "off",
					sprite: (typeof s_icon_toggle_sfx_off !== "undefined")? s_icon_toggle_sfx_off : undefined,
					textKey: "optionsSFXBig_off"
				}],
				toggleImageOffsetX: tgScale(33, "round"),
				toggleImageOffsetY: tgScale(12),
				textOffsetX : tgScale(10),
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			},
			"language_toggle": {
				type: "toggleText",
				sprite: s_btn_toggle,
				leftWidth: tgScale(106),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				states: [{
					id: "en-us",
					sprite: s_flagIcon_us,
					textKey: "optionsLang_en-us"
				}, {
					id: "en-gb",
					sprite: s_flagIcon_gb,
					textKey: "optionsLang_en-gb"
				}, {
					id: "nl-nl",
					sprite: s_flagIcon_nl,
					textKey: "optionsLang_nl-nl"
				}, {
					id: "de-de",
					sprite: s_flagIcon_de,
					textKey: "optionsLang_de-de"
				}, {
					id: "fr-fr",
					sprite: s_flagIcon_fr,
					textKey: "optionsLang_fr-fr"
				}, {
					id: "pt-br",
					sprite: s_flagIcon_br,
					textKey: "optionsLang_pt-br"
				}, {
					id: "es-es",
					sprite: s_flagIcon_es,
					textKey: "optionsLang_es-es"
				}, {
					id: "ru-ru",
					sprite: s_flagIcon_ru,
					textKey: "optionsLang_ru-ru"
				}, {
					id: "ar-eg",
					sprite: s_flagIcon_ar,
					textKey: "optionsLang_ar-eg"
				}, {
					id: "ko-kr",
					sprite: s_flagIcon_kr,
					textKey: "optionsLang_ko-kr"
				}, {
					id: "tr-tr",
					sprite: s_flagIcon_tr,
					textKey: "optionsLang_tr-tr"
				}, {
					id: "jp-jp",
					sprite: s_flagIcon_jp,
					textKey: "optionsLang_jp-jp"
				}],
				toggleImageOffsetX: tgScale(40),
				toggleImageOffsetY: tgScale(20),
				textOffsetX: tgScale(10),
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			},
			"default_text": {
				type: "text",
				sprite: s_btn_standard,
				leftWidth: tgScale(40),
				rightWidth: tgScale(40),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			},
			"default_image": {
				type: "image",
				sprite: s_btn_standard,
				leftWidth: tgScale(40),
				rightWidth: tgScale(40),
				textPaddingY: tgScale(6)
			},
			"options": {
				type: "image",
				sprite: s_btn_small_options
			}
		},
		buttonDefinitions_landscape: {
			"bigPlay": {
				type: "text",
				sprite: s_btn_bigtext,
				leftWidth: tgScale(40),
				rightWidth: tgScale(76),
				font: {
					align: "center",
					baseline: "middle",
					fontSize: tgSelect({
						big: 40,
						small: 20
					}),
					fillColor: "#01198a",
					shadow: {
						active: true,
						color: "#7bfdff",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				},
				textOffsetY: 2,
				textPaddingY: tgScale(30),
				fontSize: tgSelect({
					big: 40,
					small: 20
				})
			}
		},
		buttonColorDefinitions: {
			"green": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#018a17",
					shadow: {
						active: true,
						color: "#d2ff7b",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"blue": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#01198a",
					shadow: {
						active: true,
						color: "#7bfdff",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"bluegreen": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#004f89",
					shadow: {
						active: true,
						color: "#7bffca",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"orange": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#9a1900",
					shadow: {
						active: true,
						color: "#ffb986",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"orangeyellow": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#8d2501",
					shadow: {
						active: true,
						color: "#ffbe60",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"pink": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#c6258f",
					shadow: {
						active: true,
						color: "#ffbde9",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},
			"white": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#ffffff"
				}
			},
			 "pastel_pink": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#83574f"
				}
			},
			  "whiteWithRedBorder": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#ffffff",
					shadow: {
						active: true,
						color: "#4c0200",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			},            
			"whiteWithBlueBorder": {
				font: {
					align: "center",
					baseline: "middle",
					fillColor: "#ffffff",
					shadow: {
						active: true,
						color: "#002534",
						offsetX: 0,
						offsetY: 2,
						blur: 0
					}
				}
			}
		},
		buttons: {
			"default_color": "green"
		},
		analytics: {
			timerPoolSize: 20
		},
		loadingScreen: {
			backgroundImage: typeof s_screen_start !== "undefined"? s_screen_start : undefined,
			minimalVisibilityDuration: 0,
			backgroundFadeDuration: 500,
			maxAudioLoadDuration: 5000,
			maxWaitDuration: 5000,
			splashScreenVisibilityDuration: -1,
			// throbber position and animation variables
			throbberFrames: 12,
			throbberFrameDuration: 100,
			throbberWidth: tgScale(78),
			throbberX: {
				align: "center"
			},
			throbberY: tgScale(560),
			// loadingbar position and animations variables
			loadingBarInnerWidth: tgScale(400),
			loadingBarX: {
				align: "center"
			},
			loadingBarY: tgScale(680),
			loadingBarBackgroundLeftWidth: tgScale(16),
			loadingBarBackgroundRightWidth: tgScale(18),
			loadingBarFillLeftWidth: tgScale(8),
			loadingBarFillRightWidth: tgScale(8),
			loadingBarFillXOffset: tgScale(9),
			loadingBarFillYOffset: tgScale(9),
			// text position and appearance variables
			textFont: {
				align: "center",
				fillColor: "#3B0057",
				fontSize: tgScale(24)
			},
			textX: {
				align: "center"
			},
			textY: tgScale(620),
			textMaxWidth: tgScale(500),
			// logos positions and animation variables
			logosAlign: "center",
			logosSurfaceMaxWidth: tgScale(500),
			logosSurfaceMaxHeight: tgScale(60),
			logosSurfaceX: {
				align: "center"
			},
			logosSurfaceY: {
				align: "bottom",
				offset: tgScale(20)
			},
			logosSurfaceMinY: tgScale(806),
			logosSurfaceAnimDuration: 500,
			logosSurfaceMarginBottom: tgScale(20)
		},
		loadingScreen_landscape: {
			logosAlign: "right",
			throbberY: tgScale(280),
			loadingBarY: tgScale(430),
			logosSurfaceMaxWidth: tgScale(340),
			logosSurfaceX: {
				align: "right",
				offset: tgScale(32)
			},
			logosSurfaceY: tgScale(560),
			logosSurfaceMinY: tgScale(560)
		},
		startScreen: {
			btnMinCanvasHeight: tgScale(860),
			backgroundImage: (typeof s_screen_start !== undefined)? s_screen_start : undefined,
			// timing
			gameLogoDelay: 700,
			optinalButtonsDelay: 1800,
			menuButtonDelay: 700,
			playButtonsDelay: 2600,
			// game logo position and animation
			gameLogoImage: (typeof s_screen_start !== undefined)? s_logo : undefined,
			gameLogoAnimationDuration: 700,
			gameLogoStartX: {
				align: "center"
			},
			gameLogoEndX: {
				align: "center"
			},
			gameLogoStartY: (typeof s_logo !== undefined)? -s_logo.height : 0,
			gameLogoEndY: {
				align: "top",
				offset: tgScale(20)
			},
			gameLogoStartScaleX: 1,
			gameLogoEndScaleX: 1,
			gameLogoStartScaleY: 1,
			gameLogoEndScaleY: 1,
			gameLogoStartAlpha: 1,
			gameLogoEndAlpha: 1,
			gameLogoEaseX: tbEase.easeLinear,
			gameLogoEaseY: tbEase.easeOutOvershoot,
			gameLogoEaseScaleX: tbEase.easeLinear,
			gameLogoEaseScaleY: tbEase.easeLinear,
			gameLogoEaseAlpha: tbEase.easeLinear,
			// play button position and animation
			playButtonX: {
				align: "center"
			},
			playButtonY: tgScale(656),
			playButtonWidth: tgScale(300),
			playButtonAnimationDuration: 700,
			playButtonWiggleDelay: 700,
			// difficulty button position and animation
			difficultyButtonX: tgScale(368),
			difficultyButtonY: tgScale(796),
			difficultyButtonWidth: tgScale(440),
			difficultyButtonAnimationDuration: 700,
			// menu button position and animation
			menuButtonX: tgScale(36),
			menuButtonY: tgScale(750),
			menuButtonAnimationDuration: 500,
			// company logo positions and animation variables
			logosAlign: "center",
			logosSurfaceMaxWidth: tgScale(500),
			logosSurfaceMaxHeight: tgScale(60),
			logosSurfaceX: {
				align: "center"
			},
			logosSurfaceY: {
				align: "bottom",
				offset: tgScale(20)
			},
			logosSurfaceMinY: tgScale(806),
			logosSurfaceAnimDuration: 500,
			logosSurfaceMarginBottom: tgScale(20)
		},
		startScreen_landscape: {
			btnMinCanvasHeight: tgScale(0),
			playButtonY: tgScale(456),
			playButtonWidth: tgScale(320),
			difficultyButtonX: {
				align: "center"
			},
			difficultyButtonY: tgScale(346),
			difficultyButtonWidth: tgScale(460),
			menuButtonX: {
				align: "left",
				offset: tgScale(32)
			},
			menuButtonY: tgScale(528),
			logosAlign: "right",
			logosSurfaceMaxWidth: tgScale(340),
			logosSurfaceX: {
				align: "right",
				offset: tgScale(32)
			},
			logosSurfaceY: tgScale(560),
			logosSurfaceMinY: tgScale(560)
		},
		levelMapScreen: {
			optionsBtnX: {
				align: "center",
				offset: tgScale(-230)
			},
			optionsBtnY: {
				align: "top",
				offset: tgScale(576)
			},
			optionsBtnType: "options",
			titleFont: {
				baseline: "bottom"
			},
			titleX: {
				align: "center"
			},
			titleY: {
				align: "top",
				offset: tgScale(35, "round")
			},
			titleMaxWidth: tgScale(232),
			titleMaxHeight: tgScale(98),
			worldButtonPreviousX: {
				align: "center",
				offset: tgScale(-206)
			},
			worldButtonPreviousY: {
				align: "top",
				offset: tgScale(30)
			},
			worldButtonNextX: {
				align: "center",
				offset: tgScale(206)
			},
			worldButtonNextY: {
				align: "top",
				offset: tgScale(30)
			},
			type: "grid",
			noOfColsBig: 3,
			noOfRowsBig: 3,
			noOfColsSmall: 5,
			noOfRowsSmall: 4,
			drawNumbers: true,
			drawBackgroundImages: true,
			gridOffsetX: tgScale(78),
			gridOffsetYBig: {
				align: "top",
				offset: tgScale(140)
			},
			gridOffsetYSmall: {
				align: "top",
				offset: tgScale(140)
			},
			gridOffsetYDifferenceNoWorldButtons: tgScale(20),
			gridMarginYBig: tgScale(18),
			gridMarginYSmall: tgScale(18),
			levelFontsBig: {
				general: {
					fontSize: tgSelect({
						big: 60,
						small: 30
					}),
					fillColor: "#3F4F5E",
					align: "center",
					baseline: "middle",
					shadow: {
						active: true,
						color: "#D0D8EA",
						offsetX: 0,
						offsetY: tgScale(6),
						blur: 0
					}
				}
			},
			levelFontsSmall: {
				general: {
					fontSize: tgSelect({
						big: 32,
						small: 16
					}),
					fillColor: "#3F4F5E",
					align: "center",
					baseline: "middle",
					shadow: {
						active: true,
						color: "#D0D8EA",
						offsetX: 0,
						offsetY: tgScale(2),
						blur: 0
					}
				}
			},
			levelsMaxHeightBig: tgScale(438),
			levelsMaxHeightSmall: tgScale(438),
			levelImageOffsetX: {
				align: "center"
			},
			levelImageOffsetY: {
				align: "center"
			},
			lockOffsetX: {
				align: "center"
			},
			lockOffsetY: {
				align: "center",
				offset: tgScale(-22)
			},
			levelNoOffsetX: {
				align: "center"
			},
			levelNoOffsetY: {
				align: "center",
				offset: tgScale(-10)
			},
			starsOverviewX: {
				align: "center",
				offset: tgScale(216)
			},
			starsOverviewY: {
				align: "top",
				offset: tgScale(574)
			},
			starsOverviewFont: {
				fontSize: tgSelect({
					big: 24,
					small: 12
				}),
				fillColor: "#3F4F5E",
				align: "center"
			},
			starsTextOffsetY: tgScale(10),
			pointOverviewFont: {
				fontSize: tgSelect({
					big: 24,
					small: 12
				}),
				fillColor: "#3F4F5E",
				align: "center"
			},
			pointOverviewX: {
				align: "center"
			},
			pointOverviewY: {
				align: "top",
				offset: tgScale(588)
			},
			pointOverviewMaxWidth: tgScale(160),
			pointOverviewMaxHeight: tgScale(40),
			backgroundImage: (typeof s_screen_levelselect !== "undefined")? s_screen_levelselect : undefined,
			swipeMinDistance: tgScale(10),
			swipeSpeedThreshold: 200,
			swipeDistanceThreshold: tgScale(200),
			levelsMoveSpeed: tgScale(600),
			moveTotalDuration: 800,
			moveMinDuration: 500
		},
		levelMapScreen_landscape: {
			worldButtonPreviousY: {
				align: "top",
				offset: tgScale(20)
			},
			worldButtonNextY: {
				align: "top",
				offset: tgScale(20)
			},
			titleY: {
				align: "top",
				offset: tgScale(25, "round")
			},
			gridOffsetX: tgScale(234),
			gridOffsetYBig: {
				align: "top",
				offset: tgScale(110)
			},
			gridOffsetYSmall: {
				align: "top",
				offset: tgScale(110)
			},
			starsOverviewY: {
				align: "top",
				offset: tgScale(536)
			},
			pointOverviewY: {
				align: "top",
				offset: tgScale(550)
			},
			optionsBtnY: {
				align: "top",
				offset: tgScale(538)
			}
		},
		levelStart: {
			overlayImage: (typeof s_overlay_assignment !== "undefined")? s_overlay_assignment : undefined,
			overlayImageX: {
				align: "center"
			},
			overlayImageY: (typeof s_overlay_assignment !== "undefined")? -s_overlay_assignment.height : undefined,
			overlayAnimationIn: [{
				type: "y",
				delay: 0,
				duration: 800,
				end: {
					align: "center",
					offset: tgScale(-142)
				},
				ease: tbEase.easeOutOvershoot,
				sound: a_levelStart
			}],
			overlayAnimationOut: [{
				type: "y",
				delay: 0,
				duration: 600,
				end: (typeof s_overlay_assignment !== "undefined")? -s_overlay_assignment.height : undefined,
				ease: tbEase.easeInOvershoot,
				blocking: true
			}],
			assignmentFont: {
				align: "center",
				baseline: "middle"
			},
			assignmentOffsetX: {
				align: "center"
			},
			assignmentOffsetY: 0,
			assignmentMaxWidth: tgScale(500),
			assignmentMaxHeight: tgScale(80),
			headerFont: {
				align: "center",
				baseline: "middle"
			},
			headerOffsetX: {
				align: "center"
			},
			headerOffsetY: 0,
			headerMaxWidth: tgScale(560),
			headerMaxHeight: tgScale(80),
			pauseDuration: 3500
		},
		levelStart_landscape: {
			overlayAnimationIn: [{
				type: "y",
				delay: 0,
				duration: 800,
				end: {
					align: "center"
				},
				ease: tbEase.easeOutOvershoot,
				sound: a_levelStart
			}]
		},
		challengeStart: {
			overlayImage: (typeof s_overlay_challenge_start !== "undefined")? s_overlay_challenge_start : undefined,
			overlayImageX: {
				align: "center"
			},
			overlayImageY: tgScale(56),
			overlayOffsetX: 0,
			overlayOffsetY: 0,
			titleFont: {
				align: "center",
				baseline: "top"
			},
			titleMaxWidth: tgScale(500),
			titleMaxHeight: tgScale(100),
			titleX: {
				align: "center"
			},
			titleY: tgScale(90),
			nameFont: {
				align: "center",
				baseline: "middle"
			},
			nameMaxWidth: tgScale(500),
			nameMaxHeight: tgScale(80),
			nameX: {
				align: "center"
			},
			nameY: tgScale(250),
			scoreHeaderFont: {
				align: "center",
				baseline: "top"
			},
			scoreHeaderMaxWidth: tgScale(500),
			scoreHeaderMaxHeight: tgScale(40),
			scoreHeaderX: {
				align: "center"
			},
			scoreHeaderY: tgScale(348),
			scoreFont: {
				align: "center",
				baseline: "top"
			},
			scoreMaxWidth: tgScale(500),
			scoreMaxHeight: tgScale(50),
			scoreX: {
				align: "center"
			},
			scoreY: tgScale(388),
			winHeaderFont: {
				align: "center",
				baseline: "top"
			},
			winHeaderMaxWidth: tgScale(500),
			winHeaderMaxHeight: tgScale(40),
			winHeaderX: {
				align: "center"
			},
			winHeaderY: tgScale(442),
			winHeaderOffsetX: 0,
			winHeaderOffsetY: 0,
			winFont: {
				align: "center",
				baseline: "top"
			},
			winMaxWidth: tgScale(500),
			winMaxHeight: tgScale(50),
			winX: {
				align: "center"
			},
			winY: tgScale(482),
			winBetweenMargin: tgScale(10),
			winAmountOffsetX: 0,
			winAmountOffsetY: 0,
			animInDuration: 800,
			animInEase: tbEase.easeOutOvershoot,
			animOutDuration: 600,
			animOutEase: tbEase.easeInOvershoot,
			pauseDuration: 3500
		},
		challengeEnd: {
			animInDelay: 500,
			animInDuration: 800,
			namesShowDelay: 1500,
			namesShowDuration: 500,
			scoreShowDelay: 2500,
			scoresShowDuration: 500,
			scoresCountDelay: 3200,
			scoresCountDuration: 800,
			messageShowDelay: 4200,
			messageShowDuration: 300,
			buttonsShowDelay: 4500,
			overlayStartX: {
				align: "center"
			},
			overlayStartY: tgScale(-800),
			overlayEndX: {
				align: "center"
			},
			overlayEndY: tgScale(52),
			overlayOffsetX: 0,
			overlayOffsetY: 0,
			fadeAlpha: 0.8,
			fadeColor: "#000000",
			messageFont: {
				align: "center",
				baseline: "middle"
			},
			messageWidth: tgScale(360),
			messageHeight: tgScale(120),
			messageMarginX: tgScale(4),
			messageMarginY: tgScale(4),
			messageX: {
				align: "center"
			},
			messageY: tgScale(340),
			subMessageX: {
				align: "center"
			},
			subMessageY: tgScale(600),
			subMessageWidth: tgScale(500),
			subMessageHeight: tgScale(120),
			subMessageFont: {
				align: "center",
				baseline: "middle"
			},
			winningsFont: {
				align: "center",
				baseline: "middle"
			},
			winningsWidth: tgScale(360),
			winningsHeight: tgScale(60),
			winningsMarginX: tgScale(4),
			winningsMarginY: tgScale(4),
			winningsX: {
				align: "center"
			},
			winningsY: tgScale(480),
			textButtonsWidth: tgScale(460),
			submitBtnX: {
				align: "center"
			},
			submitBtnY: tgScale(400),
			cancelBtnX: {
				align: "center"
			},
			cancelBtnY: tgScale(500),
			namesFont: {
				align: "center",
				baseline: "middle"
			},
			namesLeftX: tgScale(75, "round"),
			namesBetweenX: tgScale(48),
			namesY: tgScale(120),
			nameWidth: tgScale(214, "round"),
			nameHeight: tgScale(40),
			nameMarginX: tgScale(4),
			nameMarginY: tgScale(4),
			nameOffsetX: 0,
			nameOffsetY: 0,
			emphasizedScoresFont: {
				align: "center",
				baseline: "middle"
			},
			emphasizedScoresY: tgScale(220),
			emphasizedScoresWidth: tgScale(180),
			emphasizedScoresHeight: tgScale(80),
			emphasizedScoresCounterMarginX: tgScale(4),
			emphasizedScoresCounterMarginY: tgScale(4)

		},
		levelEnd: {
			overlayImages: {
				difficulty: (typeof s_overlay_difficulty !== "undefined")? s_overlay_difficulty : undefined,
				endless: (typeof s_overlay_endless !== "undefined")? s_overlay_endless : undefined,
				levelWin: (typeof s_overlay_level_win !== "undefined")? s_overlay_level_win : undefined,
				levelFail: (typeof s_overlay_level_fail !== "undefined")? s_overlay_level_fail : undefined
			},
			textureSwapDelay: 500,
			animInDuration: 800,
			animInEase: tbEase.easeOutOvershoot,
			animOutDuration: 800,
			animOutEase: tbEase.easeInCubic,
			overlayImageOffsetX: {
				align: "center"
			},
			overlayImageOffsetY: 0,
			titleFont: {
				align: "center",
				baseline: "middle",
				fontSize: tgSelect({
					big: 26,
					small: 13
				})
			},
			titleX: {
				align: "center"
			},
			titleY: tgScale(58),
			titleMaxWidth: tgScale(500),
			titleMaxHeight: tgScale(100),
			subTitleFont: {
				align: "center",
				baseline: "middle",
				fontSize: tgSelect({
					big: 56,
					small: 28
				})
			},
			subTitleX: {
				align: "center"
			},
			subTitleY: tgScale(236),
			emphasizedStatsHeaderFont: {
				align: "center",
				baseline: "top",
				fontSize: tgSelect({
					big: 24,
					small: 12
				})
			},
			emphasizedStatsHeaderX: {
				align: "center"
			},
			emphasizedStatsHeaderY: tgScale(144),
			emphasizedStatsFont: {
				align: "center",
				baseline: "top",
				fontSize: tgSelect({
					big: 56,
					small: 28
				})
			},
			emphasizedStatsX: {
				align: "center"
			},
			emphasizedStatsY: tgScale(176),
			emphasizedStatsMaxWidth: tgScale(200),
			emphasizedStatsMaxHeight: tgScale(60),
			statsHeaderFont: {
				align: "center",
				baseline: "top",
				fontSize: tgSelect({
					big: 24,
					small: 12
				})
			},
			statsHeaderX: {
				align: "center"
			},
			statsHeaderY: tgScale(286),
			statsInlineOffset: tgScale(0),
			forceStatsInline: false,
			statsHeaderMarginBottom: tgScale(14),
			statsHeaderMarginRight: tgScale(10),
			statsFont: {
				align: "center",
				baseline: "top",
				fontSize: tgSelect({
					big: 24,
					small: 12
				})
			},
			statsCounterMarginX: tgScale(10),
			statsCounterMarginY: tgScale(4),
			statsMaxWidth: tgScale(200),
			statsMaxHeight: tgScale(50),
			backButtonX: {
				align: "center",
				offset: tgScale(12)
			},// tgScale(284),
			backButtonY: tgScale(549),
			forwardButtonX: {
				align: "center",
				offset: tgScale(162)
			},// tgScale(404),
			forwardButtonY: tgScale(489),
			closeButtonX: {
				align: "center",
				offset: tgScale(250)
			},// tgScale(530),
			closeButtonY: tgScale(10),
			closeButtonMaskWidth: tgScale(90),
			closeButtonMaskHeight: tgScale(90),
			star1X: {
				align: "center",
				offset: tgScale(-177, "round")
			},
			star1Y: tgScale(120),
			star2X: {
				align: "center"
			},
			star2Y: tgScale(96),
			star3X: {
				align: "center",
				offset: tgScale(179, "round")
			},
			star3Y: tgScale(120),
			starEffectDelay: 200,
			starEffectDuration: 500,
			starFillDelay: 800,
			starFillStartScale: 0,
			starFillStartAlpha: 0,
			starFillScaleDuration: 300,
			starFillAlphaDuration: 200,
			starFillDuration: 300,
			fadeAlpha: 0.8,
			fadeDuration: 800,
			fadeColor: "#000000",
			medalX: tgScale(508),
			medalY: tgScale(394),
			medalImageOriginX: tgScale(96),
			medalImageOriginY: tgScale(74),
			medalScale: 3,
			medalDuration: 400,
			medalDelay: 2500,
			medalShadowDelay: 0,
			medalShadowOffsetY: tgScale(100),
			medalShadowScaleFactor: 1.5,
			medalTextOffsetX: {
				align: "center"
			},
			medalTextOffsetY: tgScale(76),
			medalTextMaxWidth: tgScale(180),
			medalTextMaxHeight: tgScale(36),
			medalTextFont: {
				align: "center",
				baseline: "middle",
				fontSize: tgSelect({
					big: 22,
					small: 12
				}),
				fontFace: "ff_opensans_extrabold",
				fillColor: "#1d347f",
				shadow: {
					active: true,
					color: "#68cbfa",
					offsetY: tgScale(2)
				}
			},
			medalEffectDuration: 500,
			medalEffectDelay: 500,
			medalEffectOffsetY: tgScale(-30),
			medalShineDuration: 500,
			medalShineDelay: 0,
			medalShineLoopInterval: 4000,
			totalScoreCounterDelay: 600,
			totalScoreCounterDuration: 1500,
			bonusCounterDelay: 500,
			bonusCounterDuration: 750,
			highScoreBtnX: {
				align: "center"
			},
			highScoreBtnY: tgScale(290),
			highScoreBtnWidth: tgScale(350),
			moreGamesOverlayDelay: 1000,
			type: {
				"level": {
					backgroundImageType: "level",
					showStars: true,
					showMedal: true,
					titleItem: "title_level",
					statsItems: "totalScore",
					backButtonType: "retry",
					forwardButtonType: "next"
				},
				"failed": {
					backgroundImageType: "failed",
					showStars: false,
					showMedal: false,
					titleItem: "title_level",
					subTitleItem: "subtitle_failed",
					backButtonType: "exit",
					forwardButtonType: "retry"
				},
				"endless": {
					backgroundImageType: "endless",
					showStars: false,
					showMedal: true,
					titleItem: "title_endless",
					emphasizedStatsItem: "totalScore",
					statsItems: "highScore",
					backButtonType: "exit",
					forwardButtonType: "retry"
				},
				"difficulty": {
					backgroundImageType: "difficulty",
					showStars: false,
					showMedal: true,
					titleItem: "title_difficulty",
					emphasizedStatsItem: "timeLeft",
					statsItems: ["totalScore", "timeBonus"],
					backButtonType: "exit",
					forwardButtonType: "retry"
				}
			}
		},
		levelEnd_landscape: {
			closeButtonY: tgScale(0),
			titleY: tgScale(30),
			emphasizedStatsHeaderY: tgScale(114),
			emphasizedStatsY: tgScale(146),
			statsHeaderY: tgScale(266),
			backButtonY: tgScale(488),
			forwardButtonY: tgScale(428),
			medalX: {
				align: "center",
				offset: tgScale(220)
			},
			medalY: tgScale(260)
		},
		inGame: {
			backgroundImage: (typeof s_background !== "undefined")? s_background : undefined
		},
		options: {
			backgroundImage: s_overlay_options,
			overlayImageOffsetX: {
				align: "center"
			},
			overlayImageOffsetY: 0,
			titleFont: {},
			titleX: {
				align: "center"
			},
			titleY: tgScale(58),
			titleMaxWidth: tgScale(500),
			titleMaxHeight: tgScale(100),
			buttonsWidth: tgScale(460, "round"),
			buttonsOffsetX: {
				align: "center"
			},
			buttonsOffsetY: {
				align: "center",
				offset: tgScale(36)
			},
			buttonsOffsetInBetween: tgScale(10, "round"),
			closeButtonX: tgScale(510),
			closeButtonY: tgScale(10),
			closeButtonMaskWidth: tgScale(130),
			closeButtonMaskHeight: tgScale(90),
			buttons: {
				"startScreen": ["tutorial", ["music", "sfx"], "language", "moreGames", "about"],
				"levelMapScreen": ["startScreen", ["music", "sfx"], "language", "moreGames", "about"],
				"inGame": ["resume", "tutorial", ["music", "sfx"], "moreGames", "quit"]
			},
			overlayAnimInDuration: 800,
			overlayAnimInEase: tbEase.easeOutOvershoot,
			overlayAnimOutDuration: 600,
			overlayAnimOutEase: tbEase.easeInCubic,
			confirmQuestionTextX: {
				align: "center"
			},
			confirmQuestionTextY: tgScale(260),
			confirmQuestionTextWidth: tgScale(460),
			confirmQuestionTextHeight: tgScale(300),
			confirmQuestionBtnYesX: {
				align: "center"
			},
			confirmQuestionBtnYesY: tgScale(460),
			confirmQuestionBtnNoX: {
				align: "center"
			},
			confirmQuestionBtnNoY: tgScale(560, "round"),
			confirmQuestionBtnsWidth: tgScale(460, "round"),
			quitConfirmFont: {},
			tutorialBackgroundImage: (typeof s_tutorial !== "undefined")? s_tutorial : undefined,
			tutorialBackgroundOffsetX: {
				align: "center"
			},
			tutorialBackgroundOffsetY: tgScale(84, "round"),
			tutorialTitleFont: {
				align: "center",
				baseline: "top"
			},
			tutorialTitleMaxWidth: tgScale(480),
			tutorialTitleMaxHeight: tgScale(46),
			tutorialTitleOffsetX: {
				align: "center"
			},
			tutorialTitleOffsetY: tgScale(110, "round"),
			tutorialImageOffsetX: {
				align: "center"
			},
			tutorialImageOffsetY: tgScale(160, "round"),
			tutorialTextOffsetX: {
				align: "center"
			},
			tutorialTextOffsetY: tgScale(446, "round"),
			tutorialFont: {
				baseline: "middle",
				align: "center",
				fontSize: tgSelect({
					big: 36,
					small: 18
				})
			},
			tutorialTextWidth: tgScale(480),
			tutorialTextHeight: tgScale(160),
			tutorialButtonPreviousX: {
				align: "center",
				offset: tgScale(-80, "round")
			},
			tutorialButtonPreviousY: tgScale(556, "round"),
			tutorialButtonNextX: {
				align: "center",
				offset: tgScale(80, "round")
			},
			tutorialButtonNextY: tgScale(556, "round"),
			aboutHeaderFont: {
				align: "center",
				baseline: "top",
				fillColor: "#3C0058",
				fontSize: tgSelect({
					big: 26,
					small: 13
				}),
				spacing: tgScale(6)
			},
			aboutHeaderMaxWidth: tgScale(480),
			aboutHeaderMaxHeight: tgScale(50),
			aboutHeaderOffsetX: {
				align: "center"
			},
			aboutHeaderOffsetY: tgScale(106, "round"),
			aboutTextFont: {
				align: "center",
				baseline: "top",
				fillColor: "#3C0058",
				fontSize: tgSelect({
					big: 26,
					small: 13
				}),
				spacing: tgScale(6)
			},
			aboutTextMaxWidth: tgScale(480),
			aboutTextMaxHeight: tgScale(110),
			aboutTextOffsetX: {
				align: "center"
			},
			aboutTextOffsetY: tgScale(396, "round"),
			aboutLogoOffsetX: {
				align: "center"
			},
			aboutLogoOffsetY: tgScale(140),
			aboutBackBtnOffsetX: {
				align: "center"
			},
			aboutBackBtnOffsetY: tgScale(500),
			aboutBackBtnWidth: tgScale(480),
			aboutVersionFont: {
				align: "center",
				baseline: "top",
				fillColor: "#808080",
				fontSize: tgSelect({
					big: 12,
					small: 8
				})
			},
			aboutVersionOffsetX: {
				align: "center"
			},
			aboutVersionOffsetY: tgScale(610),
			aboutVersionMaxWidth: tgScale(440),
			aboutVersionMaxHeight: tgScale(20),
			aboutVersionKeyAreaWidth: tgScale(200),
			aboutVersionKeyAreaHeight: tgScale(200),
			versionsOffsetX: tgScale(80),
			versionsOffsetY: tgScale(140),
			versionsMarginY: tgScale(10)
		},
		options_landscape: {
			titleY: tgScale(12),
			buttonsOffsetY: {
				align: "center",
				offset: tgScale(16)
			},
			confirmQuestionTextY: tgScale(200),
			confirmQuestionTextHeight: tgScale(300),
			confirmQuestionBtnYesY: tgScale(400),
			confirmQuestionBtnNoY: tgScale(500, "round"),
			tutorialBackgroundOffsetY: tgScale(60, "round"),
			tutorialTitleOffsetY: tgScale(80, "round"),
			tutorialImageOffsetY: tgScale(134, "round"),
			tutorialTextOffsetY: tgScale(410, "round"),
			tutorialButtonPreviousY: tgScale(500, "round"),
			tutorialButtonNextY: tgScale(500, "round"),
			aboutHeaderOffsetY: tgScale(86, "round"),
			aboutLogoOffsetY: tgScale(126),
			aboutTextOffsetY: tgScale(392, "round"),
			aboutBackBtnOffsetY: tgScale(490),
			aboutVersionOffsetY: tgScale(590)
		},
		options_challenge: {
			backgroundImage: (typeof s_overlay_challenge_options !== "undefined")? s_overlay_challenge_options : s_overlay_options,
			overlayImageOffsetX: {
				align: "center"
			},
			overlayImageOffsetY: tgScale(120),
			titleFont: {},
			titleX: {
				align: "center"
			},
			titleY: tgScale(200),
			buttonsWidth: tgScale(460, "round"),
			buttonsOffsetX: {
				align: "center"
			},
			buttonsOffsetY: {
				align: "center",
				offset: tgScale(140)
			},
			buttonsOffsetInBetween: tgScale(10, "round"),
			closeButtonX: tgScale(510),
			closeButtonY: tgScale(10),
			closeButtonMaskWidth: tgScale(130),
			closeButtonMaskHeight: tgScale(90),
			buttons: {
				"startScreen": ["tutorial", ["music", "sfx"], "language", "about"],
				"inGame_challengee": ["resume", "tutorial", ["music", "sfx"], "forfeitChallenge"],
				"inGame_challenger": ["resume", "tutorial", ["music", "sfx"], "cancelChallenge"]
			},
			overlayAnimInDuration: 800,
			overlayAnimInEase: tbEase.easeOutOvershoot,
			overlayAnimOutDuration: 600,
			overlayAnimOutEase: tbEase.easeInCubic,
			quitConfirmFont: {},
			quitConfirmTextX: {
				align: "center"
			},
			quitConfirmTextY: tgScale(360),
			quitConfirmTextWidth: tgScale(460),
			quitConfirmTextHeight: tgScale(300),
			quitConfirmBtnYesType: "default_text",
			quitConfirmBtnYesX: {
				align: "center"
			},
			quitConfirmBtnYesY: tgScale(630),
			quitConfirmBtnNoType: "default_text",
			quitConfirmBtnNoX: {
				align: "center"
			},
			quitConfirmBtnNoY: tgScale(730, "round"),
			quitConfirmBtnsWidth: tgScale(460, "round"),
			confirmQuestionFont: {},
			confirmQuestionTextX: {
				align: "center"
			},
			confirmQuestionTextY: tgScale(200),
			confirmQuestionTextWidth: tgScale(460),
			confirmQuestionTextHeight: tgScale(250),
			confirmQuestionBtnYesX: {
				align: "center"
			},
			confirmQuestionBtnYesY: tgScale(520),
			confirmQuestionBtnNoX: {
				align: "center"
			},
			confirmQuestionBtnNoY: tgScale(620, "round"),
			confirmQuestionBtnsWidth: tgScale(460, "round"),
			messageFont: {},
			messageTextX: {
				align: "center"
			},
			messageTextY: tgScale(200),
			messageTextWidth: tgScale(460),
			messageTextHeight: tgScale(300),
			tutorialBackgroundImage: (typeof s_tutorial !== "undefined")? s_tutorial : undefined,
			tutorialBackgroundOffsetX: {
				align: "center"
			},
			tutorialBackgroundOffsetY: tgScale(0, "round"),
			tutorialTitleFont: {
				align: "center",
				baseline: "top"
			},
			tutorialTitleMaxWidth: tgScale(480),
			tutorialTitleMaxHeight: tgScale(50),
			tutorialTitleOffsetX: {
				align: "center"
			},
			tutorialTitleOffsetY: tgScale(20, "round"),
			tutorialImageOffsetX: {
				align: "center"
			},
			tutorialImageOffsetY: tgScale(70, "round"),
			tutorialTextOffsetX: {
				align: "center"
			},
			tutorialTextOffsetY: tgScale(356, "round"),
			tutorialFont: {
				baseline: "middle",
				align: "center",
				fontSize: tgSelect({
					big: 36,
					small: 18
				})
			},
			tutorialTextWidth: tgScale(480),
			tutorialTextHeight: tgScale(150),
			tutorialButtonPreviousX: tgScale(224, "round"),
			tutorialButtonPreviousY: tgScale(636, "round"),
			tutorialButtonNextX: tgScale(350, "round"),
			tutorialButtonNextY: tgScale(636, "round"),
			aboutHeaderFont: {
				align: "center",
				baseline: "top",
				fillColor: "#3C0058",
				fontSize: tgSelect({
					big: 26,
					small: 13
				}),
				spacing: tgScale(6)
			},
			aboutHeaderMaxWidth: tgScale(480),
			aboutHeaderMaxHeight: tgScale(50),
			aboutHeaderOffsetX: {
				align: "center"
			},
			aboutHeaderOffsetY: tgScale(26, "round"),
			aboutTextFont: {
				align: "center",
				baseline: "top",
				fillColor: "#3C0058",
				fontSize: tgSelect({
					big: 26,
					small: 13
				}),
				spacing: tgScale(6)
			},
			aboutTextMaxWidth: tgScale(480),
			aboutTextMaxHeight: tgScale(110),
			aboutTextOffsetX: {
				align: "center"
			},
			aboutTextOffsetY: tgScale(316, "round"),
			aboutLogoOffsetX: {
				align: "center"
			},
			aboutLogoOffsetY: tgScale(60),
			aboutBackBtnOffsetX: {
				align: "center"
			},
			aboutBackBtnOffsetY: tgScale(420),
			aboutBackBtnWidth: tgScale(480),
			aboutVersionFont: {
				align: "center",
				baseline: "top",
				fillColor: "#808080",
				fontSize: tgSelect({
					big: 12,
					small: 8
				})
			},
			aboutVersionOffsetX: {
				align: "center"
			},
			aboutVersionOffsetY: tgScale(530),
			aboutVersionMaxWidth: tgScale(440),
			aboutVersionMaxHeight: tgScale(20),
			aboutVersionKeyAreaWidth: tgScale(200),
			aboutVersionKeyAreaHeight: tgScale(200),
			versionsOffsetX: tgScale(80),
			versionsOffsetY: tgScale(100),
			versionsMarginY: tgScale(10)
		},
		dialog: {
			backgroundImage: typeof s_overlay_dialog !== "undefined"? s_overlay_dialog : s_overlay_options,
			overlayImageOffsetX: {
				align: "center"
			},
			overlayImageOffsetY: tgScale(120),
			buttonsWidth: tgScale(460, "round"),
			buttonsOffsetX: {
				align: "center"
			},
			buttonsOffsetY: {
				align: "bottom",
				offset: tgScale(20)
			},
			buttonsOffsetInBetween: tgScale(10, "round"),

			closeButtonX: tgScale(510),
			closeButtonY: tgScale(10),
			closeButtonMaskWidth: tgScale(130),
			closeButtonMaskHeight: tgScale(90),

			overlayAnimInDuration: 800,
			overlayAnimInEase: tbEase.easeOutOvershoot,
			overlayAnimOutDuration: 600,
			overlayAnimOutEase: tbEase.easeInCubic,
			questionFont: {},
			questionTextX: {
				align: "center"
			},
			questionTextY: {
				align: "center",
				offset: tgScale(40)
			},
			questionTextWidth: tgScale(460),
			questionTextHeight: tgScale(300),
			subQuestionFont: {},
			questionSubTextX: {
				align: "center"
			},
			questionSubTextY: {
				align: "center",
				offset: tgScale(160)
			},
			questionSubTextWidth: tgScale(460),
			questionSubTextHeight: tgScale(200)
		},
		gameEndScreen: {
			backgroundImage: (typeof s_screen_end !== "undefined")? s_screen_end : undefined,
			textX: {
				align: "center"
			},
			textY: tgScale(152),
			textMaxWidth: tgScale(560),
			textMaxHeight: tgScale(560),
			font: {
				align: "center",
				baseline: "middle",
				fontSize: tgSelect({
					big: 52,
					small: 26
				}),
				fillColor: "#FFFFFF"
			},
			btnX: {
				align: "center"
			},
			btnY: tgScale(600),
			btnWidth: tgScale(460),
			btnType: "default_text"
		},
		gameEndScreen_landscape: {
			btnY: tgScale(520)
		}
	};
};



//******************************************************
//*****    Script Resource: config_general
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************
/* exported game_configuration */
var configuration_general = {
	publisherId: "boostermedia",
	track: false,
	siteLock: {
		lock: false,
		domains: [
			"8784d238b5a81ab432d0bf303850ed8a",
			"9f5ffc7a10e0bad054458b089947ce2f",
			"0c73b027c77dd2c1ee6cd996c2c83efc", 
			"0e26dbc8d26a0880786e55e3e950a320", //http://games.gamesplaza.com/
			"7e77fcbd9a8ec327db79b6c753a5eef1", //http://cdn.mobinozer.com/
			"5ff97ceeea117a01f512adc7ece5ade7", //http://games.nolimitsgames.com/
			"54f5ededff723ca5c52bc75764ceb738", //http://cdn.nolimitsgames.com/
			"526ceb1f05b2418711e1eb59058e0b09", //http://games.jeux.com/
			"ae11a4449bebb3561a078bac793dc74b", //http://games.connectedgamestore.com/
			"f51482fb5ecee35fd139f89728a7a616" //http://games.bmmob.com
		]
	},
	languages: {
		initial: "en-us",
		available: ["en-us"]
	}
};



//******************************************************
//*****    Script Resource: game_settings
//******************************************************

//////////////////////////////////////////////////////
// Settings file for game bejeweled_timed           //
// Created from levels file levels.tblev            //
//                                                  //
// Created by the Tingly Level Designer             //
//     on 07/05/2015 14:37:36                       //
//////////////////////////////////////////////////////

// Directives for jshint
/* global tgScale */
/* exported gameSettingsVar */
/* exported initGameSettings */

var gameSettingsVar = null;

var initGameSettings = function() {
	"use strict";
	gameSettingsVar = {

		// System settings
		animationController : {
			cubeAnimationSpeed : 50,                       // No of milliseconds per frame of hypercube animation
			fadeDuration : 200,                            // Time in milliseconds during which the field fades to black when nova is activated
			lightningDuration : 700,                       // Duration of hypercube lightning animation in milliseconds
			lightningScaleDuration : 200,                  // Duration in milliseconds during which lightning spreads across grid when hypercube is activated
			novaAnimationSpeed : 50,                       // No of milliseconds per frame of nova animation
			novaScaleDuration : 200,                       // Duration in milliseconds during which lightning spreads across grid when nova is activated
			novaTotDuration : 1500                         // Total duration of nova effect in milliseconds
		},
		floaters : {
			floaterDelayForBonus : 800,                    // Duration floater delay if time bonus is also shown (ms)
			floaterNumberDuration : 900,                   // Duration in milliseconds a number floater will be visible
			floaterNumberMovementY : tgScale(-80),         // Floater movement in y direction
			floaterTextAppearDuration : 200,               // Duration of floater scale up animation (ms)
			floaterTextDisappearDuration : 1500,           // Duration of floater scale down animation (ms)
			floaterTextMovementY : tgScale(-60),           // Floater movement in y direction
			floaterTextVisibleDuration : 500,              // Duration of floater rest animation (ms)
			fontSize : [                                   // Font size for floaters of level {1}
				tgScale(32),
				tgScale(44),
				tgScale(56),
				tgScale(68)
			],
			stageFaileFloaterAppearDuration : 500,         // Duration of end stage floater scale up animation (ms)
			stageFailFloaterDisappearDuration : 500,       // Duration of end stage floater scale down animation (ms)
			stageFailFloaterVisibleDuration : 3500,        // Duration of end stage floater rest animation (ms)
			stageStartFloaterAppearDuration : 500,         // Duration of start stage floater scale up animation (ms)
			stageStartFloaterDisappearDuration : 500,      // Duration of start stage floater scale down animation (ms)
			stageStartFloaterVisibleDuration : 1500,       // Duration of start stage floater rest animation (ms)
			textFloaterOffset : tgScale(0)                 // Offset between points floater and text floater
		},
		framework : {
			gameId : "match3_timed",                       // The id of the game
			gameType : "endless",                          // The type of game
			showDifficulty : false,                        // Whether or not to show the difficulty button in start screen
			supportedDifficulties : [2]                    // Which difficulty level(s) is/are supported
		},
		gem : {
			bombAnimationSpeed : 100,                      // No of milliseconds per frame of bomb gem animation
			bombExplodeDuration : 400,                     // Duration in milliseconds of bomb explosion animation
			fallDuration : 300,                            // Duration in milliseconds of gems falling into grid animation
			falseMoveDuration : 300,                       // Duration in milliseconds of gems swapping back after false match animation
			gettingPowerAnimDuration : 300,                // No of milliseconds for gems to move together to form a boost
			growShrinkDuration : 400,                      // Duration in milliseconds of gem growing and shrinking animation when hit by a combo
			hyperAnimationSpeed : 100,                     // No of milliseconds per frame of hypercube gem animation
			novaAnimationSpeed : 50,                       // No of milliseconds per frame of nova gem animation
			shrinkDuration : 200,                          // Duration in milliseconds of gem shrink animation
			sparkFPS : 20,                                 // FPS for spark animation when gem disappears after hit by hyper cube
			spawnDuration : 200,                           // Duration in milliseconds
			spiralPopUpDuration : 200,                     // duration of popup effect of gems before starting sprial move (ms)
			spiralPopUpScaleFactor : 1.2,                  // scalefactor of popup effect of gems before starting sprial move
			swapDuration : 300,                            // Duration in milliseconds of gem swap animation
			timeBonusOffsetX : tgScale(0),                 // x offset with respect to center of gem where timeBonusText will be displayed
			timeBonusOffsetY : tgScale(30)                 // y offset with respect to center of gem where timeBonusText will be displayed
		},
		gridController : {
			alwaysEnsureMove : [                           // Whether or not to ensure the player can always continue in mode {0}
				true,
				false,
				true
			],
			bombExplodeDelay : 200,                        // No of milliseconds before next ring starts detonating
			bombRadius : 1,                                // No of rings of gems around bomb that will explode when bomb is detonated
			columns : 8,                                   // No of columns in grid
			hyperMaxDelayDiff : 400,                       // Maximum delay in milliseconds before lightning jumps to next gem
			hyperMinDelay : 100,                           // Number of milliseconds before hypercube animation starts
			hyperMinDelayDiff : 100,                       // Minimum delay in milliseconds before lightning jumps to next gem
			novaMinDelay : 1000,                           // Number of milliseconds before column falls down after a gem has been removed by a nova boost
			rows : 8,                                      // No of rows in grid
			swapDist : tgScale(30),                        // Minimal number of pixels the player should move a gem before a swap is initiated
			tileFlashDuration : 500,                       // Duration of coloured tile flash animation (ms)
			tileSize : tgScale(70),                        // Grid tile size
			timeBonusChanceMatch : [                       // chance of a 5, 10, 15, etc... seconds time bonus after a match of size ({0} + 3)
				[0, 0, 0],
				[1, 0, 0],
				[0, 1, 0],
				[0, 0, 1]
			]
		},
		hintController : {
			hintDuration : 2000,                           // No of milliseconds hint indicator is shown
			hintFramesPerSeconds : 20,                     // 
			timeBeforeHint : [                             // No of milliseconds in which no correct match is made by user before hint is showed, per playing mode
				6000,
				6000,
				6000
			],
			timeBetweenHint : 3000                         // No of milliseconds after first hint indicator has disappeared before the hint is shown again
		},
		levelController : {
			endGameBoostActivateDelay : 2500,              // Time in milliseconds before all boosts are activated at end the end of the game
			endGameEndScreenDelay : 500,                   // Time in milliseconds before end screen is shown after all boosts have been activated at end the end of the game
			gameTime : 60000,                              // Time in milliseconds the game lasts in timed mode
			showHints : [                                  // Whether or not to show hints in mode {0}
				true,
				true,
				true
			],
			showNumberFloaters : true,                     // Whether or not to show number floaters for points
			showTextFloaters : true,                       // Whether or not to show text floaters like Good, Great, etc.
			startStage : 1,                                // Stage to start the game from, for tweaking/debugging
			timeWarningDuration : 10000                    // Time in milliseconds before the end of the game when the gameui starts flashing and beeping
		},
		levelEnd : {
			backButtonType : "exit",                       // Level end screen definition
			emphasizedStatsItem : "totalScore",            // Level end screen definition
			forwardButtonType : "retry",                   // Level end screen definition
			showMedal : true,                              // Level end screen definition
			statsItems : ["highScore"],                    // Level end screen definition
			titleItem : "title_difficulty"                 // Level end screen definition
		},

		// Level Descriptions
		levelCount : 7,
		levels : [
			{   // LEVEL 0
				name : "level_1",
				general : {
					stageNo : 1,                                 // Indicates up until which state these settings should be taken into account
					mode : ["timed"]                             // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0.25, 0, 0.2, 0.15, 0.1, 0.1, 0.2]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 1
				name : "level_2",
				general : {
					stageNo : 1,                                 // Indicates up until which state these settings should be taken into account
					mode : ["zen", "classic"]                    // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2],
						[0, 0.25, 0.2, 0.15, 0.1, 0.1, 0.2]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 2
				name : "level_3",
				general : {
					stageNo : 3,                                 // Indicates up until which state these settings should be taken into account
					mode : ["zen", "classic"]                    // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 3
				name : "level_4",
				general : {
					stageNo : 7,                                 // Indicates up until which state these settings should be taken into account
					mode : ["zen", "classic", "timed"]           // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2],
						[14.3,14.3, 14.3, 14.3, 14.3, 14.3, 14.2]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 4
				name : "level_1_timed",
				general : {
					stageNo : 1,                                 // Indicates up until which state these settings should be taken into account
					mode : ["timed"]                             // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0],
						[0.2, 0.2, 0.2, 0.2, 0.2, 0, 0]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 5
				name : "level_2_timed",
				general : {
					stageNo : 5,                                 // Indicates up until which state these settings should be taken into account
					mode : ["timed"]                             // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0],
						[0.2, 0.2, 0.2, 0.2, 0.1, 0.1, 0]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			},
			{   // LEVEL 6
				name : "level_3_timed",
				general : {
					stageNo : 10,                                // Indicates up until which state these settings should be taken into account
					mode : ["timed"]                             // specifies in which game modes these settings should be used
				},
				discrete : {
					timeBeforeHint : 6000                        // No of milliseconds in which no correct match is made by user before hint is showed
				},
				interpolated : {
					maxComboTimer : [1200,1200,1200],            // No of milliseconds in which the player has to make a new match to keep time combo bonus active, length of array determines match combo counter
					maxChainCounter : 3,                         // Maximum value of the chain counter
					comboFactor : 1,                             // Indicates the weight of the combo counter in computing total score for match
					chainFactor : 0.5,                           // Indicates the weight of the chain counter in computing total score for match
					pointsPerGem : [16.67,18.75,20,33.33,35.71,56.25,83.33,100],// Points per gem per match size (starting from match size 3)
					falseMoveReduction : 0,                      // No of points deducted when the player makes a false move
					timeBeforeReduction : 0,                     // No of milliseconds during which the player makes no correct move before points are deducted (0 means no points will ever be deducted)
					reductionAmount : 0,                         // No of points deducted when timeBeforeReduction has passed
					floaterThresholds : [250, 750, 1500, 3000],  // No of points player must earn with a single match before a floater of that level is shown
					gemColourChance : [                          // Chance of gem of specified colour from {0}0% of the stage
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05],
						[0.25, 0.2, 0.15, 0.1, 0.1, 0.05, 0.05]
					],
					maxPoints : 5000                             // No of points that must be earned to go to the next stage
				}
			}
		]

	};
};

//******************************************************
//*****    Script Resource: dictionary
//******************************************************

// **********************************************
// Localization file for game: Match3Time
//
// created on 30-Jun-16 10:58:23 AM
// **********************************************

/*exported dictionaryVar*/
var dictionaryVar = dictionaryVar || {};
dictionaryVar["nl-nl"] = dictionaryVar["nl-nl"] || {};
dictionaryVar["nl-nl"]["floater_0"] = "Goed";
dictionaryVar["nl-nl"]["floater_1"] = "Mooi!";
dictionaryVar["nl-nl"]["floater_2"] = "Geweldig!";
dictionaryVar["nl-nl"]["floater_3"] = "Fantastisch!";
dictionaryVar["nl-nl"]["timeBonus"] = "+<TIME_BONUS> sec. tijdbonus";
dictionaryVar["nl-nl"]["inGameStageEndFloater"] = "Level\n<STAGE_NO>\nvoltooid!";
dictionaryVar["nl-nl"]["optionsDifficulty_hard"] = "Tijdsdruk";
dictionaryVar["nl-nl"]["optionsDifficulty_medium"] = "Klassiek";
dictionaryVar["nl-nl"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["nl-nl"]["inGameStageFloater"] = "Level";
dictionaryVar["nl-nl"]["inGameNoMoreMovesFloater"] = "Geen zetten meer";
dictionaryVar["nl-nl"]["inGameTimeUpFloater"] = "DE TIJD IS OM";
dictionaryVar["nl-nl"]["levelStartHeader"] = "Aan de slag!";
dictionaryVar["nl-nl"]["levelStartText"] = "Je hebt 1 minuut om dezelfde edelstenen bij elkaar te zoeken voor een topscore!";
dictionaryVar["nl-nl"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["nl-nl"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["nl-nl"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["nl-nl"]["TutorialTitle_1"] = "Voortgang";
dictionaryVar["nl-nl"]["TutorialTitle_0"] = "Zo speel je";
dictionaryVar["nl-nl"]["TutorialText_4"] = "Dit is een kleurenbom. Maak een rij van 5 gelijke items voor dit voorwerp.";
dictionaryVar["nl-nl"]["TutorialText_3"] = "Dit is een lijnbom. Maak een L- of T-vorm voor dit voorwerp.";
dictionaryVar["nl-nl"]["TutorialText_2"] = "Dit is een bom. Maak een combinatie van 4 items voor dit voorwerp.";
dictionaryVar["nl-nl"]["TutorialText_1"] = "Als je groepen van 3 maakt, verdien je punten.";
dictionaryVar["nl-nl"]["TutorialText_0"] = "#touch{Klik en versleep items om 3 van dezelfde kleur bij elkaar te zetten.}{Veeg over items om 3 van dezelfde kleur bij elkaar te zetten.}";
dictionaryVar["nl-nl"]["TutorialText_5"] = "Maak groepen van meer dan 3 voor extra tijd.";
dictionaryVar["nl-nl"]["TutorialTitle_5"] = "Extra tijd!";
dictionaryVar["en-us"] = dictionaryVar["en-us"] || {};
dictionaryVar["en-us"]["floater_0"] = "Good";
dictionaryVar["en-us"]["floater_1"] = "Nice!";
dictionaryVar["en-us"]["floater_2"] = "Great!";
dictionaryVar["en-us"]["floater_3"] = "Awesome!";
dictionaryVar["en-us"]["timeBonus"] = "+<TIME_BONUS> s Time Bonus";
dictionaryVar["en-us"]["inGameStageEndFloater"] = "Stage\n<STAGE_NO>\nComplete!";
dictionaryVar["en-us"]["optionsDifficulty_hard"] = "Timed";
dictionaryVar["en-us"]["optionsDifficulty_medium"] = "Classic";
dictionaryVar["en-us"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["en-us"]["inGameStageFloater"] = "Stage";
dictionaryVar["en-us"]["inGameNoMoreMovesFloater"] = "No more moves";
dictionaryVar["en-us"]["inGameTimeUpFloater"] = "TIME UP";
dictionaryVar["en-us"]["levelStartHeader"] = "Start matching!";
dictionaryVar["en-us"]["levelStartText"] = "You have 1 minute to match gems and get a high score.";
dictionaryVar["en-us"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["en-us"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["en-us"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["en-us"]["TutorialTitle_1"] = "Progress";
dictionaryVar["en-us"]["TutorialTitle_0"] = "How to play";
dictionaryVar["en-us"]["TutorialText_4"] = "This is a color bomb. Create one by matching 5 items in a line.";
dictionaryVar["en-us"]["TutorialText_3"] = "This is a line bomb. Create one by lining up items in an L or T shape.";
dictionaryVar["en-us"]["TutorialText_2"] = "This is a bomb. Create one by lining up 4 items.";
dictionaryVar["en-us"]["TutorialText_1"] = "Creating groups of 3 will earn you points.";
dictionaryVar["en-us"]["TutorialText_0"] = "#touch{Click and drag items to line up 3 of the same color}{Swipe items to line up 3 of the same color.}";
dictionaryVar["en-us"]["TutorialText_5"] = "Match groups of more than 3 to receive extra time rewards.";
dictionaryVar["en-us"]["TutorialTitle_5"] = "Extra time!";
dictionaryVar["en-gb"] = dictionaryVar["en-gb"] || {};
dictionaryVar["en-gb"]["floater_0"] = "Good";
dictionaryVar["en-gb"]["floater_1"] = "Nice!";
dictionaryVar["en-gb"]["floater_2"] = "Great!";
dictionaryVar["en-gb"]["floater_3"] = "Awesome!";
dictionaryVar["en-gb"]["timeBonus"] = "+<TIME_BONUS> s Time Bonus";
dictionaryVar["en-gb"]["inGameStageEndFloater"] = "Stage\n<STAGE_NO>\nComplete!";
dictionaryVar["en-gb"]["optionsDifficulty_hard"] = "Timed";
dictionaryVar["en-gb"]["optionsDifficulty_medium"] = "Classic";
dictionaryVar["en-gb"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["en-gb"]["inGameStageFloater"] = "Stage";
dictionaryVar["en-gb"]["inGameNoMoreMovesFloater"] = "No more moves";
dictionaryVar["en-gb"]["inGameTimeUpFloater"] = "TIME UP";
dictionaryVar["en-gb"]["levelStartHeader"] = "Start matching!";
dictionaryVar["en-gb"]["levelStartText"] = "You have 1 minute to match gems and get a high score.";
dictionaryVar["en-gb"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["en-gb"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["en-gb"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["en-gb"]["TutorialTitle_1"] = "Progress";
dictionaryVar["en-gb"]["TutorialTitle_0"] = "How to play";
dictionaryVar["en-gb"]["TutorialText_4"] = "This is a colour bomb. Create one by matching 5 items in a line.";
dictionaryVar["en-gb"]["TutorialText_3"] = "This is a line bomb. Create one by lining up items in an L or T shape.";
dictionaryVar["en-gb"]["TutorialText_2"] = "This is a bomb. Create one by lining up 4 items.";
dictionaryVar["en-gb"]["TutorialText_1"] = "Creating groups of 3 will earn you points.";
dictionaryVar["en-gb"]["TutorialText_0"] = "#touch{Click and drag items to line up 3 of the same colour}{Swipe items to line up 3 of the same colour.}";
dictionaryVar["en-gb"]["TutorialText_5"] = "Match groups of more than 3 to receive extra time rewards.";
dictionaryVar["en-gb"]["TutorialTitle_5"] = "Extra time!";
dictionaryVar["de-de"] = dictionaryVar["de-de"] || {};
dictionaryVar["de-de"]["floater_0"] = "Gut!";
dictionaryVar["de-de"]["floater_1"] = "Toll!";
dictionaryVar["de-de"]["floater_2"] = "Super!";
dictionaryVar["de-de"]["floater_3"] = "Fantastisch!";
dictionaryVar["de-de"]["timeBonus"] = "+<TIME_BONUS> Sek. Zeitbonus";
dictionaryVar["de-de"]["inGameStageEndFloater"] = "Stufe\n<STAGE_NO>\nabgeschlossen!";
dictionaryVar["de-de"]["optionsDifficulty_hard"] = "Auf Zeit";
dictionaryVar["de-de"]["optionsDifficulty_medium"] = "Klassisch";
dictionaryVar["de-de"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["de-de"]["inGameStageFloater"] = "Stufe";
dictionaryVar["de-de"]["inGameNoMoreMovesFloater"] = "Keine Züge mehr";
dictionaryVar["de-de"]["inGameTimeUpFloater"] = "DIE ZEIT IST UM.";
dictionaryVar["de-de"]["levelStartHeader"] = "Los geht's!";
dictionaryVar["de-de"]["levelStartText"] = "Du hast 1 Minuten, um mit passenden Edelsteinen einen Highscore zu schaffen.";
dictionaryVar["de-de"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["de-de"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["de-de"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["de-de"]["TutorialTitle_1"] = "Fortschritt";
dictionaryVar["de-de"]["TutorialTitle_0"] = "Spielregeln";
dictionaryVar["de-de"]["TutorialText_4"] = "Das ist eine Farbbombe. Du bekommst sie für 5 passende Gegenstände in einer Reihe.";
dictionaryVar["de-de"]["TutorialText_3"] = "Das ist eine Reihenbombe. Du bekommst sie für L- oder T-förmig angeordnete Gegenstände.";
dictionaryVar["de-de"]["TutorialText_2"] = "Das ist eine Bombe. Du bekommst sie für 4 passende Gegenstände in einer Reihe.";
dictionaryVar["de-de"]["TutorialText_1"] = "Für Dreiergruppen bekommst du Punkte.";
dictionaryVar["de-de"]["TutorialText_0"] = "#touch{Klicke auf Gegenstände und verschiebe sie zu gleichfarbigen Dreierreihen}{Verschiebe Gegenstände zu gleichfarbigen Dreierreihen.}";
dictionaryVar["de-de"]["TutorialText_5"] = "Für Gruppen von mehr als drei Gegenständen bekommst du Bonuszeit.";
dictionaryVar["de-de"]["TutorialTitle_5"] = "Bonuszeit!";
dictionaryVar["fr-fr"] = dictionaryVar["fr-fr"] || {};
dictionaryVar["fr-fr"]["floater_0"] = "Bien !";
dictionaryVar["fr-fr"]["floater_1"] = "Joli !";
dictionaryVar["fr-fr"]["floater_2"] = "Génial !";
dictionaryVar["fr-fr"]["floater_3"] = "Excellent !";
dictionaryVar["fr-fr"]["timeBonus"] = "+<TIME_BONUS> s de temps bonus";
dictionaryVar["fr-fr"]["inGameStageEndFloater"] = "Niveau\n<STAGE_NO>\nTerminé !";
dictionaryVar["fr-fr"]["optionsDifficulty_hard"] = "Contre la montre";
dictionaryVar["fr-fr"]["optionsDifficulty_medium"] = "Classique";
dictionaryVar["fr-fr"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["fr-fr"]["inGameStageFloater"] = "Niveau";
dictionaryVar["fr-fr"]["inGameNoMoreMovesFloater"] = "Plus de mouvements";
dictionaryVar["fr-fr"]["inGameTimeUpFloater"] = "TEMPS ÉCOULÉ";
dictionaryVar["fr-fr"]["levelStartHeader"] = "Commencez à associer !";
dictionaryVar["fr-fr"]["levelStartText"] = "Vous avez une minute pour associer les gemmes et obtenir le meilleur score.";
dictionaryVar["fr-fr"]["TutorialTitle_4"] = "Bonus";
dictionaryVar["fr-fr"]["TutorialTitle_3"] = "Bonus";
dictionaryVar["fr-fr"]["TutorialTitle_2"] = "Bonus";
dictionaryVar["fr-fr"]["TutorialTitle_1"] = "Progression";
dictionaryVar["fr-fr"]["TutorialTitle_0"] = "Comment jouer";
dictionaryVar["fr-fr"]["TutorialText_4"] = "Ceci est une bombe colorée. Associez 5 objets sur une ligne pour en créer une.";
dictionaryVar["fr-fr"]["TutorialText_3"] = "Ceci est une bombe linéaire. Associez des objets en formant un L ou un T pour en créer une.";
dictionaryVar["fr-fr"]["TutorialText_2"] = "Ceci est une bombe. Associez 4 objets sur une ligne pour en créer une.";
dictionaryVar["fr-fr"]["TutorialText_1"] = "Créer des groupes de 3 vous rapporte des points.";
dictionaryVar["fr-fr"]["TutorialText_0"] = "#touch{Cliquez et faites glisser les objets pour en associer 3 de la même couleur.}{Faites glisser les objets pour en associer 3 de la même couleur.}";
dictionaryVar["fr-fr"]["TutorialText_5"] = "Associez des groupes de 3 ou plus pour obtenir un bonus de temps.";
dictionaryVar["fr-fr"]["TutorialTitle_5"] = "Temps bonus !";
dictionaryVar["pt-br"] = dictionaryVar["pt-br"] || {};
dictionaryVar["pt-br"]["floater_0"] = "Bom";
dictionaryVar["pt-br"]["floater_1"] = "Legal!";
dictionaryVar["pt-br"]["floater_2"] = "Ótimo!";
dictionaryVar["pt-br"]["floater_3"] = "Incrível!";
dictionaryVar["pt-br"]["timeBonus"] = "+<TIME_BONUS> s Tempo Bônus";
dictionaryVar["pt-br"]["inGameStageEndFloater"] = "Fase\n<STAGE_NO>\nCompletada!";
dictionaryVar["pt-br"]["optionsDifficulty_hard"] = "Cronometrado";
dictionaryVar["pt-br"]["optionsDifficulty_medium"] = "Clássico";
dictionaryVar["pt-br"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["pt-br"]["inGameStageFloater"] = "Fase";
dictionaryVar["pt-br"]["inGameNoMoreMovesFloater"] = "Sem mais ações";
dictionaryVar["pt-br"]["inGameTimeUpFloater"] = "TEMPO ESGOTADO";
dictionaryVar["pt-br"]["levelStartHeader"] = "Começar partida!";
dictionaryVar["pt-br"]["levelStartText"] = "Você tem 1 minuto para combinar as gemas e conseguir uma grande pontuação.";
dictionaryVar["pt-br"]["TutorialTitle_4"] = "Aditivos";
dictionaryVar["pt-br"]["TutorialTitle_3"] = "Aditivos";
dictionaryVar["pt-br"]["TutorialTitle_2"] = "Aditivos";
dictionaryVar["pt-br"]["TutorialTitle_1"] = "Progresso";
dictionaryVar["pt-br"]["TutorialTitle_0"] = "Como jogar";
dictionaryVar["pt-br"]["TutorialText_4"] = "Esta é uma bomba colorida. Crie uma ao combinar 5 itens em uma linha.";
dictionaryVar["pt-br"]["TutorialText_3"] = "Esta é uma linha bomba. Crie uma ao alinhar itens no formato de um L ou um T.";
dictionaryVar["pt-br"]["TutorialText_2"] = "Isto é uma bomba. Crie uma ao alinhar 4 itens.";
dictionaryVar["pt-br"]["TutorialText_1"] = "Criar conjuntos de 3 irá lhe garantir pontos.";
dictionaryVar["pt-br"]["TutorialText_0"] = "#touch{Clique e arraste itens para alinhar 3 da mesma cor.}{Deslize itens para alinhar 3 da mesma cor}";
dictionaryVar["pt-br"]["TutorialText_5"] = "Combine conjuntos de mais de 3 para receber recompensas de tempo extra.";
dictionaryVar["pt-br"]["TutorialTitle_5"] = "Tempo extra!";
dictionaryVar["es-es"] = dictionaryVar["es-es"] || {};
dictionaryVar["es-es"]["floater_0"] = "Bien";
dictionaryVar["es-es"]["floater_1"] = "¡Guay!";
dictionaryVar["es-es"]["floater_2"] = "¡Mola!";
dictionaryVar["es-es"]["floater_3"] = "¡Estupendo!";
dictionaryVar["es-es"]["timeBonus"] = "+<TIME_BONUS> s extra";
dictionaryVar["es-es"]["inGameStageEndFloater"] = "¡Nivel\n<STAGE_NO>\nterminado!";
dictionaryVar["es-es"]["optionsDifficulty_hard"] = "Contrarreloj";
dictionaryVar["es-es"]["optionsDifficulty_medium"] = "Clásico";
dictionaryVar["es-es"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["es-es"]["inGameStageFloater"] = "Nivel";
dictionaryVar["es-es"]["inGameNoMoreMovesFloater"] = "Sin movimientos";
dictionaryVar["es-es"]["inGameTimeUpFloater"] = "TIEMPO";
dictionaryVar["es-es"]["levelStartHeader"] = "¡A combinar!";
dictionaryVar["es-es"]["levelStartText"] = "Tienes 1 minuto para combinar gemas y ganar puntos.";
dictionaryVar["es-es"]["TutorialTitle_4"] = "Potenciadores";
dictionaryVar["es-es"]["TutorialTitle_3"] = "Potenciadores";
dictionaryVar["es-es"]["TutorialTitle_2"] = "Potenciadores";
dictionaryVar["es-es"]["TutorialTitle_1"] = "Progreso";
dictionaryVar["es-es"]["TutorialTitle_0"] = "Cómo jugar";
dictionaryVar["es-es"]["TutorialText_4"] = "Esta es una bomba de color. Créala combinando 5 objetos en fila.";
dictionaryVar["es-es"]["TutorialText_3"] = "Esta es una bomba de línea. Combina objetos en forma de L o T para crearla.";
dictionaryVar["es-es"]["TutorialText_2"] = "Esta es una bomba. Créala combinando 4 objetos.";
dictionaryVar["es-es"]["TutorialText_1"] = "Ganas puntos al crear grupos de 3.";
dictionaryVar["es-es"]["TutorialText_0"] = "#touch{Haz clic y arrastra objetos para alinear 3 del mismo color.}{Desliza objetos para alinear 3 del mismo color.}";
dictionaryVar["es-es"]["TutorialText_5"] = "Combina más de 3 para recibir tiempo extra.";
dictionaryVar["es-es"]["TutorialTitle_5"] = "¡Tiempo extra!";
dictionaryVar["tr-tr"] = dictionaryVar["tr-tr"] || {};
dictionaryVar["tr-tr"]["floater_0"] = "İyi";
dictionaryVar["tr-tr"]["floater_1"] = "Güzel!";
dictionaryVar["tr-tr"]["floater_2"] = "Harika!";
dictionaryVar["tr-tr"]["floater_3"] = "Muhteşem!";
dictionaryVar["tr-tr"]["timeBonus"] = "+<TIME_BONUS> s Zaman Bonusu";
dictionaryVar["tr-tr"]["inGameStageEndFloater"] = "Bölüm\n<STAGE_NO>\nBitti!";
dictionaryVar["tr-tr"]["optionsDifficulty_hard"] = "Zamanlı";
dictionaryVar["tr-tr"]["optionsDifficulty_medium"] = "Klasik";
dictionaryVar["tr-tr"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["tr-tr"]["inGameStageFloater"] = "Bölüm";
dictionaryVar["tr-tr"]["inGameNoMoreMovesFloater"] = "Hareket kalmadı";
dictionaryVar["tr-tr"]["inGameTimeUpFloater"] = "ZAMAN DOLDU";
dictionaryVar["tr-tr"]["levelStartHeader"] = "Eşlemeye Başlayın!";
dictionaryVar["tr-tr"]["levelStartText"] = "Cevherleri eşleyip yüksek bir puan almak için 1 dakikan var.";
dictionaryVar["tr-tr"]["TutorialTitle_4"] = "Güçlendirmeler";
dictionaryVar["tr-tr"]["TutorialTitle_3"] = "Güçlendirmeler";
dictionaryVar["tr-tr"]["TutorialTitle_2"] = "Güçlendirmeler";
dictionaryVar["tr-tr"]["TutorialTitle_1"] = "İlerleme";
dictionaryVar["tr-tr"]["TutorialTitle_0"] = "Nasıl oynanır";
dictionaryVar["tr-tr"]["TutorialText_4"] = "Bu bir renk bombasıdır. 5 cevheri çizgisel birleştirdiğinde olur.";
dictionaryVar["tr-tr"]["TutorialText_3"] = "Bu bir çizgi bombasıdır. L veya T şeklinde birleştirirsen olur.";
dictionaryVar["tr-tr"]["TutorialText_2"] = "Bu bir bombadır. 4 cevheri birleştirdiğinde olur.";
dictionaryVar["tr-tr"]["TutorialText_1"] = "3'lü gruplar yaptığında puan kazanırsın.";
dictionaryVar["tr-tr"]["TutorialText_0"] = "#touch{Aynı renkten 3 tane birleştirmek için sürükle.}{Aynı renkten 3 tane birleştirmek için tıkla ve sürükle.}";
dictionaryVar["tr-tr"]["TutorialText_5"] = "3'ten fazla birleştirirsen ekstra zaman ödülleri kazanırsın.";
dictionaryVar["tr-tr"]["TutorialTitle_5"] = "Ekstra zaman!";
dictionaryVar["ru-ru"] = dictionaryVar["ru-ru"] || {};
dictionaryVar["ru-ru"]["floater_0"] = "Хорошо";
dictionaryVar["ru-ru"]["floater_1"] = "Здорово!";
dictionaryVar["ru-ru"]["floater_2"] = "Отлично!";
dictionaryVar["ru-ru"]["floater_3"] = "Круто!";
dictionaryVar["ru-ru"]["timeBonus"] = "Дополнительное время: +<TIME_BONUS> с.";
dictionaryVar["ru-ru"]["inGameStageEndFloater"] = "Stage\n<STAGE_NO>\nComplete!";
dictionaryVar["ru-ru"]["optionsDifficulty_hard"] = "Timed";
dictionaryVar["ru-ru"]["optionsDifficulty_medium"] = "Classic";
dictionaryVar["ru-ru"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["ru-ru"]["inGameStageFloater"] = "Stage";
dictionaryVar["ru-ru"]["inGameNoMoreMovesFloater"] = "No more moves";
dictionaryVar["ru-ru"]["inGameTimeUpFloater"] = "TIME UP";
dictionaryVar["ru-ru"]["levelStartHeader"] = "Start matching!";
dictionaryVar["ru-ru"]["levelStartText"] = "You have 1 minute to match gems and get a high score.";
dictionaryVar["ru-ru"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["ru-ru"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["ru-ru"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["ru-ru"]["TutorialTitle_1"] = "Progress";
dictionaryVar["ru-ru"]["TutorialTitle_0"] = "How to play";
dictionaryVar["ru-ru"]["TutorialText_4"] = "This is a color bomb. Create one by matching 5 items in a line.";
dictionaryVar["ru-ru"]["TutorialText_3"] = "This is a line bomb. Create one by lining up items in an L or T shape.";
dictionaryVar["ru-ru"]["TutorialText_2"] = "This is a bomb. Create one by lining up 4 items.";
dictionaryVar["ru-ru"]["TutorialText_1"] = "Creating groups of 3 will earn you points.";
dictionaryVar["ru-ru"]["TutorialText_0"] = "#touch{Click and drag items to line up 3 of the same color}{Swipe items to line up 3 of the same color.}";
dictionaryVar["ru-ru"]["TutorialText_5"] = "Match groups of more than 3 to receive extra time rewards.";
dictionaryVar["ru-ru"]["TutorialTitle_5"] = "Extra time!";
dictionaryVar["ar-eg"] = dictionaryVar["ar-eg"] || {};
dictionaryVar["ar-eg"]["floater_0"] = "جيد";
dictionaryVar["ar-eg"]["floater_1"] = "جيد!";
dictionaryVar["ar-eg"]["floater_2"] = "عظيم!";
dictionaryVar["ar-eg"]["floater_3"] = "رائع!";
dictionaryVar["ar-eg"]["timeBonus"] = "مكافأة وقت مقدارها +<TIME_BONUS>";
dictionaryVar["ar-eg"]["inGameStageEndFloater"] = "Stage\n<STAGE_NO>\nComplete!";
dictionaryVar["ar-eg"]["optionsDifficulty_hard"] = "Timed";
dictionaryVar["ar-eg"]["optionsDifficulty_medium"] = "Classic";
dictionaryVar["ar-eg"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["ar-eg"]["inGameStageFloater"] = "Stage";
dictionaryVar["ar-eg"]["inGameNoMoreMovesFloater"] = "No more moves";
dictionaryVar["ar-eg"]["inGameTimeUpFloater"] = "TIME UP";
dictionaryVar["ar-eg"]["levelStartHeader"] = "Start matching!";
dictionaryVar["ar-eg"]["levelStartText"] = "You have 1 minute to match gems and get a high score.";
dictionaryVar["ar-eg"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["ar-eg"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["ar-eg"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["ar-eg"]["TutorialTitle_1"] = "Progress";
dictionaryVar["ar-eg"]["TutorialTitle_0"] = "How to play";
dictionaryVar["ar-eg"]["TutorialText_4"] = "This is a color bomb. Create one by matching 5 items in a line.";
dictionaryVar["ar-eg"]["TutorialText_3"] = "This is a line bomb. Create one by lining up items in an L or T shape.";
dictionaryVar["ar-eg"]["TutorialText_2"] = "This is a bomb. Create one by lining up 4 items.";
dictionaryVar["ar-eg"]["TutorialText_1"] = "Creating groups of 3 will earn you points.";
dictionaryVar["ar-eg"]["TutorialText_0"] = "#touch{Click and drag items to line up 3 of the same color}{Swipe items to line up 3 of the same color.}";
dictionaryVar["ar-eg"]["TutorialText_5"] = "Match groups of more than 3 to receive extra time rewards.";
dictionaryVar["ar-eg"]["TutorialTitle_5"] = "Extra time!";
dictionaryVar["ko-kr"] = dictionaryVar["ko-kr"] || {};
dictionaryVar["ko-kr"]["floater_0"] = "좋아요";
dictionaryVar["ko-kr"]["floater_1"] = "멋져요!";
dictionaryVar["ko-kr"]["floater_2"] = "훌륭해요!";
dictionaryVar["ko-kr"]["floater_3"] = "굉장하군요!";
dictionaryVar["ko-kr"]["timeBonus"] = "+<TIME_BONUS> 시간 보너스";
dictionaryVar["ko-kr"]["inGameStageEndFloater"] = "Stage\n<STAGE_NO>\nComplete!";
dictionaryVar["ko-kr"]["optionsDifficulty_hard"] = "Timed";
dictionaryVar["ko-kr"]["optionsDifficulty_medium"] = "Classic";
dictionaryVar["ko-kr"]["optionsDifficulty_easy"] = "Zen";
dictionaryVar["ko-kr"]["inGameStageFloater"] = "Stage";
dictionaryVar["ko-kr"]["inGameNoMoreMovesFloater"] = "No more moves";
dictionaryVar["ko-kr"]["inGameTimeUpFloater"] = "TIME UP";
dictionaryVar["ko-kr"]["levelStartHeader"] = "Start matching!";
dictionaryVar["ko-kr"]["levelStartText"] = "You have 1 minute to match gems and get a high score.";
dictionaryVar["ko-kr"]["TutorialTitle_4"] = "Boosts";
dictionaryVar["ko-kr"]["TutorialTitle_3"] = "Boosts";
dictionaryVar["ko-kr"]["TutorialTitle_2"] = "Boosts";
dictionaryVar["ko-kr"]["TutorialTitle_1"] = "Progress";
dictionaryVar["ko-kr"]["TutorialTitle_0"] = "How to play";
dictionaryVar["ko-kr"]["TutorialText_4"] = "This is a color bomb. Create one by matching 5 items in a line.";
dictionaryVar["ko-kr"]["TutorialText_3"] = "This is a line bomb. Create one by lining up items in an L or T shape.";
dictionaryVar["ko-kr"]["TutorialText_2"] = "This is a bomb. Create one by lining up 4 items.";
dictionaryVar["ko-kr"]["TutorialText_1"] = "Creating groups of 3 will earn you points.";
dictionaryVar["ko-kr"]["TutorialText_0"] = "#touch{Click and drag items to line up 3 of the same color}{Swipe items to line up 3 of the same color.}";
dictionaryVar["ko-kr"]["TutorialText_5"] = "Match groups of more than 3 to receive extra time rewards.";
dictionaryVar["ko-kr"]["TutorialTitle_5"] = "Extra time!";
dictionaryVar["jp-jp"] = dictionaryVar["jp-jp"] || {};
dictionaryVar["jp-jp"]["floater_0"] = "Good";
dictionaryVar["jp-jp"]["floater_1"] = "Nice!";
dictionaryVar["jp-jp"]["floater_2"] = "Great!";
dictionaryVar["jp-jp"]["floater_3"] = "最高！";
dictionaryVar["jp-jp"]["timeBonus"] = "\"+<TIME_BONUS> 秒のタイムボーナス\"";
dictionaryVar["jp-jp"]["inGameStageEndFloater"] = "ステージ\n<STAGE_NO>\nクリア！";
dictionaryVar["jp-jp"]["optionsDifficulty_hard"] = "経過時間";
dictionaryVar["jp-jp"]["optionsDifficulty_medium"] = "クラシック";
dictionaryVar["jp-jp"]["optionsDifficulty_easy"] = "ステージ";
dictionaryVar["jp-jp"]["inGameStageFloater"] = "ステージ";
dictionaryVar["jp-jp"]["inGameNoMoreMovesFloater"] = "これ以上動かせないよ";
dictionaryVar["jp-jp"]["inGameTimeUpFloater"] = "タイムアップ";
dictionaryVar["jp-jp"]["levelStartHeader"] = "スタート！";
dictionaryVar["jp-jp"]["levelStartText"] = "同じアイテムを3つか\nそれ以上つなげていこう！";
dictionaryVar["jp-jp"]["TutorialTitle_4"] = "ブースト";
dictionaryVar["jp-jp"]["TutorialTitle_3"] = "ブースト";
dictionaryVar["jp-jp"]["TutorialTitle_2"] = "ブースト";
dictionaryVar["jp-jp"]["TutorialTitle_1"] = "経過";
dictionaryVar["jp-jp"]["TutorialTitle_0"] = "あそび方";
dictionaryVar["jp-jp"]["TutorialText_4"] = "5つつなげると\nカラー爆弾ができるよ";
dictionaryVar["jp-jp"]["TutorialText_3"] = "L字、T字でつなげると\nライン爆弾ができるよ";
dictionaryVar["jp-jp"]["TutorialText_2"] = "4つつなげると\n爆弾ができるよ";
dictionaryVar["jp-jp"]["TutorialText_1"] = "なにがそろったかで\n得点が変わります";
dictionaryVar["jp-jp"]["TutorialText_0"] = "スワイプして\n同じ色を3つ以上つなげよう";
dictionaryVar["jp-jp"]["TutorialText_5"] = "3つ以上でそろえると\nエクストラタイムがもらえるよ";
dictionaryVar["jp-jp"]["TutorialTitle_5"] = "エクストラタイム";

//******************************************************
//*****    Script Resource: game_configuration
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************
/* exported configuration_product */
var configuration_product = configuration_product || {};
configuration_product.ga = {
	gameKey: "879f207bb26236263180bdd6b27457df",
	secretKey: "0ab83e52020f1ce8033dccce44febc2c9092a36a"
};
configuration_product.supportedLanguages = ["nl-nl", "en-us", "en-gb", "de-de", "fr-fr", "pt-br", "es-es", "tr-tr"];
configuration_product.storageKey = "match3Time";

//******************************************************
//*****    Script Resource: theme_settings
//******************************************************

//////////////////////////////////////////////////////
// Settings file for game theme                     //
// Created from levels file theme_olympus           //
//                                                  //
// Created by the Tingly Level Editor               //
//     on 13/03/2015 14:02:44                       //
//////////////////////////////////////////////////////

// Directives for jshint
/* global tgScale */
/* global tgSelect */
/* exported themeSettingsVar */
/* exported initThemeSettings */

var themeSettingsVar = null;

var initThemeSettings = function() {
	"use strict";

	// *****************************************
	// Global Settings
	// *****************************************

	themeSettingsVar = {
		buttons : {
			"bigPlay" : "orange",                          // font color of big play button (play button on start screen)
			"default_color" : "orange"                     // default font color used on buttons
		},
		game : {
			gameName : "MythicalJewels"                    // official name of the game, capitalized but without spaces (i.e. BubbleCharms)
		},
		levelEnd : {
			emphasizedStatsFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(74), fillColor: "#1d1030", align: "center", baseline: "middle", stroke: true, strokeColor: "#ffddc6", strokeWidth: tgSelect({big: 4, small: 2}), strokeBehind: true},// font properties of big/emphasized stats text
			emphasizedStatsHeaderFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(28), fillColor: "#1d1030", align: "center", baseline: "middle"},// font properties of big/emphasized stats header
			emphasizedStatsHeaderY : tgScale(154),         // y position of emphasized header
			emphasizedStatsY : tgScale(186),               // y position of emphasized stats
			statsFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(38), fillColor: "#1d1030", align: "center", baseline: "middle", stroke: true, strokeColor: "#ffddc6", strokeWidth: tgSelect({big: 4, small: 2}), strokeBehind: true},// font properties of normal stats text
			statsHeaderFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(28), fillColor: "#1d1030", align: "center", baseline: "middle"},// font properties of normal stats header
			statsHeaderY : tgScale(286),                   // y position of stats header
			titleFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(24), fillColor: "#1d1030", align: "center", baseline: "middle"},// font properties of title
			titleMaxHeight : tgScale(26),                  // max height of title
			titleMaxWidth : tgScale(164),                  // max width of title
			titleY : tgScale(66)                           // y position of title
		},
		levelStart : {
			assignmentFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(24), fillColor: "#1d1030", align: "center", baseline: "middle", spacing: tgScale(6)},// font properties of assignment (body) content
			assignmentMaxWidth : tgScale(366),             // max width of assignment (body) text
			assignmentOffsetY : tgScale(242),              // y position of assignment (body) text
			headerFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(28), fillColor: "#1d1030", align: "center", baseline: "middle", stroke: true, strokeColor: "#ffddc6", strokeWidth: tgSelect({big: 4, small: 2}), strokeBehind: true},// font properties of header
			headerMaxWidth : tgScale(366),                 // max width of header text
			headerOffsetY : tgScale(160)                   // y position of header text
		},
		music : {
			backgroundFadeInDuration : 500                 // duration (ms) of background music fade in after clicking big play button
		},
		options : {
			confirmQuestionBtnNoY : tgScale(490),          // y position of no button
			confirmQuestionBtnYesY : tgScale(390),         // y position of yes button
			confirmQuestionTextY : tgScale(240),           // y position of question text
			quitConfirmFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(24), fillColor: "#1d1030", align: "center", baseline: "middle", spacing: tgScale(10)},// font properties for confirming quit (and other questions)
			titleFont : {fontFace: f_themeDefault.fontFace, fontSize: tgScale(24), fillColor: "#1d1030", align: "center", baseline: "middle"},// font properties of title
			titleMaxHeight : tgScale(26),                  // max height of title
			titleMaxWidth : tgScale(164),                  // max width of title
			titleY : tgScale(54),                          // y position of title
			tutorialBackgroundOffsetY : tgScale(79, "round"),// offset y of tutorial background from overlay
			tutorialFont : {align: "center", baseline:"top", fontSize: tgSelect({big: 24, small: 14}), spacing: 6, fillColor:"#1d1030"},// font properties for tutorial body
			tutorialImageOffsetY : tgScale(144, "round"),  // y offset of tutorial image from overlay
			tutorialTextOffsetY : tgScale(360, "round"),   // y offset for tutorial text
			tutorialTitleFont : {fontFace: f_default.fontFace, fontSize: tgScale(28), fillColor: "#1d1030", align: "center", baseline: "middle"},// font properties for tutorial title
			tutorialTitleOffsetY : tgScale(120, "round")   // y offset for tutorial title
		},
		startScreen : {
			playButtonY :  tgScale(496)                    // y position of play button
		}
	};
};

//******************************************************
//*****    Script Resource: game_theme_settings
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

/* exported gameThemeSettingsVar */
var gameThemeSettingsVar = {};
/* exported initGameThemeSettings */
var initGameThemeSettings = function(){
	"use strict";
	gameThemeSettingsVar = {
		gem: {
			timeBonusColours: ["#5080e3", "#ff0000", "#ff5400", "#e42b20", "#5080e3", "#ff0000", "#5080e3"]
		}
	};
};

//******************************************************
//*****    Script Resource: TG_Hooks
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.hooks = TG.hooks || {};

TG.hooks.handleGameInitialized = function(){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.informStartTextureLoaded = function(){
	"use strict";
};

TG.hooks.loadingProgress = function(progress){
	"use strict";
};

TG.hooks.handleGameLoaded = function(){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.informGameStart = function(){
	"use strict";
};

TG.hooks.informGameRestart = function(){
	"use strict";
};

TG.hooks.informStartScreenShown = function(){
	"use strict";
};

TG.hooks.informLevelStart = function(worldIdx, levelIdx){
	"use strict";
};

TG.hooks.informLevelEndScreenShown = function(){
	"use strict";
};

TG.hooks.informLevelEndScreenStart = function(){
	"use strict";
};

TG.hooks.informLevelQuit = function(){
	"use strict";
};

TG.hooks.handleLevelInitialized = function(worldIdx, levelIdx){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.handleLevelLoaded = function(worldIdx, levelIdx){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.handleLevelCompleted = function(){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.handleGameCompleted = function(){
	"use strict";
	TG.controller.continueGameFlow();
};

TG.hooks.getStoredData = function(defaultValue, callback){
	"use strict";
	if(TG.controller.persistence === undefined){
		TG.controller.persistence = new TG_Persistence(true);
	}
	return TG.controller.persistence.loadData(defaultValue, callback);
};

TG.hooks.updateStoredData = function(data){
	"use strict";
	if(TG.controller.persistence === undefined){
		TG.controller.persistence = new TG_Persistence(true);
	}
	return TG.controller.persistence.saveData(data);
};

TG.hooks.openUrl = function(url){
	"use strict";
	window.open(url);
};

TG.hooks.getStartScreenLogos = function(){
	"use strict";
	return [{image: s_logo_coolgames_start, url: TG.config.homeUrl}];
};

TG.hooks.informResize = function(){
	"use strict";
};

//******************************************************
//*****    Script Resource: TG_AdvertisementHooks
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.adHooks = TG.adHooks || {};

TG.adHooks.handleGameInitialized = function(){
    "use strict";
    TG.controller.continueGameFlowAfterAd();
};

TG.adHooks.loadingProgress = function(progress){
    "use strict";
};

TG.adHooks.handleGameLoaded = function(){
    "use strict";
    TG.controller.continueGameFlowAfterAd();
};

TG.adHooks.handleLevelCompleted = function(){
    "use strict";
    TG.controller.continueGameFlowAfterAd();
};
//******************************************************
//*****    Script Resource: Inheritance
//******************************************************


/* exported Inheritance */
var Inheritance = {
	copyPrototype: function(classFrom, classTo) {
		"use strict";
		for ( var prop in classFrom.prototype) {
			classTo.prototype[prop] = classFrom.prototype[prop];
		}
	}
};
//******************************************************
//*****    Script Resource: Easer
//******************************************************

/**
 * @constructor Easer
 */
var Easer = function(initial, defaultDuration, easeFunction, clampFunction) {
	"use strict";
	this.from = initial;
	this.to = initial;
	this.defaultDuration = defaultDuration;
	this.duration = 1;
	this.clampFunction = clampFunction;
	this.easeFunction = easeFunction;
	this.callBackFunction = null;
	this.totalTimePassed = 0;
};

Easer.prototype.addTime = function(timePassed) {
	"use strict";
	this.totalTimePassed += timePassed;
	if (this.totalTimePassed > this.duration) {
		if (this.callBackFunction) {
			this.callBackFunction();
			this.callBackFunction = null;
		}
	}
};

Easer.prototype.getValue = function() {
	"use strict";

	if (this.totalTimePassed >= this.duration) {
		return this.easeFunction(this.duration, this.from, this.to - this.from, this.duration);
	}

	var result = this.easeFunction(this.totalTimePassed, this.from, this.to - this.from, this.duration);

	if (this.clampFunction) {
		result = this.clampFunction(result);
	}
	return result;
};

Easer.prototype.easeTo = function(to, duration, callBackFunction) {
	"use strict";
	this.from = this.getValue();
	this.to = to;
	this.duration = duration || this.defaultDuration;
	this.callBackFunction = callBackFunction;
	this.totalTimePassed = 0;
};

Easer.prototype.isDone = function() {
	"use strict";
	return this.totalTimePassed > this.duration;
};
//******************************************************
//*****    Script Resource: TG_BrowserScaling
//******************************************************

/* global
	tbBrowser
 */

var TG_BrowserScaling = function() {
	"use strict";
	this.minGrade = 1;
	this.maxGrade = 3;

	this.grade = this.maxGrade;
	this._determineGrade();

	TG.version["browser_name"] = tbBrowser.name;
	TG.version["browser_version"] = tbBrowser.browser.version;
	TG.version["os_version"] = tbBrowser.os.version;
	TG.version["browser_grade"] = this.grade;

};

TG_BrowserScaling.prototype._determineGrade = function() {
	"use strict";

	if (tbBrowser.browser.stockandroid) {
		if (tbBrowser.os.versionAsNumber < 3) {
			this.grade = 1;
		}
		else if (tbBrowser.os.versionAsNumber < 4.4) {
			this.grade = 2;
		}
		else {
			this.grade = 3;
		}
	}

	if (tbBrowser.os.ios) {

		if (tbBrowser.os.versionAsNumber < 7) {
			this.grade = 2;
		}
		else {
			this.grade = 3;
		}
	}

	if (tbBrowser.os.windowsphone) {
		if (tbBrowser.os.versionAsNumber < 8) {
			this.grade = 2;
		}
		else {
			this.grade = 3;
		}
	}

};

TG_BrowserScaling.prototype.hasGrade1 = function() {
	"use strict";
	return this.grade >= 1;
};

TG_BrowserScaling.prototype.hasGrade2 = function() {
	"use strict";
	return this.grade >= 2;
};

TG_BrowserScaling.prototype.hasGrade3 = function() {
	"use strict";
	return this.grade >= 3;
};

TG_BrowserScaling.prototype.getGradedSetting = function(settingTriplet) {
	"use strict";
	if (typeof settingTriplet === "object" && typeof settingTriplet.length !== "undefined")	 {
		return settingTriplet[this.grade-1];
	}
	else {
		return settingTriplet;
	}

};

TG_BrowserScaling.prototype.forceGrade = function(grade) {
	"use strict";
	this.grade = grade;
};
//******************************************************
//*****    Script Resource: TG_Globals
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

/*
 * Fill TG object with data
 */
TG.environment = (TG["environment"] !== undefined)? TG["environment"] : "development";
TG.analyticsProperty = (TG["ga"] !== undefined)? TG["ga"] : TG.environment;
if(typeof TG["mediaUrl"] !== "undefined"){
	tbLoader.setDataLocation(TG["mediaUrl"]);
}else{
	tbLoader.setDataLocation(TG.size);
}

/*
 * Special event id's
 */
TG.backButtonEventId = "backButton";
TG.languageSetEventId = "languageSet";
TG.resizeEventId = "resizeEvent";

TG.version = {
    "builder": builderVersion,
    "build-time": buildTime,
    "build-date": buildDate,
    "audio": (tbAudio.usingWebAudio)? "web audio api" : ((tbAudio.usingHTMLAudio)? "html5 audio" : "no audio")
};

TG.browserScaling = new TG_BrowserScaling();

TG.settings = {};
// init settings
if(typeof initTGSettings === "function"){
	initTGSettings();
}
if(typeof initGameSettings === "function"){
	initGameSettings();
}
if(typeof initThemeSettings === "function"){
	initThemeSettings();
}
if(typeof initGameThemeSettings === "function"){
	initGameThemeSettings();
}

TG.settings.tg = (typeof tgSettingsVar !== "undefined")? tgSettingsVar : {};
TG.settings.game = (typeof gameSettingsVar !== "undefined")? gameSettingsVar : {};
TG.settings.theme = (typeof themeSettingsVar !== "undefined")? themeSettingsVar : {};
TG.settings.gameTheme = (typeof gameThemeSettingsVar !== "undefined")? gameThemeSettingsVar : {};
TG.publisherSettings = window["publisherSettings"];

/*
 * Set-up configuration
 */
TG.config = (typeof game_configuration !== "undefined")? game_configuration : {};
if(typeof configuration_general !== "undefined"){
	TG.config = configuration_general;
}
if(typeof configuration_product !== "undefined"){
	for( var s in configuration_product){
		TG.config[s] = configuration_product[s];
	}
}

/*
 * Set-up dictionary
 */
(function(){
	"use strict";
	var defaultLanguage, states, i, finalLanguageSet, browserLangPrefs, j, l, regex, urlLanguageSet, urlParams;
	TG.dictionary = {};
	TG.dictionary._dictionary = (typeof dictionaryVar !== "undefined")? dictionaryVar : {};

	//determine available languages based on client configuration
	TG.dictionary._availableLanguages = (TG.config.languages !== undefined && TG.config.languages.available !== undefined)? TG.config.languages.available
	        : TG.settings.tg.languages.available;
	//remove languages from available list if they are not supported by the game
	if(TG.config.supportedLanguages){
		for(i = TG.dictionary._availableLanguages.length - 1; i >= 0; i--){
			if(TG.config.supportedLanguages.indexOf(TG.dictionary._availableLanguages[i]) < 0){
				TG.dictionary._availableLanguages.splice(i, 1);
			}
		}
	}
	// check url parameters and remove languages that are not in param
	try{
		urlParams = (function(){
			var queryString, params, queries, temp, i, l;
			params = {};
			queryString = window.location.search.substring(1);
			if(queryString){
				// Split into key/value pairs
				queries = queryString.split("&");

				// Convert the array of strings into an object
				for(i = 0, l = queries.length; i < l; i++){
					temp = queries[i].split("=");
					params[temp[0]] = temp[1];
				}
			}
			return params;
		}());

		if(urlParams["lang"]){
			urlLanguageSet = urlParams["lang"].toLowerCase().split("+");
			for(i = TG.dictionary._availableLanguages.length - 1; i >= 0; i--){
				if(urlLanguageSet.indexOf(TG.dictionary._availableLanguages[i]) < 0){
					TG.dictionary._availableLanguages.splice(i, 1);
				}
			}
		}
	}catch(e){
	}
	
	// make sure available language set is non empty
	if(TG.dictionary._availableLanguages.length === 0){
		TG.dictionary._availableLanguages.push("en-us");
	}
	// try to determine browser language
	browserLangPrefs = navigator["languages"]? navigator["languages"] : [navigator["language"] || navigator["userLanguage"]];
	for(i = 0; i < browserLangPrefs.length; i++){
		if(typeof browserLangPrefs[i] === "string"){
			l = browserLangPrefs[i].toLowerCase();
			for(j = 0; j < TG.dictionary._availableLanguages.length; j++){
				if(TG.dictionary._availableLanguages[j].search(l) >= 0){
					defaultLanguage = TG.dictionary._availableLanguages[j];
					break;
				}
			}
			if(defaultLanguage !== undefined){
				break;
			}
		}
	}
	if(defaultLanguage === undefined){
		defaultLanguage = (TG.config.languages !== undefined && TG.config.languages.initial !== undefined)? TG.config.languages.initial
		        : TG.settings.tg.languages.initial;
	}
	TG.dictionary._language = (TG.dictionary._availableLanguages.indexOf(defaultLanguage) >= 0)? defaultLanguage : TG.dictionary._availableLanguages[0];
	TG.dictionary._langDictionary = TG.dictionary._dictionary[TG.dictionary._language];
	// setup language toggle button
	if(TG.settings.tg.buttonDefinitions["language_toggle"] !== undefined && TG.settings.tg.buttonDefinitions["language_toggle"].states !== undefined){
		states = TG.settings.tg.buttonDefinitions["language_toggle"].states;
		finalLanguageSet = [];
		for(i = 0; i < states.length; i++){
			if(TG.dictionary._availableLanguages.indexOf(states[i].id) >= 0){
				finalLanguageSet.push(states[i]);
			}
		}
		TG.settings.tg.buttonDefinitions["language_toggle"].states = finalLanguageSet;
	}

	// add dictionary functions
	TG.dictionary.getText = function(key, defaultText){
		var text, subStringsToReplace, replacement, i, touch;
		if(TG.dictionary._langDictionary !== undefined && TG.dictionary._langDictionary[key] !== undefined){
			text = TG.dictionary._langDictionary[key];
			subStringsToReplace = text.match(/#touch{.*}\s*{.*}/g);
			if(subStringsToReplace){
				for(i = 0; i < subStringsToReplace.length; i++){
					touch = tbBrowser.features.tablet || tbBrowser.features.mobile;
					// specifications
					replacement = touch? subStringsToReplace[i].match(/{[^}]*}/g)[1] : subStringsToReplace[i].match(/{[^}]*}/g)[0];
					replacement = replacement.substring(1, replacement.length - 1);
					text = text.replace(subStringsToReplace[i], replacement);
				}
			}
			return text;
		}
		return defaultText;
	};

	TG.dictionary.setLanguage = function(languageKey){
		TG.dictionary._language = languageKey;
		TG.dictionary._langDictionary = TG.dictionary._dictionary[languageKey];
		tbGame.addSpecial(TG.languageSetEventId, languageKey);
	};

	TG.dictionary.getLanguage = function(){
		return TG.dictionary._language;
	};

	TG.dictionary.getAvailableLanguages = function(){
		return TG.dictionary._availableLanguages;
	};
	
	TG.dictionary.isLanguageAvailable = function(languageKey){
		return TG.dictionary._availableLanguages.indexOf(languageKey) >= 0;
	};

}());

// determine device/browser data
TG.deviceData = (function(){
	"use strict";
	var data = {};
	data.os = "";
	data.os_major = "";
	data.os_minor = "";
	data.device = "";

	return data;
}());

// Set-up google analytics
/*
 * (function(){ if(TG.config.track !== false){ (function(i,s,o,g,r,a,m){
 * i["GoogleAnalyticsObject"] = r; i[r] = i[r] || function(){(i[r]["q"] =
 * i[r]["q"] || []).push(arguments);}; i[r]["l"] = 1*new Date(); a =
 * s.createElement(o); m = s.getElementsByTagName(o)[0]; a.async = 1; a.src = g;
 * m.parentNode.insertBefore(a,m); })(window, document, "script",
 * "//www.google-analytics.com/analytics.js", "gameAnalytics");
 * window["gameAnalytics"]("create", "UA-44198463-9", "tinglygames.com"); //test
 * //window["gameAnalytics"]("create", "UA-44198463-10", "greetinggames.com");
 *
 * window["gameAnalytics"]("send", "pageview"); } })(); TG.analytics =
 * window["gameAnalytics"];
 */
/*
 * Define Global functions
 */
TG.functions = {};

TG.functions.createEvent = function(eventName, params){
	"use strict";
	var e, d, b, c;
	d = params.detail || {};
	b = params.bubbles || false;
	c = params.cancelable || false;
	if(typeof CustomEvent === "function"){
		e = new CustomEvent(eventName, {
		    detail: d,
		    bubbles: b,
		    cancelable: c
		});
	}else{
		try{
			e = document.createEvent("CustomEvent");
			e.initCustomEvent(eventName, b, c, d);
		}catch(err){
			e = document.createEvent("Event");
			e.initEvent(eventName, b, c);
			e.data = d;
		}
	}
	return e;
};

TG.functions.timeToStr = function(value){
	"use strict";
	var sign = (value < 0)? "-" : "";
	var minutes = Math.floor(value / 60000);
	var seconds = Math.floor((value % 60000) / 1000);
	return sign + minutes + (seconds < 10? ":0" : ":") + seconds;
};

TG.functions.inherit = function(C, P){
	"use strict";
	var F = function(){
	};
	F.prototype = P.prototype;
	C.prototype = new F();
};

TG.functions.setupCanvas = function(id, depth, top, left, width, height){
	"use strict";
	var element, shouldAdd = false, canvas = document.getElementById(id);
	if(!canvas){
		shouldAdd = true;
		canvas = document.createElement("canvas");
		canvas.id = id;
	}
	canvas.style.zIndex = depth;
	canvas.style.top = top + "px";
	canvas.style.left = left + "px";
	canvas.width = width;
	canvas.height = height;
	if(shouldAdd){
		element = document.getElementById("viewport");
		if(element){
			element.appendChild(canvas);
		}else{
			document.body.appendChild(canvas);
		}
	}

	TG.canvasses.push(canvas);
	return canvas;
};

/*
 * Setup Canvasses & Layers
 */
(function(){
	"use strict";
	var setting, forceDOMUpdate, updateCanvases, updateViewport, startHeight, iterations, createCanvas, gameCanvasSettings, defaultCanvasSettings;

	// setup parameters for resizing/positioning script
	TG.lastWindowHeight = 0;
	TG.lastWindowWidth = 0;
	TG.scrollAndResizeTimerId = false;

	// use DOM update hack on stockbrowser for android 4 or higher
	TG.usesDOMUpdateHack = tbBrowser.browser.stockandroid && tbBrowser.browser.versionAsNumber && tbBrowser.browser.versionAsNumber >= 4;
	TG.DOMUpdating = false;

	//determine if full screen should be used (not on pc)
	TG.usesFullScreen = tbBrowser.features.tablet || tbBrowser.features.mobile;

	// determine orientation and settings
	TG.orientation = (buildTags.indexOf("landscape") >= 0)? "landscape" : "portrait";
	defaultCanvasSettings = (TG.orientation === "landscape")? TG.settings.tg.canvasses_landscape : TG.settings.tg.canvasses;
	gameCanvasSettings = (TG.orientation === "landscape")? TG.settings.game.canvasses_landscape : TG.settings.game.canvasses;
	if(gameCanvasSettings !== undefined){
		if(gameCanvasSettings.canvasGame !== undefined){
			for(setting in gameCanvasSettings.canvasGame){
				defaultCanvasSettings.canvasGame[setting] = gameCanvasSettings.canvasGame[setting];
			}
		}
		if(gameCanvasSettings.canvasGameUI !== undefined){
			for(setting in gameCanvasSettings.canvasGameUI){
				defaultCanvasSettings.canvasGameUI[setting] = gameCanvasSettings.canvasGameUI[setting];
			}
		}
	}

	forceDOMUpdate = function(){
		var canvasses, i, canvas, event, images;
		if(TG.usesDOMUpdateHack && !TG.DOMUpdating){
			TG.DOMUpdating = true;

			// find all canvases
			canvasses = document.getElementsByTagName("canvas");
			if(canvasses){
				for(i = 0; i < canvasses.length; i++){
					canvas = canvasses[i];
					if(!(canvas.getContext && canvas.getContext("2d"))){ // no
						// support
						TG.DOMUpdating = false;
						return;
					}
				}
			}

			// pause game
			event = document.createEvent("Event");
			event.params = [false];
			event.initEvent("gameSetPause", false, false);
			window.dispatchEvent(event);

			// copy canvas image data and clear canvasses
			images = [];
			for(i = 0; i < canvasses.length; i++){
				canvas = canvasses[i];
				var context = canvas.getContext("2d");
				try{
					var image = context.getImageData(0, 0, canvas.width, canvas.height);
					images.push(image);
				}catch(e){
					if(testMode){
						console.log("error in getting image data: " + e.name + ", " + e.message);
					}
				}
				context.clearRect(0, 0, canvas.width, canvas.height);
				canvas.style.visibility = "hidden";
			}

			// DOM refresh
			setTimeout(function(){
				for(var i = 0; i < canvasses.length; i++){
					var canvas = canvasses[i];
					canvas.style.visibility = "visible";
				}
			}, 1);

			// redraw image data
			setTimeout(function(){
				for(var i = 0; i < canvasses.length; i++){
					var context = canvasses[i].getContext("2d");
					try{
						context.putImageData(images[i], 0, 0);
					}catch(e){
						if(testMode){
							console.log("error in putting image data: " + e.name + ", " + e.message);
						}
					}
				}

				// activate engine
				var event = document.createEvent("Event");
				event.initEvent("gameResume", false, false);
				window.dispatchEvent(event);

				TG.DOMUpdating = false;
			}, 100);
		}
	};

	updateCanvases = function(){
		var windowSizes, settingSizes, minSize, preferredAspect, aspect, minAspect, _shortSide, _longSide, _scalingRatio, defaultOffsetTop, defaultOffsetLeft, _width, _height, divWidth, divHeight, defaultTopOffset, resizeCanvas;

		// determine aspect ratio
		if(TG.orientation === "landscape"){
			windowSizes = [window.innerWidth, window.innerHeight];
			settingSizes = [defaultCanvasSettings.defaultImageWidth, defaultCanvasSettings.defaultHeight];
			minSize = defaultCanvasSettings.minWidth;
		}else{
			windowSizes = [window.innerHeight, window.innerWidth];
			settingSizes = [defaultCanvasSettings.defaultHeight, defaultCanvasSettings.defaultWidth];
			minSize = defaultCanvasSettings.minHeight;
		}
		preferredAspect = settingSizes[0] / settingSizes[1];
		aspect = windowSizes[0] / windowSizes[1];
		minAspect = minSize / settingSizes[1];
		if(aspect < preferredAspect){
			if(aspect < minAspect){
				// resize short side to match long side
				_shortSide = Math.floor(windowSizes[0] / minAspect);
				_longSide = windowSizes[0];
			}else{
				// use total of short side and total of long side
				_shortSide = windowSizes[1];
				_longSide = windowSizes[0];
			}
		}else{
			// use total of short side, adapt long side
			_shortSide = windowSizes[1];
			_longSide = Math.floor(windowSizes[1] * preferredAspect);
		}
		_scalingRatio = _shortSide / settingSizes[1];

		// check if scaling should be applied (for example not for desktops)
		if(!TG.usesFullScreen && _scalingRatio > 1){
			// only centre screen not resize(up) except if forced
			_longSide = Math.min(windowSizes[0], settingSizes[0]);
			_shortSide = Math.min(windowSizes[1], settingSizes[1]);
			_scalingRatio = 1;
		}
		_width = (TG.orientation === "landscape")? _longSide : _shortSide;
		_height = (TG.orientation === "landscape")? _shortSide : _longSide;

		// determine position
		defaultOffsetTop = 0;
		defaultOffsetLeft = 0;
		if(window.innerHeight < Math.floor(defaultCanvasSettings.defaultHeight * _scalingRatio)){
			defaultOffsetTop = Math.max(defaultCanvasSettings.minTop, window.innerHeight - Math.floor(defaultCanvasSettings.defaultHeight * _scalingRatio));
		}
		if(window.innerWidth < Math.floor(defaultCanvasSettings.defaultWidth * _scalingRatio)){
			defaultOffsetLeft = Math.floor(Math.max((defaultCanvasSettings.defaultImageWidth - defaultCanvasSettings.defaultWidth), (window.innerWidth - Math
			        .floor(defaultCanvasSettings.defaultWidth * _scalingRatio))
			        / _scalingRatio));
			if(window.innerWidth < Math.floor(defaultCanvasSettings.defaultWidth * _scalingRatio) + defaultOffsetLeft * _scalingRatio){
				defaultOffsetLeft += Math.floor(Math.max((minSize - defaultCanvasSettings.defaultImageWidth) / 2,
				        ((window.innerWidth - (defaultCanvasSettings.defaultWidth * _scalingRatio + defaultOffsetLeft * _scalingRatio)) / 2) / _scalingRatio));
			}
		}

		// set global variables for reference outside script
		TG.backgroundOffsetTop = defaultCanvasSettings.defaultHeight - defaultCanvasSettings.defaultImageHeight;
		TG.backgroundOffsetLeft = defaultCanvasSettings.defaultWidth - defaultCanvasSettings.defaultImageWidth;
		TG.canvasTopOffset = defaultOffsetTop;
		TG.canvasLeftOffset = defaultOffsetLeft;
		TG.canvasCenterLeftOffset = Math.min(TG.backgroundOffsetLeft, TG.canvasOffsetLeft * -1);

		//only update gameUIOffsetY because it is the only canvas that moves relatively to the main canvas and only in a vertical direction
		//TG.gameUIOffsetX = (defaultCanvasSettings.canvasGameUI.left || defaultCanvasSettings.defaultLeft) - TG.canvasLeftOffset;
		TG.gameUIOffsetY = (defaultCanvasSettings.canvasGameUI.top || defaultCanvasSettings.defaultTop) - TG.canvasTopOffset;

		TG.visibleArea = {
		    top: defaultOffsetTop * -1,
		    left: defaultOffsetLeft * -1,
		    height: Math.min(defaultCanvasSettings.defaultHeight, Math.round(Math.min(_height, window.innerHeight) / _scalingRatio)),
		    width: Math.min(defaultCanvasSettings.defaultWidth, Math.round(Math.min(_width, window.innerWidth) / _scalingRatio))
		};
		
		TG.safeZone = (TG.orientation === "landscape")? {
			top: 0,
			left: Math.floor((defaultCanvasSettings.defaultImageWidth - defaultCanvasSettings.minWidth)/2),
			width: defaultCanvasSettings.minWidth,
			height: defaultCanvasSettings.minHeight
		} : {
			top: Math.abs(defaultCanvasSettings.minTop),
			left: defaultCanvasSettings.defaultLeft,
			width: defaultCanvasSettings.defaultWidth,
			height: defaultCanvasSettings.minHeight
		};
		
		// resize div
		divHeight = Math.min(window.innerHeight, _height);
		divWidth = Math.min(window.innerWidth, _width);

		if((TG.orientation === "landscape")){
			document.getElementById("viewport").setAttribute(
			        "style",
			        "position:fixed; overflow:hidden; z-index: 0; width:" + divWidth + "px; left:50%; margin-left:" + (-divWidth / 2) + "px; height: "
			                + divHeight + "px; top:50%; margin-top:" + (-divHeight / 2) + "px");
		}else{
			document.getElementById("viewport").setAttribute(
			        "style",
			        "position:absolute; overflow:hidden; z-index: 0; width:" + divWidth + "px; left:50%; margin-left:" + (-divWidth / 2) + "px; height: "
			                + divHeight + "px");

		}
		// resize canvasses
		resizeCanvas = function(canvas, settings, addTopOffset, addLeftOffset){
			var t, l, w, h;
			t = (settings.top !== undefined)? settings.top : defaultCanvasSettings.defaultTop;
			l = (settings.left !== undefined)? settings.left : defaultCanvasSettings.defaultLeft;
			w = (settings.width !== undefined)? settings.width : defaultCanvasSettings.defaultWidth;
			h = (settings.height !== undefined)? settings.height : defaultCanvasSettings.defaultHeight;
			canvas.cssTop = Math.floor(_scalingRatio * t);
			canvas.cssLeft = Math.floor(_scalingRatio * l);
			canvas.cssWidth = Math.floor(_scalingRatio * w);
			canvas.cssHeight = Math.floor(_scalingRatio * h);
			if(addTopOffset !== false){
				t += defaultOffsetTop;
			}
			if(addLeftOffset !== false){
				l += defaultOffsetLeft;
			}

			canvas.setAttribute("style", "position:absolute; left:" + Math.floor(_scalingRatio * l) + "px; top:" + Math.floor(_scalingRatio * t) + "px; width:"
			        + Math.floor(_scalingRatio * w) + "px; height:" + Math.floor(_scalingRatio * h) + "px; z-index: " + settings.depth);
		};

		resizeCanvas(TG.backgroundCanvas, defaultCanvasSettings.canvasBackground);
		resizeCanvas(TG.gameCanvas, defaultCanvasSettings.canvasGame);
		resizeCanvas(TG.gameUICanvas, defaultCanvasSettings.canvasGameUI, false, true);
		resizeCanvas(TG.mainCanvas, defaultCanvasSettings.canvasMain);

		forceDOMUpdate();
		//add additional DOM updates because sometimes the first update seems to be too early
		//and the loading bar will remain visible on top of the game
		setTimeout(forceDOMUpdate, 5000);
		setTimeout(forceDOMUpdate, 10000);
		setTimeout(forceDOMUpdate, 20000);
		tbGame.addSpecial(TG.resizeEventId);
	};

	updateViewport = function(){
		if(TG.lastWindowHeight === window.innerHeight && TG.lastWindowWidth === window.innerWidth){
			return false;
		}

		/*
		// if we are running in webapp mode (iOS), there"s nothing to scroll
		// away
		if(navigator["standalone"] || !TG.deviceNeedsResizing){
			document.documentElement["style"]["min-height"] = "100%";
			document.getElementById("viewport")["style"]["height"] = window.innerHeight + "px";
			TG.lastWindowWidth = window.innerWidth;
			TG.lastWindowHeight = window.innerHeight;
			updateCanvases();
			return false;
		}*/

		if(TG.scrollAndResizeTimerId){
			return false;
		}

		// maximise the document element"s height to be able to scroll away the
		// url bar
		document.documentElement["style"]["min-height"] = 5000;

		startHeight = window.innerHeight;
		iterations = 40;
		TG.scrollAndResizeTimerId = window.setInterval(function(){
			// set viewport and document size
			document.documentElement["style"]["minHeight"] = "";
			document.documentElement["style"]["min-height"] = "";

			// Android needs to scroll by at least 1px
			window.scrollTo(0, tbBrowser.browser.stockandroid? 1 : 0);
			iterations--;
			if((tbBrowser.browser.stockandroid? false : (window.innerHeight > startHeight)) || iterations < 0){
				TG.lastWindowWidth = window.innerWidth;
				TG.lastWindowHeight = window.innerHeight;
				clearInterval(TG.scrollAndResizeTimerId);
				TG.scrollAndResizeTimerId = false;

				// set viewport and document size
				document.documentElement["style"]["min-height"] = window.innerHeight + "px";
				document.getElementById("viewport")["style"]["height"] = window.innerHeight + "px";

				updateCanvases();
			}
		}, 10);
	};
	
	// store game and gameUI offsets
	TG.gameOffsetX = defaultCanvasSettings.canvasGame.left || defaultCanvasSettings.defaultLeft;
	TG.gameOffsetY = defaultCanvasSettings.canvasGame.top || defaultCanvasSettings.defaultTop;
	TG.gameWidth = defaultCanvasSettings.canvasGame.width || defaultCanvasSettings.defaultWidth;
	TG.gameHeight = defaultCanvasSettings.canvasGame.height || defaultCanvasSettings.defaultHeight;
	TG.gameUIOffsetX = defaultCanvasSettings.canvasGameUI.left || defaultCanvasSettings.defaultLeft;
	TG.gameUIOffsetY = defaultCanvasSettings.canvasGameUI.top || defaultCanvasSettings.defaultTop;
	TG.gameUIWidth = defaultCanvasSettings.canvasGameUI.width || defaultCanvasSettings.defaultWidth;
	TG.gameUIHeight = defaultCanvasSettings.canvasGameUI.height || defaultCanvasSettings.defaultHeight;
	TG.mainOffsetX = defaultCanvasSettings.canvasMain.left || defaultCanvasSettings.defaultLeft;
	TG.mainOffsetY = defaultCanvasSettings.canvasMain.top || defaultCanvasSettings.defaultTop;
	TG.mainWidth = defaultCanvasSettings.canvasMain.width || defaultCanvasSettings.defaultWidth;
	TG.mainHeight = defaultCanvasSettings.canvasMain.height || defaultCanvasSettings.defaultHeight;
	
	// create canvasses
	createCanvas = function(settings){
		var t = (settings.top !== undefined)? settings.top : defaultCanvasSettings.defaultTop, l = (settings.left !== undefined)? settings.left
		        : defaultCanvasSettings.defaultLeft, w = (settings.width !== undefined)? settings.width : defaultCanvasSettings.defaultWidth, h = (settings.height !== undefined)? settings.height
		        : defaultCanvasSettings.defaultHeight;
		return TG.functions.setupCanvas(settings.id, settings.depth, t, l, w, h);
	};
	TG.canvasses = [];
	TG.backgroundCanvas = createCanvas(defaultCanvasSettings.canvasBackground);
	TG.gameCanvas = createCanvas(defaultCanvasSettings.canvasGame);
	TG.gameUICanvas = createCanvas(defaultCanvasSettings.canvasGameUI);
	TG.mainCanvas = createCanvas(defaultCanvasSettings.canvasMain);

	updateCanvases();

	// add event listeners + timer to react to scrolling, size and orientation
	// changes
	document.body.addEventListener("touchmove", function(event){
		event.preventDefault();
	}, true);
	document.body.addEventListener("touchstart", updateViewport, true);
	window.addEventListener("resize", updateViewport, true);// used for
	// orientation &&
	// resize
	window.setInterval(updateViewport, 200);

	TG.layerCanvasMap = {};
	TG.layerCanvasMap[TG.gameBackgroundLayer] = TG.backgroundCanvas;
	TG.layerCanvasMap[TG.gameLayer] = TG.gameCanvas;
	TG.layerCanvasMap[TG.gameUILayer] = TG.gameUICanvas;
	TG.layerCanvasMap[TG.gameEffectsLayer] = TG.mainCanvas;
	TG.layerCanvasMap[TG.frameworkBackgroundLayer] = TG.backgroundCanvas;
	TG.layerCanvasMap[TG.frameworkUILayer] = TG.mainCanvas;
	TG.layerCanvasMap[TG.frameworkEffectsLayer] = TG.mainCanvas;
}());

TG.functions.addBanner = function(show, clickable){
	"use strict";
	var image, listenerUp, div = document.getElementById("viewport");
	if(div){
		image = document.createElement("img");
		image.className = "banner";
		image.src = tbLoader.getDataLocation() + "/media/banner_game_640x100.png";
		image.style.position = "absolute";
		image.style.bottom = "0px";
		image.style.width = "100%";
		image.style.zIndex = 300;
		div.appendChild(image);
		TG.bannerClickable = true;
		TG.bannerVisible = true;
		listenerUp = function(e){
			if(TG.bannerClickable && TG.bannerVisible){
				TG.hooks.openUrl("http://www.tinglygames.com/html5-games/");
				e.preventDefault();
				if(e.stopPropagation){
					e.stopPropagation();
				}else{
					e.cancelBubble = true;
				}
			}
		};
		image.addEventListener("mouseup", listenerUp, true);
		image.addEventListener("touchend", listenerUp, true);
		image.addEventListener("mousedown", function(e){
			if(TG.bannerVisible){
				e.preventDefault();
				if(e.stopPropagation){
					e.stopPropagation();
				}else{
					e.cancelBubble = true;
				}
			}
		}, true);
		image.addEventListener("touchstart", function(e){
			if(TG.bannerVisible){
				e.preventDefault();
				if(e.stopPropagation){
					e.stopPropagation();
				}else{
					e.cancelBubble = true;
				}
			}
		}, true);
		if(!show){
			image.style.display = "none";
			TG.bannerVisible = false;
			return true;
		}
	}
};

TG.functions.showBanner = function(){
	"use strict";
	var i, images = document.getElementsByClassName("banner"); // images =
	// document.getElementsByClassName("myAd");
	if(images){
		for(i = 0; i < images.length; i++){
			images[i].style.display = "inline";
		}
		TG.bannerVisible = true;
	}
};

TG.functions.hideBanner = function(){
	"use strict";
	var i, images = document.getElementsByClassName("banner"); // images =
	// document.getElementsByClassName("myAd");
	if(images){
		for(i = 0; i < images.length; i++){
			images[i].style.display = "none";
		}
		TG.bannerVisible = false;
	}
};

TG.functions.getCanvasMainOffset = function(canvas){
	"use strict";
	if(canvas === TG.gameCanvas){
		return {
		    x: TG.gameOffsetX,
		    y: TG.gameOffsetY
		};
	}else if(canvas === TG.gameUICanvas){
		return {
		    x: TG.gameUIOffsetX,
		    y: TG.gameUIOffsetY
		};
	}else{
		return {
		    x: TG.mainOffsetX,
		    y: TG.mainOffsetY
		};
	}
};

/**
 * Returns canvas to which the specified layer is assigned
 *
 * @param {Number}
 *            layer identifier
 * @return {Object}
 */
TG.functions.getCanvas = function(layer){
	"use strict";
	return TG.layerCanvasMap[layer];
};

/**
 * Sets active canvas (tbGraphics.canvas) to the canvas to which the specified
 * layer is assigned
 *
 * @param {Number}
 *            layer
 * @returns {Boolean} true if canvas is set, false if associated canvas could
 *          not be found
 */
TG.functions.setCanvas = function(layer){
	"use strict";
	if(TG.layerCanvasMap[layer]){
		if(tbGraphics.canvas !== TG.layerCanvasMap[layer]){
			tbGraphics.setCanvas(TG.layerCanvasMap[layer]);
		}
		return true;
	}
	return false;
};

/**
 * Sets the layer and canvas of the specified token
 *
 * @param {Object}
 *            token for which to set layer and canvas
 * @param {Number}
 *            layer identifier
 * @return {Boolean} true if canvas is set, false if associated canvas could not
 *         be found
 */
TG.functions.setLayerAndCanvas = function(token, layer){
	"use strict";
	if(TG.layerCanvasMap[layer]){
		tbGame.setTokenLayer(token, layer);
		token.canvas = TG.layerCanvasMap[layer];
		return true;
	}
	return false;
};

TG.functions.computeImgCoordinatesFromSettings = function(settingsObj, canvasDimension, imageDimension, origin){
	"use strict";
	var offset;
	canvasDimension = canvasDimension || 0;
	imageDimension = imageDimension || 0;
	origin = origin || 0;

	if(typeof settingsObj === "number"){
		return settingsObj;
	}
	if(typeof settingsObj === "object"){
		offset = settingsObj.offset || 0;
		switch(settingsObj.align){
			case "center":
				return Math.round(canvasDimension / 2 - (imageDimension / 2 - origin)) + offset;
			case "left":
			case "top":
				return offset - origin;
			case "right":
			case "bottom":
				return canvasDimension - imageDimension - offset - origin;
			default:
				return offset + 0;
		}
	}
	return 0;
};

TG.functions.computeTextCoordinatesFromSettings = function(settingsObj, canvasDimension, textDimension, textAlignment){
	"use strict";
	var offset;
	canvasDimension = canvasDimension || 0;
	textDimension = textDimension || 0;

	if(typeof settingsObj === "number"){
		return settingsObj;
	}
	if(typeof settingsObj === "object"){
		offset = settingsObj.offset || 0;
		switch(settingsObj.align){
			case "center":
				if(textAlignment === "center" || textAlignment === "middle"){
					return Math.round(canvasDimension / 2) + offset;
				}else if(textAlignment === "left" || textAlignment === "top"){
					return Math.round(canvasDimension / 2 - textDimension / 2) + offset;
				}else{
					return Math.round(canvasDimension / 2 + textDimension / 2) - offset;
				}
			break;
			case "left":
				// no break
			case "top":
				if(textAlignment === "center" || textAlignment === "middle"){
					return Math.round(textDimension / 2) + offset;
				}else if(textAlignment === "left" || textAlignment === "top"){
					return offset;
				}else{
					return textDimension + offset;
				}
			break;
			case "right":
				// no break
			case "bottom":
				if(textAlignment === "center" || textAlignment === "middle"){
					return canvasDimension - Math.round(textDimension / 2) - offset;
				}else if(textAlignment === "left" || textAlignment === "top"){
					return canvasDimension - Math.round(textDimension / 2) - offset;
				}else{
					return canvasDimension - offset;
				}
			break;
			default:
				return offset + 0;
		}
	}
	return 0;
};

TG.functions.computeTextSurfaceCoordinatesFromSettings = function(offset, canvasDimension, textDimension, textAlignment){
	"use strict";
	switch(textAlignment){
		case "center":
		case "middle":
			return Math.round(canvasDimension / 2) + offset;
		case "left":
		case "top":
			return offset;
		case "right":
		case "bottom":
			return textDimension + offset;
	}
	return 0;
};
//******************************************************
//*****    Script Resource: Provider_GameAnalytics
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.analyticsProvider = TG.analyticsProvider || {};

TG.analyticsProvider.showDebugDataAnalytics = false;


TG.analyticsProvider.initialise = function(args){
    "use strict";
    if(args instanceof Array){
        this.gameKey = args[0];
        this.secretKey = args[1];
        this.baseUrl = "http://api.gameanalytics.com/1/"+this.gameKey;
        this.initialised = true;
    }
};

TG.analyticsProvider._sendEvent = function(category, message){
    "use strict";
    var request,
        msg = JSON.stringify(message),
        md5Msg = TG_MD5.createMD5Hash(msg + this.secretKey),
        url = this.baseUrl+"/"+category;

    try{ //might fail in IE9
        //create xmlhttp request
        request=new XMLHttpRequest();
        request.open("POST", url, true);
        if(this.showDebugDataAnalytics){
            request.onreadystatechange = function(){
                if(request.readyState === 4){
                    if(request.status === 200){
                        console.log("GOOD! statusText: " + request.statusText);
                        console.log(message);
                    }
                    else{
                        console.log("ERROR ajax call error: " + request.statusText + ", url: " + url);
                    }
                }
            };
        }
        request.setRequestHeader("Content-Type", "text/plain");
        request.setRequestHeader("Authorization", md5Msg);
        request.send(msg);
    }catch(e){}
};

TG.analyticsProvider._createJSONObject = function(){
    "use strict";
    return {
        "user_id": this.userId,
        "session_id": this.sessionId,
        "build": this.build
    };
};

TG.analyticsProvider.startSession = function(sessionId, gameId, themeId, publisherId, version, userInfo, deviceInfo){
    "use strict";
    var userData;
    this.sessionId = sessionId;
    if(userInfo && typeof userInfo === "object"){
        this.userId = userInfo.userId;
    }
    this.build = version;
    this.active = true;

    //send user data
    if(this.initialised){
        userData = this._createJSONObject();
        userData["device"] = deviceInfo.device;
        userData["platform"] = deviceInfo.os;
        userData["os_major"] = deviceInfo.os_major;
        userData["os_minor"] = deviceInfo.os_minor;
        userData["install_publisher"] = publisherId;
        userData["install_site"] = (window["location"] !== window["parent"]["location"]) ? document["referrer"]: document["location"].href;
        this._sendEvent("user", userData);
    }
    this.reportEvent("start", "session");
};

TG.analyticsProvider.endSession = function(){
    "use strict";
    this.active = false;
    this.reportEvent("end", "session");
};

TG.analyticsProvider.reportEvent = function(eventName, categories, value){
    "use strict";
    //check if tracking should be done
    if(!this.active || !this.initialised){
        return;
    }

    var data,
        eventId = "";
    if(categories){
        if(categories instanceof Array){
            eventId = categories.toString().replace(",",":");
        }else{
            eventId += categories;
        }
    }
    //create JSON object
    data = this._createJSONObject();
    data["event_id"] = eventId + ":" + eventName;
    data["value"] = value;
    //send event
    this._sendEvent("design", data);
};

TG.analyticsProvider.reportTimedEvent = function(eventName, categories, duration){
    "use strict";
    this.reportEvent(eventName, categories, duration);
};

TG.analyticsProvider.reportError = function(errorMessage, errorSeverity){
    "use strict";
    var data = this._createJSONObject();
    data["message"] = errorMessage;
    data["severity"] = errorSeverity;
    this._sendEvent("error", data);
};


//******************************************************
//*****    Token Resource: TG_Analytics
//******************************************************

// ***************************************
// Definition of game token TG_Analytics
// ***************************************

// The constructor for the TG_Analytics game token.
var TG_Analytics = function() {
	"use strict";
	// Properties used by the engine
	this.depth = 0;
	this.layer = 0;
	this.visible = false;
	this.active = true;
	// Your own properties
	this.settings = TG.settings.tg.analytics;
	this.poolSize = this.settings.timerPoolSize;
	// Add it to the game engine
	tbGame.addToken(this);
	tbGame.addTokenTags(this, "system");
};

TG_Analytics.prototype._getSessionId = function(){
    "use strict";
    return this._generateUniqueId();
};

TG_Analytics.prototype._getUserId = function(){
    "use strict";
    var userId = TG.controller._getGameData("userId", "");
    if(userId === ""){
        userId = this._generateUniqueId();
        TG.controller._updateGameData("userId", userId);
    }
    return userId;
};

TG_Analytics.prototype._generateUniqueId = function(){
    "use strict";
    var guid = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx";
    return guid.replace(/[xy]/g, function(c) {
        var r = Math.random()*16|0, v = c === "x" ? r : (r&0x3|0x8);
        return v.toString(16);
    });
};

TG_Analytics.prototype.start = function(args){
    "use strict";
    TG.analyticsProvider.initialise(args);
    TG.analyticsProvider.startSession(TG.analytics._getSessionId(), TG.settings.game.framework.gameId, TG.settings.theme.id, TG.config.publisherId, TG.controller.getShortVersion(), {userId: TG.analytics._getUserId()}, TG.deviceData);
};

TG_Analytics.prototype.reportEvent = function(eventName, parents, value){
    "use strict";
    TG.analyticsProvider.reportEvent(eventName, parents, value);
};

TG_Analytics.prototype.reportTimedEventStart = function(eventName, parents, tag, reportStart){
    "use strict";
    var i;
    for(i=0; i<this.timers.length; i++){
        if(!this.timers[i].active){
            this.timers[i].timer = 0;
            this.timers[i].paused = 0;
            this.timers[i].active = true;
            this.timers[i].eventName = eventName;
            this.timers[i].parents = parents;
            this.timers[i].tag = tag;
            break;
        }
    }
    if(i === this.timers.length){
        this.timers.push({active: true, timer: 0, paused: 0, eventName: eventName, parents: parents, tag:tag});
    }

    if(reportStart){
        this.reportEvent(eventName, parents);
    }
};

TG_Analytics.prototype.reportTimedEventStop = function(tag, eventName, parents){
    "use strict";
    var i, timer;
    for(i=0; i<this.timers.length; i++){
        if(this.timers[i] !== undefined && this.timers[i].tag === tag){
            timer = this.timers[i];
            this.reportEvent((eventName !== undefined)? eventName : timer.eventName, (parents !== undefined)? parents: timer.parents, timer.timer);
            timer.active = false;
        }
    }
};

TG_Analytics.prototype.pauseTimedEvent = function(tag){
    "use strict";
    var i;
    for(i=0; i<this.timers.length; i++){
        if(this.timers[i] !== undefined && this.timers[i].tag === tag){
            this.timers[i].paused += 1;
        }
    }
};

TG_Analytics.prototype.resumeTimedEvent = function(tag){
    "use strict";
    var i;
    for(i=0; i<this.timers.length; i++){
        if(this.timers[i] !== undefined && this.timers[i].tag === tag){
            this.timers[i].paused -= 1;
            this.timers[i].paused = Math.max(this.timers[i].paused, 0);
        }
    }
};

TG_Analytics.prototype.pauseAllTimedEvents = function(){
    "use strict";
    var i;
    for(i=0; i<this.timers.length; i++){
        if(this.timers[i].active){
            this.timers[i].paused += 1;
        }
    }
};

TG_Analytics.prototype.resumeAllTimedEvents = function(){
    "use strict";
    var i;
    for(i=0; i<this.timers.length; i++){
        if(this.timers[i].active){
            this.timers[i].paused -= 1;
            this.timers[i].paused = Math.max(this.timers[i].paused, 0);
        }
    }
};

TG_Analytics.prototype.reportError = function(message, severity){
    "use strict";
    TG.analyticsProvider.reportError(message, severity);
};

// handleAdd is called when the token is (re-) added to the game.
TG_Analytics.prototype.handleAdd = function() {
	"use strict";
	this.timers = [];
};

// handleBeginStep is called each step before processing other events.
TG_Analytics.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	var i, toRemove, lastActiveTimerIdx = 0;
	for(i=0; i<this.timers.length; i++){
	    if(this.timers[i].active){
	        if(this.timers[i].paused === 0){
	            this.timers[i].timer += timePassed;
	        }
	        lastActiveTimerIdx = i;
	    }
	}
	if(lastActiveTimerIdx < this.timers.length - 1){
	    toRemove = this.timers.length - Math.max(this.poolSize, lastActiveTimerIdx + 1);
	    if(toRemove > 0){
	        this.timers.splice(this.timers.length - toRemove, toRemove);
	    }
	}
};

//******************************************************
//*****    Script Resource: TG_Persistence
//******************************************************

/* global
	tbText
 */

/** @constructor */
var TG_Persistence = function(localOnly, host, service) {
	"use strict";
    this.localOnly = localOnly || false;
    this.host = host || "http://localhost:8080";
    this.service = service || this.host+"/services/storage/gamestate";
    this.supportLocalStorage =  typeof window["localStorage"] !== "undefined";
    this.useParentWindowCommunication = false;

    this.initialized = false;
	var _this = this;

    if ((window.parent !== window && (tbBrowser.browser.safari || tbBrowser.os.ios))) {

    	window.addEventListener("message", function(event) {
    		_this.handleMessage(event);
    	}, false);

    	this.getItemCallBack = null;

    	window.parent.postMessage({"command": "init"}, "*");
    }

    this.onInitialized = [];

    window.setTimeout(function() {
    	_this.initialized = true;

    	for (var i = 0; i < _this.onInitialized.length; ++i) {
    		_this.onInitialized[i]();
    	}

    	_this.onInitialized = [];
    }, 2000);

};

TG_Persistence.prototype.handleMessage = function(event) {
	"use strict";

	var data = event.data;
	var command = data["command"];

	if (command === "init") {
	    this.useParentWindowCommunication = data["result"] === "ok";
	}
	else if (command === "getItem") {
		if (this.getItemCallBack) {
			if (data["result"] === "ok") {
				this.getItemCallBack(data["value"]);
			}
			else {
				this.getItemCallBack(data["defaultValue"]);
			}
		}
	}
	else if (command === "setItem") {
	}

};

TG_Persistence.prototype.createKey = function(){
	"use strict";
	if(typeof TG.config.storageKey === "string" && TG.config.storageKey !== ""){
		return TG.config.storageKey;
	}
	if(TG.settings.game.framework !== undefined && TG.settings.game.framework.gameId !== undefined){
		return TG.settings.game.framework.gameId;
	}
	return "0";
};

TG_Persistence.prototype.loadData = function(defaultValue, callback){
	"use strict";
	if(typeof callback === "function"){

		if (!this.initialized) {
			var _this = this;
			this.onInitialized.push(function() {
				_this.loadState(defaultValue, callback);
			});
		}
		else {
			this.loadState(defaultValue, callback);
		}
	}
};

TG_Persistence.prototype.saveData = function(data){
    "use strict";
    if (!this.initialized) {
		var _this = this;
		this.onInitialized.push(function() {
			_this.saveState(data);
		});
	}
	else {
		this.saveState(data);
	}
};


TG_Persistence.prototype.saveState = function(state) {
	"use strict";
    var stateWrapper = null, gameId = this.createKey();

    try{
        stateWrapper = JSON.stringify({
            "lastChanged": new Date(),
            "gameState": JSON.stringify(state)
        });
    }catch(e){}

    if (this.useParentWindowCommunication) {
    	window.parent.postMessage({"command": "setItem", "key": "TG_" + gameId, "value": stateWrapper}, "*");
    }
    else {
        //save locally
        if(this.supportLocalStorage){
            try {
                window["localStorage"].setItem(gameId, stateWrapper);
            }catch(e){}
        }
        //save on server
        if(!this.localOnly){
            var remoteStorageText = new tbText("gameState_" + gameId);
            remoteStorageText.stringifyJSON(state);
            remoteStorageText.post(this.service+"/my_ip/"+gameId);
        }
    }
};

TG_Persistence.prototype.loadState = function(defaultValue, callback) {
	"use strict";

    var localStateWrapper = null, remoteStateWrapper = null,
        gameId = this.createKey();


    if (this.useParentWindowCommunication) {
    	this.getItemCallBack = function(data) {
    		var value;
    		try {
        		localStateWrapper = JSON.parse(data);
        		value = JSON.parse(localStateWrapper["gameState"]);
    		} catch (e) {
    			value = defaultValue;
    		}
            callback(value);
    	};
    	window.parent.postMessage({"command": "getItem", "key": "TG_" + gameId}, "*");
    	return;
    }
    else {
        //get data from local storage
        if(this.supportLocalStorage) {
            try {
                localStateWrapper = window["localStorage"].getItem(gameId);
                if (localStateWrapper) {
                    localStateWrapper = JSON.parse(localStateWrapper);
                }
            }catch(e){
            	callback(defaultValue);
                return;
            }
        }


        //get data from server
        if(!this.localOnly){
            var remoteStorageText = new tbText("gameState_" + gameId);
            remoteStateWrapper = null;
            if (remoteStorageText.getSynchronous(TG_Persistence.remoteStorageService+"/my_ip/"+gameId)) {
                remoteStateWrapper = remoteStorageText.parseJSON();
            }
        }

        try{
            //compare timestamps
            if(localStateWrapper){
                if(remoteStateWrapper && Date.parse(remoteStateWrapper["lastChanged"]) > Date.parse(localStateWrapper["lastChanged"]) ){
                	callback(JSON.parse(remoteStateWrapper["gameState"]));
                	return;
                }
                callback(JSON.parse(localStateWrapper["gameState"]));
                return;
            }else if(remoteStateWrapper){
            	callback(JSON.parse(remoteStateWrapper["gameState"]));
                return;
            }
        }catch(e){
        	callback(defaultValue);
            return;
        }
    }

    callback(defaultValue);
    return;
};
//******************************************************
//*****    Token Resource: TG_LoadingScreen
//******************************************************

// ***************************************
// Definition of game token TG_LoadingScreen
// ***************************************

// The constructor for the TG_LoadingScreen game token.
var TG_LoadingScreen = function(callbackEnd, callbackProgress, visible){
	"use strict";
	//Report what URL we are running on. This is what we site-lock against, so we report it a reasonable distance away from where
	//the sitelocking code actually happens.
	if(console && console.log) {
		var site = (window.location.origin)? window.location.origin : window.location.protocol + "//" + window.location.hostname;
		console.log("Hosted on: " + site);
	}	
	// Properties used by the engine
	this.depth = 1000;
	this.visible = (visible !== false);
	this.forceDraw = this.visible;
	this.active = true;
	//use background canvas to avoid android back buffer bug
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	var setting;
	this.settings = TG.settings.tg.loadingScreen;
	if(TG.orientation === "landscape" && TG.settings.tg.loadingScreen_landscape){
		for(setting in TG.settings.tg.loadingScreen_landscape){
			this.settings[setting] = TG.settings.tg.loadingScreen_landscape[setting];
		}
	}
	//apply theme specific setting overwrites
	for(setting in TG.settings.theme.loadingScreen){
		this.settings[setting] = TG.settings.theme.loadingScreen[setting];
	}
	//apply customer specific setting overwrites
	if(TG.config.loadingScreen){
		for(setting in TG.config.loadingScreen){
			this.settings[setting] = TG.config.loadingScreen[setting];
		}
	}
	this.callback = callbackEnd;
	this.callbackProgress = callbackProgress;
	this.done = false;
	
	//background fade in
	this.backgroundAlpha = 0;
	this.backgroundFadeInRunning = false;
	this.backgroundFadeInTimer = 0;
	this.backgroundFadeInDuration = this.settings.backgroundFadeDuration;

	//if set to true the background will be drawn each step on the main canvas instead of once on the background canvas
	//true is the default value to avoid artifacts caused by the canvasses resizing a lot at the start of a game on mobile devices
	this.redrawBackground = true;
	
	this.loadingbarFillMultiplier = 0.6 / Math.log(this.settings.maxAudioLoadDuration + 1);
	this.waitAfterReadyTimer = (TG.config.loadingScreenMinimalVisibilityDuration !== undefined)? TG.config.loadingScreenMinimalVisibilityDuration
	        : this.settings.minimalVisibilityDuration;
	this.maxLoadDuration = this.waitAfterReadyTimer + this.settings.maxWaitDuration;
	
	// Add it to the game engine
	tbGame.addToken(this);
};

/*
 * Replaces loading bar and throbber with error message
 */
TG_LoadingScreen.prototype.showError = function(errorKey){
	"use strict";
	var font, text;
	TG.functions.setCanvas(TG.frameworkBackgroundLayer);
	tbGraphics.drawRectangle(0, 0, this.canvas.width, this.canvas.height, "white", false);
	font = f_default.clone();
	if((TG.config.loadingScreen && TG.config.loadingScreen.textFont) || this.settings.textFont){
		font.setProperties((TG.config.loadingScreen && TG.config.loadingScreen.textFont)? TG.config.loadingScreen.textFont : this.settings.textFont);
	}
	text = TG.dictionary.getText(errorKey, "<" + errorKey.toUpperCase() + ">");
	font.drawSimple(text, this.canvas.width / 2, this.canvas.height / 2, this.settings.textMaxWidth);

	this.error = true;
	this.forceDraw = false;
	this.visible = false;
	this.canvas.drawTokens = true;
};

/*
 * Updates the position of all docked elements
 */
TG_LoadingScreen.prototype.positionDockedElements = function(){
	"use strict";
	//the logo-surface should always be as close to the bottom of the screen as possible, but in portrait mode
	//the bottom of the screen and the bottom of the canvas are often not the same, therefore the positions of the
	//logo-surface has to be adjusted to the visible area
	if(this.logosSurface){
		this.logosSurfaceX = TG.functions.computeImgCoordinatesFromSettings(this.settings.logosSurfaceX, TG.visibleArea.width, this.logosSurface.width) + TG.visibleArea.left;
		this.logosSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.settings.logosSurfaceY, TG.visibleArea.height, this.logosSurface.height) + TG.visibleArea.top;
	}
};

/*
 * Creates and draws the surface containing all logos
 */
TG_LoadingScreen.prototype.createLogosSurface = function(){
	"use strict";
	var x, y, i, scale, l, maxLogoWidth;
	if(typeof TG.hooks.getStartScreenLogos === "function"){
		maxLogoWidth = this.settings.logosSurfaceMaxWidth;
		this.logos = TG.hooks.getStartScreenLogos();
		if(this.logos && this.logos.length > 0){
			if(this.logosSurface){
				this.logosSurface.clear();
			}else{
				this.logosSurface = new tbSurface(this.settings.logosSurfaceMaxWidth, this.settings.logosSurfaceMaxHeight);
			}
			this.logosSurface.setTarget();
			maxLogoWidth /= this.logos.length;
			for(i=0; i<this.logos.length; i++){
				try{
					l = this.logos[i].image;
					scale = Math.min(1, Math.min((maxLogoWidth - 20) / l.width, this.settings.logosSurfaceMaxHeight / l.height));
					if(this.settings.logosAlign === "center"){
						x = maxLogoWidth * i + Math.round((maxLogoWidth - l.width * scale) / 2);
					}else{
						x = maxLogoWidth * i + Math.round((maxLogoWidth - l.width * scale)) - 10;
					}
					y = this.logosSurface.height - l.height * scale;
					if(l instanceof tbImage){
						l.drawExt(0, x, y, scale, scale, 0, 1);
					}else{
						tbGraphics.context.drawImage(l, x, y, l.width * scale, l.height * scale);
					}
				}catch(e){
					if(testMode){
						console.log("error in drawing logo: " + e);
					}
				}
			}
			this.logosSurface.resetTarget();
			this.logosSurfaceAlpha = 0;
			this.logosSurfaceAnimating = true;
			this.logosSurfaceAlphaTimer = 0;
			this.logoSurfaceMask = tbMaskRectangle(0, 0, this.logosSurface.width, this.logosSurface.height);
			this.positionDockedElements();
		}
	}
};

/*
 * Draws the background of the Loading Screen, scaling it up when necessary
 */
TG_LoadingScreen.prototype.drawBackground = function(){
	"use strict";
	var top, left, scale, font, text, fillwidth, width, height, i, logo, img, x, y;
	// set canvas
	if(!this.redrawBackground){
		TG.functions.setCanvas(TG.frameworkBackgroundLayer);
	}else{
		tbGraphics.clear();
	}
	// draw background
	if(this.settings.backgroundImage){
		img = this.settings.backgroundImage;
		top = Math.abs(TG.canvasTopOffset);
		
		//in portrait mode the background might have to be scaled up a bit to accomodate
		//taller screens. Check the canvasTopOffset to see if this is the case. If the canvasTopOffset is
		//-80 the canvas (which is bigger than the background) is partially out of view and the background scale is 1,
		//if the offset becomes bigger (less negative) more of the canvas will be in view and the background should become bigger 
		if(img.number > 1){
			scale = (tbGraphics.canvas.height - top) / img.combinedHeight;
			left = -(img.combinedWidth * scale - tbGraphics.canvas.width) / 2;
			img.drawCombinedAlpha(left, top, this.backgroundAlpha);
		}else{
			scale = (this.canvas.height - top) / img.height;
			left = -Math.floor((img.width * scale - this.canvas.width) / 2);
			if(img instanceof tbImage){
				img.drawExt(0, left, top, scale, scale, 0, this.backgroundAlpha);
			}else if(img instanceof tbSurface){
				img.drawExt(left, top, scale, scale, 0, this.backgroundAlpha);
			}
		}
	}
	//draw loading bar background
	width = this.settings.loadingBarBackgroundLeftWidth + this.settings.loadingBarBackgroundRightWidth + this.settings.loadingBarInnerWidth;
	height = s_loadingbar_background.height;
	fillwidth = s_loadingbar_background.width - (this.settings.loadingBarBackgroundLeftWidth + this.settings.loadingBarBackgroundRightWidth);
	this.loadingBarX = TG.functions.computeImgCoordinatesFromSettings(this.settings.loadingBarX, tbGraphics.canvas.width, width);
	this.loadingBarY = TG.functions.computeImgCoordinatesFromSettings(this.settings.loadingBarY, tbGraphics.canvas.height, height);
	s_loadingbar_background.drawPart(0, 0, 0, this.settings.loadingBarBackgroundLeftWidth, height, this.loadingBarX, this.loadingBarY, 1);
	s_loadingbar_background.drawPartTiled(0, this.settings.loadingBarBackgroundLeftWidth, 0, fillwidth, height, this.loadingBarX
	        + this.settings.loadingBarBackgroundLeftWidth, this.loadingBarY, this.settings.loadingBarInnerWidth, height, 1);
	s_loadingbar_background.drawPart(0, this.settings.loadingBarBackgroundLeftWidth + fillwidth, 0, this.settings.loadingBarBackgroundRightWidth, height,
	        this.loadingBarX + this.settings.loadingBarBackgroundLeftWidth + this.settings.loadingBarInnerWidth, this.loadingBarY, 1);
};

/*
 * Starts the loading of the "optional" loading moments and updates the visuals of the loading screen
 */
TG_LoadingScreen.prototype.handleStartTextureLoaded = function(){
	"use strict";
	var i, l, x, y, btn, callback, logos;

	TG.hooks.informStartTextureLoaded();
	
	if(this.redrawBackground){
		this.backgroundFadeInRunning = true;
	}

	if(this.visible){
		//draw background
		this.drawBackground();

		//add logos
		this.createLogosSurface();
		
		//splashscreen
		if(typeof TG.hooks.getSplashScreenSurface === "function"){
			this.splashScreenSurface = TG.hooks.getSplashScreenSurface();
			if(this.splashScreenSurface instanceof tbSurface){
				this.splashScreenVisible = true;
				this.splashScreenSurfaceX = Math.floor((this.canvas.width - this.splashScreenSurface.width) / 2);
				this.splashScreenSurfaceY = Math.floor((this.canvas.height - this.splashScreenSurface.height) / 2);
			}
		}
	}
	//start loading audio
	if(TG.controller.loadSfx){
		tbLoader.startLoading("audio");
	}
	if(TG.controller.loadMusic){
		tbLoader.startLoading("audio_music");
	}
	tbLoader.startLoading("fonts");
};

// handleAdd is called when the token is (re-) added to the game.
TG_LoadingScreen.prototype.handleAdd = function(){
	"use strict";
	//check site-lock
	var i, proceed = false; //&& TG.config.siteLock.domainMD5Hashed
	if(TG.config.siteLock !== undefined){
		if(TG.config.siteLock.lock === false){
			proceed = true;
		}else{
			if(TG.config.siteLock.domains !== undefined){
				for(i = 0; i < TG.config.siteLock.domains.length; i++){
					if(TG_SiteLock.checkSiteLock(TG.config.siteLock.domains[i])){
						proceed = true;
						break;
					}
				}
			}
		}
	}else{
		proceed = true;
	}
	//check time-lock
	if(proceed && typeof TG.config.timeLock === "number"){
		if(new Date().getTime() > TG.config.timeLock){
			proceed = false;
		}
	}
	if(!proceed){
		tbGame.pause();
		return;
	}

	//initialise variables
	this.btns = [];
	this.error = false;
	this.timer = 0;
	this.audioTimer = 0;
	this.fontsTimer = 0;
	this.throbberFrame = 0;
	this.splashScreenVisible = false;
	this.ready = false;
	this.loadingBarXDiff = (this.settings.loadingBarFillXOffset !== undefined)? this.settings.loadingBarFillXOffset
	        : this.settings.loadingBarBackgroundLeftWidth - this.settings.loadingBarFillLeftWidth;
	this.loadingBarYDiff = (this.settings.loadingBarFillYOffset !== undefined)? this.settings.loadingBarFillYOffset : Math
	        .floor((s_loadingbar_background.height - s_loadingbar_fill.height) / 2);
	this.loadingBarMiddleWidth = s_loadingbar_fill.width - (this.settings.loadingBarFillLeftWidth + this.settings.loadingBarFillRightWidth);

	//check what is and isn't already loaded
	this.audioLoaded = false;
	this.musicLoaded = false;
	this.fontsLoaded = false;
	this.startTextureLoaded = tbLoader.complete("start");
	if(this.startTextureLoaded){
		this.handleStartTextureLoaded();
	}
	this.loadTextureLoaded = tbLoader.complete("load");

	//initialise throbber
	if(this.visible){
		this.throbberImage = document.getElementById("throbber_image");
		this.throbberWidth = this.settings.throbberWidth;
		this.throbberX = TG.functions.computeImgCoordinatesFromSettings(this.settings.throbberX, this.canvas.width, this.throbberWidth);
		this.throbberY = TG.functions.computeImgCoordinatesFromSettings(this.settings.throbberY, this.canvas.height, this.throbberWidth);
	}
};

// handleBeginStep is called each step before processing other events.
TG_LoadingScreen.prototype.handleBeginStep = function(timePassed){
	"use strict";
	this.timer += timePassed;

	//splashscreen
	if(typeof TG.hooks.getSplashScreenSurface === "function" && this.splashScreenSurface === undefined){
		this.splashScreenSurface = TG.hooks.getSplashScreenSurface();
		if(this.splashScreenSurface instanceof tbSurface){
			this.splashScreenVisible = true;
			this.splashScreenSurfaceX = Math.floor((this.canvas.width - this.splashScreenSurface.width) / 2);
			this.splashScreenSurfaceY = Math.floor((this.canvas.height - this.splashScreenSurface.height) / 2);
		}
	}
	if(this.splashScreenVisible && this.settings.splashScreenVisibilityDuration >= 0 && this.timer >= this.settings.splashScreenVisibilityDuration){
		this.splashScreenVisible = false;
	}

	// update fadein animation
	if (this.backgroundFadeInRunning) {
		this.backgroundFadeInTimer += timePassed;
		if (this.backgroundFadeInTimer >= this.backgroundFadeInDuration) {
			this.backgroundFadeInRunning = false;
			this.backgroundAlpha = 1;
		}
		else {
			this.backgroundAlpha = tbEase.easeInCubic(this.backgroundFadeInTimer, 0, 1, this.backgroundFadeInDuration);
		}

	}

	//update loading timers
	if(this.startTextureLoaded){
		this.audioTimer += timePassed;
		this.fontsTimer += timePassed;
	}

	//update throbber
	this.throbberFrame = Math.round((this.timer / this.settings.throbberFrameDuration) % (this.settings.throbberFrames - 1));

	//update logo animation
	if(this.logosSurfaceAnimating ){
		this.logosSurfaceAlpha = tbEase.easeLinear(this.logosSurfaceAlphaTimer, 0, 1, this.settings.logosSurfaceAnimDuration);
		this.logosSurfaceAlphaTimer += timePassed;
		if(this.logosSurfaceAlphaTimer >= this.settings.logosSurfaceAnimDuration){
			this.logosSurfaceAnimating = false;
			this.logosSurfaceAlpha = 1;
		}
	}

	//send progress reports if required
	if(typeof this.callbackProgress === "function"){
		this.callbackProgress(Math.round((tbLoader.percentage("load") + tbLoader.percentage("audio") + tbLoader.percentage("audio_music")) / 2));
	}

	//check if "start" and "load" moment are completed and if either "audio", "audio_music" and "fonts" have completed
	//or the max loading time has been exceeded
	if(!this.ready && this.loadTextureLoaded
			&& (this.fontsLoaded || (this.fontsTimer >= this.settings.maxAudioLoadDuration))
	        && (!TG.controller.loadSfx || this.audioLoaded || (tbAudio.usingHTMLAudio && this.audioTimer >= this.settings.maxAudioLoadDuration))
	        && (!TG.controller.loadMusic || this.musicLoaded || (tbAudio.usingHTMLAudio && this.audioTimer >= this.settings.maxAudioLoadDuration))){
		this.ready = true;
	}

	//if loading conditions have been met, check if the loading screen has to remain visible still
	//(because the framework is still waiting for a backe-nd or for promotional reasons)
	//and if not signal main controller to proceed
	if(!this.done && !this.error && this.ready && ((this.timer >= this.waitAfterReadyTimer) || testMode) && (TG.controller._readyForStart() || this.timer >= this.maxLoadDuration)){
		this.done = true;
		this.callback();
	}
};

TG_LoadingScreen.prototype.handleImmediateTouchDown = function(finger, x, y){
	"use strict";
	//check if one of the logos has been touched
    if(!this.splashScreenVisible && this.logoSurfaceMask && this.logoSurfaceMask.collisionPoint(this.logosSurfaceX, this.logosSurfaceY, x, y)){
        this.logoClickIdx = Math.floor((x - this.logosSurfaceX)/(this.logosSurface.width/this.logos.length));
    }
};

TG_LoadingScreen.prototype.handleImmediateTouchUp = function(finger, x, y){
	"use strict";
	//check if one of the logos has been touched and if so open url or call function associated with logo
    if(this.logoClickIdx !== undefined && (this.logos[this.logoClickIdx].url || this.logos[this.logoClickIdx].action)){
        if(this.logoSurfaceMask.collisionPoint(this.logosSurfaceX, this.logosSurfaceY, x, y)){
            x -= this.logosSurfaceX;
            if(x >= this.logoClickIdx * (this.logosSurface.width/this.logos.length) && x < (this.logoClickIdx + 1)* (this.logosSurface.width/this.logos.length)){
                if(this.logos[this.logoClickIdx].url){
                    TG.hooks.openUrl(this.logos[this.logoClickIdx].url);
                }else{
                    this.logos[this.logoClickIdx].action();
                }
            }
        }
    }
    this.logoClickIdx = undefined;
};

// handleSpecial is called if a special or user-defined event happens.
TG_LoadingScreen.prototype.handleSpecial = function(id, args){
	"use strict";
	//check for the completion of loading moments
	if(id === "Load Complete" && args.moment === "start"){
		this.startTextureLoaded = true;
		this.handleStartTextureLoaded();
	}else if(id === "Load Complete" && args.moment === "load"){
		this.loadTextureLoaded = true;
	}else if(id === "Load Complete" && args.moment === "audio"){
		this.audioLoaded = true;
	}else if(id === "Load Complete" && args.moment === "audio_music"){
		this.musicLoaded = true;
	}else if(id === "Load Complete" && args.moment === "fonts"){
		this.fontsLoaded = true;
	}
	if(id === TG.resizeEventId){
        this.positionDockedElements();
    }
};

// handleDraw is called at the end of a step to draw the token.
TG_LoadingScreen.prototype.handleDraw = function(){
	"use strict";
	if(!this.error){
		//draw background (image and loading bar background)
		if(this.redrawBackground && this.startTextureLoaded){
			this.drawBackground();
		}else{
			tbGraphics.clear();
		}
		
		//draw throbber, has to be done in try catch because sometime the throbber image might not be ready yet
		try{
			if(this.throbberImage){
				tbGraphics.context.drawImage(this.throbberImage, this.throbberWidth * this.throbberFrame, 0, this.throbberWidth, this.throbberWidth, this.throbberX, this.throbberY,
				        this.throbberWidth, this.throbberWidth);
			}
		}catch(e){
			if(testMode){
				console.log("error in drawing throbber: " + e.name + ", " + e.message);
			}
		}
		if(this.startTextureLoaded){
			//draw loading bar fill
			var xSource = 0, x = this.loadingBarX + this.loadingBarXDiff, y = this.loadingBarY + this.loadingBarYDiff, fillHeight = s_loadingbar_fill.height;
			s_loadingbar_fill.drawPart(0, xSource, 0, this.settings.loadingBarFillLeftWidth, fillHeight, x, y, 1);
			xSource += this.settings.loadingBarFillLeftWidth;
			x += this.settings.loadingBarFillLeftWidth;
			if(!this.ready){
				s_loadingbar_fill.drawPartTiled(0, xSource, 0, this.loadingBarMiddleWidth, fillHeight, x, y, Math.floor(Math.min(
				        ((tbLoader.percentage("load") + tbLoader.percentage("audio")) / 500 + this.loadingbarFillMultiplier * Math.log(this.timer + 1)), 1)
				        * this.settings.loadingBarInnerWidth), fillHeight, 1);
			}else{
				s_loadingbar_fill.drawPartTiled(0, xSource, 0, this.loadingBarMiddleWidth, fillHeight, x, y, this.settings.loadingBarInnerWidth, fillHeight, 1);
				xSource += this.loadingBarMiddleWidth;
				x += this.settings.loadingBarInnerWidth;
				s_loadingbar_fill.drawPart(0, xSource, 0, this.settings.loadingBarFillRightWidth, fillHeight, x, y, 1);
			}
			//draw logo surface
			if(this.logosSurface){
				this.logosSurface.drawAlpha(this.logosSurfaceX, this.logosSurfaceY, this.logosSurfaceAlpha);
			}
		}
		//draw splash screen
		if(this.splashScreenVisible){
			this.splashScreenSurface.drawSimple(this.splashScreenSurfaceX, this.splashScreenSurfaceY);
		}
	}
};

//******************************************************
//*****    Token Resource: TG_StartScreen
//******************************************************

// ***************************************
// Definition of game token TG_StartScreen
// ***************************************

// The constructor for the TG_StartScreen game token.
var TG_StartScreen = function() {
	"use strict";
	var setting, self, playbuttonDelay;
	self = this;
	// Properties used by the engine
	this.depth = 100;
	this.visible = true;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
    //read settings
    this.settings = TG.settings.tg.startScreen;
    if(TG.orientation === "landscape" && TG.settings.tg.startScreen_landscape){
		for(setting in TG.settings.tg.startScreen_landscape){
			this.settings[setting] = TG.settings.tg.startScreen_landscape[setting];
		}
	}
    this.buttonDefinitions = TG.settings.tg.buttonDefinitions;
    if(TG.orientation === "landscape" && TG.settings.tg.buttonDefinitions_landscape){
		for(setting in TG.settings.tg.buttonDefinitions_landscape){
			this.buttonDefinitions[setting] = TG.settings.tg.buttonDefinitions_landscape[setting];
		}
	}
    this.buttonColors = TG.settings.tg.buttonColorDefinitions;
    //apply theme specific setting overwrites
    for(setting in TG.settings.theme.startScreen){
        this.settings[setting] = TG.settings.theme.startScreen[setting];
    }
    
    //init variables
    this.btns = [];
    var challengeInfo = TG.controller.getChallengeInformation();
    this.challenge = challengeInfo !== undefined && challengeInfo !== null;
    
    //define timeLine
    this.timeLine = new tbTimeLine();
	this.timeLine.addMoment(this.settings.gameLogoDelay, function(){
		self.startGameLogoAnimation.call(self);
	});
	this.timeLine.addMoment(this.settings.optinalButtonsDelay, function(){
		self.startOptionalButtonsAnimation.call(self);
	});
	playbuttonDelay = (TG.gameFlowSettings.showDifficulty && !this.challenge)? this.settings.playButtonsDelay : this.settings.optinalButtonsDelay;
	this.timeLine.addMoment(playbuttonDelay, function(){
		self.startPlayButtonsAnimation.call(self);
	});
	this.timeLine.addMoment(this.settings.menuButtonDelay, function(){
		self.startMenuAnimation.call(self);
	});
	    
    // Add it to the game engine
	tbGame.addToken(this, false);
};

/*
 * Defines and starts the animated appearance of the game logo
 */
TG_StartScreen.prototype.startGameLogoAnimation = function(){
	"use strict";
	this.gameLogoShown = true;
	if(this.settings.gameLogoImage){
		this.gameLogoStartX = TG.functions.computeImgCoordinatesFromSettings(this.settings.gameLogoStartX, this.canvas.width, s_logo.width);
		this.gameLogoEndX = TG.functions.computeImgCoordinatesFromSettings(this.settings.gameLogoEndX, this.canvas.width, s_logo.width);
		this.gameLogoStartY = TG.functions.computeImgCoordinatesFromSettings(this.settings.gameLogoStartY, this.canvas.height, s_logo.height);
		this.gameLogoEndY = TG.functions.computeImgCoordinatesFromSettings(this.settings.gameLogoEndY, this.canvas.height, s_logo.height);
		this.gameLogoX = this.gameLogoStartX;
		this.gameLogoY = this.gameLogoStartY;
		this.gameLogoScaleX = this.settings.gameLogoStartScaleX;
		this.gameLogoScaleY = this.settings.gameLogoStartScaleY;
		this.gameLogoAlpha = this.settings.gameLogoStartAlpha;
		this.gameLogoTimer = 0;
		this.positionDockedElements();
	}
};

/*
 * Defines and starts the animated appearance of the difficulty button
 */
TG_StartScreen.prototype.startOptionalButtonsAnimation = function(animate){
	"use strict";
	var x, y, btn, ease = function(t, s, c, d){
		return tbEase.easeOutElastic(t, s, c, d, 3, 15);
	};
	this.optionalButtonsShown = true;
	if(TG.gameFlowSettings.showDifficulty && !this.challenge){
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.difficultyButtonX, this.canvas.width, this.settings.difficultyButtonWidth, Math.floor(this.settings.difficultyButtonWidth / 2));
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.difficultyButtonY, this.canvas.height, s_btn_toggle.height, Math.floor(s_btn_toggle.height / 2));
		btn = new TG_ButtonToggleText("difficulty_toggle", x, y, this.depth - 20, TG.controller.getDifficulty() + "",
		        this.settings.difficultyButtonWidth, {callbackFunc: function(toggleValue){
			        TG.controller._setDifficulty(parseInt(toggleValue, 10));
			        return true;
		        }, immediate: true});
		btn.originX = Math.floor(this.settings.difficultyButtonWidth / 2);
		btn.originY = Math.floor(s_btn_toggle.height / 2);
		if(animate !== false){
			btn.addAnimation("xScale", ease, 0, 1, this.settings.difficultyButtonAnimationDuration);
			btn.addAnimation("yScale", ease, 0, 1, this.settings.difficultyButtonAnimationDuration);
		}
		this.difficultyButton = btn;
		this.difficultyButtonY = btn.y;
		this.btns.push(btn);
		this.positionDockedElements();
	}
};

/*
 * Defines and starts the animated appearance of the play button
 */
TG_StartScreen.prototype.startPlayButtonsAnimation = function(animate){
	"use strict";
	var x, y, btn, self = this, ease = function(t, s, c, d){
		return tbEase.easeOutElastic(t, s, c, d, 3, 15);
	};
	this.playButtonsShown = true;
	x = TG.functions.computeImgCoordinatesFromSettings(this.settings.playButtonX, this.canvas.width, this.settings.playButtonWidth, Math.floor(this.settings.playButtonWidth / 2));
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.playButtonY, this.canvas.height, s_btn_bigtext.height, Math.floor(s_btn_bigtext.height / 2));
	btn = new TG_ButtonText("bigPlay", x, y, this.depth - 20, "startScreenPlay", this.settings.playButtonWidth, {
	    callbackFunc: function(){
		    tbGame.removeToken(self);
		    TG.controller._startGame();
		    return true;
	    },
	    immediate: true
	});
	btn.originX = Math.floor(this.settings.playButtonWidth / 2);
	btn.originY = Math.floor(s_btn_bigtext.height / 2);
	if(animate !== false){
		btn.addAnimation("xScale", ease, 0, 1, this.settings.playButtonAnimationDuration);
		btn.addAnimation("yScale", ease, 0, 1, this.settings.playButtonAnimationDuration);
		this.playButtonAnimationTimer = 0;
	}else{
		this.playButtonAnimationTimer = this.settings.playButtonAnimationDuration;
	}
	this.playButton = btn;
	this.playButtonY = btn.y;
	this.btns.push(btn);
	this.positionDockedElements();
};

/*
 * Defines and starts the play button "wiggle" animation
 */
TG_StartScreen.prototype.startPlayButtonWiggleAnimation = function(){
	"use strict";
	var elasticEase = function(time, begin, change, duration, oscillations, stiffness){
		return tbEase.easeOutElastic(time, begin, change, duration, 3, 2);
	};
	var startButtonEaseFunction = tbEase.easeCreateCombined([tbEase.easeOutOvershoot, elasticEase, tbEase.easeConstant], [true, false, false], [0.02,
	        0.1, 0.88]);
	this.playButtonAnimating = true;
	this.playButton.addAnimation("xScale", tbEase.easeCreateRepeat(startButtonEaseFunction, tbEase.repeatRepeat), 1, 0.25, 4000, false);
	this.playButton.addAnimation("yScale", tbEase.easeCreateRepeat(startButtonEaseFunction, tbEase.repeatRepeat), 1, -0.1, 4000, false);
};

/*
 * Defines and starts the animated appearance of the options button
 */
TG_StartScreen.prototype.startMenuAnimation = function(animate){
	"use strict";
	var x, y, btn;
	this.menuButtonShown = true;
	x = TG.functions.computeImgCoordinatesFromSettings(this.settings.menuButtonX, this.canvas.width, s_btn_small_options.width);
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.menuButtonY, this.canvas.height, s_btn_small_options.height);
	btn = new TG_ButtonBase(x, y, this.depth - 20, new tbMaskImage(
	        s_btn_small_options), [s_btn_small_options], {callbackFunc: TG.controller.showOptions, immediate: true});
	if(animate !== false){
		btn.addAnimation("alpha", tbEase.easeLinear, 0, 1, this.settings.menuButtonAnimationDuration);
	}
	this.menuButton = btn;
	this.menuButtonY = btn.y;
	this.btns.push(btn);
	this.positionDockedElements();
};


/*
 * Draws the background of the Start Screen, scaling it up when necessary
 */
TG_StartScreen.prototype.drawBackground = function(){
	"use strict";
	var image, scale, top, left, x, y;
	// draw background
	image = this.settings.backgroundImage;
	if(image){
		TG.functions.setCanvas(TG.frameworkBackgroundLayer);
		top = Math.abs(TG.canvasTopOffset);
		if(image.number > 1){
			scale = (tbGraphics.canvas.height - top) / image.combinedHeight;
			left = -(image.combinedWidth * scale - tbGraphics.canvas.width) / 2;
			image.drawCombinedSimple(left, top);
		}else{
			scale = (tbGraphics.canvas.height - top) / image.height;
			left = -Math.floor((image.width * scale - this.canvas.width) / 2);
			image.drawExt(0, left, top, scale, scale, 0, 1);
		}
	}
};

/*
 * Creates and draws the surface containing all logos
 */
TG_StartScreen.prototype.createLogosSurface = function(){
	"use strict";
	var x, y, i, scale, l, maxLogoWidth, extraLogo;
	if(typeof TG.hooks.getStartScreenLogos === "function"){
		maxLogoWidth = this.settings.logosSurfaceMaxWidth;
		this.logos = TG.hooks.getStartScreenLogos();
		if(this.logos && this.logos.length > 0){
			if(this.logosSurface){
				this.logosSurface.clear();
			}else{
				this.logosSurface = new tbSurface(this.settings.logosSurfaceMaxWidth, this.settings.logosSurfaceMaxHeight);
			}
			this.logosSurface.setTarget();
			maxLogoWidth /= this.logos.length;
			for(i in this.logos){
				try{
					l = this.logos[i].image;
					scale = Math.min(1, Math.min((maxLogoWidth - 20) / l.width, this.settings.logosSurfaceMaxHeight / l.height));
					if(this.settings.logosAlign === "center"){
						x = maxLogoWidth * i + Math.round((maxLogoWidth - l.width * scale) / 2);
					}else{
						x = maxLogoWidth * i + Math.round((maxLogoWidth - l.width * scale)) - 10;
					}
					y = this.logosSurface.height - l.height * scale;
					if(l instanceof tbImage){
						l.drawExt(0, x, y, scale, scale, 0, 1);
					}else{
						tbGraphics.context.drawImage(l, x, y, l.width * scale, l.height * scale);
					}
				}catch(e){
					if(testMode){
						console.log("error in drawing logo: " + e);
					}
				}
			}
			this.logosSurface.resetTarget();
			this.logosSurfaceAlpha = 0;
			this.logosSurfaceAnimating = true;
			this.logosSurfaceAlphaTimer = 0;
			this.logoSurfaceMask = tbMaskRectangle(0, 0, this.logosSurface.width, this.logosSurface.height);
			this.positionDockedElements();
		}
	}
};

/*
 * Updates the position of all docked elements
 */
TG_StartScreen.prototype.positionDockedElements = function(){
    "use strict";
    var i, btn, buttonOffsetY, buttonsMaxY = 0;
    //move buttons up if necessary
    buttonOffsetY = 0;
    if(TG.visibleArea.height < this.settings.btnMinCanvasHeight){
        buttonOffsetY = this.settings.btnMinCanvasHeight - TG.visibleArea.height;
    }
    if(this.playButtonsShown){
		this.playButton.y = this.playButtonY - buttonOffsetY;
	}
	if(this.menuButtonShown){
		this.menuButton.y = this.menuButtonY - buttonOffsetY;
		this.menuButton.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.menuButtonX, TG.visibleArea.width, s_btn_small_options.width) + TG.visibleArea.left;
	}
	if(this.difficultyButton){
		this.difficultyButton.y = this.difficultyButtonY - buttonOffsetY;
	}
	
	//move game logo
	if(this.gameLogoShown && this.gameLogoTimer >= this.settings.gameLogoAnimationDuration){
		this.gameLogoY = this.gameLogoEndY - TG.canvasTopOffset;
	}
	
	//move company logos
    if(this.logosSurface){
		this.logosSurfaceX = TG.functions.computeImgCoordinatesFromSettings(this.settings.logosSurfaceX, TG.visibleArea.width, this.logosSurface.width) + TG.visibleArea.left;
		this.logosSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.settings.logosSurfaceY, TG.visibleArea.height, this.logosSurface.height) + TG.visibleArea.top;
	}
};

// handleAdd is called when the token is (re-) added to the game.
TG_StartScreen.prototype.handleAdd = function() {
	"use strict";
	var i, btn, btnType, btnX = 0, btnY, state = "medium", startButton,
    buttons = this.settings.btns, buttonOffsetY = 0;

	//draw background
    this.drawBackground();

	//draw game logo outside screen to activate texture swap
	if(this.settings.gameLogoImage){
		TG.functions.setCanvas(TG.frameworkUILayer);
		this.settings.gameLogoImage.drawSimple(0, 0, -this.settings.gameLogoImage.height - 10);
	}
	
	//create company logosSurface
	this.createLogosSurface();
    
	this.timeLine.start();
};

// handleRemove is called when the token is removed from the game.
TG_StartScreen.prototype.handleRemove = function() {
	"use strict";
	var i;
	for(i=0; i<this.btns.length; i++){
		tbGame.removeToken(this.btns[i]);
	}
};

//handleBeginStep is called each step before processing other events.
TG_StartScreen.prototype.handleBeginStep = function(timePassed){
	"use strict";
	this.canvas.drawTokens = true;
	if(this.gameLogoShown && this.gameLogoTimer < this.settings.gameLogoAnimationDuration){
		this.gameLogoX = this.settings.gameLogoEaseX(this.gameLogoTimer, this.gameLogoStartX, this.gameLogoEndX - this.gameLogoStartX,
		        this.settings.gameLogoAnimationDuration);
		this.gameLogoY = this.settings.gameLogoEaseY(this.gameLogoTimer, this.gameLogoStartY, this.gameLogoEndY - this.gameLogoStartY,
		        this.settings.gameLogoAnimationDuration) - TG.canvasTopOffset;
		this.gameLogoScaleX = this.settings.gameLogoEaseScaleX(this.gameLogoTimer, this.settings.gameLogoStartScaleX, this.settings.gameLogoEndScaleX
		        - this.settings.gameLogoStartScaleX, this.settings.gameLogoAnimationDuration);
		this.gameLogoScaleY = this.settings.gameLogoEaseScaleY(this.gameLogoTimer, this.settings.gameLogoStartScaleY, this.settings.gameLogoEndScaleY
		        - this.settings.gameLogoStartScaleY, this.settings.gameLogoAnimationDuration);
		this.gameLogoAlpha = this.settings.gameLogoEaseAlpha(this.gameLogoTimer, this.settings.gameLogoStartAlpha, this.settings.gameLogoEndAlpha
		        - this.settings.gameLogoStartAlpha, this.settings.gameLogoAnimationDuration);
		this.gameLogoTimer += timePassed;
		if(this.gameLogoTimer >= this.settings.gameLogoAnimationDuration){
			this.gameLogoX = this.gameLogoEndX;
			this.gameLogoY = this.gameLogoEndY - TG.canvasTopOffset;
			this.gameLogoScaleX = this.settings.gameLogoEndScaleX;
			this.gameLogoScaleY = this.settings.gameLogoEndScaleY;
			this.gameLogoAlpha = this.settings.gameLogoEndAlpha;
		}
	}
	if(this.playButtonsShown){
		if(!this.playButtonAnimating && this.playButtonAnimationTimer >= this.settings.playButtonAnimationDuration + this.settings.playButtonWiggleDelay){
			this.startPlayButtonWiggleAnimation();
		}
		this.playButtonAnimationTimer += timePassed;
	}
};

TG_StartScreen.prototype.handleImmediateTouchDown = function(finger, x, y){
	"use strict";
	//check if one of the logos has been touched
    if(this.logoSurfaceMask && this.logoSurfaceMask.collisionPoint(this.logosSurfaceX, this.logosSurfaceY, x, y)){
        this.logoClickIdx = Math.floor((x - this.logosSurfaceX)/(this.logosSurface.width/this.logos.length));
    }
};

TG_StartScreen.prototype.handleImmediateTouchUp = function(finger, x, y){
	"use strict";
	//check if one of the logos has been touched and if so open url or call function associated with logo
    if(this.logoClickIdx !== undefined && (this.logos[this.logoClickIdx].url || this.logos[this.logoClickIdx].action)){
        if(this.logoSurfaceMask.collisionPoint(this.logosSurfaceX, this.logosSurfaceY, x, y)){
            x -= this.logosSurfaceX;
            if(x >= this.logoClickIdx * (this.logosSurface.width/this.logos.length) && x < (this.logoClickIdx + 1)* (this.logosSurface.width/this.logos.length)){
                if(this.logos[this.logoClickIdx].url){
                    TG.hooks.openUrl(this.logos[this.logoClickIdx].url);
                }else{
                    this.logos[this.logoClickIdx].action();
                }
            }
        }
    }
    this.logoClickIdx = undefined;
};

//handleTouchDown is called when a touch is started or if a mouse button is
//pressed.
TG_StartScreen.prototype.handleTouchDown = function(finger, x, y){
	"use strict";
	this.down = true;
};

//handleTouchUp is called when a touch is ended or if a mouse button is
//released.
TG_StartScreen.prototype.handleTouchUp = function(finger, x, y){
	"use strict";
	if(this.down){
		//end all animations and add missing elements to screen
		this.timeLine.stop();
		
		//game logo
		if(this.gameLogoShown){
			if(this.gameLogoTimer < this.settings.gameLogoAnimationDuration){
				this.gameLogoTimer = this.settings.gameLogoAnimationDuration - 1;
			}
		}else{
			this.startGameLogoAnimation();
			this.gameLogoTimer = this.settings.gameLogoAnimationDuration - 1;
		}
		//difficulty button
		if(this.difficultyButton){
			this.difficultyButton.endAnimations(true);
		}else{
			this.startOptionalButtonsAnimation(false);
		}
		//menu
		if(this.menuButtonShown){
			this.menuButton.endAnimations(true);
		}else{
			this.startMenuAnimation(false);
		}
		//play button
		if(this.playButtonsShown){
			this.playButton.endAnimations(true);
			if(this.playButtonAnimating){
				this.startPlayButtonWiggleAnimation();
			}
		}else{
			this.startPlayButtonsAnimation(false);
		}
		this.down = false;
	}
};


TG_StartScreen.prototype.handleSpecial = function(id, args) {
    "use strict";
    if(id === TG.resizeEventId){
        this.drawBackground();
        this.positionDockedElements();
    }
};

TG_StartScreen.prototype.handleDraw = function(){
    "use strict";
    //draw game logo
	if(this.gameLogoShown && this.settings.gameLogoImage){
		this.settings.gameLogoImage.drawExt(0, this.gameLogoX, this.gameLogoY, this.gameLogoScaleX, this.gameLogoScaleY, 0, this.gameLogoAlpha);
	}
	//draw logo surface
	if(this.logosSurface){
		this.logosSurface.drawSimple(this.logosSurfaceX, this.logosSurfaceY);
	}
    this.forceDraw = false;
};


//******************************************************
//*****    Token Resource: TG_LevelMapScreen
//******************************************************

// ***************************************
// Definition of game token TG_LevelMapScreen
// ***************************************

/*global f_default, s_level_stars, s_level_lock, s_level_0, s_level_1, s_level_2, s_level_3 */
// The constructor for the TG_LevelMapScreen game token.
var TG_LevelMapScreen = function(){
	"use strict";
	// Properties used by the engine
	this.depth = 100;
	this.visible = true;
	this.active = true;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	var s;
	//read framework settings
	this.settings = TG.settings.tg.levelMapScreen;
	if(TG.orientation === "landscape"){
		for(s in TG.settings.tg.levelMapScreen_landscape){
			this.settings[s] = TG.settings.tg.levelMapScreen_landscape[s];
		}
	}
	//read game settings
	this.gameSettings = TG.settings.game.levelMap;
	if(TG.settings.game.levelMapScreen){
		for(s in TG.settings.game.levelMapScreen){
			this.settings[s] = TG.settings.game.levelMapScreen[s];
		}
	}
	//read button settings
	this.buttonSettings = TG.settings.tg.buttonDefinitions;
	//apply theme specific setting overwrites
	for( var setting in TG.settings.theme.levelMapScreen){
		this.settings[setting] = TG.settings.theme.levelMapScreen[setting];
	}

	this.levelPressed = -1;
	this.swipeOffsetX = 0;
	this.levels = [];
	// Add it to the game engine
	tbGame.addToken(this);
};

TG_LevelMapScreen.prototype.drawBackground = function(){
	"use strict";
	var image, scale, top, left, x, y;
	TG.functions.setCanvas(TG.frameworkBackgroundLayer);
	// draw background
	image = (this.settings.backgroundImages && this.settings.backgroundImages[0])? this.settings.backgroundImages[0].sprite : undefined;
	image = (this.settings.backgroundImage)? this.settings.backgroundImage : undefined;
	if(image){
		top = Math.abs(TG.canvasTopOffset);
		if(image.number > 1){
			scale = (tbGraphics.canvas.height - top) / image.combinedHeight;
			left = -(image.combinedWidth * scale - tbGraphics.canvas.width) / 2;
			image.drawCombinedSimple(left, top);
		}else{
			scale = (tbGraphics.canvas.height - top) / image.height;
			left = -Math.floor((image.width * scale - this.canvas.width) / 2);
			image.drawExt(0, left, top, scale, scale, 0, 1);
		}
	}
	this.drawStats();
};

TG_LevelMapScreen.prototype.drawLevels = function(){
	"use strict";
	if(this.settings.type === "grid"){
		this.levelSurface.setTarget();
		tbGraphics.clear();
		this.levelPositions = [];
		var self = this, drawOneWorld;

		drawOneWorld = function(worldIdx, offset, mapLevels){
			var colDistribution, r, c, idx, savedData, image, backgroundImage, x, y, text, font, state, stars, xOff, yOff, drawNumber, drawBackgroundImages, drawLock, scale, backgroundImageScale, tileWidth, tileHeight, noOfLevels, c_max, gridMarginY, xOffset, yOffset, c_tot, r_tot, centerLevels;

			noOfLevels = TG.gameFlowSettings.noOfLevels[worldIdx];
			gridMarginY = (self.useBigImages)? self.settings.gridMarginYBig : self.settings.gridMarginYSmall;
			xOffset = self.settings.gridOffsetX;
			yOffset = gridMarginY;

			//read or create the column distribution (the amount of columns per row)
			if (self.settings.colDistribution){
				colDistribution = self.settings.colDistribution[worldIdx];
			} else {
				c_max = (self.useBigImages)? self.settings.noOfColsBig : self.settings.noOfColsSmall;
				c_tot = c_max;
				r_tot = Math.floor(noOfLevels / c_tot);

				while(Math.abs(r_tot - c_tot) > 1){
					c_tot -= 1;
					r_tot = Math.floor(noOfLevels / c_tot);
				}

				colDistribution = [];
				while(noOfLevels > 0){
					colDistribution.push(Math.min(c_tot, noOfLevels));
					noOfLevels -= c_tot;
				}
				noOfLevels = TG.gameFlowSettings.noOfLevels[worldIdx];
			}

			r_tot = colDistribution.length;
			tileHeight = Math.round((((self.useBigImages)? self.settings.levelsMaxHeightBig : self.settings.levelsMaxHeightSmall) - (r_tot + 1) * gridMarginY) / r_tot);

			var centerLevels = (self.settings.centerLevels)?self.settings.centerLevels:false;
			if(!centerLevels){
				c_max = 1;
				for (r = 0; r < r_tot; r++){
					c_max = Math.max(colDistribution[r], c_max);
				}
				tileWidth = Math.round((self.canvas.width - 2 * xOffset) / c_max);
			}

			idx = 0;
			for (r = 0; r < r_tot; r++){
				c_tot = colDistribution[r];
				if(centerLevels){
					tileWidth = Math.round((self.canvas.width - 2 * xOffset) / c_tot);
				}
				for (c = 0; c < c_tot; c++){
					drawNumber = self.settings.drawNumbers;
					drawBackgroundImages = self.settings.drawBackgroundImages;

					//check saved data
					state = TG.gameFlowSettings.defaultLevelState || "locked";
					stars = 0;
					savedData = TG.controller.getLevelData(worldIdx, idx);
					if(typeof savedData === "object" && savedData !== null){
						if(savedData["state"] !== undefined){
							state = savedData["state"];
						}
						if(typeof savedData["stats"] === "object" && savedData["stats"] !== null){
							stars = savedData["stats"]["stars"] || 0;
						}
					}
					drawLock = state === "locked";

					//check if game wants to show custom level images
					if(typeof TG.game.getLevelImage === "function"){
						image = TG.game.getLevelImage(TG.controller.getSequentialLevelIdx(worldIdx, idx), worldIdx, idx, state);
						if(image){
							drawNumber = false;
							drawLock = false;
							drawBackgroundImages = false;
						}
					}

					//draw level
					x = xOffset + offset;
					y = yOffset;
					scale = 1;
					backgroundImageScale = 1;

					if(drawBackgroundImages !== false){
						backgroundImage = self.useBigImages? s_level_0 : s_level2_0;
						if(state === "played"){
							switch(stars){
								case 1:
									backgroundImage = self.useBigImages? s_level_1 : s_level2_1;
								break;
								case 2:
									backgroundImage = self.useBigImages? s_level_2 : s_level2_2;
								break;
								case 3:
									backgroundImage = self.useBigImages? s_level_3 : s_level2_3;
								break;
							}
						}else if(!self.useBigImages && state === "locked"){
							backgroundImage = s_level2_lock;
						}
						if(backgroundImage.width > tileWidth){
							backgroundImageScale = tileWidth / backgroundImage.width;
						}
						if(backgroundImage.height > tileHeight){
							backgroundImageScale = Math.min(scale, tileHeight / backgroundImage.height);
						}
						x += Math.round((tileWidth - backgroundImage.width * backgroundImageScale) / 2);
						y += Math.round((tileHeight - backgroundImage.height * backgroundImageScale) / 2);
						backgroundImage.drawExt(0, x, y, backgroundImageScale, backgroundImageScale, 0, 1);
						if(mapLevels){
							self.levelPositions[idx] = {
							    x: x,
							    y: y
							};
						}
					}
					if(image){
						if(image.width > tileWidth){
							scale = tileWidth / image.width;
						}
						if(image.height > tileHeight){
							scale = Math.min(scale, tileHeight / image.height);
						}
						if(backgroundImage !== undefined){
							xOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.levelImageOffsetX,
							        backgroundImage.width * backgroundImageScale, image.width * scale);
							yOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.levelImageOffsetY, backgroundImage.height
							        * backgroundImageScale, image.height * scale);
						}else{
							xOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.levelImageOffsetX, tileWidth, image.width * scale);
							yOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.levelImageOffsetY, tileHeight, image.height * scale);
							if(mapLevels){
								self.levelPositions[idx] = {
								    x: x + xOff,
								    y: y + yOff
								};
							}
						}

						if(image instanceof tbSurface){
							image.drawExt(x + xOff, y + yOff, scale, scale, 0, 1);
						}else{
							image.drawExt(0, x + xOff, y + yOff, scale, scale, 0, 1);
						}
					}
					if(drawNumber !== false && !drawLock){
						text = ""+ (TG.gameFlowSettings.usePerWorldLevelNumbering? (idx + 1) : (TG.controller.getSequentialLevelIdx(worldIdx, idx) + 1));
						font = self.fonts.general;
						if(state === "locked" && self.fonts.locked !== undefined){
							font = self.fonts.locked;
						}else if(state === "unlocked" && self.fonts.unlocked !== undefined){
							font = self.fonts.unlocked;
						}else if(state === "played" && self.fonts.played !== undefined){
							font = self.fonts.played;
						}
						if(backgroundImage !== undefined){
							xOff = TG.functions.computeTextCoordinatesFromSettings(self.settings.levelNoOffsetX, backgroundImage.width * backgroundImageScale,
							        font.textWidth(text), font.getAlign());
							yOff = TG.functions.computeTextCoordinatesFromSettings(self.settings.levelNoOffsetY, backgroundImage.height * backgroundImageScale,
							        font.textHeight(text), font.getBaseline());
						}else{
							xOff = TG.functions.computeTextCoordinatesFromSettings(self.settings.levelNoOffsetX, tileWidth, font.textWidth(text), font
							        .getAlign());
							yOff = TG.functions.computeTextCoordinatesFromSettings(self.settings.levelNoOffsetY, tileHeight, font.textHeight(text), font
							        .getBaseline());
						}
						font.drawSimple(text, x + xOff, y + yOff);
					}
					if(self.useBigImages && drawLock){
						if(backgroundImage !== undefined){
							xOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.lockOffsetX, backgroundImage.width * backgroundImageScale,
							        s_level_lock.width);
							yOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.lockOffsetY, backgroundImage.height * backgroundImageScale,
							        s_level_lock.height);
						}else{
							xOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.lockOffsetX, tileWidth, s_level_lock.width);
							yOff = TG.functions.computeImgCoordinatesFromSettings(self.settings.lockOffsetY, tileHeight, s_level_lock.height);
						}
						s_level_lock.drawSimple(0, x + xOff, y + yOff);

					}
					xOffset += tileWidth;

					idx++;
				}
				xOffset = self.settings.gridOffsetX;
				yOffset += tileHeight + gridMarginY;
			}
		};

		if(this.hasPrevious){
			drawOneWorld(this.currentWorldIdx - 1, 0);
		}
		drawOneWorld(this.currentWorldIdx, this.canvas.width, true);
		if(this.hasNext){
			drawOneWorld(this.currentWorldIdx + 1, this.canvas.width * 2);
		}
		this.levelSurface.resetTarget();
	}
};

TG_LevelMapScreen.prototype.animateToWorld = function(worldIdx){
	"use strict";
	var idxDifference = worldIdx - this.currentWorldIdx;

	this.nextWorldIdx = worldIdx;
	switch(idxDifference){
		case 0:
			this.nextWorldX = 0;
		break;
		case 1:
			this.nextWorldX = -this.canvas.width;
		break;
		case -1:
			this.nextWorldX = this.canvas.width;
		break;
	}
	this.moving = true;
	this.moveTimer = 0;
	this.moveStart = this.swipeOffsetX;
	this.moveChange = this.nextWorldX - this.swipeOffsetX;
	this.moveDuration = Math.min(this.settings.moveTotalDuration - this.swipeTimer, Math.round(Math.abs(this.moveChange) / (this.swipeSpeed / 1000)));
	this.moveDuration = Math.max(this.settings.moveMinDuration, this.moveDuration);
};

TG_LevelMapScreen.prototype.addWorldButtons = function(){
	"use strict";
	if(TG.gameFlowSettings.noOfLevels.length > 1){
		var x, y, self = this;

		//previous button
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonPreviousX, this.canvas.width, s_level2_arrow_left.width);
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonPreviousY, TG.visibleArea.height, s_level2_arrow_left.height) + TG.visibleArea.top;
		this.previousBtn = new TG_ButtonBase(x, y, this.depth - 20, new tbMaskImage(s_level2_arrow_left), [s_level2_arrow_left], function(){
			self.swipeDirection = "previous";
			self.animateToWorld(self.currentWorldIdx - 1);
			return true;
		});

		//next button
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonNextX, this.canvas.width, s_level2_arrow_right.width);
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonNextY, TG.visibleArea.height, s_level2_arrow_right.height) + TG.visibleArea.top;
		this.nextBtn = new TG_ButtonBase(x, y, this.depth - 20, new tbMaskImage(s_level2_arrow_right), [s_level2_arrow_right], function(){
			self.swipeDirection = "next";
			self.animateToWorld(self.currentWorldIdx + 1);
			return true;
		});

		this.updateWorldButtons();
	}else{
		this.levelSurfaceY -= this.settings.gridOffsetYDifferenceNoWorldButtons;
	}
};

TG_LevelMapScreen.prototype.updateWorldButtons = function(){
	"use strict";
	if(TG.gameFlowSettings.noOfLevels.length > 1){
		var image;
		if(!this.hasPrevious){
			image = [new tbSurface(s_level2_arrow_left.width, s_level2_arrow_left.height)];
			image[0].setTarget();
			s_level2_arrow_left.drawSimple(1, 0, 0);
			image[0].resetTarget();
			this.previousBtn.enabled = false;
		}else{
			image = [s_level2_arrow_left];
			this.previousBtn.enabled = true;
		}
		this.previousBtn.setImages(image);

		if(!this.hasNext){
			image = [new tbSurface(s_level2_arrow_right.width, s_level2_arrow_right.height)];
			image[0].setTarget();
			s_level2_arrow_right.drawSimple(1, 0, 0);
			image[0].resetTarget();
			this.nextBtn.enabled = false;
		}else{
			image = [s_level2_arrow_right];
			this.nextBtn.enabled = true;
		}
		this.nextBtn.setImages(image);
	}
};

TG_LevelMapScreen.prototype.drawTitle = function(){
	"use strict";
	var font, text, maxFontSize;

	this.titleSurface.setTarget();

	tbGraphics.clear();
	font = f_default.clone();
	if(this.settings.titleFont){
		font.setProperties(this.settings.titleFont);
	}
	font.setAlign("center");
	font.setBaseline("middle");
	text = TG.dictionary.getText("levelMapScreenWorld_" + this.currentWorldIdx, "<LEVELMAPSCREENWORLD_" + this.currentWorldIdx + ">");
	maxFontSize = font.textFitSize(text, this.settings.titleMaxWidth - (font.getStroke()?font.getStrokeWidth():0), this.settings.titleMaxHeight - (font.getStroke()?font.getStrokeWidth():0), false);
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
	font.drawSimple(text, this.titleSurface.width / 2, this.titleSurface.height / 2);
	this.titleSurface.resetTarget();
	this.canvas.drawTokens = true;
};

TG_LevelMapScreen.prototype.drawStats = function(){
	"use strict";
	var font, text, x, y, maxFontSize, gameShowStars, gameShowScore, starsOverviewX, starsOverviewY;

	gameShowStars = TG.settings.tg.levelEnd.type[TG.gameFlowSettings.gameType].showStars;
	if(TG.settings.game.levelEnd && TG.settings.game.levelEnd.type && TG.settings.game.levelEnd.type[TG.gameFlowSettings.gameType]
	        && TG.settings.game.levelEnd.type[TG.gameFlowSettings.gameType]){
		gameShowStars = (TG.settings.game.levelEnd.type[TG.gameFlowSettings.gameType].showStars === false)? false : gameShowStars;
	}
	if(this.gameSettings !== undefined && this.gameSettings.showStars !== undefined){
		gameShowStars = this.gameSettings.showStars;
	}

	starsOverviewX = TG.functions.computeImgCoordinatesFromSettings(this.settings.starsOverviewX, this.canvas.width, s_level_stars.width);
	starsOverviewY = TG.functions.computeImgCoordinatesFromSettings(this.settings.starsOverviewY, TG.visibleArea.height, s_level_stars.height) + TG.visibleArea.top;
	if(gameShowStars){
		//draw stars
		s_level_stars.drawSimple(0, starsOverviewX, starsOverviewY);
		font = f_default.clone();
		font.setProperties(this.settings.starsOverviewFont);
		font.setAlign("center");
		font.drawSimple(this.stars + " / " + this.totStars, starsOverviewX + Math.floor(s_level_stars.width/2), starsOverviewY + s_level_stars.height + this.settings.starsTextOffsetY);
	}

	//draw total score
	gameShowScore = (this.gameSettings !== undefined && this.gameSettings.showScore !== undefined)? this.gameSettings.showScore : true;
	if(gameShowScore){
		font = f_default.clone();
		if(this.settings.pointOverviewHeaderFont !== undefined){
			font.setProperties(this.settings.pointOverviewHeaderFont);
		}else{
			font.setProperties(this.settings.pointOverviewFont);
		}
		text = TG.dictionary.getText("levelMapScreenTotalScore", "<TOTAL SCORE:>");
		maxFontSize = font.textFitSize(text, this.settings.pointOverviewMaxWidth, this.settings.pointOverviewMaxHeight);
		if(maxFontSize < font.getFontSize()){
			font.setFontSize(maxFontSize);
		}
		x = TG.functions.computeTextCoordinatesFromSettings(this.settings.pointOverviewX, this.canvas.width, font.textWidth(text), font.getAlign());
		y = TG.functions.computeTextCoordinatesFromSettings(this.settings.pointOverviewY, TG.visibleArea.height, font.textHeight(text), font.getBaseline()) + TG.visibleArea.top;
		font.drawSimple(text, x, y);
		text = "" + this.points;
		font.setProperties(this.settings.pointOverviewFont);
		x = TG.functions.computeTextCoordinatesFromSettings(this.settings.pointOverviewX, this.canvas.width, font.textWidth(text), font.getAlign());
		font.drawSimple(text, x, starsOverviewY + s_level_stars.height + this.settings.starsTextOffsetY);
	}
};

// handleAdd is called when the token is (re-) added to the game.
TG_LevelMapScreen.prototype.handleAdd = function(){
	"use strict";
	var btnType, btnX, btnY, setFonts, i, w, toManyLevels, stats, self = this;

	//determine whether to use big or small level images
	this.useBigImages = this.settings.useBigImages? true : false;
	if(!this.useBigImages){
		for(i = 0; i < TG.gameFlowSettings.noOfLevels.length; i++){
			if(TG.gameFlowSettings.noOfLevels[i] > 9){
				toManyLevels = true;
				break;
			}
		}
		if(!toManyLevels){
			this.useBigImages = true;
		}
	}

	//create surfaces and masks
	this.levelSurface = new tbSurface(this.canvas.width * 3, (this.useBigImages)? this.settings.levelsMaxHeightBig : this.settings.levelsMaxHeightSmall);
	this.levelSurfaceX = -this.canvas.width;
	this.levelSurfaceYSettings = ((this.useBigImages)? this.settings.gridOffsetYBig : this.settings.gridOffsetYSmall);
	this.levelSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.levelSurfaceYSettings, TG.visibleArea.height, this.levelSurface.height) + TG.visibleArea.top;

	this.titleSurface = new tbSurface(this.settings.titleMaxWidth, this.settings.titleMaxHeight);
	this.titleSurfaceX = TG.functions.computeImgCoordinatesFromSettings(this.settings.titleX, this.canvas.width, this.settings.titleMaxWidth);
	this.titleSurfaceClippingX = TG.functions.computeImgCoordinatesFromSettings(this.settings.titleX, this.canvas.width, this.settings.titleMaxWidth);
	this.titleSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.settings.titleY, TG.visibleArea.height, this.titleSurface.height) + TG.visibleArea.top;
	this.levelMask = (typeof s_level_mask !== "undefined")? s_level_mask : (this.useBigImages? tbMaskImage(s_level_0) : tbMaskImage(s_level2_0));

	//setup fonts
	if(this.settings.drawNumbers){
		this.fonts = {};
		setFonts = function(setting){
			var f, font;
			for(f in setting){
				font = f_default.clone();
				font.setProperties(setting[f]);
				self.fonts[f] = font;
			}
		};
		this.fonts = {};
		this.fonts.general = f_default;
		if(this.useBigImages){
			setFonts(this.settings.levelFontsBig);
		}else{
			setFonts(this.settings.levelFontsSmall);
		}
	}

	//initialise variables
	this.currentWorldIdx = TG.controller.getCurrentWorldIdx();
	this.noOfLevels = TG.gameFlowSettings.noOfLevels[this.currentWorldIdx];
	this.swiping = false;
	this.swipeTimer = 0;
	this.swipeDistance = 0;
	this.swipeSpeed = 0;
	this.swipeXStart = this.levelSurfaceX;
	this.swipeOffsetX = 0;
	this.hasPrevious = (this.currentWorldIdx > 0);
	this.hasNext = (this.currentWorldIdx < TG.gameFlowSettings.noOfLevels.length - 1);

	//request stats
	this.stars = 0;
	this.points = 0;
	this.totStars = 0;
	for(w = 0; w < TG.gameFlowSettings.noOfLevels.length; w++){
		for(i = 0; i < TG.gameFlowSettings.noOfLevels[w]; i++){
			stats = TG.controller.getLevelStatistics(undefined, i, w);
			this.totStars += 3;
			if(typeof stats === "object" && stats !== null){
				this.stars += (stats["stars"] !== undefined)? stats["stars"] : 0;
				this.points += (stats["highScore"] !== undefined)? stats["highScore"] : 0;
			}
		}
	}

	if (TG.game.getTotalScoreText) {
		this.points = TG.game.getTotalScoreText();
	}

	//draw background
	this.drawBackground();

	//create options buttons
	btnType = this.buttonSettings[this.settings.optionsBtnType];
	btnX = TG.functions.computeImgCoordinatesFromSettings(this.settings.optionsBtnX, this.canvas.width, btnType.sprite.width);
	btnY = TG.functions.computeImgCoordinatesFromSettings(this.settings.optionsBtnY, TG.visibleArea.height, btnType.sprite.height) + TG.visibleArea.top;
	this.optionsBtn = new TG_ButtonBase(btnX, btnY, this.depth - 20, new tbMaskImage(btnType.sprite), [btnType.sprite], {
	    callbackFunc: TG.controller.showOptions,
	    callbackObj: this
	});

	//add world buttons
	this.addWorldButtons();

	//add levels
	this.drawLevels();

	//add title
	this.drawTitle();

	this.forceDraw = true;
};

// handleRemove is called when the token is removed from the game.
TG_LevelMapScreen.prototype.handleRemove = function(){
	"use strict";
	if(this.previousBtn){
		tbGame.removeToken(this.previousBtn);
	}
	if(this.nextBtn){
		tbGame.removeToken(this.nextBtn);
	}
	tbGame.removeToken(this.optionsBtn);
};

// handleTouchDown is called when a touch is started or if a mouse button is
// pressed.
TG_LevelMapScreen.prototype.handleTouchDown = function(finger, x, y){
	"use strict";
	var i;
	if(!this.moving){
		for(i = 0; i < this.levelPositions.length; i++){
			if(this.levelMask.collisionPoint(this.levelPositions[i].x - this.canvas.width, this.levelPositions[i].y + this.levelSurfaceY, x, y)){
				this.levelPressed = i;
				break;
			}
		}
	}
	this.moving = false;
	// only check for swiping if there are worlds to swipe to
	if(TG.gameFlowSettings.noOfLevels.length > 1){
		this.swiping = true;
		this.swipeTimer = 0;
		this.swipeXStart = x;
		this.swipePreviousX = x;
		this.swipeDistance = 0;
		this.swipeSpeed = 0;
	}

};

// handleTouchUp is called when a touch is ended or if a mouse button is
// released.
TG_LevelMapScreen.prototype.handleTouchUp = function(finger, x, y){
	"use strict";
	// handle level buttons
	if(!this.moving
	        && this.levelPressed !== -1
	        && this.levelMask.collisionPoint(this.levelPositions[this.levelPressed].x - this.canvas.width, this.levelPositions[this.levelPressed].y
	                + this.levelSurfaceY, x, y)){
		var state = TG.gameFlowSettings.defaultLevelState || "locked", levelData = TG.controller.getLevelData(this.currentWorldIdx, this.levelPressed);
		if(typeof levelData === "object" && levelData !== null && levelData["state"] !== undefined){
			state = levelData["state"];
		}
		if(state !== "locked"){
			tbGame.removeToken(this);
			TG.controller._initGameLevel(this.levelPressed, this.currentWorldIdx);
			return true;
		}
	}
	this.levelPressed = -1;

	// handle swiping
	this.swiping = false;
	// only check for swiping if there are worlds to swipe to
	if(TG.gameFlowSettings.noOfLevels.length > 1){
		if(Math.abs(this.swipeOffsetX) >= this.settings.swipeMinDistance
		        && (this.swipeSpeed >= this.settings.swipeSpeedThreshold || Math.abs(this.swipeOffsetX) >= this.settings.swipeDistanceThreshold)){
			if(this.swipeDirection === "previous"){
				if(this.hasPrevious && this.swipeOffsetX >= 0 && this.swipeOffsetX <= this.canvas.width / 2){
					this.animateToWorld(this.currentWorldIdx - 1);
				}else if(this.swipeOffsetX < 0){
					this.animateToWorld(this.currentWorldIdx);
				}else{
					this.swipeDirection = "next";
					this.animateToWorld(this.currentWorldIdx);
				}
			}else if(this.swipeDirection === "next"){
				if(this.hasNext && this.swipeOffsetX <= 0 && this.swipeOffsetX >= -this.canvas.width / 2){
					this.animateToWorld(this.currentWorldIdx + 1);
				}else if(this.swipeOffsetX > 0){
					this.animateToWorld(this.currentWorldIdx);
				}else{
					this.swipeDirection = "previous";
					this.animateToWorld(this.currentWorldIdx);
				}
			}
		}else if(Math.abs(this.swipeOffsetX) > 0){
			this.swipeDirection = this.swipeDirection === "next"? "previous" : "next";
			this.animateToWorld(this.currentWorldIdx);
		}
	}
	return true;
};

TG_LevelMapScreen.prototype.handleSpecial = function(id){
	"use strict";
	if(id === TG.languageSetEventId || id === TG.resizeEventId){
		this.canvas.drawTokens = true;
		this.drawBackground();
		if(id === TG.resizeEventId){ //resize Event
			this.titleSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.settings.titleY, TG.visibleArea.height, this.titleSurface.height)
			        + TG.visibleArea.top;
			this.levelSurfaceY = TG.functions.computeImgCoordinatesFromSettings(this.levelSurfaceYSettings, TG.visibleArea.height, this.levelSurface.height)
			        + TG.visibleArea.top;
			this.optionsBtn.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.optionsBtnY, TG.visibleArea.height,
			        this.optionsBtn.images[0].height)
			        + TG.visibleArea.top;
			if(this.previousBtn){
				this.previousBtn.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonPreviousY, TG.visibleArea.height,
				        s_level2_arrow_left.height)
				        + TG.visibleArea.top;
			}
			if(this.nextBtn){
				this.nextBtn.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.worldButtonNextY, TG.visibleArea.height,
				        s_level2_arrow_right.height)
				        + TG.visibleArea.top;
			}
			if(this.nextBtn === undefined && this.previousBtn === undefined){
				this.levelSurfaceY -= this.settings.gridOffsetYDifferenceNoWorldButtons;
			}
		}else{ //languageSet Event
			this.drawTitle();
			this.drawLevels();
		}
	}
};

TG_LevelMapScreen.prototype.handleEndStep = function(timePassed){
	"use strict";
	var x = tbGame.touchX(0);

	if(this.swiping){
		this.swipeDistance = Math.abs(this.swipeXStart - x);
		if(this.swipeTimer > 0){
			this.swipeSpeed = this.swipeDistance / (this.swipeTimer / 1000);
		}
		this.swipeDirection = (x > this.swipeXStart)? "previous" : "next";
		this.swipeTimer += timePassed;

		this.swipeOffsetX += x - this.swipePreviousX;
		this.swipePreviousX = x;
		this.canvas.drawTokens = true;
	}
	if(this.moving){
		this.swipeOffsetX = tbEase.easeOutQuadratic(this.moveTimer, this.moveStart, this.moveChange, this.moveDuration);
		if(this.moveTimer >= this.moveDuration){
			this.moving = false;
			this.swipeOffsetX = 0;
		}
		this.moveTimer += timePassed;
		this.canvas.drawTokens = true;
	}

	if(this.moving || this.swiping){
		if(this.swipeDirection === "previous" && this.swipeOffsetX >= this.canvas.width / 2){
			if(this.currentWorldIdx - 1 >= 0){
				this.currentWorldIdx -= 1;
				this.noOfLevels = TG.gameFlowSettings.noOfLevels[this.currentWorldIdx];
				this.hasPrevious = (this.currentWorldIdx > 0);
				this.hasNext = (this.currentWorldIdx < TG.gameFlowSettings.noOfLevels.length - 1);
				this.updateWorldButtons();
				this.swipeOffsetX -= this.canvas.width;
				this.drawTitle();
				this.drawLevels();
				this.canvas.drawTokens = true;
				this.moveStart -= this.canvas.width;
			}else{
				this.swipeOffsetX = Math.round(this.canvas.width / 2);
			}
		}else if(this.swipeDirection === "next" && this.swipeOffsetX <= -this.canvas.width / 2){
			if(this.currentWorldIdx + 1 < TG.gameFlowSettings.noOfLevels.length){
				this.currentWorldIdx += 1;
				this.noOfLevels = TG.gameFlowSettings.noOfLevels[this.currentWorldIdx];
				this.hasPrevious = (this.currentWorldIdx > 0);
				this.hasNext = (this.currentWorldIdx < TG.gameFlowSettings.noOfLevels.length - 1);
				this.updateWorldButtons();
				this.swipeOffsetX += this.canvas.width;
				this.drawTitle();
				this.drawLevels();
				this.canvas.drawTokens = true;
				this.moveStart += this.canvas.width;
			}else{
				this.swipeOffsetX = Math.round(-this.canvas.width / 2);
			}
		}
	}

};

TG_LevelMapScreen.prototype.handleDraw = function(){
	"use strict";
	this.titleSurface.drawSimple(this.titleSurfaceX, this.titleSurfaceY);
	this.levelSurface.drawSimple(Math.round(this.levelSurfaceX + this.swipeOffsetX), this.levelSurfaceY);
	this.forceDraw = false;
};

//******************************************************
//*****    Token Resource: TG_LevelEndDialog
//******************************************************

// ***************************************
// Definition of game token TG_LevelEndDialog
// ***************************************

/* global
	TG_MainController
 */

// The constructor for the TG_LevelEndDialog game token.
var TG_LevelEndDialog = function(type, stats, callbackFunc, callbackObj) {
	"use strict";
	// Properties used by the engine
	this.depth = 10;
	this.visible = true;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
    var setting;
    this.type = stats["failed"]? "failed" : type;
    this.settings = TG.settings.tg.levelEnd;
    this.gameSettings = this.settings.type[this.type];
    if(TG.orientation === "landscape"){
        for(setting in TG.settings.tg.levelEnd_landscape){
            this.settings[setting] = TG.settings.tg.levelEnd_landscape[setting];
        }
    }
    //apply theme and game specific setting overwrites
    for(setting in TG.settings.theme.levelEnd){
        this.settings[setting] = TG.settings.theme.levelEnd[setting];
    }
    if(TG.settings.theme.levelEnd && TG.settings.theme.levelEnd.type && TG.settings.theme.levelEnd.type[this.type]){
        for(setting in TG.settings.theme.levelEnd.type[this.type]){
            this.settings[setting] = TG.settings.theme.levelEnd.type[this.type][setting];
        }
    }
    if(this.type === "failed"){
        if(TG.settings.game.levelEnd !== undefined && TG.settings.game.levelEnd.type && TG.settings.game.levelEnd.type["failed"] !== undefined){
            for(setting in TG.settings.game.levelEnd.type[this.type]){
                this.gameSettings[setting] = TG.settings.game.levelEnd.type[this.type][setting];
            }
        }
    }else{
        if(TG.settings.game.levelEnd !== undefined && TG.settings.game.levelEnd.type !== undefined){
            for(setting in TG.settings.game.levelEnd.type[this.type]){
                this.gameSettings[setting] = TG.settings.game.levelEnd.type[this.type][setting];
            }
        }
        for(setting in TG.settings.game.levelEnd){
            this.gameSettings[setting] = TG.settings.game.levelEnd[setting];
        }
    }

    this.stats = stats;
    this.callbackFunc = callbackFunc;
    this.callbackObj = callbackObj;

    this.showStarEffect = true;
    this.showCounterEffects = true;

    this.starAudio = [a_levelend_star_01, a_levelend_star_02, a_levelend_star_03];
    this.bonusCounters = [];
    this.timeLine = new tbTimeLine();
    this.timeLine.parent = this;
	// Add it to the game engine
	tbGame.addToken(this, false);
};

TG_LevelEndDialog.prototype.handleBtnClick = function() {
    "use strict";
    var i;
    //force end star animations
    for(i=0; i<this.stars.length; i++){
        this.stars[i].removeAnimator("all", true);
    }
    for(i=0; i<this.starEffects.length; i++){
        tbGame.removeToken(this.starEffects[i]);
    }
    this.starEffects = [];

    //force end medal
    if(this.medal){
        this.medal.removeAnimator("all", true);
    }
    this.medal = undefined;

    //disable buttons
    for(i=0; i<this.buttons.length; i++){
        this.buttons[i].enabled = false;
    }

    //stop time line
    this.timeLine.stop();
    this.timeLine = undefined;

    //start animation out
    this.startOverlayOutAnimation();

    return true;
};

TG_LevelEndDialog.prototype.addTitleItem = function(type){
    "use strict";
    var text;
    switch(type){
        case "title_level": text = TG.dictionary.getText("levelEndScreenTitle_level", "<LEVELENDSCREENTITLE_LEVEL>").replace("<VALUE>", this.stats["level"]); break;
        case "title_endless": text = TG.dictionary.getText("levelEndScreenTitle_endless", "<LEVELENDSCREENTITLE_ENDLESS>").replace("<VALUE>", this.stats["stage"]); break;
        case "title_difficulty": text = TG.dictionary.getText("levelEndScreenTitle_difficulty", "<LEVELENDSCREENTITLE_DIFFICULTY>"); break;
    }
    if(text !== undefined){
        this.drawText(this.settings.titleFont, text,this.settings.titleX, this.settings.titleY, this.settings.titleMaxWidth, this.settings.titleMaxHeight);
    }
};

TG_LevelEndDialog.prototype.addSubTitleItem = function(type){
    "use strict";
    var text;
    switch(type){
        case "subtitle_failed": text = TG.dictionary.getText("levelEndScreenSubTitle_levelFailed", "<LEVEL_FAILED>"); break;
    }
    if(text !== undefined){
        this.drawText(this.settings.subTitleFont, text, this.settings.subTitleX, this.settings.subTitleY);
    }
};

TG_LevelEndDialog.prototype.addStatsObjectToOverlay = function(obj, inLine){
	"use strict";
	var text, header, x, y, fontHeader, fontText, h, w, fontHeaderSettings, fontTextSettings, counter, counterX, counterY, counterHeight;
	header = TG.dictionary.getText(obj.key, "<" + obj.key.toUpperCase() + ">");
	text = obj.animate? obj.toString(obj.startValue) : obj.toString(obj.endValue);

	fontHeaderSettings = this.settings.statsHeaderFont;
	fontHeaderSettings.align = "left";
	fontHeaderSettings.baseline = "top";
	fontHeader = f_default.clone();
	fontHeader.setProperties(fontHeaderSettings);
	if(!inLine){
		// draw header
		if(header !== undefined){
			this.drawText(fontHeaderSettings, header, this.settings.statsHeaderX, this.settings.statsHeaderY);
			y = this.settings.statsHeaderY;
			if(typeof y === "object"){
				y.offset = (y.offset !== undefined)? y.offset + this.settings.statsHeaderMarginBottom : this.settings.statsHeaderMarginBottom;
				y.offset += fontHeader.textHeight(header);
			}else if(typeof y === "number"){
				y += this.settings.statsHeaderMarginBottom + fontHeader.textHeight(header);
			}
		}
		if(text !== undefined){
			fontTextSettings = this.settings.statsFont;
			fontTextSettings.baseline = "top";
			if(obj.animate){
				fontText = f_default.clone();
				fontTextSettings.align = "center";
				fontText.setProperties(fontTextSettings);
				counterX = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderX, this.canvas.width, this.settings.statsMaxWidth)
				        - this.overlay.x;
				counterY = y - this.overlay.y;
				obj.counter = new TG_Counter(counterX, counterY, this.settings.statsMaxWidth, fontText.textHeight(text), this.depth - 100, obj.startValue,
				        fontText, this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay, obj.toString);
				//obj.counter.setBackgroundImage(this.image, counterX, counterY);
			}else{
				this.drawText(fontTextSettings, text, this.settings.statsHeaderX, y);
			}
		}
	}else{
		fontHeader.setBaseline("bottom");
		fontTextSettings = this.settings.statsFont;
		fontTextSettings.align = "left";
		fontTextSettings.baseline = "bottom";
		fontText = f_default.clone();
		fontText.setProperties(fontTextSettings);
		h = 0;
		w = 0;

		// compute width of header and text together to compute x position
		// header
		if(header !== undefined){
			w += fontHeader.textWidth(header) + this.settings.statsHeaderMarginRight;
		}
		if(text !== undefined){
			w += fontText.textWidth(text);
		}
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderX, this.canvas.width, w) - this.overlay.x;

		// draw header and update x coordinate for x
		if(header !== undefined){
			fontHeader.drawSimple(header, x, this.statsY + fontHeader.getFontSize());
			if (testMode) {
				TG_MainController.drawTextRect(fontHeader, x, this.statsY + fontHeader.getFontSize());
			}
			x += fontHeader.textWidth(header) + this.settings.statsHeaderMarginRight;
			h += fontHeader.textHeight(header);
		}

		// draw text
		if(text !== undefined){
			if(obj.animate){
				counterHeight = fontText.textHeight(text);
				counterY = this.statsY + fontHeader.getFontSize() - counterHeight;
				obj.counter = new TG_Counter(x, counterY, this.settings.statsMaxWidth, counterHeight, this.depth - 100, obj.startValue, fontText,
				        this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay, obj.toString);
				//obj.counter.setBackgroundImage(this.image, x, counterY);
				h = Math.max(h, counterHeight);
			}else{
				fontText.drawSimple(text, x, this.statsY + fontHeader.getFontSize() + this.settings.statsInlineOffset);
				if (testMode) {
					TG_MainController.drawTextRect(fontText, x, this.statsY + fontHeader.getFontSize() + this.settings.statsInlineOffset);
				}
				h = Math.max(h, fontText.textHeight(text));
			}
		}

		// update global y variable
		if(h > 0){
			this.statsY += h + this.settings.statsHeaderMarginBottom;
		}
	}
};

TG_LevelEndDialog.prototype.addStatsItem = function(type, inLine){
    "use strict";
    var text, header, x, y, fontHeader, fontText, h, w, fontHeaderSettings, fontTextSettings, counter, counterX, counterY, counterHeight;

    switch(type){
        case "totalScore":
            text = ""+ this.stats["totalScore"];
            header = TG.dictionary.getText("levelEndScreenTotalScore", "<LEVENENDSCREENTOTALSCORE>");
            counter = 0;
        break;
        case "highScore":
            header = TG.dictionary.getText("levelEndScreenHighScore", "<LEVENENDSCREENHIGHSCORE>");
            text = "" + this.stats["highScore"];
        break;
        case "timeLeft":
            header = TG.dictionary.getText("levelEndScreenTimeLeft", "<LEVENENDSCREENTIMELEFT>");
            text = "" + this.stats["timeLeft"];
        break;
        case "timeBonus":
            header = TG.dictionary.getText("levelEndScreenTimeBonus", "<LEVENENDSCREENTIMEBONUS>");
            text = "" + this.stats["timeBonus"];
            counter = this.stats["timeBonus"];
        break;
    }

    fontHeaderSettings = this.settings.statsHeaderFont;
    fontHeaderSettings.align = "left";
    fontHeaderSettings.baseline = "top";
    fontHeader = f_default.clone();
    fontHeader.setProperties(fontHeaderSettings);
    if(!inLine){
        if(header !== undefined){
            this.drawText(fontHeaderSettings, header, this.settings.statsHeaderX, this.settings.statsHeaderY);
            y = this.settings.statsHeaderY;
            if(typeof y === "object") {
                y.offset = (y.offset !== undefined) ? y.offset + this.settings.statsHeaderMarginBottom : this.settings.statsHeaderMarginBottom;
                y.offset += fontHeader.textHeight(header);
            }else if (typeof y === "number") {
                y += this.settings.statsHeaderMarginBottom + fontHeader.textHeight(header);
            }
        }
        if(text !== undefined){
            fontTextSettings = this.settings.statsFont;
            fontTextSettings.baseline = "top";
            if(counter !== undefined){
                fontText = f_default.clone();
                fontTextSettings.align = "center";
                fontText.setProperties(fontTextSettings);
                counterX = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderX, this.canvas.width, this.settings.statsMaxWidth) - this.overlay.x;
                counterY = y - this.overlay.y;
                counter = new TG_Counter(counterX, counterY, this.settings.statsMaxWidth, fontText.textHeight(text), this.depth - 100, counter, fontText, this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay);
                //counter.setBackgroundImage(this.image, counterX, counterY);
            }else{
                this.drawText(fontTextSettings, text, this.settings.statsHeaderX, y);
            }
        }
    }else{
        fontHeader.setBaseline("bottom");
        fontTextSettings = this.settings.statsFont;
        fontTextSettings.align = "left";
        fontTextSettings.baseline = "bottom";
        fontText = f_default.clone();
        fontText.setProperties(fontTextSettings);
        h = 0;
        w = 0;

        //compute width of header and text together to compute x position header
        if(header !== undefined){
            w += fontHeader.textWidth(header) + this.settings.statsHeaderMarginRight;
        }
        if(text !== undefined){
            w += fontText.textWidth(text);
        }
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderX, this.canvas.width, w) - this.overlay.x;

        //draw header and update x coordinate for x
        if(header !== undefined){
            fontHeader.drawSimple(header, x, this.statsY + fontHeader.getFontSize());
            if (testMode) {
				TG_MainController.drawTextRect(fontHeader, x, this.statsY + fontHeader.getFontSize());
			}
            x += fontHeader.textWidth(header) + this.settings.statsHeaderMarginRight;
            h += fontHeader.textHeight(header);
        }

        //draw text
        if(text !== undefined){
            if(counter !== undefined){
                counterHeight = fontText.textHeight(text);
                counterY = this.statsY + fontHeader.getFontSize() - counterHeight;
                counter = new TG_Counter(x, counterY, this.settings.statsMaxWidth, counterHeight, this.depth - 100, counter, fontText, this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay);
                //counter.setBackgroundImage(this.image, x, counterY);
                h = Math.max(h, counterHeight);
            }else{
                fontText.drawSimple(text, x, this.statsY + fontHeader.getFontSize() + this.settings.statsInlineOffset);
                if (testMode) {
    				TG_MainController.drawTextRect(fontText, x, this.statsY + fontHeader.getFontSize() + this.settings.statsInlineOffset);
    			}
                h = Math.max(h, fontText.textHeight(text));
            }
        }

        //update global y variable
        if(h > 0){
            this.statsY += h + this.settings.statsHeaderMarginBottom;
        }
    }

    if(counter instanceof TG_Counter){
        if(type === "totalScore"){
            this.totalScoreCounter = counter;
        }else{
            this.bonusCounters.push(counter);
        }
    }
};

TG_LevelEndDialog.prototype.addEmphasizedStatsObjectToOverlay = function(obj){
    "use strict";
    var header, text, fontText, counter, counterX, counterY, fontTextSettings;
    header = TG.dictionary.getText(obj.key, "<" + obj.key.toUpperCase() + ">");
	text = obj.animate? obj.toString(obj.startValue) : obj.toString(obj.endValue);
    if(header !== undefined){
        this.drawText(this.settings.emphasizedStatsHeaderFont, header, this.settings.emphasizedStatsHeaderX, this.settings.emphasizedStatsHeaderY);
    }
    if(text !== undefined){
        if(obj.animate){
            fontText = f_default.clone();
            fontTextSettings = this.settings.emphasizedStatsFont;
            if (!this.settings.emphasizedStatsFontDontOverrideTextAlign) {
            	//By default the text font align setting is overridden with center,
            	//   but we are afraid to disable this therefore you can turn this
            	//   behavior off by setting emphasizedStatsFontDontOverrideTextAlign
            	//   to true
            	fontTextSettings.align = "center";
            }
            fontText.setProperties(fontTextSettings);
            counterX = TG.functions.computeImgCoordinatesFromSettings(this.settings.emphasizedStatsX, this.canvas.width, this.settings.emphasizedStatsMaxWidth) - this.overlay.x;
            counterY = TG.functions.computeImgCoordinatesFromSettings(this.settings.emphasizedStatsY, this.canvas.height, this.settings.emphasizedStatsMaxHeight) - this.overlay.y;
            obj.counter = new TG_Counter(counterX, counterY, this.settings.emphasizedStatsMaxWidth, this.settings.emphasizedStatsMaxHeight, this.depth - 100, obj.startValue, fontText, this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay, obj.toString);
            //obj.counter.setBackgroundImage(this.image, counterX, counterY);
        }else{
            this.drawText(this.settings.emphasizedStatsFont, text, this.settings.emphasizedStatsX, this.settings.emphasizedStatsY);
        }
    }
};

TG_LevelEndDialog.prototype.addEmphasizedStatsItem = function(type){
    "use strict";
    var header, text, fontText, counter, counterX, counterY, fontTextSettings;
    switch(type){
        case "totalScore":
            header = TG.dictionary.getText("levelEndScreenTotalScore", "<LEVENENDSCREENTOTALSCORE>");
            text = "" + this.stats["totalScore"];
            counter = 0;
        break;
        case "timeLeft":
            header = TG.dictionary.getText("levelEndScreenTimeLeft", "<LEVENENDSCREENTIMELEFT>");
            text = "" + this.stats["timeLeft"];
        break;
    }
    if(header !== undefined){
        this.drawText(this.settings.emphasizedStatsHeaderFont, header, this.settings.emphasizedStatsHeaderX, this.settings.emphasizedStatsHeaderY);
    }
    if(text !== undefined){
        if(counter !== undefined){
            fontText = f_default.clone();
            fontTextSettings = this.settings.emphasizedStatsFont;
            fontTextSettings.align = "center";
            fontText.setProperties(fontTextSettings);
            counterX = TG.functions.computeImgCoordinatesFromSettings(this.settings.emphasizedStatsX, this.canvas.width, this.settings.emphasizedStatsMaxWidth) - this.overlay.x;
            counterY = TG.functions.computeImgCoordinatesFromSettings(this.settings.emphasizedStatsY, this.canvas.height, this.settings.emphasizedStatsMaxHeight) - this.overlay.y;
            counter = new TG_Counter(counterX, counterY, this.settings.emphasizedStatsMaxWidth, this.settings.emphasizedStatsMaxHeight, this.depth - 100, counter, fontText, this.settings.statsCounterMarginX, this.settings.statsCounterMarginY, this.overlay);
            //counter.setBackgroundImage(this.image, counterX, counterY);
        }else{
            this.drawText(this.settings.emphasizedStatsFont, text, this.settings.emphasizedStatsX, this.settings.emphasizedStatsY);
        }
    }

    if(counter instanceof TG_Counter){
        if(type === "totalScore"){
            this.totalScoreCounter = counter;
        }else{
            this.bonusCounters.push(counter);
        }
    }
};

TG_LevelEndDialog.prototype.drawText = function(fontProperties, text, xSetting, ySetting, maxWidth, maxHeight){
    "use strict";
    var width, height, x, y, maxFontSize,
        font = f_default.clone();
    font.setProperties(fontProperties);
    if(maxWidth !== undefined && maxHeight !== undefined){
        maxFontSize = font.textFitSize(text, maxWidth, maxHeight, maxWidth);
        if(font.getFontSize() > maxFontSize){
            font.setFontSize(maxFontSize);
        }
    }
    width = font.textWidth(text);
    height = font.textHeight(text);
    x = TG.functions.computeTextCoordinatesFromSettings(xSetting, this.canvas.width, width, font.getAlign()) - this.overlay.x;
    y = TG.functions.computeTextCoordinatesFromSettings(ySetting, this.canvas.height, height, font.getBaseline()) - this.overlay.y;
    font.drawSimple(text, x, y, maxWidth);

    if (testMode) {
    	TG_MainController.drawTextRect(font, x, y, maxWidth, maxHeight);
    }
};

TG_LevelEndDialog.prototype.addBackButton = function(type){
    "use strict";
    var backButtonImage, backButtonCallback, x, y, self = this;
    switch(type) {
        case "retry":
            backButtonImage = s_btn_small_retry;
            backButtonCallback = function () {
                self.callbackParam = "retry";
                self.handleBtnClick();
            };
            break;
        case "exit":
            backButtonImage = s_btn_small_exit;
            backButtonCallback = function () {
                self.callbackParam = "exit";
                self.handleBtnClick();
            };
        break;
    }
    if (backButtonImage !== undefined) {
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.backButtonX, this.canvas.width, backButtonImage.width) - this.overlay.x;
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.backButtonY, this.canvas.height, backButtonImage.height) - this.overlay.y;
        this.buttons.push(new TG_ButtonBase(x, y, this.depth - 20, new tbMaskImage(backButtonImage), [backButtonImage], backButtonCallback, this.overlay));
    }
};

TG_LevelEndDialog.prototype.addForwardButton = function(type){
    "use strict";
    var forwardButtonImage, forwardButtonCallback, x, y, self = this;
    switch(type) {
        case "retry":
            forwardButtonImage = s_btn_big_restart;
            forwardButtonCallback = function () {
                self.callbackParam = "retry";
                self.handleBtnClick();
            };
            break;
        case "exit":
            forwardButtonImage = s_btn_big_start;
            forwardButtonCallback = function () {
                self.callbackParam = "exit";
                self.handleBtnClick();
            };
            break;
        case "next":
            forwardButtonImage = s_btn_big_start;
            forwardButtonCallback = function () {
                self.callbackParam = "next";
                self.handleBtnClick();
            };
        break;
    }
    if (forwardButtonImage !== undefined) {
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.forwardButtonX, this.canvas.width, forwardButtonImage.width) - this.overlay.x;
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.forwardButtonY, this.canvas.height, forwardButtonImage.height) - this.overlay.y;
        this.buttons.push(new TG_ButtonBase(x, y, this.depth - 20, new tbMaskImage(forwardButtonImage), [forwardButtonImage], forwardButtonCallback, this.overlay));
    }
};

// handleAdd is called when the token is (re-) added to the game.
TG_LevelEndDialog.prototype.handleAdd = function() {
	"use strict";
	this.timer = 0;
    this.stars = [];
    this.starEffects = [];
    this.buttons = [];
    this.canvas.drawTokens = true;
    this.callbackParam = "";
    this.failed = (this.stats["failed"])? true: false;
    this.showStars = (this.gameSettings.showStars && !this.failed);
    this.showMedal = (this.gameSettings.showMedal && !this.failed && this.stats.improvedHighScore);
    this.bonusCounterNextIdx = 0;
    this.alpha = 0;
    this.alphaTimer = 0;
    //pause all tokens
    TG.controller._pauseAll(this);

    //define local variables and helper functions
    var moment, updateHighScoreBtn, x, y, i, totalHeight, fontHeader, fontText, length, totalScoreCounterDuration, bonusMoment, bonusCounterCallback, obj,
        self = this;

       switch (this.gameSettings.backgroundImageType) {
           case "failed": this.image = this.settings.overlayImages.levelFail; break;
           case "level": this.image = this.settings.overlayImages.levelWin; break;
           case "difficulty": this.image = this.settings.overlayImages.difficulty; break;
           case "endless": this.image = this.settings.overlayImages.endless; break;
       }


    //create overlay surface
    this.overlay = new TG_AnimatedImage(this.depth - 10, this.layer, new tbSurface(this.image.width, this.image.height));
    this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetX, this.canvas.width, this.image.width);
    this.overlay.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetY, this.canvas.height, this.image.height);
    this.overlay.image.setTarget();

	    // draw overlay background
	this.image.drawSimple(0, 0, 0);
	if(!this.failed && this.showStars){
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.star1X, this.canvas.width, 0) - this.overlay.x;
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.star1Y, this.canvas.height, s_star01_fill.height) - this.overlay.y
		        + Math.round(s_star01_empty.height / 2);
		s_star01_empty.drawSimple(0, x, y);
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.star2X, this.canvas.width, 0) - this.overlay.x;
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.star2Y, this.canvas.height, s_star02_fill.height) - this.overlay.y
		        + Math.round(s_star02_empty.height / 2);
		s_star02_empty.drawSimple(0, x, y);
		x = TG.functions.computeImgCoordinatesFromSettings(this.settings.star3X, this.canvas.width, 0) - this.overlay.x;
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.star3Y, this.canvas.height, s_star03_fill.height) - this.overlay.y
		        + Math.round(s_star03_empty.height / 2);
		s_star03_empty.drawSimple(0, x, y);
	}

    // add items to overlay
    if(this.gameSettings.titleItem !== undefined){
        this.addTitleItem(this.gameSettings.titleItem);
    }
    if(this.gameSettings.subTitleItem !== undefined){
        this.addSubTitleItem(this.gameSettings.subTitleItem);
    }

    // add emphasized stats object/item
	this.statsObjects = {};
	if(this.stats.emphasizedStatsObject !== undefined){
		obj = this.stats.emphasizedStatsObject;
		if(obj.visible){
			this.addEmphasizedStatsObjectToOverlay(obj);
		}
		this.statsObjects[obj.id] = obj;
	}else if(this.gameSettings.emphasizedStatsItem !== undefined){
		this.addEmphasizedStatsItem(this.gameSettings.emphasizedStatsItem);
	}

    // add stats objects/items
    if(this.stats.statsObjects !== undefined){
		length = this.stats.statsObjects.length;
		fontHeader = f_default.clone();
		fontHeader.setProperties(this.settings.statsHeaderFont);
		fontText = f_default.clone();
		fontText.setProperties(this.settings.statsFont);
		totalHeight = Math.max(fontHeader.textHeight("g"), fontText.textHeight("g")) * length + this.settings.statsHeaderMarginBottom * (length - 1);
		this.statsY = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderY, this.canvas.height, totalHeight) - this.overlay.y;
		for(i = 0; i < length; i++){
			obj = this.stats.statsObjects[i];
			if(obj.visible){
				this.addStatsObjectToOverlay(this.stats.statsObjects[i], length > 1);
			}
			this.statsObjects[obj.id] = obj;
		}
	}else if(this.gameSettings.statsItems !== undefined){
		if(typeof this.gameSettings.statsItems === "string"){
			this.addStatsItem(this.gameSettings.statsItems, this.settings.forceStatsInline);
		}else if(this.gameSettings.statsItems instanceof Array){
			length = this.gameSettings.statsItems.length;
			fontHeader = f_default.clone();
			fontHeader.setProperties(this.settings.statsHeaderFont);
			fontText = f_default.clone();
			fontText.setProperties(this.settings.statsFont);
			totalHeight = Math.max(fontHeader.textHeight("g"), fontText.textHeight("g")) * length + this.settings.statsHeaderMarginBottom * (length - 1);
			this.statsY = TG.functions.computeImgCoordinatesFromSettings(this.settings.statsHeaderY, this.canvas.height, totalHeight) - this.overlay.y;
			for(i = 0; i < length; i++){
				this.addStatsItem(this.gameSettings.statsItems[i], length > 1 || this.settings.forceStatsInline);
			}
		}
	}
    this.overlay.image.resetTarget();

    // add forward/back buttons
    this.addBackButton(this.gameSettings.backButtonType);
    this.addForwardButton(this.gameSettings.forwardButtonType);

    //add highscore button
    if(TG.controller.useExternalHighScoreSystem){
        x = TG.functions.computeImgCoordinatesFromSettings(self.settings.highScoreBtnX, self.canvas.width, self.settings.highScoreBtnWidth) - this.overlay.x;
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.highScoreBtnY, this.canvas.height, this.settings.closeButtonMaskHeight) - this.overlay.y;
        this.highScoreBtn = new TG_ButtonText("default_text", x, y, self.depth - 20, "levelEndScreenViewHighscoreBtn", self.settings.highScoreBtnWidth, {
		    callbackFunc: function(){
			    TG_HighScores.submitHighScore(self.stats["totalScore"]);
		    },
		    immediate: true
		}, self.overlay);
        this.buttons.push(this.highScoreBtn);
        updateHighScoreBtn = function(isHighScore){
            if(isHighScore){
                self.highScoreBtn.updateText("levelEndScreenSubmitHighscoreBtn");
                self.isHighScore = isHighScore;
            }
        };
        TG_HighScores.checkHighScore(this.stats["totalScore"], updateHighScoreBtn);
    }

    //add close button
    x = TG.functions.computeImgCoordinatesFromSettings(this.settings.closeButtonX, this.canvas.width, this.settings.closeButtonMaskWidth) - this.overlay.x;
    y = TG.functions.computeImgCoordinatesFromSettings(this.settings.closeButtonY, this.canvas.height, this.settings.closeButtonMaskHeight) - this.overlay.y;

    this.buttons.push(new TG_ButtonBase(x, y, this.depth - 20,
        new tbMaskRectangle(0,0,this.settings.closeButtonMaskWidth, this.settings.closeButtonMaskHeight),
        undefined, function(){self.callbackParam = "exit"; self.handleBtnClick();}, this.overlay));
    for(i=0; i<this.buttons.length; i++){
        this.buttons[i].enabled = false;
    }

    this.overlay.y = -this.overlay.height;

    //setup timeLine
    moment = this.settings.textureSwapDelay;

    //start overlay animation in
    this.timeLine.addMoment(moment, this.startOverlayInAnimation);
    moment += this.settings.animInDuration;
    bonusMoment = 0;

    totalScoreCounterDuration = this.settings.totalScoreCounterDuration;
	if(this.showStars){
		totalScoreCounterDuration = Math.max(totalScoreCounterDuration, this.settings.starFillDuration + this.settings.starFillDelay * this.stats["stars"]);
	}

	if(this.totalScoreCounter){
		this.timeLine.addMoment(moment + this.settings.totalScoreCounterDelay, function(m, tl){
			tl.parent.totalScoreCounter.startCount(tl.parent.stats["totalScore"], totalScoreCounterDuration, tbEase.easeLinear);
		});
		bonusMoment = moment + this.settings.totalScoreCounterDelay + totalScoreCounterDuration;
		if(this.bonusCounters.length > 0){
			bonusCounterCallback = function(m, tl){
				var self = tl.parent, bonusCounter = self.bonusCounters[self.bonusCounterNextIdx];
				self.totalScoreCounter.startCount(self.totalScoreCounter.currentValue() + bonusCounter.currentValue(), self.settings.bonusCounterDuration,
				        tbEase.easeLinear);
				bonusCounter.startCount(0, self.settings.bonusCounterDuration, tbEase.easeLinear);
				self.bonusCounterNextIdx += 1;
			};
			for(i = 0; i < this.bonusCounters.length; i++){
				bonusMoment += this.settings.bonusCounterDelay;
				this.timeLine.addMoment(bonusMoment, bonusCounterCallback);
			}
			bonusMoment += this.settings.bonusCounterDuration;
		}
	}
	if(this.statsObjects !== undefined){
		bonusMoment = moment;
		bonusCounterCallback = function(m, tl){
			var self = tl.parent, statsObj = self.statUpdaters[self.bonusCounterNextIdx || 0], otherStatsObj = self.statsObjects[statsObj.statToUpdate];
			if(statsObj.updateFunction !== undefined){
				if(otherStatsObj.visible && otherStatsObj.animate){
					otherStatsObj.counter.startCount(statsObj.updateFunction(otherStatsObj.counter.currentValue()), self.settings.bonusCounterDuration,
				        tbEase.easeLinear);
				}else{
					otherStatsObj.endValue = statsObj.updateFunction(otherStatsObj.endValue);
				}
			}
			if(statsObj.visible && statsObj.animate){
				statsObj.counter.startCount(statsObj.endValue, self.settings.bonusCounterDuration, tbEase.easeLinear);
			}
			self.bonusCounterNextIdx += 1;
		};
		this.statUpdaters = [];
		if(this.stats.emphasizedStatsObject !== undefined && this.stats.emphasizedStatsObject.updateFunction !== undefined){
			this.timeLine.addMoment(moment + this.settings.totalScoreCounterDelay, bonusCounterCallback);
			this.statUpdaters.push(this.stats.emphasizedStatsObject);
			bonusMoment += this.settings.totalScoreCounterDelay + bonusCounterDuration;
		}
		if(this.stats.statsObjects !== undefined){
			for(i = 0; i < this.stats.statsObjects.length; i++){
				obj = this.stats.statsObjects[i];
				if(obj.updateFunction !== undefined){
					bonusMoment += this.settings.bonusCounterDelay;
					this.timeLine.addMoment(bonusMoment, bonusCounterCallback);
					this.statUpdaters.push(obj);
					bonusMoment += this.settings.bonusCounterDuration;
				}
			}
		}
	}


    // start star animations
    if(this.showStars){
        for(i=0; i<this.stats["stars"]; i++){
            moment += this.settings.starFillDelay;
            this.timeLine.addMoment(moment, this.startStarAnimation);
            this.timeLine.addMoment(moment, this.startStarEffectAnimation);
        }
        moment += this.settings.starFillDuration;
    }

    //start medal animation
    moment = Math.max(moment, bonusMoment);
    if(this.showMedal){
        moment += this.settings.medalDelay;
        this.timeLine.addMoment(moment, this.startMedalShadowAnimation);
        this.timeLine.addMoment(moment, this.startMedalAnimation);
        this.timeLine.addMoment(moment + this.settings.medalEffectDelay, this.startMedalEffectAnimation);
    }

    //inform portals that level end screen is shown
    this.timeLine.addMoment(moment + this.settings.moreGamesOverlayDelay, TG.hooks.informLevelEndScreenShown);

    TG.hooks.informLevelEndScreenStart();

    //start timeLine
    this.timeLine.start();

    //play sound
    if(this.failed){
        tbAudio.play(a_levelend_fail);
    }else{
        tbAudio.play(a_levelComplete);
    }
};

//handleBeginStep is called each step before processing other events.
TG_LevelEndDialog.prototype.handleBeginStep = function(timePassed){
	"use strict";
	this.alpha = this.settings.fadeAlpha * this.alphaTimer/this.settings.fadeDuration;
	this.alphaTimer += timePassed;
	if(this.alpha >= this.settings.fadeAlpha){
		this.alpha = this.settings.fadeAlpha;
		this.active = false;
	}
	this.canvas.drawTokens = true;
};

TG_LevelEndDialog.prototype.startOverlayInAnimation = function(moment, tl){
    "use strict";
    var self = tl.parent, end, direction,
        callback = function(){
            var i;
            for(i=0; i<self.buttons.length; i++){
                self.buttons[i].enabled = true;
            }
        };
    switch(self.settings.animInDirection){
        case "fromLeft":
            direction = "horizontal";
            end = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.overlay.width);
            self.overlay.x = -self.overlay.width;
            self.overlay.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height, self.overlay.height) + Math.abs(TG.canvasTopOffset);
        break;
        case "fromRight":
            direction = "horizontal";
            end = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.overlay.width);
            self.overlay.x = self.canvas.width;
            self.overlay.y = TG.functions.computeImgCoordinatesFromSettings(this.parent.settings.overlayImageOffsetY, self.canvas.height, selft.overlay.height) + Math.abs(TG.canvasTopOffset);
        break;
        case "fromBottom":
            direction = "vertical";
            end = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height, self.overlay.height) + Math.abs(TG.canvasTopOffset);
            self.overlay.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.overlay.width);
            self.overlay.y = self.canvas.height + self.overlay.height;
        break;
        default:
            direction = "vertical";
            end = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height, self.overlay.height) + Math.abs(TG.canvasTopOffset);
            self.overlay.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.overlay.width);
            self.overlay.y = -self.overlay.height;
        break;
    }

    if(direction === "vertical"){
        self.overlay.addAnimator("y", end, self.settings.animInDuration, self.settings.animInEase, callback);
    }else{
        self.overlay.addAnimator("x", end, self.settings.animInDuration, self.settings.animInEase, callback);
    }
};

TG_LevelEndDialog.prototype.startOverlayOutAnimation = function(){
    "use strict";
    var self = this, end, direction,
        callback = function(){
            tbGame.removeToken(self);
            if(self.callbackObj){
                self.callbackFunc.call(self.callbackObj, self.callbackParam);
            }else{
                self.callbackFunc(self.callbackParam);
            }
        };
    switch(self.settings.animOutDirection){
        case "toLeft":
            direction = "horizontal";
            end = -self.overlay.width;
        break;
        case "toRight":
            direction = "horizontal";
            end = self.canvas.width;
        break;
        case "toBottom":
            direction = "vertical";
            end = self.canvas.height + self.overlay.height;
        break;
        default:
            direction = "vertical";
            end = -self.overlay.height;
        break;
    }

    if(direction === "vertical"){
        self.overlay.addAnimator("y", end, self.settings.animOutDuration, self.settings.animOutEase, callback);
    }else{
        self.overlay.addAnimator("x", end, self.settings.animOutDuration, self.settings.animOutEase, callback);
    }
};

TG_LevelEndDialog.prototype.startStarAnimation = function(moment, tl){
    "use strict";
    var star, self = tl.parent, top = Math.abs(TG.canvasTopOffset);
    function addStarToOverlay(starIdx) {
        var x, y, image;
        self.overlay.image.setTarget();
        switch (starIdx) {
            case 1:
                image = s_star01_fill;
                x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star1X, self.canvas.width, 0) - self.overlay.x;// + Math.round(s_star01_empty.width/2);
                y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star1Y, self.canvas.height, s_star01_fill.height) - self.overlay.y + top + Math.round(s_star01_empty.height/2);
                break;
            case 2:
                image = s_star02_fill;
                x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star2X, self.canvas.width, 0) - self.overlay.x;// + Math.round(s_star02_empty.width/2);
                y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star2Y, self.canvas.height, s_star01_fill.height) - self.overlay.y + top + Math.round(s_star02_empty.height/2);
                break;
            case 3:
                image = s_star03_fill;
                x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star3X, self.canvas.width, 0) - self.overlay.x;// + Math.round(s_star03_empty.width/2);
                y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star3Y, self.canvas.height, s_star01_fill.height) - self.overlay.y + top + Math.round(s_star03_empty.height/2);
                break;
        }
        image.drawSimple(0,x,y);
        self.overlay.image.resetTarget();
        self.overlay.forceDraw = true;
        tbGame.removeToken(self.stars[starIdx-1]);
    }
    if(self.stars.length < self.stats["stars"]) {
        switch (self.stars.length + 1) {
            case 1:
                star = new TG_AnimatedImage(self.depth - 30, TG.frameworkEffectsLayer, s_star01_fill);
                star.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star1X, self.canvas.width, 0);
                star.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star1Y, self.canvas.height, s_star01_fill.height) + top + Math.round(s_star01_empty.height/2);
                break;
            case 2:
                star = new TG_AnimatedImage(self.depth - 30, TG.frameworkEffectsLayer, s_star02_fill);
                star.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star2X, self.canvas.width, 0);
                star.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star2Y, self.canvas.height, s_star02_fill.height) + top + Math.round(s_star02_empty.height/2);
                break;
            case 3:
                star = new TG_AnimatedImage(self.depth - 30, TG.frameworkEffectsLayer, s_star03_fill);
                star.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.star3X, self.canvas.width, 0);
                star.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.star3Y, self.canvas.height, s_star03_fill.height) + top + Math.round(s_star03_empty.height/2);
                break;
        }
        star.xScale = self.settings.starFillStartScale;
        star.yScale = self.settings.starFillStartScale;
        star.alpha = self.settings.starFillStartAlpha;
        star.addAnimator("scale", 1, self.settings.starFillScaleDuration, tbEase.easeOutOvershoot, function(){addStarToOverlay(self.stars.length);});
        star.addAnimator("alpha", 1, self.settings.starFillAlphaDuration, tbEase.easeInQuadratic);
        self.stars.push(star);
        tbAudio.play(self.starAudio[self.stars.length - 1]);
    }
};

TG_LevelEndDialog.prototype.startStarEffectAnimation = function(moment, tl){
    "use strict";
    var self = tl.parent, star, starEffect;
    star = self.stars[self.starEffects.length];
    starEffect = new TG_AnimatedImage(self.depth - 50, TG.frameworkEffectsLayer, s_sfx_star);
    starEffect.x = star.x;
    starEffect.y = star.y;
    starEffect.addAnimator("subImage", s_sfx_star.number - 1, self.settings.starEffectDuration, undefined, function(){tbGame.removeToken(starEffect);});
    self.starEffects.push(starEffect);
};

TG_LevelEndDialog.prototype.startMedalAnimation = function(moment, tl) {
    "use strict";
    var self = tl.parent, surfaces, surf, font, maxFontSize, text, x, y, i;

    surfaces = [];

    //determine font properties
    font = f_default.clone();
    text = TG.dictionary.getText("levelEndScreenMedal", "<LEVELENDSCREENMEDAL>");
    if(self.settings.medalTextFont){
        font.setProperties(self.settings.medalTextFont);
    }
    maxFontSize = font.textFitSize(text, self.settings.medalTextMaxWidth, self.settings.medalTextMaxHeight, true);
    if(maxFontSize < font.getFontSize()){
        font.setFontSize(maxFontSize);
    }
    x = TG.functions.computeTextCoordinatesFromSettings(self.settings.medalTextOffsetX, s_medal_shine.width, font.textWidth(text, self.settings.medalTextMaxWidth), font.getAlign());
    y = TG.functions.computeTextCoordinatesFromSettings(self.settings.medalTextOffsetY, s_medal_shine.height, font.textHeight(text, self.settings.medalTextMaxWidth), font.getBaseline());

    //draw surface
    for(i=0; i<s_medal_shine.number; i++){
        surf = new tbSurface(s_medal_shine.width, s_medal_shine.height);
        surf.setTarget();
        s_medal_shine.drawSimple(i, 0, 0);
        font.drawSimple(text, x, y, self.settings.medalTextMaxWidth);
        surf.resetTarget();
        surfaces.push(surf);
    }

    //created animated surface
    self.medal = new TG_AnimatedImage(self.depth - 120, TG.frameworkEffectsLayer, surfaces);
    self.medal.originX = self.settings.medalImageOriginX;
    self.medal.originY = self.settings.medalImageOriginY;
    self.medal.x = TG.functions.computeImgCoordinatesFromSettings({align:"center"}, self.overlay.canvas.width, self.medal.width) - self.overlay.x;
    self.medal.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.medalY, self.medal.canvas.height, self.medal.height)- self.overlay.y + Math.abs(TG.canvasTopOffset);
    x = TG.functions.computeImgCoordinatesFromSettings(self.settings.medalX, self.medal.canvas.width, self.medal.width) - self.overlay.x;
    self.medal.xScale = self.settings.medalScale;
    self.medal.yScale = self.settings.medalScale;
    self.medal.parent = self.overlay;
    self.medal.alpha = 0;
    self.medal.centerOrigin = true;
    self.medal.addAnimator("scale", 1, self.settings.medalDuration, tbEase.easeInQuadratic, function(){tbGame.removeToken(self.medalShadow); self.medalShadow = undefined;});
    self.medal.addAnimator("x", x, self.settings.medalDuration, tbEase.easeInQuadratic);
    self.medal.addAnimator("alpha", 1, 0, tbEase.easeInQuadratic);
    self.medal.addAnimator("subImage", s_medal_shine.number, self.settings.medalShineDuration, tbEase.easeInQuadratic, undefined, self.settings.medalDuration + self.settings.medalEffectDuration + self.settings.medalShineDelay, true, self.settings.medalShineLoopInterval);
};

TG_LevelEndDialog.prototype.startMedalShadowAnimation = function(moment, tl) {
    "use strict";
    var y, self = tl.parent;
    self.medalShadow = new TG_AnimatedImage(self.depth - 110, TG.frameworkEffectsLayer, s_medal_shadow);
    self.medalShadow.y = TG.functions.computeImgCoordinatesFromSettings(self.settings.medalY, self.medalShadow.canvas.height, s_medal_shadow.height)- self.overlay.y + self.settings.medalShadowOffsetY;
    self.medalShadow.originX = self.settings.medalImageOriginX;
    self.medalShadow.originY = self.settings.medalImageOriginY;
    self.medalShadow.x = TG.functions.computeImgCoordinatesFromSettings(self.settings.medalX, self.medalShadow.canvas.width, self.medalShadow.width) - self.overlay.x;
    y = TG.functions.computeImgCoordinatesFromSettings(self.settings.medalY, self.medalShadow.canvas.height, s_medal_shadow.height)- self.overlay.y + Math.abs(TG.canvasTopOffset);
    self.medalShadow.xScale = self.settings.medalScale*self.settings.medalShadowScaleFactor;
    self.medalShadow.yScale = self.settings.medalScale*self.settings.medalShadowScaleFactor;
    self.medalShadow.alpha = 0;
    self.medalShadow.parent = self.overlay;
    self.medalShadow.addAnimator("y", y, self.settings.medalDuration, tbEase.easeInQuadratic);
    self.medalShadow.addAnimator("scale", 1, self.settings.medalDuration, tbEase.easeInQuadratic);
    self.medalShadow.addAnimator("alpha", 1, self.settings.medalDuration, tbEase.easeInQuadratic);
};

TG_LevelEndDialog.prototype.startMedalEffectAnimation = function(moment, tl) {
    "use strict";
    var self = tl.parent;

    self.medalEffect = new TG_AnimatedImage(self.depth - 130, TG.frameworkEffectsLayer, s_pop_medal);
    self.medalEffect.parent = self.overlay;
    self.medalEffect.x = self.medal.x;
    self.medalEffect.y = self.medal.y + self.settings.medalEffectOffsetY;
    self.medalEffect.addAnimator("subImage", s_pop_medal.number - 1, self.settings.medalEffectDuration, undefined, function(){tbGame.removeToken(self.medalEffect); self.medalEffect = undefined;});
    tbAudio.play(a_medal);
};

// handleRemove is called when the token is removed from the game.
TG_LevelEndDialog.prototype.handleRemove = function() {
	"use strict";
    var i;
    for(i=0; i<this.buttons.length; i++){
        tbGame.removeToken(this.buttons[i]);
    }
    for(i=0; i<this.stars.length; i++){
        tbGame.removeToken(this.stars[i]);
    }
    for(i=0; i<this.starEffects.length; i++){
        tbGame.removeToken(this.starEffects[i]);
    }

    //remove medal
    if(this.medal){
        tbGame.removeToken(this.medal);
        if(this.medalEffect){
            tbGame.removeToken(this.medalEffect);
        }
        if(this.medalShadow){
            tbGame.removeToken(this.medalShadow);
        }
    }
    tbGame.removeToken(this.overlay);

    if(this.timeLine){
        this.timeLine.stop();
    }

    //remove counter objects
    if(this.totalScoreCounter){
        tbGame.removeToken(this.totalScoreCounter);
    }
    for(i=0; i<this.bonusCounters.length; i++){
        tbGame.removeToken(this.bonusCounters[i]);
    }

    TG.controller._resumeAll();
};

// handleDraw is called at the end of a step to draw the token.
TG_LevelEndDialog.prototype.handleDraw = function() {
	"use strict";
    var alpha = tbGraphics.context.globalAlpha;
    tbGraphics.context.globalAlpha = this.alpha; //this.settings.fadeAlpha;
    tbGraphics.drawRectangle(0, 0, tbGraphics.canvas.width, tbGraphics.canvas.height, this.settings.fadeColor, false);
    tbGraphics.context.globalAlpha = alpha;
};
//******************************************************
//*****    Token Resource: TG_ChallengeEndDialog
//******************************************************

// ***************************************
// Definition of game token TG_ChallengeEndDialog
// ***************************************

/* global
	TG_MainController
 */


// The constructor for the TG_ChallengeEndDialog game token.
var TG_ChallengeEndDialog = function(score, submitFunction, cancelFunction){
	"use strict";
	// Properties used by the engine
	this.depth = 10;
	this.visible = true;
	this.active = false;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	this.settings = TG.settings.tg.challengeEnd;
	// apply theme and game specific setting overwrites
	var setting;
	for(setting in TG.settings.theme.challengeEnd){
		this.settings[setting] = TG.settings.theme.challengeEnd[setting];
	}
	this.submitFunction = submitFunction;
	this.cancelFunction = cancelFunction;
	this.challengeInfo = TG.controller.getChallengeInformation();
	this.challengeInfo.playerValues[this.challengeInfo.currentPlayerIdx] = score;
	this.timeLine = new tbTimeLine();
	this.timeLine.parent = this;
	this.buttons = [];
	// Add it to the game engine
	tbGame.addToken(this, false);
};

TG_ChallengeEndDialog.prototype.showMessage = function(text){
	"use strict";
	var s, font, maxFontSize, anim, self = this;
	anim = function(){
		self.message.addAnimator("xScale", 1, self.settings.messageShowDuration, tbEase.easeOutOvershoot);
		self.message.addAnimator("yScale", 1, self.settings.messageShowDuration, tbEase.easeOutOvershoot);
		self.message.addAnimator("alpha", 1, self.settings.messageShowDuration, tbEase.easeLinear);
	};
	// create animated image if one does not already exists
	if(this.message === undefined){
		s = new tbSurface(this.settings.messageWidth + 2 * this.settings.messageMarginX, this.settings.messageHeight + 2 * this.settings.messageMarginY);
		this.message = new TG_AnimatedImage(this.depth - 20, TG.frameworkUILayer, s);
		this.message.xScale = 0;
		this.message.yScale = 0;
		this.message.alpha = 0;
		this.message.originX = Math.floor(s.width / 2);
		this.message.originY = Math.floor(s.height / 2);
		this.message.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.messageX, this.canvas.width, 0) - this.overlayEndX;
		this.message.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.messageY, this.canvas.height, s.height) + Math.floor(s.height / 2)
		        - this.overlayEndY;
		this.message.parent = this.overlay;
		if(this.timeLine.getMoment() >= this.settings.messageShowDelay){
			anim();
		}else{
			this.timeLine.addMoment(this.settings.messageShowDelay, anim);
		}
	}

	// clear text
	this.message.image.setTarget();
	tbGraphics.clear();

	// draw new text
	font = f_default.clone();
	if(this.settings.messageFont !== undefined){
		font.setProperties(this.settings.messageFont);
	}
	font.setAlign("center");
	font.setBaseline("middle");
	maxFontSize = font.textFitSize(text, this.settings.messageWidth, this.settings.messageHeight, true);
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
	font.drawSimple(text, Math.floor(this.message.width / 2), Math.floor(this.message.height / 2), this.settings.messageWidth);
	if (testMode) {
		TG_MainController.drawSurfaceBoundaries();
	}

	this.message.image.resetTarget();
	if(this.message.xScale > 0){
		this.message.forceDraw = true;
	}
};

TG_ChallengeEndDialog.prototype.showSubMessage = function(text){
	"use strict";
	var font, maxFontSize, x, y;
	font = f_default.clone();
	//font.setProperties(this.settings.subMessageFont);
	if(this.settings.winningsFont !== undefined){
		font.setProperties(this.settings.winningsFont);
	}
	maxFontSize = font.textFitSize(text, this.settings.subMessageWidth, this.settings.subMessageHeight, true);
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
	x = TG.functions.computeTextCoordinatesFromSettings(this.settings.subMessageX, this.overlay.width, this.settings.subMessageWidth, font.getAlign());
	y = TG.functions.computeTextCoordinatesFromSettings(this.settings.subMessageY, this.overlay.height, this.settings.subMessageHeight, font.getBaseline());
	this.overlay.image.setTarget();
	font.drawSimple(text, x, y, this.settings.subMessageWidth);
	if (testMode) {
		TG_MainController.drawTextRect(font, x, y, this.settings.subMessageWidth, this.settings.subMessageHeight);
	}
	this.overlay.image.resetTarget();
};

TG_ChallengeEndDialog.prototype.showWinnings = function(){
	"use strict";
	var s, text, font, maxFontSize, anim, self = this;

	anim = function(){
		self.winnings.addAnimator("xScale", 1, self.settings.messageShowDuration, tbEase.easeOutOvershoot);
		self.winnings.addAnimator("yScale", 1, self.settings.messageShowDuration, tbEase.easeOutOvershoot);
		self.winnings.addAnimator("alpha", 1, self.settings.messageShowDuration, tbEase.easeLinear);
	};

	// create animated image
	s = new tbSurface(this.settings.winningsWidth + 2 * this.settings.winningsMarginX, this.settings.winningsHeight + 2 * this.settings.winningsMarginY);
	this.winnings = new TG_AnimatedImage(this.depth - 20, TG.frameworkUILayer, s);
	this.winnings.xScale = 0;
	this.winnings.yScale = 0;
	this.winnings.alpha = 0;
	this.winnings.originX = Math.floor(s.width / 2);
	this.winnings.originY = Math.floor(s.height / 2);
	this.winnings.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.winningsX, this.canvas.width, 0) - this.overlayEndX;
	this.winnings.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.winningsY, this.canvas.height, s.height) + Math.floor(s.height / 2)
	        - this.overlayEndY;
	this.winnings.parent = this.overlay;
	if(this.timeLine.getMoment() >= this.settings.messageShowDelay){
		anim();
	}else{
		this.timeLine.addMoment(this.settings.messageShowDelay, anim);
	}


	// draw text
	text = TG.dictionary.getText("challengeEndScreenWinnings", "CHALLENGEENDSCREENWINNINGS");
	text = text.replace("<AMOUNT>", this.challengeInfo.wager);
	this.winnings.image.setTarget();
	font = f_default.clone();
	if(this.settings.winningsFont !== undefined){
		font.setProperties(this.settings.winningsFont);
	}
	font.setAlign("center");
	font.setBaseline("middle");
	maxFontSize = font.textFitSize(text, this.settings.winningsWidth, this.settings.winningsHeight, true);
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
	font.drawSimple(text, Math.floor(this.winnings.width / 2), Math.floor(this.winnings.height / 2), this.settings.winningsWidth);
	if (testMode) {
		TG_MainController.drawSurfaceBoundaries();
	}
	this.winnings.image.resetTarget();
	if(this.winnings.xScale > 0){
		this.winnings.forceDraw = true;
	}
};

TG_ChallengeEndDialog.prototype.updateOutcome = function(outcome){
	"use strict";
	if(typeof outcome === "string"){
		var text = TG.dictionary.getText("challengeEndScreenOutcomeMessage_" + outcome, "<CHALLENGEENDSCREEN_CHALLENGE" + outcome.toUpperCase() + ">");
		this.showMessage(text);
		if(outcome === "WON"){
			this.showWinnings();
		}
	}
};

TG_ChallengeEndDialog.prototype.addSubmitCancelButtons = function(){
	"use strict";
	var submit, cancel, x, y, self = this;

	submit = function(){
		var i;
		self.submitFunction();
		for(i = 0; i < self.buttons.length; i++){
			tbGame.removeToken(self.buttons[i]);
		}
		return true;
	};
	cancel = function(){
		var dialog, yesFunction, noFunction, i;
		yesFunction = function(){
			dialog.close();
			self.cancelFunction();
			for(i = 0; i < self.buttons.length; i++){
				tbGame.removeToken(self.buttons[i]);
			}
			return true;
		};
		noFunction = function(){
			dialog.close();
		};
		dialog = new TG_Dialog(TG.dictionary.getText("challengeCancelConfirmText", "<CHALLENGECANCELCONFIRMTEXT>"), [{
		    textKey: "challengeCancelConfirmBtn_yes",
		    callbackFunc: yesFunction
		}, {
		    textKey: "challengeCancelConfirmBtn_no",
		    callbackFunc: noFunction
		}], true);
	};

	// add submit button
	x = TG.functions.computeImgCoordinatesFromSettings(this.settings.submitBtnX, this.overlay.width, this.settings.textButtonsWidth);
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.submitBtnY, this.overlay.height, 0);
	this.buttons.push(new TG_ButtonText("default_text", x, y, this.depth - 20, "challengeEndScreensBtn_submit", this.settings.textButtonsWidth, submit,
	        this.overlay));
	// add cancel button
	x = TG.functions.computeImgCoordinatesFromSettings(this.settings.cancelBtnX, this.overlay.width, this.settings.textButtonsWidth);
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.cancelBtnY, this.overlay.height, 0);
	this.buttons.push(new TG_ButtonText("default_text", x, y, this.depth - 20, "challengeEndScreenBtn_cancel", this.settings.textButtonsWidth, cancel,
	        this.overlay));
};

TG_ChallengeEndDialog.prototype.handleAdd = function(){
	"use strict";
	var overlaySurface, titleSurface, font, maxFontSize, text, i, s, name, score, key, x, y, counter, fontHeader, order, idx, backLog, statsObj, nameTexts, setupAnimatedImageForScaleAnimation = function(
	        image){
		image.xScale = 0;
		image.yScale = 0;
		image.originX = Math.floor(image.image.width / 2);
		image.originY = Math.floor(image.image.height / 2);
	}, self = this;

	//pause everything else
	TG.controller._pauseAll(this);

	// create overlay
	if(typeof s_overlay_challenge_end !== "undefined"){
		overlaySurface = new tbSurface(s_overlay_challenge_end.width, s_overlay_challenge_end.height);
		overlaySurface.setTarget();
		s_overlay_challenge_end.drawSimple(0, 0, 0);
		overlaySurface.resetTarget();
	}else{
		overlaySurface = new tbSurface(tgScale(560), tgScale(560));
	}
	this.overlay = new TG_AnimatedImage(this.depth - 10, TG.frameworkUILayer, overlaySurface);
	this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayStartX, this.canvas.width, this.overlay.width)
	        + this.settings.overlayOffsetX;
	this.overlay.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayStartY, this.canvas.height, this.overlay.height)
	        + TG.backgroundOffsetTop + this.settings.overlayOffsetY;
	this.overlayEndX = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayEndX, this.canvas.width, this.overlay.width)
	        + this.settings.overlayOffsetX;
	this.overlayEndY = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayEndY, this.canvas.height, this.overlay.height)
	        + this.settings.overlayOffsetY;

	// determine order in which player statistics should be shown
	order = new Array(this.challengeInfo.playerNames.length);
	idx = 0;
	backLog = [];
	for(i = 0; i < this.challengeInfo.playerNames.length; i++){
		if(i !== this.challengeInfo.currentPlayerIdx){
			if(this.challengeInfo.playerValues[i] !== undefined){
				order[i] = idx;
				idx++;
			}else{
				backLog.push(i);
			}
		}
	}
	order[this.challengeInfo.currentPlayerIdx] = idx;
	idx++;
	for(i = 0; i < backLog.length; i++){
		order[backLog[i]] = idx;
		idx++;
	}

	// create names
	font = f_default.clone();
	if(this.settings.namesFont !== undefined){
		font.setProperties(this.settings.namesFont);
	}
	font.setAlign("center");
	font.setBaseline("middle");
	//abbreviate names that are too long
	nameTexts = [];
	for(i = 0; i < this.challengeInfo.playerNames.length; i++){
		text = (i === this.challengeInfo.currentPlayerIdx)? TG.dictionary.getText("challengeEndScreenName_you", "<YOU>") : this.challengeInfo.playerNames[i];
		if(text.length > 13){
			text = text.substr(0, 10) + "...";
		}
		nameTexts.push(text);
	}
	maxFontSize = font.getFontSize();
	for(i = 0; i < nameTexts.length; i++){
		maxFontSize = Math.min(maxFontSize, font.textFitSize(nameTexts[i], this.settings.nameWidth, this.settings.nameHeight, false));
	}
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
	this.names = [];

	for(i = 0; i < nameTexts.length; i++){
		text = nameTexts[i];
		s = new tbSurface(this.settings.nameWidth + 2 * this.settings.nameMarginX, this.settings.nameHeight + 2 * this.settings.nameMarginY);
		s.setTarget();
		font.drawSimple(text, Math.floor(s.width / 2) + this.settings.nameOffsetX, Math.floor(s.height / 2) + this.settings.nameOffsetY);
		if (testMode) {
			TG_MainController.drawSurfaceBoundaries();
		}
		s.resetTarget();
		name = new TG_AnimatedImage(this.depth - 20, this.layer, s);
		name.alpha = 0;
		setupAnimatedImageForScaleAnimation(name);
		name.x = this.settings.namesLeftX + order[i] * (s.width + this.settings.namesBetweenX) + Math.floor(s.width / 2)
		        - this.overlayEndX;
		name.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.namesY, this.canvas.height, s.height) + Math.floor(s.height / 2)
		        - this.overlayEndY;
		name.parent = this.overlay;
		this.names.push(name);
	}

	// create score
	this.emphasizedScores = [];
	font = f_default.clone();
	if(this.settings.emphasizedScoresFont !== undefined){
		font.setProperties(this.settings.emphasizedScoresFont);
	}
	font.setAlign("center");
	font.setBaseline("middle");
	for(i = 0; i < this.challengeInfo.playerNames.length; i++){
		if(this.challengeInfo.playerValues[i] !== undefined){
			statsObj = new TG_StatObject("player_" + i, TG.gameFlowSettings.scoreType, "", this.challengeInfo.playerValues[i]);
		}else{
			statsObj = new TG_StatObject("player_" + i, "text", "", "?");
		}
		x = this.names[i].x + this.settings.emphasizedScoresCounterMarginX;
		y = TG.functions.computeImgCoordinatesFromSettings(this.settings.emphasizedScoresY, this.canvas.height, this.settings.emphasizedScoresHeight)
		        + Math.floor(s.height / 2) - this.overlayEndY;
		counter = new TG_Counter(x, y, this.settings.emphasizedScoresWidth, this.settings.emphasizedScoresHeight, this.depth - 18,
		        (i === this.challengeInfo.currentPlayerIdx)? 0 : statsObj.endValue, font, this.settings.emphasizedScoresCounterMarginX,
		        this.settings.emphasizedScoresCounterMarginY, this.overlay, statsObj.toString);
		setupAnimatedImageForScaleAnimation(counter.animatedImage);
		counter.animatedImage.alpha = 0;
		this.emphasizedScores.push(counter);
	}

	// set-up timeLine
	this.timer = 0;
	this.timeLine.addMoment(this.settings.animInDelay, function(){
		self.overlay.addAnimator("x", self.overlayEndX, self.settings.animInDuration, tbEase.easeOutOvershoot);
		self.overlay.addAnimator("y", self.overlayEndY + TG.backgroundOffsetTop, self.settings.animInDuration, tbEase.easeOutOvershoot);
	});
	this.timeLine.addMoment(this.settings.namesShowDelay, function(){
		var i;
		for(i = 0; i < self.names.length; i++){
			self.names[i].addAnimator("xScale", 1, self.settings.namesShowDuration, tbEase.easeOutOvershoot);
			self.names[i].addAnimator("yScale", 1, self.settings.namesShowDuration, tbEase.easeOutOvershoot);
			self.names[i].addAnimator("alpha", 1, self.settings.namesShowDuration, tbEase.easeLinear);
		}
	});
	this.timeLine.addMoment(this.settings.scoreShowDelay, function(){
		var i;
		for(i = 0; i < self.names.length; i++){
			self.emphasizedScores[i].animatedImage.addAnimator("xScale", 1, self.settings.scoresShowDuration, tbEase.easeOutOvershoot);
			self.emphasizedScores[i].animatedImage.addAnimator("yScale", 1, self.settings.scoresShowDuration, tbEase.easeOutOvershoot);
			self.emphasizedScores[i].animatedImage.addAnimator("alpha", 1, self.settings.scoresShowDuration, tbEase.easeLinear);
		}
	});

	this.timeLine.addMoment(this.settings.scoresCountDelay, function(){
		self.emphasizedScores[self.challengeInfo.currentPlayerIdx].startCount(self.challengeInfo.playerValues[self.challengeInfo.currentPlayerIdx],
		        self.settings.scoresCountDuration, tbEase.easeLinear);
	});

	if(typeof this.submitFunction === "function" && typeof this.cancelFunction === "function"){
		this.timeLine.addMoment(self.settings.buttonsShowDelay, function(){
			self.addSubmitCancelButtons();
		});
	}

	//inform portals that level end screen is shown
    this.timeLine.addMoment(this.settings.animInDelay, TG.hooks.informLevelEndScreenShown);

	this.timeLine.start();

	if(typeof this.submitFunction === "function" && typeof this.cancelFunction !== "function"){
		this.submitFunction();
	}

};

TG_ChallengeEndDialog.prototype.handleRemove = function(){
	"use strict";
	var i;
	if(this.message){
		tbGame.removeToken(this.message);
	}
	for(i=0; i<this.buttons.length; i++){
		tbGame.removeToken(this.buttons[i]);
	}
	for(i=0; i<this.names.length; i++){
		tbGame.removeToken(this.names[i]);
	}
	for(i=0; i<this.emphasizedScores.length; i++){
		tbGame.removeToken(this.emphasizedScores[i]);
	}

	TG.controller._resumeAll();
};

// handleDraw is called at the end of a step to draw the token.
TG_ChallengeEndDialog.prototype.handleDraw = function(){
	"use strict";
	var alpha = tbGraphics.context.globalAlpha;
	tbGraphics.context.globalAlpha = this.settings.fadeAlpha;
	tbGraphics.drawRectangle(0, 0, tbGraphics.canvas.width, tbGraphics.canvas.height, this.settings.fadeColor, false);
	tbGraphics.context.globalAlpha = alpha;
};
//******************************************************
//*****    Token Resource: TG_InterstitialController
//******************************************************

// ***************************************
// Definition of game token TG_InterstitialController
// ***************************************

// The constructor for the TG_InterstitialController game token.
var TG_InterstitialController = function(x, y, depth) {
	"use strict";
	// Properties used by the engine
	this.depth = depth;
	this.visible = true;
	this.active = true;
	// Your own properties
	this.x = x;
	this.y = y;
	// Add it to the game engine
	tbGame.addToken(this);
};

// handleAdd is called when the token is (re-) added to the game.
TG_InterstitialController.prototype.handleAdd = function() {
	"use strict";
};

// handleRemove is called when the token is removed from the game.
TG_InterstitialController.prototype.handleRemove = function() {
	"use strict";
};

// handleBeginStep is called each step before processing other events.
TG_InterstitialController.prototype.handleBeginStep = function(timePassed) {
	"use strict";
};

// handleTouchDown is called when a touch is started or if a mouse button is
// pressed.
TG_InterstitialController.prototype.handleTouchDown = function(finger, x, y) {
	"use strict";
};

// handleTouchUp is called when a touch is ended or if a mouse button is
// released.
TG_InterstitialController.prototype.handleTouchUp = function(finger, x, y) {
	"use strict";
};

// handleKeyDown is called if the key with the given keyCode is pressed.
TG_InterstitialController.prototype.handleKeyDown = function(keyCode) {
	"use strict";
};

// handleKeyUp is called if the key with the given keyCode is released.
TG_InterstitialController.prototype.handleKeyUp = function(keyCode) {
	"use strict";
};

// handleAlarm is called if a global alarm is reached.
TG_InterstitialController.prototype.handleAlarm = function(alarm) {
	"use strict";
};

// handleSpecial is called if a special or user-defined event happens.
TG_InterstitialController.prototype.handleSpecial = function(id, args) {
	"use strict";
};

// handleEndStep is called each step after processing other events (before
// draw).
TG_InterstitialController.prototype.handleEndStep = function(timePassed) {
	"use strict";
};

// handleDraw is called at the end of a step to draw the token.
TG_InterstitialController.prototype.handleDraw = function() {
	"use strict";
};

//******************************************************
//*****    Token Resource: TG_LevelPreviewDialog
//******************************************************

// ***************************************
// Definition of game token TG_LevelPreviewDialog
// ***************************************

// The constructor for the TG_LevelPreviewDialog game token.
var TG_LevelPreviewDialog = function(levelIdx, levelDescription, startLevelCallback, cancelCallback) {
	"use strict";
	// Properties used by the engine
	this.depth = 100;
	this.visible = true;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
    this.settings = TG.settings.theme.levelPreview;
    this.levelIdx = levelIdx;
    this.levelDescription = levelDescription;
    this.startLevelCallback = startLevelCallback;
    this.cancelCallback = cancelCallback;
    this.overlay = {x: 0, y: 0, xScale: 1, yScale: 1, angle: 0, alpha: 1};
    this.touchStarted = false;

	// Add it to the game engine
	tbGame.addToken(this);
};

TG_LevelPreviewDialog.prototype.startLevel = function() {
    "use strict";
    tbGame.removeToken(this);
    this.startLevelCallback(this.levelIdx);
};

TG_LevelPreviewDialog.prototype.close = function() {
    "use strict";
    tbGame.removeToken(this);
    this.cancelCallback();
};

// handleAdd is called when the token is (re-) added to the game.
TG_LevelPreviewDialog.prototype.handleAdd = function() {
	"use strict";
    var canvas, context, alpha, img, width, height, imgIdx, font, txt, btnType, x, y,
        background = this.settings.background;
    //draw background
    if(background.type === "fade"){
        //draw background fade
        canvas = TG.functions.getCanvas(TG.frameworkBackgroundLayer);
        if(tbGraphics.canvas !== canvas){
            tbGraphics.setCanvas(canvas);
        }
        context = tbGraphics.context;
        alpha = context.globalAlpha;
        context.globalAlpha = background.alpha;
        tbGraphics.drawRectangle(0,0,tbGraphics.width, tbGraphics.height, background.color, false);
        context.globalAlpha = alpha;
    }

    //create overlay surface
    img = this.settings.popupImage;
    width = img.width;
    height = img.height;
    this.overlay.mask = new tbMaskRectangle(0,0,img.width, img.height);
    this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.popupImageX, this.canvas.width, width);
    this.overlay.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.popupImageY, this.canvas.height, height);
    this.overlay.surface = new tbSurface(width, height);
    this.overlay.surface.setTarget();
    //draw overlay background
    img.drawSimple(0, 0, 0);
    //drawLevelTitle
    if(this.settings.levelTitleFont){
        font = f_default.clone();
        font.setProperties(this.settings.levelTitleFont);
        font.drawSimple(TG.dictionary.getText("levelTitle ", "<levelTitle> ")+(this.levelIdx+1),
            TG.functions.computeImgCoordinatesFromSettings(this.settings.levelTitleXOffset, width, 0),
            TG.functions.computeImgCoordinatesFromSettings(this.settings.levelTitleYOffset, height, 0));
    }
    //draw stars
    if(this.settings.starsImage) {
        img = this.settings.starsImage;
        imgIdx = this.levelDescription.stars || 0;
        img.drawSimple(imgIdx, TG.functions.computeImgCoordinatesFromSettings(this.settings.starsImageXOffset, width, img.width),
            TG.functions.computeImgCoordinatesFromSettings(this.settings.starsImageYOffset, height, img.height));
    }
    //draw assignment
    if(this.levelDescription.assignmentTextKey){
        font = f_default.clone();
        if(this.settings.assignmentFont) {
            font.setProperties(this.settings.assignmentFont);
        }
        font.drawSimple(TG.dictionary.getText(this.levelDescription.assignmentTextKey, "<"+this.levelDescription.assignmentTextKey+">"), TG.functions.computeImgCoordinatesFromSettings(this.settings.assignmentXOffset, width, 0),
            TG.functions.computeImgCoordinatesFromSettings(this.settings.assignmentYOffset, height, 0));
    }
    //draw target
    if(this.levelDescription.nextTarget !== undefined){
        txt =  TG.dictionary.getText("target", "<target <POINTS> points>").replace("<POINTS>", this.levelDescription.nextTarget+"");
        font = f_default.clone();
        if(this.settings.targetFont) {
            font.setProperties(this.settings.targetFont);
        }
        font.drawSimple(txt, TG.functions.computeImgCoordinatesFromSettings(this.settings.targetXOffset, width, 0),
            TG.functions.computeImgCoordinatesFromSettings(this.settings.targetYOffset, height, 0));
    }

    this.overlay.surface.resetTarget();


    //create buttons
    this.buttons = [];
    //close button
    btnType = TG.settings.theme.buttons[this.settings.closeBtnType];
    if(btnType.type === "image"){
        img = btnType.sprite;
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.closeBtnXOffset, this.overlay.surface.width, img.width);
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.closeBtnYOffset, this.overlay.surface.height, img.height);
        this.buttons.push(new TG_ButtonBase(x, y, this.depth - 10, new tbMaskImage(img), [img], {callbackFunc: this.close, callbackObj: this}, this.overlay));
    }
    //play button
    btnType = TG.settings.theme.buttons[this.settings.playBtnType];
    y = TG.functions.computeImgCoordinatesFromSettings(this.settings.playBtnYOffset, this.overlay.surface.height, btnType.sprite.height);
    if(btnType.type === "text"){
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.playBtnXOffset, this.overlay.surface.width, this.settings.playBtnWidth);
        this.buttons.push(new TG_ButtonToggleText(btnType, x, y, this.depth - 10, TG.dictionary.getText("play", "play"), this.settings.playBtnWidth, {callbackFunc: this.startLevel, callbackObj: this}, this.overlay));
    }else if(btnType.type === "image"){
        img = btnType.sprite;
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.playBtnXOffset, this.overlay.surface.width, img.width);
        this.buttons.push(new TG_ButtonBase(x, y, this.depth - 10, new tbMaskImage(img), [img], {callbackFunc: this.startLevel, callbackObj: this}, this.overlay));
    }

    //start overlay in-animation
    this.state = "animIn";
    this.canvas.drawTokens = true;
};

// handleRemove is called when the token is removed from the game.
TG_LevelPreviewDialog.prototype.handleRemove = function() {
	"use strict";
    var i;
    for(i=0; i<this.buttons.length; i++){
        tbGame.removeToken(this.buttons[i]);
    }
    this.canvas.drawTokens = true;
 };

// handleBeginStep is called each step before processing other events.
TG_LevelPreviewDialog.prototype.handleBeginStep = function(timePassed) {
	"use strict";

};

// handleTouchDown is called when a touch is started or if a mouse button is pressed.
TG_LevelPreviewDialog.prototype.handleTouchDown = function(finger, x, y) {
	"use strict";
    if(!this.overlay.mask.collisionPoint(this.overlay.x, this.overlay.y, x, y)){
        this.touchStarted = true;
    }
	return true;
};

// handleTouchUp is called when a touch is ended or if a mouse button is released.
TG_LevelPreviewDialog.prototype.handleTouchUp = function(finger, x, y) {
	"use strict";
    if(this.touchStarted && !this.overlay.mask.collisionPoint(this.overlay.x, this.overlay.y, x, y)){
        this.close();
    }
    this.touchStarted = false;
    return true;
};

// handleKeyDown is called if the key with the given keyCode is pressed.
TG_LevelPreviewDialog.prototype.handleKeyDown = function(keyCode) {
	"use strict";
};

// handleKeyUp is called if the key with the given keyCode is released.
TG_LevelPreviewDialog.prototype.handleKeyUp = function(keyCode) {
	"use strict";
};

// handleAlarm is called if a global alarm is reached.
TG_LevelPreviewDialog.prototype.handleAlarm = function(alarm) {
	"use strict";
};

// handleSpecial is called if a special or user-defined event happens.
TG_LevelPreviewDialog.prototype.handleSpecial = function(id, args) {
	"use strict";
};

// handleEndStep is called each step after processing other events (before draw).
TG_LevelPreviewDialog.prototype.handleEndStep = function(timePassed) {
	"use strict";
};

// handleDraw is called at the end of a step to draw the token.
TG_LevelPreviewDialog.prototype.handleDraw = function() {
	"use strict";
    this.overlay.surface.drawExt(this.overlay.x, this.overlay.y, this.overlay.xScale, this.overlay.yScale, this.overlay.angle, this.overlay.alpha);
};


//******************************************************
//*****    Token Resource: TG_Options
//******************************************************

// ***************************************
// Definition of game token TG_Options
// ***************************************

/* global
	TG_MainController,
	TG_ButtonBase,
	TG_ButtonImage,
	TG_ButtonToggleText,
	TG_ButtonText,
	TG_ButtonToggle,
	TG_AnimatedImage,
	f_default,
	s_tutorialButton_previous,
	s_tutorialButton_close,
	s_tutorialButton_next,
	s_logo_coolgames,
	tbKeyArea
 */

/**
 * The constructor for the TG_Options game token.
 *
 * @constructor
 */
function TG_Options(type, tutorial, challenge, paused) {
	"use strict";
	// Properties used by the engine
	this.depth = -100;
	this.visible = false;
	this.active = true;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	var setting, landscapeSettings, themeSettings;
	this.settings = challenge ? TG.settings.tg.options_challenge : TG.settings.tg.options;
	if (TG.orientation === "landscape") {
		landscapeSettings = challenge ? TG.settings.tg.options_challenge_landscape : TG.settings.tg.options_landscape;
		for (setting in landscapeSettings) {
			this.settings[setting] = landscapeSettings[setting];
		}
	}
	this.buttonSettings = TG.settings.tg.buttonDefinitions;
	// apply theme specific setting overwrites
	themeSettings = challenge ? TG.settings.theme.options_challenge : TG.settings.theme.options;
	for (setting in themeSettings) {
		this.settings[setting] = themeSettings[setting];
	}
	// allow custom buttons
	if (TG.config.options && TG.config.options.buttons) {
		for (setting in TG.config.options.buttons) {
			this.settings.buttons[setting] = TG.config.options.buttons[setting];
		}
	}

	this.type = type;
	this.tutorialVisibleOnStart = tutorial;
	this.challengeMode = challenge;
	this.shouldPause = paused !== false;
	// Add it to the game engine
	tbGame.addToken(this);
}

TG_Options.prototype.addButton = function(type, buttonX, buttonY, buttonWidth, enableButton) {
	"use strict";
	var btnType = undefined, textKey = undefined, state = undefined, callbackFunction = undefined, callbackImmediate = undefined, image = undefined;
	switch (type) {
	case "music":
		btnType = "music_toggle";
		callbackFunction = this.toggleMusic;
		state = (TG.controller.playMusic()) ? "on" : "off";
		break;
	case "music_big":
		btnType = "music_big_toggle";
		callbackFunction = this.toggleMusic;
		state = (TG.controller.playMusic()) ? "on" : "off";
		break;
	case "sfx_big":
		btnType = "sfx_big_toggle";
		callbackFunction = this.toggleSFX;
		state = (TG.controller.playSFX()) ? "on" : "off";
		break;
	case "sfx":
		btnType = "sfx_toggle";
		callbackFunction = this.toggleSFX;
		state = (TG.controller.playSFX()) ? "on" : "off";
		break;
	case "language":
		btnType = "language_toggle";
		callbackFunction = this.toggleLang;
		state = TG.controller.language();
		break;
	case "tutorial":
		btnType = "default_text";
		textKey = "optionsTutorial";
		callbackFunction = this.showTutorial;
		break;
	case "highScores":
		btnType = "default_text";
		textKey = "optionsHighScore";
		callbackFunction = this.sendAway;
		this.buttonCallback = this.showHighScores;
		break;
	case "moreGames":
		if (TG.config.moreGamesImage !== undefined) {
			btnType = "default_image";
			image = TG.config.moreGamesImage;
		}
		else {
			btnType = "default_text";
			textKey = "optionsMoreGames";
		}
		callbackFunction = this.showMoreGames;
		callbackImmediate = true;
		break;
	case "resume":
		btnType = "default_text";
		textKey = "optionsResume";
		callbackFunction = this.close;
		break;
	case "exit":
		btnType = "default_text";
		textKey = "optionsExit";
		callbackFunction = (TG.publisherSettings["customFunctions"] && typeof TG.publisherSettings["customFunctions"]["exit"] === "function") ? TG.publisherSettings["customFunctions"]["exit"]
				: function() {
				};
		break;
	case "quit":
		btnType = "default_text";
		textKey = "optionsQuit";
		callbackFunction = this.quit;
		break;
	case "restart":
		btnType = "default_text";
		textKey = "optionsRestart";
		callbackFunction = this.restart;
		break;
	case "startScreen":
		btnType = "default_text";
		textKey = "optionsStartScreen";
		callbackFunction = this.sendAway;
		this.buttonCallback = this.showStartScreen;
		break;
	case "about":
		btnType = "default_text";
		textKey = "optionsAbout";
		callbackFunction = this.showAbout;
		break;
	case "forfeitChallenge":
		btnType = "default_text";
		textKey = "optionsChallengeForfeit";
		callbackFunction = this.forfeitChallenge;
		break;
	case "cancelChallenge":
		btnType = "default_text";
		textKey = "optionsChallengeCancel";
		callbackFunction = this.cancelChallenge;
		break;
	}
	if (btnType !== undefined && callbackFunction !== undefined) {

		if (this.buttonSettings[btnType].type === "image") {
			this.buttons.push(new TG_ButtonImage(btnType, buttonX, buttonY, this.depth - 20, image, buttonWidth, {
				callbackFunc: callbackFunction,
				callbackObj: this,
				immediate: callbackImmediate
			}, this.overlay));
		}
		else if (this.buttonSettings[btnType].type === "toggleText") {
			this.buttons.push(new TG_ButtonToggleText(btnType, buttonX, buttonY, this.depth - 20, state, buttonWidth, {
				callbackFunc: callbackFunction,
				callbackObj: this,
				immediate: callbackImmediate
			}, this.overlay));
		}
		else if (this.buttonSettings[btnType].type === "text") {
			this.buttons.push(new TG_ButtonText(btnType, buttonX, buttonY, this.depth - 20, textKey, buttonWidth, {
				callbackFunc: callbackFunction,
				callbackObj: this,
				immediate: callbackImmediate
			}, this.overlay));
		}
		else if (this.buttonSettings[btnType].type === "toggle") {
			this.buttons.push(new TG_ButtonToggle(btnType, buttonX, buttonY, this.depth - 20, state, {
				callbackFunc: callbackFunction,
				callbackObj: this,
				immediate: callbackImmediate
			}, this.overlay));
		}
		this.buttons[this.buttons.length - 1].enabled = enableButton || false;
	}
};

TG_Options.prototype.sendAway = function() {
	"use strict";
	var end = (this.type !== "inGame") ? -this.overlay.image.height : this.canvas.height;
	var self = this;
	function callback() {
		tbGame.removeToken(self);
		if (self.buttonCallback !== undefined) {
			self.buttonCallback.call(self);
		}
	}
	self.overlay.addAnimator("y", end, this.settings.overlayAnimOutDuration, this.settings.overlayAnimOutEase, callback);
	return true;
};

TG_Options.prototype.drawBackground = function(overlayXOffset, overlayYOffset) {
	"use strict";
	var image, titleText, titleFont, titleX, titleY, maxFontSize;
	this.overlay.image.setTarget();
	tbGraphics.clear();

	// draw overlay background
	image = this.settings.backgroundImage;
	image.drawSimple(0, 0, 0);

	// draw overlay title
	titleText = TG.dictionary.getText("optionsTitle", "<OPTIONS_TITLE>");
	titleFont = f_default.clone();

	if (this.settings.titleFont) {
		titleFont.setProperties(this.settings.titleFont);
	}
	if (this.settings.titleMaxWidth !== undefined && this.settings.titleMaxHeight !== undefined) {
		maxFontSize = titleFont.textFitSize(titleText, this.settings.titleMaxWidth, this.settings.titleMaxHeight, this.settings.titleMaxWidth);
		if (titleFont.getFontSize() > maxFontSize) {
			titleFont.setFontSize(maxFontSize);
		}
	}
	titleX = TG.functions.computeTextCoordinatesFromSettings(this.settings.titleX, this.canvas.width, titleFont.textWidth(titleText), titleFont.getAlign())
			- overlayXOffset;
	titleY = TG.functions
			.computeTextCoordinatesFromSettings(this.settings.titleY, this.canvas.height, titleFont.textHeight(titleText, titleFont.getBaseline()))
			- overlayYOffset + TG.canvasTopOffset * -1;
	titleFont.drawSimple(titleText, titleX, titleY);

	if (testMode) {
		TG_MainController.drawTextRect(titleFont, titleX, titleY);
	}

	this.overlay.image.resetTarget();
};

TG_Options.prototype.addInitButtons = function(overlayXOffset, overlayYOffset, enableButtons) {
	"use strict";
	var i, j, buttonHeight, buttonMaxWidth, left, totalHeight, row, buttonX, buttonY, buttonWidth, stop = false, buttonsToShow = this.settings.buttons[this.type];
	if (this.type === "inGame" && TG.settings.game.framework.menuButtons) {
		buttonsToShow = TG.settings.game.framework.menuButtons;
	}

	// remove more games button if no more games url
	if (typeof TG.controller.getMoreGamesFunction() !== "function") {
		for (i = 0; i < buttonsToShow.length; i++) {
			if (typeof buttonsToShow[i] === "string" && buttonsToShow[i] === "moreGames") {
				buttonsToShow.splice(i, 1);
				break;
			}
			for (j = 0; j < buttonsToShow[i].length; j++) {
				if (buttonsToShow[i][j] === "moreGames") {
					buttonsToShow[i].splice(j, 1);
					break;
				}
			}
		}
	}

	// remove music button if no music
	if (TG.config.playMusic === false || TG.controller.loadMusic === false) {
		for (i = 0; i < buttonsToShow.length; i++) {
			if (buttonsToShow[i] instanceof Array) {
				for (j = 0; j < buttonsToShow[i].length; j++) {
					if (buttonsToShow[i][j] === "music") {
						if (TG.controller.loadSfx) {
							buttonsToShow[i] = "sfx_big";
						}
						else {
							buttonsToShow.splice(i, 1);
						}
						stop = true;
						break;
					}
				}
				if (stop) {
					break;
				}
			}
			else if (buttonsToShow[i] === "music_big") {
				buttonsToShow.splice(i, 1);
				break;
			}
		}
	}
	// remove sfx button if no sfx
	if (!TG.controller.loadSfx) {
		for (i = 0; i < buttonsToShow.length; i++) {
			if (buttonsToShow[i] instanceof Array) {
				for (j = 0; j < buttonsToShow[i].length; j++) {
					if (buttonsToShow[i][j] === "sfx") {
						if (TG.config.playMusic !== false && TG.controller.loadMusic) {
							buttonsToShow[i] = "music_big";
						}
						else {
							buttonsToShow.splice(i, 1);
						}
						stop = true;
						break;
					}
				}
				if (stop) {
					break;
				}
			}
			else if (buttonsToShow[i] === "sfx_big") {
				buttonsToShow.splice(i, 1);
				break;
			}
		}
	}

	// remove language
	if (TG.dictionary.getAvailableLanguages().length === 1) {
		for (i = 0; i < buttonsToShow.length; i++) {
			if (buttonsToShow[i] === "language") {
				buttonsToShow.splice(i, 1);
				break;
			}
		}
	}

	buttonHeight = this.buttonSettings["default_text"].sprite.height;
	buttonMaxWidth = this.settings.buttonsWidth;
	left = TG.functions.computeImgCoordinatesFromSettings(this.settings.buttonsOffsetX, this.canvas.width, buttonMaxWidth) - overlayXOffset;
	totalHeight = buttonHeight * buttonsToShow.length + this.settings.buttonsOffsetInBetween * (buttonsToShow.length - 1);
	buttonY = TG.functions.computeImgCoordinatesFromSettings(this.settings.buttonsOffsetY, this.overlay.image.height, totalHeight) - overlayYOffset
			+ TG.canvasTopOffset * -1;

	for (i = 0; i < buttonsToShow.length; i++) {
		buttonX = left;
		buttonWidth = buttonMaxWidth;
		if (typeof buttonsToShow[i] === "string") {
			this.addButton(buttonsToShow[i], buttonX, buttonY, buttonWidth, enableButtons);
		}
		else {
			row = buttonsToShow[i];
			buttonWidth = (buttonMaxWidth - (row.length - 1) * this.settings.buttonsOffsetInBetween) / row.length;
			for (j = 0; j < row.length; j++) {
				this.addButton(row[j], buttonX, buttonY, buttonWidth, enableButtons);
				buttonX += buttonWidth + this.settings.buttonsOffsetInBetween;
			}
		}
		buttonY += buttonHeight + this.settings.buttonsOffsetInBetween;
	}
};

TG_Options.prototype.toggleMusic = function(toggleValue) {
	"use strict";
	var value = true;
	if (toggleValue === "off") {
		value = false;
		TG.analytics.reportEvent("off", "options:music");
	}
	else {
		TG.analytics.reportEvent("on", "options:music");
	}
	TG.controller.playMusic(value);
	return true;
};

TG_Options.prototype.toggleSFX = function(toggleValue) {
	"use strict";
	var value = true;
	if (toggleValue === "off") {
		value = false;
		TG.analytics.reportEvent("off", "options:sfx");
	}
	else {
		TG.analytics.reportEvent("on", "options:sfx");
	}
	TG.controller.playSFX(value);
	return true;
};

TG_Options.prototype.toggleLang = function(langKey) {
	"use strict";
	TG.dictionary.setLanguage(langKey);
	TG.analytics.reportEvent(langKey, "options:language");
	return true;
};

TG_Options.prototype.showTutorial = function() {
	"use strict";
	var i, x, y, font, text, maxFontSize, image, self = this;
	var backgroundX = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.settings.backgroundImage.width);
	var backgroundY = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height,
			self.settings.backgroundImage.height)
			+ TG.canvasTopOffset * -1;
	function next() {
		self.tutorialIdx += 1;
		self.showTutorial();
		return true;
	}
	function previous() {
		self.tutorialIdx -= 1;
		self.showTutorial();
		return true;
	}
	function close() {
		var i;
		// redraw background
		self.drawBackground(backgroundX, backgroundY);
		self.closeButton.enabled = true;
		// remove quit buttons
		for (i = 0; i < self.buttons.length; i++) {
			tbGame.removeToken(self.buttons[i]);
		}
		self.buttons = [];
		// add normal buttons
		self.addInitButtons(backgroundX, backgroundY, true);
	}
	if (self.tutorialIdx === undefined) {
		self.tutorialIdx = 0;
	}
	self.tutorialData = (TG.game.getTutorialData !== undefined) ? TG.game.getTutorialData(TG.controller.currentLevelIdx, TG.controller.getDifficulty()) : [];
	TG.analytics.reportEvent(((self.tutorialIdx < 10) ? "0" : "") + self.tutorialIdx, "options:tutorial");
	// remove all buttons except close button
	for (i = 0; i < self.buttons.length; i++) {
		tbGame.removeToken(self.buttons[i]);
	}
	self.buttons = [];

	if (!this.challengeMode) {
		self.drawBackground(backgroundX, backgroundY);
	}
	else {
		self.overlay.image.setTarget();
		tbGraphics.clear();
		self.closeButton.enabled = false;
	}
	self.overlay.image.setTarget();
	// draw tutorial background
	if (self.settings.tutorialBackgroundImage !== undefined) {
		x = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialBackgroundOffsetX, self.overlay.image.width,
				self.settings.tutorialBackgroundImage.width);
		y = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialBackgroundOffsetY, self.overlay.image.height,
				self.settings.tutorialBackgroundImage.height);
		self.settings.tutorialBackgroundImage.drawSimple(0, x, y);
	}

	// draw tutorial title
	text = self.tutorialData[self.tutorialIdx].title;
	if (text !== undefined && text !== "") {
		font = f_default.clone();
		if (self.settings.tutorialTitleFont) {
			font.setProperties(self.settings.tutorialTitleFont);
		}
		maxFontSize = font.textFitSize(text, self.settings.tutorialTitleMaxWidth, self.settings.tutorialTitleMaxHeight, self.settings.tutorialTitleMaxWidth);
		if (font.getFontSize() > maxFontSize) {
			font.setFontSize(maxFontSize);
		}
		x = TG.functions.computeTextCoordinatesFromSettings(self.settings.tutorialTitleOffsetX, self.overlay.image.width, font.textWidth(text,
				self.settings.tutorialTitleMaxWidth), font.getAlign());
		y = TG.functions.computeTextCoordinatesFromSettings(self.settings.tutorialTitleOffsetY, self.overlay.image.height, font.textHeight(text,
				self.settings.tutorialTitleMaxHeight), font.getBaseline());
		font.drawSimple(text, x, y);

		if (testMode) {
			TG_MainController.drawTextRect(font, x, y, self.settings.tutorialTitleMaxWidth, self.settings.tutorialTitleMaxHeight);
		}
	}

	// draw tutorial data
	if (self.tutorialIdx < self.tutorialData.length) {
		image = self.tutorialData[self.tutorialIdx].image;
		x = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialImageOffsetX, self.overlay.image.width, image.width);
		y = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialImageOffsetY, self.overlay.image.height, image.height);
		image.drawSimple(0, x, y);
		text = self.tutorialData[self.tutorialIdx].text;
		font = f_default.clone();
		if (self.settings.tutorialFont) {
			font.setProperties(self.settings.tutorialFont);
		}
		maxFontSize = font.textFitSize(text, self.settings.tutorialTextWidth, self.settings.tutorialTextHeight, self.settings.tutorialTextWidth);
		if (font.getFontSize() > maxFontSize) {
			font.setFontSize(maxFontSize);
		}
		x = TG.functions.computeTextCoordinatesFromSettings(self.settings.tutorialTextOffsetX, self.overlay.image.width, font.textWidth(text,
				self.settings.tutorialTextWidth), font.getAlign());
		y = TG.functions.computeTextCoordinatesFromSettings(self.settings.tutorialTextOffsetY, self.overlay.image.height, font.textHeight(text,
				self.settings.tutorialTextWidth), font.getBaseline());
		font.drawSimple(text, x, y, self.settings.tutorialTextWidth);

		if (testMode) {
			TG_MainController.drawTextRect(font, x, y, self.settings.tutorialTextWidth, self.settings.tutorialTextHeight);
		}

	}
	self.overlay.image.resetTarget();

	// add buttons
	image = s_tutorialButton_previous;
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialButtonPreviousX, self.canvas.width, image.width) - self.overlay.x;
	y = TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialButtonPreviousY, self.canvas.height, image.height) - self.overlay.y
			- TG.canvasTopOffset;
	if (self.tutorialIdx - 1 >= 0) {
		self.buttons.push(new TG_ButtonBase(x, y, self.depth - 20, new tbMaskImage(image), [image], {
			callbackFunc: previous,
			callbackObj: self
		}, self.overlay));
	}
	else {
		image = s_tutorialButton_close;
		self.buttons.push(new TG_ButtonBase(x, y, self.depth - 20, new tbMaskImage(image), [image], {
			callbackFunc: close,
			callbackObj: self
		}, self.overlay));
	}

	image = s_tutorialButton_next;
	x = TG.functions.computeImgCoordinatesFromSettings(this.settings.tutorialButtonNextX, self.canvas.width, image.width) - self.overlay.x;
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.tutorialButtonNextY, self.canvas.height, image.height) - self.overlay.y
			- TG.canvasTopOffset;
	if (self.tutorialIdx + 1 < self.tutorialData.length) {
		self.buttons.push(new TG_ButtonBase(x, y, self.depth - 20, new tbMaskImage(image), [image], {
			callbackFunc: next,
			callbackObj: self
		}, self.overlay));
	}
	else {
		image = s_tutorialButton_close;
		self.buttons.push(new TG_ButtonBase(x, y, self.depth - 20, new tbMaskImage(image), [image], {
			callbackFunc: close,
			callbackObj: self
		}, self.overlay));
	}

	return true;
};

TG_Options.prototype.showAbout = function() {
	"use strict";
	var i, x, y, self = this, t;

	var backgroundX = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.settings.backgroundImage.width);
	var backgroundY = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height,
			self.settings.backgroundImage.height)
			+ TG.canvasTopOffset * -1;

	function drawText(text, fontDefinition, xOffset, yOffset, maxWidth, maxHeight) {
		var font, x, y, maxFontSize;
		font = f_default.clone();
		if (fontDefinition) {
			font.setProperties(fontDefinition);
		}
		maxFontSize = font.textFitSize(text, maxWidth, maxHeight, maxWidth);
		if (font.getFontSize() > maxFontSize) {
			font.setFontSize(maxFontSize);
		}
		x = TG.functions.computeTextCoordinatesFromSettings(xOffset, self.overlay.image.width, font.textWidth(text, maxWidth), font.getAlign());
		y = TG.functions.computeTextCoordinatesFromSettings(yOffset, self.overlay.image.height, font.textHeight(text, maxHeight), font.getBaseline());
		font.drawSimple(text, x, y, maxWidth);

		if (testMode) {
			TG_MainController.drawTextRect(font, x, y, maxWidth, maxHeight);
		}

		return y + maxHeight;
	}
	function drawSimpleImage(image, xOffset, yOffset) {
		var x, y;
		x = TG.functions.computeImgCoordinatesFromSettings(xOffset, self.overlay.image.width, image.width);
		y = TG.functions.computeImgCoordinatesFromSettings(yOffset, self.overlay.image.height, image.height);
		image.drawSimple(0, x, y);
		return y + image.height;
	}
	function close() {
		var i;
		// redraw background
		self.drawBackground(backgroundX, backgroundY);
		self.closeButton.enabled = true;
		// remove quit buttons
		for (i = 0; i < self.buttons.length; i++) {
			tbGame.removeToken(self.buttons[i]);
		}
		self.buttons = [];
		// add normal buttons
		self.addInitButtons(backgroundX, backgroundY, true);
		self.showingAbout = false;
	}

	TG.analytics.reportEvent("about", "options");
	// remove all buttons except close button
	for (i = 0; i < self.buttons.length; i++) {
		tbGame.removeToken(self.buttons[i]);
	}
	self.buttons = [];

	if (!this.challengeMode) {
		self.drawBackground(backgroundX, backgroundY);
	}
	else {
		self.overlay.image.setTarget();
		tbGraphics.clear();
		self.closeButton.enabled = false;
	}
	self.overlay.image.setTarget();

	// draw about background
	if (self.settings.tutorialBackgroundImage !== undefined) {
		drawSimpleImage(self.settings.tutorialBackgroundImage, self.settings.tutorialBackgroundOffsetX, self.settings.tutorialBackgroundOffsetY);
	}

	var aboutButtonSpec = null;

	if (typeof TG.hooks.drawAboutScreen === "function") {
		aboutButtonSpec = TG.hooks.drawAboutScreen(self.settings, drawText, drawSimpleImage, self.overlay.image);
	}
	else {
		t = TG.dictionary.getText("optionsAbout_header", "<OPTIONSABOUT_HEADER>");
		drawText(t, self.settings.aboutHeaderFont, self.settings.aboutHeaderOffsetX, self.settings.aboutHeaderOffsetY, self.settings.aboutHeaderMaxWidth,
				self.settings.aboutHeaderMaxHeight);

		// draw logo
		drawSimpleImage(s_logo_coolgames, self.settings.aboutLogoOffsetX, self.settings.aboutLogoOffsetY);

		// draw text
		t = TG.dictionary.getText("optionsAbout_text", "<OPTIONSABOUT_TEXT>");
		drawText(t, self.settings.aboutTextFont, self.settings.aboutTextOffsetX, self.settings.aboutTextOffsetY, self.settings.aboutTextMaxWidth,
				self.settings.aboutTextMaxHeight);
	}
	// draw version
	drawText(TG.dictionary.getText("optionsAbout_version", "<OPTIONSABOUT_VERSION>") + " " + TG.controller.getShortVersion()
			+ ((TG.size === "big") ? "b" : "s"), self.settings.aboutVersionFont, self.settings.aboutVersionOffsetX, self.settings.aboutVersionOffsetY,
			self.settings.aboutVersionMaxWidth, self.settings.aboutVersionMaxHeight);
	self.overlay.image.resetTarget();

	if (aboutButtonSpec) {
		var buttonSpecs = aboutButtonSpec;
		for (i = 0; i < buttonSpecs.length; ++i) {
			var buttonSpec = buttonSpecs[i];
			self.buttons.push(new TG_ButtonBase(buttonSpec.x, buttonSpec.y, self.depth - 10, tbMaskRectangle(0, 0, buttonSpec.width, buttonSpec.height), null,
					{
						callbackFunc: (function(url) {
							return function() {
								TG.hooks.openUrl(url);
							};
						})(buttonSpec.url),
						immediate: true
					}, self.overlay));
		}

	}
	else {
		// add invisible link button
		if (TG.config.homeUrl !== undefined) {

			var btn_x = TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutLogoOffsetX, self.overlay.image.width, s_logo_coolgames.width);
			var btn_y = TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutLogoOffsetY, self.overlay.image.height, s_logo_coolgames.height);

			btn_x = Math.min(btn_x, TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutTextOffsetX, self.overlay.image.width, self.settings.aboutTextMaxWidth));
			btn_y = Math.min(btn_y, TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutTextOffsetY, self.overlay.image.height, self.settings.aboutTextMaxHeight));

			var btn_w = Math.max(self.settings.aboutTextMaxWidth, s_logo_coolgames.width);
			var btn_h = TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutTextOffsetY, self.overlay.image.height, self.settings.aboutTextMaxHeight) + self.settings.aboutTextMaxHeight - btn_y;

			self.buttons.push(new TG_ButtonBase(btn_x, btn_y, self.depth - 10, tbMaskRectangle(0, 0, btn_w, btn_h), null, {
				callbackFunc: function() {
					TG.hooks.openUrl(TG.config.homeUrl);
				},
				immediate: true
			}, self.overlay));
		}
	}

	// add backbutton
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutBackBtnOffsetX, self.overlay.image.width, self.settings.aboutBackBtnWidth);
	y = self.settings.aboutBackBtnOffsetY;
	self.buttons.push(new TG_ButtonText("default_text", x, y, self.depth - 20, "optionsAbout_backBtn", self.settings.aboutBackBtnWidth, {
		callbackFunc: close,
		callbackObj: self
	}, self.overlay));

	this.showingAbout = true;
	return true;
};

TG_Options.prototype.showVersions = function() {
	"use strict";
	var i, x, y, self = this, t, f;
	var backgroundX = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.settings.backgroundImage.width);
	var backgroundY = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height,
			self.settings.backgroundImage.height)
			+ TG.canvasTopOffset * -1;
	function close() {
		var i;
		// redraw background
		self.drawBackground(backgroundX, backgroundY);
		// remove quit buttons
		for (i = 0; i < self.buttons.length; i++) {
			tbGame.removeToken(self.buttons[i]);
		}
		self.buttons = [];
		// add normal buttons
		self.addInitButtons(backgroundX, backgroundY, true);
	}

	TG.analytics.reportEvent("versions", "options");
	// remove all buttons except close button
	for (i = 0; i < self.buttons.length; i++) {
		tbGame.removeToken(self.buttons[i]);
	}
	self.buttons = [];

	self.drawBackground(backgroundX, backgroundY);
	self.overlay.image.setTarget();
	// draw about background
	if (self.settings.tutorialBackgroundImage !== undefined) {
		self.settings.tutorialBackgroundImage.drawSimple(0, TG.functions.computeImgCoordinatesFromSettings(self.settings.tutorialBackgroundOffsetX,
				self.overlay.width, self.settings.tutorialBackgroundImage.width), TG.functions.computeImgCoordinatesFromSettings(
				self.settings.tutorialBackgroundOffsetY, self.overlay.height, self.settings.tutorialBackgroundImage.height));
	}

	// draw text
	f = f_default.clone();
	f.setProperties(self.settings.aboutVersionFont);
	f.setAlign("left");
	x = self.settings.versionsOffsetX;
	y = self.settings.versionsOffsetY;
	for (i in TG.version) {
		t = i + ": " + TG.version[i];
		f.drawSimple(t, x, y);
		y += f.textHeight(t) + self.settings.versionsMarginY;
	}

	// add backbutton
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.aboutBackBtnOffsetX, self.overlay.image.width, self.settings.aboutBackBtnWidth);
	y = self.settings.aboutBackBtnOffsetY;
	self.buttons.push(new TG_ButtonText("default_text", x, y, self.depth - 20, "optionsAbout_backBtn", self.settings.aboutBackBtnWidth, {
		callbackFunc: close,
		callbackObj: self
	}, self.overlay));

	return true;
};

TG_Options.prototype.showHighScores = function() {
	"use strict";
	return true;
};

TG_Options.prototype.showMoreGames = function() {
	"use strict";
	TG.analytics.reportEvent("moreGames", "options");
	var moreGamesFunction = TG.controller.getMoreGamesFunction();
	if (typeof moreGamesFunction === "function") {
		moreGamesFunction();
	}
	return true;
};

TG_Options.prototype.quit = function() {
	"use strict";
	var quit, self = this;
	quit = function() {
		TG.analytics.reportEvent("confirm_yes", "options:quit");
		tbGame.removeToken(self);
		TG.controller._quitGame();
		return true;
	};
	this.showConfirmQuestion("optionsQuitConfirmationText", "optionsQuitConfirmBtn_Yes", "optionsQuitConfirmBtn_No", quit);
};

TG_Options.prototype.restart = function() {
	"use strict";
	var restart, self = this;
	restart = function() {
		TG.analytics.reportEvent("confirm_yes", "options:restart");
		tbGame.removeToken(self);
		TG.controller._restartGame();
		return true;
	};
	this.showConfirmQuestion("optionsRestartConfirmationText", "optionsQuitConfirmBtn_Yes", "optionsQuitConfirmBtn_No", restart);
};

TG_Options.prototype.forfeitChallenge = function() {
	"use strict";
	var callback, forfeit, self = this;
	callback = function(succes) {
		var key = succes ? "challengeForfeitMessage_success" : "challengeForfeitMessage_error";
		self.showMessage(TG.dictionary.getText(key, "<" + key.toUpperCase() + ">"));
		if (succes) {
			self.closeButton.enabled = false;
			if (!self.shouldPause) {
				TG.controller._pauseAll();
			}
		}
	};
	forfeit = function() {
		TG.controller.forfeitChallenge(callback);
		return true;
	};
	this.showConfirmQuestion("challengeForfeitConfirmText", "challengeForfeitConfirmBtn_yes", "challengeForfeitConfirmBtn_no", forfeit);
};

TG_Options.prototype.cancelChallenge = function() {
	"use strict";
	var callback, cancel, self = this;
	callback = function(succes) {
		var key = succes ? "challengeCancelMessage_success" : "challengeCancel_error";
		self.showMessage(TG.dictionary.getText(key, "<" + key.toUpperCase() + ">"));
		if (succes) {
			self.closeButton.enabled = false;
			if (!self.shouldPause) {
				TG.controller._pauseAll();
			}
		}
	};
	cancel = function() {
		TG.controller.cancelChallenge(callback);
		return true;
	};
	this.showConfirmQuestion("challengeCancelConfirmText", "challengeCancelConfirmBtn_yes", "challengeCancelConfirmBtn_no", cancel);
};

TG_Options.prototype.showConfirmQuestion = function(questionTextKey, confirmTextKey, cancelTextKey, confirmCallback, cancelCallback) {
	"use strict";
	var i, text, font, maxFontSize, x, y, self = this, w, h, yesCallback, noCallback;
	yesCallback = function() {
		if (typeof confirmCallback === "function") {
			return confirmCallback();
		}
		return true;
	};
	noCallback = function() {
		var i, x, y;
		x = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.settings.backgroundImage.width);
		y = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height, self.settings.backgroundImage.height)
				+ TG.canvasTopOffset * -1;
		// redraw background
		self.drawBackground(x, y);
		// remove quit buttons
		for (i = 0; i < self.buttons.length; i++) {
			tbGame.removeToken(self.buttons[i]);
		}
		self.buttons = [];
		// add normal buttons
		self.addInitButtons(x, y, true);
		if (typeof cancelCallback === "function") {
			return cancelCallback();
		}
		return true;
	};

	// remove all buttons except close button
	for (i = 0; i < self.buttons.length; i++) {
		tbGame.removeToken(self.buttons[i]);
	}
	self.buttons = [];

	// draw confirm text
	text = TG.dictionary.getText(questionTextKey, "<" + questionTextKey.toUpperCase() + ">");
	font = f_default.clone();
	if (self.settings.confirmQuestionFont) {
		font.setProperties(self.settings.confirmQuestionFont);
	}
	else if (self.settings.quitConfirmFont) {
		font.setProperties(self.settings.quitConfirmFont);
	}
	maxFontSize = font.textFitSize(text, self.settings.confirmQuestionTextWidth, self.settings.confirmQuestionTextHeight, true);
	if (maxFontSize < font.getFontSize()) {
		font.setFontSize(maxFontSize);
	}
	w = font.textWidth(text, self.settings.confirmQuestionTextWidth) + 10;
	h = font.textHeight(text, self.settings.confirmQuestionTextWidth) + 10;
	x = TG.functions.computeTextCoordinatesFromSettings(self.settings.confirmQuestionTextX, self.overlay.image.width, w, font.getAlign());
	y = TG.functions.computeTextCoordinatesFromSettings(self.settings.confirmQuestionTextY, self.overlay.image.height, h, font.getBaseline());
	self.overlay.image.setTarget();
	font.drawSimple(text, x, y, w);
	if (testMode) {
		TG_MainController.drawTextRect(font, x, y, self.settings.confirmQuestionTextWidth, self.settings.confirmQuestionTextHeight);
	}
	self.overlay.image.resetTarget();
	// add buttons
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.confirmQuestionBtnNoX, self.canvas.width, self.settings.confirmQuestionBtnsWidth)
			- self.overlay.x;
	y = TG.functions.computeImgCoordinatesFromSettings(self.settings.confirmQuestionBtnNoY, self.canvas.height,
			this.buttonSettings["default_text"].sprite.height)
			- self.overlay.y - TG.canvasTopOffset;
	this.buttons.push(new TG_ButtonText("default_text", x, y, this.depth - 20, cancelTextKey, self.settings.confirmQuestionBtnsWidth, {
		callbackFunc: noCallback,
		callbackObj: self
	}, self.overlay));
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.confirmQuestionBtnYesX, self.canvas.width, self.settings.confirmQuestionBtnsWidth)
			- self.overlay.x;
	y = TG.functions.computeImgCoordinatesFromSettings(self.settings.confirmQuestionBtnYesY, self.canvas.height,
			this.buttonSettings["default_text"].sprite.height)
			- self.overlay.y - TG.canvasTopOffset;
	this.buttons.push(new TG_ButtonText("default_text", x, y, this.depth - 20, confirmTextKey, self.settings.confirmQuestionBtnsWidth, {
		callbackFunc: yesCallback,
		callbackObj: self
	}, self.overlay));
	return true;
};

TG_Options.prototype.showMessage = function(text) {
	"use strict";
	var i, font, maxFontSize, x, y, self = this, w, h;

	// remove all buttons except close button
	for (i = 0; i < self.buttons.length; i++) {
		tbGame.removeToken(self.buttons[i]);
	}
	self.buttons = [];

	// redraw background
	x = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetX, self.canvas.width, self.settings.backgroundImage.width);
	y = TG.functions.computeImgCoordinatesFromSettings(self.settings.overlayImageOffsetY, self.canvas.height, self.settings.backgroundImage.height)
			+ TG.canvasTopOffset * -1;
	// redraw background
	self.drawBackground(x, y);

	// draw message text
	font = f_default.clone();
	if (self.settings.messageFont) {
		font.setProperties(self.settings.messageFont);
	}
	maxFontSize = font.textFitSize(text, self.settings.messageTextWidth, self.settings.messageTextHeight, true);
	if (maxFontSize < font.getFontSize()) {
		font.setFontSize(maxFontSize);
	}
	w = font.textWidth(text, self.settings.messageTextWidth) + 10;
	h = font.textHeight(text, self.settings.messageTextWidth) + 10;
	x = TG.functions.computeTextCoordinatesFromSettings(self.settings.messageTextX, self.overlay.image.width, w, font.getAlign());
	y = TG.functions.computeTextCoordinatesFromSettings(self.settings.messageTextY, self.overlay.image.height, h, font.getBaseline());
	self.overlay.image.setTarget();
	font.drawSimple(text, x, y, w);
	if (testMode) {
		TG_MainController.drawTextRect(font, x, y, self.settings.messageTextWidth, self.settings.messageTextHeight);
	}
	self.overlay.image.resetTarget();
};

TG_Options.prototype.showStartScreen = function() {
	"use strict";
	TG.analytics.reportEvent("startScreen", "options");
	TG.controller._showStartScreen();
	return true;
};

TG_Options.prototype.close = function() {
	"use strict";
	tbGame.removeToken(this);
	this.canvas.drawTokens = true;
	return true;
};

// handleAdd is called when the token is (re-) added to the game.
TG_Options.prototype.handleAdd = function() {
	"use strict";
	var y, activateButtons, image, self;
	// pause all tokens
	if (this.shouldPause) {
		TG.controller._pauseAll(this);
	}
	TG.controller.activeModelDialog = this;
	this.fireLanguageEvent = false;
	this.fireResizeEvent = false;

	// create overlay surface
	image = this.settings.backgroundImage;
	this.overlay = new TG_AnimatedImage(this.depth - 10, this.layer, new tbSurface(image.width, image.height));
	this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetX, this.canvas.width, image.width);
	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetY, this.canvas.height, image.height) + TG.canvasTopOffset * -1;
	this.overlay.y = y;
	// draw background
	this.drawBackground(this.overlay.x, this.overlay.y);

	this.buttons = [];
	if (this.tutorialVisibleOnStart) {
		this.showTutorial();
	}
	else {
		// add buttons
		this.addInitButtons(this.overlay.x, this.overlay.y);
	}
	// add close button
	this.closeButton = new TG_ButtonBase(this.settings.closeButtonX, this.settings.closeButtonY, this.depth - 20, new tbMaskRectangle(0, 0,
			this.settings.closeButtonMaskWidth, this.settings.closeButtonMaskHeight), undefined, {
		callbackFunc: this.close,
		callbackObj: this
	}, this.overlay);

	// add keyAreas
	this.versionsString = "versions";
	this.area = new tbKeyArea();
	TG.functions.setLayerAndCanvas(this.area, TG.frameworkUILayer);
	tbGame.setTokenDepth(this.area, this.depth - 1);
	this.area.addRectangle("keyAreaLeft", this.overlay.x, this.overlay.y + this.settings.tutorialBackgroundOffsetY, this.settings.aboutVersionKeyAreaWidth,
			this.settings.aboutVersionKeyAreaHeight, tbKeyCodes.vkL);
	this.area.addRectangle("keyAreaRight", this.overlay.x + this.overlay.width - this.settings.aboutVersionKeyAreaWidth, this.overlay.y
			+ this.settings.tutorialBackgroundOffsetY, this.settings.aboutVersionKeyAreaWidth, this.settings.aboutVersionKeyAreaHeight, tbKeyCodes.vkR);
	this.area.addRectangle("keyAreaCentre", TG.mainWidth / 2 - this.settings.aboutVersionKeyAreaWidth / 2, this.overlay.y
			+ this.settings.tutorialBackgroundOffsetY, this.settings.aboutVersionKeyAreaWidth, this.settings.aboutVersionKeyAreaHeight, tbKeyCodes.vkC);

	// start animation
	self = this;
	activateButtons = function() {
		var i;
		for (i = 0; i < self.buttons.length; i++) {
			self.buttons[i].enabled = true;
		}
	};
	if (this.type !== "inGame") {
		// slide in from bottom
		this.overlay.y = this.canvas.height;
	}
	else {
		// slide in from top
		this.overlay.y = -this.overlay.image.height;
	}
	this.overlay.addAnimator("y", y, this.settings.overlayAnimInDuration, this.settings.overlayAnimInEase, activateButtons);
};

// handleRemove is called when the token is removed from the game.
TG_Options.prototype.handleRemove = function() {
	"use strict";
	var i;
	if (this.shouldPause) {
		TG.controller._resumeAll();
	}
	if (this.fireLanguageEvent) {
		tbGame.addSpecial(TG.languageSetEventId, TG.dictionary.getLanguage());
	}
	if (this.fireResizeEvent) {
		tbGame.addSpecial(TG.resizeEventId);
	}
	for (i = 0; i < this.buttons.length; i++) {
		tbGame.removeToken(this.buttons[i]);
	}
	this.area.clear();
	tbGame.removeToken(this.area);
	tbGame.removeToken(this.closeButton);
	tbGame.removeToken(this.overlay);
	TG.controller.activeModelDialog = null;
};

TG_Options.prototype.handleTouchUp = function() {
	"use strict";
	return true;
};

TG_Options.prototype.handleTouchDown = function() {
	"use strict";
	return true;
};

TG_Options.prototype.handleKeyUp = function(keyCode) {
	"use strict";
	if (this.showingAbout) {
		if (keyCode === tbKeyCodes.vkC) {
			this.versionsString = "";
		}
		else if (keyCode === tbKeyCodes.vkL) {
			this.versionsString += "l";
		}
		else if (keyCode === tbKeyCodes.vkR) {
			this.versionsString += "r";
		}
		if (this.versionsString === "lrl") {
			this.showVersions();
		}
	}
};

TG_Options.prototype.handleSpecial = function(id) {
	"use strict";
	if (id === TG.languageSetEventId) {
		this.drawBackground(this.overlay.x, this.overlay.y);
		this.fireLanguageEvent = true;
	}
	else if (id === TG.resizeEventId) {
		this.fireResizeEvent = true;
	}
	else if (id === TG.backButtonEventId) {
		this.close();
	}
};

//******************************************************
//*****    Token Resource: TG_GameEndScreen
//******************************************************

// ***************************************
// Definition of game token TG_GameEndScreen
// ***************************************

// The constructor for the TG_GameEndScreen game token.
var TG_GameEndScreen = function(){
	"use strict";
	// Properties used by the engine
	this.depth = -200;
	this.visible = true;
	this.active = true;
	TG.functions.setLayerAndCanvas(this, TG.frameworkBackgroundLayer);
	// Your own properties
	var setting;
	this.settings = TG.settings.tg.gameEndScreen;
	if(TG.orientation === "landscape" && TG.settings.tg.gameEndScreen_landscape){
		for(setting in TG.settings.tg.gameEndScreen_landscape){
			this.settings[setting] = TG.settings.tg.gameEndScreen_landscape[setting];
		}
	}
	this.buttonSettings = TG.settings.tg.buttonDefinitions;
	// apply theme specific setting overwrites
	for(setting in TG.settings.theme.gameEndScreen){
		this.settings[setting] = TG.settings.theme.gameEndScreen[setting];
	}
	// Add it to the game engine
	tbGame.addToken(this);
};

TG_GameEndScreen.prototype.drawBackground = function(){
    "use strict";
    var x, y, font, text, maxFontSize,
        top = Math.abs(TG.canvasTopOffset),
        image = this.settings.backgroundImage,
        scale = (TG.mainHeight - top)/image.combinedHeight;
    this.overlay.image = new tbSurface(scale*image.combinedWidth, scale*image.combinedHeight);
    this.overlay.image.setTarget();
    this.overlay.y = Math.abs(TG.canvasTopOffset);
    image.drawCombinedExt(0,0, scale, scale, 0, 1);
    font = f_default.clone();
    font.setProperties(this.settings.font);
    text = TG.dictionary.getText("gameEndScreenTitle", "<GAMEENDSCREENTITLE>");
	maxFontSize = font.textFitSize(text, this.settings.textMaxWidth - (font.getStroke()?font.getStrokeWidth():0), this.settings.textMaxHeight - (font.getStroke()?font.getStrokeWidth():0), true);
	if(maxFontSize < font.getFontSize()){
		font.setFontSize(maxFontSize);
	}
    x = TG.functions.computeTextCoordinatesFromSettings(this.settings.textX, this.canvas.width, font.textWidth(text), font.getAlign());
    y = TG.functions.computeTextCoordinatesFromSettings(this.settings.textY, this.canvas.height, font.textHeight(text), font.getBaseline());
    font.drawSimple(text, x, y, this.settings.textMaxWidth);
    this.overlay.image.resetTarget();
    this.overlay.canvas.drawTokens = true;
};

// handleAdd is called when the token is (re-) added to the game.
TG_GameEndScreen.prototype.handleAdd = function() {
	"use strict";
    var x, y, callback, self = this,
        image = this.settings.backgroundImage,
        surface = new tbSurface(image.width, image.height);
    //create screen
    this.overlay = new TG_AnimatedImage(this.depth, TG.frameworkUILayer, surface);
    this.overlay.x = 0;
    this.overlay.y = Math.abs(TG.canvasTopOffset);
    this.drawBackground();

    //add button
    callback = function(){
        tbGame.removeToken(self);
        TG.controller._showStartScreen();
    };
    x = TG.functions.computeImgCoordinatesFromSettings(this.settings.btnX, this.canvas.width, this.settings.btnWidth);
    y = TG.functions.computeImgCoordinatesFromSettings(this.settings.btnY, this.canvas.height, this.buttonSettings[this.settings.btnType].sprite.height);
    this.button = new TG_ButtonText(this.settings.btnType, x, y, this.depth - 10, "gameEndScreenBtnText", this.settings.btnWidth, callback, this.overlay);

};

// handleRemove is called when the token is removed from the game.
TG_GameEndScreen.prototype.handleRemove = function() {
	"use strict";
    tbGame.removeToken(this.overlay);
    tbGame.removeToken(this.button);
};

TG_GameEndScreen.prototype.handleSpecial = function(id) {
    "use strict";
    if(id === TG.languageSetEventId || id === TG.resizeEventId){
        this.drawBackground();
    }
};
//******************************************************
//*****    Token Resource: TG_ButtonBase
//******************************************************

// ***************************************
// Definition of game token TG_ButtonBase
// ***************************************

// The constructor for the TG_ButtonBase game token.
var TG_ButtonBase = function(x, y, depth, mask, images, callback, parent){
	"use strict";
	// Properties used by the engine
	this.depth = depth;
	this.visible = true;
	this.active = true;
	this.group = "TG_Token";
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	this.originX = 0;
	this.originY = 0;
	this.x = x;
	this.y = y;
	this.width = (images)? images[0].width : (mask.bBoxRight - mask.bBoxLeft);
	this.height = (images)? images[0].height : (mask.bBoxBottom - mask.bBoxTop);
	this.xScale = 1;
	this.yScale = 1;
	this.alpha = 1;
	this.angle = 0;
	this.mask = mask;
	this.images = images;
	this.imageIdx = 0;
	this.subImageIdx = 0;
	this.pressed = false;
	this.enabled = true;
	this.parent = (parent !== undefined)? parent : {
	    x: 0,
	    y: 0
	};
	this.scaleOffsetX = 0;
	this.scaleOffsetY = 0;
	this.forceDraw = true;

	// Set-up button click handling
	this.onClick = function(){
	};
	this.immediate = false;
	if(typeof callback === "object"){
		this.onClick = callback.callbackFunc;
		this.callbackObj = callback.callbackObj;
		this.immediate = callback.immediate;
	}else if(typeof callback === "function"){
		this.onClick = callback;
	}

	var self = this, handleDown = function(finger, touchX, touchY){
		var x, y, canvasOffset;
		canvasOffset = TG.functions.getCanvasMainOffset(self.canvas);
		x = Math.round(self.x + self.parent.x - self.originX * self.xScale);
		y = Math.round(self.y + self.parent.y - self.originY * self.yScale);
		if(self.enabled && self.mask.collisionPoint(x, y, touchX - canvasOffset.x, touchY - canvasOffset.y)){
			self.pressed = true;
			if(self.images){
				if(self.images.length > 1){
					self.imageIdx = 1;
					self.canvas.drawTokens = true;
				}else if(self.images[0].number > 1){
					self.subImageIdx = 1;
					self.canvas.drawTokens = true;
				}
			}
			if(typeof a_mouseDown !== undefined){
				tbAudio.play(a_mouseDown);
			}
			self.finger = finger;
			return true;
		}
	}, handleUp = function(finger, touchX, touchY){
		var x, y, canvasOffset;
		canvasOffset = TG.functions.getCanvasMainOffset(self.canvas);
		x = Math.round(self.x + self.parent.x - self.originX * self.xScale);
		y = Math.round(self.y + self.parent.y - self.originY * self.yScale);
		if(self.images && self.imageIdx > 0 || self.subImageIdx > 0){
			self.imageIdx = 0;
			self.subImageIdx = 0;
			self.canvas.drawTokens = true;
		}
		if(self.pressed && self.enabled && self.mask.collisionPoint(x, y, touchX - canvasOffset.x, touchY - canvasOffset.y)){
			self.pressed = false;
			if(self.callbackObj !== undefined){
				return self.onClick.call(self.callbackObj, self);
			}
			return self.onClick(self);
		}
	};

	if(this.immediate){
		this.handleImmediateTouchDown = handleDown;
		this.handleImmediateTouchUp = handleUp;
	}else{
		this.handleTouchDown = handleDown;
		this.handleTouchUp = handleUp;
	}

	// Add it to the game engine
	tbGame.addToken(this);
};

TG_ButtonBase.prototype.addAnimation = function(type, ease, start, change, duration){
	"use strict";
	if(this.animations === undefined){
		this.animations = [];
	}
	this.animations.push({
	    type: type,
	    start: start,
	    change: change,
	    ease: ease,
	    duration: duration,
	    timer: 0
	});
};

TG_ButtonBase.prototype.endAnimations = function(forceToEndValue){
	"use strict";
	var i, animation;
	if(this.animations !== undefined){
		for(i = 0; i < this.animations.length; i++){
			animation = this.animations[i];
			if(animation.active){
				if(forceToEndValue){
					switch(animation.type){
						case "xScale":
							this.xScale = animation.start + animation.change;
						break;
						case "yScale":
							this.yScale = animation.start + animation.change;
						break;
						case "alpha":
							this.alpha = animation.start + animation.change;
						break;
						case "angle":
							this.angle = animation.start + animation.change;
						break;
						case "x":
							this.x = animation.start + animation.change;
						break;
						case "y":
							this.y = animation.start + animation.change;
						break;
					}
				}
				animation.active = false;
			}
		}
		this.canvas.drawTokens = true;
	}
};

TG_ButtonBase.prototype.setImages = function(images){
	"use strict";
	this.images = images;
	this.canvas.drawTokens = true;
};

TG_ButtonBase.prototype.setPosition = function(x, y){
	"use strict";
	this.x = x;
	this.y = y;
	if(this.images){
		this.canvas.drawTokens = true;
	}
};

TG_ButtonBase.prototype.setVisible = function(visible){
	"use strict";
	this.active = visible;
	this.visible = visible;
};

// The following event handlers describe the behavior of the token

// handleRemove is called when the token is removed from the game.
TG_ButtonBase.prototype.handleRemove = function(){
	"use strict";
	if(this.images){
		this.canvas.drawTokens = true;
	}
};

//handleBeginStep is called each step before processing other events
TG_ButtonBase.prototype.handleBeginStep = function(timePassed){
	"use strict";
	var i, animation;
	if(this.animations !== undefined){
		for(i = 0; i < this.animations.length; i++){
			animation = this.animations[i];
			animation.timer += timePassed;
			switch(animation.type){
				case "xScale":

					var oldScaleX = this.xScale;
					var scaleOffsetX = this.scaleOffsetX;

					this.xScale = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
					this.scaleOffsetX = -(this.images[0].width * this.xScale - this.images[0].width * animation.start) / 2;

					if (isNaN(this.xScale) || isNaN(this.scaleOffsetX)) {
						this.xScale = oldScaleX;
						this.scaleOffsetX = scaleOffsetX;
					}

				break;
				case "yScale":
					var oldScaleY = this.yScale;
					var scaleOffsetY = this.scaleOffsetY;

					this.yScale = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
					this.scaleOffsetY = -(this.images[0].height * this.yScale - this.images[0].height * animation.start) / 2;

					if (isNaN(this.yScale) || isNaN(this.scaleOffsetY)) {
						this.yScale = oldScaleY;
						this.scaleOffsetY = scaleOffsetY;
					}

				break;
				case "alpha":
					this.alpha = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
				break;
				case "angle":
					this.angle = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
				break;
				case "x":
					var oldX = this.x;
					this.x = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
					if (isNaN(this.x)) {
						this.x = oldX;
					}
				break;
				case "y":
					var oldY = this.y;
					this.y = animation.ease(animation.timer, animation.start, animation.change, animation.duration);
					if (isNaN(this.y)) {
						this.y = oldY;
					}
				break;
			}
		}
		this.canvas.drawTokens = true;
	}
};

// handleEndStep is called each step after processing other events (before
// draw).
TG_ButtonBase.prototype.handleEndStep = function(){
	"use strict";
	var x, y, canvasOffset;
	canvasOffset = TG.functions.getCanvasMainOffset(this.canvas);
	x = Math.round(this.x + this.parent.x - this.originX * this.xScale);
	y = Math.round(this.y + this.parent.y - this.originY * this.yScale);
	// reset button if no longer pressed
	if(this.pressed && !this.mask.collisionPoint(x, y, tbGame.touchX(this.finger) - canvasOffset.x, tbGame.touchY(this.finger) - canvasOffset.y)){
		if(this.images){
			this.imageIdx = 0;
			this.subImageIdx = 0;
			this.canvas.drawTokens = true;
		}
		this.pressed = false;
	}
};

// handleDraw is called at the end of a step to draw the token.
TG_ButtonBase.prototype.handleDraw = function(){
	"use strict";
	var x, y;
	x = Math.round(this.x + this.parent.x - this.originX * this.xScale);
	y = Math.round(this.y + this.parent.y - this.originY * this.yScale);
	if(this.images){
		if(this.images[this.imageIdx] instanceof tbSurface){
			this.images[this.imageIdx].drawExt(x, y, this.xScale, this.yScale, this.angle, this.alpha);
		}else{
			this.images[this.imageIdx].drawExt(this.subImageIdx, x, y, this.xScale, this.yScale, this.angle, this.alpha);
		}
	}

	if (testMode && TG_MainController.drawDebugInfo) {
		this.mask.drawBox(x, y, "red");
	}

	this.forceDraw = false;
};



//******************************************************
//*****    Token Resource: TG_ButtonText
//******************************************************

// ***************************************
// Definition of game token TG_ButtonText
// ***************************************

// The constructor for the TG_ButtonText game token.
function TG_ButtonText(btnType, x, y, depth, textKey, width, onClick, parent) {
	"use strict";
	var s, m, buttonColor;

	// define fonts and images
    this.buttonType = TG.settings.tg.buttonDefinitions[btnType];
    buttonColor = (TG.settings.theme.buttons !== undefined)? TG.settings.tg.buttonColorDefinitions[TG.settings.theme.buttons[btnType] || TG.settings.theme.buttons["default_color"]] : TG.settings.tg.buttonColorDefinitions[TG.settings.tg.buttons["default_color"]];
    this.font = f_default.clone();
    if(buttonColor.font) {
        this.font.setProperties(buttonColor.font);
    }
    if(this.buttonType.fontSize){
        this.font.setFontSize(this.buttonType.fontSize);
    }
    this.textKey = textKey;
	this.text = TG.dictionary.getText(this.textKey,"<"+textKey.toUpperCase()+">");
	if (width !== undefined) {
		this.width = width;
	}
	this.height = this.buttonType.sprite.height;
    this.image = {source: this.buttonType.sprite, leftWidth: this.buttonType.leftWidth, rightWidth: this.buttonType.rightWidth};

	s = this.createBtnSurfaces(this.image);
	m = new tbMaskRectangle(0, 0, s[0].width, s[0].height);
	TG_ButtonBase.call(this, x, y, depth, m, s, onClick, parent);
}

TG.functions.inherit(TG_ButtonText, TG_ButtonBase);

TG_ButtonText.prototype.redrawText = function(fontProperties){
    "use strict";
    this.text = TG.dictionary.getText(this.textKey,"<"+this.textKey.toUpperCase()+">");
    if(fontProperties) {
        this.font.setProperties(fontProperties);
    }
    this.setImages(this.createBtnSurfaces(this.image));
};

TG_ButtonText.prototype.updateText = function(textKey, fontProperties){
    "use strict";
    this.textKey = textKey;
    this.redrawText(fontProperties);
};

TG_ButtonText.prototype.updateBtnVisuals = function(image, text, fontProperties) {
	"use strict";
    if(typeof text === "string"){
        this.text = text;
    }
	if(fontProperties){
        this.font.setProperties(fontProperties);
    }
    if(image instanceof tbImage){
        this.image.source = image;
    }else if(image.leftWidth !== undefined && image.rightWidth !== undefined && image.source !== undefined){
        this.image = image;
    }
	this.setImages(this.createBtnSurfaces(this.image));
};

TG_ButtonText.prototype.createBtnSurfaces = function(image) {
	"use strict";
	var middleWidth, txtWidth, txtAlign, maxFontSize, textXOffset, textYOffset, i, surfaces, s,
		sideWidth = image.leftWidth + image.rightWidth,
		middleHeight = this.height - (this.buttonType.textPaddingY || 0),
		sprite = image.source;

	txtWidth = this.font.textWidth(this.text);

	// determine middleWidth and max font size
	if (this.width === undefined) {
		middleWidth = txtWidth;
	}
	else if (typeof this.width === "number") {
		middleWidth = this.width - sideWidth;
	}
	else if (typeof this.width === "object") {
		if (this.width.width !== undefined) {
			middleWidth = this.width.width - sideWidth;
		}
		else {
			if (this.width.minWidth !== undefined) {
				middleWidth = Math.max(this.width.minWidth - sideWidth, txtWidth);
			}
			if (this.width.maxWidth !== undefined) {
				middleWidth = Math.min(this.width.maxWidth - sideWidth, txtWidth);
			}
		}
	}
	maxFontSize = this.font.textFitSize(this.text, middleWidth, middleHeight, true);
	if(maxFontSize < this.buttonType.fontSize){
		this.font.setFontSize(maxFontSize);
	}
	else {
		this.font.setFontSize(this.buttonType.fontSize);
	}

	//align font
	textXOffset = image.leftWidth;
    txtAlign = this.font.getAlign();
	if(txtAlign === "center") {
		textXOffset += Math.round(middleWidth/2);
	}else if(txtAlign === "right"){
		textXOffset += middleWidth;
	}
	textYOffset = Math.round(this.height/2);
    if(this.buttonType.textOffsetY !== undefined){
        textYOffset += this.buttonType.textOffsetY;
    }

	// draw surfaces
	surfaces = [];
	for(i=0; i<sprite.number; i++){
		s = new tbSurface(middleWidth + sideWidth, this.height);
		s.setTarget();

		// draw images
		sprite.drawPart(i, 0, 0, image.leftWidth, this.height, 0, 0, 1);
		sprite.drawPartStretched(i, image.leftWidth, 0, sprite.width - sideWidth, this.height, image.leftWidth, 0, middleWidth, this.height, 1);
		sprite.drawPart(i, image.leftWidth + sprite.width - sideWidth, 0, image.rightWidth, this.height, image.leftWidth + middleWidth, 0, 1);

		// drawText
		this.font.drawSimple(this.text, textXOffset, textYOffset, middleWidth);

		s.resetTarget();
		surfaces.push(s);
	}
	return surfaces;
};

TG_ButtonText.prototype.handleSpecial = function(id){
    "use strict";
    if(id === TG.languageSetEventId){
        this.redrawText();
    }
};
//******************************************************
//*****    Token Resource: TG_ButtonImage
//******************************************************

// ***************************************
// Definition of game token TG_ButtonImage
// ***************************************

// The constructor for the TG_ButtonImage game token.
function TG_ButtonImage(btnType, x, y, depth, img, width, onClick, parent) {
	"use strict";
	var s, m;

	// define fonts and images
    this.buttonType = TG.settings.tg.buttonDefinitions[btnType];
    if (width !== undefined) {
		this.width = width;
	}
	this.height = this.buttonType.sprite.height;
    this.buttonImage = {source: this.buttonType.sprite, leftWidth: this.buttonType.leftWidth, rightWidth: this.buttonType.rightWidth};
    this.image = img;
    
	s = this.createBtnSurfaces();
	m = new tbMaskRectangle(0, 0, s[0].width, s[0].height);
	TG_ButtonBase.call(this, x, y, depth, m, s, onClick, parent);
}

TG.functions.inherit(TG_ButtonImage, TG_ButtonBase);

TG_ButtonImage.prototype.createBtnSurfaces = function() {
	"use strict";
	var middleWidth, i, surfaces, s, imageXOffset, imageYOffset, imageScale,
		sideWidth = this.buttonImage.leftWidth + this.buttonImage.rightWidth,
		middleHeight = this.height - (this.buttonType.textPaddingY || 0),
		sprite = this.buttonImage.source;

	// determine middleWidth and max font size
	if (this.width === undefined) {
		middleWidth = this.image.width;
	}
	else if (typeof this.width === "number") {
		middleWidth = this.width - sideWidth;
	}
	else if (typeof this.width === "object") {
		if (this.width.width !== undefined) {
			middleWidth = this.width.width - sideWidth;
		}
		else {
			if (this.width.minWidth !== undefined) {
				middleWidth = Math.max(this.width.minWidth - sideWidth, this.image.width);
			}
			if (this.width.maxWidth !== undefined) {
				middleWidth = Math.min(this.width.maxWidth - sideWidth, this.image.width);
			}
		}
	}
	imageScale = Math.min(middleWidth/this.image.width, middleHeight/this.image.height);
	imageScale = Math.min(imageScale, 1);
	
	//align image
	imageXOffset = Math.round(this.buttonImage.leftWidth + ((middleWidth - this.image.width * imageScale)/2));
	imageYOffset = Math.round((middleHeight - this.image.height * imageScale)/2);

	// draw surfaces
	surfaces = [];
	for(i=0; i<sprite.number; i++){
		s = new tbSurface(middleWidth + sideWidth, this.height);
		s.setTarget();

		// draw button images
		sprite.drawPart(i, 0, 0, this.buttonImage.leftWidth, this.height, 0, 0, 1);
		sprite.drawPartStretched(i, this.buttonImage.leftWidth, 0, sprite.width - sideWidth, this.height, this.buttonImage.leftWidth, 0, middleWidth, this.height, 1);
		sprite.drawPart(i, this.buttonImage.leftWidth + sprite.width - sideWidth, 0, this.buttonImage.rightWidth, this.height, this.buttonImage.leftWidth + middleWidth, 0, 1);

		// draw image
		try{
		    tbGraphics.context.drawImage(this.image, imageXOffset, imageYOffset, this.image.width*imageScale, this.image.height*imageScale);
		}catch(e){}

		s.resetTarget();
		surfaces.push(s);
	}
	return surfaces;
};
//******************************************************
//*****    Token Resource: TG_ButtonCustomImage
//******************************************************

// ***************************************
// Definition of game token TG_ButtonImage
// ***************************************

// The constructor for the TG_ButtonImage game token.
function TG_ButtonCustomImage(x, y, depth, img, mask, onClick, parent) {
	"use strict";
	TG_ButtonBase.call(this, x, y, depth, mask, img, onClick, parent);
}

TG.functions.inherit(TG_ButtonCustomImage, TG_ButtonBase);
//******************************************************
//*****    Token Resource: TG_ButtonToggleText
//******************************************************

// ***************************************
// Definition of game token TG_ButtonToggleText
// ***************************************

// The constructor for the TG_ButtonToggleText game token.
function TG_ButtonToggleText(btnType, x, y, depth, state, width, onClick, parent) {
	"use strict";
	// define fonts and images
    var toggleAction, self, s, m, i, buttonColor;
    this.buttonType = TG.settings.tg.buttonDefinitions[btnType];
    buttonColor = (TG.settings.theme.buttons !== undefined)? TG.settings.tg.buttonColorDefinitions[TG.settings.theme.buttons[btnType] || TG.settings.theme.buttons["default_color"]] : TG.settings.tg.buttonColorDefinitions[TG.settings.tg.buttons["default_color"]];
    this.font = f_default.clone();
    if(buttonColor.font) {
        this.font.setProperties(buttonColor.font);
    }
    if(this.buttonType.fontSize){
        this.font.setFontSize(this.buttonType.fontSize);
    }
    
	if (width !== undefined) {
		this.width = width;
	}
	this.height = this.buttonType.sprite.height;
    this.states = this.buttonType.states;
    if(!this.states.length) {
        return;
    }
    for(i=0; i<this.states.length; i++){
        if(this.states[i].id === state){
            this.currentStateIdx = i;
            break;
        }
    }
    if(this.currentStateIdx === undefined){
        this.currentStateIdx = 0;
    }

    this.text = TG.dictionary.getText(this.states[this.currentStateIdx].textKey, "<"+this.states[this.currentStateIdx].id.toUpperCase()+">");
    this.toggleImage = this.states[this.currentStateIdx].sprite;

    s = this.createBtnSurfaces();
	m = new tbMaskRectangle(0, 0, s[0].width, s[0].height);

    self = this;
    if(typeof onClick === "function"){
        toggleAction = function(){
            self.setState();
            return onClick(self.states[self.currentStateIdx].id);
        };
    }else if (typeof onClick === "object"){
        toggleAction = {};
        toggleAction.immediate = onClick.immediate;
        toggleAction.callbackObj = this;
        toggleAction.callbackFunc = function(){
            self.setState();
            return onClick.callbackFunc.call(onClick.callbackObj, self.states[self.currentStateIdx].id);
        };
    }else{
        toggleAction = function(){
            self.setState();
        };
    }

    TG_ButtonBase.call(this, x, y, depth, m, s, toggleAction, parent);
}

TG.functions.inherit(TG_ButtonToggleText, TG_ButtonBase);

TG_ButtonToggleText.prototype.setState = function(state){
    "use strict";
    var i;
    if(state === undefined){
        this.currentStateIdx = (this.currentStateIdx + 1)%this.states.length;
    }else{
        for(i=0; i<this.states.length; i++){
            if(this.states[i].id === state){
                this.currentStateIdx = i;
                break;
            }
        }
    }
    this.updateBtnVisuals(this.states[this.currentStateIdx].sprite, TG.dictionary.getText(this.states[this.currentStateIdx].textKey, "<"+this.states[this.currentStateIdx].id.toUpperCase()+">"));
};

TG_ButtonToggleText.prototype.redrawText = function(fontProperties){
    "use strict";
    if(fontProperties) {
        this.font.setProperties(fontProperties);
    }
    this.text = TG.dictionary.getText(this.states[this.currentStateIdx].textKey, "<"+this.states[this.currentStateIdx].id.toUpperCase()+">");
    this.setImages(this.createBtnSurfaces());
};

TG_ButtonToggleText.prototype.updateBtnVisuals = function(image, text, fontProperties) {
	"use strict";
    this.text = text;
    this.toggleImage = image;
    if(fontProperties){
        this.font.setProperties(fontProperties);
    }
	this.setImages(this.createBtnSurfaces());
};

TG_ButtonToggleText.prototype.createBtnSurfaces = function() {
	"use strict";
	var middleWidth, txtWidth, txtAlign, maxFontSize, textXOffset, textYOffset, i, surfaces, s,
        image = {source: this.buttonType.sprite, leftWidth: this.buttonType.leftWidth, rightWidth : this.buttonType.rightWidth},
		sideWidth = image.leftWidth + image.rightWidth,
        sideDiff = Math.abs(image.leftWidth - image.rightWidth),
        middleHeight = this.height - (this.buttonType.textPaddingY || 0),
        sprite = image.source,
        textFont = this.font.clone();

	txtWidth = textFont.textWidth(this.text);

	//determine middleWidth and max font size
	if(this.width === undefined){
		middleWidth = txtWidth;
	}else if (typeof this.width === "number") {
		middleWidth = this.width - sideWidth;
	}
	else if(typeof this.width === "object") {
		if(this.width.width !== undefined) {
			middleWidth = this.width.width - sideWidth;
		}else{
			if(this.width.minWidth !== undefined) {
				middleWidth = Math.max(this.width.minWidth - sideWidth, txtWidth);
			}
			if(this.width.maxWidth !== undefined) {
				middleWidth = Math.min(this.width.maxWidth - sideWidth, txtWidth);
			}
		}
	}
	maxFontSize = textFont.textFitSize(this.text, middleWidth, middleHeight, true);
    if(maxFontSize < textFont.getFontSize()){
		textFont.setFontSize(maxFontSize);
	}
    txtWidth = textFont.textWidth(this.text, middleWidth);

	// align font
	textXOffset = image.leftWidth;
    txtAlign = textFont.getAlign();
	if(txtAlign === "center") {
		if(middleWidth - sideDiff >= txtWidth){
		    textXOffset += Math.round((middleWidth - sideDiff)/2);
		}else{
			textXOffset += this.buttonType.textOffsetX + Math.round(txtWidth/2);
		}
	}else if(txtAlign === "left"){
	    textXOffset += this.buttonType.textOffsetX;
	}else if(txtAlign === "right"){
		textXOffset += middleWidth;
	}
	textYOffset = Math.round(this.height/2);
    if(this.buttonType.textOffsetY !== undefined){
        textYOffset += this.buttonType.textOffsetY;
    }

	// draw surfaces
	surfaces = [];
	for(i=0; i<sprite.number; i++){
		s = new tbSurface(middleWidth + sideWidth, this.height);
		s.setTarget();

		// draw images
		sprite.drawPart(i, 0, 0, image.leftWidth, this.height, 0, 0, 1);
		sprite.drawPartStretched(i, image.leftWidth, 0, sprite.width - sideWidth, this.height, image.leftWidth, 0, middleWidth, this.height, 1);
		sprite.drawPart(i, image.leftWidth + sprite.width - sideWidth, 0, image.rightWidth, this.height, image.leftWidth + middleWidth, 0, 1);

        // draw toggle image
        this.toggleImage.drawSimple(0, this.buttonType.toggleImageOffsetX, this.buttonType.toggleImageOffsetY);

		// drawText
        textFont.drawSimple(this.text, textXOffset, textYOffset, middleWidth);

		s.resetTarget();
		surfaces.push(s);
	}
	return surfaces;
};

TG_ButtonToggleText.prototype.handleSpecial = function(id){
    "use strict";
    if(id === TG.languageSetEventId){
        this.redrawText();
    }
};
//******************************************************
//*****    Token Resource: TG_ButtonToggle
//******************************************************

// ***************************************
// Definition of game token TG_ButtonToggle
// ***************************************

// The constructor for the TG_ButtonToggle game token.
function TG_ButtonToggle(buttonType, x, y, depth, state, onClick, parent) {
	"use strict";
	// define fonts and images
    var toggleAction, self, m, i;
    this.states = TG.settings.tg.buttonDefinitions[buttonType].states;
    if(!this.states.length) {
        return;
    }
    for(i=0; i<this.states.length; i++){
        if(this.states[i].id === state){
            this.currentStateIdx = i;
            break;
        }
    }
    if(this.currentStateIdx === undefined){
        this.currentStateIdx = 0;
    }
    this.toggleImage = this.states[this.currentStateIdx].sprite;
    m = new tbMaskImage(this.toggleImage);

    self = this;
    if(typeof onClick === "function"){
        toggleAction = function(){
            self.setState();
            return onClick(self.states[self.currentStateIdx].id);
        };
    }else if (typeof onClick === "object"){
        toggleAction = {};
        toggleAction.callbackObj = this;
        toggleAction.callbackFunc = function(){
            self.setState();
            return onClick.callbackFunc.call(onClick.callbackObj, self.states[self.currentStateIdx].id);
        };
    }else{
        toggleAction = function(){
            self.setState();
        };
    }

    TG_ButtonBase.call(this, x, y, depth, m,  [this.toggleImage], toggleAction, parent);
}

TG.functions.inherit(TG_ButtonToggle, TG_ButtonBase);

TG_ButtonToggle.prototype.setState = function(state){
    "use strict";
    var i;
    if(state === undefined){
        this.currentStateIdx = (this.currentStateIdx + 1)%this.states.length;
    }else{
        for(i=0; i<this.states.length; i++){
            if(this.states[i].id === state){
                this.currentStateIdx = i;
                break;
            }
        }
    }
    this.updateBtnVisuals(this.states[this.currentStateIdx].sprite);
};

TG_ButtonToggle.prototype.updateBtnVisuals = function(image) {
	"use strict";
    var array = [];
    this.toggleImage = image;
    this.setImages(array.concat(this.toggleImage));
};

//******************************************************
//*****    Token Resource: TG_LevelStartDialog
//******************************************************

// ***************************************
// Definition of game token TG_LevelStartDialog
// ***************************************

/* global
	TG_MainController
 */

// The constructor for the TG_LevelStartDialog game token.
var TG_LevelStartDialog = function(headerText, assignmentText, callbackFunc, callbackObj) {
	"use strict";
	// Properties used by the engine
	this.depth = 10;
	this.visible = false;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
    var setting;
    this.settings = TG.settings.tg.levelStart;
    if(TG.orientation === "landscape" && TG.settings.tg.levelStart_landscape){
		for(setting in TG.settings.tg.levelStart_landscape){
			this.settings[setting] = TG.settings.tg.levelStart_landscape[setting];
		}
	}
    //apply theme specific setting overwrites
    for(setting in TG.settings.theme.levelStart){
        this.settings[setting] = TG.settings.theme.levelStart[setting];
    }
    this.headerText = headerText;
    this.assignmentText = assignmentText;
    this.callbackFunc = callbackFunc;
	this.callbackObj = callbackObj;
	this.phase = "entering";
    this.touchStarted = false;
    // Add it to the game engine
	tbGame.addToken(this, false);
    tbGame.addTokenTags(this, "LevelStartDialog");
};

TG_LevelStartDialog.prototype.sendUp = function(){
    "use strict";
    var callbackFunction, i, animation, callback, end,
        self = this;
    if(this.phase !== "leaving") {
        this.phase = "leaving";
        //start overlay out-animations
        this.blockingAnimsCount = 0;
        callbackFunction = function () {
            tbGame.removeToken(self);
            if(self.callbackObj) {
                self.callbackFunc.call(self.callbackObj);
            }else if(self.callbackFunc){
                self.callbackFunc();
            }
        };
        if (this.settings.overlayAnimationOut !== undefined) {
            for (i = 0; i < this.settings.overlayAnimationOut.length; i++) {
                animation = this.settings.overlayAnimationOut[i];
                callback = undefined;
                if (animation.blocking) {
                    this.blockingAnimsCount++;
                    callback = callbackFunction;
                }
                end = animation.end;
                if (animation.type === "x") {
                    end = TG.functions.computeImgCoordinatesFromSettings(end, this.canvas.width, this.overlay.image.width);
                } else if (animation.type === "y") {
                    end = TG.functions.computeImgCoordinatesFromSettings(end, this.canvas.height, this.overlay.image.height) + Math.abs(TG.canvasTopOffset);
                }
                this.overlay.addAnimator(animation.type, end, animation.duration, animation.ease, callback, animation.delay, animation.loop, animation.loopInterval, animation.activationEvent);
            }
        }
        if (this.blockingAnimsCount === 0) {
            callbackFunction();
        }
    }
};

// handleAdd is called when the token is (re-) added to the game.
TG_LevelStartDialog.prototype.handleAdd = function() {
    "use strict";
    var font, i, x, y,animation, callbackFunction, callback, end, maxFontSize,
        self = this,
        img = this.settings.overlayImage,
        width = img.width,
        height = img.height;
    this.overlay = new TG_AnimatedImage(this.depth + 10, this.layer, new tbSurface(width, height));

    //draw overlay surface
    this.overlay.image.setTarget();
    img.drawSimple(0, 0, 0);
    if (this.assignmentText !== "") {
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.assignmentOffsetX, width, 0);
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.assignmentOffsetY, height, 0);
        font = f_default.clone();
        font.setProperties(this.settings.assignmentFont);
        if(this.settings.assignmentMaxWidth !== undefined && this.settings.assignmentMaxHeight !== undefined){
            maxFontSize = font.textFitSize(this.assignmentText, this.settings.assignmentMaxWidth, this.settings.assignmentMaxHeight, this.settings.assignmentMaxWidth);
            if(font.getFontSize() > maxFontSize){
                font.setFontSize(maxFontSize);
            }
        }
        font.drawSimple(this.assignmentText, x, y, this.settings.assignmentMaxWidth);
        if (testMode) {
        	TG_MainController.drawTextRect(font, x, y, this.settings.assignmentMaxWidth, this.settings.assignmentMaxHeight);
        }
    }
    if (this.headerText !== "") {
        x = TG.functions.computeImgCoordinatesFromSettings(this.settings.headerOffsetX, width, 0);
        y = TG.functions.computeImgCoordinatesFromSettings(this.settings.headerOffsetY, height, 0);
        font = f_default.clone();
        font.setProperties(this.settings.headerFont);
        if(this.settings.headerMaxWidth !== undefined && this.settings.headerMaxHeight !== undefined){
            maxFontSize = font.textFitSize(this.headerText, this.settings.headerMaxWidth, this.settings.headerMaxHeight, this.settings.headerMaxWidth);
            if(font.getFontSize() > maxFontSize){
                font.setFontSize(maxFontSize);
            }
        }
        font.drawSimple(this.headerText, x, y, this.settings.headerMaxWidth);
        if (testMode) {
        	TG_MainController.drawTextRect(font, x, y, this.settings.headerMaxWidth, this.settings.headerMaxHeight);
        }
    }
    this.overlay.image.resetTarget();

    //setup overlay vars
    this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageX, this.canvas.width, width);
    this.overlay.y = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageY, this.canvas.height, height) + Math.abs(TG.canvasTopOffset);

    //setup overlay in-animations
    this.blockingAnimsCount = 0;
    callbackFunction = function () {
        self.blockingAnimsCount--;
        if (self.blockingAnimsCount === 0) {
            self.phase = "paused";
        }
    };
    if (this.settings.overlayAnimationIn !== undefined) {
        for (i = 0; i < this.settings.overlayAnimationIn.length; i++) {
            animation = this.settings.overlayAnimationIn[i];
            callback = undefined;
            if (animation.blocking) {
                this.blockingAnimsCount++;
                callback = callbackFunction;
            }
            end = animation.end;
            if (animation.type === "x") {
                end = TG.functions.computeImgCoordinatesFromSettings(end, this.canvas.width, this.overlay.image.width);
            } else if (animation.type === "y") {
                end = TG.functions.computeImgCoordinatesFromSettings(end, this.canvas.height, this.overlay.image.height)+ Math.abs(TG.canvasTopOffset);
            }
            this.overlay.addAnimator(animation.type, end, animation.duration, animation.ease, callback, animation.delay, animation.loop, animation.loopInterval, animation.activationEvent);
            if (animation.sound !== undefined) {
                tbAudio.play(animation.sound);
            }
        }
    }
    if (this.blockingAnimsCount === 0) {
        this.phase = "paused";
    }
    this.timer = 0;
};

// handleRemove is called when the token is removed from the game.
TG_LevelStartDialog.prototype.handleRemove = function() {
	"use strict";
    tbGame.removeToken(this.overlay);
};

// handleBeginStep is called each step before processing other events.
TG_LevelStartDialog.prototype.handleBeginStep = function(timePassed) {
	"use strict";
    if(this.state !== "paused") {
        this.timer += timePassed;
        if(this.timer >= this.settings.pauseDuration && !(testMode && TG_MainController.drawDebugInfo)){
            this.sendUp();
        }
    }
};

// handleTouchDown is called when a touch is started or if a mouse button is pressed.
TG_LevelStartDialog.prototype.handleTouchDown = function() {
	"use strict";
    this.touchStarted = true;
    return true;
};

// handleTouchUp is called when a touch is ended or if a mouse button is released.
TG_LevelStartDialog.prototype.handleTouchUp = function() {
	"use strict";
    if( this.touchStarted && this.phase === "paused"){
		this.sendUp();
	}
    return true;
};

//******************************************************
//*****    Token Resource: TG_ChallengeStartDialog
//******************************************************

// ***************************************
// Definition of game token TG_ChallengeStartDialog
// ***************************************

// The constructor for the TG_ChallengeStartDialog game token.
var TG_ChallengeStartDialog = function(textKey, callbackFunc, callbackObj) {
	"use strict";
	// Properties used by the engine
	this.depth = 10;
	this.visible = false;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
    this.settings = TG.settings.tg.challengeStart;
     //apply theme specific setting overwrites
    var setting;
    for(setting in TG.settings.theme.challengeStart){
        this.settings[setting] = TG.settings.theme.challengeStart[setting];
    }
    this.textKey = textKey;
    this.callbackFunc = callbackFunc;
	this.callbackObj = callbackObj;
	this.phase = "entering";
    this.touchStarted = false;
    this.challengeInfo = TG.controller.getChallengeInformation();
    if(this.challengeInfo.currentPlayerIdx === this.challengeInfo.challengePlayerIdx){
		this.challenger = true;
	}
    // Add it to the game engine
	tbGame.addToken(this, false);
    tbGame.addTokenTags(this, "LevelStartDialog");
};

TG_ChallengeStartDialog.prototype.sendUp = function(){
    "use strict";
    var callbackFunction, end,
        self = this;
    if(this.phase !== "leaving") {
        this.phase = "leaving";
        //start overlay out-animations
        callbackFunction = function () {
            tbGame.removeToken(self);
            if(self.callbackObj) {
                self.callbackFunc.call(self.callbackObj);
            }else if(self.callbackFunc){
                self.callbackFunc();
            }
        };
        this.overlay.addAnimator("y", -this.overlay.height, this.settings.animOutDuration, this.settings.animOutEase, callbackFunction);
    }
};

// handleAdd is called when the token is (re-) added to the game.
TG_ChallengeStartDialog.prototype.handleAdd = function() {
    "use strict";
    var self, img, width, height, drawText, t, w, f, textWidth, textHeight, textX, textY, winX, winY, end, callback, i, name;
    self = this;
    img = this.settings.overlayImage;
    width = img !== undefined? img.width : tgScale(600);
    height = img !== undefined? img.height : tgScale(700);
	drawText = function(text, fontProps, maxWidth, maxHeight, wrap, xSetting, ySetting){
		var font, maxFontSize, x, y;
		font = f_default.clone();
		if(fontProps !== undefined){
			font.setProperties(fontProps);
		}
		maxFontSize = font.textFitSize(text, maxWidth, maxHeight, wrap);
		if(font.getFontSize() > maxFontSize){
			font.setFontSize(maxFontSize);
		}
		x = TG.functions.computeTextCoordinatesFromSettings(xSetting, width, font.textWidth(text, wrap), font.getAlign());
		y = TG.functions.computeTextCoordinatesFromSettings(ySetting, height, font.textHeight(text, wrap), font.getAlign());
		font.drawSimple(text, x, y, wrap? maxWidth : undefined);

		if (testMode) {
			TG_MainController.drawTextRect(font, x, y, maxWidth, maxHeight);
		}
	};

	this.overlay = new TG_AnimatedImage(this.depth + 10, this.layer, new tbSurface(width, height));

	// draw overlay surface
	this.overlay.image.setTarget();
	if(img !== undefined){
		img.drawSimple(0, 0, 0);
	}

    // draw title
	if(this.challenger){
		if(this.challengeInfo.stranger){
			t = TG.dictionary.getText("challengeStartScreenTitle_challenger_stranger", "<CHALLENGESTARTSCREENTITLE_CHALLENGER>");
		}else{
			t = TG.dictionary.getText("challengeStartScreenTitle_challenger_friend", "<CHALLENGESTARTSCREENTITLE_CHALLENGER>");
		}
	}else{
		if(this.challengeInfo.stranger){
			t = TG.dictionary.getText("challengeStartScreenTitle_challengee_stranger", "<CHALLENGESTARTSCREENTITLE_CHALLENGEE>");
		}else{
			t = TG.dictionary.getText("challengeStartScreenTitle_challengee_friend", "<CHALLENGESTARTSCREENTITLE_CHALLENGEE>");
		}
	}
    drawText(t, this.settings.titleFont, this.settings.titleMaxWidth, this.settings.titleMaxHeight, true, this.settings.titleX, this.settings.titleY);

    //draw name
    if(this.challenger){
		t = "";
		for(i = 1; i < this.challengeInfo.playerNames.length; i++){
			name = this.challengeInfo.playerNames[i];
			if(name.length > 13){
				t += name.substr(0, 10) + "...";
			}else{
				t += name;
			}
			if(i + 1 < this.challengeInfo.playerNames.length){
				t += ", ";
			}
		}
	}else{
		name = this.challengeInfo.playerNames[this.challengeInfo.challengePlayerIdx];
		t = (name.length > 13)? name.substr(0, 10) + "..." : name;
	}
    drawText(t, this.settings.nameFont, this.settings.nameMaxWidth, this.settings.nameMaxHeight, false, this.settings.nameX, this.settings.nameY);


    // draw score header
	if(!this.challenger){
		t = TG.dictionary.getText(this.textKey, "<" + this.textKey.toUpperCase() + ">");
		name = this.challengeInfo.playerNames[this.challengeInfo.challengePlayerIdx];
		t = t.replace("<NAME>", (name.length > 13)? name.substr(0, 10) + "..." : name);
	}else{
		t = TG.dictionary.getText(this.textKey + "_challenger", "<" + this.textKey.toUpperCase() + "_CHALLENGER>");
	}
		drawText(t, this.settings.scoreHeaderFont, this.settings.scoreHeaderMaxWidth, this.settings.scoreHeaderMaxHeight, false, this.settings.scoreHeaderX,
		        this.settings.scoreHeaderY);


	// draw score
	if(!this.challenger){
		t = TG.controller.getFormattedScoreString(this.challengeInfo.playerValues[0]);
		drawText(t, this.settings.scoreFont, this.settings.scoreMaxWidth, this.settings.scoreMaxHeight, false, this.settings.scoreX, this.settings.scoreY);
	}

    // draw to win header
    t = TG.dictionary.getText("challengeStartScreenToWin", "<CHALLENGESTARTSCREENTOWIN>");
    drawText(t, this.settings.winHeaderFont, this.settings.winHeaderMaxWidth, this.settings.winHeaderMaxHeight, true, this.settings.winHeaderX, this.settings.winHeaderY);

  //draw to win header
    t = this.challengeInfo.wager +"";
    drawText(t, this.settings.winFont, this.settings.winMaxWidth, this.settings.winMaxHeight, false, this.settings.winX, this.settings.winY);

	this.overlay.image.resetTarget();

    // setup overlay vars
    this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageX, this.canvas.width, width) + this.settings.overlayOffsetX;
    this.overlay.y = -this.overlay.height;

    // setup overlay in-animations
	callback = function(){
		self.phase = "paused";
	};
    end = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageY, this.canvas.height, this.overlay.image.height)+ Math.abs(TG.canvasTopOffset);
    this.overlay.addAnimator("y", end, this.settings.animInDuration, this.settings.animInEase, callback);
    //this.overlay.y = end;
    tbAudio.play(a_levelStart);

    this.timer = 0;
};

// handleRemove is called when the token is removed from the game.
TG_ChallengeStartDialog.prototype.handleRemove = function() {
	"use strict";
    tbGame.removeToken(this.overlay);
};

// handleBeginStep is called each step before processing other events.
TG_ChallengeStartDialog.prototype.handleBeginStep = function(timePassed) {
	"use strict";
    if(this.state !== "paused") {
        this.timer += timePassed;
        if(this.timer >= this.settings.pauseDuration && !(testMode && TG_MainController.drawDebugInfo)){
            this.sendUp();
        }
    }
};

// handleTouchDown is called when a touch is started or if a mouse button is pressed.
TG_ChallengeStartDialog.prototype.handleTouchDown = function() {
	"use strict";
    this.touchStarted = true;
    return true;
};

// handleTouchUp is called when a touch is ended or if a mouse button is released.
TG_ChallengeStartDialog.prototype.handleTouchUp = function() {
	"use strict";
    if( this.touchStarted && this.phase === "paused"){
		this.sendUp();
	}
    return true;
};

//******************************************************
//*****    Token Resource: TG_AnimatedImage
//******************************************************

// ***************************************
// Definition of game token TG_AnimatedImage
// ***************************************

// The constructor for the TG_AnimatedImage game token.
var TG_AnimatedImage = function(depth, layer, image) {
	"use strict";
	// Properties used by the engine
	this.depth = depth;
	this.visible = true;
	this.active = true;
    TG.functions.setLayerAndCanvas(this, layer);
	// Your own properties
    this.image = image;
    this.subImage = 0;
    this.width = image.width;
    this.height = image.height;
	this.x = 0;
	this.y = 0;
	this.originX = 0;
	this.originY = 0;
    this.xScale = 1;
    this.yScale = 1;
    this.angle = 0;
    this.alpha = 1;
	this.animators = [];
    this.animatorCount = 0;
    this.parent = {x: 0, y: 0};
    this.forceDraw = true;
	// Add it to the game engine
	tbGame.addToken(this, false);
};

// handleAdd is called when the token is (re-) added to the game.
TG_AnimatedImage.prototype.addAnimator = function(type, end, duration, ease, callback, delay, loop, loopInterval) {
    "use strict";
    var startValue,
        active = true,
        delayed = delay > 0;
    switch(type){
        case "x": startValue = this.x; break;
        case "y": startValue = this.y; break;
        case "xScale": startValue = this.xScale; break;
        case "yScale": startValue = this.yScale; break;
        case "scale": type = "xScale";
            startValue = this.xScale;
            this.addAnimator("yScale", end, duration, ease, undefined, delay, loop, loopInterval);
        break;
        case "angle": startValue = this.angle; break;
        case "alpha": startValue = this.alpha; break;
        case "subImage": startValue = 0; break;
    }
    this.animators.push({id: this.animatorCount, timer: 0, active: active, delayed: delayed, type: type, start: startValue, end: end, callback: callback, duration: duration, ease: ease, delay: delay, loop: loop, loopInterval: loopInterval});
    return this.animatorCount++;
};

// handleAdd is called when the token is (re-) added to the game.
TG_AnimatedImage.prototype.removeAnimator = function(id, forceToEndValue) {
    "use strict";
    var i;
    for(i=this.animators.length-1; i>=0; i--){
        if(id ==="all" || id === this.animators[i].id){
            if(forceToEndValue) {
                switch(this.animators[i].type){
                    case "x": this.x = this.animators[i].end; break;
                    case "y": this.y = this.animators[i].end; break;
                    case "xScale": this.xScale = this.animators[i].end; break;
                    case "yScale": this.yScale = this.animators[i].end; break;
                    case "angle": this.angle = this.animators[i].end; break;
                    case "alpha": this.alpha = this.animators[i].end; break;
                    case "subImage": this.subImage = this.animators[i].end; break;
                }
                if(typeof this.animators[i].callback === "function"){
                    this.animators[i].callback.call(this);
                }
            }
            if(id !== "all") {
                return this.animators.split(i, 1);
            }
        }
    }
};

// handleBeginStep is called each step before processing other events.
TG_AnimatedImage.prototype.handleBeginStep = function(timePassed) {
	"use strict";
    var i, animator, value, n;
    for(i=0; i<this.animators.length; i++){
        animator = this.animators[i];
        if(animator.active){
            animator.timer += timePassed;
            if(animator.delayed){
                if(animator.timer >= animator.delay){
                    animator.timer = animator.timer % animator.delay;
                    animator.delayed = false;
                }
            }
            if(!animator.delayed){
                if(animator.timer >= animator.duration){
                    value = animator.end;
                    if(animator.loop){
                        animator.delayed = true;
                        animator.delay = animator.loopInterval;
                        animator.timer = animator.timer % animator.duration;
                    }else{
                        if(typeof animator.callback === "function"){
                            animator.callback.call(this);
                        }
                        this.animators[i] = undefined;
                    }
                }else {
                    if(animator.type === "subImage"){
                        n = (this.image instanceof Array)? this.image.length : this.image.number;
                        value = Math.floor(animator.timer * n/animator.duration);
                    }else{
                        value = animator.ease(animator.timer, animator.start, animator.end - animator.start, animator.duration);
                    }
                }
                switch(animator.type){
                    case "x": this.x = value; break;
                    case "y": this.y = value; break;
                    case "xScale": this.xScale = value; break;
                    case "yScale": this.yScale = value; break;
                    case "angle": this.angle = value; break;
                    case "alpha": this.alpha = value; break;
                    case "subImage": this.subImage = value; break;
                }
                this.canvas.drawTokens = true;
            }
        }
    }
    for(i=this.animators.length - 1; i>=0; i--){
        if(this.animators[i] === undefined){
            this.animators.splice(i, 1);
        }
    }
};

// handleDraw is called at the end of a step to draw the token.
TG_AnimatedImage.prototype.handleDraw = function() {
	"use strict";
	var image, x, y;
	x = Math.round(this.x - this.xScale*this.originX) + this.parent.x;
	y = Math.round(this.y - this.yScale*this.originY) + this.parent.y;
    image = this.image;
	if(image instanceof Array){
        image = this.image[this.subImage%this.image.length];
    }
	if(image instanceof tbSurface) {
	    image.drawExt(x, y, this.xScale, this.yScale, this.angle, this.alpha);
    }else{
        image.drawExt(this.subImage, x, y, this.xScale, this.yScale, this.angle, this.alpha);
    }
	this.forceDraw = false;
};


//******************************************************
//*****    Token Resource: TG_Counter
//******************************************************

// ***************************************
// Definition of game token TG_Counter
// ***************************************

/* global
	TG_MainController
 */

// The constructor for the TG_Counter game token.
var TG_Counter = function(x, y, w, h, depth, value, font, marginX, marginY, parent, toString) {
	"use strict";
	// Properties used by the engine
	this.depth = depth;
	this.visible = true;
	this.active = false;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	this.x = x;
	this.y = y;
	this.marginX = marginX;
    this.marginYTop = (typeof marginY === "object")? marginY.top : marginY;
    this.marginYBottom = (typeof marginY === "object")? marginY.bottom : marginY;
	this.textWidth = w;
	this.textHeight = h;
	this.width = this.textWidth + 2*this.marginX;
	this.height = this.textHeight + this.marginYTop + this.marginYBottom;
	this.value = value || 0;
	this.parent = parent || {x: 0, y: 0};
	this.font = font;
	this.toString = (typeof toString === "function")? toString : function(value){return value + "";};

	this.alpha = 1;
	this.originX = 0;
	this.originY = 0;
	this.scaleX = 0;
	this.scaleY = 0;
	var s = new tbSurface(this.width, this.height);
	this.animatedImage = new TG_AnimatedImage(this.depth, this.layer, s);
	this.animatedImage.x = x - this.marginX;
	this.animatedImage.y = y - this.marginYTop;
	this.animatedImage.parent = parent;
	this.surface = this.animatedImage.image;
	this.redrawSurface();
	// Add it to the game engine
	tbGame.addToken(this);
};

TG_Counter.prototype.handleRemove = function(){
    "use strict";
    tbGame.removeToken(this.animatedImage);
};

TG_Counter.prototype.startCount = function(endValue, duration, ease, beginValue){
    "use strict";
    this.active = true;
    this.begin = (beginValue !== undefined)? beginValue : this.value;
    this.value = this.begin;
    this.end = endValue;
    this.duration = duration;
    this.ease = ease;
    this.timer = 0;
};

TG_Counter.prototype.stopCount = function(forceToEnd){
    "use strict";
    if(this.active && this.forceToEnd){
        this.value = this.end;
        this.redrawSurface();
    }
    this.active = false;
};

TG_Counter.prototype.currentValue = function(value){
    "use strict";
    if(value !== undefined){
        this.value = value;
        this.forceDraw = true;
    }
    return this.value;
};

TG_Counter.prototype.redrawSurface = function(){
    "use strict";
    var maxFontSize, text,
        align = this.font.getAlign(),
        baseline = this.font.getBaseline(),
        x = this.marginX,
        y = this.marginYTop;
    if(!this.backgroundImg){
		this.surface.clear();
		this.canvas.drawTokens = true;
	}
    this.surface.setTarget();
    if(this.backgroundImg){
        this.backgroundImg.drawPart(0, this.backgroundX, this.backgroundY, this.backgroundW, this.backgroundH, 0, 0, 1);
    }
    if(align === "center"){
        x += Math.round(this.textWidth/2);
    }else if(align === "right"){
        x += this.textWidth;
    }
    if(baseline === "middle"){
        y += Math.round(this.textHeight/2);
    }else if(baseline === "bottom"){
        y += this.textHeight;
    }
    text = this.toString(this.value);
    maxFontSize = this.font.textFitSize(text, this.textWidth, this.textHeight, true);
    if(maxFontSize < this.font.getFontSize()){
        this.font.setFontSize(maxFontSize);
    }
    this.font.drawSimple(text, x, y, this.textWidth);

    if (testMode) {
		TG_MainController.drawSurfaceBoundaries();
	}

    this.surface.resetTarget();
    this.animatedImage.forceDraw = true;
};

TG_Counter.prototype.setBackgroundImage = function(image, x, y, w, h){
    "use strict";
    this.backgroundImg = image;
    this.backgroundX = x - this.marginX || 0;
    this.backgroundY = y - this.marginYTop || 0;
    this.backgroundW = w + 2*this.marginX || this.width;
    this.backgroundH = h + this.marginYTop + this.marginYBottom || this.height;
    this.redrawSurface();
};

// handleBeginStep is called each step before processing other events.
TG_Counter.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	var newValue;
	newValue = Math.round(this.ease(this.timer, this.begin, this.end - this.begin, this.duration));
	if(this.timer >= this.duration){
        this.value = this.end;
        this.active = false;
        this.redrawSurface();
    }else if(newValue !== this.value){
	    this.value = newValue;
	    this.redrawSurface();
	}
	this.timer += timePassed;
};
//******************************************************
//*****    Token Resource: TG_Dialog
//******************************************************

// ***************************************
// Definition of game token TG_Dialog
// ***************************************

/* global
	TG_ButtonText,
	f_default
 */

// The constructor for the TG_Dialog game token.
var TG_Dialog = function(question, buttonDescriptions, pauseGame) {
	"use strict";
	// Properties used by the engine
	this.depth = -100;
	this.visible = false;
	this.active = true;
	this.question = question;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);
	// Your own properties
	// apply theme specific setting overwrites
	this.settings = TG.settings.tg.dialog;
	this.buttonSettings = TG.settings.tg.buttonDefinitions;
	this.buttonDescriptions = buttonDescriptions;
	// apply theme specific setting overwrites
	for (var setting in TG.settings.theme.dialog) {
		this.settings[setting] = TG.settings.theme.dialog[setting];
	}
	this.pauseGame = pauseGame !== false;
	// Add it to the game engine
	tbGame.addToken(this);
};

TG_Dialog.prototype.toggleLang = function () {};

TG_Dialog.prototype.addButton = function(buttonDescription, buttonX, buttonY, buttonWidth, enableButton) {
	"use strict";

	var textKey = buttonDescription.textKey || "NO_TEXT_KEY_GIVEN";

	var newButton = new TG_ButtonText("default_text", buttonX, buttonY, this.depth - 20, textKey, buttonWidth, {
		callbackFunc: function () {
			//this.close();
			if (buttonDescription.callbackFunc) {
				if (buttonDescription.callbackObj) {
					buttonDescription.callbackFunc.call(buttonDescription.callbackObj, buttonDescription);
				}
				else {
					buttonDescription.callbackFunc(buttonDescription);
				}
			}
		},
		callbackObj: this
	}, this.overlay);

	this.buttons.push(newButton);

	if (buttonDescription.text) {
		newButton.updateBtnVisuals(newButton.image, buttonDescription.text);
	}

	this.buttons[this.buttons.length - 1].enabled = enableButton || false;
};

TG_Dialog.prototype.drawBackground = function(overlayXOffset, overlayYOffset, message) {
	"use strict";
	var image;
	this.overlay.image.setTarget();
	tbGraphics.clear();
	// draw overlay background
	image = this.settings.backgroundImage;
	image.drawSimple(0, 0, 0);

	// draw confirm text
	var text = message? message : this.question;
	var font = f_default.clone();
	if (this.settings.questionFont) {
		font.setProperties(this.settings.questionFont);
	}
	var maxFontSize = font.textFitSize(text, this.settings.questionTextWidth, this.settings.questionTextHeight, true);
	if (maxFontSize < font.getFontSize()) {
		font.setFontSize(maxFontSize);
	}
	var w = font.textWidth(text, this.settings.questionTextWidth) + 10;
	var h = font.textHeight(text, this.settings.questionTextHeight) + 10;
	var x = TG.functions.computeTextCoordinatesFromSettings(this.settings.questionTextX, this.overlay.image.width, w, font.getAlign());
	var y = TG.functions.computeTextCoordinatesFromSettings(this.settings.questionTextY, this.overlay.image.height - this.getButtonsY(), h, font.getBaseline());
	font.drawSimple(text, x, y, w);
	if (testMode) {
		TG_MainController.drawTextRect(font, x, y, this.settings.questionTextWidth, this.settings.questionTextHeight);
	}
	this.overlay.image.resetTarget();
};

TG_Dialog.prototype.getButtonsY = function() {
	"use strict";
	var buttonsToShow = this.buttonDescriptions;
	var buttonHeight = this.buttonSettings["default_text"].sprite.height;
	var totalHeight = buttonHeight * buttonsToShow.length + this.settings.buttonsOffsetInBetween * (buttonsToShow.length - 1);
	return TG.functions.computeImgCoordinatesFromSettings(this.settings.buttonsOffsetY, this.overlay.image.height, totalHeight);

};

TG_Dialog.prototype.addInitButtons = function(overlayXOffset, enableButtons) {
	"use strict";
	var i, j, buttonHeight, buttonMaxWidth, left, row, buttonX, buttonY, buttonWidth, buttonsToShow = [];

	buttonsToShow = this.buttonDescriptions;

	buttonHeight = this.buttonSettings["default_text"].sprite.height;
	buttonMaxWidth = this.settings.buttonsWidth;
	left = TG.functions.computeImgCoordinatesFromSettings(this.settings.buttonsOffsetX, this.canvas.width, buttonMaxWidth) - overlayXOffset;
	buttonY = this.getButtonsY();


	for (i = buttonsToShow.length-1; i >= 0; i--) {
		buttonX = left;
		buttonWidth = buttonMaxWidth;

		if (typeof buttonsToShow[i] === "object" && buttonsToShow[i].hasOwnProperty("length") && buttonsToShow[i].length) {
			row = buttonsToShow[i];
			buttonWidth = (buttonMaxWidth - (row.length - 1) * this.settings.buttonsOffsetInBetween) / row.length;
			for (j = 0; j < row.length; j++) {
				this.addButton(row[j], buttonX, buttonY, buttonWidth, enableButtons);
				buttonX += buttonWidth + this.settings.buttonsOffsetInBetween;
			}
		}
		else {
			this.addButton(buttonsToShow[i], buttonX, buttonY, buttonWidth, enableButtons);
		}

		buttonY -= buttonHeight + this.settings.buttonsOffsetInBetween;
	}
};

TG_Dialog.prototype.hide = function(){
	"use strict";
	var i, b;
	for(i=0; i<this.buttons.length; i++){
		b = this.buttons[i];
		b.setVisible(false);
	}
	this.overlay.visible = false;
};

TG_Dialog.prototype.show = function(){
	"use strict";
	var i, b;
	for(i=0; i<this.buttons.length; i++){
		b = this.buttons[i];
		b.setVisible(true);
	}
	this.overlay.visible = true;
};

TG_Dialog.prototype.close = function() {
	"use strict";
	tbGame.removeToken(this);
	this.canvas.drawTokens = true;
	return true;
};

TG_Dialog.prototype.showConfirmMessage = function(message) {
	"use strict";
	this.drawBackground(this.overlay.x, this.overlay.y, message);
	var i;
	for(i = 0; i < this.buttons.length; i++) {
		tbGame.removeToken(this.buttons[i]);
	}
	this.canvas.drawTokens = true;
};

TG_Dialog.prototype.showConfirmSubMessage = function(message) {
	"use strict";
	var font, maxFontSize, w, h, x, y;
	font = f_default.clone();
	if(this.settings.subQuestionFont) {
		font.setProperties(this.settings.subQuestionFont);
	}
	maxFontSize = font.textFitSize(message, this.settings.questionSubTextWidth, this.settings.questionSubTextHeight, true);
	if(maxFontSize < font.getFontSize()) {
		font.setFontSize(maxFontSize);
	}
	w = font.textWidth(message, this.settings.questionTextWidth) + 10;
	h = font.textHeight(message, this.settings.questionTextHeight) + 10;
	x = TG.functions.computeTextCoordinatesFromSettings(this.settings.questionSubTextX, this.overlay.image.width, w, font.getAlign());
	y = TG.functions.computeTextCoordinatesFromSettings(this.settings.questionSubTextY, this.overlay.image.height - this.getButtonsY(), h, font.getBaseline());
	this.overlay.image.setTarget();
	font.drawSimple(message, x, y, w);
	if (testMode) {
		TG_MainController.drawTextRect(font, x, y, this.settings.questionSubTextWidth, this.settings.questionSubTextHeight);
	}
	this.overlay.image.resetTarget();
};

// handleAdd is called when the token is (re-) added to the game.
TG_Dialog.prototype.handleAdd = function() {
	"use strict";
	var y, activateButtons, image, self;
	// pause all tokens
	if(this.pauseGame){
		TG.controller._pauseAll(this);
	}

	// create overlay surface
	image = this.settings.backgroundImage;
	this.overlay = new TG_AnimatedImage(this.depth - 10, this.layer, new tbSurface(image.width, image.height));
	this.overlay.x = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetX, this.canvas.width, image.width);

	var defaultCanvasSettings = (TG.orientation === "landscape")? TG.settings.tg.canvasses_landscape : TG.settings.tg.canvasses;

	y = TG.functions.computeImgCoordinatesFromSettings(this.settings.overlayImageOffsetY, this.canvas.height, image.height) + defaultCanvasSettings.minTop* -1;
	this.overlay.y = y;
	// draw background
	this.drawBackground(this.overlay.x, this.overlay.y);

	this.buttons = [];
	// add buttons
	this.addInitButtons(this.overlay.x);

	// start animation
	self = this;
	activateButtons = function() {
		var i;
		for (i = 0; i < self.buttons.length; i++) {
			self.buttons[i].enabled = true;
		}
	};
	// slide in from top
	this.overlay.y = -this.overlay.image.height;
	this.overlay.addAnimator("y", y, this.settings.overlayAnimInDuration, this.settings.overlayAnimInEase, activateButtons);

};

// handleRemove is called when the token is removed from the game.
TG_Dialog.prototype.handleRemove = function() {
	"use strict";
	var i;
	if(this.pauseGame){
		TG.controller._resumeAll();
	}
	for (i = 0; i < this.buttons.length; i++) {
		tbGame.removeToken(this.buttons[i]);
	}
	tbGame.removeToken(this.overlay);
	if(TG.controller.activeModelDialog === this){
		TG.controller.activeModelDialog = null;
	}
};

TG_Dialog.prototype.handleTouchUp = function(){
	"use strict";
	return true;
};

TG_Dialog.prototype.handleTouchDown = function(){
	"use strict";
	return true;
};



//******************************************************
//*****    Script Resource: TG_SiteLock
//******************************************************

/* global
	window
 */

/*jslint bitwise: true */

var TG_MD5 = {
	utf8_encode: function(argString) {
		"use strict";
		// discuss at: http://phpjs.org/functions/utf8_encode/
		// original by: Webtoolkit.info (http://www.webtoolkit.info/)
		// improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		// improved by: sowberry
		// improved by: Jack
		// improved by: Yves Sucaet
		// improved by: kirilloid
		// bugfixed by: Onno Marsman
		// bugfixed by: Onno Marsman
		// bugfixed by: Ulrich
		// bugfixed by: Rafal Kukawski
		// bugfixed by: kirilloid
		// example 1: utf8_encode('Kevin van Zonneveld');
		// returns 1: 'Kevin van Zonneveld'

		if (argString === null || typeof argString === "undefined") {
			return "";
		}

		// .replace(/\r\n/g, "\n").replace(/\r/g, "\n");
		var string = (argString + "");
		var utftext = "", start, end, stringl = 0;

		start = end = 0;
		stringl = string.length;
		for (var n = 0; n < stringl; n++) {
			var c1 = string.charCodeAt(n);
			var enc = null;

			if (c1 < 128) {
				end++;
			}
			else if (c1 > 127 && c1 < 2048) {
				enc = String.fromCharCode((c1 >> 6) | 192, (c1 & 63) | 128);
			}
			else if ((c1 & 0xF800) !== 0xD800) {
				enc = String.fromCharCode((c1 >> 12) | 224, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128);
			}
			else {
				// surrogate pairs
				if ((c1 & 0xFC00) !== 0xD800) {
					throw new RangeError("Unmatched trail surrogate at " + n);
				}
				var c2 = string.charCodeAt(++n);
				if ((c2 & 0xFC00) !== 0xDC00) {
					throw new RangeError("Unmatched lead surrogate at " + (n - 1));
				}
				c1 = ((c1 & 0x3FF) << 10) + (c2 & 0x3FF) + 0x10000;
				enc = String.fromCharCode((c1 >> 18) | 240, ((c1 >> 12) & 63) | 128, ((c1 >> 6) & 63) | 128, (c1 & 63) | 128);
			}
			if (enc !== null) {
				if (end > start) {
					utftext += string.slice(start, end);
				}
				utftext += enc;
				start = end = n + 1;
			}
		}

		if (end > start) {
			utftext += string.slice(start, stringl);
		}

		return utftext;
	},

	createMD5Hash: function(str) {
		"use strict";
		// discuss at: http://phpjs.org/functions/md5/
		// original by: Webtoolkit.info (http://www.webtoolkit.info/)
		// improved by: Michael White (http://getsprink.com)
		// improved by: Jack
		// improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		// input by: Brett Zamir (http://brett-zamir.me)
		// bugfixed by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
		// depends on: utf8_encode
		// example 1: md5('Kevin van Zonneveld');
		// returns 1: '6e658d4bfcb59cc13f96c14450ac40b9'

		var xl;

		var rotateLeft = function(lValue, iShiftBits) {
			return (lValue << iShiftBits) | (lValue >>> (32 - iShiftBits));
		};

		var addUnsigned = function(lX, lY) {
			var lX4, lY4, lX8, lY8, lResult;
			lX8 = (lX & 0x80000000);
			lY8 = (lY & 0x80000000);
			lX4 = (lX & 0x40000000);
			lY4 = (lY & 0x40000000);
			lResult = (lX & 0x3FFFFFFF) + (lY & 0x3FFFFFFF);
			if (lX4 & lY4) {
				return (lResult ^ 0x80000000 ^ lX8 ^ lY8);
			}
			if (lX4 | lY4) {
				if (lResult & 0x40000000) {
					return (lResult ^ 0xC0000000 ^ lX8 ^ lY8);
				}
				return (lResult ^ 0x40000000 ^ lX8 ^ lY8);
						}
			return (lResult ^ lX8 ^ lY8);
		};

		var _F = function(x, y, z) {
			return (x & y) | ((~x) & z);
		};
		var _G = function(x, y, z) {
			return (x & z) | (y & (~z));
		};
		var _H = function(x, y, z) {
			return (x ^ y ^ z);
		};
		var _I = function(x, y, z) {
			return (y ^ (x | (~z)));
		};

		var _FF = function(a, b, c, d, x, s, ac) {
			a = addUnsigned(a, addUnsigned(addUnsigned(_F(b, c, d), x), ac));
			return addUnsigned(rotateLeft(a, s), b);
		};

		var _GG = function(a, b, c, d, x, s, ac) {
			a = addUnsigned(a, addUnsigned(addUnsigned(_G(b, c, d), x), ac));
			return addUnsigned(rotateLeft(a, s), b);
		};

		var _HH = function(a, b, c, d, x, s, ac) {
			a = addUnsigned(a, addUnsigned(addUnsigned(_H(b, c, d), x), ac));
			return addUnsigned(rotateLeft(a, s), b);
		};

		var _II = function(a, b, c, d, x, s, ac) {
			a = addUnsigned(a, addUnsigned(addUnsigned(_I(b, c, d), x), ac));
			return addUnsigned(rotateLeft(a, s), b);
		};

		var convertToWordArray = function(s) {
			var lWordCount;
			var lMessageLength = s.length;
			var lNumberOfWords_temp1 = lMessageLength + 8;
			var lNumberOfWords_temp2 = (lNumberOfWords_temp1 - (lNumberOfWords_temp1 % 64)) / 64;
			var lNumberOfWords = (lNumberOfWords_temp2 + 1) * 16;
			var lWordArray = new Array(lNumberOfWords - 1);
			var lBytePosition = 0;
			var lByteCount = 0;
			while (lByteCount < lMessageLength) {
				lWordCount = (lByteCount - (lByteCount % 4)) / 4;
				lBytePosition = (lByteCount % 4) * 8;
				lWordArray[lWordCount] = (lWordArray[lWordCount] | (s.charCodeAt(lByteCount) << lBytePosition));
				lByteCount++;
			}
			lWordCount = (lByteCount - (lByteCount % 4)) / 4;
			lBytePosition = (lByteCount % 4) * 8;
			lWordArray[lWordCount] = lWordArray[lWordCount] | (0x80 << lBytePosition);
			lWordArray[lNumberOfWords - 2] = lMessageLength << 3;
			lWordArray[lNumberOfWords - 1] = lMessageLength >>> 29;
			return lWordArray;
		};

		var wordToHex = function(lValue) {
			var wordToHexValue = "", wordToHexValue_temp = "", lByte, lCount;
			for (lCount = 0; lCount <= 3; lCount++) {
				lByte = (lValue >>> (lCount * 8)) & 255;
				wordToHexValue_temp = "0" + lByte.toString(16);
				wordToHexValue = wordToHexValue + wordToHexValue_temp.substr(wordToHexValue_temp.length - 2, 2);
			}
			return wordToHexValue;
		};

		var x = [], k, AA, BB, CC, DD, a, b, c, d, S11 = 7, S12 = 12, S13 = 17, S14 = 22, S21 = 5, S22 = 9, S23 = 14, S24 = 20, S31 = 4, S32 = 11, S33 = 16, S34 = 23, S41 = 6, S42 = 10, S43 = 15, S44 = 21;

		str = TG_MD5.utf8_encode(str);
		x = convertToWordArray(str);
		a = 0x67452301;
		b = 0xEFCDAB89;
		c = 0x98BADCFE;
		d = 0x10325476;

		xl = x.length;
		for (k = 0; k < xl; k += 16) {
			AA = a;
			BB = b;
			CC = c;
			DD = d;
			a = _FF(a, b, c, d, x[k + 0], S11, 0xD76AA478);
			d = _FF(d, a, b, c, x[k + 1], S12, 0xE8C7B756);
			c = _FF(c, d, a, b, x[k + 2], S13, 0x242070DB);
			b = _FF(b, c, d, a, x[k + 3], S14, 0xC1BDCEEE);
			a = _FF(a, b, c, d, x[k + 4], S11, 0xF57C0FAF);
			d = _FF(d, a, b, c, x[k + 5], S12, 0x4787C62A);
			c = _FF(c, d, a, b, x[k + 6], S13, 0xA8304613);
			b = _FF(b, c, d, a, x[k + 7], S14, 0xFD469501);
			a = _FF(a, b, c, d, x[k + 8], S11, 0x698098D8);
			d = _FF(d, a, b, c, x[k + 9], S12, 0x8B44F7AF);
			c = _FF(c, d, a, b, x[k + 10], S13, 0xFFFF5BB1);
			b = _FF(b, c, d, a, x[k + 11], S14, 0x895CD7BE);
			a = _FF(a, b, c, d, x[k + 12], S11, 0x6B901122);
			d = _FF(d, a, b, c, x[k + 13], S12, 0xFD987193);
			c = _FF(c, d, a, b, x[k + 14], S13, 0xA679438E);
			b = _FF(b, c, d, a, x[k + 15], S14, 0x49B40821);
			a = _GG(a, b, c, d, x[k + 1], S21, 0xF61E2562);
			d = _GG(d, a, b, c, x[k + 6], S22, 0xC040B340);
			c = _GG(c, d, a, b, x[k + 11], S23, 0x265E5A51);
			b = _GG(b, c, d, a, x[k + 0], S24, 0xE9B6C7AA);
			a = _GG(a, b, c, d, x[k + 5], S21, 0xD62F105D);
			d = _GG(d, a, b, c, x[k + 10], S22, 0x2441453);
			c = _GG(c, d, a, b, x[k + 15], S23, 0xD8A1E681);
			b = _GG(b, c, d, a, x[k + 4], S24, 0xE7D3FBC8);
			a = _GG(a, b, c, d, x[k + 9], S21, 0x21E1CDE6);
			d = _GG(d, a, b, c, x[k + 14], S22, 0xC33707D6);
			c = _GG(c, d, a, b, x[k + 3], S23, 0xF4D50D87);
			b = _GG(b, c, d, a, x[k + 8], S24, 0x455A14ED);
			a = _GG(a, b, c, d, x[k + 13], S21, 0xA9E3E905);
			d = _GG(d, a, b, c, x[k + 2], S22, 0xFCEFA3F8);
			c = _GG(c, d, a, b, x[k + 7], S23, 0x676F02D9);
			b = _GG(b, c, d, a, x[k + 12], S24, 0x8D2A4C8A);
			a = _HH(a, b, c, d, x[k + 5], S31, 0xFFFA3942);
			d = _HH(d, a, b, c, x[k + 8], S32, 0x8771F681);
			c = _HH(c, d, a, b, x[k + 11], S33, 0x6D9D6122);
			b = _HH(b, c, d, a, x[k + 14], S34, 0xFDE5380C);
			a = _HH(a, b, c, d, x[k + 1], S31, 0xA4BEEA44);
			d = _HH(d, a, b, c, x[k + 4], S32, 0x4BDECFA9);
			c = _HH(c, d, a, b, x[k + 7], S33, 0xF6BB4B60);
			b = _HH(b, c, d, a, x[k + 10], S34, 0xBEBFBC70);
			a = _HH(a, b, c, d, x[k + 13], S31, 0x289B7EC6);
			d = _HH(d, a, b, c, x[k + 0], S32, 0xEAA127FA);
			c = _HH(c, d, a, b, x[k + 3], S33, 0xD4EF3085);
			b = _HH(b, c, d, a, x[k + 6], S34, 0x4881D05);
			a = _HH(a, b, c, d, x[k + 9], S31, 0xD9D4D039);
			d = _HH(d, a, b, c, x[k + 12], S32, 0xE6DB99E5);
			c = _HH(c, d, a, b, x[k + 15], S33, 0x1FA27CF8);
			b = _HH(b, c, d, a, x[k + 2], S34, 0xC4AC5665);
			a = _II(a, b, c, d, x[k + 0], S41, 0xF4292244);
			d = _II(d, a, b, c, x[k + 7], S42, 0x432AFF97);
			c = _II(c, d, a, b, x[k + 14], S43, 0xAB9423A7);
			b = _II(b, c, d, a, x[k + 5], S44, 0xFC93A039);
			a = _II(a, b, c, d, x[k + 12], S41, 0x655B59C3);
			d = _II(d, a, b, c, x[k + 3], S42, 0x8F0CCC92);
			c = _II(c, d, a, b, x[k + 10], S43, 0xFFEFF47D);
			b = _II(b, c, d, a, x[k + 1], S44, 0x85845DD1);
			a = _II(a, b, c, d, x[k + 8], S41, 0x6FA87E4F);
			d = _II(d, a, b, c, x[k + 15], S42, 0xFE2CE6E0);
			c = _II(c, d, a, b, x[k + 6], S43, 0xA3014314);
			b = _II(b, c, d, a, x[k + 13], S44, 0x4E0811A1);
			a = _II(a, b, c, d, x[k + 4], S41, 0xF7537E82);
			d = _II(d, a, b, c, x[k + 11], S42, 0xBD3AF235);
			c = _II(c, d, a, b, x[k + 2], S43, 0x2AD7D2BB);
			b = _II(b, c, d, a, x[k + 9], S44, 0xEB86D391);
			a = addUnsigned(a, AA);
			b = addUnsigned(b, BB);
			c = addUnsigned(c, CC);
			d = addUnsigned(d, DD);
		}

		var temp = wordToHex(a) + wordToHex(b) + wordToHex(c) + wordToHex(d);

		return temp.toLowerCase();
	}
};

/* Add this setting to game_configuration:
 * 	siteLock: {
		domainMD5Hashed: "8fac1f8e1858745191f2db8612565d3d" // put the MD5 hash of your website origin here (An origin is for example: "http://www.google.nl")
	}
 *
 */
/* exported TG_SiteLock */
var TG_SiteLock = {
	checkSiteLock: function(domainMD5Hashed) {
		"use strict";
		var i, j, subStrings, begin, stringToCompare, o, origin, localString = "file://";
		origin = (window.location.origin)? window.location.origin : window.location.protocol + "//" + window.location.hostname;
		if(origin.indexOf(localString) === 0 && domainMD5Hashed === TG_MD5.createMD5Hash(localString)){
			return true;
		}
		subStrings = origin.split(".");
		begin = subStrings.shift().split("://");
		begin[0] += "://";
		subStrings = begin.concat(subStrings);
		stringToCompare = "";
		for(i=subStrings.length - 1; i >= 0; i--){
			stringToCompare = subStrings[i] + ((i > 0 && i < subStrings.length - 1)? "." : "") + stringToCompare;
			if(TG_MD5.createMD5Hash(stringToCompare) === domainMD5Hashed){
				return true;
			}
		}
		
		return false;
	}
};
//******************************************************
//*****    Script Resource: TG_HighScores
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

var TG_HighScores = {};

TG_HighScores._postJSON = function(url, paramString, callback) {
	"use strict";
	//TODO try using XDomainRequest object instead
	try { //might fail in IE9
		var request = new XMLHttpRequest();
		request.open("POST", url);
		request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
		request.onload = function() {
			if (request.getResponseHeader("Content-Type") === "application/json") {
				callback(JSON.parse(request.responseText));
			}
		};
		request.onerror = function(e) {
			console.log("error: " + e);
		};
		request.send(paramString);
	} catch (e) {
	}
};

TG_HighScores.checkActive = function(callback) {
	"use strict";
	TG_HighScores._postJSON(TG.config.highScoreSystem.url + "api", "call=api_is_valid", function(response) {
		callback(response["is_valid"]);
	});
};

TG_HighScores.checkHighScore = function(score, callback) {
	"use strict";
	TG_HighScores._postJSON(TG.config.highScoreSystem.url + "api", "call=is_highscore&score=" + score, function(response) {
		if (response["position"] >= 0) {
			TG_HighScores.highScoreId = response["code"];
			callback(TG_HighScores.highScoreId !== undefined);
		}
		else {
			callback(false);
		}
	});
};

TG_HighScores.submitHighScore = function(score) {
	"use strict";
	if (TG_HighScores.highScoreId !== undefined) {
		TG.hooks.openUrl(TG.config.highScoreSystem.url + "submit/" + TG_HighScores.highScoreId + "/" + score);
	}
	else {
		TG.hooks.openUrl(TG.config.highScoreSystem.url + "submit/");
	}
};

//******************************************************
//*****    Script Resource: TG_ScoreObject
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG_StatObjectFactory = {};

TG_StatObjectFactory.createTotalScoreStat = function(type){
	"use strict";
	return new TG_StatObject("totalScore", type, "levelEndScreenTotalScore_"+type, 0, 0, true, true);
};

TG_StatObjectFactory.createHighScoreStat = function(type){
	"use strict";
	return new TG_StatObject("highScore", type, "levelEndScreenHighScore_"+type, TG.controller.getLevelHighScore(), TG.controller.getLevelHighScore(), true);
};

TG_StatObjectFactory.createBonusStat = function(id, type, key, value, visible){
	"use strict";
	var updateFunction;
	if(TG.gameFlowSettings.computeHighScore === "max"){
		updateFunction = function(score){return score + value;};
	}else {
		updateFunction = function(score){return score - value;};
	}
	return new TG_StatObject(id, type, key, 0, value, visible, true, updateFunction);
};

TG_StatObjectFactory.createPenaltyStat = function(id, type, key, value, visible){
	"use strict";
	var updateFunction;
	if(TG.gameFlowSettings.computeHighScore === "max"){
		updateFunction = function(score){return score - value;};
	}else {
		updateFunction = function(score){return score + value;};
	}
	return new TG_StatObject(id, type, key, 0, value, visible, true, updateFunction);
};


TG_StatObject = function(id, type, key, endValue, startValue, visible, animate, updateFunction, statToUpdate){
	"use strict";
	this.id = id;
	this.type = type;
	this.key = key;
	this.endValue = endValue;
	this.startValue = (startValue !== undefined)? startValue : this.endValue;
	this.visible = (visible !== undefined)? visible : true;
	this.animate = (animate !== undefined)? animate : this.endValue !== this.startValue;
	this.updateFunction = updateFunction;
	this.statToUpdate = (statToUpdate !== undefined)? statToUpdate : "totalScore";
	switch(this.type){
		case "text": this.toString = function(value){return value;}; break;
		case "number": this.toString = function(value){return value +"";}; break;
		case "time": this.toString = function(value){return TG.functions.timeToStr(value*1000);};
	}
};

TG_StatObject.prototype.clone = function(){
	"use strict";
	return new TG_StatObject(this.id, this.type, this.key, this.endValue, this.startValue, this.visible, this.animate, this.updateFunction, this.statToUpdate);
};

//******************************************************
//*****    Script Resource: TG_version
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.version = TG.version || {};
TG.version["tg"] = "2.13.0";







//******************************************************
//*****    Script Resource: Behaviors
//******************************************************

var Behaviors = {};

Behaviors._methodDispatchers = {};

Behaviors._createMethodDispatcher = function(func){
	"use strict";

	if (Behaviors._methodDispatchers[func]) {
		return Behaviors._methodDispatchers[func];
	}

	var result = function() {

		var i;
		for (i = this.behaviors.length - 1; i >= 0; --i) {
			if (this.behaviors[i].replace && this.behaviors[i].replace[func]) {
				return this.behaviors[i].replace[func].apply(this, arguments);
			}
		}

		var result = undefined;

		for (i = 0; i < this.behaviors.length; ++i) {
			if (this.behaviors[i].prototype[func]) {
				result = this.behaviors[i].prototype[func].apply(this, arguments);
			}
		}
		return result;
	};

	Behaviors._methodDispatchers[func] = result;

	return result;
};

Behaviors.addBehavior = function(obj, behaviorType) {
	"use strict";

	if (typeof obj.behaviors === "undefined") {
		obj.behaviors = [];
	}

	for (var i = 0; i < obj.behaviors.length; ++i) {
		if (obj.behaviors[i] === behaviorType) {
			if (behaviorType.prototype.onAdd) {
				behaviorType.prototype.onAdd.apply(this);
			}
			return;
		}
	}

	obj.behaviors.push(behaviorType);

	for ( var func in behaviorType.prototype) {
		if (!obj[func]) {
			obj[func] = Behaviors._createMethodDispatcher(func);
		}
	}

	behaviorType.call(obj);

};

Behaviors.removeBehaviorsWithTag = function(obj, tagsQuery) {
	"use strict";

	if (typeof tagsQuery !== "object" || tagsQuery.length === undefined) {
		tagsQuery = [tagsQuery];
	}

	for (var i = 0; i < obj.behaviors.length; ++i) {
		if (obj.behaviors[i].tags) {
			var tags = obj.behaviors[i].tags;
			var foundTag = false;
			for (var j=0;j<tags.length;++j) {
				for (var k=0;k<tagsQuery.length;++k) {
					if (tags[j] === tagsQuery[k]) {
						obj.behaviors.splice(i, 1);
						--i;
						foundTag = true;
						break;
					}
				}
				if (foundTag) {
					break;
				}
			}
		}
	}
	return false;
};


Behaviors.hasBehavior = function(obj, behaviorType) {
	"use strict";
	for (var i = 0; i < obj.behaviors.length; ++i) {
		if (obj.behaviors[i] === behaviorType) {
			return true;
		}
	}
	return false;
};

Behaviors.hasBehaviorTag = function(obj, tagsQuery) {
	"use strict";

	if (typeof tagsQuery !== "object" || tagsQuery.length === undefined) {
		tagsQuery = [tagsQuery];
	}

	for (var i = 0; i < obj.behaviors.length; ++i) {
		if (obj.behaviors[i].tags) {
			var tags = obj.behaviors[i].tags;
			for (var j=0;j<tags.length;++j) {
				for (var k=0;k<tagsQuery.length;++k) {
					if (tags[j] === tagsQuery[k]) {
						return true;
					}
				}
			}
		}
	}
	return false;
};

Behaviors.removeBehavior = function(obj, behaviorType) {
	"use strict";
	for (var i = 0; i < obj.behaviors.length; ++i) {
		if (obj.behaviors[i] === behaviorType) {
			obj.behaviors.splice(i, 1);
			return;
		}
	}
};

Behaviors.removeAllBehaviors = function(obj, exceptType) {
	"use strict";

	if (!obj.behaviors) {
		return;
	}

	var i = 0;

	while (i < obj.behaviors.length) {
		if (obj.behaviors[i] !== exceptType) {
			obj.behaviors.splice(i, 1);
		}
		else {
			++i;
		}
	}

};
//******************************************************
//*****    Script Resource: Distribution
//******************************************************


/**
 * @param {number|Array} eventsToChance
 * Array of chances, or a single number with which a uniform distribution will be created with n event types
 *
 * @param {boolean} perfectDistribution
 * Should the events be perfectly distributed as described by the chances at each moment?
 *
 * @param {boolean} preventRepeats
 * Can an event happen twice in a row
 *
 */
var Distribution = function(eventsToChance, perfectDistribution, preventRepeats) {
	"use strict";

	this.chances = [];
	this.eventsGenerated = [];
	this.perfectDistribution = false;
	this.preventRepeats = false || preventRepeats;
	this.randomNoGenerator = new tbRandom();

	this.oneTolastEvent = -1;
	this.lastEvent = -1;

	this.updateChances(eventsToChance);

	if (perfectDistribution) {
		this.perfectDistribution = true;
	}

	this.resetHistory();
};

Distribution.prototype.setRandomNoGenerator = function(randomNoGenerator) {
	"use strict";
	this.randomNoGenerator = randomNoGenerator;
};

Distribution.prototype.updateChances = function(eventsToChance) {
	"use strict";
	var totalChance = 0;
	var i;

	if (typeof eventsToChance === "number") {
		for (i = 0; i < eventsToChance; ++i) {
			this.chances[i] = 1 / eventsToChance;
		}
		return;
	}

	for (i = 0; i < eventsToChance.length; ++i) {
		totalChance += eventsToChance[i];
		if (eventsToChance[i] < 0) {
			throw "Invalid distribution, chance < 0";
		}
	}

	if (totalChance === 0) {
		for (i = 0; i < eventsToChance.length; ++i) {
			this.chances[i] = 0;
		}
	}
	else {
		// normalize chances
		for (i = 0; i < eventsToChance.length; ++i) {
			this.chances[i] = eventsToChance[i] / totalChance;
		}
	}
};

Distribution.prototype.combine = function(otherDistribution) {
	"use strict";

	var totalChance = 0;

	for (var i = 0; i < this.chances.length; ++i) {
		this.chances[i] *= otherDistribution.chances[i];
		totalChance += this.chances[i];
	}

	if (totalChance === 0) {
		for (i = 0; i < this.chances.length; ++i) {
			this.chances[i] = 0;
		}
	}
	else {
		// normalize chances
		for (i = 0; i < this.chances.length; ++i) {
			this.chances[i] = this.chances[i] / totalChance;
		}
	}
};

Distribution.prototype.resetHistory = function() {
	"use strict";
	this.totalEventsGenerated = 0;
	this.oneTolastEvent = -1;
	this.lastEvent = -1;

	if (this.perfectDistribution) {
		for (var i = 0; i < this.chances.length; ++i) {
			this.eventsGenerated[i] = 0;
		}
	}
};

Distribution.prototype.generateEvent = function(exceptFor, onlyFor) {
	"use strict";
	var subsetTotalEventsGenerated, subsetTotalChanceSum, i;
	//check if subsets are requested
	if(!(exceptFor instanceof Array)){
		exceptFor = [exceptFor];
	}
	if(onlyFor && !(onlyFor instanceof Array)){
		onlyFor = [onlyFor];
	}
	//normalize chance sum and total events generated to subset
	subsetTotalEventsGenerated = 0;
	subsetTotalChanceSum = 0;
	for(i=0; i<this.chances.length; i++){
		if(exceptFor.indexOf(i) < 0 && (!onlyFor || onlyFor.indexOf(i) >= 0)){
			if (this.perfectDistribution) {
				subsetTotalEventsGenerated += this.eventsGenerated[i];
			}
			subsetTotalChanceSum += this.chances[i];
		}
	}

	if(subsetTotalChanceSum === 0){
		this.oneTolastEvent = this.lastEvent;
		this.lastEvent = -1;
		return null;
	}

	while (true) {
		var diceRoll = this.randomNoGenerator.random(1);
		var chanceSum = 0;
		var skippedEventsForPerfectDist = false;
		for (i = 0; i < this.chances.length; ++i) {

			if(exceptFor.indexOf(i) >= 0) {
				continue;
			}

			if (onlyFor && onlyFor.indexOf(i) < 0) {
				continue;
			}

			if (this.preventRepeats && this.lastEvent === i) {
				continue;
			}

			var distChance = this.chances[i]/subsetTotalChanceSum;
			if (this.perfectDistribution){ //&& exceptFor === undefined && onlyFor === undefined) {
				var chanceBasedOnPreviousEvents = this.eventsGenerated[i] / subsetTotalEventsGenerated;//this.totalEventsGenerated;
				if (distChance < chanceBasedOnPreviousEvents) {
					skippedEventsForPerfectDist = true;
					continue;
				}
			}
			chanceSum += distChance;
			if (chanceSum > diceRoll) {

				if (this.perfectDistribution) {
					++this.totalEventsGenerated;
					++this.eventsGenerated[i];
				}

				this.oneTolastEvent = this.lastEvent;
				this.lastEvent = i;
				return i;
			}
		}

		if (chanceSum === 0) {
			if (skippedEventsForPerfectDist) {
				subsetTotalEventsGenerated++;
			}
			else {
				this.oneTolastEvent = this.lastEvent;
				this.lastEvent = -1;
				return null;
			}
		}
	}
};

Distribution.prototype.cancelEvent = function(eventIndex) {
	"use strict";
	this.lastEvent = this.oneTolastEvent;
	this.oneTolastEvent = -1;

	if (this.perfectDistribution) {
		--this.totalEventsGenerated;
		--this.eventsGenerated[eventIndex];
	}
};

//******************************************************
//*****    Script Resource: Collections
//******************************************************




function ArrayList() {
	"use strict";
	this.data = [];
	this.count = 0;
}

ArrayList.prototype.push = function(item) {
	"use strict";
	this.data[this.count] = item;
	this.data[this.count + 1] = undefined;
	++this.count;
};

ArrayList.prototype.pop = function() {
	"use strict";
	var result = this.data[this.count - 1];
	--this.count;
	this.data[this.count] = undefined;
	return result;
};

ArrayList.prototype.clear = function() {
	"use strict";
	this.data[0] = undefined;
	this.count = 0;
};

function Queue() {
	"use strict";

	this.inbox = new ArrayList();
	this.outbox = new ArrayList();

	this.count = 0;
}

Queue.prototype.clear = function() {
	"use strict";
	this.inbox.clear();
	this.outbox.clear();
	this.count = 0;
};

Queue.prototype.queue = function(item) {
	"use strict";
	this.inbox.push(item);
	++this.count;
};

Queue.prototype.dequeue = function() {
	"use strict";
	if (this.outbox.count === 0) {
		while (this.inbox.count !== 0) {
			this.outbox.push(this.inbox.pop());
		}
	}
	--this.count;
	return this.outbox.pop();
};
//******************************************************
//*****    Script Resource: RandomSounds
//******************************************************

/* global
	Distribution
 */

/**
 * Arguments should be a list of sounds
 */
function RandomSounds() {
	"use strict";

	this.sounds = [];

	for (var i = 0; i < arguments.length; ++i) {
		this.sounds.push(arguments[i]);
	}

	this.distribution = new Distribution(this.sounds.length, true, true);

}

RandomSounds.maxInstances = {};

RandomSounds.prototype.setMaxInstances = function(tag, count) {
	"use strict";
	RandomSounds.maxInstances[tag] = count;
	this.tag = tag;

	for (var i = 0; i < this.sounds.length; ++i) {
		this.sounds[i].addTags(tag);
	}
};

RandomSounds.prototype.play = function() {
	"use strict";

	if (this.tag !== undefined) {
		if (tbAudio.getInstances(this.tag).length >= RandomSounds.maxInstances[this.tag]) {
			return;
		}
	}

	var sound = this.sounds[this.distribution.generateEvent()];

	if (sound) {
		return tbAudio.play(sound);
	}

	return undefined;
};
//******************************************************
//*****    Script Resource: DictUtils
//******************************************************

/* globals
	tbBrowser
 */

function DictUtils() {
	"use strict";
}

DictUtils.touchRegex = /#touch{([^}]*)}{([^}]*)}/gim;

DictUtils.lookup = function(word) {
	"use strict";
	/** @type String */
	var result = TG.dictionary.getText(word, word);

	if (tbBrowser.features.tablet || tbBrowser.features.mobile) {
		result = result.replace(DictUtils.touchRegex, "$2");
	}
	else {
		result = result.replace(DictUtils.touchRegex, "$1");
	}

	return result;
};
//******************************************************
//*****    Script Resource: GameUIControllerBase
//******************************************************

// ***************************************
// Definition of game token GameUIControllerBase
// ***************************************

/* global
	 tbGame,
	 GameUIImage,
	 GameUIText,
	 GameUISettingsOffsets
 */

function GameUIBounds(top, left, bottom, right) {
	"use strict";
	this.top = top;
	this.left = left;
	this.bottom = bottom;
	this.right = right;
}

GameUIBounds.prototype.getWidth = function() {
	"use strict";
	return this.right - this.left;
};

GameUIBounds.prototype.getHeight = function() {
	"use strict";
	return this.bottom - this.top;
};

function GameUIInputProxy(controller) {
	"use strict";
	this.depth = -99;
	TG.functions.setLayerAndCanvas(this, TG.frameworkUILayer);

	this.active = true;
	this.visible = false;

	this.controller = controller;

	tbGame.addToken(this);
}

GameUIInputProxy.prototype.handleKeyDown = function(keycode) {
	"use strict";
	return this.controller.handleKeyDownProxy(keycode);
};

GameUIInputProxy.prototype.handleKeyUp = function(keycode) {
	"use strict";
	return this.controller.handleKeyUpProxy(keycode);
};


GameUIInputProxy.prototype.handleTouchDown = function(finger, x, y) {
	"use strict";
	return this.controller.handleTouchDownProxy(finger, x, y);
};

GameUIInputProxy.prototype.handleTouchUp = function(finger, x, y) {
	"use strict";
	return this.controller.handleTouchUpProxy(finger, x, y);
};


function GameUIControllerBase() {
	"use strict";
	// Properties used by the engine
	this.depth = 0;
	this.layer = 0;
	this.visible = false;
	this.active = false;
	this.redraw = false;

	this.drawDebugInfo = false;

	this._components = [];

	this.entrance = {};
	this.entrance.running = false;
	this.exit = {};
	this.exit.running = false;

	this.paused = false;
	/** @type tbSurface */
	this.staticSurface = new tbSurface(0, 0);

	this.staticSurfaceOffset_x = 0;
	this.staticSurfaceOffset_y = 0;

	this.keyCtrlDown = false;
	this.keyAltDown = false;

	this.userComponentMove = null;
	this.userComponentMove_touchstart_x = -1;
	this.userComponentMove_touchstart_y = -1;
	this.userComponentMove_touchfinger = -1;
	this.userComponentMove_touching = false;

	this.offset_x = 0;
	this.offset_y = 0;

	this.inputProxy = null;
}

GameUIControllerBase.prototype.handleAdd = function() {
	"use strict";
	this.inputProxy = new GameUIInputProxy(this);
};

GameUIControllerBase.prototype.handleRemove = function() {
	"use strict";

	if (this.inputProxy) {
		tbGame.removeToken(this.inputProxy);
		this.inputProxy = null;
	}
};

GameUIControllerBase.prototype.readComponents = function(componentDefinitions, result) {
	"use strict";

	for ( var key in componentDefinitions) {
		var item = componentDefinitions[key];

		if (item.image) {
			result[key] = new GameUIImage(this, item);
		}
		else if (item.text_key) {
			result[key] = new GameUIText(this, TG.dictionary.getText(item.text_key, "<" + item.text_key + ">"), item);
		}
		else if (item.textKey) {
			result[key] = new GameUIText(this, TG.dictionary.getText(item.textKey, "<" + item.textKey + ">"), item);
		}
		else if (item.text) {
			result[key] = new GameUIText(this, item.text, item);
		}
	}

};

GameUIControllerBase.prototype.addComponent = function(component) {
	"use strict";
	this._components.push(component);
};

GameUIControllerBase.prototype.startEntranceAnimation = function(callback) {
	"use strict";
	this.entrance.running = true;
	this.entrance.timer = 0;
	this.entrance.duration = 0;
	this.entrance.callback = callback;
	for (var j = 0; j < this._components.length; ++j) {
		this._components[j].startEntranceAnimation();
		this.entrance.duration = Math.max(this.entrance.duration, this._components[j].getEntranceDuration());
	}
};

GameUIControllerBase.prototype.setLayer = function(layerId) {
	"use strict";
	this.canvas.drawTokens = true;
	TG.functions.setLayerAndCanvas(this, layerId);
};

GameUIControllerBase.prototype.setOffset = function(x, y) {
	"use strict";
	this.offset_x = x;
	this.offset_y = y;
};

GameUIControllerBase.prototype.startExitAnimation = function(callback, canUseEntranceAnimation) {
	"use strict";
	this.exit.running = true;
	this.exit.timer = 0;
	this.exit.duration = 0;
	this.exit.callback = callback;
	for (var j = 0; j < this._components.length; ++j) {
		this._components[j].startExitAnimation(canUseEntranceAnimation);
		this.exit.duration = Math.max(this.exit.duration, this._components[j].getExitDuration());
	}
};

GameUIControllerBase.prototype.calculateBounds = function(result) {
	"use strict";

	if (!result) {
		result = new GameUIBounds(Number.MAX_VALUE,Number.MAX_VALUE,-Number.MAX_VALUE,-Number.MAX_VALUE);
	}

	result.left = Number.MAX_VALUE;
	result.top = Number.MAX_VALUE;
	result.right = -Number.MAX_VALUE;
	result.bottom = -Number.MAX_VALUE;

	for (var j = 0; j < this._components.length; ++j) {
		var component = this._components[j];

		if (component.x < result.left) {
			result.left = component.x;
		}
		if (component.y < result.top) {
			result.top = component.y;
		}
		var right = component.x + component.width;
		var bottom = component.y + component.height;

		if (right > result.right) {
			result.right = right;
		}
		if (bottom > result.bottom) {
			result.bottom = bottom;
		}
	}
	return result;
};

GameUIControllerBase.prototype.setPaused = function(paused) {
	"use strict";

	if (this.paused === paused) {
		return;
	}

	this.paused = paused;

	if (this.paused) {
		var bounds = this.calculateBounds();

		var width = bounds.getWidth();
		var height = bounds.getHeight();

		if (this.staticSurface) {

			if (this.staticSurface.width === width && this.staticSurface.height === height) {
				// keep the surface, it is big enough
			}
			else {
				this.staticSurface = null;
			}
		}

		if (!this.staticSurface) {
			this.staticSurface = new tbSurface(width, height);
		}
		this.staticSurface.setTarget();
		this.staticSurface.clear();

		this.drawWithOffset(-bounds.left, -bounds.top);

		this.staticSurface.resetTarget();

		this.staticSurfaceOffset_x = bounds.left;
		this.staticSurfaceOffset_y = bounds.top;
	}

};

GameUIControllerBase.prototype._updateAnimation = function(animData, timePassed) {
	"use strict";

	if (animData.running) {
		animData.timer += timePassed;
		if (animData.timer >= animData.duration) {
			animData.running = false;
			if (animData.callback) {
				animData.callback();
			}
		}
	}
};

// handleBeginStep is called each step before processing other events.
GameUIControllerBase.prototype.handleBeginStep = function(timePassed) {
	"use strict";

	this._updateAnimation(this.entrance, timePassed);
	this._updateAnimation(this.exit, timePassed);

	for (var j = 0; j < this._components.length; ++j) {
		this._components[j].handleBeginStep(timePassed);
	}

	if (this.userComponentMove && this.userComponentMove_touching) {

		var touch_x = tbGame.touchX(this.userComponentMove_touchfinger);
		var touch_y = tbGame.touchY(this.userComponentMove_touchfinger);

		var backgroundCanvas = TG.functions.getCanvas(TG.gameBackgroundLayer);
		if (this.canvas === backgroundCanvas) {
			this.userComponentMove.handleDrawGameBackground(this.offset_x + TG.gameUIOffsetX, this.offset_y + TG.gameUIOffsetY);
		}

		var distance_x = touch_x - this.userComponentMove_touchstart_x;
		var distance_y = touch_y - this.userComponentMove_touchstart_y;
		this.userComponentMove.x += distance_x;
		this.userComponentMove.y += distance_y;

		this.userComponentMove.userComponentMoveDistance.x += distance_x;
		this.userComponentMove.userComponentMoveDistance.y += distance_y;

		this.userComponentMove_touchstart_x = touch_x;
		this.userComponentMove_touchstart_y = touch_y;

		this.redraw = true;
	}
};


GameUIControllerBase.prototype.handleTouchDownProxy = function(finger, x, y) {
	"use strict";
	var j;
	if (testMode && this.keyAltDown && this.keyCtrlDown) {
		for (j = this._components.length - 1; j >= 0; --j) {
			if (this._components[j].visible) {
				if (this._components[j].isPointInside(x - TG.gameUIOffsetX, y - TG.gameUIOffsetY)) {
					this.userComponentMove = this._components[j];
					this.userComponentMove_touchstart_x = x;
					this.userComponentMove_touchstart_y = y;
					this.userComponentMove_touchfinger = finger;
					this.userComponentMove_touching = true;

					if (!this.userComponentMove.userComponentMoveDistance) {
						this.userComponentMove.userComponentMoveDistance = new tbVector(0, 0);
					}
					return true;
				}
			}
		}
		this.userComponentMove = null;
	}
	for (j = 0; j < this._components.length; ++j) {
		if (this._components[j].handleTouchDown) {
			if (this._components[j].handleTouchDown(finger, x, y)) {
				return true;
			}
		}
	}
	return false;
};

GameUIControllerBase.prototype.handleTouchUpProxy = function(finger, x, y) {
	"use strict";

	if (this.userComponentMove && finger === this.userComponentMove_touchfinger) {

		var originalX = this.userComponentMove.settings.x;
		var originalY = this.userComponentMove.settings.y;

		if (this.userComponentMove.originalSettings) {
			originalX = this.userComponentMove.originalSettings.x;
			originalY = this.userComponentMove.originalSettings.y;
		}

		if (GameUISettingsOffsets) {
			console.log("Component:\n"+
					" x: tgScale(" + (originalX + this.userComponentMove.userComponentMoveDistance.x - GameUISettingsOffsets.X) + ") + GameUISettingsOffsets.X,\n"+
					" y: tgScale(" + (originalY + this.userComponentMove.userComponentMoveDistance.y - GameUISettingsOffsets.Y) + ") + GameUISettingsOffsets.Y,");
		}
		else {
			console.log("Component:\n"+
					" x: tgScale(" + (originalX + this.userComponentMove.userComponentMoveDistance.x) + "),\n"+
					" y: tgScale(" + (originalY + this.userComponentMove.userComponentMoveDistance.y) + "),");
		}

		this.userComponentMove_touching = false;
		return true;
	}
	else {
		for (var j = 0; j < this._components.length; ++j) {
			if (this._components[j].handleTouchUp) {
				if (this._components[j].handleTouchUp(finger, x, y)) {
					return true;
				}
			}
		}
	}
	return false;
};

GameUIControllerBase.prototype.handleKeyDownProxy = function(keycode) {
	"use strict";

	if (testMode) {

		var j, component;

		if (keycode === tbKeyCodes.vkD) {
			this.drawDebugInfo = !this.drawDebugInfo;

			for (j = 0; j < this._components.length; ++j) {
				component = this._components[j];

				if (component.redrawSurface) {
					component.redrawSurface();
				}
			}
		}

		if (keycode === tbKeyCodes.vkE) {
			for (j = 0; j < this._components.length; ++j) {
				component = this._components[j];

				if (component.showAttentionEffect) {
					component.showAttentionEffect();
				}
			}
		}
	}

	if (keycode === tbKeyCodes.vkControl) {
		this.keyCtrlDown = true;
	}
	if (keycode === tbKeyCodes.vkAlt) {
		this.keyAltDown = true;
	}
};

GameUIControllerBase.prototype.handleKeyUpProxy = function(keycode) {
	"use strict";

	if (keycode === tbKeyCodes.vkControl) {
		this.keyCtrlDown = false;
	}
	if (keycode === tbKeyCodes.vkAlt) {
		this.keyAltDown = false;
	}
};


// handleEndStep is called each step after processing other events (before
// draw).
GameUIControllerBase.prototype.handleEndStep = function(timePassed) {
	"use strict";
	if (this.redraw) {
		var backgroundCanvas = TG.functions.getCanvas(TG.gameBackgroundLayer);
		if (this.canvas !== backgroundCanvas) {
			this.canvas.drawTokens = this.redraw;
		}
		else {
			tbGraphics.setCanvas(backgroundCanvas);
			this.handleDraw();
		}
	}
};

GameUIControllerBase.prototype.drawWithOffset = function(offset_x, offset_y) {
	"use strict";
	var backgroundCanvas = TG.functions.getCanvas(TG.gameBackgroundLayer);
	var isDrawnOnBackgroundCanvas = backgroundCanvas === this.canvas;

	for (var j = 0; j < this._components.length; ++j) {
		var component = this._components[j];
		if (component.visible) {
			if (isDrawnOnBackgroundCanvas) {
				component.handleDrawGameBackground(offset_x, offset_y);
			}

			component.handleDraw(offset_x, offset_y);
		}
	}
};

// handleDraw is called at the end of a step to draw the token.
GameUIControllerBase.prototype.handleDraw = function() {
	"use strict";
	var gameUiCanvas = TG.functions.getCanvas(TG.gameUILayer);
	var isDrawnOnGameUiCanvas = gameUiCanvas === this.canvas;

	var gameUiOffset_x = 0;
	var gameUiOffset_y = 0;

	if (!isDrawnOnGameUiCanvas) {
		gameUiOffset_x = TG.gameUIOffsetX;
		gameUiOffset_y = TG.gameUIOffsetY;
	}

	if (this.paused) {
		this.staticSurface.drawSimple(this.staticSurfaceOffset_x + this.offset_x + gameUiOffset_x, this.staticSurfaceOffset_y + this.offset_y + gameUiOffset_y);
	}
	else {
		this.drawWithOffset(this.offset_x + gameUiOffset_x, this.offset_y + gameUiOffset_y);
	}
	this.redraw = false;

};

//******************************************************
//*****    Token Resource: GameUIController
//******************************************************

// ***************************************
// Definition of game token GameUIController
// ***************************************

/* global
	 TG,
	 tbGame,
	 gameUISettingsVar,
	 tbGraphics,
	 GameUIValue,
	 GameUIProgress,
	 GameUIRoundProgress,
	 GameUIImage,
	 GameUIDummy,
	 GameUIText,
	 GameUIButton,
	 s_ui_smiley_easy,
	 s_ui_smiley_medium,
	 s_ui_smiley_hard,
	 a_gameui_timewarning_second,
	 GameUIControllerBase,
	 Inheritance,
	 a_gameui_timewarning_second,
	 GameUIBounds
 */

/**
 * @enum GameUIAutoTime
 */
var GameUITimeMode = {
	NONE: 0,
	AUTO_INCREMENT: 1,
	AUTO_DECREMENT: 2
};

function GameUIRuntimeOptions() {
	"use strict";

	this.hiddenComponents = [];
	this.extraComponents = [];

	this.settingsObject = null;

	this.autoUpdateHighScore = undefined;

	this.gameCanBePaused = true;
}

// The constructor for the GameUIController game token.
/**
 * @constructor
 * @extends GameUIControllerBase
 */
function GameUIController(argument) {
	"use strict";

	/** @type GameUIRuntimeOptions */
	var runtimeOptions;

	// Check if the function is called correctly
	if (argument instanceof GameUIRuntimeOptions) {

		if (arguments.length !== 1) {
			throw "When using GameUIOptions as argument to GameUIController constructor you should not use extraComponents of gameUiSettings as parameters anymore.";
		}
		runtimeOptions = argument;
	}
	else {
		runtimeOptions = new GameUIRuntimeOptions();

		if (testMode) {
			console.warn("Please use the new GameUIRuntimeOptions object instead of the old calling convention. It will be deprecated. See wiki for documentation.");
		}

		runtimeOptions.hiddenComponents = arguments[0];
		runtimeOptions.extraComponents = arguments[1];
		runtimeOptions.settingsObject = arguments[2];
	}

	var hiddenComponents = null;
	var extraComponents = null;
	var gameUiSettings = null;

	hiddenComponents = runtimeOptions.hiddenComponents;
	extraComponents = runtimeOptions.extraComponents;

	gameUiSettings = runtimeOptions.settingsObject;

	this.runtimeOptions = runtimeOptions;

	if (this.runtimeOptions.autoUpdateHighScore === undefined) {
		this.runtimeOptions.autoUpdateHighScore = !TG.controller.getChallengeInformation();
	}

	GameUIControllerBase.apply(this, arguments);

	// Add it to the game engine
	tbGame.addToken(this);

	this.visible = true;
	this.active = true;

	extraComponents = extraComponents || [];
	hiddenComponents = hiddenComponents || [];

	var hasHiddenComponent = function (name) {
		for (var option in hiddenComponents) {
			if (option === name) {
				return true;
			}
			if (hiddenComponents[option] === name) {
				return true;
			}
		}
		return false;
	};

	var hasExtraComponent = function (name) {
		for (var option in extraComponents) {
			if (option === name) {
				return true;
			}
			if (extraComponents[option] === name) {
				return true;
			}
		}
		return false;
	};

	var shouldBeAdded = function(names, setting) {

		if (!setting) {
			return false;
		}

		var i = 0;

		if (typeof names === "string") {
			if (hasHiddenComponent(names)) {
				return false;
			}
		}
		else {
			for (i = 0; i < names.length; ++i) {
				if (hasHiddenComponent(names[i])) {
					return false;
				}
			}
		}

		if (setting.defaultHide) {
			if (typeof names === "string") {
				if (hasExtraComponent(names)) {
					return true;
				}
			}
			else {
				for (i = 0; i < names.length; ++i) {
					if (hasExtraComponent(names[i])) {
						return true;
					}
				}
			}
			return false;
		}

		return true;
	};

	// public properties
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideScore = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideLives = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideProgress = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideStars = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideStage = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideTime = null;
	/** @deprecated Use hiddenComponents in GameUIRuntimeOptions */
	this.forceHideHighscore = null;

	this.timeMode = GameUITimeMode.AUTO_DECREMENT;

	// private fields

	this.bonusActive = false;

	this._settings = gameUiSettings || gameUISettingsVar;

	this.defaultLayer = TG.gameUILayer;

	if (this._settings.layer !== undefined) {
		this.defaultLayer = this._settings.layer;
	}

	TG.functions.setLayerAndCanvas(this, this.defaultLayer);

	this._background_image_x = 0;
	this._background_image_y = 0;

	if (this._settings.background.image_x) {
		this._background_image_x = this._settings.background.image_x;
	}
	if (this._settings.background.image_y) {
		this._background_image_y = this._settings.background.image_y;
	}

	if (!this._settings.background.elements) {
		/**
		 * @type tbImage
		 */
		this._background_image = this._settings.background.image;
	}
	if (this._settings.background.backgroundElementsAsComponents) {
		this.readComponents(this._settings.background.elements, {});
		this._background_image = this._settings.background.image;
	}
	else {

		var backgroundImage = this._settings.background.image;

		var components = [];
		var virtualGameController = new GameUIControllerBase();

		virtualGameController.readComponents(this._settings.background.elements, components);


		if (!backgroundImage && this.layer === TG.gameBackgroundLayer) {
			tbGraphics.setCanvas(TG.functions.getCanvas(this.layer));

			virtualGameController.handleDraw();
		}
		else {
			this._background_image = new tbSurface(backgroundImage.width, backgroundImage.height);
			this._background_image.setTarget();

			backgroundImage.drawSimple(0, 0, 0);

			virtualGameController.drawWithOffset(-this._background_image_x, -this._background_image_y);

			this._background_image.resetTarget();
		}

	}

	this._mode = this._settings.mode;

	this.initialHighScore = 0;

	if (shouldBeAdded("score", this._settings.score)) {
		this.scoreComponent = new GameUIValue(this, this._settings.score, "SCORE", 0, true);
		if (this._settings.scoreImage) {
			this.scoreImageComponent = new GameUIImage(this, this._settings.scoreImage);
		}
	}
	else {
		this.scoreComponent = new GameUIDummy(this, 0);
	}

	if (shouldBeAdded("highScore", this._settings.highScore)) {
		this.highScoreComponent = new GameUIValue(this, this._settings.highScore, "HIGHSCORE", 0, false);
	}
	else {
		this.highScoreComponent = new GameUIDummy(this, 0);
	}

	if (shouldBeAdded("highScore", this._settings.highScoreImage)) {
		this.highScoreImageComponent = new GameUIImage(this, this._settings.highScoreImage);
	}

	if (shouldBeAdded(["stage", "level"], this._settings.stage)) {
		this.stageComponent = new GameUIValue(this, this._settings.stage, "STAGE", 0, false);
	}
	else {
		this.stageComponent = new GameUIDummy(this, 0);
	}

	if (shouldBeAdded("lives", this._settings.lives)) {
		this.livesComponent = new GameUIValue(this, this._settings.lives, "LIVES", 0, false);
	}
	else {
		this.livesComponent = new GameUIDummy(this, 0);
	}

	if (shouldBeAdded("time", this._settings.time)) {
		this.timeComponent = new GameUIValue(this, this._settings.time, "TIME", 0, false, function(value) {
			return GameUIController.timeToStr(value);
		});
	}
	else {
		this.timeComponent = new GameUIDummy(this, 0);
	}

	this.timeComponent.setValue(1000*6*60);

	if (this._settings.progress && this._settings.progressFill) {
		throw "Don't define both progress and progressFill in your game_ui settings";
	}

	if (shouldBeAdded("progress", this._settings.progress)) {
		if (this._settings.progress.round) {
			this.progressComponent = new GameUIRoundProgress(this, this._settings.progress);
		}
		else {
			this.progressComponent = new GameUIProgress(this, this._settings.progress);
		}
	}
	else {
		if (shouldBeAdded("progress", this._settings.progressFill)) {
			this.progressComponent = new GameUIProgress(this, this._settings.progressFill);
		}
		else {
			this.progressComponent = new GameUIDummy(this, 0);
		}
	}


	if (shouldBeAdded("lives", this._settings.heart)) {
		this.heartComponent = new GameUIImage(this, this._settings.heart);
	}

	if (shouldBeAdded("difficulty", this._settings.difficulty)) {
		this.difficultyComponent = new GameUIText(this, TG.controller.getDifficultyText().toUpperCase(), this._settings.difficulty);
	}
	else {
		this.difficultyComponent = new GameUIDummy(this, TG.controller.getDifficultyText().toUpperCase());
	}
	if (shouldBeAdded("difficulty", this._settings.difficultyImage)) {
		var difficultyImage = s_ui_smiley_medium;
		var difficultyImages;
		if (this._settings.difficultyImage.images) {
			difficultyImages = this._settings.difficultyImage.images;
		}
		else {
			difficultyImages = [s_ui_smiley_easy, s_ui_smiley_medium, s_ui_smiley_hard];
		}

		difficultyImage = difficultyImages[TG.controller.getDifficulty()];

		if (!this._settings.difficultyImage.image) {
			this._settings.difficultyImage.image = difficultyImage;
		}

		this.difficultyImageComponent = new GameUIImage(this, this._settings.difficultyImage);
		this.difficultyImageComponent.setImage(difficultyImage);
	}

	// translate old object definitions to arrays
	if (this._settings.itemImage && !this._settings.itemImage.length) {
		this._settings.itemImage = [this._settings.itemImage];
	}
	if (this._settings.itemText && !this._settings.itemText.length) {
		this._settings.itemText = [this._settings.itemText];
	}

	this.itemImageComponent = [];
	this.itemTextComponent = [];

	if (shouldBeAdded(["item", "item0"], this._settings.itemImage)) {
		this.itemImageComponent[0] = new GameUIImage(this, this._settings.itemImage[0]);
	} else {
		this.itemImageComponent[0] = new GameUIDummy(this, "");
	}

	if (shouldBeAdded(["item", "item0"], this._settings.itemText)) {
		this.itemTextComponent[0] = new GameUIText(this, "", this._settings.itemText[0]);
	}
	else {
		this.itemTextComponent[0] = new GameUIDummy(this, "");
	}

	if (this._settings.itemImage && this._settings.itemText) {

		for (var index = 1; index < this._settings.itemText.length; ++index) {
			if (shouldBeAdded("item" + index, this._settings.itemText[index])) {
				this.itemTextComponent[index] = new GameUIText(this, "0 / 0", this._settings.itemText[index]);
				this.itemImageComponent[index] = new GameUIImage(this, this._settings.itemImage[index]);
			}
		}

	}

	for (var key in this._settings) {
		var item = this._settings[key];

		if (item.textKey) {
			new GameUIText(this, TG.dictionary.getText(item.textKey, "<"+ item.textKey+">") + (item.separator?item.separator:"") , item);
		}
	}

	this.timeWarningSeconds = 0;


	this.lifeTime = 0;

	this.buttons = {};
	var button;

	for (key in this._settings.buttons) {
		button = this._createButton(key, this._settings.buttons[key]);
		this.buttons[key] = button;
	}

	if (this._settings.pauseButton) {
		button = this._createButton("pauseButton", this._settings.pauseButton);
		this.buttons["pauseButton"] = button;
	}

	this.customComponents = {};

	for (key in this._settings.customComponents) {
		var customComponentSettings = this._settings.customComponents[key];
		var componentType = GameUIController._customComponentRegistry[customComponentSettings.componentType];
		var customComponent = new componentType(this, customComponentSettings);
		this.customComponents[key] = customComponent;
	}

	this.setOffset(0, 0);
}

Inheritance.copyPrototype(GameUIControllerBase, GameUIController);

GameUIController._customComponentRegistry = {};

GameUIController.registerComponent = function(name, customComponentType) {
	"use strict";
	GameUIController._customComponentRegistry[name] = customComponentType;
};

GameUIController.prototype.getCustomComponent = function(componentName) {
	"use strict";
	return this.customComponents[componentName];
};

GameUIController.prototype.setLayer = function(layerId) {
	"use strict";
	GameUIControllerBase.prototype.setLayer.apply(this, arguments);
	for (var key in this.buttons) {
		TG.functions.setLayerAndCanvas(this.buttons[key], layerId);
	}
};

GameUIController.prototype._createButton = function(buttonName, settings) {
	"use strict";

	var result = new GameUIButton(this, settings, settings.buttonType, buttonName);
	this._components.push(result);
	result.gameui_buttonSettings = settings;

	return result;
};

GameUIController.prototype.setButtonType = function(buttonType, buttonKey) {
	"use strict";
	buttonKey = buttonKey || "pauseButton";
	var button = this.buttons[buttonKey];
	button.setButtonType(buttonType);
};

GameUIController.prototype.setButtonCallback = function(callback, buttonKey) {
	"use strict";
	buttonKey = buttonKey || "pauseButton";

	var oldButton = this.buttons[buttonKey];

	oldButton.setCallback(callback);
};

GameUIController.prototype.setItemText = function(value, index) {
	"use strict";
	index = index || 0;
	this.itemTextComponent[index].setText(value);
};

GameUIController.prototype.setItemImage = function(image, index) {
	"use strict";
	index = index || 0;
	this.itemImageComponent[index].setImage(image);
};

GameUIController.prototype.setItemImageNumber = function(number, index) {
	"use strict";
	index = index || 0;
	this.itemImageComponent[index].setNumber(number);
};

GameUIController.prototype._createItemImageOverlay = function(index) {
	"use strict";
	index = index || 0;

	var gameUiImage;

	if (!this.itemImageComponent[index].overlay) {

		var fromSettings = this.itemImageComponent[index].settings;
		var settings = {};

		for (var key in fromSettings) {
			if (fromSettings.hasOwnProperty(key)) {
				settings[key] = fromSettings[key];
			}
		}

		gameUiImage = new GameUIImage(this, settings);
		gameUiImage.center = false;
		this.itemImageComponent[index].overlay = gameUiImage;
	}
	else {
		gameUiImage = this.itemImageComponent[index].overlay;
	}

	return gameUiImage;
};

GameUIController.prototype.setItemImageOverlay = function(overlayImage, offsetx, offsety, index) {
	"use strict";
	index = index || 0;
	var gameUiImage = this._createItemImageOverlay(index);

	gameUiImage.x = gameUiImage.settings.x + (offsetx || 0);
	gameUiImage.y = gameUiImage.settings.y + (offsety || 0);

	gameUiImage.setImage(overlayImage);

};

GameUIController.prototype.getItemImageOverlay = function(index) {
	"use strict";
	index = index || 0;
	var gameUiImage = this.itemImageComponent[index].overlay;
	if (!gameUiImage) {
		return null;
	}
	return gameUiImage.getImage();
};

GameUIController.prototype.setItemImageOverlayEntrance = function(entranceSettings, index) {
	"use strict";
	var gameUiImage = this._createItemImageOverlay(index);

	gameUiImage.settings.entrance = entranceSettings;
	gameUiImage.startEntranceAnimation();

};
GameUIController.prototype.getItemImagePosition = function(index) {
	"use strict";
	index = index || 0;
	return this.itemImageComponent[index].getPosition();
};

GameUIController.prototype.setTimeWarningSeconds = function(seconds) {
	"use strict";
	this.timeWarningSeconds = seconds;
};

GameUIController.timeToStr = function(value) {
	"use strict";
	var minutes = Math.floor(value / 60000);
	var seconds = Math.floor((value % 60000) / 1000);
	return minutes + (seconds < 10 ? ":0" : ":") + seconds;
};

GameUIController.prototype.setProgress = function(percentage) {
	"use strict";
	this.progressComponent.setProgress(percentage);
	return this;
};

GameUIController.prototype.getProgress = function() {
	"use strict";
	return this.progressComponent.getProgress();
};

GameUIController.prototype.setBonusActive = function(active) {
	"use strict";
	this.progressComponent.setBonusActive(active);
};

GameUIController.prototype.setTime = function(time) {
	"use strict";
	this.timeComponent.setValue(time);
	return this;
};

GameUIController.prototype.setTimeMmSs = function(minutes, seconds) {
	"use strict";
	this.timeComponent.setValue((minutes * 60 + seconds) * 1000);
	return this;
};

GameUIController.prototype.getTime = function() {
	"use strict";
	return this.timeComponent.getValue();
};

GameUIController.prototype.setLives = function(lives) {
	"use strict";
	this.livesComponent.setValue(lives);
	return this;
};

GameUIController.prototype.getLives = function() {
	"use strict";
	return this.livesComponent.getValue();
};

GameUIController.prototype.setScore = function(score) {
	"use strict";

	this.scoreComponent.setValue(score);

	if (this.runtimeOptions.autoUpdateHighScore) {
		if (this.highScoreComponent.getValue() < score) {
			this.highScoreComponent.setValue(score);
		}
		else {
			if (score < this.highScoreComponent.getValue()) {
				this.highScoreComponent.setValue(Math.max(score, this.initialHighScore));
			}
		}
	}

	return this;
};

/** @deprecated Use GameUIController.setHighScore */
GameUIController.prototype.setHighscore = function(highScore) {
	"use strict";
	return this.setHighScore(highScore);
};

GameUIController.prototype.setHighScore = function(highScore) {
	"use strict";
	this.highScoreComponent.setValue(highScore);
	this.initialHighScore = highScore;
	return this;
};

GameUIController.prototype.setHighScoreFormatter = function(formatter) {
	"use strict";
	this.highScoreComponent.setFormatter(formatter);
};

GameUIController.prototype.setHighScoreDictionaryKey = function(dictionaryKey) {
	"use strict";
	this.highScoreComponent.setDictionaryKey(dictionaryKey);
};

GameUIController.prototype.getScore = function() {
	"use strict";
	return this.scoreComponent.getValue();
};

/** @deprecated Use GameUIController.getHighScore */
GameUIController.prototype.getHighscore = function() {
	"use strict";
	return this.getHighScore();
};


GameUIController.prototype.getHighScore = function() {
	"use strict";
	return this.highScoreComponent.getValue();
};

GameUIController.prototype.setStage = function(stage) {
	"use strict";
	this.stageComponent.setValue(stage);
	if (stage > 1) {
		if (this.progressComponent) {
			if (this.progressComponent.handleNextStage) {
				this.progressComponent.handleNextStage(stage);
			}
		}
	}
	return this;
};

GameUIController.prototype.getStage = function() {
	"use strict";
	return this.stageComponent.getValue();
};

GameUIController.prototype.setLevel = function(level) {
	"use strict";
	this.stageComponent.setValue(level);
	return this;
};

GameUIController.prototype.getLevel = function() {
	"use strict";
	return this.stageComponent.getValue();
};

GameUIController.prototype.incStage = function(stagesToAdd) {
	"use strict";
	if (typeof stagesToAdd === "undefined") {
		stagesToAdd = 1;
	}
	this.setStage(this.getStage() + stagesToAdd);
	return this;
};

GameUIController.prototype.getTimeAsString = function() {
	"use strict";
	return GameUIController.timeToStr(this.timeComponent.getValue());
};

GameUIController.prototype.addScore = function(points) {
	"use strict";
	this.setScore(this.getScore() + points);
	return this;
};

GameUIController.prototype.addStar = function(percentage) {
	"use strict";
	return this.progressComponent.addStar(percentage);
};

GameUIController.prototype.setStarComplete = function(starIndex, isComplete) {
	"use strict";
	this.progressComponent.setStarComplete(starIndex, isComplete);
	return this;
};

GameUIController.prototype.getStarComplete = function(starIndex) {
	"use strict";
	return this.progressComponent.getStarComplete(starIndex);
};

GameUIController.prototype.calculateBounds = function(result) {
	"use strict";

	if (!result) {
		result = new GameUIBounds(0,0,0,0);
	}

	result.left = 0;
	result.top = 0;
	result.right = this._background_image.width;
	result.bottom = this._background_image.height;

	return result;
};

// handleRemove is called when the token is removed from the game.
GameUIController.prototype.handleRemove = function() {
	"use strict";

	GameUIControllerBase.prototype.handleRemove.apply(this, arguments);

	tbGraphics.setCanvas(this.canvas);
	tbGraphics.clear();

	for (var key in this.buttons) {
		tbGame.removeToken(this.buttons[key]);
	}
};

// handleBeginStep is called each step before processing other events.
GameUIController.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	if (this.timeMode === GameUITimeMode.AUTO_INCREMENT) {
		this.setTime(this.getTime() + timePassed);
	}
	if (this.timeMode === GameUITimeMode.AUTO_DECREMENT) {


		if (this.timeWarningSeconds && this.timeWarningSeconds*1000 >= this.getTime()) {
			var lastSeconds = Math.floor(this.getTime() / 1000);
			var currSeconds = Math.floor(Math.max(this.getTime() - timePassed, 0) / 1000);

			if (lastSeconds !== currSeconds) {
				this.timeComponent.startHighlight();

				if (typeof a_gameui_timewarning_second !== "undefined") {
					tbAudio.play(a_gameui_timewarning_second);
				}
			}
		}
		this.setTime(Math.max(this.getTime() - timePassed, 0));
	}

	GameUIControllerBase.prototype.handleBeginStep.apply(this, arguments);

	this.lifeTime += timePassed;

};

/**
 * @deprecated This function cannot be used anymore
 */
GameUIController.prototype.setBackgroundAlpha = function() {
};

// handleDraw is called at the end of a step to draw the token.
GameUIController.prototype.drawWithOffset = function(offset_x, offset_y) {
	"use strict";

	if (this._background_image) {
		if (this._background_image instanceof tbImage) {
			this._background_image.drawAlpha(0, offset_x + this._background_image_x, offset_y + this._background_image_y, 1);
		}
		else {
			this._background_image.drawAlpha(offset_x + this._background_image_x, offset_y + this._background_image_y, 1);
		}
	}
	GameUIControllerBase.prototype.drawWithOffset.apply(this, arguments);

	if (this.drawDebugInfo && this._background_image) {
		tbGraphics.drawRectangle(offset_x, offset_y, this._background_image.width, this._background_image.height, "blue", true);
	}
};

//******************************************************
//*****    Script Resource: GameUIStar
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

/* exported GameUIStar
 */
var GameUIStar = function(position) {
	"use strict";
	this.complete = false;
	this.position = position;

};
//******************************************************
//*****    Script Resource: GameUIComponent
//******************************************************

/* global
	tbSurface
 */

/**
 * @constructor GameUIComponent
 */
function GameUIComponent(controller, settings, x, y, width, height) {
	"use strict";
	/**
	 * @type GameUIController
	 */
	this.controller = controller;

	this.width = width;
	this.height = height;

	this.surface = null;

	this.x = x;
	this.y = y;

	this.visible = true;

	this.settings = settings;
	this.alpha = settings.alpha !== undefined ? settings.alpha : 1;

	this.scale = settings.scale !== undefined ? settings.scale: 1;

	this.drawing = {};
	this.drawing.offset_x = 0;
	this.drawing.offset_y = 0;
	this.drawing.scale = this.scale;
	this.drawing.alpha = this.alpha;
	this.drawing.angle = 0;

	this.animation = {};
	this.animation.running = false;

	this.animation.origin = {};
	this.animation.target = {};

	this.animation.timer = 0;

	if (this.settings.entrance) {
		this.startAnimationFrom(this.settings.entrance);
		this.animation.running = false;
	}

	this.controller._components.push(this);


	if (!GameUIComponent.attentionEffectFloaterKind) {
		GameUIComponent.attentionEffectFloaterKind = {
			init: function(floater) {
				if (floater.value instanceof tbSurface) {
					floater.surface = floater.value;
				}
				else {
					floater.image = floater.value;
					floater.subImage = 0;
				}
			},
			update: tbFloaters.defaultUpdate,
			draw: tbFloaters.defaultDraw,
			end: tbFloaters.defaultEnd,
			easeX: tbEase.easeLinear,
			easeY: tbEase.easeLinear,
			easeAlpha: function(time, begin, change, duration) {
				return 1 - tbEase.easeOutCubic(time, begin, change, duration);
			},
			easeScaleX: function(a, b, c, d) {
				return tbEase.easeOutCubic(a, b, c, d) * 1 + 1;
			},
			easeScaleY: function(a, b, c, d) {
				return tbEase.easeOutCubic(a, b, c, d) * 1 + 1;
			}
		};
	}

}


GameUIComponent.prototype._getAnimationDuration = function(settings) {
	"use strict";
	if (settings) {
		var duration = settings.duration;
		if (settings.delay) {
			duration += settings.delay;
		}
		return duration;
	}
	else {
		return 0;
	}
};


GameUIComponent.prototype.getEntranceDuration = function() {
	"use strict";
	return this._getAnimationDuration(this.settings.entrance);
};

GameUIComponent.prototype.getExitDuration = function() {
	"use strict";
	return this._getAnimationDuration(this.settings.exit);
};


GameUIComponent.prototype.startExitAnimation = function(canUseEntranceAnimation) {
	"use strict";
	if (this.settings.exit) {
		this.startAnimationTo(this.settings.exit);
	}
	else if (canUseEntranceAnimation && this.settings.entrance) {
		this.startAnimationTo(this.settings.entrance);
	}
};

GameUIComponent.prototype.startEntranceAnimation = function() {
	"use strict";
	if (this.settings.entrance) {
		this.startAnimationFrom(this.settings.entrance);
	}
};

GameUIComponent.prototype._setGenericAnimationParameters = function(settings) {
	"use strict";
	this.animation.duration = settings.duration;
	this.animation.running = true;
	this.animation.easeFunction = settings.easeFunction || tbEase.easeOutCubic;
	this.animation.timer = 0;
	this.animation.delay = settings.delay || 0;

	this.updateAnimation();
};

GameUIComponent.prototype.startAnimationFrom = function(settings) {
	"use strict";

	this.animation.origin.x = settings.x === undefined ? this.x : settings.x;
	this.animation.origin.y = settings.y === undefined ? this.y : settings.y;
	this.animation.origin.alpha = settings.alpha !== undefined ? settings.alpha : 1;
	this.animation.origin.scale = settings.scale !== undefined ? settings.scale : 1;

	this.animation.target.x = this.x;
	this.animation.target.y = this.y;
	this.animation.target.alpha = this.alpha;
	this.animation.target.scale = this.scale;

	this._setGenericAnimationParameters(settings);
};

GameUIComponent.prototype.startAnimationTo = function(settings) {
	"use strict";

	this.animation.target.x = settings.x === undefined ? this.x : settings.x;
	this.animation.target.y = settings.y === undefined ? this.y : settings.y;
	this.animation.target.alpha = settings.alpha !== undefined ? settings.alpha : 1;
	this.animation.target.scale = settings.scale !== undefined ? settings.scale : 1;

	this.animation.origin.x = this.x;
	this.animation.origin.y = this.y;
	this.animation.origin.alpha = this.alpha;
	this.animation.origin.scale = this.scale;

	this._setGenericAnimationParameters(settings);
};

GameUIComponent.prototype.updateAnimation = function() {
	"use strict";

	if (this.animation.timer >= this.animation.duration) {
		this.animation.timer = this.animation.duration;
		this.animation.running = false;
	}

	var virtual_x = this.animation.easeFunction(this.animation.timer, this.animation.origin.x, this.animation.target.x - this.animation.origin.x, this.animation.duration);
	var virtual_y = this.animation.easeFunction(this.animation.timer, this.animation.origin.y, this.animation.target.y - this.animation.origin.y, this.animation.duration);

	this.drawing.offset_x = virtual_x - this.x;
	this.drawing.offset_y = virtual_y - this.y;

	this.drawing.alpha = this.animation.easeFunction(this.animation.timer, this.animation.origin.alpha, this.animation.target.alpha - this.animation.origin.alpha, this.animation.duration);
	this.drawing.scale = this.animation.easeFunction(this.animation.timer, this.animation.origin.scale, this.animation.target.scale - this.animation.origin.scale, this.animation.duration);

	this.controller.redraw = true;

};

GameUIComponent.prototype.createSurface = function() {
	"use strict";
	this.surface = new tbSurface(this.width, this.height);
};

GameUIComponent.prototype.handleDraw = function(offset_x, offset_y) {
	"use strict";
	if (this.surface) {
		this.surface.drawExt(this.x + this.drawing.offset_x + offset_x, this.y + this.drawing.offset_y + offset_y, this.drawing.scale, this.drawing.scale, 0, this.drawing.alpha);
	}
};


GameUIComponent.prototype.handleDrawGameBackground = function(offset_x, offset_y) {
	"use strict";
	TG.controller.drawInGameBackground(this.x + this.drawing.offset_x + offset_x, this.y + this.drawing.offset_y + offset_y, this.width * this.drawing.scale,
			this.height * this.drawing.scale);
};

GameUIComponent.prototype.isPointInside = function(x, y) {
	"use strict";
	return x > this.x + this.drawing.offset_x && x < this.x + this.drawing.offset_x + this.width * this.drawing.scale && y > this.y + this.drawing.offset_y
			&& y < this.y + this.drawing.offset_y + this.height * this.drawing.scale;
};


GameUIComponent.prototype.showAttentionEffect = function() {
	"use strict";

	if (this.surface) {
		if (!tbFloaters.visible) {
			tbFloaters.initFloaters();
		}

		var offset_x = this.controller.offset_x + TG.gameUIOffsetX + this.surface.width / 2;
		var offset_y = this.controller.offset_y + TG.gameUIOffsetY + this.surface.height / 2;

		tbFloaters.addFloater(GameUIComponent.attentionEffectFloaterKind, this.surface, this.x + offset_x, this.y + offset_y, this.x + offset_x, this.y + offset_y, 900, 0, false);
	}
};


GameUIComponent.prototype.setVisible = function(visible) {
	"use strict";
	if (this.visible === visible) {
		return;
	}
	this.visible = visible;
	this.controller.redraw = true;
};

GameUIComponent.prototype.handleBeginStep = function(timePassed) {
	"use strict";

	if (this.animation.running) {

		if (this.animation.delay > 0) {
			this.animation.delay -= timePassed;
		}
		else {
			this.animation.timer += -this.animation.delay;
			this.animation.delay = 0;
			this.animation.timer += timePassed;
			this.updateAnimation();
		}

	}
};


//******************************************************
//*****    Script Resource: GameUIDummy
//******************************************************

var GameUIDummy = function(controller, initialValue) {
	"use strict";
	this.initialValue = initialValue;
	this.value = initialValue;
	this.progress = initialValue;
};

GameUIDummy.prototype.setValue = function(value) {
	"use strict";
	this.value = value;
};

GameUIDummy.prototype.getValue = function() {
	"use strict";
	return this.value;
};

GameUIDummy.prototype.setProgress = function(percentage) {
	"use strict";

	if (percentage < 0) {
		percentage = 0;
	}
	if (percentage > 100) {
		percentage = 100;
	}

	this.progress = percentage;
};

GameUIDummy.prototype.getProgress = function() {
	"use strict";
	return this.progress;
};

GameUIDummy.prototype.addStar = function() {
	"use strict";
};

GameUIDummy.prototype.setBonusActive = function() {
	"use strict";
};

GameUIDummy.prototype.setText = function() {
	"use strict";
};

GameUIDummy.prototype.setImage = function() {
	"use strict";
};

GameUIDummy.prototype.setFormatter = function() {
	"use strict";
};

GameUIDummy.prototype.setDictionaryKey = function() {
	"use strict";
};

//******************************************************
//*****    Script Resource: GameUIImage
//******************************************************

/* global
	Inheritance,
	GameUIComponent,
	tbImage
 */

/**
 * @constructor GameUIImage
 * @base GameUIComponent
 */
function GameUIImage(controller, settings) {
	"use strict";

	this.originalSettings = settings;
	this.settings = {};

	for (var key in settings) {
		this.settings[key] = settings[key];
	}

	this.image = this.settings.image;
	this.number = 0;

	this.center = false || this.settings.center;

	if (this.settings.subtractOrigin) {

		this.settings.x += this.image.xOrigin;
		this.settings.y += this.image.yOrigin;
	}

	GameUIComponent.call(this, controller, this.settings, this.settings.x, this.settings.y, this.image?this.image.width:1, this.image?this.image.height:1);

}

Inheritance.copyPrototype(GameUIComponent, GameUIImage);
GameUIController.registerComponent("GameUIImage", GameUIImage);

GameUIImage.prototype.setNumber = function(number) {
	"use strict";
	if (this.number !== number) {
		this.number = number;
		this.controller.redraw = true;
	}
};

GameUIImage.prototype.showAttentionEffect = function() {
	"use strict";
	if (!tbFloaters.visible) {
		tbFloaters.initFloaters();
	}

	var offset_x = this.controller.offset_x + TG.gameUIOffsetX;
	var offset_y = this.controller.offset_y + TG.gameUIOffsetY;

	if (this.center) {
		offset_x += -Math.floor(this.image.width / 2);
		offset_y += -Math.floor(this.image.height / 2);
	}

	tbFloaters.addFloater(GameUIComponent.attentionEffectFloaterKind, this.image, this.x + offset_x, this.y + offset_y, this.x + offset_x, this.y + offset_y, 900, 0, false);
};

GameUIImage.prototype.handleDraw = function(offset_x, offset_y) {
	"use strict";

	if (this.image) {

		if (this.center) {
			offset_x += -Math.floor(this.image.width / 2);
			offset_y += -Math.floor(this.image.height / 2);
		}

		if (this.image instanceof tbImage) {
			this.image.drawExt(this.number, this.x + offset_x + this.drawing.offset_x, this.y + offset_y + this.drawing.offset_y, this.drawing.scale, this.drawing.scale, 0, this.drawing.alpha);
		}
		else {
			this.image.drawExt(this.x + offset_x + this.drawing.offset_x, this.y + offset_y + this.drawing.offset_y, this.drawing.scale, this.drawing.scale, 0, this.drawing.alpha);
		}

		if (this.controller.drawDebugInfo) {
			tbGraphics.drawRectangle(this.x + offset_x - this.image.xOrigin + 1, this.y + offset_y - this.image.yOrigin + 1, this.image.width - 2, this.image.height - 2,
					"black", true);
		}
	}
};


GameUIImage.prototype.isPointInside = function(x, y) {
	"use strict";
	if (!this.image) {
		return false;
	}

	var offset_x = 0;
	var offset_y = 0;
	if (this.center) {
		offset_x += -Math.floor(this.image.width / 2);
		offset_y += -Math.floor(this.image.height / 2);
	}
	offset_x -= this.image.xOrigin;
	offset_y -= this.image.yOrigin;

	return x > offset_x + this.x + this.drawing.offset_x && x < offset_x + this.x + this.drawing.offset_x + this.width * this.drawing.scale && y > offset_y + this.y + this.drawing.offset_y
			&& y < offset_y + this.y + this.drawing.offset_y + this.height * this.drawing.scale;
};


GameUIImage.prototype.handleDrawGameBackground = function(offset_x, offset_y) {
	"use strict";
	if (this.image) {
		if (this.center) {
			offset_x += -Math.floor(this.image.width / 2);
			offset_y += -Math.floor(this.image.height / 2);
		}
		offset_x -= this.image.xOrigin;
		offset_y -= this.image.yOrigin;

		TG.controller.drawInGameBackground(this.x + this.drawing.offset_x + offset_x, this.y + this.drawing.offset_y + offset_y, this.width * this.drawing.scale,
				this.height * this.drawing.scale);
	}
};


GameUIImage.prototype.getPosition = function(result) {
	"use strict";

	if (!result) {
		result = new tbVector(0, 0);
	}

	result.x = this.x + TG.gameUIOffsetX + this.controller.offset_x;
	result.y = this.y + TG.gameUIOffsetY + this.controller.offset_y;

	return result;
};

GameUIImage.prototype.setImage = function(image) {
	"use strict";

	if (image !== this.image) {
		this.image = image;
		this.controller.redraw = true;
		if (this.image) {
			this.width = this.image.width;
			this.height = this.image.height;
		}
	}

};

GameUIImage.prototype.getImage = function() {
	"use strict";
	return this.image;
};

//******************************************************
//*****    Script Resource: GameUIText
//******************************************************

/* global
	Inheritance,
	GameUIComponent,
	tbGraphics
 */

/**
 * @constructor GameUIText
 * @extends GameUIComponent
 * @param {String} text
 * @param {tbVectorFont} font
 */
var GameUIText = function(controller, text, settings) {
	"use strict";

	if (typeof text === "object") {
		settings = text;
		if (settings.textKey) {
			text = TG.dictionary.getText(settings.textKey, "<" + settings.textKey + ">");
		}
		else {
			text = settings.text || "";
		}
	}

	this.text = text;

	/**
	 * @type tbVectorFont
	 */
	this.font = settings.font.clone();
	if (settings.font_properties) {
		this.font.setProperties(settings.font_properties);
	}

	this.originalX = settings.x;
	this.originalY = settings.y;

	this.originalW = settings.w;
	this.originalH = settings.h;

	this.originalColor = this.font.getFillColor();

	this.textYOffsetFactor = settings.textYOffsetFactor === undefined ? 0.2 : settings.textYOffsetFactor;

	GameUIComponent.call(this, controller, settings, Math.floor(settings.x - settings.w * 0.1), Math.floor(settings.y - settings.h * 0.1), Math.floor(settings.w * 1.2), Math.floor(settings.h * 1.2));

	this.createSurface();

	switch (this.font.getAlign()) {
	case "left":
		this.text_x = Math.floor(settings.w * 0.1);
		break;
	case "right":
		this.text_x = Math.floor(settings.w * 1.1);
		break;
	case "center":
		this.text_x = Math.floor(settings.w * 0.6);
		break;
	default:
		throw "Unknown alignment: "+this.font.getAlign();
	}

	// this.textYOffsetFactor = tgSelect({big: 0.2;)

	var textYOffset = Math.floor(this.textYOffsetFactor * this.font.getFontSize());

	switch (this.font.getBaseline()) {
	case "top":
		this.text_y = Math.floor(settings.h * 0.1);
		break;
	case "bottom":
		this.text_y = Math.floor(settings.h * 1.1) + textYOffset;
		break;
	case "middle":
		this.text_y = Math.floor(settings.h * 0.6) + textYOffset;
		break;
	default:
		throw "Unknown baseline: "+this.font.getBaseline();
	}

	this.highlight = {};
	this.highlight.color = "red";
	this.highlight.duration = 200;
	this.highlight.timer = 0;
	this.highlight.started = false;

	this.redrawSurface();
};

Inheritance.copyPrototype(GameUIComponent, GameUIText);
GameUIController.registerComponent("GameUIText", GameUIText);

GameUIText.prototype.startHighlight = function(color, duration) {
	"use strict";

	this.highlight.timer = 0;
	this.highlight.started = true;

	if (color) {
		this.highlight.color = color;
	}

	if (duration) {
		this.highlight.duration = duration;
	}
	this.font.setFillColor(this.highlight.color);
	this.redrawSurface();
};


GameUIText.prototype.handleBeginStep = function(timePassed) {
	"use strict";

	GameUIComponent.prototype.handleBeginStep.apply(this, arguments);

	if (this.highlight.started) {
		this.highlight.timer += timePassed;

		if (this.highlight.duration <= this.highlight.timer)	{
			this.highlight.started = false;
			this.font.setFillColor(this.originalColor);
			this.redrawSurface();
		}
	}
};

GameUIText.prototype.redrawSurface = function() {
	"use strict";
	this.surface.clear();
	this.surface.setTarget();

	var textWidth = this.font.textWidth(this.text);

	var scale = 1;

	if (textWidth > this.originalW) {
		scale = this.originalW / textWidth;
	}

	this.font.drawExt(this.text, this.text_x, this.text_y, scale, scale, 0, 1);

	if (this.controller.drawDebugInfo) {
		tbGraphics.drawRectangle(0, 0, this.surface.width, this.surface.height, "black", true);
		tbGraphics.drawRectangle(this.originalX - this.x, this.originalY - this.y, this.surface.width - (this.originalX - this.x)*2, this.surface.height - (this.originalY - this.y)*2, "red", true);

		tbGraphics.drawLine(this.text_x - 5, this.text_y, this.text_x + 5, this.text_y, "green", 1);
		tbGraphics.drawLine(this.text_x, this.text_y - 5, this.text_x, this.text_y + 5, "green", 1);
	}
	this.controller.redraw = true;

	this.surface.resetTarget();
};

GameUIText.prototype.setText = function(newText) {
	"use strict";
	if (this.text !== newText) {
		this.text = newText;
		this.redrawSurface();
	}
};

//******************************************************
//*****    Script Resource: GameUIValue
//******************************************************

/* global
	Inheritance,
	GameUIText,
	tbEase,
	Easer,
	TG,
	tbGame
 */


var GameUIValueFormatters = {};

GameUIValueFormatters.defaultFormatter = function(value) {
	"use strict";
	return ""+value;
};

GameUIValueFormatters.decimalUsingLocale = function(value) {
	"use strict";
	return value.toLocaleString(TG.dictionary.getLanguage());
};

GameUIValueFormatters.timeFormatter = function(value) {
	"use strict";
	var minutes = Math.floor(value / 60000);
	var seconds = Math.floor((value % 60000) / 1000);
	return minutes + (seconds < 10 ? ":0" : ":") + seconds;
};

var easeFunctionConstantHigh = function(time, begin, change, duration) {
	"use strict";
	return begin + change;
};


var GameUIValue = function(controller, settings, valueName, initialValue, useEasing, formatter) {
	"use strict";
	this.initialValue = initialValue || 0;
	this.value = this.initialValue;
	this.valueLastDraw = -1;
	this.valueName = valueName;
	this.settings = settings;
	this.valueLastDrawnTime = -99999;
	this.timeBetweenValueUpdates = settings.timeBetweenValueUpdates || 0;

	if (settings.formatter) {
		this.formatter = settings.formatter;
	}
	else {
		this.formatter = formatter || GameUIValueFormatters.defaultFormatter;
	}

	var easeFunction = easeFunctionConstantHigh;

	if (useEasing && !(this.settings.easeDuration === 0)) {
		easeFunction = tbEase.easeInOutCubic;
	}

	this.easer = new Easer(this.initialValue, this.settings.easeDuration === undefined ? 500 : this.settings.easeDuration, easeFunction);
	if(settings.dictionaryKey){
		this.dictionaryKey = "game_ui_"+ settings.dictionaryKey;
	}
	this.text = this._getCaptionText() + this.formatter(this.initialValue);


	GameUIText.call(this, controller, this.text, settings);
};

Inheritance.copyPrototype(GameUIText, GameUIValue);
GameUIController.registerComponent("GameUIValue", GameUIValue);

GameUIValue.prototype.setValue = function(value) {
	"use strict";
	this.value = value;
	this.easer.easeTo(this.value);
};


GameUIValue.prototype.setFormatter = function(formatter) {
	"use strict";
	if (!formatter) {
		formatter = GameUIValueFormatters.defaultFormatter;
	}
	this.formatter = formatter;
	this.updateText(true);
};


GameUIValue.prototype.setDictionaryKey = function(dictionaryKey) {
	"use strict";
	this.dictionaryKey = dictionaryKey;
	this.updateText(true);
};


GameUIValue.prototype.getValue = function() {
	"use strict";
	return this.value;
};

GameUIValue.prototype.updateText = function(force) {
	"use strict";

	var valueToDraw = this.valueLastDraw;
	if (force || (tbGame.gameTime - this.valueLastDrawnTime > this.timeBetweenValueUpdates)) {
		valueToDraw = this.formatter(Math.floor(this.easer.getValue()));
	}

	if (this.valueLastDraw !== valueToDraw) {
		this.valueLastDrawnTime = tbGame.gameTime;
		this.valueLastDraw = valueToDraw;
		this.text = this._getCaptionText() + valueToDraw;
		this.redrawSurface();
	}
};

GameUIValue.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	GameUIText.prototype.handleBeginStep.apply(this, arguments);

	this.easer.addTime(timePassed);
	var newValue = Math.floor(this.easer.getValue());

	if (newValue !== this.valueLastDraw) {
		this.updateText();
	}
};

GameUIValue.prototype._getCaptionText = function() {
	"use strict";
	var t = "";
	if(this.settings.useCaption){
		t = this.dictionaryKey? TG.dictionary.getText(this.dictionaryKey, "<"+ this.dictionaryKey.toUpperCase()+">") : TG.dictionary.getText("game_ui_"+this.valueName, "<"+this.valueName+">");
	}
	return t + (this.settings.separator?this.settings.separator:"");
};

//******************************************************
//*****    Script Resource: GameUIProgress
//******************************************************

/* global
	Inheritance,
	GameUIComponent,
	tbEase,
	Easer,
	s_ui_level_star_fill,
	s_ui_level_star_empty,
	GameUIStar
 */
/**
 * @enum GameUIProgressDirection
 */
var GameUIProgressDirection = {
	LEFT_TO_RIGHT: 0,
	BOTTOM_TO_TOP: 1,
	TOP_TO_BOTTOM: 2
};

var GameUIProgress = function(controller, settings) {
	"use strict";
	this.progress = 0;
	this.bonusProgress = 0;

	this.settings = settings;
	this.previousValue = 0;
	this.previousBonusValue = 0;

	this.image = settings.image;
	this.bonusImage = settings.bonusImage || settings.image;
	this.imageOverlay = settings.imageOverlay || null;
	this.settings.overlayOffsetX = this.settings.overlayOffsetX || 0;
	this.settings.overlayOffsetY = this.settings.overlayOffsetY || 0;

	this.autoCompleteStars = true;
	this.progressDirection = settings.progressDirection || GameUIProgressDirection.LEFT_TO_RIGHT;
	this.stars = [];

	this.bonusActive = false;

	this.easer = new Easer(0, 200, tbEase.easeOutOvershoot);
	this.bonusEaser = new Easer(0, 200, tbEase.easeOutOvershoot);

	GameUIComponent.call(this, controller, settings, settings.x, settings.y, this.image.width, this.image.height);
};

Inheritance.copyPrototype(GameUIComponent, GameUIProgress);
GameUIController.registerComponent("GameUIProgress", GameUIProgress);

GameUIProgress.prototype.setProgress = function(percentage) {
	"use strict";

	if (percentage < 0) {
		percentage = 0;
	}
	if (percentage > 100) {
		percentage = 100;
	}

	if (this.bonusActive) {
		this.bonusProgress = percentage - this.progress;
		this.bonusEaser.easeTo(this.bonusProgress);
	}
	else {
		this.easer.easeTo(percentage);
		this.progress = percentage;
	}
};

GameUIProgress.prototype.getProgress = function() {
	"use strict";
	return this.progress;
};

GameUIProgress.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	this.easer.addTime(timePassed);
	var newValue = this.easer.getValue();

	if (newValue !== this.previousValue) {
		this.controller.redraw = true;
		this.previousValue = newValue;
	}

	this.bonusEaser.addTime(timePassed);
	var newBonusValue = this.bonusEaser.getValue();

	if (newBonusValue !== this.previousBonusValue) {
		this.controller.redraw = true;
		this.previousBonusValue = newBonusValue;
	}

	newValue += newBonusValue;

	if (this.autoCompleteStars) {
		for (var i=0;i<this.stars.length;++i) {

			var isComplete = (newValue >= this.stars[i].position) && (this.progress + this.bonusProgress >= this.stars[i].position);

			if (this.stars[i].complete !== isComplete) {
				if (this.settings.stars) {
					this.controller.redraw = true;
					this.previousValue = newValue;
				}
				this.stars[i].complete = isComplete;
			}
		}
	}
};

GameUIProgress.prototype.setBonusActive = function(active) {
	"use strict";

	if (!active && this.bonusActive) {
		var bonusProgress = this.bonusProgress;
		this.progress += this.bonusProgress;
		this.easer.easeTo(this.progress, bonusProgress * 50);
		this.bonusProgress = 0;
		this.bonusEaser.easeTo(this.bonusProgress, bonusProgress * 50);
	}
	this.bonusActive = active;
};

GameUIProgress.prototype.handleDraw = function(offset_x, offset_y) {
	"use strict";
	var i, star, image;
	
	if(this.progressDirection === GameUIProgressDirection.LEFT_TO_RIGHT){
		if (this.bonusEaser.getValue() > 0) {

			this.bonusImage.drawPart(0, this.width * this.easer.getValue() / 100, 0, this.bonusImage.width * this.bonusEaser.getValue() / 100,
					this.bonusImage.height, offset_x + this.x + this.width * this.easer.getValue() / 100, offset_y + this.y);
		}

		this.image.drawPart(0, 0, 0, this.width * this.easer.getValue() / 100, this.height, offset_x + this.x, offset_y + this.y);

		if (this.settings.stars) {
			for (i = 0; i < this.stars.length; ++i) {
				/**
				 * @type GameUIStar
				 */
				star = this.stars[i];
				if (star.complete) {
					image = s_ui_level_star_fill;
				}
				else {
					image = s_ui_level_star_empty;
				}
				image.drawSimple(0, offset_x + this.x + this.width / 100 * star.position, offset_y + this.y + this.settings.stars.y);
			}
		}
	}
	
	if(this.progressDirection === GameUIProgressDirection.BOTTOM_TO_TOP){
		if (this.bonusEaser.getValue() > 0) {
			this.bonusImage.drawPart(0, 0, this.height - this.height *this.easer.getValue()/100, this.width, this.height, offset_x + this.x, offset_y + this.y + (this.height - this.height * this.easer.getValue()/100));
		}

		this.image.drawPart(0, 0, this.height - this.height *this.easer.getValue()/100, this.width, this.height, offset_x + this.x, offset_y + this.y + (this.height - this.height * this.easer.getValue()/100));
		
		if (this.settings.stars) {
			for (i = 0; i < this.stars.length; ++i) {
				/**
				 * @type GameUIStar
				 */
				star = this.stars[i];
				if (star.complete) {
					image = s_ui_level_star_fill;
				}
				else {
					image = s_ui_level_star_empty;
				}
				image.drawSimple(0, offset_x + this.x + this.settings.stars.x, offset_y + this.y + this.height - this.height/ 100 * star.position);
			}
		}
	}
	
	if(this.progressDirection === GameUIProgressDirection.TOP_TO_BOTTOM){
		if (this.bonusEaser.getValue() > 0) {

			this.bonusImage.drawPart(0, 0, this.height * this.easer.getValue() / 100, this.bonusImage.width,
					this.bonusImage.height * this.bonusEaser.getValue() / 100, offset_x + this.x + this.width * this.easer.getValue() / 100, offset_y + this.y);
		}

		this.image.drawPart(0, 0, 0, this.width, this.height * this.easer.getValue()/100, offset_x + this.x, offset_y + this.y);

		if (this.settings.stars) {
			for (i = 0; i < this.stars.length; ++i) {
				star = this.stars[i];
				if (star.complete) {
					image = s_ui_level_star_fill;
				}
				else {
					image = s_ui_level_star_empty;
				}
				image.drawSimple(0, offset_x + this.x + this.settings.stars.x, offset_y + this.y + this.height/ 100 * star.position);
			}
		}
	}
	
	if(this.imageOverlay){
		this.imageOverlay.drawSimple(0, offset_x + this.x + this.settings.overlayOffsetX, offset_y + this.y + this.settings.overlayOffsetY);
	}
};

GameUIProgress.prototype.addStar = function(percentage) {
	"use strict";
	this.stars.push(new GameUIStar(percentage));
	return this.stars.length-1;
};

GameUIProgress.prototype.setStarComplete = function(starIndex, isComplete) {
	"use strict";
	this.stars[starIndex].complete = isComplete;
	return this;
};

GameUIProgress.prototype.getStarComplete = function(starIndex) {
	"use strict";
	return this.stars[starIndex].complete;
};

//******************************************************
//*****    Script Resource: GameUIButton
//******************************************************

/* global
	Inheritance,
	GameUIImage,
	s_btn_small_pause,
	s_btn_small_options,
	s_btn_small_retry,
	s_btn_small_quit
 */


/**
 * @enum GameUIButtonType
 */
var GameUIButtonType = {
	PAUSE: 0,
	RESTART: 1,
	RESIGN: 2,
	CUSTOM: 3
};

/**
 * @constructor GameUIButton
 * @base GameUIImage
 */
function GameUIButton(controller, settings, buttonType, buttonName) {
	"use strict";

	this.touch_touching = false;
	this.touch_finger = -1;

	this.buttonName = buttonName;

	this.controller = controller;
	this.settings = settings;

	this.setButtonType(buttonType);

	GameUIImage.call(this, controller, settings);

}

Inheritance.copyPrototype(GameUIImage, GameUIButton);
GameUIController.registerComponent("GameUIButton", GameUIButton);

GameUIButton.prototype.setButtonType = function(buttonType) {
	"use strict";
	var buttonImage = null;
	var callBack = null;
	var controller = this.controller;
	var settings = this.settings;

	if (buttonType === undefined) {
		if (!settings.buttonType) {
			buttonType = GameUIButtonType.PAUSE;
		}
		else {
			buttonType = settings.buttonType;
		}
	}

	var self = this;

	switch (buttonType) {
	case GameUIButtonType.PAUSE:

		if (controller.runtimeOptions.gameCanBePaused) {
			buttonImage = s_btn_small_pause;
		}
		else {
			buttonImage = s_btn_small_options;
		}
		callBack = function() {

			if (TG.controller.getChallengeInformation()) {
				TG.controller.showOptionsDuringChallenge(controller.runtimeOptions.gameCanBePaused);
			}
			else {
				TG.controller.showOptions();
			}

			return true;
		};
		break;
	case GameUIButtonType.RESTART:
		buttonImage = s_btn_small_retry;
		callBack = function() {
			TG.controller.showOptions();
			return true;
		};
		break;
	case GameUIButtonType.RESIGN:
		buttonImage = s_btn_small_quit;
		callBack = function() {
			TG.controller.showForfeitDialog(controller.runtimeOptions.gameCanBePaused);
			return true;
		};
		break;
	case GameUIButtonType.CUSTOM:
		buttonImage = settings.image;
		break;
	}

	this.callback = callBack;
	this.settings.image = buttonImage;

};


GameUIButton.prototype.setCallback = function(callback) {
	"use strict";
	this.callback = callback;
};

GameUIButton.prototype.handleTouchDown = function(finger, x, y) {
	"use strict";
	if (this.isPointInside(x - TG.gameUIOffsetX, y - TG.gameUIOffsetY)) {
		this.touch_touching = true;
		this.touch_finger = finger;
		this.setNumber(1);
		return true;
	}
	return false;
};

GameUIButton.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	GameUIImage.prototype.handleBeginStep.apply(this, arguments);

	if (this.touch_touching) {
		var x = tbGame.touchX(this.touch_finger);
		var y = tbGame.touchY(this.touch_finger);
		if (this.isPointInside(x - TG.gameUIOffsetX, y - TG.gameUIOffsetY)) {
			this.setNumber(1);
		}
		else {
			this.setNumber(0);
		}
	}
};

GameUIButton.prototype.handleTouchUp = function(finger, x, y) {
	"use strict";
	if (this.touch_touching && finger === this.touch_finger) {
		this.setNumber(0);
		if (this.isPointInside(x - TG.gameUIOffsetX, y - TG.gameUIOffsetY)) {
			if (this.callback) {
				this.callback();
			}
		}
		this.touch_touching = false;
		this.touch_finger = -1;
		return true;
	}
	return false;

};
//******************************************************
//*****    Script Resource: GameUIRoundProgress
//******************************************************

/* global
	Inheritance,
	GameUIComponent,
	tbEase,
	Easer,
	s_star_filled,
	s_star_empty,
	GameUIStar
 */

var GameUIRoundProgress = function(controller, settings) {
	"use strict";
	this.progress = 0;
	this.bonusProgress = 0;

	this.settings = settings;
	this.previousValue = 0;
	this.previousBonusValue = 0;

	this.autoCompleteStars = true;

	this.stars = [];

	this.color = settings.color || "#00AEEF";
	this.bonusColor = settings.bonusColor || "#FF0F64";
	this.blinkColor = settings.blinkColor || "#FFED93";

	this.blinkEnabled = (settings.blink === undefined) || settings.blink;

	this.overlayImage = settings.overlayImage;

	this.bonusActive = false;

	this.blinkAnimationRunning = false;
	this.blinkAnimationTimer = 0;
	this.blinkAnimationDuration = 1000;
	this.blinkIntensity = 0;
	this.blinkProgress = 0;

	this.easer = new Easer(0, 200, tbEase.easeOutOvershoot);
	this.bonusEaser = new Easer(0, 200, tbEase.easeOutOvershoot);

	GameUIComponent.call(this, controller, settings, settings.x, settings.y, 1, 1);
};

Inheritance.copyPrototype(GameUIComponent, GameUIRoundProgress);
GameUIController.registerComponent("GameUIRoundProgress", GameUIRoundProgress);

GameUIRoundProgress.prototype.startBlinkAnimation = function() {
	"use strict";

	if (!this.blinkEnabled) {
		return;
	}

	if (this.blinkAnimationRunning) {
		this.blinkAnimationTimer -= this.blinkAnimationDuration;
	}
	else {
		this.blinkAnimationRunning = true;
		this.blinkAnimationTimer = 0;
		this.blinkIntensity = 0;
		this.easer.easeTo(100);
	}
};

GameUIRoundProgress.prototype.addStar = function(percentage) {
	"use strict";
	this.stars.push(new GameUIStar(percentage));
	return this.stars.length - 1;
};


GameUIRoundProgress.prototype.setProgress = function(percentage) {
	"use strict";

	if (percentage < 0) {
		percentage = 0;
	}
	if (percentage > 100) {
		percentage = 100;
	}

	if (this.bonusActive) {
		this.bonusProgress = percentage - this.progress;
		this.bonusEaser.easeTo(this.bonusProgress);
	}
	else {

		if (!this.blinkAnimationRunning) {
			if (percentage === 100 && this.blinkEnabled) {
				this.startBlinkAnimation();
			}
			else {
				this.easer.easeTo(percentage);
			}
		}
		this.progress = percentage;
	}
};

GameUIRoundProgress.prototype.getProgress = function() {
	"use strict";
	return this.progress;
};

GameUIRoundProgress.prototype.handleNextStage = function() {
	"use strict";
	this.startBlinkAnimation();
};

GameUIRoundProgress.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	this.easer.addTime(timePassed);
	var newValue = this.easer.getValue();

	if (newValue !== this.previousValue) {
		this.controller.redraw = true;
		this.previousValue = newValue;
	}

	this.bonusEaser.addTime(timePassed);
	var newBonusValue = this.bonusEaser.getValue();

	if (newBonusValue !== this.previousBonusValue) {
		this.controller.redraw = true;
		this.previousBonusValue = newBonusValue;
	}

	if (this.blinkAnimationRunning) {
		this.blinkAnimationTimer += timePassed;

		if (this.blinkAnimationTimer >= this.blinkAnimationDuration) {

			if (this.progress === 100) {
				this.blinkAnimationRunning = false;
				this.startBlinkAnimation();
			}
			else {
				this.blinkAnimationRunning = false;
				this.blinkIntensity = 0;
				this.easer.from = 0;
				this.easer.to = 0;
				this.easer.easeTo(this.progress);
			}
		}
		else {
			this.blinkIntensity = (-Math.cos(this.blinkAnimationTimer / this.blinkAnimationDuration * 5 * Math.PI * 2) + 1) / 2;
		}
		this.controller.redraw = true;
	}

	newValue += newBonusValue;

	if (this.autoCompleteStars) {
		for (var i=0;i<this.stars.length;++i) {

			var isComplete = (newValue >= this.stars[i].position) && (this.progress + this.bonusProgress >= this.stars[i].position);

			if (this.stars[i].complete !== isComplete) {
				if (this.settings.stars) {
					this.controller.redraw = true;
					this.previousValue = newValue;
				}
				this.stars[i].complete = isComplete;
			}
		}
	}

};


GameUIRoundProgress.prototype.handleDrawGameBackground = function(offset_x, offset_y) {
	"use strict";
	if (this.overlayImage) {
		TG.controller.drawInGameBackground(this.x + this.drawing.offset_x + offset_x - this.overlayImage.xOrigin, this.y + this.drawing.offset_y + offset_y
				- this.overlayImage.yOrigin, this.overlayImage.width * this.drawing.scale, this.overlayImage.height * this.drawing.scale);
	}
};

GameUIRoundProgress.prototype.setBonusActive = function(active) {
	"use strict";
	if (!active && this.bonusActive) {
		this.progress += this.bonusProgress;
		this.easer.easeTo(this.progress);
		this.bonusProgress = 0;
		this.bonusEaser.easeTo(this.bonusProgress);
	}
	this.bonusActive = active;
};

GameUIRoundProgress.prototype.handleDraw = function(offset_x, offset_y) {
	"use strict";

	/**
	 * @type GameUIStar
	 */
	var star;
	var i;
	var angle;

	if (this.overlayImage) {

		var percentage = this.easer.getValue() / 100;

		percentage = Math.max(percentage, 0);
		percentage = Math.min(percentage, 1);

		var context = tbGraphics.context;
		var radius = this.overlayImage.width / 2 - tgScale(4);
		var oldFillStyle = context.fillStyle;

		if (this.bonusEaser.getValue() > 0) {

			var bonusPercentage = this.bonusEaser.getValue() / 100;

			context.beginPath();
			context.arc(this.x + offset_x, this.y + offset_y, radius, -Math.PI * 0.5 + 2 * percentage * Math.PI, 2 * (percentage + bonusPercentage) * Math.PI - Math.PI * 0.5, false);
			context.lineTo(this.x + offset_x, this.y + offset_y);
			context.fillStyle = this.bonusColor;
			context.fill();
		}

		context.beginPath();
		context.arc(this.x + offset_x, this.y + offset_y, radius, -Math.PI * 0.5, 2 * percentage * Math.PI - Math.PI * 0.5, false);
		context.lineTo(this.x + offset_x, this.y + offset_y);
		context.fillStyle = this.color;
		context.fill();

		if (this.blinkAnimationDuration) {
			var globalAlpha = context.globalAlpha;
			context.globalAlpha *= this.blinkIntensity;
			context.beginPath();
			context.arc(this.x + offset_x, this.y + offset_y, radius, -Math.PI * 0.5, 2 * percentage * Math.PI - Math.PI * 0.5, false);
			context.lineTo(this.x + offset_x, this.y + offset_y);
			context.fillStyle = this.blinkColor;
			context.fill();
			context.globalAlpha = globalAlpha;
		}

		if (this.settings.stars) {
			var oldStrokeStyle = context.strokeStyle;
			var oldLineWidth = context.lineWidth;

			context.strokeStyle = "white";
			context.lineWidth = tgScale(2);

			for (i = 0; i < this.stars.length; ++i) {

				star = this.stars[i];
				angle = star.position / 100 * Math.PI * 2;

				var line_x = Math.cos(Math.PI * -0.5 + angle) * radius;
				var line_y = Math.sin(Math.PI * -0.5 + angle) * radius;

				context.beginPath();
				context.moveTo(Math.round(offset_x + this.x), Math.round(offset_y + this.y));
				context.lineTo(Math.round(offset_x + this.x + line_x), Math.round(offset_y + this.y + line_y));
				context.stroke();
			}

			context.strokeStyle = oldStrokeStyle;
			context.lineWidth = oldLineWidth;
		}


		this.overlayImage.drawSimple(0, offset_x + this.x, offset_y + this.y);

		if (this.settings.stars) {

			for (i = 0; i < this.stars.length; ++i) {

				star = this.stars[i];
				/**
				 * @type tbImage
				 */
				var image;
				if (star.complete) {
					image = s_star_filled;
				}
				else {
					image = s_star_empty;
				}

				angle = star.position / 100 * Math.PI * 2;

				var star_x = Math.cos(Math.PI * -0.5 + angle) * this.settings.stars.radius * 0.5;
				var star_y = Math.sin(Math.PI * -0.5 + angle) * this.settings.stars.radius * 0.5;

				image.drawSimple(0, Math.round(offset_x + this.x + star_x), Math.round(offset_y + this.y + star_y));
			}
		}
		context.fillStyle = oldFillStyle;

	}

};

//******************************************************
//*****    Script Resource: game_ui_version
//******************************************************


TG.version = TG.version || {};
TG.version["game_ui"] = "2.1.0";




//******************************************************
//*****    Script Resource: game_ui_settings_difficulty
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

/* global
	tgScale,
	f_game_ui,
	s_ui_background_blank,
	f_game_ui_tiny,
	s_ui_heart,
	s_ui_smiley_medium,
	s_ui_timeleft,
	s_ui_highscore,
	s_ui_divider */

/* exported GameUISettings */
var GameUISettings = GameUISettings || {};

var GameUISettingsOffsets = {
	X: tgScale(14),
	Y: tgScale(40)
};

GameUISettings.Difficulty = {
	background: {
		image: s_ui_background_blank,
		image_x: tgScale(0),
		image_y: tgScale(34),
		elements: [{
			image: s_ui_timeleft,
			x: tgScale(46) + GameUISettingsOffsets.X,
			y: tgScale(16) + GameUISettingsOffsets.Y
		}, {
			textKey: "game_ui_time_left",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(52) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#9fa9bf",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(124) + GameUISettingsOffsets.Y
		}, {
			textKey: "game_ui_SCORE",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(140) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#9fa9bf",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(200) + GameUISettingsOffsets.Y
		}, {
			textKey: "game_ui_HIGHSCORE",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(258) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#9fa9bf",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(318) + GameUISettingsOffsets.Y
		}]
	},
	pauseButton: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(538) - tgScale(86) + GameUISettingsOffsets.Y
	},
	time: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(80) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(38),
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		/**
		 * @type tbVectorFont
		 */
		font: f_game_ui,
		font_properties: {
			fontSize: tgScale(38),
			fillColor: "#172348",
			align: "center",
			baseline: "top"
		}
	},
	score: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(166) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(24),
		timeBetweenValueUpdates: 50, // ms
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		/**
		 * @type tbVectorFont
		 */
		font: f_game_ui,
		font_properties: {
			fontSize: tgScale(24),
			fillColor: "#172348",
			align: "center",
			baseline: "top"
		}
	},
	highScoreImage: {
		x: tgScale(43, "round") + GameUISettingsOffsets.X,
		y: tgScale(212) + GameUISettingsOffsets.Y,
		image: s_ui_highscore,
		center: false,
		subtractOrigin: true
	},
	highScore: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(284) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(20),
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		font: f_game_ui,
		font_properties: {
			fillColor: "#59668e",
			fontSize: tgScale(20),
			align: "center",
			baseline: "top"
		}
	},
	difficulty: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(362) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(40),
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		font: f_game_ui,
		font_properties: {
			fillColor: "#9fa9bf",
			fontSize: tgScale(20),
			align: "center",
			baseline: "top",
			fontCase: "lower"
		}
	},
	difficultyImage: {
		x: tgScale(56) + GameUISettingsOffsets.X,
		y: tgScale(340) + GameUISettingsOffsets.Y,
		image: s_ui_smiley_medium,
		center: false,
		subtractOrigin: false
	}
};

/* exported gameUISettingsVar */
var gameUISettingsVar = GameUISettings.Difficulty;

//******************************************************
//*****    Script Resource: tbFloaters
//******************************************************

/////////////////////////////////////////////////////////////
// tbFloaters
//
// tbFloaters object contains the functionality for floaters
//
// Copyright 2014 Tingly Games
// Last Changed: 21-05-2014
/////////////////////////////////////////////////////////////

var tbFloaters = {


/////////////////////////////////////////////////////////////
// Floater kinds
/////////////////////////////////////////////////////////////

	textFloater1 : { },
	textFloater2 : { },
	textFloater3 : { },
	textFloater4 : { },
	numberFloater : { },
	numberSignFloater : { },
	surfaceFloater : { },
	imageFloater : { },

	// Initialize all the default floater kinds
	_initDefaultFloaters : function () {
		"use strict";
		tbFloaters.textFloater1 = {
			init : tbFloaters.defaultTextInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			font : floaterFontText1,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.textFloater2 = {
			init : tbFloaters.defaultTextInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			font : floaterFontText2,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.textFloater3 = {
			init : tbFloaters.defaultTextInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			font : floaterFontText3,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.textFloater4 = {
			init : tbFloaters.defaultTextInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			font : floaterFontText4,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.numberFloater = {
			init : tbFloaters.defaultNumberInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			fontPositive : floaterFontNumberPositive,
			fontNegative : floaterFontNumberNegative,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.numberSignFloater = {
			init : tbFloaters.defaultNumberSignInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			fontPositive : floaterFontNumberPositive,
			fontNegative : floaterFontNumberNegative,
			margin : 20,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.surfaceFloater = {
			init : tbFloaters.defaultSurfaceInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
		tbFloaters.imageFloater = {
			init : tbFloaters.defaultImageInit,
			update : tbFloaters.defaultUpdate,
			draw : tbFloaters.defaultDraw,
			end : tbFloaters.defaultEnd,
			easeX : tbEase.easeLinear,
			easeY : tbEase.easeLinear,
			easeAlpha : tbEase.easeCreateCombined([tbEase.easeOutCubic, tbEase.easeConstant, tbEase.easeOutCubic], [false, false, true], [0.1, 0.8, 0.1])
		};
	},

	// Create a clone of a kind of floater
	cloneFloaterKind : function (kind) {
		"use strict";
		var	clone = function(source){
			var attr, target = {};
			for (attr in source) {
				if(typeof source[attr] === "object" && source[attr] !== null){
						target[attr] = clone(source[attr]);
				}else{
					target[attr] = source[attr]; 
				}
			}
			return target;
		};
		return clone(kind);
	},

	// Set the font face for all text floaters
	setTextFontFace : function (fontFace) {
		"use strict";
		tbFloaters.textFloater1.font.setFontFace(fontFace);
		tbFloaters.textFloater2.font.setFontFace(fontFace);
		tbFloaters.textFloater3.font.setFontFace(fontFace);
		tbFloaters.textFloater4.font.setFontFace(fontFace);
	},

	// Set the font face for all number floaters
	setNumberFontFace : function (fontFace) {
		"use strict";
		tbFloaters.numberFloater.fontPositive.setFontFace(fontFace);
		tbFloaters.numberFloater.fontNegative.setFontFace(fontFace);
		tbFloaters.numberSignFloater.fontPositive.setFontFace(fontFace);
		tbFloaters.numberSignFloater.fontNegative.setFontFace(fontFace);
	},


/////////////////////////////////////////////////////////////
// Surface functions
/////////////////////////////////////////////////////////////

	_bufferSurfaces : false,				// Whether to buffer surfaces
	_buffers : [],						// The buffered surfaces

	// Indicate whether to default buffer surfaces
	setBuffering : function(buffer) {
		"use strict";
		tbFloaters._bufferSurfaces = buffer;
	},

	// Returns whether to default buffer surfaces
	getBuffering : function() {
		"use strict";
		return tbFloaters._bufferSurfaces;
	},

	// Remove all buffers of the given kind if no floater with it exists
	removeBuffers : function(kind) {
		"use strict";
		var ind, buffer;
		for (ind = 0; ind < tbFloaters._buffers.length; ind += 1) {
			buffer = tbFloaters._buffers[ind];
			if (buffer === undefined) { continue; }
			if (kind !== undefined && buffer.kind !== kind) { continue; }
			if (buffer.refCount > 0) { continue; }
			tbFloaters._buffers[ind] = undefined;
		}
	},

	// Initialize the buffers
	_initBuffers : function() {
		"use strict";
		tbFloaters._bufferSurfaces = false;
		tbFloaters._buffers = [];
	},

	// Create a buffer or return an existing one if it already exists
	_createBuffer : function(kind, font, text, keepIt) {
		"use strict";
		var buffer, w, h, ind, oldAlign, oldBaseline;
		if (keepIt === undefined) { keepIt = tbFloaters._bufferSurfaces; }
		// Check whether it exists
		if (keepIt) {
			for (ind = 0; ind < tbFloaters._buffers.length; ind += 1) {
				buffer = tbFloaters._buffers[ind];
				if (buffer !== undefined && buffer.keepIt && buffer.kind === kind &&
					buffer.font === font && buffer.text === text) {
					buffer.refCount += 1;
					return ind;
				}
			}
		}
		// Create the surface buffer
		buffer = { kind: kind, font: font, text: text, refCount: 1, keepIt: keepIt };
		oldAlign = font.getAlign();
		oldBaseline = font.getBaseline();
		font.setAlign("center");
		font.setBaseline("middle");
		w = font.textWidth(text) + 2 * kind.margin;
		h = font.textHeight(text) + 2 * kind.margin;
		buffer.surface = new tbSurface(w, h);
		buffer.surface.setTarget();
		font.drawSimple(text, w/2, h/2);
		buffer.surface.resetTarget();
		font.setAlign(oldAlign);
		font.setBaseline(oldBaseline);
		// Find an empty spot
		for (ind = 0; ind < tbFloaters._buffers.length; ind += 1) {
			if (tbFloaters._buffers[ind] === undefined) {
				tbFloaters._buffers[ind] = buffer;
				return ind;
			}
		}
		tbFloaters._buffers.push(buffer);
		return tbFloaters._buffers.length-1;
	},

	// End a buffer when it no longer needs drawing for a floater
	_endBuffer : function(ind) {
		"use strict";
		var buffer = tbFloaters._buffers[ind];
		buffer.refCount -= 1;
		if (buffer.refCount <= 0 && !buffer.keepIt) {
			tbFloaters._buffers[ind] = undefined;
		}
	},


/////////////////////////////////////////////////////////////
// Floater functions
/////////////////////////////////////////////////////////////

	// Default text floater init method
	defaultTextInit : function(floater) {
		"use strict";
		floater.buffer = tbFloaters._createBuffer(floater.kind, floater.kind.font, floater.value, floater.keepIt);
	},

	// Default number floater init method
	defaultNumberInit : function(floater) {
		"use strict";
		var text = floater.value.toString();
		if (floater.value >= 0) {
			floater.buffer = tbFloaters._createBuffer(floater.kind, floater.kind.fontPositive, text, floater.keepIt);
		} else {
			floater.buffer = tbFloaters._createBuffer(floater.kind, floater.kind.fontNegative, text, floater.keepIt);
		}
	},

	// Default signed number floater init method
	defaultNumberSignInit : function(floater) {
		"use strict";
		var text = floater.value.toString();
		if (floater.value > 0) { text = "+" + text; }
		if (floater.value >= 0) {
			floater.buffer = tbFloaters._createBuffer(floater.kind, floater.kind.fontPositive, text, floater.keepIt);
		} else {
			floater.buffer = tbFloaters._createBuffer(floater.kind, floater.kind.fontNegative, text, floater.keepIt);
		}
	},

	// Default surface floater init method
	defaultSurfaceInit : function(floater) {
		"use strict";
		floater.surface = floater.value;
	},

	// Default image floater init method
	defaultImageInit : function(floater) {
		"use strict";
		floater.image = floater.value;
		floater.subImage = 0;
	},

	// Default floater update method
	defaultUpdate : function(floater) {
		"use strict";
		if (floater.kind.easeX !== undefined) {
			floater.x = floater.kind.easeX(floater.time, floater.startX, floater.endX-floater.startX, floater.duration);
		} else {
			floater.x = tbEase.easeLinear(floater.time, floater.startX, floater.endX-floater.startX, floater.duration);
		}
		if (floater.kind.easeY !== undefined) {
			floater.y = floater.kind.easeY(floater.time, floater.startY, floater.endY-floater.startY, floater.duration);
		} else {
			floater.y = tbEase.easeLinear(floater.time, floater.startY, floater.endY-floater.startY, floater.duration);
		}
		if (floater.kind.easeScaleX !== undefined) {
			floater.scaleX = floater.kind.easeScaleX(floater.time, 0, 1, floater.duration);
		}
		if (floater.kind.easeScaleY !== undefined) {
			floater.scaleY = floater.kind.easeScaleY(floater.time, 0, 1, floater.duration);
		}
		if (floater.kind.easeAlpha !== undefined) {
			floater.alpha = floater.kind.easeAlpha(floater.time, 0, 1, floater.duration);
		}
		if (floater.kind.easeAngle !== undefined) {
			floater.angle = floater.kind.easeAngle(floater.time, 0, 360, floater.duration) % 360;
		}
		// Special processing for the image floater
		if (floater.image !== undefined) {
			floater.subImage = floater.time * floater.image.number / floater.duration;
		}
	},

	// Default floater draw method
	defaultDraw : function(floater) {
		"use strict";
		var context = tbGraphics.context, surface;
		if (floater.image !== undefined && floater.images !== null) {
			if (floater.scaleX === 1 && floater.scaleY === 1 && floater.angle === 0) {
				floater.image.drawAlpha(Math.floor(floater.subImage), floater.x, floater.y, floater.alpha);
			} else {
				floater.image.drawExt(Math.floor(floater.subImage), floater.x, floater.y,
					floater.scaleX, floater.scaleY, floater.angle, floater.alpha);
			}
		} else {
			if (floater.surface !== undefined && floater.surface !== null) {
				surface = floater.surface;
			} else {
				surface = tbFloaters._buffers[floater.buffer].surface;
			}
			if (floater.scaleX === 1 && floater.scaleY === 1 && floater.angle === 0) {
				surface.drawAlpha(floater.x - surface.width/2, floater.y - surface.height/2, floater.alpha);
			} else {
				if (Math.abs(floater.scaleX) < 0.0001 || Math.abs(floater.scaleY) < 0.0001) { return; }
				context.save();
				context.translate(floater.x, floater.y);
				context.rotate(-floater.angle * Math.PI/180);
				context.scale(floater.scaleX, floater.scaleY);
				surface.drawAlpha(-surface.width/2, -surface.height/2, floater.alpha);
				context.restore();
			}
		}
	},

	// Default end floater method
	defaultEnd : function(floater) {
		"use strict";
		// Special processing for text and number floaters
		if (floater.buffer !== undefined) {
			tbFloaters._endBuffer(floater.buffer);
		}
	},


/////////////////////////////////////////////////////////////
// The token callback methods
/////////////////////////////////////////////////////////////

	// Handles the step event for the floaters token
	handleEndStep : function(timePassed) {
		"use strict";
		var ind, floater, mustDraw = false;
		for (ind = 0; ind < tbFloaters._floaters.length; ind += 1) {
			floater = tbFloaters._floaters[ind];
			if (floater === undefined) { continue; }
			// Adapt the time
			if (floater.delay > 0) {
				floater.delay -= timePassed;
				if (floater.delay < 0) {
					floater.time += -floater.delay;
					floater.delay = 0;
				}
			} else {
				floater.time += timePassed;
			}
			if (floater.delay > 0) { continue; }
			// Remove if the time is up
			if (floater.time >= floater.duration) {
				floater.kind.end(floater);
				tbFloaters._floaters[ind] = undefined;
				mustDraw = true;		// To erase it
				continue;
			}
			// Update it
			floater.kind.update(floater);
			mustDraw = true;
		}
		if (mustDraw) { tbFloaters.canvas.drawTokens = true; }
	},

	// Handles the draw event for the floaters token
	handleDraw : function() {
		"use strict";
		var ind, floater;
		for (ind = 0; ind < tbFloaters._floaters.length; ind += 1) {
			floater = tbFloaters._floaters[ind];
			if (floater === undefined) { continue; }
			if (floater.delay > 0) { continue; }
			floater.kind.draw(floater);
		}
	},


/////////////////////////////////////////////////////////////
// Global maintenance
/////////////////////////////////////////////////////////////

	_floaters : [],					// The array of floaters

	// Initialize the floaters
	initFloaters : function (layer, depth, tags) {
		"use strict";
		tbFloaters.endFloaters();
		// Token related aspects
		if (layer === undefined) { layer = TG.gameEffectsLayer; }
		if (depth === undefined) { depth = -1000000; }
		if (tags === undefined) { tags = ["game"]; }
		tbFloaters.visible = true;
		tbFloaters.active = true;
		TG.functions.setLayerAndCanvas(tbFloaters, layer);
		tbFloaters.depth = depth;
		tbGame.addToken(tbFloaters);
		tbGame.addTokenTags(tbFloaters, tags);
		// Initialize with the default floater types
		tbFloaters._initBuffers();
		tbFloaters._initDefaultFloaters();
	},

	// Add a floater; returns the id
	addFloater : function (kind, value, startX, startY, endX, endY, duration, delay, keepIt) {
		"use strict";
		var floater, ind;
		if (delay === undefined) { if(kind.delay !== undefined){delay = kind.delay;}else{delay = 0;}}
		if (keepIt === undefined) { keepIt = tbFloaters._bufferSurfaces; }
		if(endX === undefined && kind.movementX !== undefined) { endX = startX + kind.movementX; }
		if(endY === undefined && kind.movementY !== undefined) { endY = startY + kind.movementY; }
		if(duration === undefined && kind.duration !== undefined) { duration = kind.duration; }
		// Create the floater object
		floater = {
				kind : kind,
				value: value,
				startX : startX,
				startY : startY,
				endX : endX,
				endY : endY,
				x : startX,
				y : startY,
				scaleX : 1,
				scaleY : 1,
				alpha : 1,
				angle : 0,
				time : 0,
				duration : duration,
				delay : delay,
				keepIt : keepIt
			};
		// Initialize it
		floater.kind.init(floater);
		// Put it in the list
		for (ind = 0; ind < tbFloaters._floaters.length; ind += 1) {
			if (tbFloaters._floaters[ind] === undefined) {
				tbFloaters._floaters[ind] = floater;
				return ind;
			}
		}
		tbFloaters._floaters.push(floater);
		return tbFloaters._floaters.length-1;
	},

	// Remove a floater
	removeFloater : function (ind) {
		"use strict";
		var floater;
		if (ind < 0 || ind >= tbFloaters._floaters.length) { return; }
		floater = tbFloaters._floaters[ind];
		if (floater === undefined) { return; }
		floater.kind.end(floater);
		tbFloaters._floaters[ind] = undefined;
	},

	// Remove a floater
	removeAllFloaters : function () {
		"use strict";
		var ind, floater;
		for (ind = 0; ind < tbFloaters._floaters.length; ind += 1) {
			floater = tbFloaters._floaters[ind];
			if (floater === undefined) { continue; }
			floater.kind.end(floater);
			tbFloaters._floaters[ind] = undefined;
		}
		tbFloaters._floaters = [];
	},

	// End the floaters
	endFloaters : function () {
		"use strict";
		tbFloaters.removeAllFloaters();
		tbFloaters.removeBuffers();
		tbGame.removeToken(tbFloaters);
	}


};
//******************************************************
//*****    Script Resource: game_version
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.version = TG.version || {};
TG.version["game"] = "1.5.3";
//******************************************************
//*****    Token Resource: LevelController
//******************************************************

// ***************************************
// Definition of game token LevelController
// ***************************************

// The constructor for the LevelController game token.
function LevelController() {
	"use strict";
	// Properties used by the engine
	this.depth = 1000;
	this.visible = testMode;
	this.active = true;
	TG.functions.setLayerAndCanvas(this, TG.gameLayer);

	// Your own properties
	this.settings = TG.settings.game.levelController;
	this.floaterSettings = TG.settings.game.floaters;
	tbFloaters.initFloaters();
	tbFloaters.setBuffering(true);

	this.comboSounds = [a_combo_01, a_combo_02, a_combo_03, a_combo_04];
	this.finger = null;
	this.currentFloaterSound = null;
	this.updateField = false;
	this.allowUserInteractions = false;

	this.mode = TG.controller.getDifficulty();
	this.showHints = this.settings.showHints[this.mode];
	this.random = new tbRandom();
	// Add it to the game engine
	tbGame.addToken(this);
	tbGame.addTokenTags(this, "game");

	this.initGame();
}

LevelController.GameState = {
	GAMESTART : 0,
	STAGESTART : 1,
	PLAY : 2,
	PAUSED : 3,
	STAGEEND : 4,
	GAMEEND: 5
};

LevelController.prototype.updateLevelSettings = function(){
	"use strict";
	var self = this, difficultyMap = ["zen", "classic", "timed"],
		interpolateSetting = function(valueBegin, valueEnd, stageIdxBegin, stageIdxEnd){
			var obj;
			if(valueEnd === undefined || valueEnd === null){
				return valueBegin;
			}
			if(typeof valueBegin === "object"){
				obj = (valueBegin instanceof  Array)? [] : {};
				for(var o in valueBegin){
					if(valueBegin.hasOwnProperty(o)) {
						obj[o]= interpolateSetting(valueBegin[o], valueEnd[o], stageIdxBegin, stageIdxEnd);
					}
				}
				return obj;
			}else if(typeof  valueBegin === "number"){
				if(stageIdxBegin === stageIdxEnd){
					return valueBegin;
				}
				return valueBegin + (valueEnd - valueBegin)/(stageIdxEnd - stageIdxBegin) *(self.stageIdx - stageIdxBegin);
			}
			return valueEnd;
		},
		replaceMissingValues = function(settingsArray){
			var i, prev = settingsArray[0];
			if( prev === null){
				return;
			}
			for(i=0; i<settingsArray.length; i++){
				if(settingsArray[i] !== null){
					prev = settingsArray[i];
				}else{
					settingsArray[i] = prev;
				}
			}
		};

	this.levelSettings = {};
	this.levelSettings.interpolated = {};

	//update level files
	var l = this.findNextLevelFile(this.stageIdx + 1, difficultyMap[this.mode]);
	if(l !== undefined){
		this.levelSettingsFirst = l;
	}
	l = this.findNextLevelFile(this.stageIdx + 2, difficultyMap[this.mode]);
	if(l !== undefined){
		this.levelSettingsSecond = l;
	}

	if(this.levelSettingsFirst.general.stageNo !== undefined && this.levelSettingsFirst.general.stageNo === this.stageIdx + 1){
		this.levelSettings = this.levelSettingsFirst;
	}else{
		//interpolate settings
		for(var s in this.levelSettingsFirst.interpolated){
			if(this.levelSettingsFirst.interpolated.hasOwnProperty(s)) {
				if(this.levelSettingsFirst.interpolated[s] instanceof Array ){
					replaceMissingValues(this.levelSettingsFirst.interpolated[s]);
					replaceMissingValues(this.levelSettingsSecond.interpolated[s]);
				}
				this.levelSettings.interpolated[s] = interpolateSetting(this.levelSettingsFirst.interpolated[s], this.levelSettingsSecond.interpolated[s], this.levelSettingsFirst.general.stageNo - 1, this.levelSettingsSecond.general.stageNo - 1);
			}
		}
		//get settings from second level
		this.levelSettings.discrete = this.levelSettingsSecond.discrete;
	}
	this.grid.handleUpdateSettings();

};

LevelController.prototype.findNextLevelFile = function(stageNo, mode){
	"use strict";
	var i, level, diff, bestMatch;
	for(i=0; i<TG.settings.game.levels.length; i++){
		level = TG.settings.game.levels[i];
		diff = level.general.stageNo - stageNo;
		if(diff >= 0 && (bestMatch === undefined || diff < bestMatch.general.stageNo - stageNo)){
		   if((typeof level.general.mode === "string" && level.general.mode === mode) || level.general.mode.indexOf(mode) >= 0){
				bestMatch = level;
			}
		}
	}
	return bestMatch;
};

LevelController.prototype.initGame = function() {
	"use strict";
	this.gameState = LevelController.GameState.GAMESTART;
	this.stageIdx = (testMode && this.settings.startStage !== undefined)? this.settings.startStage - 1 : 0;

	this.grid = new GridController(this);
	var gameUiOptions = new GameUIRuntimeOptions();
	gameUiOptions.hiddenComponents = ["lives"];
	gameUiOptions.settingsObject = gameUISettingsVar;
	this.gameUI = new GameUIController(gameUiOptions);
	this.gameUI.setScore(0);
	this.gameUI.setHighScore(TG.controller.getLevelHighScore());

	if(this.mode < 2){
		this.gameUI.setProgress(0);
		this.gameUI.setStage(this.stageIdx + 1);
	}else{
		this.gameUI.setTime(this.settings.gameTime);
		this.gameUI.timeMode = GameUITimeMode.NONE;
		this.gameUI.setTimeWarningSeconds(this.settings.timeWarningDuration/1000);
	}

	//initialise settings
	this.levelSettingsFirst = TG.settings.game.levels[0];
	this.levelSettingsSecond = TG.settings.game.levels[0];
	this.updateLevelSettings();
	if(this.levelSettings === undefined){
		this.endGame();
		return;
	}


	this.previousMaxPoints = 0;
	this.totalScore = 0;
	this.stageScore = 0;
	this.gameEnded = false;
	this.updateField = true;
	this.grid.init();
	this.grid.fillGrid(true);

	TG.controller.showStartLevelScreen(TG.dictionary.getText("levelStartHeader", "<LEVELSTARTHEADER>"), TG.dictionary.getText("levelStartText", "LEVELSTARTTEXT"), this.startStage, this);
};

LevelController.prototype.endGame = function(){
	"use strict";
	this.active = false;
	var stats = {
		"totalScore": this.totalScore,
		"stage": this.stageIdx + 1
	};
	TG.controller.levelEnded(stats);
};

LevelController.prototype.startStage = function() {
	"use strict";
	if(this.grid.animationPlaying()){
		this.gameState = LevelController.GameState.STAGEEND;
		return;
	}
	this.updateLevelSettings();


	this.gameUI.setStage(this.stageIdx + 1);
	if(this.settings.takeProgressToNextStage){
		this.stageScore = this.stageScore - this.previousMaxPoints;
	}else{
		this.stageScore = 0;
	}
	this.previousMaxPoints = this.levelSettings.interpolated.maxPoints;

	if(this.stageScore >= this.levelSettings.interpolated.maxPoints){
		this.stageIdx += 1;
		this.startStage();
	}else{
		this.gameUI.setProgress(100*this.stageScore/this.levelSettings.interpolated.maxPoints);
		this.comboTimer = 0;
		this.comboCounter = 0;
		this.maxComboCounter = this.levelSettings.interpolated.maxComboTimer.length;
		this.matchTimer = 0;
		this.grid.updateProgress(this.getProgress()/100);
		this.gameState = LevelController.GameState.PLAY;
		this.setAllowUserInteraction(true);

		if(this.mode < 2){ //in all modes except timed mode
			var kind = tbFloaters.cloneFloaterKind(tbFloaters.textFloater1);
			kind.font = f_stage_start;
			kind.easeScaleX = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot],[false, false, true],[this.floaterSettings.stageStartFloaterAppearDuration, this.floaterSettings.stageStartFloaterVisibleDuration, this.floaterSettings.stageStartFloaterDisappearDuration]);
			kind.easeScaleY = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot],[false, false, true],[this.floaterSettings.stageStartFloaterAppearDuration, this.floaterSettings.stageStartFloaterVisibleDuration, this.floaterSettings.stageStartFloaterDisappearDuration]);
			if(this.stageFloater !== undefined){
				tbFloaters.removeFloater(this.stageFloater);
				tbGame.removeToken(this.stageFloaterImage);
			}
			this.stageFloater = tbFloaters.addFloater(kind, TG.dictionary.getText("inGameStageFloater", "<STAGEFlOATER>")+" "+(this.stageIdx + 1),
					Math.round(TG.gameOffsetX + TG.gameWidth/2), Math.round(TG.gameOffsetY + TG.gameHeight/2), Math.round(TG.gameOffsetX + TG.gameWidth/2), Math.round(TG.gameOffsetY + TG.gameHeight/2),
					this.floaterSettings.stageStartFloaterAppearDuration + this.floaterSettings.stageStartFloaterVisibleDuration + this.floaterSettings.stageStartFloaterDisappearDuration,
					0, false);

			this.stageFloaterImage = new TG_AnimatedImage(-1000, TG.gameEffectsLayer, s_pop_floater);
			this.stageFloaterImage.x = Math.round(TG.gameOffsetX + TG.gameWidth/2);
			this.stageFloaterImage.y = Math.round(TG.gameOffsetY + TG.gameHeight/2);
			this.stageFloaterImage.addAnimator("subImage", s_pop_floater.number - 1, 600, undefined, function(){TG.functions.getCanvas(TG.gameEffectsLayer).drawTokens = true; tbGame.removeToken(this);});

		}else{
			this.gameUI.timeMode = GameUITimeMode.AUTO_DECREMENT;
		}
	}
	this.grid.flashGrid();
};

LevelController.prototype.endStage = function() {
	"use strict";
	var kind, text;
	//set state variables
	this.stageIdx += 1;
	this.gameState = LevelController.GameState.STAGEEND;
	if(this.mode < 2){
		this.setAllowUserInteraction(false);
		this.endAnimationStarted = false;

		//show floater
		kind = tbFloaters.cloneFloaterKind(tbFloaters.textFloater1);
		kind.font = f_stage_end;
		kind.easeScaleX = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot],[false, false, true],[this.floaterSettings.stageStartFloaterAppearDuration, 10000, this.floaterSettings.stageStartFloaterDisappearDuration]);
		kind.easeScaleY = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot],[false, false, true],[this.floaterSettings.stageStartFloaterAppearDuration, 10000, this.floaterSettings.stageStartFloaterDisappearDuration]);
		if(this.stageFloater){
			tbFloaters.removeFloater(this.stageFloater);
			if(this.stageFloaterImage){
				tbGame.removeToken(this.stageFloaterImage);
			}
		}
		text = TG.dictionary.getText("inGameStageEndFloater", "<STAGEENDFlOATER>").replace("<STAGE_NO>", this.stageIdx);
		this.stageFloater = tbFloaters.addFloater(kind, text,
				Math.round(TG.gameOffsetX + TG.gameWidth/2), Math.round(TG.gameOffsetY + TG.gameHeight/2), Math.round(TG.gameOffsetX + TG.gameWidth/2), Math.round(TG.gameOffsetY + TG.gameHeight/2),
				this.floaterSettings.stageStartFloaterAppearDuration + 10000 + this.floaterSettings.stageStartFloaterDisappearDuration,
				0, false);
		this.stageFloaterImage = new TG_AnimatedImage(-1000, TG.gameEffectsLayer, s_pop_floater);
		this.stageFloaterImage.x = Math.round(TG.gameOffsetX + TG.gameWidth/2);
		this.stageFloaterImage.y = Math.round(TG.gameOffsetY + TG.gameHeight/2);
		this.stageFloaterImage.addAnimator("subImage", s_pop_floater.number - 1, 600, undefined, function(){TG.functions.getCanvas(TG.gameEffectsLayer).drawTokens = true; tbGame.removeToken(this);});
	}else{
		this.endAnimationStarted = true;
	}
};

LevelController.prototype.forceGameEnd = function() {
	"use strict";
	this.gameState = LevelController.GameState.GAMEEND;
	this.allowUserInteractions = false;
	this.grid.setAllowUserInteraction(false);
};

LevelController.prototype.updateProgress = function(x, y, points, timeBonus){
	"use strict";
	if(this.gameState === LevelController.GameState.PLAY || this.gameState === LevelController.GameState.STAGEEND || this.gameState === LevelController.GameState.GAMEEND){
		// init vars
		var i, progress, kind, text, floaterX, delay,
			defaultTextFloaters = [tbFloaters.textFloater1, tbFloaters.textFloater2, tbFloaters.textFloater3, tbFloaters.textFloater4];

		// show score floater
		if(this.settings.showNumberFloaters && points !== 0){
			tbFloaters.addFloater(tbFloaters.numberSignFloater, points, x, y, x, y + this.floaterSettings.floaterNumberMovementY, this.floaterSettings.floaterNumberDuration, 0, false);
		}

		if(timeBonus !== undefined && timeBonus.bonus > 0){
			if(testMode && timeBonus.bonus > 60000){
				console.log("timeBonus: " + timeBonus.bonus);
			}
			timeBonus.bonus = Math.min(timeBonus.bonus, 60000);
		}

		// show text floater if earned
		if(this.settings.showTextFloaters){
			for(i = 0; i <  this.levelSettings.interpolated.floaterThresholds.length; i += 1){
				if(points < this.levelSettings.interpolated.floaterThresholds[i]){
					break;
				}
			}
			if(i > 0){
				i = Math.min(i, defaultTextFloaters.length);
				kind = tbFloaters.cloneFloaterKind(defaultTextFloaters[i-1]);
				//kind.font.setFontFace(f_themeDefault.fontFace);
				kind.font.setFontSize(this.floaterSettings.fontSize[i - 1]);
				kind.easeY = tbEase.easeCreateCombined([tbEase.easeConstant, tbEase.easeLinear], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeAlpha = tbEase.easeCreateCombined([tbEase.easeLinear, tbEase.easeConstant, tbEase.easeLinear], [false, false, true], [0, this.floaterSettings.floaterTextAppearDuration + this.floaterSettings.floaterTextVisibleDuration, this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeScaleX = tbEase.easeCreateCombined([function(t,b,c,d){return tbEase.easeOutOvershoot(t,b,c,d,6);}, tbEase.easeConstant], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeScaleY = tbEase.easeCreateCombined([function(t,b,c,d){return tbEase.easeOutOvershoot(t,b,c,d,6);}, tbEase.easeConstant], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);

				if((this.settings.showNumberFloaters && points !== 0) || (timeBonus !== undefined && timeBonus.bonus > 0)){
					y += this.floaterSettings.textFloaterOffset;
					if(timeBonus !== undefined && timeBonus.bonus > 0){
						delay = this.floaterSettings.floaterDelayForBonus;
					}
				}
				if(this.textFloater !== undefined){
					tbFloaters.removeFloater(this.textFloater);
				}

				text = TG.dictionary.getText("floater_"+(i-1), "<FLOATER_"+(i-1)+">");
				floaterX = Math.max(x, kind.font.textWidth(text)/2);
				floaterX = Math.min(x, TG.mainWidth - kind.font.textWidth(text)/2);
				this.textFloater = tbFloaters.addFloater(kind, text, floaterX, y,
						floaterX, y + this.floaterSettings.floaterTextMovementY,
						this.floaterSettings.floaterTextAppearDuration + this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration, delay);
				if(this.currentFloaterSound){
					tbAudio.stop(this.currentFloaterSound);
				}
				this.currentFloaterSound = tbAudio.play(this.comboSounds[i - 1]);
			}
		}

		if(this.gameState === LevelController.GameState.PLAY){
			//update time and show floater if timeBonus earned
			if(timeBonus !== undefined && timeBonus.bonus > 0){
				//update gameUI
				this.gameUI.setTime(this.gameUI.getTime() + timeBonus.bonus);
				//show floater
				text = TG.dictionary.getText("timeBonus", "+<TIME_BONUS> s Time Bonus").replace("<TIME_BONUS>", timeBonus.bonus/1000);
				kind = tbFloaters.cloneFloaterKind(defaultTextFloaters[0]);
				kind.font = f_timeBonus;
				kind.font.setFillColor(TG.settings.gameTheme.gem.timeBonusColours[timeBonus.colour]);
				kind.font.setFontSize(this.floaterSettings.fontSize[0]);
				kind.easeY = tbEase.easeCreateCombined([tbEase.easeConstant, tbEase.easeLinear], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeAlpha = tbEase.easeCreateCombined([tbEase.easeLinear, tbEase.easeConstant, tbEase.easeLinear], [false, false, true], [0, this.floaterSettings.floaterTextAppearDuration + this.floaterSettings.floaterTextVisibleDuration, this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeScaleX = tbEase.easeCreateCombined([function(t,b,c,d){return tbEase.easeOutOvershoot(t,b,c,d,6);}, tbEase.easeConstant], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);
				kind.easeScaleY = tbEase.easeCreateCombined([function(t,b,c,d){return tbEase.easeOutOvershoot(t,b,c,d,6);}, tbEase.easeConstant], [false, false], [this.floaterSettings.floaterTextAppearDuration, this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration]);
				floaterX = Math.max(x, kind.font.textWidth(text)/2);
				floaterX = Math.min(x, TG.mainWidth - kind.font.textWidth(text)/2);
				tbFloaters.addFloater(kind, text, floaterX, y,
						floaterX, y + this.floaterSettings.floaterTextMovementY + this.random.iRandomRange(0, this.floaterSettings.floaterTextMovementY/2),
						this.floaterSettings.floaterTextAppearDuration + this.floaterSettings.floaterTextVisibleDuration + this.floaterSettings.floaterTextDisappearDuration + this.random.iRandomRange(-200, 200));
			}
		}

		//add points
		this.totalScore += points;
		this.stageScore += points;
		this.gameUI.addScore(points);

		//update progress
		if(this.gameState === LevelController.GameState.PLAY){
			progress = this.getProgress() + (points/this.levelSettings.interpolated.maxPoints * 100);
			this.gameUI.setProgress(progress);
			// check if level end has been reached
			if(progress >= 100){
				if(this.mode === 0){//zen mode
					TG.controller.setLevelHighScore(this.totalScore);
				}
				this.endStage();
			}
			// if level is not completed, updated grid progress dependent settings
			this.grid.updateProgress(progress);
		}
	}
};

LevelController.prototype.getProgress = function(){
	"use strict";
	return this.gameUI.getProgress();
};

LevelController.prototype.setAllowUserInteraction = function(allow){
	"use strict";
	this.allowUserInteractions = allow;
	this.grid.setAllowUserInteraction(allow);
	if(TG.controller.getDifficulty() === 2){
		if(!allow){
			this.gameUI.timeMode = GameUITimeMode.NONE;
		}else{
			this.gameUI.timeMode = GameUITimeMode.AUTO_DECREMENT;
		}
	}
};

LevelController.prototype.handleNoMoreMoves = function(){
	"use strict";
	if(this.gameState === LevelController.GameState.PLAY){
		this.endFloaterText = TG.dictionary.getText("inGameNoMoreMovesFloater", "<NOMOREMOVESFLOATER>");
		this.forceGameEnd();
	}
};

LevelController.prototype.handleMatch = function(x, y, noOfGems, chainDepth, timeBonus) {
	"use strict";
	var multiplier, points;
	if(chainDepth === undefined){
		chainDepth = 0;
	}
	multiplier = 1 + this.levelSettings.interpolated.comboFactor * this.comboCounter + this.levelSettings.interpolated.chainFactor * chainDepth;
	points =  Math.round(multiplier * noOfGems * this.levelSettings.interpolated.pointsPerGem[Math.min(Math.max(noOfGems - 3, 0), this.levelSettings.interpolated.pointsPerGem.length -1)]);
	this.updateProgress(x + TG.gameOffsetX, y + TG.gameOffsetY, points, timeBonus);
	this.matchTimer = 0;
	this.comboTimer = 0;
	if(chainDepth === 0){
		this.comboCounter = Math.min(this.comboCounter + 1, this.maxComboCounter);
	}
};

LevelController.prototype.handleFalseMove = function(x, y) {
	"use strict";
	this.updateProgress(x + TG.gameOffsetX, y + TG.gameOffsetY, Math.round(-this.levelSettings.interpolated.falseMoveReduction));
	this.comboCounter = 0;
	this.comboTimer = 0;
};

LevelController.prototype.handleBoost = function(x, y, noOfGems, chainDepth, timeBonus){
	"use strict";
	var multiplier, points;
	if(chainDepth === undefined){
		chainDepth = 0;
	}
	multiplier = 1 + this.levelSettings.interpolated.comboFactor * this.comboCounter + this.levelSettings.interpolated.chainFactor * chainDepth;
	points = Math.round(multiplier * noOfGems * this.levelSettings.interpolated.pointsPerGem[Math.min(Math.max(noOfGems - 3, 0), this.levelSettings.interpolated.pointsPerGem.length -1)]);
	this.updateProgress(x + TG.gameOffsetX, y + TG.gameOffsetY, points, timeBonus);
	this.matchTimer = 0;
	this.comboTimer = 0;
	if(chainDepth === 0){
		this.comboCounter = Math.min(this.comboCounter + 1, this.maxComboCounter);
	}
};

// The following event handlers describe the behavior of the token

// handleRemove is called when the token is removed from the game.
LevelController.prototype.handleRemove = function() {
	"use strict";
	tbFloaters.endFloaters();
	tbGame.removeToken(this.grid);
	tbGame.removeToken(this.gameUI);
};

// handleBeginStep is called each step before processing other events.
LevelController.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	var kind, time;
	if(this.updateField){
		TG.functions.getCanvas(TG.gameLayer).drawTokens = true;
	}

	if(this.gameState === LevelController.GameState.STAGEEND){
		if(!this.grid.animationPlaying()){
			if(!this.endAnimationStarted){
				this.grid.startStageEndAnimation();
				this.endAnimationStarted = true;
			}else{
				this.endAnimationStarted = false;
				tbFloaters.removeFloater(this.stageFloater);
				if(this.mode < 2){
					this.grid.fillGrid(true);
				}
				this.startStage();
			}
		}
	}

	if(this.gameState === LevelController.GameState.GAMEEND){
		if(this.grid.ready()){
			if(this.endFloater === undefined){
				// show game over floater
				kind = tbFloaters.cloneFloaterKind(tbFloaters.textFloater1);
				kind.font = f_stage_fail;
				kind.easeScaleX = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot], [false, false, true], [
						this.floaterSettings.stageFaileFloaterAppearDuration, this.floaterSettings.stageFailFloaterVisibleDuration,
						this.floaterSettings.stageFailFloaterDisappearDuration]);
				kind.easeScaleY = tbEase.easeCreateCombined([tbEase.easeOutElastic, tbEase.easeConstant, tbEase.easeInOvershoot], [false, false, true], [
						this.floaterSettings.stageFaileFloaterAppearDuration, this.floaterSettings.stageFailFloaterVisibleDuration,
						this.floaterSettings.stageFailFloaterDisappearDuration]);
				kind.duration = this.floaterSettings.stageFailFloaterVisibleDuration;
				tbFloaters.removeAllFloaters();
				if(this.mode === 2){
					tbAudio.play(a_combo_04);
				}
				this.endFloater = tbFloaters.addFloater(kind, this.endFloaterText, Math.round(TG.gameOffsetX + TG.gameWidth / 2), Math.round(TG.gameOffsetY
						+ TG.gameHeight / 2), Math.round(TG.gameOffsetX + TG.gameWidth / 2), Math.round(TG.gameOffsetY + TG.gameHeight / 2), undefined, 0,
						false);
				this.endTimer = 0;
			}else if(this.endTimer < this.settings.endGameBoostActivateDelay){
				this.endTimer += timePassed;
				if(this.endTimer >= this.settings.endGameBoostActivateDelay){
					this.grid.activateAllBoosts();
				}
			}else if(!this.endAnimationStarted){
				this.grid.startStageEndAnimation();
				this.endAnimationStarted = true;
			}/*else{
				this.endTimer += timePassed;
				if(this.endTimer >= this.settings.endGameBoostActivateDelay + this.settings.endGameEndScreenDelay){
					this.endGame();
				}
			}*/
		}else if(this.endAnimationStarted && !this.grid.animationPlaying()){
			this.endTimer += timePassed;
			if(this.endTimer >= this.settings.endGameBoostActivateDelay + this.settings.endGameEndScreenDelay){
				this.endGame();
			}
		}
	}else if(this.mode === 2){
		time = this.gameUI.getTime();
		if(time <= 0){
			this.endFloaterText = TG.dictionary.getText("inGameTimeUpFloater", "<TIMEUPFLOATER>");
			this.forceGameEnd();
		}
	}
};

// handleTouchDown is called when a touch is started or if a mouse button is pressed.
LevelController.prototype.handleTouchDown = function(finger, x, y) {
	"use strict";
	if(this.gameState === LevelController.GameState.PLAY && this.allowUserInteractions && this.finger === null &&
		x >= TG.gameOffsetX && x < TG.gameOffsetX + TG.gameWidth &&
		y >= TG.gameOffsetY && y < TG.gameOffsetY + TG.gameHeight){
		this.grid.touchDown(x-TG.gameOffsetX, y-TG.gameOffsetY);
		this.finger = finger;
	}
};

// handleTouchUp is called when a touch is ended or if a mouse button is released.
LevelController.prototype.handleTouchUp = function(finger, x, y) {
	"use strict";
	if(this.gameState === LevelController.GameState.PLAY && this.allowUserInteractions && finger === this.finger &&
		x >= TG.gameOffsetX && x < TG.gameOffsetX + TG.gameWidth &&
		y >= TG.gameOffsetY && y < TG.gameOffsetY + TG.gameHeight){
		this.grid.touchUp(x-TG.gameOffsetX, y-TG.gameOffsetY);
		this.finger = null;
	}
};

LevelController.prototype.handleKeyDown = function(keyCode) {
	"use strict";
	if (keyCode === tbKeyCodes.vkL && testMode) {
		this.handleNoMoreMoves();
	}
};

// handleEndStep is called each step after processing other events (before draw).
LevelController.prototype.handleEndStep = function(timePassed) {
	"use strict";
	if(this.gameState === LevelController.GameState.PLAY && this.allowUserInteractions){
		//update comboCounter
		this.comboTimer += timePassed;
		if(this.comboTimer > this.levelSettings.interpolated.maxComboTimer[Math.min(this.comboCounter, this.maxComboCounter - 1)]){
			this.comboTimer = 0;
			if(this.comboCounter !== 0){
				this.comboCounter = 0;
			}
		}
		//update matchTimer and reduce points if necessary
		this.matchTimer += timePassed;
		if(this.matchTimer > this.levelSettings.interpolated.timeBeforeReduction){
			this.matchTimer = 0;
			if(this.progress > 0 && this.levelSettings.interpolated.reductionAmount > 0){
				this.updateProgress(Math.round(TG.gameUIOffsetX + TG.gameUIWidth/2), TG.gameUIOffsetY + TG.gameUIHeight -tgScale(20), -this.levelSettings.interpolated.reductionAmount);
			}
		}
	}
};

/*
LevelController.prototype.handleDraw = function(){
	"use strict";
	var f = f_default.clone();
	f.setProperties({fillColor: "white", fontSize: 20});
	f.drawSimple("cc: "+this.comboCounter, 45, 0);
};
*/


//******************************************************
//*****    Token Resource: GridController
//******************************************************

// ***************************************
// Definition of game token GridController
// ***************************************

// The constructor for the GridController game token.
function GridController(parentController) {
	"use strict";
	// Properties used by the engine
	this.depth = 800;
	this.visible = true;
	this.active = false;
	//this.group = "gameControllers";
	TG.functions.setLayerAndCanvas(this, TG.gameLayer);
	// Your own properties
	this.random = new tbRandom();
	this.settings = TG.settings.game.gridController;
	this.controller = parentController;
	this.ensureMove = this.settings.alwaysEnsureMove[this.controller.mode];
	this.r = this.settings.rows;
	this.c = this.settings.columns;
    this.tileSize = this.settings.tileSize;
	this.tiles = this.r * this.c;
	this.grid = [];
    this.dirtyColumns = [];
    this.tileFlashes = [];
    this.tileFlashColour = (TG.settings.gameTheme.gridController !== undefined)? TG.settings.gameTheme.gridController.tileFlashColours : undefined;
    this.tileFlashEase = tbEase.easeCreateCombined([tbEase.easeInQuartic, tbEase.easeInQuartic], [false, true], [0.2, 0.8]);
    this.showTileFlash = this.tileFlashColour !== undefined;
    var c_idx, r_idx, i;
	for(c_idx=0; c_idx<this.c; c_idx+=1){
		this.dirtyColumns[c_idx] = [];
		for(r_idx=0; r_idx<this.r; r_idx+=1){
			this.dirtyColumns[c_idx].push(new Gem(this));
		}
	}
    this.activeBoosts = [];
    this.activateAllBoostsFlag = false;
    for(i = 0; i < 3; i += 1){
        this.activeBoosts[i] = 0;
    }
    this.matchSounds = [a_match_01, a_match_02, a_match_03, a_match_04];
    this.lastMatchSoundIdx = -1;
    this.lastMatchSound = null;
    this.timeBonusses = [];
    this.timeBonusCount = 0;
    this.clearFieldAnimationPlaying = false;
	// Add it to the game engine
	tbGame.addToken(this);
}

GridController.prototype.ready = function(ignoreActivateAllBoostFlag){
    "use strict";
    var i;
    if(this.animationController.active){
        return false;
    }
    for(i=0; i<this.c*this.r; i+=1){
        if(this.grid[i] === null){
            return false;
        }
        if(this.grid[i].playState !== Gem.PlayState.NORMAL){
            return false;
        }
    }
    if(!ignoreActivateAllBoostFlag){
        return !this.activateAllBoostsFlag;
    }
    return true;
};

GridController.prototype.forceReady = function(){
    "use strict";
    this.active = false;
    this.animationController.stopAllAnimations();
    var i;
    for(i=0; i<this.c*this.r; i+=1){
        if(this.grid[i]){
            this.grid[i].forceRemove();
        }
	}
};

GridController.prototype.animationPlaying = function(){
    "use strict";
    return this.animationController.active || this.clearFieldAnimationPlaying;
};

GridController.prototype.getXByIdx = function(idx) {
	"use strict";
	return (Math.floor(idx/this.r) + 0.5) * this.tileSize;
};

GridController.prototype.getYByIdx = function(idx) {
	"use strict";
	return (Math.round(idx % this.r) + 0.5) * this.tileSize;
};

GridController.prototype.getIdxByXY = function(x, y){
    "use strict";
    var row, col;
    row = Math.round((y + 0.5)/this.tileSize);
    col = Math.round((x + 0.5)/this.tileSize);
    return (col * this.r + row);
};

GridController.prototype.init = function(){
	"use strict";
    this.dragging = false;
	this.dragStart = null;
	this.fallenGems = [];
	this.swappedGems = [];
    this.matchedGems = [];
    this.powerQueue = [];
    this.chainingGrid = [];
    this.nextChainId = 0;
    this.chainingCounter = [];
    this.gettingPowerQueue = [];
    this.updateProgress(0);
    this.active = true;
    this.updateGrid = true;
    this.selectionController.initLevel();
    if(this.showTileFlash){
        var i = 0;
        for(i=0; i<this.r*this.c; i++){
            this.tileFlashes[i] = {active: false};
        }
    }
};

GridController.prototype.flashGrid = function(){
	"use strict";
	var i, dist;
	for(i=0; i<this.grid.length; i++){
		if(this.grid[i]){
			dist = Math.max(Math.floor(i/this.r), i%this.r);
			this.grid[i].showShineAnimation(dist * 100 + defaultRandom.random()*200);
		}
	}
};

GridController.prototype.setAllowUserInteraction = function(allowUserInteraction) {
    "use strict";
    if(allowUserInteraction && this.controller.gameState !== LevelController.GameState.PLAY){
        return;
    }
    this.hintController.setActivate(allowUserInteraction);
    this.selectionController.setActivate(allowUserInteraction);
};

GridController.prototype.chooseGemColour = function(forbiddenColours) {
	"use strict";
    var r, count, i, repeat, colour;
    do{
        r = this.random.random();
        count = 0;
        for(colour=0; colour<this.gemColourChance.length; colour+=1){
           count += this.gemColourChance[colour];
            if(r < count){
                break;
            }
        }
        repeat = false;
        if(forbiddenColours !== undefined){
            for(i=0; i<forbiddenColours.length; i+=1){
                if(colour === forbiddenColours[i]){
                    repeat = true;
                    break;
                }
            }
        }
	}
	while (repeat);
    return colour;
};

GridController.prototype.handleUpdateSettings = function() {
    "use strict";
    this.gamePlaySettings = this.controller.levelSettings;
};

GridController.prototype.getProgressBasedSettingsValue = function(setting, range, integer) {
    "use strict";
    var i, progress = this.controller.getProgress(),
        idx = Math.ceil(progress/10 + 0.1) - 1,
        value = setting[idx];
    if(value === null || value === undefined){
        for(i = idx - 1; i >= 0; i--){
            value = setting[i];
            if(value !== null && value !== undefined){
                break;
            }
        }
    }
    if(!range){
        return value;
    }
    return this.getRandomSettingsValueFromRange(value, integer);
};

// handleAdd is called when the token is (re-) added to the game.
GridController.prototype.getRandomSettingsValueFromRange = function(setting, integer) {
    "use strict";
    if(setting[0] === setting[1]){
        return setting[0];
    }
    if(integer){
        return this.random.iRandomRange(setting[0], setting[1]);
    }
    return this.random.randomRange(setting[0], setting[1]);
};

GridController.prototype.updateProgress = function(progress){
	"use strict";
	var i, tot = 0;
	progress /= 100;
	this.gemColourChance = this.getProgressBasedSettingsValue(this.gamePlaySettings.interpolated.gemColourChance);
	for(i=0; i<this.gemColourChance.length; i++){
	    tot += this.gemColourChance[i];
	}
	for(i=0; i<this.gemColourChance.length; i++){
        this.gemColourChance[i] = this.gemColourChance[i]/tot;
    }
};

GridController.prototype.fillGrid = function(repeatTillMatchFound) {
	"use strict";
    var totDelay, c, r, gem, index, matches, forbiddenColours, i, boostCount, rowDelay;
	totDelay = 0;
    // assign random colours
    for(c=0; c<this.c; c+=1){
        for(r=0; r<this.r; r+=1){
            gem = this.dirtyColumns[c].shift();
            index = c * this.r + r;
            gem.init(this.getXByIdx(index), -this.tileSize/2, this.chooseGemColour());
            this.grid[index] = gem;
            this.chainingGrid[index] = -1;
        }
    }
    // ready grid (i.e. remove matches, add boost, check if a move can be made, if not
    // reassign random colours)
	do{
		// find and remove matches
        for(index=0; index<this.tiles; index+=1){
			matches = this.checkMatch(index);
            if(matches !== 0){
                forbiddenColours = [];
                if(index >= this.r){
                    forbiddenColours.push(this.grid[index - this.r].colour);
                }
                if(index % this.r !==0 ){
                    forbiddenColours.push(this.grid[index - 1].colour);
                }
				this.grid[index].colour = this.chooseGemColour(forbiddenColours);
			}
		}

        // redistribute boosts carried over from the previous level (if any)
        for(i = 0; i < this.activeBoosts.length; i += 1){
			boostCount = this.activeBoosts[i];
			while(boostCount > 0){
				index = this.random.iRandom(this.tiles-1);
				while(this.grid[index].powerType !== undefined){
					index = this.random.iRandom(this.tiles-1);
                }
                gem = this.grid[index];
                gem.forcePower(i);
                boostCount -= 1;
			}
		}

        // check if field contains match possibilities, if not reassign colours
		if(repeatTillMatchFound){
            this.match = this.findPossibleMatch();
            if(this.match >= 0){
                repeatTillMatchFound = false;
                this.hintController.updateHint(this.grid[this.match]);
            }
		}
        if(!repeatTillMatchFound){
			// grid is ready, make gems fall
            for(c=0; c<this.c; c+=1){
				rowDelay = 500;
				for(r=this.r-1; r>=0; r-=1){
					index = c * this.r + r;
					this.grid[index].fall(index, rowDelay);
					rowDelay += 50 + this.random.random(r*20);
					totDelay = Math.max(rowDelay, totDelay);
				}
			}
		}else{
            // grid not ready reassign random colours and start over
            for(index = 0; index < this.tiles; index += 1){
                this.grid[index].colour = this.chooseGemColour();
                this.grid[index].powerType = undefined;
            }
        }
	}
	while (repeatTillMatchFound);
	this.updateGrid = true;
	return totDelay;
};

GridController.prototype.emptyGrid = function() {
    "use strict";
    var idx, r, c, delayStep, gem;
    idx = 0;
    delayStep = 100;
    for(r=0; r<this.r; r+=1){
        idx = r;
        for(c=0; c<this.c; c+=1){
            gem = this.grid[idx];
            if(gem !== null){
               gem.remove();
            }
            idx += this.r;
        }
    }
    this.active = false;
    this.hintController.reset();
    return delayStep * this.r + TG.settings.game.gem.shrinkDuration;
};

GridController.prototype.startStageEndAnimation = function(){
    "use strict";
    this.clearFieldAnimationPlaying = true;
    this.clearFieldAnimationStarted = false;
    this.clearFieldTimer = 0;
};

GridController.prototype.addTimeBonus = function(bonus){
    "use strict";
    var i;
    this.timeBonusCount += 1;
    for(i=0; i<this.timeBonusses.length; i++){
        if(this.timeBonusses[i] === 0){
            this.timeBonusses[i] = bonus;
            return;
        }
    }
    this.timeBonusses.push(bonus);
};

GridController.prototype.activateAllBoosts = function(){
    "use strict";
    var i, otherGem;
    for(i=0; i<this.c*this.r; i+=1){
        if(this.grid[i] && this.grid[i].powerType !== undefined){
            otherGem = this.areNeighbours(i, i+1)? this.grid[i + 1] : this.grid[i - 1];
            this.executePower(this.grid[i], 0, otherGem);
            this.active = true;
            this.activateAllBoostsFlag = true;
            return;
        }
    }
    this.activateAllBoostsFlag = false;
};

GridController.prototype.resetChains = function(){
    "use strict";
    var i;
    for(i = 0; i < this.tiles; i += 1){
        this.chainingGrid[i] = -1;
    }
    this.nextChainId = 0;
    this.chainingCounter = [];
};

GridController.prototype.touchDown = function(x, y){
	"use strict";
    var row, col, idx, selected;
    row = Math.floor(y/this.tileSize);
	col = Math.floor(x/this.tileSize);
	idx = col * this.r + row;
    selected = this.selectionController.selectedGem;
    if(selected !== null && this.areNeighbours(selected.currentIdx, idx)){
        this.swapGems(selected.currentIdx, idx);
        this.dragging = false;
    }
    else{
        this.dragging = true;
        this.dragStart = {x: x, y: y};
        this.selectionController.setSelection(this.grid[idx]);
    }
};

GridController.prototype.touchUp = function(x,y){
	"use strict";
    this.dragging = false;
};

GridController.prototype.areNeighbours = function(idx1, idx2){
	"use strict";
    return((idx1 + 1 === idx2 && idx2 % this.r !== 0) || (idx1 - 1 === idx2 && idx1 % this.r !== 0) || (idx1 + this.r === idx2) || (idx1 - this.r === idx2));
};

GridController.prototype.swapGems = function(idx1, idx2, back){
	"use strict";
    var gem1, gem2;
    gem1 = this.grid[idx1];
    gem2 = this.grid[idx2];
    if(gem1 !== null && gem2 !== null){
        // if hypercube immediately activate
        if(gem1.powerType === 2 && gem2.powerType === 2){
            this.handleHyperCubeCombo(gem1, gem2);
        }else if(gem1.powerType === 2){
            gem1.colour = gem2.colour;
            this.handleHyperCube(gem1, undefined, gem2);
        }else if(gem2.powerType === 2){
            gem2.colour = gem1.colour;
            this.handleHyperCube(gem2, undefined, gem1);
        }// else swap gems
        else if(gem1.playState === Gem.PlayState.NORMAL && gem2.playState === Gem.PlayState.NORMAL){
            this.grid[idx1] = gem2;
            this.grid[idx2] = gem1;
            gem1.swap(idx2, back);
            gem2.swap(idx1, back);
        }
    }
    //this.hintController.reset();
};

GridController.prototype.checkForSwap = function(){
	"use strict";
    var selected, x, y, xDiff, yDiff, other, selectedIdx;
    selected = this.selectionController.selectedGem;
    if(selected === null || this.controller.finger === null){
		return;
    }

	//determine swap direction
	x = tbGame.touchX(this.controller.finger) - TG.gameOffsetX;
	y = tbGame.touchY(this.controller.finger) - TG.gameOffsetY;
	xDiff = x - this.dragStart.x;
	yDiff = y - this.dragStart.y;
	other = -1;
    selectedIdx = selected.currentIdx;
	if(xDiff >= this.settings.swapDist){
		other = selectedIdx + this.r;
    }else if(xDiff <= -this.settings.swapDist){
		other = selectedIdx - this.r;
    }else if(yDiff >= this.settings.swapDist){
		other = selectedIdx + 1;
    }else if(yDiff <= -this.settings.swapDist){
		other = selectedIdx - 1;
    }

	//make swap
	if(other >= 0 && other < this.tiles && this.areNeighbours(selectedIdx, other)){
        this.swapGems(selectedIdx, other, false);
		this.dragging = false;
	}
};

GridController.prototype.checkAndHandleMatch = function(idx, chained){
    "use strict";
    var special, gem, x, y, match, vCount, hCount, tCount, v, h, that, power, setInMatch, chainId, timeBonus, i, chance, tot, soundIdx;
    special = -1;
    gem = this.grid[idx];
    x = gem.x;
    y = gem.y;
    if(gem.powerType === 2){
        return 0;
    }
    // determine if match has been made
    match = this.checkMatch(idx);
    if(match === 0){
        return 0;
    }
    if(match.extendOptions){
        return 0;
    }
    hCount = match.horizontal.length;
    vCount = match.vertical.length;
    tCount = 0;

     //start new chain if necessary
    if(!chained){
        chainId = this.nextChainId;
        this.nextChainId += 1;
        this.chainingCounter[chainId] = 0;
    }else{
        chainId = this.chainingGrid[idx];
        this.chainingCounter[chainId] = (this.chainingCounter[chainId] === undefined)? 1 : this.chainingCounter[chainId] + 1;
    }

    // determine if special combo was made
    if (vCount > 3 || hCount > 3){ // Hypercube 5 or more
        special = 2;
    }else if (hCount > 1 && vCount > 1){ // SuperNova L or T shape
        special = 1;
    }else if (vCount > 2 || hCount > 2){ // Bomb 4 in a row
       special = 0;
    }

    if(special >= 0){
		this.gettingPowerQueue.push({
		    gem: gem,
		    power: special
		});
	}else{
		if(!this.lastMatchSound || !tbAudio.isPlaying(this.lastMatchSound)
		        || tbAudio.getPosition(this.lastMatchSound) > this.matchSounds[this.lastMatchSoundIdx].duration / 2){
			if(this.lastMatchSound && tbAudio.isPlaying(this.lastMatchSound)){
				tbAudio.stop(this.lastMatchSound);
			}
			soundIdx = Math.min(this.lastMatchSoundIdx + 1, this.matchSounds.length - 1);
			soundIdx = Math.min(soundIdx, Math.max(Math.min(this.chainingCounter[chainId], this.matchSounds.length - 1), Math.min(this.controller.comboCounter,
			        this.matchSounds.length - 1)));
			this.lastMatchSound = tbAudio.play(this.matchSounds[soundIdx]);
			this.lastMatchSoundIdx = soundIdx;
		}
	}

    // define function to set gems in match
    that = this;
    power = false;
    setInMatch = function(index){
        gem = that.grid[index];
        gem.inMatch = true;
        that.chainingGrid[index] = chainId;
        if(gem.powerType === undefined){
            that.matchedGems.push({gem: gem, causeIdx: idx, special: special, chained: (that.chainingCounter[chainId] > 0)});
            tCount += 1;
        }else{
            that.powerQueue.push({gem: gem, delay:0, depth: 0, chainId: chainId});
            power = true;
        }
        that.addTileFlash(index, gem.colour, 0);
        timeBonus = that.updateTimeBonus(timeBonus, gem);
    };

    // set gems in match
    setInMatch(idx);
    if(vCount > 1){
        for(v=0; v < vCount; v+=1){
            setInMatch(match.vertical[v]);
        }
    }
    if(hCount > 1){
        for (h=0; h < hCount; h+=1){
            setInMatch(match.horizontal[h]);
        }
    }
    //add time bonus
    if(this.controller.mode === 2 && this.settings.timeBonusChanceMatch[tCount - 3] !== undefined){
        chance = this.random.random();
        tot = 0;
        for(i=0; i<this.settings.timeBonusChanceMatch[tCount - 3].length; i++){
            tot += this.settings.timeBonusChanceMatch[tCount - 3][i];
            if(chance < tot){
                this.addTimeBonus((i+1)*5000);
                break;
            }
        }
    }

    if(!power){
        this.controller.handleMatch(x, y, tCount, this.chainingCounter[chainId], timeBonus);
    }
    return tCount;
};

GridController.prototype.canGemMatchWithColour = function(gem, colour){
    "use strict";
    return (gem.playState === Gem.PlayState.NORMAL && gem.powerType !== 2 && gem.colour === colour && !gem.inMatch);
};

GridController.prototype.couldGemMatchWithColour = function(gem, colour){
    "use strict";
    return ((gem.playState === Gem.PlayState.FALLING || gem.playState === Gem.PlayState.SWAPPING || gem.playState === Gem.PlayState.GETTING_POWER) && gem.powerType !== 2 && gem.colour === colour);
};

GridController.prototype.isPossibleMatch = function(idx){
    "use strict";
    var colour, can_top, can_bot, topleftRuby, botleftRuby, toprightRuby, botrightRuby,
        c = Math.floor(idx/this.r),
        r = idx%this.c,
        gem = this.grid[idx],
        checkGem = function(gem, colour){
            return gem !== null && gem.colour === colour && gem.powerType !== 2 &&
                (gem.playState !== Gem.PlayState.REMOVING &&
                 gem.playState !== Gem.PlayState.MOVING &&
                 gem.playState !== Gem.PlayState.EXPLODING);
        };

    if(gem !== null){
        //hypercube means always match possibilities!
        if (gem.powerType === 2){
            return idx;
        }
        colour = gem.colour;
        can_top = r > 0;
        can_bot = r < this.r - 1;
        topleftRuby = false;
        botleftRuby = false;
        toprightRuby = false;
        botrightRuby = false;
        //find cross possibilities
        if(c > 0){
            if(can_top){
                gem = this.grid[(c-1) * this.r + (r-1)];
                if(checkGem(gem, colour)){
                    topleftRuby = true;
                }
            }
            if (can_bot){
                gem = this.grid[(c-1) * this.r + (r+1)];
                if(checkGem(gem, colour)){
                    botleftRuby = true;
                }
            }
        }
        if(c < this.c - 1){
            if(can_top){
                gem = this.grid[(c+1) * this.r + (r-1)];
                if(checkGem(gem, colour)){
                    toprightRuby = true;
                }
            }
            if (can_bot){
                gem = this.grid[(c+1) * this.r + (r+1)];
                if(checkGem(gem, colour)){
                    botrightRuby = true;
                }
            }
        }
        if ((topleftRuby && toprightRuby) || (topleftRuby && botleftRuby) ||
            (botleftRuby && botrightRuby) || (botrightRuby && toprightRuby)){
            return idx;
        }

        //find corner double possibilities && find straight triple possibilities
        if(r > 1){ //Top
            //check topleft/toptopleft
            if(topleftRuby){
                gem = this.grid[(c-1) * this.r + (r-2)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(toprightRuby){
                gem = this.grid[(c+1) * this.r + (r-2)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(r > 2){
                gem = this.grid[c * this.r + (r-2)];
                if(checkGem(gem, colour)){
                    gem = this.grid[c * this.r + (r-3)];
                    if(checkGem(gem, colour)){
                        return idx;
                    }
                }
            }
        }
        if (r < this.r - 2){ //Bottom
            //check botleft/botbotleft
            if(botleftRuby){
                gem = this.grid[(c-1) * this.r + (r+2)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(botrightRuby){
                gem = this.grid[(c+1) * this.r + (r+2)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(r < this.r - 3){
                gem = this.grid[c * this.r + (r+2)];
                if(checkGem(gem, colour)){
                    gem = this.grid[c * this.r + (r+3)];
                    if(checkGem(gem, colour)){
                        return idx;
                    }
                }
            }
        }
        if(c > 1){ //Left
            //check botleft/botleftleft
            if(botleftRuby){
                gem = this.grid[(c-2) * this.r + (r+1)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(topleftRuby){
                gem = this.grid[(c-2) * this.r + (r-1)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(c > 2){
                gem = this.grid[(c-2) * this.r + r];
                if(checkGem(gem, colour)){
                    gem = this.grid[(c-3) * this.r + r];
                    if(checkGem(gem, colour)){
                        return idx;
                    }
                }
            }
        }

        if (c < this.c - 2){ //Right
            //check botleft/botleftleft
            if(botrightRuby){
                gem = this.grid[(c+2) * this.r + (r+1)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(toprightRuby){
                gem = this.grid[(c+2) * this.r + (r-1)];
                if(checkGem(gem, colour)){
                    return idx;
                }
            }
            if(c < this.c - 3){
                gem = this.grid[(c+2) * this.r + r];
                if(checkGem(gem, colour)){
                    gem = this.grid[(c+3) * this.r + r];
                    if(checkGem(gem, colour)){
                        return idx;
                    }
                }
            }
        }
    }
    return -1;
};

GridController.prototype.findPossibleMatch = function(){
    "use strict";
    var i, match;
    for(i=0; i<this.tiles; i++){
        match = this.isPossibleMatch(i);
        if(match > 0){
            return match;
        }
    }
    return -1;
};

GridController.prototype.checkMatch = function(idx){
	"use strict";
    var gem, neighbour, horizontalMatch, verticalMatch, l, r, u, d, hcount, vcount,  matchCouldExtend;
    gem = this.grid[idx];
	if(gem === null){
		return 0;
    }
    //find matching group
	if(gem.playState === Gem.PlayState.NORMAL){
		horizontalMatch = [];
		verticalMatch = [];
		//check left
        for(l=idx-this.r; l>=0; l-=this.r){
            neighbour = this.grid[l];
            if(neighbour !== null && this.canGemMatchWithColour(neighbour, gem.colour)){
                horizontalMatch.push(l);
            }else{
                break;
            }
        }
        //check right
        for(r=idx+this.r; r<this.tiles; r+=this.r){
            neighbour = this.grid[r];
            if(neighbour !== null && this.canGemMatchWithColour(neighbour, gem.colour)){
                horizontalMatch.push(r);
            }else{
                break;
            }
        }
        //check up
        for(u=idx-1; (u+1)%this.r !== 0; u-=1){
            neighbour = this.grid[u];
            if(neighbour !== null && this.canGemMatchWithColour(neighbour, gem.colour)){
                verticalMatch.push(u);
            }else{
                break;
            }
        }
        //check down
        for(d=idx+1; d%this.r !== 0; d+=1){
            neighbour = this.grid[d];
            if(neighbour !== null && this.canGemMatchWithColour(neighbour, gem.colour)){
                verticalMatch.push(d);
            }else{
                break;
            }
        }
		hcount = horizontalMatch.length;
		vcount = verticalMatch.length;
        matchCouldExtend = false;
        //check if the match could be extended (i.e. more gems could be included in the match) after neighbouring
        //gems finish there current action (i.e. they are now excluded because of their playstate, but are of the
        //right colour
        if(hcount > 1){
            if((r < this.tiles && this.grid[r] !== null && this.couldGemMatchWithColour(this.grid[r], gem.colour))
                || (l >= 0 && this.grid[l] !== null && this.couldGemMatchWithColour(this.grid[l], gem.colour))){
                matchCouldExtend = true;
            }
        }
        if(vcount > 1){
            if(((u+1)%this.r !== 0 && this.grid[u] !== null && this.couldGemMatchWithColour(this.grid[u], gem.colour))
                || ( d%this.r !== 0 && this.grid[d] !== null && this.couldGemMatchWithColour(this.grid[d], gem.colour))){
                matchCouldExtend = true;
            }
        }

        if (vcount > 1 || hcount > 1){
			return {horizontal: horizontalMatch, vertical: verticalMatch, extendOptions: matchCouldExtend};
		}
	}
	return 0;
};

GridController.prototype.handleGemFallFinished = function(gem){
	"use strict";
    this.fallenGems.push(gem);
};

GridController.prototype.handleGemDestroyFinished = function(gem){
    "use strict";
    var col = Math.floor(gem.currentIdx/this.r);
    this.grid[gem.currentIdx] = null;
    this.dirtyColumns[col].push(gem);
    this.needUpdate = true;
};

GridController.prototype.addTileFlash = function(idx, colour, delay){
    "use strict";
    if(this.showTileFlash){
        this.tileFlashes[idx].timer = 0;
        this.tileFlashes[idx].active = true;
        this.tileFlashes[idx].colour = colour;
        this.tileFlashes[idx].alpha = 0;
        this.tileFlashes[idx].delay = delay || 0;
    }
};

GridController.prototype.handleGemSwapFinished = function(gem){
    "use strict";
    this.swappedGems.push(gem);
};

GridController.prototype.executePower = function(powerGem, chainId, otherGem){
    "use strict";
    switch(powerGem.powerType){
        case 0 : this.handleBomb(powerGem, chainId); break;
        case 1 : this.handleNova(powerGem, chainId); break;
        case 2 : this.handleHyperCube(powerGem, chainId, otherGem); break;
    }
};

GridController.prototype.handleHyperCube = function(powerGem, chainId, otherGem){
    "use strict";
    var i, gemList, noOfGems, gem, that, choose, cube, maxD, animDuration, chooseAndDestroyGems, delay, timeBonus, self;
    self = this;
    timeBonus = this.updateTimeBonus(timeBonus, otherGem);
    //find same colour gems
    gemList = [];
    if(chainId === undefined){
        chainId = this.nextChainId;
        this.nextChainId += 1;
        this.chainingCounter[chainId] = 0;
    }
    for(i=0; i<this.tiles; i+=1){
        gem = this.grid[i];
        if(gem !== otherGem){
            if(gem !== null && gem.powerType !== 2 && gem.colour === powerGem.colour && gem.playState !== Gem.PlayState.EXPLODING && gem.playState !== Gem.PlayState.REMOVING){
                gemList.push(gem);
            }
        }
    }
    noOfGems = gemList.length + 2;
    //function to select gems in random order from gem list
    that = this;
    choose = function(number){
        var res, j, idx;
        res = [];
        for(j=0; j<number; j+=1){
            if(gemList.length === 0){
                break;
            }
            idx = that.random.iRandom(gemList.length-1);
            res.push(gemList.splice(idx, 1)[0]);
        }
        return res;
    };

    //set-up cube animation
    cube = {};
    cube.gem = powerGem;
    cube.animations = [];
    maxD = 0;
    animDuration = this.animationController.totLightningDuration;

    //function to generate lightning rays from parent gem to child gems
    chooseAndDestroyGems = function(parent, parentDelay){
        var count, children, idx, d, g;
        count = that.random.iRandomRange(0, 2);
        children = choose(count);
        for(idx = 0; idx<children.length; idx += 1){
            d = parentDelay + that.random.iRandomRange(that.settings.hyperMinDelayDiff, that.settings.hyperMaxDelayDiff);
            g = children[idx];
            cube.animations.push({startX: parent.x, startY: parent.y, endX: g.x, endY: g.y, timer: 0, delay: d});
            that.chainingGrid[idx] = chainId;
            timeBonus = that.updateTimeBonus(timeBonus, g);
            if(g.powerType !== undefined){
                tbGame.setTokenDepth(g, 100);
                that.powerQueue.push({gem: g, delay:d + animDuration + that.settings.hyperMaxDelayDiff, chainId: chainId, otherGem: powerGem});
            }else{
                tbGame.setTokenDepth(g, 100);
                g.remove(d + animDuration + that.settings.hyperMaxDelayDiff, false, true);
                that.addTileFlash(g.currentIdx, g.colour, d + animDuration + that.settings.hyperMaxDelayDiff);
            }
            if(d > maxD){
                maxD = d;
            }
            chooseAndDestroyGems(g, d);
        }
    };

    //shoot lightning to "swapped" gem
    if(otherGem !== undefined){
        delay = this.settings.hyperMinDelay;
        cube.animations.push({startX: powerGem.x, startY: powerGem.y, endX: otherGem.x, endY: otherGem.y, timer: 0, delay: delay});
        if(otherGem.powerType !== undefined){
            this.powerQueue.push({gem: otherGem, delay:delay + animDuration + this.settings.hyperMaxDelayDiff, chainId: chainId, otherGem: powerGem});
        }else{
            tbGame.setTokenDepth(otherGem, 100);
            otherGem.remove(delay + animDuration + this.settings.hyperMaxDelayDiff, false, true);
        }
    }
    //shoot lightning to all other gems
    i = 0;
    while(gemList.length > 0){
        chooseAndDestroyGems(powerGem, this.settings.hyperMinDelay*i);
        i += 1;
    }
    powerGem.remove(maxD + animDuration);
    cube.totDuration = maxD + animDuration;
    
    this.activeBoosts[2] -= 1;
    this.animationController.startCubeAnimation(cube, function(){TG.level.handleBoost(powerGem.x, powerGem.y, noOfGems, self.chainingCounter[chainId], timeBonus);});
    //this.controller.handleBoost(powerGem.x, powerGem.y, noOfGems, this.chainingCounter[chainId], timeBonus);
};

GridController.prototype.handleHyperCubeCombo = function(swappedGem, otherGem){
    "use strict";
    var gemLists, i, r, c, gem, noOfGems, cube, animDuration, maxDelay, list, j, delay, gem1, gem2, timeBonus, self;
    self = this;
    //collect all gems
    gemLists = [];
    for(i = 0; i < 5; i += 1){
        gemLists[i] = [];
    }
    noOfGems = 2;
    for(r = this.r - 1; r >= 0; r -= 1){
        for(c = this.c - 1; c >= 0; c -= 1){
            gem = this.grid[c * this.r + r];
            if(gem !== null && gem !== swappedGem && gem !== otherGem){
                if(gem.playState !== Gem.PlayState.EXPLODING && gem.playState !== Gem.PlayState.REMOVING && gem.playState !== Gem.PlayState.MOVING){
                    i = this.random.iRandom(gemLists.length - 1);
                    gemLists[i].push(gem);
                    noOfGems += 1;
                }
            }
        }
    }

    //set-up cube animation
    cube = {};
    cube.gem = swappedGem;
    cube.animations = [];
    animDuration = this.animationController.totLightningDuration;
    maxDelay = 0;
    for(i = 0; i < gemLists.length; i += 1){
        list = gemLists[i];
        gem1 = (this.random.random() > 0.5)? swappedGem : otherGem;
        delay = 0;
        for(j = 0; j < list.length; j += 1){
            gem2 = list[j];
            delay += this.random.iRandomRange(this.settings.hyperMinDelayDiff, this.settings.hyperMaxDelayDiff);
            cube.animations.push({startX: gem1.x, startY: gem1.y, endX: gem2.x, endY: gem2.y, timer: 0, delay: delay});
            if(j > 0){
                timeBonus = this.updateTimeBonus(timeBonus, gem1);
                if(gem1.powerType !== undefined){
                    tbGame.setTokenDepth(gem1, 100);
                    this.powerQueue.push({gem: gem1, delay:delay + animDuration, depth: 1, otherGem: swappedGem});
                }else{
                    tbGame.setTokenDepth(gem1, 100);
                    gem1.remove(delay + animDuration);
                    this.addTileFlash(gem1.currentIdx, gem1.colour, delay + animDuration);
                }
            }
            gem1 = gem2;
        }
        //remove 'leave' gems
        timeBonus = this.updateTimeBonus(timeBonus, gem2);
        if(gem2.powerType !== undefined){
            tbGame.setTokenDepth(gem2, 100);
            this.powerQueue.push({gem: gem2, delay:delay + animDuration, depth: 1, otherGem: swappedGem});
        }else{
            tbGame.setTokenDepth(gem2, 100);
            gem2.remove(delay + animDuration);
        }
        maxDelay = Math.max(delay, maxDelay);
    }
    // remove hypercubes
    swappedGem.remove(this.settings.hyperMaxDelayDiff + animDuration);
    otherGem.remove(this.settings.hyperMaxDelayDiff + animDuration);
    // start animation
    cube.totDuration = maxDelay + animDuration;
    
    // update grid en game
    this.activeBoosts[2] -= 2;
    this.resetChains();
    this.animationController.startCubeAnimation(cube, function(){TG.level.handleBoost(swappedGem.x, swappedGem.y, noOfGems, 0, timeBonus);});
    //this.controller.handleBoost(swappedGem.x, swappedGem.y, noOfGems, 0, timeBonus);
};

GridController.prototype.handleNova = function(gem, chainId){
	"use strict";
	var c, r, max, i, delay, noOfGems, timeBonus, self, handleNeighbour;
	self = this;
	c = Math.floor(gem.currentIdx / this.r);
	r = gem.currentIdx % this.r;
	max = Math.max(Math.max(c, this.c - c - 1), Math.max(r, this.r - r - 1));
	timeBonus = this.updateTimeBonus(timeBonus, gem);
	gem.remove(this.animationController.settings.novaTotDuration);
	this.chainingGrid[gem.currentIdx] = chainId;
	noOfGems = 1;
	handleNeighbour = function(idx){
		var g;
		g = self.grid[idx];
		if(g !== null){
			self.chainingGrid[idx] = chainId;
			timeBonus = self.updateTimeBonus(timeBonus, g);
			if(g.powerType !== undefined){
				self.powerQueue.push({
				    gem: g,
				    delay: delay,
				    chainId: chainId,
				    otherGem: gem
				});
				self.addTileFlash(idx, g.colour, delay);
			}else{
				g.remove(delay);
				self.addTileFlash(idx, g.colour, delay);
				noOfGems += 1;
			}
		}
	};
	for(i = 1; i <= max; i += 1){
		delay = this.settings.novaMinDelay + this.settings.bombExplodeDelay * i;
		if(i <= r){ // up
			handleNeighbour((r - i) + c * this.r);
		}
		if(i < this.r - r){ // down
			handleNeighbour((r + i) + c * this.r);
		}
		if(i <= c){ // left
			handleNeighbour(r + (c - i) * this.r);
		}
		if(i < this.c - c){ // right
			handleNeighbour(r + (c + i) * this.r);
		}
	}
	this.activeBoosts[1] -= 1;
	this.animationController.startNovaAnimation(gem, function(){
		TG.level.handleBoost(gem.x, gem.y, noOfGems, self.chainingCounter[chainId], timeBonus);
	});
};

GridController.prototype.handleBomb = function(gem, chainId){
    "use strict";
    var row, col, minROffset, maxROffset, minCOffset, maxCOffset, rOffset, cOffset, i, g, d, noOfGems, timeBonus;
    timeBonus = this.updateTimeBonus(timeBonus, gem);
    gem.explode();
    this.chainingGrid[gem.currentIdx] = chainId;
    tbAudio.play(a_bomb_used);
    noOfGems = 1;
    col = Math.floor(gem.currentIdx/this.r);
    row = gem.currentIdx % this.r;
    minCOffset = Math.max(-this.settings.bombRadius, -col);
    maxCOffset = Math.min(this.settings.bombRadius, (this.c - 1) - col);
    minROffset = Math.max(-this.settings.bombRadius, -row);
    maxROffset = Math.min(this.settings.bombRadius, (this.r - 1) - row);

    //locate and destroy neighbours
    for(cOffset = minCOffset; cOffset <= maxCOffset; cOffset += 1){
        for(rOffset = minROffset; rOffset <= maxROffset; rOffset += 1){
            i = gem.currentIdx + this.r*cOffset + rOffset;
            if(i !== gem.currentIdx){
                g = this.grid[i];
                d = this.settings.bombExplodeDelay * Math.max(Math.abs(cOffset), Math.abs(rOffset));
                if(g !== null){
                    timeBonus = this.updateTimeBonus(timeBonus, g);
                    this.chainingGrid[i] = chainId;
                    if(g.powerType !== undefined){
                        if(g.playState !== Gem.PlayState.EXPLODING && g.playState !== Gem.PlayState.REMOVING){
                            this.powerQueue.push({gem: g, delay:d, chainId: chainId, otherGem: gem});
                            this.addTileFlash(i, g.colour, d);
                        }
                    }else{
                        g.explode(d);
                        this.addTileFlash(i, g.colour, d);
                        noOfGems += 1;
                    }
                }
            }
        }
    }
    this.activeBoosts[0] -= 1;
    this.controller.handleBoost(gem.x, gem.y, noOfGems, this.chainingCounter[chainId], timeBonus);
};

GridController.prototype.updateTimeBonus = function(timeBonus, gem){
    "use strict";
    if(timeBonus === undefined){
        timeBonus = {bonus: 0, colour: undefined};
    }
    if(gem.timeBonus){
        timeBonus.bonus += gem.timeBonus;
        timeBonus.colour = gem.colour;
    }
    return timeBonus;
};

GridController.prototype.updateGems = function(){
    "use strict";
    var tempList, c, r, rowDelay, count, idx, gem, newIdx;
    tempList = [];
    for(c=0; c<this.c; c+=1){
        //check if new gems should be spawned
        if(this.dirtyColumns[c].length > 0){
            rowDelay = 0;
            count = 0;
            //apply gravity to remaining gems
            for(r=this.r-1; r>=0; r-=1){
                idx = c * this.r + r;
                gem = this.grid[idx];
                if(gem === null){
                    count+=1;
                }else if(gem.playState !== Gem.PlayState.NORMAL && gem.playState !== Gem.PlayState.FALLING){
                    count = 0;
                }else if(count > 0){
                    newIdx = idx+count;
                    this.grid[newIdx] = gem;
                    this.chainingGrid[idx] = this.chainingGrid[newIdx];
                    this.grid[idx] = null;
                    gem.fall(newIdx, rowDelay);
                    rowDelay += 1 + this.random.random(2 * (newIdx % this.r));
                }
            }
        }
    }
};

// The following event handlers describe the behavior of the token

// handleAdd is called when the token is (re-) added to the game.
GridController.prototype.handleAdd = function() {
	"use strict";
    this.animationController = new AnimationController(this);
    this.hintController = new HintController(this.controller.showHints);
    this.selectionController = new SelectionController(this);
};

// handleRemove is called when the token is removed from the game.
GridController.prototype.handleRemove = function() {
	"use strict";
    var c, i, gem;
    tbGame.removeToken(this.animationController);
    tbGame.removeToken(this.hintController);
    tbGame.removeToken(this.selectionController);

    // remove gems
    for(i = 0; i < this.tiles; i += 1){
        gem = this.grid[i];
        if(gem !== null && gem !== undefined){
            tbGame.removeToken(gem);
        }
    }
    for(c = 0; c < this.c; c += 1){
		for(i = 0; i < this.dirtyColumns[c].length; i += 1){
            gem = this.dirtyColumns[c][i];
            if(gem !== null && gem !== undefined){
                tbGame.removeToken(gem);
            }
        }
	}
    this.canvas.drawTokens = true;
};

// handleBeginStep is called each step before processing other events.
GridController.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	 var i = 0, tile, c, done;
    if(!this.paused){
        //update grid
        if(this.updateGrid){
            // check if the player is swapping gems
            if(this.dragging){
                this.checkForSwap();
            }
            // refill the grid if necessary
            /*if(this.needUpdate){
                this.needUpdate = false;
                this.updateGems();
            }*/
        }

        //update tile flashes
        if(this.showTileFlash){
            for(i=0; i<this.r*this.c; i++){
                tile = this.tileFlashes[i];
                if(tile.active){
                    if(tile.timer >= tile.delay){
                        tile.alpha = this.tileFlashEase(tile.timer - tile.delay, 0, 1, this.settings.tileFlashDuration);
                        if(tile.timer - tile.delay >= this.settings.tileFlashDuration){
                            tile.active = false;
                            tile.timer = 0;
                            tile.alpha = 0;
                            continue;
                        }
                    }
                    tile.timer += timePassed;
                }
            }
        }
    }

    //check if all boost should be activated
    if(this.activateAllBoostsFlag && this.ready(true)){
        this.activateAllBoosts();
    }

    //check if clear field animation should be played
    if(this.clearFieldAnimationPlaying){
        if(!this.clearFieldAnimationStarted && this.ready(true)){
            for(i=0; i<this.tiles; i++){
                if(this.grid[i]){
                    this.grid[i].spiralAway();
                }
            }
            this.clearFieldAnimationStarted = true;
            this.updateGrid = false;
        }
        //check if clear field animation is done
        if(this.clearFieldAnimationStarted){
            done = 0;
            for(c=0; c<this.c; c++){
                done += this.dirtyColumns[c].length;
            }
            if(done === this.tiles){
                this.clearFieldAnimationPlaying = false;
                //this.fillGrid(true);
                //this.updateGrid = true;
                //tbFloaters.removeFloater(this.controller.stageFloater);
            }
        }
        this.clearFieldTimer += timePassed;
    }
};

// handleEndStep is called each step after processing other events (before draw).
GridController.prototype.handleEndStep = function(timePassed) {
    "use strict";
    var gem, gem2, item, i, tempList, c, r, rowDelay, idx, t, randomDelay;

    if(this.updateGrid){
		// check for all gems that have finished falling whether or not
		// they create a match
		while(this.fallenGems.length > 0){
			gem = this.fallenGems.shift();
			this.checkAndHandleMatch(gem.currentIdx, true);
		}

        // check for all gems that have finished swapping (or swapping back)
		// whether or not they create a match
        for(i=0; i<this.swappedGems.length; i+=1){
            gem = this.swappedGems[i];
            this.checkAndHandleMatch(gem.currentIdx, false);
        }

        // check power queue to see if any powers have to be activated
        tempList = [];
        while(this.powerQueue.length > 0){
            item = this.powerQueue.shift();
            item.delay -= timePassed;
            if(item.delay < 0){
                if(item.gem.playState === Gem.PlayState.NORMAL){
                    this.executePower(item.gem, item.chainId, item.otherGem);
                }
            }else{
                tempList.push(item);
            }
        }
        this.powerQueue = tempList;

        // remove all gems that were part of a match
        while(this.matchedGems.length > 0){
            item = this.matchedGems.shift();
            gem = item.gem;
            if(item.special < 0){
                gem.remove(0, item.chained);
            }else{
                if(gem.currentIdx !== item.causeIdx){
                    gem.move(this.getXByIdx(item.causeIdx), this.getYByIdx(item.causeIdx));
                }
            }
        }

        // add powers
        while(this.gettingPowerQueue.length > 0){
            item = this.gettingPowerQueue.shift();
            item.gem.forcePower(item.power);
            this.activeBoosts[item.power] += 1;
        }

        // swap back all gems that finished swapping but did not create a match
        for(i=0; i<this.swappedGems.length; i+=1){
            gem = this.swappedGems[i];
            if(!gem.back && gem.playState === Gem.PlayState.NORMAL){
                gem2 = this.grid[gem.oldIdx];
                if(gem2.playState === Gem.PlayState.NORMAL){
                    this.swapGems(gem.currentIdx, gem.oldIdx, true);
                    this.controller.handleFalseMove(gem.x, gem.y);
                }
            }
        }
        this.swappedGems = [];

        //add new gems if required
        if(this.needUpdate){
            this.needUpdate = false;
            
            // refill the grid if necessary
            //if(this.needUpdate){
                //this.needUpdate = false;
                this.updateGems();
            //}

            tempList = [];
            for(c=0; c<this.c; c+=1) {
                //check if new gems should be spawned
                if(this.dirtyColumns[c].length > 0){
                    rowDelay = 0;
                    //spawn new gems
                    for(r=0; r<this.r; r++){
                        if(this.grid[c * this.r + r] === null){
                            idx = c * this.r + r;
                            gem = this.dirtyColumns[c].shift();
                            gem.init(this.getXByIdx(idx), -this.tileSize / 2, this.chooseGemColour());
							this.grid[idx] = gem;
							gem.fall(idx, rowDelay);
							randomDelay = (idx % this.r !== 0)? this.random.random(2 * (idx % this.r)) : 0;
							rowDelay += 1 + randomDelay;
                            tempList.push(idx);
                            if(this.controller.mode === 2 && this.timeBonusCount > 0){
                                for(i=0; i<this.timeBonusses.length; i++){
                                    if(this.timeBonusses[i] > 0){
                                        gem.timeBonus = this.timeBonusses[i];
                                        this.timeBonusses[i] = 0;
                                        this.timeBonusCount -= 1;
                                        break;
                                    }
                                }
                            }
                        }else{
                            break;
                        }
                    }

                    if(this.dirtyColumns[c].length > 0){
                        this.needUpdate = true;
                    }
                }
            }
        }

        if(this.controller.gameState === LevelController.GameState.PLAY && (this.match < 0 || this.isPossibleMatch(this.match) === -1)){
            this.match = this.findPossibleMatch();
            t = 0;
            while(t<100){
                if(this.match >= 0 || !this.ensureMove){
                    break;
                }
                for(i=0; i<tempList.length; i+=1){
                    this.grid[tempList[i]].colour = this.chooseGemColour();
                }
                this.match = this.findPossibleMatch();
                t+=1;
            }
            if(this.match < 0 && this.ready()){
                this.hintController.reset();
                this.controller.handleNoMoreMoves();
            }
        }
        if(this.match >= 0 && this.hintController.hint !== this.grid[this.match]){
            this.hintController.updateHint(this.grid[this.match]);
        }
    }
};

GridController.prototype.handleDraw = function(){
    "use strict";
    if(this.showTileFlash){
        var t, i = 0;
        for(i=0; i<this.tileFlashes.length; i++){
            t = this.tileFlashes[i];
            if(t.active && t.alpha > 0){
                s_tile_flash.drawAlpha(0, this.getXByIdx(i), this.getYByIdx(i), t.alpha);
                tbGraphics.context.globalCompositeOperation = "source-atop";
                tbGraphics.drawRectangle(this.getXByIdx(i) - this.tileSize/2, this.getYByIdx(i) - this.tileSize/2, this.tileSize, this.tileSize, this.tileFlashColour[t.colour], false);
                tbGraphics.context.globalCompositeOperation = "source-over";
            }
        }
    }
};
//******************************************************
//*****    Token Resource: Gem
//******************************************************

// ***************************************
// Definition of game token Gem
// ***************************************

// The constructor for the Gem game token.
function Gem(controller) {
	"use strict";
	// Properties used by the engine
	this.depth = 500;
	this.visible = false;
	this.active = false;
	TG.functions.setLayerAndCanvas(this, TG.gameLayer);
	// Your own properties
	this.settings = TG.settings.game.gem;
	this.controller = controller;
	//support both flashing and non-flashing gems
	if(typeof s_gems !== "undefined"){
		this.gemImage = s_gems;
	}
	if(typeof s_gem_blue !== "undefined"){
		this.gemShineImages = [s_gem_blue, s_gem_yellow, s_gem_green, s_gem_red, s_gem_purple, s_gem_orange, s_gem_white];
	}
	this.hyperCubeImages = s_hypercube;
    this.novaImages = s_nova_gem;
    this.bombImages = s_bomb_gem;
    this.bombExplosionImages = s_bomb_explosion;
    this.bombExplosionCenterImages = s_bomb_explosion_center;
    this.selectorImage = s_selection;
    this.sparkImage = s_bomb_explosion;
    this.powerAnimationSpeeds = [this.settings.bombAnimationSpeed, this.settings.novaAnimationSpeed, this.settings.hyperAnimationSpeed];
	this.powerAnimationNumbers = [this.bombImages.number, this.novaImages.number, this.hyperCubeImages.number];
    this.powerGetSounds=[a_bomb_creation, a_lightning_creation, a_superLightning_creation];
    this.random = new tbRandom();
    // Add it to the game engine
	tbGame.addToken(this);
}

Gem.PlayState = {
	NORMAL : 0,
	FALLING : 1,
	REMOVING : 2,
	EXPLODING : 3,
	SWAPPING : 4,
    GETTING_POWER : 5,
    MOVING : 6,
    SPIRAL : 7
};

Gem.prototype.init = function(x, y, colour) {
	"use strict";
	this.scale = 1;
	this.alpha = 1;
	this.imageIdx = 1;
	this.drawExplosion = false;
	this.x = x;
	this.y = y;
	this.colour = colour;
	this.playState = Gem.PlayState.NORMAL;
	this.currentIdx = -1;
    this.oldIdx = -1;
	this.visible = true;
	this.active = false;
    this.powerType = undefined;
    this.inMatch = false;
    this.spark = false;
    this.showSpark = false;
    this.timeBonus = 0;
    this.shining = false;
    this.shineIdx = 0;
    this.destroyed = false;
    tbGame.setTokenDepth(this, 500);
};

Gem.prototype.addPower = function(powerType){
    "use strict";
    this.powerType = powerType;
    this.active = true;
    this.visualTimer = 0;
    if(this.y > 0){
        this.playState = Gem.PlayState.GETTING_POWER;
        tbGame.setTokenDepth(this, this.depth - 100);
        this.playStateTimer = 0;
        this.imageIdx = 0;
        tbAudio.play(this.powerGetSounds[powerType]);
    }
};

Gem.prototype.forcePower = function(powerType){
    "use strict";
    this.scale = 1;
    this.addPower(powerType);
    this.inMatch = false;
};

Gem.prototype.remove = function(delay, easeIn, spark) {
	"use strict";
    switch(this.playState){
        case Gem.PlayState.FALLING:
        case Gem.PlayState.GETTING_POWER:
        case Gem.PlayState.NORMAL:
            this.active = true;
            this.playState = Gem.PlayState.REMOVING;
            this.playStateTimer = 0;
            this.delay = delay || 0;
            this.ease = (easeIn)? tbEase.easeInOvershoot : tbEase.easeLinear;
            this.duration = (easeIn)? this.settings.growShrinkDuration : this.settings.shrinkDuration;
            if(spark){
                this.spark = true;
                this.sparkImageIdx = 0;
            }
        break;
    }
};

Gem.prototype.spiralAway = function() {
    "use strict";
    this.playState = Gem.PlayState.SPIRAL;
    this.playStateTimer = 0;
    this.active = true;
    this.delay = this.random.iRandomRange(0, 200);

    this.spiralAngle = Math.atan2(this.x- TG.gameWidth/2, this.y- TG.gameHeight/2);
    this.spiralSpeed = 0;

    var dx = this.x- TG.gameWidth/2;
    var dy = this.y- TG.gameHeight/2;

    var distance = Math.sqrt(dx * dx + dy * dy);

    this.spiralRadius = distance;
    this.spiralStartRadius = distance;

};

Gem.prototype.forceRemove = function(){
    "use strict";
    switch(this.playState){
        case Gem.PlayState.SWAPPING:
            this.x = this.endX;
            this.y = this.endY;
        break;
        case Gem.PlayState.FALLING:
            this.y = this.endY;
        break;
        case Gem.PlayState.GETTING_POWER:
            tbGame.setTokenDepth(this, this.depth + 100);
        break;
    }
    this.visible = false;
    this.active = false;
    this.controller.handleGemDestroyFinished(this);
    this.destroyed = true;
};

Gem.prototype.explode = function(delay){
    "use strict";
    if(this.playState !== Gem.PlayState.EXPLODING){
        this.playState = Gem.PlayState.EXPLODING;
        this.playStateTimer = 0;
        this.delay = (delay !== undefined)? delay : 0;
        this.active = true;
        if(this.delay === 0){
			this.drawExplosion = true;
			this.imageIdx = 0;
		}
        //move gem to front for explosion effect
        if(this.powerType === 0){
			this.explosionImage = this.bombExplosionCenterImages;
			tbGame.setTokenDepth(this, 400);
		}else{
			this.explosionImage = this.bombExplosionImages;
		}
    }
};

Gem.prototype.move = function(goalX, goalY){
    "use strict";
	if(this.playState === Gem.PlayState.NORMAL){
        this.active = true;
        this.startX = this.x;
        this.endX = goalX;
        this.startY = this.y;
        this.endY = goalY;
		this.playState = Gem.PlayState.MOVING;
		this.playStateTimer = 0;
	}
};

Gem.prototype.fall = function(idx, delay) {
	"use strict";
	if(this.playState !== Gem.PlayState.NORMAL && this.playState !== Gem.PlayState.FALLING){
        return;
    }
    this.oldIdx = this.currentIdx;
    this.currentIdx = idx;
    this.endY = this.controller.getYByIdx(idx);
	if(this.playState !== Gem.PlayState.FALLING){
		this.active = true;
		this.playState = Gem.PlayState.FALLING;
		this.fallDuration = this.settings.fallDuration;
		this.fallDelay = 0;
		if(delay !== undefined){
			this.fallDelay = delay;
        }
		this.playStateTimer = 0;
		this.startY = this.y;
		this.visible = true;
	}
	else{
		this.fallDuration += this.settings.fallDuration;
	}
	if(isNaN(delay)){
		console.log("nan detected");
		return true;
	}
};

Gem.prototype.swap = function(idx, back) {
	"use strict";
	if(this.playState !== Gem.PlayState.NORMAL){
        return;
    }
    this.oldIdx = this.currentIdx;
    this.currentIdx = idx;
    this.startY = this.y;
    this.endY = this.controller.getYByIdx(idx);
    this.startX = this.x;
    this.endX = this.controller.getXByIdx(idx);

	this.playState = Gem.PlayState.SWAPPING;
	this.playStateTimer = 0;
    this.active = true;
    this.back = back;
};

Gem.prototype.showShineAnimation = function(delay){
	"use strict";
	if(this.gemShineImages !== undefined){
		this.shining = true;
		this.shineTimer = 0;
		this.active = true;
		this.shineDelay = delay || 0;
		this.shineTimer = 0;
	}
};

// The following event handlers describe the behavior of the token

// handleBeginStep is called each step before processing other events.
Gem.prototype.handleBeginStep = function(timePassed) {
	"use strict";
	switch(this.playState){
        case Gem.PlayState.NORMAL :
            if(this.powerType === undefined && !this.shining){
                this.active = false;
            }
        break;
		case Gem.PlayState.FALLING :
			if(this.playStateTimer < this.fallDelay){
				this.playStateTimer += timePassed;
            }
			else if(this.playStateTimer < this.fallDuration + this.fallDelay){
				this.playStateTimer += timePassed;
				this.y = tbEase.easeOutBounce(this.playStateTimer - this.fallDelay, this.startY, this.endY-this.startY, this.fallDuration);
				if(this.playStateTimer >= this.fallDuration + this.fallDelay){
					this.y = this.endY;
					this.playState = Gem.PlayState.NORMAL;
                    this.controller.handleGemFallFinished(this);
				}
			}
        break;
		case Gem.PlayState.REMOVING :
			if(this.playStateTimer < this.delay){
                this.playStateTimer += timePassed;
                if(this.spark && this.playStateTimer >= this.delay){
                    this.showSpark = true;
                }
			}
            else if(this.playStateTimer < this.duration  + this.delay){
				this.playStateTimer += timePassed;
                this.scale = this.ease(this.playStateTimer - this.delay, 1, -1, this.duration);
                this.sparkImageIdx += this.settings.sparkFPS * timePassed/1000;
                if(this.playStateTimer >= this.duration + this.delay){
					this.visible = false;
					this.active = false;
                    this.controller.handleGemDestroyFinished(this);
                    this.destroyed = true;
                    this.showSpark = false;
				}
			}
        break;
		case Gem.PlayState.EXPLODING:
            if(this.playStateTimer < this.delay){
				this.playStateTimer += timePassed;
				if(this.playStateTimer >= this.delay){
					this.playStateTimer = this.delay;
					this.drawExplosion = true;
					this.imageIdx = 0;
				}
			}else{
				this.imageIdx += 20 * timePassed / 1000;
				if(Math.round(this.imageIdx) >= this.explosionImage.number){
					this.active = false;
					this.visible = false;
					if(this.depth !== 500){
						tbGame.setTokenDepth(this, 500);
					}
					this.controller.handleGemDestroyFinished(this);
					this.destroyed = true;
				}
			}
        break;
		case Gem.PlayState.SWAPPING :
            if(this.playStateTimer < this.settings.swapDuration){
				this.playStateTimer += timePassed;
				if(this.endX !== this.startX){
                    this.x = tbEase.easeInQuadratic(this.playStateTimer, this.startX, this.endX-this.startX, this.settings.swapDuration);
                }
                if(this.endY !== this.startY){
                    this.y = tbEase.easeInQuadratic(this.playStateTimer, this.startY, this.endY-this.startY, this.settings.swapDuration);
                }
                if(this.playStateTimer >= this.settings.swapDuration){
					this.x = this.endX;
                    this.y = this.endY;
					this.playState = Gem.PlayState.NORMAL;
                    this.controller.handleGemSwapFinished(this);
				}
			}
        break;
        case Gem.PlayState.GETTING_POWER :
            if(this.playStateTimer < this.settings.gettingPowerAnimDuration){
                this.playStateTimer += timePassed;
                if(this.playStateTimer >= this.settings.gettingPowerAnimDuration){
                    this.playState = Gem.PlayState.NORMAL;
                    tbGame.setTokenDepth(this, this.depth + 100);
                }
            }
        break;
        case Gem.PlayState.MOVING :
            if(this.playStateTimer < this.settings.gettingPowerAnimDuration){
                this.playStateTimer += timePassed;
                this.x = tbEase.easeInQuadratic(this.playStateTimer, this.startX, this.endX-this.startX, this.settings.gettingPowerAnimDuration);
                this.y = tbEase.easeInQuadratic(this.playStateTimer, this.startY, this.endY-this.startY, this.settings.gettingPowerAnimDuration);
                if(this.playStateTimer >= this.settings.gettingPowerAnimDuration){
                    this.visible = false;
					this.active = false;
                    this.controller.handleGemDestroyFinished(this);
                    this.destroyed = true;
                }
            }
        break;
        case Gem.PlayState.SPIRAL :
            if(this.playStateTimer < this.delay){
                this.playStateTimer += timePassed;
            }else if(this.playStateTimer < this.delay + this.settings.spiralPopUpDuration){
                this.scale = tbEase.easeOutOvershoot(this.playStateTimer - this.delay, 1, this.settings.spiralPopUpScaleFactor - 1, this.settings.spiralPopUpDuration);
                this.playStateTimer += timePassed;
                if(this.playStateTimer >= this.delay + this.settings.spiralPopUpDuration){
                    this.scale = this.settings.spiralPopUpScaleFactor;
                }
            }else{
                if(this.spiralRadius > 10){

                    this.spiralSpeed += timePassed * 0.02 / this.spiralRadius;
                    this.spiralAngle += this.spiralSpeed * 3;
                    this.spiralRadius -= this.spiralSpeed * tgScale(120);

                    this.x = TG.gameWidth/2 + this.spiralRadius*Math.sin(this.spiralAngle);
                    this.y = TG.gameHeight/2 + this.spiralRadius*Math.cos(this.spiralAngle);

                    this.scale = this.settings.spiralPopUpScaleFactor*this.spiralRadius / this.spiralStartRadius;
                }
                else {
                    this.visible = false;
                    this.active = false;
                    this.controller.handleGemDestroyFinished(this);
                    this.destroyed = true;
                }
            }
        break;
	}
    //update power visuals
    if(this.powerType !== undefined && this.playState !== Gem.PlayState.EXPLODING){
        this.visualTimer = (this.visualTimer + timePassed)%(this.powerAnimationSpeeds[this.powerType]*this.powerAnimationNumbers[this.powerType]);
        this.imageIdx = Math.floor(this.visualTimer/this.powerAnimationSpeeds[this.powerType]);
    }
    if(this.shining){
		if(this.shineTimer >= this.shineDelay){
			this.shineIdx += 20 * timePassed / 1000;
			if(this.shineIdx >= this.gemShineImages[0].number){
				this.shineIdx = 0;
				this.shining = false;
			}
		}else{
			this.shineTimer += timePassed;
		}
	}
};

// handleDraw is called at the end of a step to draw the token.
Gem.prototype.handleDraw = function() {
	"use strict";
	if(this.showSpark){
        this.sparkImage.drawSimple(this.sparkImageIdx, this.x, this.y);
    }
	if(this.playState === Gem.PlayState.EXPLODING && this.drawExplosion){
		this.explosionImage.drawSimple(this.imageIdx, this.x, this.y);
    }else if(this.powerType === 2){
        this.hyperCubeImages.drawExt(this.imageIdx, this.x, this.y, this.scale, this.scale, 0, this.alpha);
    }
    else{
        if(this.gemShineImages){
			this.gemShineImages[this.colour].drawExt(this.shineIdx, this.x, this.y, this.scale, this.scale, 0, this.alpha);
		}else{
			this.gemImage.drawExt(this.colour, this.x, this.y, this.scale, this.scale, 0, this.alpha);
		}
        if(this.powerType === 0){
            this.bombImages.drawExt(this.imageIdx, this.x, this.y, this.scale, this.scale, 0, this.alpha);
        }else if(this.powerType === 1){
            this.novaImages.drawExt(this.imageIdx, this.x, this.y, this.scale, this.scale, 0, this.alpha);
        }
    }
	if(this.timeBonus > 0){
	    f_timeBonus.setFillColor(TG.settings.gameTheme.gem.timeBonusColours[this.colour]);
	    f_timeBonus.drawSimple("+" + Math.round(this.timeBonus/1000), this.x + this.settings.timeBonusOffsetX, this.y + this.settings.timeBonusOffsetY);
    }
};


//******************************************************
//*****    Token Resource: AnimationController
//******************************************************

// ***************************************
// Definition of game token AnimationController
// ***************************************

// The constructor for the AnimationController game token.
function AnimationController(parentController) {
	"use strict";
	// Properties used by the engine
	this.depth = 200;
	this.visible = false;
	this.active = false;
	//this.group = "gameControllers";
	TG.functions.setLayerAndCanvas(this, TG.gameLayer);
	// Your own properties
	this.settings = TG.settings.game.animationController;
    this.paused = false;
    this.controller = parentController;
    this.novaImages = [s_nova_lightning_01, s_nova_lightning_02, s_nova_lightning_03];
    this.cubeImage = s_hypercube_lightning;
    this.novaSound = a_lightning_used;
    this.cubeSound = a_superLightning_used;
    this.activeNovas = [];
    this.activeCubes = [];
    this.totLightningDuration = this.settings.lightningScaleDuration + this.settings.lightningDuration;
    this.totCount = 0;
    this.fadingIn = false;
    this.fadingOut = false;
    this.fadeTimer = 0;
    this.alpha = 0;
    // Add it to the game engine
	tbGame.addToken(this);
}

AnimationController.prototype.stopAllAnimations = function() {
	"use strict";
	this.activeNovas = [];
    this.activeCubes = [];
    this.totCount = 0;
    this.fadingIn = false;
    this.fadingOut = false;
    this.fadeTimer = 0;
    this.alpha = 0;
    this.active = false;
    this.visible = false;
    TG.level.setAllowUserInteraction(true);
    this.controller.updateGrid = true;
    tbGame.setTokenDepth(this.controller, 800);
};

AnimationController.prototype.startNovaAnimation = function(gem, callback){
    "use strict";
    this._startAnimation();
    this.activeNovas.push({timer: 0, imageIdx: 0, scale: 0, x: gem.x, y: gem.y - this.controller.tileSize/2, gem: gem, callback: callback});
    tbAudio.play(this.novaSound);
    
    return this.settings.novaTotDuration;
};

AnimationController.prototype.startCubeAnimation = function(cube, callback){
    "use strict";
    this._startAnimation();
    tbAudio.play(this.cubeSound);
    cube.callback = callback;
    this.activeCubes.push(cube);
};

AnimationController.prototype._startAnimation = function(){
    "use strict";
    if(this.totCount === 0){
        TG.level.setAllowUserInteraction(false);
        this.controller.updateGrid = false;
        this.visible = true;
        this.active = true;
        this.fadingIn = true;
        this.fadingOut = false;
    }
    this.totCount += 1;
};

// The following event handlers describe the behavior of the token
// handleBeginStep is called each step before processing other events.
AnimationController.prototype.handleBeginStep = function(timePassed) {
	"use strict";
    if(!this.paused){
        var i, animation, count, cube, maxTimer;
        // update nova animations
        count = this.activeNovas.length;
        while(count > 0){
            animation = this.activeNovas.shift();
            animation.imageIdx = (Math.floor(animation.timer/this.settings.novaAnimationSpeed))%this.novaImages.length;
            if(animation.timer < this.settings.novaScaleDuration){
                animation.timer += timePassed;
                animation.scale = animation.timer/this.settings.novaScaleDuration;
                if(animation.timer >= this.settings.novaScaleDuration){
                    animation.scale = 1;
                }
                this.activeNovas.push(animation);
            }else if(animation.timer < this.settings.novaTotDuration){
                animation.timer += timePassed;
                this.activeNovas.push(animation);
            }else{
                this.totCount -= 1;
                if(typeof animation.callback === "function"){
					animation.callback();
				}
            }
            count -= 1;
        }

        // update cube animations
        count = this.activeCubes.length;
        while(count > 0){
            cube = this.activeCubes.shift();
            for(i = 0; i < cube.animations.length; i += 1){
                animation = cube.animations[i];
                if(animation.timer < animation.delay){
                    animation.timer += timePassed;
                    if(animation.timer >= animation.delay){
                        animation.imageIdx = 0;
                        animation.scale = 0;
                        animation.endScale = Math.sqrt(Math.pow(animation.endX - animation.startX, 2) + Math.pow(animation.startY - animation.endY, 2))/this.cubeImage.width;
                        animation.angle = tbMath.radToDeg(Math.atan2(animation.startY - animation.endY, animation.endX - animation.startX));
                    }
                }else if(animation.timer < animation.delay + this.settings.lightningScaleDuration){
                    animation.timer += timePassed;
                    animation.imageIdx = (animation.timer/this.settings.cubeAnimationSpeed);

                    animation.scale = animation.endScale * ((animation.timer - animation.delay)/this.settings.lightningScaleDuration);
                    if(animation.timer >= animation.delay + this.settings.lightningScaleDuration){
                        animation.scale = animation.endScale;
                    }
                }else if(animation.timer < animation.delay + this.totLightningDuration){
                    animation.timer += timePassed;
                    animation.imageIdx = (animation.timer/this.settings.cubeAnimationSpeed);
                    if(animation.timer >= animation.delay + this.totLightningDuration){
                        animation.scale = 0;
                    }
                }
                if(maxTimer === undefined || animation.timer > maxTimer){
                    maxTimer = animation.timer;
                }
            }
            if(maxTimer < cube.totDuration){
                this.activeCubes.push(cube);
            }else{
                this.totCount -= 1;
                if(typeof cube.callback === "function"){
					cube.callback();
				}
            }
            count -= 1;
        }

        //update alpha
        if(this.fadingIn){
            this.fadeTimer += timePassed;
            this.alpha = 0.8 * (this.fadeTimer/this.settings.fadeDuration);
            if(this.fadeTimer >= this.settings.fadeDuration){
                this.fadingIn = false;
                this.alpha = 0.8;
                this.fadeTimer = this.settings.fadeDuration;
            }
        }else if(this.fadingOut){
            this.fadeTimer -= timePassed;
            this.alpha = 0.8 * (this.fadeTimer/this.settings.fadeDuration);
            if(this.fadeTimer <= 0){
                this.fadingOut = false;
                this.alpha = 0;
                this.fadeTimer = 0;
                this.active = false;
                this.visible = false;
            }
        }

        // if no more animations are playing, reactivate game
        if(this.totCount === 0){
            TG.level.setAllowUserInteraction(true);
            this.controller.updateGrid = true;
            tbGame.setTokenDepth(this.controller, 800);
            this.fadingOut = true;
        }
    }
};

// handleDraw is called at the end of a step to draw the token.
AnimationController.prototype.handleDraw = function() {
	"use strict";
	var ctx, i, animation, height, widthL, widthR, lengthT, lengthB, cube, j;
    if(this.alpha > 0){
        ctx = tbGraphics.context;
		ctx.fillStyle="rgba(30,34,56,"+this.alpha+")";
		ctx.fillRect(0,0,tbGraphics.width,tbGraphics.height);
    }

    // draw nova animations
    for(i = 0; i<this.activeNovas.length; i += 1){
        animation = this.activeNovas[i];
        animation.gem.handleDraw();
        height = this.novaImages[animation.imageIdx].height;
        widthL =  animation.x * animation.scale;
        widthR = (TG.gameWidth - animation.x) * animation.scale;
        lengthT = animation.y * animation.scale;
        lengthB = (TG.gameHeight - animation.y) * animation.scale;
        this.novaImages[animation.imageIdx].drawPart(0, animation.x - widthL, 0,
            widthL + widthR, height, animation.x - widthL, animation.y, 1);
        this.novaImages[animation.imageIdx].drawPartExt(0, animation.y - lengthT, 0,
            lengthT + lengthB, height, animation.x + height/2, animation.y - lengthT, 1, 1, -90, 1);
    }

    // draw cube animations
    for(i = 0; i < this.activeCubes.length; i += 1){
        cube = this.activeCubes[i];
        cube.gem.handleDraw();
        for(j = 0; j < cube.animations.length; j += 1){
            animation = cube.animations[j];
            if(animation.scale !== undefined && animation.scale > 0){
                this.cubeImage.drawExt(animation.imageIdx, animation.startX, animation.startY, animation.scale, 1, animation.angle, 1);
            }
        }
    }
};


//******************************************************
//*****    Token Resource: HintController
//******************************************************

// ***************************************
// Definition of game token HintController
// ***************************************

// The constructor for the HintController game token.
function HintController(showHints) {
    "use strict";
    // Properties used by the engine
    this.depth = 400;
    this.active = false;
    this.visible = false;
    TG.functions.setLayerAndCanvas(this, TG.gameLayer);
    // Your own properties
    this.settings = TG.settings.game.hintController;
    this.image = s_hint;
    this.animTimer = 0;
    this.imageIdx = 0;
    this.reset();
    // Add it to the game engine
    //this.setActivate(showHints);
    if(showHints){
        this.active = true;
        tbGame.addToken(this);
        tbGame.addTokenTags(this, "game");
    }
}

HintController.prototype.setActivate = function(active) {
    "use strict";
    this.active = active;
    this.visible = (active && this.show);
};

HintController.prototype.reset = function() {
    "use strict";
    this.hint = undefined;
    this.visible = false;
    this.show = false;
    this.hintTimer = 0;
};

HintController.prototype.updateHint = function(hintGem) {
    "use strict";
    this.visible = false;
    this.show = false;
    this.hintTimer = 0;
    this.hint = hintGem;
};

// The following event handlers describe the behavior of the token

// handleBeginStep is called each step before processing other events.
HintController.prototype.handleBeginStep = function(timePassed) {
    "use strict";
    if(this.hint !== undefined){
        this.hintTimer += timePassed;
        if(!this.visible){
            if(!this.show && this.hintTimer >= this.settings.timeBeforeHint[TG.level.mode]){
                this.hintTimer = 0;
                this.animTimer = 0;
                this.visible = true;
                this.show = true;
                this.imageIdx = 0;
            }
            else if(this.show && this.hintTimer >= this.settings.timeBetweenHint){
                this.hintTimer = 0;
                this.animTimer = 0;
                this.visible = true;
                this.imageIdx = 0;
            }
        }
        else{
            if(this.hintTimer >= this.settings.hintDuration){
                this.hintTimer = 0;
                this.visible = false;
            }
            else{
                this.animTimer += timePassed;
                this.imageIdx += this.settings.hintFramesPerSeconds * timePassed/1000;
            }
        }
    }
};

// handleDraw is called at the end of a step to draw the token.
HintController.prototype.handleDraw = function() {
    "use strict";
    if(this.hint.y > 0){
        this.image.drawSimple(this.imageIdx, this.hint.x, this.hint.y);
    }
};


//******************************************************
//*****    Token Resource: SelectionController
//******************************************************

// ***************************************
// Definition of game token SelectionController
// ***************************************

// The constructor for the SelectionController game token.
var SelectionController = function(controller) {
	"use strict";
	// Properties used by the engine
	this.depth = 400;
	this.visible = false;
	this.active = false;
	TG.functions.setLayerAndCanvas(this, TG.gameLayer);
	// Your own properties
    this.controller = controller;
    this.selectedGem = null;
	this.setActivate(true);
};

SelectionController.prototype.setActivate = function(active) {
    "use strict";
    if(active && !this.active){
        this.active = true;
        this.visible = true;
        tbGame.addToken(this);
        tbGame.addTokenTags(this, "game");
    }else if(!active && this.active){
        this.active = false;
        this.visible = false;
        tbGame.removeToken(this);
    }
};


SelectionController.prototype.initLevel = function(){
    "use strict";
    this.selectedGem = null;
};

SelectionController.prototype.setSelection = function(gem){
	"use strict";
    this.selectedGem = gem;
};

// The following event handlers describe the behaviour of the token

// handleEndStep is called each step after processing other events (before draw).
SelectionController.prototype.handleEndStep = function(timePassed) {
	"use strict";
    if(this.selectedGem !== null && this.selectedGem.playState !== Gem.PlayState.NORMAL && this.selectedGem.playState !== Gem.PlayState.FALLING){
        this.selectedGem = null;
    }
};

// handleDraw is called at the end of a step to draw the token.
SelectionController.prototype.handleDraw = function() {
	"use strict";
	if(this.selectedGem !== null){
        s_selection.drawSimple(0, this.selectedGem.x, this.selectedGem.y);
    }
};


//******************************************************
//*****    Token Resource: GameController
//******************************************************

// ***************************************
// Definition of game token GameController
// ***************************************

// The constructor for the GameController game token.
/* exported GameController */
var GameController = function() {
    "use strict";
    // Properties used by the engine
    this.depth = 0;
    this.visible = false;
    this.active = false;
    TG.functions.setLayerAndCanvas(this, TG.frameworkBackgroundLayer);
    // Your own properties

    // Add it to the game engine
    tbGame.addToken(this);
};

GameController.prototype.getTutorialData = function(idx, difficulty){
    "use strict";
    var data, images, i, txt, header;
    data = [];
    images = [s_tutorial_01, s_tutorial_02, s_tutorial_03, s_tutorial_04, s_tutorial_05];
	if(typeof s_tutorial_06 !== "undefined"){
		images.push(s_tutorial_06);
	}
    for(i = 0; i < images.length; i += 1){
        txt = TG.dictionary.getText("TutorialText_"+i, "<TUTORIAL_TEXT_"+i+">");
        header = TG.dictionary.getText("TutorialTitle_"+i, "");
        data.push({image: images[i], text: txt, title: header });
    }
    return data;
};

GameController.prototype.getLevelImage = function(worldIdx, levelIdx, state){
    "use strict";
    return null;
};


//******************************************************
//*****    Script Resource: theme_version
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.version = TG.version || {};
TG.version["theme"] = "1.1";
//******************************************************
//*****    Script Resource: game_ui_settings
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

/* global
	tgScale,
	f_game_ui,
	s_ui_background_blank,
	f_game_ui_tiny,
	s_ui_heart,
	s_ui_smiley_medium,
	s_ui_timeleft,
	s_ui_highscore,
	s_ui_divider */

/* exported GameUISettings */
var GameUISettings = GameUISettings || {};

var GameUISettingsOffsets = {X: tgScale(14), Y: tgScale(48)};

GameUISettings.Difficulty = {
	background: {
		image: s_ui_background_bejeweled,
		image_x: tgScale(0),
		image_y: tgScale(30),
		elements: [
		{
			textKey: "game_ui_time_left",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(52) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#d47171",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(124) + GameUISettingsOffsets.Y
		}, {
			textKey: "game_ui_SCORE",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(140) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#d47171",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(200) + GameUISettingsOffsets.Y
		}, {
			textKey: "game_ui_HIGHSCORE",
			x: tgScale(6) + GameUISettingsOffsets.X,
			y: tgScale(258) + GameUISettingsOffsets.Y,
			w: tgScale(100),
			h: tgScale(20),
			textYOffsetFactor: 0.2,
			font: f_game_ui,
			font_properties: {
				fillColor: "#d47171",
				fontSize: tgScale(20),
				fontCase: "lower",
				align: "center",
				baseline: "top"
			}
		}, {
			image: s_ui_divider,
			x: tgScale(9, "round") + GameUISettingsOffsets.X,
			y: tgScale(318) + GameUISettingsOffsets.Y
		}]
	},
	pauseButton: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(520) - tgScale(80) + GameUISettingsOffsets.Y
	},
	time: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(80) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(38),
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		/**
		 * @type tbVectorFont
		 */
		font: f_game_ui,
		font_properties: {
			fontSize: tgScale(38),
			fillColor: "#ff6c00",
			align: "center",
			baseline: "top"
		}
	},
	score: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(166) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(24),
		timeBetweenValueUpdates: 50, // ms
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		/**
		 * @type tbVectorFont
		 */
		font: f_game_ui,
		font_properties: {
			fontSize: tgScale(24),
			fillColor: "#ff6c00",
			align: "center",
			baseline: "top"
		}
	},
	highScoreImage: {
		x: tgScale(43, "round") + GameUISettingsOffsets.X,
		y: tgScale(212) + GameUISettingsOffsets.Y,
		image: s_ui_highscore,
		center: false,
		subtractOrigin: true
	},
	highScore: {
		x: tgScale(6) + GameUISettingsOffsets.X,
		y: tgScale(284) + GameUISettingsOffsets.Y,
		w: tgScale(100),
		h: tgScale(20),
		textYOffsetFactor: 0.2,
		useCaption: false,
		separator: "",
		font: f_game_ui,
		font_properties: {
			fillColor: "#c83939",
			fontSize: tgScale(24),
			align: "center",
			baseline: "top"
		}
	}
};

/* exported gameUISettingsVar */
var gameUISettingsVar = GameUISettings.Difficulty;


//******************************************************
//*****    Script Resource: hooks
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.hooks = TG.hooks || {};
var publisherSettings = window["publisher"];
var _firstAdsInit = false;
var _standardAdsShow = false;

TG.hooks.handleGameInitialized = function(){
	"use strict";
	var Booster = window.Booster || {};
	new BoosterSdk(publisherSettings, TG.controller.getGameOrientation(), TG.controller.handlePause, TG.controller.handleResume, TG.controller.continueGameFlow, this);
	
	//handle sound issues, by disabling sfx and/or music at specific devices 
	var ua = window["navigator"]["userAgent"];
	var ieDetection = false;
	var isChrome = /Chrome/.test(window["navigator"]["userAgent"]) && /Google Inc/.test(window["navigator"]["vendor"]);
	var isSharp = ((/SHL24|SHL23|SHL22|SHL25|SHL21/i).test(ua)) && !isChrome;
	var isSharpNormal = ((/SH-01F/i).test(ua)) && !isChrome;
	var androidDetection = /Android/.test(ua) && (!isSharp || !isSharpNormal);

	function isIE(){
		if(window["ActiveXObject"] || "ActiveXObject" in window){
		// Always true if browser is Internet Explorer
		ieDetection = true;
		}
	}

	// Checks if the browser is Internet Explorer
	isIE();
	
	//checks for listed devices if the music should be enabled or not.
	if(isSharp || isSharpNormal || androidDetection){
		TG.controller.loadMusic = false;
	}

	//disable SFX if IE = true
	if(ieDetection || isSharpNormal || isSharp){
		TG.controller.loadSfx = false;
	}

	//TG.controller.continueGameFlow();
	
	//if moreGames is false, set the handleMoreGamesButtonPressed function to undefined (so the framework won't create a button')
	if(!publisherSettings["moreGames"]){  
		TG.hooks.handleMoreGamesButtonPressed = undefined;
	}
};

TG.hooks.informStartScreenShown = function(){
	"use strict";
	try{
		analytics["menu"]();
	} catch(e){
		console.log("error calling analytics.menu: " + e); 
	}
};

TG.hooks.informLevelStart = function(worldIdx, levelIdx){
	"use strict";
	var timer, secTimer, self = this;
//  console.log("LEVEL STARTED");
	if(publisherSettings["enableAds"] && !_firstAdsInit){
		timer = setTimeout(function(){self.handleAdsShow();}, (publisherSettings["firstAd"] * 1000));
		secTimer = setTimeout(function(){self.standardTimer();}, (publisherSettings["firstAd"] * 1000));
		_firstAdsInit = true;
	}
};

TG.hooks.standardTimer = function(){
	"use strict";
//  console.log("set second timer");
	var self = this;
	setInterval(function(){self.handleAdsShow();}, (publisherSettings["adFreq"] * 1000));
};

TG.hooks.handleAdsShow = function(){
	"use strict";
//  console.log("Can show ads");
	TG.config.hasAdBeenShown = true;
};

TG.hooks.informLevelEndScreenStart = function(){
	"use strict";
	try { 
		var points = TG.controller.getTotalScoreAllLevels();
		community["submitScore"]({
			"score": points, // This is an int value
			"callback": function () {
				// Call method(s) in the game that should happen after the high score popup is closed, like showing advertisement.
				if(TG.config.hasAdBeenShown){
					try{
						TG.controller.handlePause();
						adSense["showAdvertising"]({"callback": TG.controller.handleResume});
						TG.config.hasAdBeenShown = false;
						//_standardAdsShow = false;
					} catch(e){
						console.log("error calling adSense.showAdvertising: " + e);
						TG.controller.handleResume();
					}
				}
			}
		});
		analytics["score"](points);
	}catch(e){ 
		console.log("error submitting highscore: " + points +", " + e); 
	} 
};

TG.hooks.handleLevelCompleted = function(levelData){
	"use strict";
	if(levelData["failed"]){
		try { 
			analytics["levelFailed"](levelData["level"]);
		} catch(e){ 
			console.log("error calling analytics.levelFailed: " + e); 
		} 
	}else{
		try { 
			analytics["level"](levelData["level"]);
		} catch(e){ 
			console.log("error calling analytics.level: " + e);
		}       
	}
	//trigger ad if ad has not yet been shown 
	//to support showing ad after level restart (where no level end screen is shown)
	if(TG.config.hasAdBeenShown){
		try{
			TG.controller.handlePause();
			adSense["showAdvertising"]({"callback": function(){TG.controller.handleResume(); TG.controller.continueGameFlow();}});
			TG.config.hasAdBeenShown = false;
			//_standardAdsShow = false;
		} catch(e){
			console.log("error calling adSense.showAdvertising: " + e);
			TG.controller.handleResume();
			TG.controller.continueGameFlow();
		}
	}else{
		TG.controller.continueGameFlow();
	}
};

TG.hooks.informLevelQuit = function(){
	"use strict";
	//also show advertisement on level quit
	if(TG.config.hasAdBeenShown){
		try{
			TG.controller.handlePause();
			adSense["showAdvertising"]({"callback": function(){TG.controller.handleResume();}});
			TG.config.hasAdBeenShown = false;
			//_standardAdsShow = false;
		}catch(e){
			console.log("error calling adSense.showAdvertising: " + e);
			TG.controller.handleResume();
		}
	}
};

TG.hooks.handleMoreGamesButtonPressed = function(){
	"use strict";
	if(!publisherSettings["moreGamesUrl"] || publisherSettings["moreGamesUrl"] === ""){        
		try{ 
			moregames["redirect"](); 
		}catch(e){ 
			console.log("error calling moregames.redirect: " + e); 
		} 
	} else {
		TG.hooks.openUrl(publisherSettings["moreGamesURL"]);
	}
};



TG.hooks.getStartScreenLogos = function(){
	"use strict";
	var self = this;
	
	return [{image: s_logo_coolgames_start, url: TG.config.homeUrl, callback: self.handleMoreGamesButtonPressed}];
};

TG.hooks.payment = function(pricePoint, price, description, callback, context){
	freemium["purchase"](pricePoint, price, description, callback, context);
};









//******************************************************
//*****    Script Resource: configuration_version
//******************************************************

// ******************************
// Tingly Builder JavaScript file
// ******************************

TG.version = TG.version || {};
TG.version["configuration_boostermedia"] = "2.6";
//******************************************************
//*****    Token Resource: TG_MainController
//******************************************************

// ***************************************
// Definition of game token TG_MainController
// ***************************************

// The constructor for the TG_MainController game token.
var TG_MainController = function(){
	"use strict";
	// Properties used by the engine
	this.depth = -1000000;
	this.visible = true;
	this.active = true;
	this.layer = TG.frameworkEffectsLayer;
	// Your own properties
	this.init = false;
	this.load = false;
	this.levelInit = false;
	this.levelLoad = false;
	this.levelEnd = false;
	this.end = false;
	this.flowPausedCounter = 0;
	this.waitingForAds = false;
	this.waitingForPortals = false;
	this.state = "GAME_INIT";
	this.screen = null;
	this.currentWorldIdx = 0;
	this.currentLevelIdx = 0;
	this.noOfPlays = 0;
	this.fontsLoaded = false;
	this.loadSfx = true;
	this.loadMusic = true;
	this.musicGain = 1;
	this.challengeMode = false;
	this.storedData = {};
	this.playerPreferences = {
	    "difficulty": 1,
	    "playMusic": true,
	    "playSFX": true,
	    "language": TG.dictionary.getLanguage()
	};
	var self = this;
	window.addEventListener("gameSetPause", self.handlePause, false);
	window.addEventListener("gameResume", self.handleResume, false);
	document.addEventListener("visibilitychange", self.handleVisibilityChange, false);
	this.timedLevelEvents = "timedLevelEvent";
};

TG_MainController.prototype.handlePause = function(e){
	"use strict";
	tbAudio.pause("master");
	tbGame.pause();
};

TG_MainController.prototype.handleResume = function(e){
	"use strict";
	tbAudio.resume("master");
	tbGame.clearClickEvents();
	tbGame.clearTouchEvents();
	tbGame.clearKeyboardEvents();
	tbGame.resume();
};

TG_MainController.prototype.handleVisibilityChange = function(){
	"use strict";
	if(document.hidden){
		TG.controller.handlePause();
	}else{
		TG.controller.handleResume();
	}
};

if (testMode) {

	TG_MainController.drawDebugInfo = false;

	TG_MainController.prototype.handleKeyDown = function(keyCode) {
		"use strict";

		if (keyCode === tbKeyCodes.vkD) {

			TG_MainController.drawDebugInfo = !TG_MainController.drawDebugInfo;

		}
	};


	TG_MainController.drawTextRect = function(font, x, y, maxWidth, maxHeight) {
		"use strict";

		if (TG_MainController.drawDebugInfo) {
			if (maxWidth && maxHeight) {

				var xOffset = 0;

				if (font.getAlign() === "center") {
					xOffset -= maxWidth / 2;
				}
				if (font.getAlign() === "right") {
					xOffset -= maxWidth;
				}

				var yOffset = 0;

				if (font.getBaseline() === "middle") {
					yOffset -= maxHeight / 2;
				}
				if (font.getBaseline() === "bottom") {
					yOffset -= maxHeight;
				}

				tbGraphics.drawRectangle(x + xOffset, y + yOffset, maxWidth, maxHeight, undefined, "blue", 1);
			}
			else {
				tbGraphics.drawDisk(x, y, 5, undefined, "red", 1);
			}
		}
	};

	TG_MainController.drawSurfaceBoundaries = function() {
		"use strict";
		if (TG_MainController.drawDebugInfo) {
			tbGraphics.drawRectangle(0, 0, tbGraphics.canvas.width, tbGraphics.canvas.height, undefined, "blue", 2);
		}
	};

}

TG_MainController.prototype.handleBackButton = function(){
	"use strict";
	tbGame.addSpecial(TG.backButtonEventId);
};

TG_MainController.prototype._init = function(){
	"use strict";
	var version, self = this, handleStoredDataRetrieved;
	if(TG.settings.theme.background !== undefined && TG.settings.theme.background.color !== undefined){
		document.body.style.background = TG.settings.theme.background.color;
	}

	//initialise analytics
	TG.analytics = new TG_Analytics();

	//test for external highscore system
	if(TG.config.highScoreSystem && TG.config.highScoreSystem.active){
		self.useExternalHighScoreSystem = TG_HighScores.checkActive(function(isActive){
			self.useExternalHighScoreSystem = isActive;
		});
	}

	//read game settings
	TG.gameFlowSettings = TG.settings.game.framework || {};
	TG.gameFlowSettings.gameType = TG.gameFlowSettings.gameType || "level";
	TG.gameFlowSettings.showLevelMap = (TG.gameFlowSettings.showLevelMap !== undefined)? TG.gameFlowSettings.showLevelMap
	        : (TG.gameFlowSettings.gameType === "level");
	TG.gameFlowSettings.noOfLevels = (TG.gameFlowSettings.noOfLevels !== undefined)? ((TG.gameFlowSettings.noOfLevels instanceof Array)? TG.gameFlowSettings.noOfLevels
	        : [TG.gameFlowSettings.noOfLevels])
	        : [20];
	TG.gameFlowSettings.defaultLevelState = (TG.gameFlowSettings.defaultLevelState !== undefined)? TG.gameFlowSettings.defaultLevelState : "locked";
	TG.gameFlowSettings.showDifficulty = (TG.gameFlowSettings.showDifficulty !== undefined)? TG.gameFlowSettings.showDifficulty
	        : (TG.gameFlowSettings.gameType === "difficulty");
	TG.gameFlowSettings.usePerWorldLevelNumbering = (TG.gameFlowSettings.usePerWorldLevelNumbering !== undefined)? TG.gameFlowSettings.usePerWorldLevelNumbering
	        : false;
	TG.gameFlowSettings.showGameEndScreen = (TG.gameFlowSettings.showGameEndScreen !== undefined)? TG.gameFlowSettings.showGameEndScreen
	        : (TG.gameFlowSettings.gameType === "level");
	TG.gameFlowSettings.computeHighScore = (TG.gameFlowSettings.computeHighScore !== undefined)? TG.gameFlowSettings.computeHighScore : "max";
	TG.gameFlowSettings.scoreType = (TG.gameFlowSettings.scoreType !== undefined)? TG.gameFlowSettings.scoreType : "number";

	// get stored data and update player preferences
	handleStoredDataRetrieved = function(data){
		var l, w, lvlState, wIdxSet;
		if(data){
			self.storedData = data;
		}
		self.playerPreferences = self._getGameData("preferences", {});
		self.playerPreferences["difficulty"] = (self.playerPreferences["difficulty"] !== undefined)? self.playerPreferences["difficulty"] : 1;
		if(TG.gameFlowSettings.supportedDifficulties !== undefined && TG.gameFlowSettings.supportedDifficulties.indexOf(self.getDifficulty()) < 0){
			self.playerPreferences["difficulty"] = TG.gameFlowSettings.supportedDifficulties[0];
		}
		self.playerPreferences["playMusic"] = (self.playerPreferences["playMusic"] !== undefined)? self.playerPreferences["playMusic"] : true;
		self.playMusic(self.playerPreferences["playMusic"]);
		self.playerPreferences["playSFX"] = (self.playerPreferences["playSFX"] !== undefined)? self.playerPreferences["playSFX"] : true;
		self.playSFX(self.playerPreferences["playSFX"]);
		self.playerPreferences["language"] = (self.playerPreferences["language"] !== undefined && TG.dictionary
		        .isLanguageAvailable(self.playerPreferences["language"]))? self.playerPreferences["language"] : TG.dictionary.getLanguage();
		TG.dictionary.setLanguage(self.playerPreferences["language"]);

		// unlock first level
		if(self.getLevelData(self.currentWorldIdx, 0, "state") === undefined){
			self.updateLevelData(self.currentWorldIdx, 0, "state", "unlocked");
		}

		//determine current world idx
		if(TG.gameFlowSettings.showLevelMap){
			if(TG.gameFlowSettings.defaultLevelState === "locked"){
				//find first locked level
				wIdxSet = false;
				for(w = 0; w < TG.gameFlowSettings.noOfLevels.length; w++){
					for(l = 0; l < TG.gameFlowSettings.noOfLevels[w]; l++){
						lvlState = self.getLevelData(w, l, "state", "locked");
						if(lvlState === "locked"){
							self.currentWorldIdx = (l - 1 >= 0)? w : ((w - 1 >= 0)? w - 1 : 0);
							wIdxSet = true;
							break;
						}
					}
					if(wIdxSet){
						break;
					}
				}

			}else if(self.playerPreferences["lastPlayed"] !== undefined){
				self.currentWorldIdx = self.playerPreferences["lastPlayed"]["world"] || 0;
			}
		}

	};
	TG.hooks.getStoredData(null, handleStoredDataRetrieved);

	// parse url
	self.urlData = self._parseUrl();
	if(self.urlData["authToken"] !== undefined && self.urlData["challengeId"] !== undefined){
		self.challengeMode = true;
	}
	if(self.urlData["testChallengeMode"] !== undefined && testMode){
		self.challengeMode = true;
		self.testChallengeMode = true;
	}

	// setup backend service provider
	if(TG.config.useBackendServiceProvider){
		if(!this.testChallengeMode){
			this.backendServiceProvider = new BackendServiceProvider();
		}else{
			this.backendServiceProvider = new TestBackendServiceProvider();
		}
		this.backendServiceProvider.initialise(function(init){
			if(init){
				TG.controller.backendServiceProvider.logIn(self.urlData["authToken"]);
			}
		});
	}

	//test for windows phone ie browsers and old Android stock browsers because they do not support
	//multiple sounds
	version = parseFloat(tbBrowser.browser.version);
	if(tbAudio.usingHTMLAudio && ((tbBrowser.os.windowsphone && tbBrowser.browser.msie) || (tbBrowser.browser.stockandroid && version && version < 4.4))){
		tbAudio.maxConcurrentSounds(1);
	}
	this.init = true;
	this.initWaitTimer = 0;
};

TG_MainController.prototype._parseUrl = function(){
	"use strict";
	var queryString, params, queries, temp, i, l;
	params = {};
	queryString = window.location.search.substring(1);
	if(queryString){
		// Split into key/value pairs
		queries = queryString.split("&");

		// Convert the array of strings into an object
		for(i = 0, l = queries.length; i < l; i++){
			temp = queries[i].split("=");
			params[temp[0]] = temp[1];
		}
	}
	return params;
};

TG_MainController.prototype.getUrlData = function(key){
	"use strict";
	if(TG.controller.urlData){
		return TG.controller.urlData[key];
	}
};

TG_MainController.prototype._readyForStart = function(){
	"use strict";
	if(!this.challengeMode || !this.backendServiceProvider || this.backendServiceProvider.isChallengeStarted()){
		return true;
	}
	return false;
};

TG_MainController.prototype._load = function(){
	"use strict";
	this.state = "GAME_LOAD";
	this.screen = new TG_LoadingScreen(function(){
		TG.controller.load = true;
		TG.controller.pauseFlow(true, true);
		TG.adHooks.handleGameLoaded();
		TG.hooks.handleGameLoaded();
	}, function(progress){
		TG.adHooks.loadingProgress(progress);
		TG.hooks.loadingProgress(progress);
	}, TG.config.showLoadingScreen);
};

TG_MainController.prototype.pauseFlow = function(waitingForPortals, waitingForAds){
	"use strict";
	this.waitingForAds = waitingForAds || false;
	this.waitingForPortals = waitingForPortals || false;
	this.flowPausedCounter++;
};

TG_MainController.prototype.resumeFlow = function(){
	"use strict";
	var self = TG.controller;
	//updated flowPausedCounter
	self.flowPausedCounter--;
	if(testMode && self.flowPausedCounter < 0){
		console.log("flowPausedCounter < 0!!!: " + self.flowPausedCounter);
	}
	//go to next state
	switch(self.state){
		case "GAME_INIT":
			if(self.init && !self.waitingForAd){
				if(self.challengeMode && self.backendServiceProvider){
					self.backendServiceProvider.startChallenge(self.urlData["challengeId"], function(challengeValid){
						if(!challengeValid && self.screen && typeof self.screen.showError === "function"){
							self.screen.showError("challengeLoadingError_notValid");
						}
					});
				}
				self._load();
			}
		break;
		case "GAME_LOAD":
			if(self.load){
				if(self.challengeMode && self.backendServiceProvider){
					if(!self.backendServiceProvider.isChallengeStarted()){
						self.screen.showError("challengeLoadingError_notStarted");
						return;
					}else{
						self.getChallengeInformation();
						self._setDifficulty(self._challengeInformation.mode);
					}
				}
				tbGame.removeToken(self.screen);
				if(typeof GameController === "function"){
					TG.game = new GameController();
				}
				if(TG.config.banner !== undefined && TG.config.banner.show !== false){
					TG.functions.addBanner(true, (TG.config.banner === undefined)? true : TG.config.banner.clickable !== false);
				}
				self._showStartScreen();
			}
		break;
		case "LEVEL_INIT":
			if(self.levelInit){
				self._loadGameLevel();
			}
		break;
		case "LEVEL_LOAD":
			if(self.levelLoad){
				self._startGameLevel();
			}
		break;
		case "LEVEL_END":
			if(self.levelEnd){
				self._levelEndCallback();
			}
		break;
		case "GAME_END":
			if(self.end){
				self.end = false;
				self._showEndGameScreen();
			}
		break;
	}
};

TG_MainController.prototype.continueGameFlow = function(){
	"use strict";
	TG.controller.waitingForPortals = false;
};

TG_MainController.prototype.continueGameFlowAfterAd = function(){
	"use strict";
	TG.controller.waitingForAds = false;
};

TG_MainController.prototype.getGameOrientation = function(){
	"use strict";
	if(TG.orientation){
		TG.orientation = (buildTags.indexOf("landscape") >= 0)? "landscape" : "portrait";
	}
	return TG.orientation;
};

TG_MainController.prototype.getGameName = function(){
	"use strict";
	var name = TG.settings.theme.gameName || TG.settings.theme.game.gameName;
	if(typeof name === "string"){
		return name;
	}
	return "";
};

TG_MainController.prototype.getMoreGamesFunction = function(){
	"use strict";
	var url;
	if(TG.controller.urlData["more_games"] !== undefined){
		try{
			url = decodeURIComponent(TG.controller.urlData["more_games"]);
			return function(){
				TG.hooks.openUrl(url);
			};
		}catch(e){
		}
	}
	if(typeof TG.publisherSettings["moreGamesUrl"] === "string" && TG.publisherSettings["moreGamesUrl"] !== ""){
		return function(){
			TG.hooks.openUrl(TG.publisherSettings["moreGamesUrl"]);
		};
	}
	if(TG.config.moreGamesUrl !== undefined){
		return function(){
			TG.hooks.openUrl(TG.config.moreGamesUrl);
		};
	}
	if(typeof TG.hooks.handleMoreGamesButtonPressed === "function"){
		return TG.hooks.handleMoreGamesButtonPressed;
	}
};

TG_MainController.prototype.getCurrentWorldIdx = function(){
	"use strict";
	return this.currentWorldIdx;
};

TG_MainController.prototype._setCurrentWorldIdx = function(idx){
	"use strict";
	if(typeof idx === "number" && idx < TG.gameFlowSettings.noOfLevels.length){
		this.currentWorldIdx = idx;
	}
};

TG_MainController.prototype.getChallengeInformation = function(update, callback){
	"use strict";
	var cf, self = this;
	if(this.challengeMode && this.backendServiceProvider !== undefined){
		if(update){
			cf = function(){
				self._challengeInformation = self.backendServiceProvider.getChallengeInfo();
				if(typeof callback === "function"){
					callback(self._challengeInformation);
				}
			};
			this.backendServiceProvider.updateChallengeInfo(cf);
		}else{
			if(this._challengeInformation === undefined){
				this._challengeInformation = this.backendServiceProvider.getChallengeInfo();
			}
			return this._challengeInformation;
		}
	}
};

TG_MainController.prototype.forfeitChallenge = function(callback){
	"use strict";
	if(TG.controller.challengeMode && TG.controller.backendServiceProvider){
		TG.controller.backendServiceProvider.forfeitChallenge(callback);
	}
};

TG_MainController.prototype.cancelChallenge = function(callback){
	"use strict";
	if(TG.controller.challengeMode && TG.controller.backendServiceProvider){
		TG.controller.backendServiceProvider.cancelChallenge(callback);
	}
};

TG_MainController.prototype.getDifficulty = function(){
	"use strict";
	return TG.controller.playerPreferences["difficulty"];
};

TG_MainController.prototype.getDifficultyId = function(){
	"use strict";
	switch(this.getDifficulty()){
		case 0:
			return "easy";
		case 1:
			return "medium";
		case 2:
			return "hard";
		default:
			throw "Unknown difficulty: " + this.getDifficulty();
	}
};

TG_MainController.prototype.getDifficultyText = function(){
	"use strict";
	var key = "optionsDifficulty_" + this.getDifficultyId();
	return TG.dictionary.getText(key, "<" + key + ">");
};

TG_MainController.prototype._setDifficulty = function(difficulty){
	"use strict";
	TG.controller.playerPreferences["difficulty"] = difficulty;
	TG.controller._updateGameData("preferences", TG.controller.playerPreferences);
};

TG_MainController.prototype.setWinningCondition = function(condition){
	"use strict";
	if(condition === "max" || condition === "min"){
		TG.gameFlowSettings.computeHighScore = condition;
	}
};

TG_MainController.prototype.setScoreType = function(type){
	"use strict";
	if(type === "number" || type === "time"){
		TG.gameFlowSettings.scoreType = type;
	}
};

TG_MainController.prototype.playMusic = function(playMusic){
	"use strict";
	if(playMusic !== undefined){
		TG.controller.playerPreferences["playMusic"] = playMusic;
		TG.controller._updateGameData("preferences", TG.controller.playerPreferences);
		if(playMusic){
			tbAudio.unmute("music");
		}else{
			tbAudio.mute("music");
		}
	}
	return TG.controller.playerPreferences["playMusic"];
};

TG_MainController.prototype.playSFX = function(playSFX){
	"use strict";
	if(playSFX !== undefined){
		TG.controller.playerPreferences["playSFX"] = playSFX;
		TG.controller._updateGameData("preferences", TG.controller.playerPreferences);
		if(playSFX){
			tbAudio.unmute("game");
			tbAudio.unmute("sfx");
		}else{
			tbAudio.mute("game");
			tbAudio.mute("sfx");
		}
	}
	return TG.controller.playerPreferences["playSFX"];
};

TG_MainController.prototype.language = function(language){
	"use strict";
	if(language !== undefined){
		TG.controller.playerPreferences["language"] = language;
		TG.controller._updateGameData("preferences", TG.controller.playerPreferences);
		//TODO set dictionary language from here not the other way around
	}
	return TG.controller.playerPreferences["language"];
};

TG_MainController.prototype.getFormattedScoreString = function(score){
	"use strict";
	if(TG.gameFlowSettings.scoreType === "time"){
		return ((score < 0)? "-" : "") + Math.floor(Math.abs(score) / 60) + ((Math.abs(score % 60) < 10)? ":0" : ":") + Math.abs(score % 60);
	}
	return "" + score;
};

TG_MainController.prototype.getLevelData = function(worldIdx, levelIdx, key, defaultValue){
	"use strict";
	return TG.controller._getLevelData(worldIdx, levelIdx, key, defaultValue, "game");
};

TG_MainController.prototype.updateLevelData = function(worldIdx, levelIdx, key, value){
	"use strict";
	return TG.controller._updateLevelData(worldIdx, levelIdx, key, value, "game");
};

TG_MainController.prototype.getGameData = function(key, defaultValue){
	"use strict";
	return TG.controller._getGameData(key, defaultValue, "game");
};

TG_MainController.prototype.updateGameData = function(key, value){
	"use strict";
	return TG.controller._updateGameData(key, value, "game");
};

TG_MainController.prototype._getLevelData = function(worldIdx, levelIdx, key, defaultValue, origin){
	"use strict";
	var data;
	if(origin !== "game"){
		origin = "tg";
	}
	data = TG.controller.storedData["level_" + worldIdx + "_" + levelIdx];
	if(data !== undefined){
		data = (key === undefined)? data[origin] : data[origin][key];
		if(data !== undefined){
			return data;
		}
	}
	return defaultValue;
};

TG_MainController.prototype._updateLevelData = function(worldIdx, levelIdx, key, value, origin){
	"use strict";
	if(origin !== "game"){
		origin = "tg";
	}
	if(TG.controller.storedData["level_" + worldIdx + "_" + levelIdx] === undefined){
		TG.controller.storedData["level_" + worldIdx + "_" + levelIdx] = {
		    "tg": {},
		    "game": {}
		};
	}

	if(key === undefined){
		TG.controller.storedData["level_" + worldIdx + "_" + levelIdx][origin] = value;
	}else{
		TG.controller.storedData["level_" + worldIdx + "_" + levelIdx][origin][key] = value;
	}
	return TG.hooks.updateStoredData(TG.controller.storedData);
};

TG_MainController.prototype._getGameData = function(key, defaultValue, origin){
	"use strict";
	var data;
	if(origin !== "game"){
		origin = "tg";
	}
	data = TG.controller.storedData["game"];
	if(data !== undefined){
		data = (key === undefined)? data[origin] : data[origin][key];
		if(data !== undefined){
			return data;
		}
	}
	return defaultValue;
};

TG_MainController.prototype._updateGameData = function(key, value, origin){
	"use strict";
	if(origin !== "game"){
		origin = "tg";
	}
	if(TG.controller.storedData["game"] === undefined){
		TG.controller.storedData["game"] = {
		    "tg": {},
		    "game": {}
		};
	}

	if(key === undefined){
		TG.controller.storedData["game"][origin] = value;
	}else{
		TG.controller.storedData["game"][origin][key] = value;
	}
	return TG.hooks.updateStoredData(TG.controller.storedData);
};

TG_MainController.prototype.getLevelStatistics = function(statisticKey, levelIdx, worldIdx){
	"use strict";
	if(levelIdx === undefined){
		levelIdx = this.currentLevelIdx;
	}
	if(worldIdx === undefined){
		worldIdx = this.getCurrentWorldIdx();
	}
	if(statisticKey === undefined){
		return this.getLevelData(worldIdx, levelIdx, "stats", {});
	}
	return this.getLevelData(worldIdx, levelIdx, "stats", {})[statisticKey];
};

TG_MainController.prototype.getLevelHighScore = function(levelIdx, worldIdx, levelWorldIdx){
	"use strict";
	if(levelWorldIdx === undefined){
		if(levelIdx !== undefined){
			levelWorldIdx = this.getWorldLevelIdx(levelIdx);
			if(worldIdx === undefined){
				worldIdx = this.getWorldIdx(levelIdx);
			}
		}
	}
	var score = this.getLevelStatistics("highScore", levelWorldIdx, worldIdx);
	if(typeof score !== "number"){
		return 0;
	}
	return score;
};

TG_MainController.prototype.setLevelHighScore = function(score, levelIdx, worldIdx){
	"use strict";
	if(levelIdx === undefined){
		levelIdx = this.currentLevelIdx;
	}
	if(worldIdx === undefined){
		worldIdx = this.getCurrentWorldIdx();
	}
	if(typeof score === "number" && score > this.getLevelHighScore(levelIdx, worldIdx)){
		var stats = this.getLevelStatistics(undefined, levelIdx, worldIdx);
		stats["highScore"] = score;
		this.updateLevelData(worldIdx, levelIdx, "stats", stats);
	}
};

TG_MainController.prototype.getTotalScoreAllLevels = function(){
	"use strict";
	var w, i, stats, points = 0;
	for(w = 0; w < TG.gameFlowSettings.noOfLevels.length; w++){
		for(i = 0; i < TG.gameFlowSettings.noOfLevels[w]; i++){
			stats = TG.controller.getLevelStatistics(undefined, i, w);
			if(typeof stats === "object" && stats !== null){
				points += (stats["highScore"] !== undefined)? stats["highScore"] : 0;
			}
		}
	}
	return points;
};

TG_MainController.prototype.getTotalStarsAllLevels = function(){
	"use strict";
	var w, i, stats, stars = 0;
	for(w = 0; w < TG.gameFlowSettings.noOfLevels.length; w++){
		for(i = 0; i < TG.gameFlowSettings.noOfLevels[w]; i++){
			stats = TG.controller.getLevelStatistics(undefined, i, w);
			if(typeof stats === "object" && stats !== null){
				stars += (stats["stars"] !== undefined)? stats["stars"] : 0;
			}
		}
	}
	return stars;
};

TG_MainController.prototype._startGame = function(){
	"use strict";
	var sound, delay, fadeDuration;
	if(TG.controller.music === undefined){
		if(TG.settings.theme.music !== undefined){
			if(TG.settings.theme.music.backgroundSource !== undefined){
				sound = TG.settings.theme.music.backgroundSource;
			}
			if(TG.settings.theme.music.backgroundGain !== undefined){
				tbAudio.setGain("music", TG.settings.theme.music.backgroundGain);
				if(!this.playMusic()){
					tbAudio.mute("music");
				}
				TG.controller.musicGain = TG.settings.theme.music.backgroundGain;
			}
			delay = (TG.settings.theme.music.backgroundDelay !== undefined)? TG.settings.theme.music.backgroundDelay : 0;
			fadeDuration = (TG.settings.theme.music.backgroundFadeInDuration !== undefined)? TG.settings.theme.music.backgroundFadeInDuration : 0;
		}
		if(sound === undefined && typeof a_music !== "undefined"){
			sound = a_music;
		}
		if(sound !== undefined){
			TG.controller.music = tbAudio.play(sound, delay, fadeDuration);
			if(TG.controller.music){
				tbAudio.addTags(TG.controller.music, "music");
				tbAudio.setLooping(TG.controller.music, true);
			}
		}
	}

	if(this.noOfPlays === 0){
		TG.hooks.informGameStart();
	}else{
		TG.hooks.informGameRestart();
	}

	if(TG.gameFlowSettings.showLevelMap && !this.challengeMode){
		this._showLevelMap();
	}else{
		this._initGameLevel(0);
	}
};

TG_MainController.prototype._showStartScreen = function(){
	"use strict";
	// create and show startScreen
	if(this.screen){
		tbGame.removeToken(this.screen);
	}
	this.screen = new TG_StartScreen();
	this.currentLevelIdx = -1;
	TG.hooks.informStartScreenShown();
};

TG_MainController.prototype._showLevelMap = function(){
	"use strict";
	// create and show levelMap
	this.screen = new TG_LevelMapScreen();
};

TG_MainController.prototype.drawInGameBackground = function(x, y, width, height){
	"use strict";
	var image;
	if(TG.settings.theme.inGame !== undefined && TG.settings.theme.inGame.backgroundImage !== undefined){
		image = TG.settings.theme.inGame.backgroundImage;
	}else if(TG.settings.tg.inGame !== undefined){
		image = TG.settings.tg.inGame.backgroundImage;
	}else{
		image = undefined;
	}
	TG.functions.setCanvas(TG.frameworkBackgroundLayer);
	x = x || 0;
	y = y || 0;
	width = width || tbGraphics.width;
	height = height || tbGraphics.height;

	if(!image){
		return;
	}

	width = Math.min(Math.min(width, tbGraphics.width), image.combinedWidth);
	height = Math.min(Math.min(height, tbGraphics.height), image.combinedHeight);
	if(image !== undefined){
		image.drawCombinedPart(x, y - TG.backgroundOffsetTop, width, height, x, y, 1);
	}else{
		tbGraphics.drawRectangle(x, y, width, height, "white", false);
	}
};

TG_MainController.prototype._initGameLevel = function(levelIdx, worldIdx){
	"use strict";
	this.state = "LEVEL_INIT";
	if(worldIdx === undefined){
		worldIdx = this.currentWorldIdx;
	}else{
		this.currentWorldIdx = worldIdx;
	}
	this.currentLevelIdx = levelIdx;
	this.levelInit = true;
	this.pauseFlow(true, false);
	if(TG.gameFlowSettings.showLevelMap){
		TG.hooks.handleLevelInitialized(this.currentWorldIdx, this.currentLevelIdx);
	}else{
		TG.hooks.handleLevelInitialized(this.currentWorldIdx, this.noOfPlays);
	}
};

TG_MainController.prototype._loadGameLevel = function(){
	"use strict";
	this.state = "LEVEL_LOAD";
	this.levelLoad = true;
	this.pauseFlow(true, false);
	TG.hooks.handleLevelLoaded(this.currentWorldIdx, this.currentLevelIdx);
};

TG_MainController.prototype._startGameLevel = function(){
	"use strict";
	var i, idx = 0;
	if(this.currentWorldIdx < TG.gameFlowSettings.noOfLevels.length && this.currentLevelIdx < TG.gameFlowSettings.noOfLevels[this.currentWorldIdx]){
		this.state = "LEVEL_PLAY";
		this.noOfPlays += 1;
		this.levelEnd = false;
		this.screen = null;
		// start game level
		this.drawInGameBackground(0, TG.backgroundOffsetTop);
		TG.analytics.reportTimedEventStart(this.getLevelString(3), "progression:levelStarted:" + this.getDifficultyId(), this.timedLevelEvents, true);
		for(i = 0; i < this.currentWorldIdx; i++){
			idx += TG.gameFlowSettings.noOfLevels[i];
		}
		TG.hooks.informLevelStart(this.currentWorldIdx, this.currentLevelIdx);
		this.playerPreferences["lastPlayed"] = {
		    "world": this.currentWorldIdx,
		    "level": this.currentLevelIdx
		};
		TG.level = new LevelController(idx + this.currentLevelIdx, this.currentWorldIdx, this.currentLevelIdx);
	}
};

TG_MainController.prototype.getSequentialLevelIdx = function(worldIdx, levelIdx){
	"use strict";
	var i, idx = 0;
	if(worldIdx === undefined){
		worldIdx = this.currentWorldIdx;
	}
	if(levelIdx === undefined){
		levelIdx = this.currentLevelIdx;
	}
	for(i = 0; i < worldIdx; i++){
		idx += TG.gameFlowSettings.noOfLevels[i];
	}
	return idx + levelIdx;
};

TG_MainController.prototype.getWorldLevelIdx = function(sequentialLevelIdx){
	"use strict";
	var i, idx = sequentialLevelIdx;
	for(i = 0; i < TG.gameFlowSettings.noOfLevels.length; i++){
		if(idx < TG.gameFlowSettings.noOfLevels[i]){
			return idx;
		}
		idx -= TG.gameFlowSettings.noOfLevels[i];
	}
	return idx;
};

TG_MainController.prototype.getWorldIdx = function(sequentialLevelIdx){
	"use strict";
	var i, world = 0, levels = 0;
	for(i = 0; i < TG.gameFlowSettings.noOfLevels.length; i++){
		levels += TG.gameFlowSettings.noOfLevels[i];
		if(levels > sequentialLevelIdx){
			return world;
		}
		world += 1;
	}
	return world;
};

TG_MainController.prototype._showEndGameScreen = function(){
	"use strict";
	TG.controller.screen = null;
	TG.controller.screen = new TG_GameEndScreen();
};

TG_MainController.prototype._showInterstitial = function(){
	// create and show appropriate interstitial
};

TG_MainController.prototype.showStartLevelScreen = function(headerText, assignmentText, callbackFunc, callbackObj){
	"use strict";
	// creates TG_LevelStartDialog
	new TG_LevelStartDialog(headerText, assignmentText, callbackFunc, callbackObj);
};

TG_MainController.prototype.showStartChallengeScreen = function(textKey, callbackFunc, callbackObj){
	"use strict";
	if(this.challengeMode
	        && (this._challengeInformation.currentPlayerIdx !== this._challengeInformation.challengePlayerIdx || (TG.config.challenges && TG.config.challenges.showChallengerStartScreen))){
		new TG_ChallengeStartDialog(textKey, callbackFunc, callbackObj);
	}else{
		if(typeof callbackFunc === "function"){
			if(callbackObj){
				callbackObj.callbackFunc();
			}else{
				callbackFunc();
			}
		}
	}
};

TG_MainController.prototype.getLevelString = function(minNoOfChars){
	"use strict";
	var i, levelString = this.currentLevelIdx + "", leadingZerosRequired = minNoOfChars - levelString.length;
	if(typeof minNoOfChars === "number" && minNoOfChars > 1){
		for(i = 0; i < leadingZerosRequired; i++){
			levelString = "0" + levelString;
		}
	}
	return levelString;
};

TG_MainController.prototype.challengeEnded = function(score){
	"use strict";
	var submit, cancel, handleSubmit, handleOutcome, handleCancel, name, self = this;
	if(self.challengeMode){
		score = Math.floor(score);
		if(self._challengeInformation.challengePlayerIdx === self._challengeInformation.currentPlayerIdx){
			handleSubmit = function(succes){
				if(self.levelEndDialog && typeof self.levelEndDialog.showMessage === "function"){
					if(!succes){
						self.levelEndDialog.showMessage(TG.dictionary.getText("challengeEndScreenChallengeSend_error", "<CHALLENGESENDERROR>"));
					}else{
						self.levelEndDialog.showMessage(TG.dictionary.getText("challengeEndScreenChallengeSend_success", "<CHALLENGESENDTEXT>"));
						if(!self._challengeInformation.stranger){
							name = self._challengeInformation.playerNames[1];
							name = (name.length > 13)? name.substr(0, 10) + "..." : name;
							self.levelEndDialog.showSubMessage((TG.dictionary
							        .getText("challengeEndScreenChallengeSend_submessage", "<CHALLENGESENDSUBMESSAGE>")).replace(/<NAME>/g, name));
						}else{
							self.levelEndDialog.showSubMessage(TG.dictionary.getText("challengeEndScreenChallengeSend_submessage_stranger", ""));
						}
					}
				}
			};
			handleCancel = function(succes){
				if(!succes){
					self.levelEndDialog.showMessage(TG.dictionary.getText("challengeCancelMessage_error", "<CHALLENGECANCELERROR>"));
				}else{
					self.levelEndDialog.showMessage(TG.dictionary.getText("challengeCancelMessage_success", "<CHALLENGECANCELSUCCESS>"));
				}
			};
			cancel = function(){
				if(self.backendServiceProvider){
					self.backendServiceProvider.cancelChallenge(handleCancel);
				}
			};
		}else{
			handleSubmit = function(succes){
				if(self.levelEndDialog && typeof self.levelEndDialog.showMessage === "function"){
					if(!succes){
						self.levelEndDialog.showMessage(TG.dictionary.getText("challengeEndScreenScoreSend_error", "<CHALLENGESCORESENDERROR>"));
					}
				}
			};
			handleOutcome = function(outcome){
				if(self.levelEndDialog && typeof self.levelEndDialog.updateOutcome === "function"){
					self.levelEndDialog.updateOutcome(outcome);
				}
			};
		}
		submit = function(){
			if(self.backendServiceProvider){
				self.backendServiceProvider.submitChallengeScore(score, TG.gameFlowSettings.scoreType, handleSubmit, handleOutcome);
			}
		};

		this.state = "LEVEL_END";

		// show end screen
		this.levelEndDialog = new TG_ChallengeEndDialog(score, submit, cancel);
	}else{
		self.levelEnded({
			"totalScore": score
		});
	}

};

TG_MainController.prototype.levelEnded = function(stats){
	"use strict";
	this.state = "LEVEL_END";

	var oldStats, newStats, callback, totalScoreNew, i, s, highScoreObj, statsList = {}, leaderboardScore, levelString = this.getLevelString(3), isHighScoreImproved = function(
	        newScore, oldScore){
		if(typeof newScore !== "number"){
			return false;
		}
		if(typeof oldScore !== "number"){
			return true;
		}
		if(TG.gameFlowSettings.computeHighScore === "max" && newScore > oldScore){
			return true;
		}
		if(TG.gameFlowSettings.computeHighScore === "min" && newScore < oldScore){
			return true;
		}
		return false;
	};

	// save stats
	stats = stats || {};
	stats["level"] = TG.gameFlowSettings.usePerWorldLevelNumbering? this.currentLevelIdx + 1 : this.getSequentialLevelIdx() + 1;
	stats.improvedHighScore = false;

	oldStats = this.getLevelData(this.currentWorldIdx, this.currentLevelIdx, "stats", undefined);
	newStats = oldStats || {};

	// compute new total score and compare to old high score
	if(stats.emphasizedStatsObject !== undefined || stats.statsObjects !== undefined){
		if(stats.emphasizedStatsObject !== undefined){
			statsList[stats.emphasizedStatsObject.id] = stats.emphasizedStatsObject.clone();
			if(stats.emphasizedStatsObject.id === "highScore"){
				highScoreObj = stats.emphasizedStatsObject;
			}
		}
		if(stats.statsObjects !== undefined){
			for(i = 0; i < stats.statsObjects.length; i++){
				statsList[stats.statsObjects[i].id] = stats.statsObjects[i].clone();
				if(stats.statsObjects[i].id === "highScore"){
					highScoreObj = stats.statsObjects[i];
				}
			}
		}
		for(i in statsList){
			s = statsList[i];
			if(s.updateFunction !== undefined){
				statsList[s.statToUpdate].endValue = s.updateFunction(statsList[s.statToUpdate].endValue);
			}
		}
		if(statsList["totalScore"] !== undefined){
			totalScoreNew = statsList["totalScore"].endValue;
		}
	}else{
		totalScoreNew = stats["totalScore"];
		if(totalScoreNew !== undefined){
			if(stats["timeBonus"] !== undefined){
				totalScoreNew += stats["timeBonus"];
			}
		}
	}

	if(stats["failed"] !== true){
		// update highScore
		if(totalScoreNew !== undefined){
			TG.analytics.reportEvent("points", "progression:score:" + this.getDifficultyId() + ":" + levelString, totalScoreNew);

			if(oldStats === undefined || isHighScoreImproved(totalScoreNew, oldStats["highScore"])){
				newStats["highScore"] = totalScoreNew;
				stats.improvedHighScore = true;
				TG.analytics.reportEvent("highScore", "progression:score:" + this.getDifficultyId() + ":" + levelString, totalScoreNew);
			}
			if(highScoreObj !== undefined){
				highScoreObj.endValue = newStats["highScore"];
			}
			stats["highScore"] = newStats["highScore"];
		}
		// update stars
		if(stats["stars"] !== undefined){
			newStats["stars"] = stats["stars"];
			TG.analytics.reportEvent("stars", "progression:score:" + this.getDifficultyId() + ":" + levelString, stats["stars"]);
		}
		// unlock next level in required
		if(this.currentLevelIdx + 1 < TG.gameFlowSettings.noOfLevels[this.currentWorldIdx]){
			if(this.getLevelData(this.currentWorldIdx, this.currentLevelIdx + 1, "state", "locked") === "locked"){
				this.updateLevelData(this.currentWorldIdx, this.currentLevelIdx + 1, "state", "unlocked");
			}
		}else if(this.currentWorldIdx + 1 < TG.gameFlowSettings.noOfLevels.length){
			if(this.getLevelData(this.currentWorldIdx + 1, 0, "state", "locked") === "locked"){
				this.updateLevelData(this.currentWorldIdx + 1, 0, "state", "unlocked");
			}
		}
		// store new level data
		this.updateLevelData(this.currentWorldIdx, this.currentLevelIdx, undefined, {
		    "stats": newStats,
		    "state": "played"
		});
		//send high score to leaderboards
		if(this.backendServiceProvider !== undefined){
			var totalScore;

			if (TG.game && TG.game.getTotalChallengeScore) {
				totalScore = TG.game.getTotalChallengeScore();
			}
			else {
				totalScore = TG.controller.getTotalScoreAllLevels();
			}

			if (totalScore !== undefined) {
				this.backendServiceProvider.submitLeaderboardScore(totalScore, TG.gameFlowSettings.scoreType);
			}
		}
		// report level successfully completed
		TG.analytics.reportTimedEventStop(this.timedLevelEvents, levelString, "progression:levelCompleted:" + this.getDifficultyId());
	}else{
		// report level failed
		TG.analytics.reportTimedEventStop(this.timedLevelEvents, levelString, "progression:levelFailed:" + this.getDifficultyId());
	}

	callback = function(callbackAction){
		TG.controller.levelEnd = true;
		TG.controller.levelEndAction = callbackAction;
		TG.controller.pauseFlow(true, true);

		var levelData = {
			    "totalScore": totalScoreNew,
			    "level": stats["level"],
			    "highScore": stats["highScore"],
			    "failed": stats["failed"] === true,
			    "stars": stats["stars"],
			    "stage": stats["stage"]
			};

		TG.hooks.handleLevelCompleted(levelData);
		TG.adHooks.handleLevelCompleted(levelData);
	};

	//show end screen
	this.levelEndDialog = new TG_LevelEndDialog(TG.gameFlowSettings.gameType, stats, callback);
};

TG_MainController.prototype._levelEndCallback = function(callbackAction){
	"use strict";
	//clear previous level
	this._deleteGame();
	TG.controller.levelInit = false;
	TG.controller.levelLoad = false;
	TG.level = undefined;
	TG.functions.getCanvas(TG.gameLayer).drawTokens = true;
	TG.functions.getCanvas(TG.gameUILayer).drawTokens = true;

	//determine next screen
	switch(TG.controller.levelEndAction){
		case "retry":
			TG.controller._initGameLevel(TG.controller.currentLevelIdx);
		break;
		case "next":
			if(TG.gameFlowSettings.showLevelMap){
				if(TG.controller.currentLevelIdx + 1 < TG.gameFlowSettings.noOfLevels[TG.controller.getCurrentWorldIdx()]){
					TG.controller._initGameLevel(TG.controller.currentLevelIdx + 1);
				}else if(TG.controller.getCurrentWorldIdx() + 1 < TG.gameFlowSettings.noOfLevels.length){
					TG.controller._initGameLevel(0, TG.controller.getCurrentWorldIdx() + 1);
				}else{
					if(TG.gameFlowSettings.showGameEndScreen){
						TG.controller.state = "GAME_END";
						TG.controller.end = true;
						TG.controller.pauseFlow(true, false);
						TG.hooks.handleGameCompleted();
					}else{
						TG.controller._showLevelMap();
					}
				}
			}else{
				TG.controller._initGameLevel(0);
			}
		break;
		case "exit":
			if(TG.gameFlowSettings.showLevelMap){
				TG.controller._showLevelMap();
			}else{
				TG.controller._showStartScreen();
			}
		break;
	}
};

TG_MainController.prototype.showTutorial = function(){
	"use strict";
	// show tutorial
	TG.controller.showOptions(true);
};

TG_MainController.prototype.showOptions = function(tutorial, challenge, pauseGame){
	"use strict";
	var type = "inGame";
	if(TG.controller.screen instanceof TG_StartScreen){
		type = "startScreen";
	}else if(TG.controller.screen instanceof TG_LevelMapScreen){
		type = "levelMapScreen";
	}else if(challenge){
		if(TG.controller._challengeInformation.currentPlayerIdx === TG.controller._challengeInformation.challengePlayerIdx){
			type = "inGame_challenger";
		}else{
			type = "inGame_challengee";
		}
	}
	if(!TG.controller.activeModelDialog){
		TG.controller.activeModelDialog = new TG_Options(type, tutorial === true, challenge, pauseGame);
	}
};

TG_MainController.prototype.showOptionsDuringChallenge = function(pauseGame){
	"use strict";
	TG.controller.showOptions(false, true, pauseGame);
};

TG_MainController.prototype.showForfeitDialog = function(pauseGame, callbackFunction){
	"use strict";
	var buttons = [], question, yes, no, yesFunction, callback;
	if(!TG.controller.activeModelDialog){
		if(!TG.controller.activeDialog){
			if(TG.controller._challengeInformation.currentPlayerIdx === TG.controller._challengeInformation.challengePlayerIdx){
				question = TG.dictionary.getText("challengeCancelConfirmText", "<CHALLENGECANCELCONFIRMTEXT>");
				yes = "challengeCancelConfirmBtn_yes";
				no = "challengeCancelConfirmBtn_no";
				callback = function(success){
					var message = success? "challengeCancelMessage_success" : "challengeCancelMessage_error";
					message = TG.dictionary.getText(message, "<" + message.toUpperCase() + "<");
					if(TG.controller.activeDialog){
						TG.controller.activeDialog.showConfirmMessage(message);
					}
					if(success){
						TG.controller._pauseAll();
					}
				};
				yesFunction = function(){
					if(typeof callbackFunction === "function"){
						callbackFunction(true);
					}
					TG.controller.cancelChallenge(callback);
					return true;
				};
			}else{
				question = TG.dictionary.getText("challengeForfeitConfirmText", "<CHALLENGEFORFEITCONFIRMTEXT>");
				yes = "challengeForfeitConfirmBtn_yes";
				no = "challengeForfeitConfirmBtn_no";
				callback = function(success){
					var message = success? "challengeForfeitMessage_success" : "challengeForfeitMessage_error";
					message = TG.dictionary.getText(message, "<" + message.toUpperCase() + "<");
					if(TG.controller.activeDialog){
						TG.controller.activeDialog.showConfirmMessage(message);
						if(success){
							message = TG.dictionary.getText("challengeForfeitMessage_winnings", "");
							message = message.replace("<NAME>",
							        TG.controller._challengeInformation.playerNames[TG.controller._challengeInformation.challengePlayerIdx]);
							message = message.replace("<AMOUNT>", TG.controller._challengeInformation.wager);
							TG.controller.activeDialog.showConfirmSubMessage(message);
						}
					}
					if(success){
						TG.controller._pauseAll();
					}
				};
				yesFunction = function(){
					if(typeof callbackFunction === "function"){
						callbackFunction(true);
					}
					TG.controller.forfeitChallenge(callback);
					return true;
				};
			}
			buttons.push({
			    textKey: yes,
			    callbackFunc: yesFunction,
			    callbackObj: TG.controller
			});
			buttons.push({
			    textKey: no,
			    callbackFunc: function(){
				    TG.controller.activeDialog.close();
				    TG.controller.activeDialog = null;
				    if(typeof callbackFunction === "function"){
					    callbackFunction(false);
				    }
				    return true;
			    }
			});
			TG.controller.activeDialog = new TG_Dialog(question, buttons, pauseGame);
			TG.controller.activeModelDialog = TG.controller.activeDialog;
		}
	}
};

TG_MainController.prototype.pauseGame = function(){
	"use strict";
	var i, tokens;

	// inform all tokens that they are going to be paused
	tokens = tbGame.findTokens("game");
	for(i = 0; i < tokens.length; i++){
		if(typeof tokens[i].handleTokenPause === "function"){
			tokens[i].handleTokenPause();
		}
	}

	// pause level timing
	TG.analytics.pauseTimedEvent(TG.controller.timedLevelEvents);

	// pause tokens
	tbGame.pauseTokens("game");
	tbGame.pauseAllAlarms();
};

TG_MainController.prototype.resumeGame = function(){
	"use strict";
	var i, tokens;

	// inform all tokens that they are going to be unpaused
	tokens = tbGame.findTokens("game");
	for(i = 0; i < tokens.length; i++){
		if(typeof tokens[i].handleTokenResume === "function"){
			tokens[i].handleTokenResume();
		}
	}

	// pause tokens
	tbGame.resumeTokens("game");
	tbGame.resumeAllAlarms();

	// resume level timing
	TG.analytics.resumeTimedEvent(TG.controller.timedLevelEvents);
};

TG_MainController.prototype._pauseAll = function(exceptionToken){
	"use strict";
	var i, tokens;

	// inform all tokens that they are going to be paused
	tokens = tbGame.findTokens();
	for(i = 0; i < tokens.length; i++){
		if(typeof tokens[i].handleTokenPause === "function"){
			tokens[i].handleTokenPause();
		}
	}

	// pause all tokens and alarms
	tbGame.pauseTokens();
	tbGame.pauseAllAlarms();

	// pause level timing
	TG.analytics.pauseTimedEvent(TG.controller.timedLevelEvents);

	// resume exception token and system tokens
	if(exceptionToken){
		tbGame.resumeToken(exceptionToken);
	}
	tbGame.resumeTokens("system");
};

TG_MainController.prototype._resumeAll = function(){
	"use strict";
	var i, tokens;

	// inform all tokens that they are going to be unpaused
	tokens = tbGame.findTokens();
	for(i = 0; i < tokens.length; i++){
		if(typeof tokens[i].handleTokenResume === "function"){
			tokens[i].handleTokenResume();
		}
	}
	// pause tokens
	tbGame.resumeTokens();
	tbGame.resumeAllAlarms();

	// resume level timing
	TG.analytics.resumeTimedEvent(TG.controller.timedLevelEvents);
};

TG_MainController.prototype._deleteGame = function(){
	"use strict";
	var levelStart;
	//pause tokens
	if(TG.level){
		tbGame.removeToken(TG.level);
	}
	levelStart = tbGame.findTokens("LevelStartDialog");
	while(levelStart.length > 0){
		tbGame.removeToken(levelStart.pop());
	}
};

TG_MainController.prototype._quitGame = function(){
	"use strict";
	TG.analytics.reportTimedEventStop(TG.controller.timedLevelEvents, TG.controller.getLevelString(), "progression:levelQuit:"
	        + TG.controller.getDifficultyId());
	TG.hooks.informLevelQuit();
	TG.controller._deleteGame();
	TG.controller._showStartScreen();
};

TG_MainController.prototype._restartGame = function(){
	"use strict";
	this.state = "LEVEL_END";
	TG.analytics.reportTimedEventStop(TG.controller.timedLevelEvents, TG.controller.getLevelString(), "progression:levelRestart:"
	        + TG.controller.getDifficultyId());

	var level = TG.gameFlowSettings.usePerWorldLevelNumbering? this.currentLevelIdx + 1 : this.getSequentialLevelIdx() + 1;
	TG.controller.levelEnd = true;
	TG.controller.levelEndAction = "retry";
	TG.controller.pauseFlow(true, true);

	var levelData = {
		    "failed": true,
		    "level": level
		};
	TG.hooks.handleLevelCompleted(levelData);
	TG.adHooks.handleLevelCompleted(levelData);
};

TG_MainController.prototype.getShortVersion = function(){
	"use strict";
	var version = "";
	if(TG.version["builder"]){
		version = TG.version["builder"];
	}
	if(TG.version["tg"]){
		version += "-" + TG.version["tg"];
	}
	if(TG.version["game"]){
		version += "-" + TG.version["game"];
	}
	if(TG.version["config"]){
		version += "-" + TG.version["config"];
	}
	return version;
};

TG_MainController.prototype.handleAdd = function(){
	"use strict";
	if(!this.init){
		this._init();
		TG.controller.pauseFlow(true, true);
		TG.adHooks.handleGameInitialized();
		TG.hooks.handleGameInitialized();
	}
};

TG_MainController.prototype.handleBeginStep = function(timePassed){
	"use strict";
	//poll if portal configuration is ready for next step
	if(typeof this.gameFlowWaiting === "function"){
		this.gameFlowWaiting();
		if(!this.gameFlowWaiting){
			TG.controller.continueGameFlow();
		}
	}

	//poll if flow should continue after having to wait for either or both portal and ads
	if(this.flowPausedCounter > 0){
		if(!this.waitingForAds && !this.waitingForPortals){
			this.resumeFlow();
		}
	}

	if(this.initWaitTimer < 700){
		this.initWaitTimer += timePassed;
		if(this.initWaitTimer >= 700){
			// start analytics
			if(TG.config.track && TG.config.ga !== undefined && TG.config.ga.gameKey && TG.config.ga.secretKey){
				TG.analytics.start([TG.config.ga.gameKey, TG.config.ga.secretKey]);
			}
			// unlock first level
			if(this.getLevelData(this.currentWorldIdx, 0, "state") === undefined){
				this.updateLevelData(this.currentWorldIdx, 0, "state", "unlocked");
			}
		}
	}
};

// handleSpecial is called if a special or user-defined event happens.
TG_MainController.prototype.handleSpecial = function(id, args){
	"use strict";
	if(id === "languageSet"){
		TG.controller.language(args);
	}else if(id === TG.resizeEventId){
		TG.hooks.informResize();
	}
};

// handleEndStep is called each step after processing other events (before
// draw).
TG_MainController.prototype.handleEndStep = function(){
	"use strict";
	// clear canvasses if required
	var i, canvas;
	for(i = 0; i < TG.canvasses.length; i++){
		canvas = TG.canvasses[i];
		if(canvas.drawTokens){
			tbGraphics.setCanvas(canvas);
			tbGraphics.clear();
		}
	}
};

// handleDraw is called at the end of a step to draw the token.
TG_MainController.prototype.handleDraw = function(){
	"use strict";
	// reset drawTokens property for all canvases
	var i;
	for(i = 0; i < TG.canvasses.length; i++){
		TG.canvasses[i].drawTokens = false;
	}
};

TG.startFramework = function() {
	"use strict";
	//Create an add it to the game engine
	TG.controller = new TG_MainController();
	tbGame.addToken(TG.controller);
	tbGame.addTokenTags(TG.controller, "system");
};

if (TG.autoStartFramework === undefined || TG.autoStartFramework) {
	TG.startFramework();
}
